@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2020 Intel Corporation
+* Copyright 2019-2022 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -182,7 +182,8 @@ void getting_started_tutorial(engine::kind engine_kind) {
     /// To initialize the @ref dnnl::memory::desc, we need to pass:
     ///   1. The tensor's dimensions, **the semantic order** of which is
     ///      defined by **the primitive** that will use this memory
-    ///      (descriptor). Which leads to the following:
+    ///      (descriptor).
+    ///
     ///      @warning
     ///         Memory descriptors and objects are not aware of any meaning of
     ///         the data they describe or contain.
@@ -255,6 +256,7 @@ void getting_started_tutorial(engine::kind engine_kind) {
     /// as required by CNN primitives. To define the physical memory format,
     /// the strides are passed as the third parameter. Note that the order of
     /// the strides corresponds to the order of the tensor's dimensions.
+    ///
     /// @warning
     ///     Using the wrong order might lead to incorrect results or even a
     ///     crash.
@@ -275,7 +277,7 @@ void getting_started_tutorial(engine::kind engine_kind) {
 
     /// We already have a memory buffer for the source memory object.  We pass
     /// it to the
-    /// @ref dnnl::memory::memory(const desc &, const engine &, void *)
+    /// @ref dnnl::memory::memory(const dnnl::memory::desc &, const dnnl::engine &, void *)
     /// constructor that takes a buffer pointer as its last argument.
     ///
     /// Let's use a constructor that instructs the library to allocate a
@@ -303,19 +305,16 @@ void getting_started_tutorial(engine::kind engine_kind) {
     /// function to each and every element of the source tensor.
     ///
     /// Just as in the case of @ref dnnl::memory, a user should always go
-    /// through (at least) three creation steps (which however, can be sometimes
+    /// through (at least) two creation steps (which however, can be sometimes
     /// combined thanks to C++11):
-    /// 1. Initialize an operation descriptor (in this example,
-    ///    @ref dnnl::eltwise_forward::desc), which defines the operation
-    ///    parameters.
-    /// 2. Create an operation primitive descriptor (here @ref
-    ///    dnnl::eltwise_forward::primitive_desc), which is a
-    ///    **lightweight** descriptor of the actual algorithm that
-    ///    **implements** the given operation. The user can query different
-    ///    characteristics of the chosen implementation such as memory
-    ///    consumptions and some others that will be covered in the next topic
-    ///    (@ref memory_format_propagation_cpp).
-    /// 3. Create a primitive (here @ref dnnl::eltwise_forward) that can be
+    /// 1. Create an operation primitive descriptor (here @ref
+    ///    dnnl::eltwise_forward::primitive_desc) that defines operation
+    ///    parameters and is a **lightweight** descriptor of the actual
+    ///    algorithm that **implements** the given operation.
+    ///    The user can query different characteristics of the chosen
+    ///    implementation such as memory consumptions and some others that will
+    ///    be covered in the next topic (@ref memory_format_propagation_cpp).
+    /// 2. Create a primitive (here @ref dnnl::eltwise_forward) that can be
     ///    executed on memory objects to compute the operation.
     ///
     /// oneDNN separates steps 2 and 3 to enable the user to inspect details of a
@@ -330,21 +329,17 @@ void getting_started_tutorial(engine::kind engine_kind) {
     /// The code:
     /// @snippet getting_started.cpp Create a ReLU primitive
     // [Create a ReLU primitive]
-    //  ReLU op descriptor (no engine- or implementation-specific information)
-    auto relu_d = eltwise_forward::desc(
+    // ReLU primitive descriptor, which corresponds to a particular
+    // implementation in the library
+    auto relu_pd = eltwise_forward::primitive_desc(
+            eng, // an engine the primitive will be created for
             prop_kind::forward_inference, algorithm::eltwise_relu,
-            src_md, // the memory descriptor for an operation to work on
+            src_md, // source memory descriptor for an operation to work on
+            src_md, // destination memory descriptor for an operation to work on
             0.f, // alpha parameter means negative slope in case of ReLU
             0.f // beta parameter is ignored in case of ReLU
     );
 
-    // ReLU primitive descriptor, which corresponds to a particular
-    // implementation in the library
-    auto relu_pd
-            = eltwise_forward::primitive_desc(relu_d, // an operation descriptor
-                    eng // an engine the primitive will be created for
-            );
-
     // ReLU primitive
     auto relu = eltwise_forward(relu_pd); // !!! this can take quite some time
     // [Create a ReLU primitive]
@@ -487,6 +482,7 @@ int main(int argc, char **argv) {
 
     std::cout << "Example " << (exit_code ? "failed" : "passed") << " on "
               << engine_kind2str_upper(engine_kind) << "." << std::endl;
+    finalize();
     return exit_code;
 }
 // [Main]
