@@ -37,25 +37,6 @@
 #include "oneapi/dnnl/dnnl.hpp"
 #include "oneapi/dnnl/dnnl_sycl.h"
 
-#if defined(__INTEL_LLVM_COMPILER)
-#if (__INTEL_LLVM_COMPILER < 20220000)
-#define DNNL_SYCL_INTEROP_USE_SYCL121
-#endif
-#elif defined(__LIBSYCL_MAJOR_VERSION) && defined(__LIBSYCL_MINOR_VERSION)
-#if (__LIBSYCL_MAJOR_VERSION == 5 && __LIBSYCL_MINOR_VERSION < 4)
-#define DNNL_SYCL_INTEROP_USE_SYCL121
-#endif
-#else
-#error "Unsupported compiler"
-#endif
-
-#ifdef DNNL_SYCL_INTEROP_USE_SYCL121
-// oneAPI DPC++/C++ Compiler issues deprecation warnings for SYCL 1.2.1 API.
-// We suppress the warnings to avoid getting them into the user application.
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-#endif
-
 /// @endcond
 
 /// @addtogroup dnnl_api
@@ -195,11 +176,7 @@ sycl::buffer<T, ndims> get_buffer(const memory &amemory) {
 
     auto &buf_u8 = *static_cast<sycl::buffer<uint8_t, 1> *>(handle_ptr);
 
-#ifdef DNNL_SYCL_INTEROP_USE_SYCL121
-    auto range = sycl::range<1>(buf_u8.get_size() / sizeof(T));
-#else
     auto range = sycl::range<1>(buf_u8.byte_size() / sizeof(T));
-#endif
     return buf_u8.reinterpret<T, 1>(range);
 }
 
@@ -211,35 +188,10 @@ sycl::buffer<T, ndims> get_buffer(const memory &amemory) {
 /// @param abuffer SYCL buffer.
 template <typename T, int ndims>
 void set_buffer(memory &amemory, sycl::buffer<T, ndims> &abuffer) {
-#ifdef DNNL_SYCL_INTEROP_USE_SYCL121
-    auto range = sycl::range<1>(abuffer.get_size());
-#else
     auto range = sycl::range<1>(abuffer.byte_size());
-#endif
     auto buf_u8 = abuffer.template reinterpret<uint8_t, 1>(range);
-    error::wrap_c_api(dnnl_sycl_interop_memory_set_buffer(amemory.get(),
-                              static_cast<void *>(&buf_u8), nullptr),
-            "could not set SYCL buffer object");
-}
-
-/// Sets SYCL buffer associated with a memory object in a specified stream.
-///
-/// @tparam T Type of the buffer.
-/// @tparam ndims Number of dimensions of the buffer.
-/// @param amemory Memory object to change.
-/// @param abuffer SYCL buffer.
-/// @param astream Stream to use to execute padding in.
-template <typename T, int ndims>
-void set_buffer(memory &amemory, sycl::buffer<T, ndims> &abuffer,
-        const stream &astream) {
-#ifdef DNNL_SYCL_INTEROP_USE_SYCL121
-    auto range = sycl::range<1>(abuffer.get_size());
-#else
-    auto range = sycl::range<1>(abuffer.byte_size());
-#endif
-    auto buf_u8 = abuffer.template reinterpret<uint8_t, 1>(range);
-    error::wrap_c_api(dnnl_sycl_interop_memory_set_buffer(amemory.get(),
-                              static_cast<void *>(&buf_u8), astream.get(true)),
+    error::wrap_c_api(dnnl_sycl_interop_memory_set_buffer(
+                              amemory.get(), static_cast<void *>(&buf_u8)),
             "could not set SYCL buffer object");
 }
 
@@ -289,7 +241,7 @@ inline memory make_memory(const memory::desc &memory_desc,
         void *handle = DNNL_MEMORY_ALLOCATE) {
     dnnl_memory_t c_memory;
     error::wrap_c_api(
-            dnnl_sycl_interop_memory_create(&c_memory, &memory_desc.data,
+            dnnl_sycl_interop_memory_create(&c_memory, memory_desc.get(),
                     aengine.get(), convert_to_c(kind), handle),
             "could not create a memory");
     return memory(c_memory);
@@ -353,9 +305,4 @@ inline sycl::event execute(const dnnl::primitive &aprimitive,
 
 /// @} dnnl_api
 
-#ifdef DNNL_SYCL_INTEROP_USE_SYCL121
-#undef DNNL_SYCL_INTEROP_USE_SYCL121
-#pragma clang diagnostic pop
-#endif
-
 #endif // DNNL_SYCL_HPP
