@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -173,9 +173,12 @@ status_t memory_desc_wrapper::compute_blocking(
         C(bca, {1, 2, 0}, {}, {});
         C(bcda, {1, 2, 3, 0}, {}, {});
         C(bcdea, {1, 2, 3, 4, 0}, {}, {});
+        C(cab, {2, 0, 1}, {}, {});
         C(cba, {2, 1, 0}, {}, {});
+        C(cdab, {2, 3, 0, 1}, {}, {});
         C(cdba, {2, 3, 1, 0}, {}, {});
         C(dcab, {3, 2, 0, 1}, {}, {});
+        C(cdeab, {2, 3, 4, 0, 1}, {}, {});
         C(cdeba, {2, 3, 4, 1, 0}, {}, {});
         C(decab, {3, 4, 2, 0, 1}, {}, {});
         C(defcab, {3, 4, 5, 2, 0, 1}, {}, {});
@@ -195,8 +198,22 @@ status_t memory_desc_wrapper::compute_blocking(
         C(BA16a32b4a, {1, 0}, {16, 32, 4}, {0, 1, 0});
         C(BA16a48b4a, {1, 0}, {16, 48, 4}, {0, 1, 0});
         C(BA16a64b4a, {1, 0}, {16, 64, 4}, {0, 1, 0});
+        C(aCB16b16c, {0, 2, 1}, {16, 16}, {1, 2});
+        C(aCB16b32c, {0, 2, 1}, {16, 32}, {1, 2});
+        C(aCB16b48c, {0, 2, 1}, {16, 48}, {1, 2});
+        C(aCB16b64c, {0, 2, 1}, {16, 64}, {1, 2});
+        C(aCB16b16c2b, {0, 2, 1}, {16, 16, 2}, {1, 2, 1});
+        C(aCB16b32c2b, {0, 2, 1}, {16, 32, 2}, {1, 2, 1});
+        C(aCB16b48c2b, {0, 2, 1}, {16, 48, 2}, {1, 2, 1});
+        C(aCB16b64c2b, {0, 2, 1}, {16, 64, 2}, {1, 2, 1});
+        C(aCB16b16c4b, {0, 2, 1}, {16, 16, 4}, {1, 2, 1});
+        C(aCB16b32c4b, {0, 2, 1}, {16, 32, 4}, {1, 2, 1});
+        C(aCB16b48c4b, {0, 2, 1}, {16, 48, 4}, {1, 2, 1});
+        C(aCB16b64c4b, {0, 2, 1}, {16, 64, 4}, {1, 2, 1});
 
+        C(ABc4b8a4b, {0, 1, 2}, {4, 8, 4}, {1, 0, 1});
         C(ABc4b16a4b, {0, 1, 2}, {4, 16, 4}, {1, 0, 1});
+        C(ABc4b24a4b, {0, 1, 2}, {4, 24, 4}, {1, 0, 1});
         C(ABc4b32a4b, {0, 1, 2}, {4, 32, 4}, {1, 0, 1});
         C(ABc4b64a4b, {0, 1, 2}, {4, 64, 4}, {1, 0, 1});
         C(ABc2b8a4b, {0, 1, 2}, {2, 8, 4}, {1, 0, 1});
@@ -263,7 +280,9 @@ status_t memory_desc_wrapper::compute_blocking(
         C(AB8b16a2b, {0, 1}, {8, 16, 2}, {1, 0, 1});
         C(AB8b32a2b, {0, 1}, {8, 32, 2}, {1, 0, 1});
         C(AB8b64a2b, {0, 1}, {8, 64, 2}, {1, 0, 1});
+        C(AB4b8a4b, {0, 1}, {4, 8, 4}, {1, 0, 1});
         C(AB4b16a4b, {0, 1}, {4, 16, 4}, {1, 0, 1});
+        C(AB4b24a4b, {0, 1}, {4, 24, 4}, {1, 0, 1});
         C(AB16b16a4b, {0, 1}, {16, 16, 4}, {1, 0, 1});
         C(AB16b32a4b, {0, 1}, {16, 32, 4}, {1, 0, 1});
         C(AB16b48a4b, {0, 1}, {16, 48, 4}, {1, 0, 1});
@@ -320,7 +339,9 @@ status_t memory_desc_wrapper::compute_blocking(
         C(ABcd16b64a, {0, 1, 2, 3}, {16, 64}, {1, 0});
         C(aBCd16b16c, {0, 1, 2, 3}, {16, 16}, {1, 2});
         C(aBCd16c16b, {0, 1, 2, 3}, {16, 16}, {2, 1});
+        C(ABcd4b8a4b, {0, 1, 2, 3}, {4, 8, 4}, {1, 0, 1});
         C(ABcd4b16a4b, {0, 1, 2, 3}, {4, 16, 4}, {1, 0, 1});
+        C(ABcd4b24a4b, {0, 1, 2, 3}, {4, 24, 4}, {1, 0, 1});
         C(ABcd4b32a4b, {0, 1, 2, 3}, {4, 32, 4}, {1, 0, 1});
         C(ABcd4b64a4b, {0, 1, 2, 3}, {4, 64, 4}, {1, 0, 1});
         C(ABcd2b8a4b, {0, 1, 2, 3}, {2, 8, 4}, {1, 0, 1});
@@ -384,7 +405,9 @@ status_t memory_desc_wrapper::compute_blocking(
         C(ABcde8b32a2b, {0, 1, 2, 3, 4}, {8, 32, 2}, {1, 0, 1});
         C(ABcde8b64a2b, {0, 1, 2, 3, 4}, {8, 64, 2}, {1, 0, 1});
         C(BAcde8a16b2a, {1, 0, 2, 3, 4}, {8, 16, 2}, {0, 1, 0});
+        C(ABcde4b8a4b, {0, 1, 2, 3, 4}, {4, 8, 4}, {1, 0, 1});
         C(ABcde4b16a4b, {0, 1, 2, 3, 4}, {4, 16, 4}, {1, 0, 1});
+        C(ABcde4b24a4b, {0, 1, 2, 3, 4}, {4, 24, 4}, {1, 0, 1});
         C(ABcde4b32a4b, {0, 1, 2, 3, 4}, {4, 32, 4}, {1, 0, 1});
         C(ABcde4b64a4b, {0, 1, 2, 3, 4}, {4, 64, 4}, {1, 0, 1});
         C(ABcde16b16a4b, {0, 1, 2, 3, 4}, {16, 16, 4}, {1, 0, 1});
@@ -417,29 +440,37 @@ status_t memory_desc_wrapper::compute_blocking(
         C(aBdC16b2c, {0, 1, 3, 2}, {16, 2}, {1, 2});
         C(aBdC16b4c, {0, 1, 3, 2}, {16, 4}, {1, 2});
         C(aBdc8b, {0, 1, 3, 2}, {8}, {1});
+        C(aBdC8b2c, {0, 1, 3, 2}, {8, 2}, {1, 2});
         C(aBdec16b, {0, 1, 3, 4, 2}, {16}, {1});
         C(aBdeC16b2c, {0, 1, 3, 4, 2}, {16, 2}, {1, 2});
         C(aBdeC16b4c, {0, 1, 3, 4, 2}, {16, 4}, {1, 2});
         C(aBdec8b, {0, 1, 3, 4, 2}, {8}, {1});
+        C(aBdeC8b2c, {0, 1, 3, 4, 2}, {8, 2}, {1, 2});
         C(aBdefc16b, {0, 1, 3, 4, 5, 2}, {16}, {1});
         C(aBdefC16b2c, {0, 1, 3, 4, 5, 2}, {16, 2}, {1, 2});
         C(aBdefC16b4c, {0, 1, 3, 4, 5, 2}, {16, 4}, {1, 2});
         C(aBdefc8b, {0, 1, 3, 4, 5, 2}, {8}, {1});
+        C(aBdefC8b2c, {0, 1, 3, 4, 5, 2}, {8, 2}, {1, 2});
+        C(aBdfec16b, {0, 1, 3, 5, 4, 2}, {16}, {1});
         C(aBedc16b, {0, 1, 4, 3, 2}, {16}, {1});
         C(Acb16a, {0, 2, 1}, {16}, {0});
         C(AcB16a2b, {0, 2, 1}, {16, 2}, {0, 1});
         C(AcB16a4b, {0, 2, 1}, {16, 4}, {0, 1});
         C(Acb8a, {0, 2, 1}, {8}, {0});
+        C(AcB8a2b, {0, 2, 1}, {8, 2}, {0, 1});
         C(aCBd16b16c, {0, 2, 1, 3}, {16, 16}, {1, 2});
         C(aCBde16b16c, {0, 2, 1, 3, 4}, {16, 16}, {1, 2});
         C(Acdb16a, {0, 2, 3, 1}, {16}, {0});
         C(AcdB16a2b, {0, 2, 3, 1}, {16, 2}, {0, 1});
         C(AcdB16a4b, {0, 2, 3, 1}, {16, 4}, {0, 1});
         C(Acdb8a, {0, 2, 3, 1}, {8}, {0});
+        C(AcdB8a2b, {0, 2, 3, 1}, {8, 2}, {0, 1});
         C(Acdeb16a, {0, 2, 3, 4, 1}, {16}, {0});
         C(AcdeB16a2b, {0, 2, 3, 4, 1}, {16, 2}, {0, 1});
         C(AcdeB16a4b, {0, 2, 3, 4, 1}, {16, 4}, {0, 1});
         C(Acdeb8a, {0, 2, 3, 4, 1}, {8}, {0});
+        C(AcdeB8a2b, {0, 2, 3, 4, 1}, {8, 2}, {0, 1});
+        C(Acedb16a, {0, 2, 4, 3, 1}, {16}, {0});
         C(Adcb16a, {0, 3, 2, 1}, {16}, {0});
         C(BAc16a16b, {1, 0, 2}, {16, 16}, {0, 1});
         C(BAcd16a16b, {1, 0, 2, 3}, {16, 16}, {0, 1});
@@ -519,12 +550,22 @@ status_t memory_desc_wrapper::compute_blocking(
         C(AB8a4b, {0, 1}, {8, 4}, {0, 1});
         C(AB32a32b8a2b, {0, 1}, {32, 32, 8, 2}, {0, 1, 0, 1});
         C(AB8a2b, {0, 1}, {8, 2}, {0, 1});
+        C(abDc16d, {0, 1, 3, 2}, {16}, {3});
         C(abDc32d, {0, 1, 3, 2}, {32}, {3});
         C(abDC32d4c, {0, 1, 3, 2}, {32, 4}, {3, 2});
+        C(abCd4c, {0, 1, 2, 3}, {4}, {2});
+        C(abCde4c, {0, 1, 2, 3, 4}, {4}, {2});
+        C(abCdef4c, {0, 1, 2, 3, 4, 5}, {4}, {2});
         C(abCd32c, {0, 1, 2, 3}, {32}, {2});
+        C(abCde32c, {0, 1, 2, 3, 4}, {32}, {2});
+        C(abCdef32c, {0, 1, 2, 3, 4, 5}, {32}, {2});
+        C(abdEc16e, {0, 1, 3, 4, 2}, {16}, {4});
         C(abdEc32e, {0, 1, 3, 4, 2}, {32}, {4});
         C(abdEC32e2c, {0, 1, 3, 4, 2}, {32, 2}, {4, 2});
         C(abdEC32e4c, {0, 1, 3, 4, 2}, {32, 4}, {4, 2});
+        C(abdEC64e2c, {0, 1, 3, 4, 2}, {64, 2}, {4, 2});
+        C(abdEC64e4c, {0, 1, 3, 4, 2}, {64, 4}, {4, 2});
+        C(abdCe16c, {0, 1, 3, 2, 4}, {16}, {2});
         C(abdCe32c, {0, 1, 3, 2, 4}, {32}, {2});
         C(abdCE32c2e, {0, 1, 3, 2, 4}, {32, 2}, {2, 4});
 
@@ -644,6 +685,7 @@ status_t memory_desc_wrapper::compute_blocking(
         C(AcdeB16b64a2b, {0, 2, 3, 4, 1}, {16, 64, 2}, {1, 0, 1});
         C(AcdeB16b64a4b, {0, 2, 3, 4, 1}, {16, 64, 4}, {1, 0, 1});
         C(decbA16a, {3, 4, 2, 1, 0}, {16}, {0});
+        C(decbA8a, {3, 4, 2, 1, 0}, {8}, {0});
         C(aCB16c2b, {0, 2, 1}, {16, 2}, {2, 1});
         C(aCB16c4b, {0, 2, 1}, {16, 4}, {2, 1});
         C(BA16b2a, {1, 0}, {16, 2}, {1, 0});
@@ -652,7 +694,164 @@ status_t memory_desc_wrapper::compute_blocking(
         C(aBC16b32c, {0, 1, 2}, {16, 32}, {1, 2});
         C(AB16a16b, {0, 1}, {16, 16}, {0, 1});
         C(AB16a32b, {0, 1}, {16, 32}, {0, 1});
+        C(ABcd16a32b, {0, 1, 2, 3}, {16, 32}, {0, 1});
 
+        C(aCdefB16b32c2b, {0, 2, 3, 4, 5, 1}, {16, 32, 2}, {1, 2, 1});
+        C(aCdefB16b32c4b, {0, 2, 3, 4, 5, 1}, {16, 32, 4}, {1, 2, 1});
+        C(aCdefB16b48c2b, {0, 2, 3, 4, 5, 1}, {16, 48, 2}, {1, 2, 1});
+        C(aCdefB16b48c4b, {0, 2, 3, 4, 5, 1}, {16, 48, 4}, {1, 2, 1});
+        C(aCdefB16b64c2b, {0, 2, 3, 4, 5, 1}, {16, 64, 2}, {1, 2, 1});
+        C(aCdefB16b64c4b, {0, 2, 3, 4, 5, 1}, {16, 64, 4}, {1, 2, 1});
+        C(BcdeA16a32b2a, {1, 2, 3, 4, 0}, {16, 32, 2}, {0, 1, 0});
+        C(BcdeA16a32b4a, {1, 2, 3, 4, 0}, {16, 32, 4}, {0, 1, 0});
+        C(BcdeA16a48b2a, {1, 2, 3, 4, 0}, {16, 48, 2}, {0, 1, 0});
+        C(BcdeA16a48b4a, {1, 2, 3, 4, 0}, {16, 48, 4}, {0, 1, 0});
+        C(BcdeA16a64b2a, {1, 2, 3, 4, 0}, {16, 64, 2}, {0, 1, 0});
+        C(BcdeA16a64b4a, {1, 2, 3, 4, 0}, {16, 64, 4}, {0, 1, 0});
+        C(aCdefb32c, {0, 2, 3, 4, 5, 1}, {32}, {2});
+        C(aCdefB32c2b, {0, 2, 3, 4, 5, 1}, {32, 2}, {2, 1});
+        C(aCdefB32c4b, {0, 2, 3, 4, 5, 1}, {32, 4}, {2, 1});
+        C(aCdefb48c, {0, 2, 3, 4, 5, 1}, {48}, {2});
+        C(aCdefB48c2b, {0, 2, 3, 4, 5, 1}, {48, 2}, {2, 1});
+        C(aCdefB48c4b, {0, 2, 3, 4, 5, 1}, {48, 4}, {2, 1});
+        C(aCdefb64c, {0, 2, 3, 4, 5, 1}, {64}, {2});
+        C(aCdefB64c2b, {0, 2, 3, 4, 5, 1}, {64, 2}, {2, 1});
+        C(aCdefB64c4b, {0, 2, 3, 4, 5, 1}, {64, 4}, {2, 1});
+        C(Bcdea32b, {1, 2, 3, 4, 0}, {32}, {1});
+        C(BcdeA32b2a, {1, 2, 3, 4, 0}, {32, 2}, {1, 0});
+        C(BcdeA32b4a, {1, 2, 3, 4, 0}, {32, 4}, {1, 0});
+        C(Bcdea48b, {1, 2, 3, 4, 0}, {48}, {1});
+        C(BcdeA48b2a, {1, 2, 3, 4, 0}, {48, 2}, {1, 0});
+        C(BcdeA48b4a, {1, 2, 3, 4, 0}, {48, 4}, {1, 0});
+        C(Bcdea64b, {1, 2, 3, 4, 0}, {64}, {1});
+        C(BcdeA64b2a, {1, 2, 3, 4, 0}, {64, 2}, {1, 0});
+        C(BcdeA64b4a, {1, 2, 3, 4, 0}, {64, 4}, {1, 0});
+        C(Bca32b, {1, 2, 0}, {32}, {1});
+        C(BcA32b2a, {1, 2, 0}, {32, 2}, {1, 0});
+        C(BcA32b4a, {1, 2, 0}, {32, 4}, {1, 0});
+        C(Bca48b, {1, 2, 0}, {48}, {1});
+        C(BcA48b2a, {1, 2, 0}, {48, 2}, {1, 0});
+        C(BcA48b4a, {1, 2, 0}, {48, 4}, {1, 0});
+        C(Bca64b, {1, 2, 0}, {64}, {1});
+        C(BcA64b2a, {1, 2, 0}, {64, 2}, {1, 0});
+        C(BcA64b4a, {1, 2, 0}, {64, 4}, {1, 0});
+        C(aCdb32c, {0, 2, 3, 1}, {32}, {2});
+        C(aCdB32c2b, {0, 2, 3, 1}, {32, 2}, {2, 1});
+        C(aCdB32c4b, {0, 2, 3, 1}, {32, 4}, {2, 1});
+        C(aCdb48c, {0, 2, 3, 1}, {48}, {2});
+        C(aCdB48c2b, {0, 2, 3, 1}, {48, 2}, {2, 1});
+        C(aCdB48c4b, {0, 2, 3, 1}, {48, 4}, {2, 1});
+        C(aCdb64c, {0, 2, 3, 1}, {64}, {2});
+        C(aCdB64c2b, {0, 2, 3, 1}, {64, 2}, {2, 1});
+        C(aCdB64c4b, {0, 2, 3, 1}, {64, 4}, {2, 1});
+        C(BcA16a16b2a, {1, 2, 0}, {16, 16, 2}, {0, 1, 0});
+        C(BcA16a16b4a, {1, 2, 0}, {16, 16, 4}, {0, 1, 0});
+        C(BcdA16a16b2a, {1, 2, 3, 0}, {16, 16, 2}, {0, 1, 0});
+        C(BcdA16a16b4a, {1, 2, 3, 0}, {16, 16, 4}, {0, 1, 0});
+        C(BcdeA16a16b2a, {1, 2, 3, 4, 0}, {16, 16, 2}, {0, 1, 0});
+        C(BcdeA16a16b4a, {1, 2, 3, 4, 0}, {16, 16, 4}, {0, 1, 0});
+        C(aCdB16b16c2b, {0, 2, 3, 1}, {16, 16, 2}, {1, 2, 1});
+        C(aCdB16b16c4b, {0, 2, 3, 1}, {16, 16, 4}, {1, 2, 1});
+        C(aCdeB16b16c2b, {0, 2, 3, 4, 1}, {16, 16, 2}, {1, 2, 1});
+        C(aCdeB16b16c4b, {0, 2, 3, 4, 1}, {16, 16, 4}, {1, 2, 1});
+        C(aCdefB16b16c2b, {0, 2, 3, 4, 5, 1}, {16, 16, 2}, {1, 2, 1});
+        C(aCdefB16b16c4b, {0, 2, 3, 4, 5, 1}, {16, 16, 4}, {1, 2, 1});
+        C(BcA16a32b2a, {1, 2, 0}, {16, 32, 2}, {0, 1, 0});
+        C(BcA16a32b4a, {1, 2, 0}, {16, 32, 4}, {0, 1, 0});
+        C(BcA16a48b2a, {1, 2, 0}, {16, 48, 2}, {0, 1, 0});
+        C(BcA16a48b4a, {1, 2, 0}, {16, 48, 4}, {0, 1, 0});
+        C(BcA16a64b2a, {1, 2, 0}, {16, 64, 2}, {0, 1, 0});
+        C(BcA16a64b4a, {1, 2, 0}, {16, 64, 4}, {0, 1, 0});
+        C(aCdB16b32c2b, {0, 2, 3, 1}, {16, 32, 2}, {1, 2, 1});
+        C(aCdB16b32c4b, {0, 2, 3, 1}, {16, 32, 4}, {1, 2, 1});
+        C(aCdB16b48c2b, {0, 2, 3, 1}, {16, 48, 2}, {1, 2, 1});
+        C(aCdB16b48c4b, {0, 2, 3, 1}, {16, 48, 4}, {1, 2, 1});
+        C(aCdB16b64c2b, {0, 2, 3, 1}, {16, 64, 2}, {1, 2, 1});
+        C(aCdB16b64c4b, {0, 2, 3, 1}, {16, 64, 4}, {1, 2, 1});
+        C(BcdA16a32b2a, {1, 2, 3, 0}, {16, 32, 2}, {0, 1, 0});
+        C(BcdA16a32b4a, {1, 2, 3, 0}, {16, 32, 4}, {0, 1, 0});
+        C(BcdA16a48b2a, {1, 2, 3, 0}, {16, 48, 2}, {0, 1, 0});
+        C(BcdA16a48b4a, {1, 2, 3, 0}, {16, 48, 4}, {0, 1, 0});
+        C(BcdA16a64b2a, {1, 2, 3, 0}, {16, 64, 2}, {0, 1, 0});
+        C(BcdA16a64b4a, {1, 2, 3, 0}, {16, 64, 4}, {0, 1, 0});
+        C(aCdeB16b32c2b, {0, 2, 3, 4, 1}, {16, 32, 2}, {1, 2, 1});
+        C(aCdeB16b32c4b, {0, 2, 3, 4, 1}, {16, 32, 4}, {1, 2, 1});
+        C(aCdeB16b48c2b, {0, 2, 3, 4, 1}, {16, 48, 2}, {1, 2, 1});
+        C(aCdeB16b48c4b, {0, 2, 3, 4, 1}, {16, 48, 4}, {1, 2, 1});
+        C(aCdeB16b64c2b, {0, 2, 3, 4, 1}, {16, 64, 2}, {1, 2, 1});
+        C(aCdeB16b64c4b, {0, 2, 3, 4, 1}, {16, 64, 4}, {1, 2, 1});
+        C(Bca16b, {1, 2, 0}, {16}, {1});
+        C(BcA16b2a, {1, 2, 0}, {16, 2}, {1, 0});
+        C(BcA16b4a, {1, 2, 0}, {16, 4}, {1, 0});
+        C(Bcda16b, {1, 2, 3, 0}, {16}, {1});
+        C(BcdA16b2a, {1, 2, 3, 0}, {16, 2}, {1, 0});
+        C(BcdA16b4a, {1, 2, 3, 0}, {16, 4}, {1, 0});
+        C(Bcdea16b, {1, 2, 3, 4, 0}, {16}, {1});
+        C(BcdeA16b2a, {1, 2, 3, 4, 0}, {16, 2}, {1, 0});
+        C(BcdeA16b4a, {1, 2, 3, 4, 0}, {16, 4}, {1, 0});
+        C(aCdb16c, {0, 2, 3, 1}, {16}, {2});
+        C(aCdB16c2b, {0, 2, 3, 1}, {16, 2}, {2, 1});
+        C(aCdB16c4b, {0, 2, 3, 1}, {16, 4}, {2, 1});
+        C(aCdeb16c, {0, 2, 3, 4, 1}, {16}, {2});
+        C(aCdeB16c2b, {0, 2, 3, 4, 1}, {16, 2}, {2, 1});
+        C(aCdeB16c4b, {0, 2, 3, 4, 1}, {16, 4}, {2, 1});
+        C(aCdefb16c, {0, 2, 3, 4, 5, 1}, {16}, {2});
+        C(aCdefB16c2b, {0, 2, 3, 4, 5, 1}, {16, 2}, {2, 1});
+        C(aCdefB16c4b, {0, 2, 3, 4, 5, 1}, {16, 4}, {2, 1});
+        C(Bcda32b, {1, 2, 3, 0}, {32}, {1});
+        C(BcdA32b2a, {1, 2, 3, 0}, {32, 2}, {1, 0});
+        C(BcdA32b4a, {1, 2, 3, 0}, {32, 4}, {1, 0});
+        C(Bcda48b, {1, 2, 3, 0}, {48}, {1});
+        C(BcdA48b2a, {1, 2, 3, 0}, {48, 2}, {1, 0});
+        C(BcdA48b4a, {1, 2, 3, 0}, {48, 4}, {1, 0});
+        C(Bcda64b, {1, 2, 3, 0}, {64}, {1});
+        C(BcdA64b2a, {1, 2, 3, 0}, {64, 2}, {1, 0});
+        C(BcdA64b4a, {1, 2, 3, 0}, {64, 4}, {1, 0});
+        C(aCdeb32c, {0, 2, 3, 4, 1}, {32}, {2});
+        C(aCdeB32c2b, {0, 2, 3, 4, 1}, {32, 2}, {2, 1});
+        C(aCdeB32c4b, {0, 2, 3, 4, 1}, {32, 4}, {2, 1});
+        C(aCdeb48c, {0, 2, 3, 4, 1}, {48}, {2});
+        C(aCdeB48c2b, {0, 2, 3, 4, 1}, {48, 2}, {2, 1});
+        C(aCdeB48c4b, {0, 2, 3, 4, 1}, {48, 4}, {2, 1});
+        C(aCdeb64c, {0, 2, 3, 4, 1}, {64}, {2});
+        C(aCdeB64c2b, {0, 2, 3, 4, 1}, {64, 2}, {2, 1});
+        C(aCdeB64c4b, {0, 2, 3, 4, 1}, {64, 4}, {2, 1});
+
+        C(Acb24a, {0, 2, 1}, {24}, {0});
+        C(Acdb24a, {0, 2, 3, 1}, {24}, {0});
+        C(Acdeb24a, {0, 2, 3, 4, 1}, {24}, {0});
+        C(aBdc24b, {0, 1, 3, 2}, {24}, {1});
+        C(aBdec24b, {0, 1, 3, 4, 2}, {24}, {1});
+        C(aBdefc24b, {0, 1, 3, 4, 5, 2}, {24}, {1});
+        C(AcB24a2b, {0, 2, 1}, {24, 2}, {0, 1});
+        C(AcdB24a2b, {0, 2, 3, 1}, {24, 2}, {0, 1});
+        C(AcdeB24a2b, {0, 2, 3, 4, 1}, {24, 2}, {0, 1});
+        C(aBdC24b2c, {0, 1, 3, 2}, {24, 2}, {1, 2});
+        C(aBdeC24b2c, {0, 1, 3, 4, 2}, {24, 2}, {1, 2});
+        C(aBdefC24b2c, {0, 1, 3, 4, 5, 2}, {24, 2}, {1, 2});
+
+        C(AB8b32a, {0, 1}, {8, 32}, {1, 0});
+        C(ABc8b32a, {0, 1, 2}, {8, 32}, {1, 0});
+        C(ABcd8b32a, {0, 1, 2, 3}, {8, 32}, {1, 0});
+        C(ABcde8b32a, {0, 1, 2, 3, 4}, {8, 32}, {1, 0});
+        C(AB8b24a, {0, 1}, {8, 24}, {1, 0});
+        C(ABc8b24a, {0, 1, 2}, {8, 24}, {1, 0});
+        C(ABcd8b24a, {0, 1, 2, 3}, {8, 24}, {1, 0});
+        C(ABcde8b24a, {0, 1, 2, 3, 4}, {8, 24}, {1, 0});
+        C(AB8b16a, {0, 1}, {8, 16}, {1, 0});
+        C(ABc8b16a, {0, 1, 2}, {8, 16}, {1, 0});
+        C(ABcd8b16a, {0, 1, 2, 3}, {8, 16}, {1, 0});
+        C(ABcde8b16a, {0, 1, 2, 3, 4}, {8, 16}, {1, 0});
+        C(AB8b8a, {0, 1}, {8, 8}, {1, 0});
+
+        C(AB8b24a2b, {0, 1}, {8, 24, 2}, {1, 0, 1});
+        C(ABc8b24a2b, {0, 1, 2}, {8, 24, 2}, {1, 0, 1});
+        C(ABcd8b24a2b, {0, 1, 2, 3}, {8, 24, 2}, {1, 0, 1});
+        C(ABcde8b24a2b, {0, 1, 2, 3, 4}, {8, 24, 2}, {1, 0, 1});
+        C(AB8b8a2b, {0, 1}, {8, 8, 2}, {1, 0, 1});
+        C(ABc8b8a2b, {0, 1, 2}, {8, 8, 2}, {1, 0, 1});
+        C(ABcd8b8a2b, {0, 1, 2, 3}, {8, 8, 2}, {1, 0, 1});
+        C(ABcde8b8a2b, {0, 1, 2, 3, 4}, {8, 8, 2}, {1, 0, 1});
         default: break;
     }
 
