@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -121,11 +121,10 @@ private:
     DNNL_DISALLOW_COPY_AND_ASSIGN(rnn_tparams_t);
 };
 
+// Note: keep for RNN quantization
 struct scales_t : public c_compatible {
-    scales_t() : count_(1), mask_(0), scales_(scales_buf_) { set(1.); }
-    scales_t(dim_t count, int mask, const float *scales)
-        : scales_(scales_buf_) {
-        set(count, mask, scales);
+    scales_t() : count_(1), mask_(0), scales_(scales_buf_) {
+        set_single_scale(1.);
     }
 
     ~scales_t() { cleanup(); }
@@ -149,8 +148,12 @@ struct scales_t : public c_compatible {
 
     bool defined() const { return !is_runtime_value(scales_[0]); }
 
+    void set_single_scale(float single_scale);
     status_t set(dim_t count, int mask, const float *scales);
-    status_t set(float single_scale) { return this->set(1, 0, &single_scale); }
+    status_t set(float single_scale) {
+        set_single_scale(single_scale);
+        return status::success;
+    }
 
     status_t copy_from(const scales_t &other) {
         return set(other.count_, other.mask_, other.scales_);
@@ -175,15 +178,42 @@ private:
     DNNL_DISALLOW_COPY_AND_ASSIGN(scales_t);
 };
 
-struct arg_scales_t : public c_compatible {
-    arg_scales_t() {
-        for (const auto &sa : {DNNL_ARG_SRC_0, DNNL_ARG_SRC_1}) {
-            set(sa, 1.f);
-        }
+struct runtime_scales_t : public c_compatible {
+    // Clang-3.8.1 raises an error for a default initialization of a const
+    // object. Const runtime_scales_t object is used as default_scales.
+    // runtime_scales_t() = default;
+    runtime_scales_t() {}
+
+    status_t set(int mask) {
+        mask_ = mask;
+        is_set_ = true;
+        return status::success;
     }
 
-    const scales_t &get(int arg) const {
-        static const scales_t default_scales;
+    bool operator==(const runtime_scales_t &rhs) const {
+        return mask_ == rhs.mask_ && is_set_ == rhs.is_set_;
+    }
+
+    bool has_default_values() const { return !is_set_; }
+
+    bool defined() const { return has_default_values(); }
+
+    void reset() {
+        mask_ = 0;
+        is_set_ = false;
+    }
+
+    // TODO: replace with `-1` to remove `is_set_`.
+    // Hide `mask_` under `private:` to force interface usage.
+    int mask_ = 0;
+    bool is_set_ = false;
+};
+
+struct arg_scales_t : public c_compatible {
+    arg_scales_t() = default;
+
+    const runtime_scales_t &get(int arg) const {
+        static const runtime_scales_t default_scales;
         const auto it = scales_.find(arg);
         if (it == scales_.end()) return default_scales;
         return it->second;
@@ -193,119 +223,107 @@ struct arg_scales_t : public c_compatible {
         return scales_ == rhs.scales_;
     }
 
-    bool has_default_values() const {
+    bool has_default_values(const std::vector<int> &skip_args = {}) const {
         for (const auto &s : scales_) {
-            if (!s.second.has_default_values()) return false;
+            if (!s.second.has_default_values()) {
+                bool skip = false;
+                for (const auto &skip_a : skip_args)
+                    if (s.first == skip_a) {
+                        skip = true;
+                        break;
+                    }
+                if (skip) continue;
+                return false;
+            }
         }
         return true;
     }
 
-    bool defined() const {
-        for (const auto &s : scales_) {
-            if (!s.second.defined()) return false;
-        }
-        return true;
+    status_t set(int arg, int mask) {
+        if (!check_arg(arg)) return status::invalid_arguments;
+        return scales_[arg].set(mask);
     }
 
-    status_t get(int arg, dim_t *count, int *mask, const float **scales) const;
-    status_t set(int arg, dim_t count, int mask, const float *scales);
-    status_t set(int arg, float single_scale) {
-        return set(arg, 1, 0, &single_scale);
+    status_t get(int arg, int *mask, bool *is_set) const {
+        if (!check_arg(arg)) return status::invalid_arguments;
+        const auto &s = get(arg);
+        if (mask) *mask = s.mask_;
+        if (is_set) *is_set = s.is_set_;
+        return status::success;
     }
 
+    status_t reset(int arg) {
+        if (!check_arg(arg)) return status::invalid_arguments;
+        const auto it = scales_.find(arg);
+        if (it != scales_.end()) scales_.erase(it);
+        return status::success;
+    }
+
+    bool defined() const { return has_default_values(); }
+
     status_t copy_from(const arg_scales_t &other) {
         for (auto it = other.scales_.begin(); it != other.scales_.end(); ++it) {
             // Find an entry that can match the arguments without constructing a
             // new object.
             if (scales_.count(it->first) == 1) {
                 auto &entry = scales_[it->first];
-                bool exists = entry.count_ == it->second.count_
-                        && entry.mask_ == it->second.mask_
-                        && !utils::any_null(entry.scales_, it->second.scales_)
-                        && !is_runtime_value(entry.scales_[0])
-                                == !is_runtime_value(it->second.scales_[0])
-                        && IMPLICATION(!is_runtime_value(entry.scales_[0]),
-                                utils::array_cmp(entry.scales_,
-                                        it->second.scales_, it->second.count_));
-
+                bool exists = entry.mask_ == it->second.mask_;
                 if (exists) continue;
             }
 
-            CHECK(set(it->first, it->second.count_, it->second.mask_,
-                    it->second.scales_));
+            CHECK(set(it->first, it->second.mask_));
         }
         return status::success;
     }
 
-    int get_index_val(int arg) const {
-        switch (arg) {
-            case DNNL_ARG_SRC_0: return 0;
-            case DNNL_ARG_SRC_1: return 1;
-            default: assert(!"unsupported arg");
-        }
-        return -1;
-    }
-
-    std::map<int, scales_t> scales_;
+    std::map<int, runtime_scales_t> scales_;
 
 private:
     bool check_arg(int arg) const {
+        // binary
         for (const auto &sa : {DNNL_ARG_SRC_0, DNNL_ARG_SRC_1}) {
             if (arg == sa) return true;
         }
+        // concat
+        if (arg & DNNL_ARG_MULTIPLE_SRC) return true;
+        // convolution
+        for (const auto &sa : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
+            if (arg == sa) return true;
+        }
+        // depth-wise convolution post op
+        for (const auto &sa : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | sa)) return true;
+        }
         return false;
     }
 };
 
 struct zero_points_t : public c_compatible {
     bool operator==(const zero_points_t &rhs) const {
-        auto eq = [](int a, int b) {
-            return a == b || (is_runtime_value(a) && is_runtime_value(b));
-        };
-        return eq(zero_point_src, rhs.zero_point_src)
-                && eq(mask_src, rhs.mask_src)
-                && eq(zero_point_wei, rhs.zero_point_wei)
-                && eq(mask_wei, rhs.mask_wei)
-                && eq(zero_point_dst, rhs.zero_point_dst)
-                && eq(mask_dst, rhs.mask_dst);
+        return mask_src == rhs.mask_src && mask_wei == rhs.mask_wei
+                && mask_dst == rhs.mask_dst && is_set_src == rhs.is_set_src
+                && is_set_wei == rhs.is_set_wei && is_set_dst == rhs.is_set_dst;
     }
 
     // arg-specific checks
     bool common(int arg) const { return get_mask(arg) == 0; }
-    bool defined(int arg) const { return !is_runtime_value(*get(arg)); }
-    bool has_default_values(int arg) const {
-        return *get(arg) == 0 && get_mask(arg) == 0;
-    }
+    bool defined(int arg) const { return has_default_values(arg); }
+    bool has_default_values(int arg) const { return is_set(arg) == false; }
 
     // same checks but for all supported arguments at once
     bool common() const { return check_all(&zero_points_t::common); }
-    bool defined() const { return check_all(&zero_points_t::defined); }
+    bool defined() const { return has_default_values(); }
     bool has_default_values() const {
         return check_all(&zero_points_t::has_default_values);
     }
 
-    const int *get(int arg) const {
-        arg &= ~DNNL_ARG_ATTR_ZERO_POINTS;
-        switch (arg) {
-            case DNNL_ARG_SRC: return &zero_point_src;
-            case DNNL_ARG_WEIGHTS: return &zero_point_wei;
-            case DNNL_ARG_DST: return &zero_point_dst;
-        }
-        static int zero = 0;
-        return &zero;
-    }
+    status_t get(int arg, int *mask) const;
 
-    status_t get(
-            int arg, dim_t *count, int *mask, const int **zero_points) const;
-
-    status_t set(int arg, dim_t count, int mask, const int *zero_points);
-    status_t set(int arg, int single_zero_points) {
-        return set(arg, 1, 0, &single_zero_points);
-    }
+    status_t set(int arg, int mask);
+    status_t set(int arg) { return set(arg, 0); }
 
 private:
-    // TODO: support count and mask
-    int zero_point_src = 0, zero_point_wei = 0, zero_point_dst = 0;
+    bool is_set_src = false, is_set_wei = false, is_set_dst = false;
     int mask_src = 0, mask_wei = 0, mask_dst = 0;
 
     int get_mask(int arg) const {
@@ -319,6 +337,17 @@ private:
         return mask;
     }
 
+    bool is_set(int arg) const {
+        bool arg_is_set = false;
+        switch (arg) {
+            case DNNL_ARG_SRC: arg_is_set = is_set_src; break;
+            case DNNL_ARG_WEIGHTS: arg_is_set = is_set_wei; break;
+            case DNNL_ARG_DST: arg_is_set = is_set_dst; break;
+            default: arg_is_set = 0;
+        }
+        return arg_is_set;
+    }
+
     bool check_all(bool (zero_points_t::*f)(int) const) const {
         for (int arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST})
             if (!(this->*f)(arg)) return false;
@@ -326,6 +355,17 @@ private:
     }
 };
 
+struct serialization_stream_t;
+
+struct primitive_attr_item_t {
+    virtual std::unique_ptr<primitive_attr_item_t> clone() const = 0;
+    virtual bool has_default_values() const = 0;
+    virtual bool is_equal(const primitive_attr_item_t &other) const = 0;
+    virtual size_t get_hash() const = 0;
+    virtual void serialize(serialization_stream_t &stream) const = 0;
+    virtual ~primitive_attr_item_t() = default;
+};
+
 } // namespace impl
 } // namespace dnnl
 
@@ -334,17 +374,13 @@ struct dnnl_post_ops : public dnnl::impl::c_compatible {
         entry_t() : kind(dnnl::impl::primitive_kind::undefined) {}
         entry_t(const entry_t &other) { copy_from(other); }
 
-        dnnl::impl::status_t copy_from(const entry_t &other) {
-            clear();
-            return set(other);
-        }
+        void copy_from(const entry_t &other) { set(other); }
 
         // TODO: This operator has to be deleted, and its usage has to be
         // replaced with copy_from() or copy/move constructors in order to
         // extract a status.
         entry_t &operator=(const entry_t &other) {
             DNNL_SHORT_CIRCUIT_SELF_ASSIGN(other);
-            clear();
             set(other);
             return *this;
         }
@@ -361,9 +397,6 @@ struct dnnl_post_ops : public dnnl::impl::c_compatible {
             dnnl::impl::data_type_t wei_dt;
             dnnl::impl::data_type_t bias_dt;
             dnnl::impl::data_type_t dst_dt;
-            dnnl::impl::dim_t count;
-            int mask;
-            float *scales;
         };
 
         struct binary_t {
@@ -461,18 +494,7 @@ struct dnnl_post_ops : public dnnl::impl::c_compatible {
                             && depthwise_conv.bias_dt
                                     == rhs.depthwise_conv.bias_dt
                             && depthwise_conv.dst_dt
-                                    == rhs.depthwise_conv.dst_dt
-                            && depthwise_conv.count == rhs.depthwise_conv.count
-                            && depthwise_conv.mask == rhs.depthwise_conv.mask;
-                    if (!ret) break;
-
-                    // only call memcmp with valid pointers
-                    if (depthwise_conv.count == 0) break;
-                    ret = !utils::any_null(depthwise_conv.scales,
-                                  rhs.depthwise_conv.scales)
-                            && !std::memcmp(depthwise_conv.scales,
-                                    rhs.depthwise_conv.scales,
-                                    sizeof(float) * depthwise_conv.count);
+                                    == rhs.depthwise_conv.dst_dt;
                     break;
                 case primitive_kind::binary:
                     ret = binary.alg == rhs.binary.alg
@@ -491,32 +513,19 @@ struct dnnl_post_ops : public dnnl::impl::c_compatible {
             return !this->operator==(rhs);
         }
 
-        ~entry_t() { clear(); }
-
     private:
-        void clear() {
-            if (is_convolution() && depthwise_conv.count
-                    && depthwise_conv.scales)
-                dnnl::impl::free(depthwise_conv.scales);
-            depthwise_conv.scales = nullptr;
-            return;
-        }
-
-        dnnl::impl::status_t set(const entry_t &other) {
-
+        void set(const entry_t &other) {
             // Copying by if (is_convolution()) {} else if(is_sum()) {}
             // else if(is_relu()) {} seems to be unreliable. memcpying for now.
             dnnl::impl::utils::array_copy(
                     (char *)this, (char *)&other, sizeof(*this));
-            if (other.is_convolution()) {
-                return set_depthwise_scales(other.depthwise_conv.scales);
-            }
-            return dnnl::impl::status::success;
         }
     };
 
     dnnl_post_ops() : entry_() {}
 
+    dnnl_post_ops(const dnnl_post_ops &other) { copy_from(other); }
+
     dnnl::impl::status_t append_sum(float scale, int32_t zero_point = 0,
             dnnl::impl::data_type_t dt = dnnl_data_type_undef);
     dnnl::impl::status_t append_eltwise(
@@ -524,12 +533,14 @@ struct dnnl_post_ops : public dnnl::impl::c_compatible {
     dnnl::impl::status_t append_dw(dnnl::impl::data_type_t wei_dt,
             dnnl::impl::data_type_t bias_dt, dnnl::impl::data_type_t dst_dt,
             dnnl::impl::dim_t kernel_size, dnnl::impl::dim_t stride_size,
-            dnnl::impl::dim_t padding_l_size, dnnl::impl::dim_t count, int mask,
-            const float *scales);
+            dnnl::impl::dim_t padding_l_size);
     dnnl::impl::status_t append_binary(dnnl::impl::alg_kind_t alg,
             const dnnl::impl::memory_desc_t *user_src1_desc);
     dnnl::impl::status_t append_prelu(int mask);
 
+    dnnl::impl::status_t prepend_binary(dnnl::impl::alg_kind_t alg,
+            const dnnl::impl::memory_desc_t *user_src1_desc);
+
     int find(dnnl::impl::primitive_kind_t kind, int start = 0,
             int stop = -1) const {
         if (stop == -1) stop = len();
@@ -576,7 +587,7 @@ struct dnnl_post_ops : public dnnl::impl::c_compatible {
         return ret;
     }
 
-    dnnl::impl::status_t copy_from(const dnnl_post_ops &other) {
+    void copy_from(const dnnl_post_ops &other) {
         using namespace dnnl::impl;
 
         for (int idx = 0; idx < other.len(); ++idx) {
@@ -585,17 +596,21 @@ struct dnnl_post_ops : public dnnl::impl::c_compatible {
             } else {
                 entry_.emplace_back();
             }
-            CHECK(entry_[idx].copy_from(other.entry_[idx]));
+            entry_[idx].copy_from(other.entry_[idx]);
         }
-
-        return status::success;
     }
 
+    bool is_initialized() const { return is_initialized_; }
+
     std::vector<entry_t> entry_;
 
     // Since binary post op accepts no more than 32 memory arguments by
     // design, we limit the amount of post-ops to 32.
     static constexpr int post_ops_limit = 32;
+
+private:
+    dnnl::impl::status_t validate_binary(dnnl::impl::alg_kind_t alg,
+            const dnnl::impl::memory_desc_t *user_src1_desc) const;
 };
 
 struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
@@ -615,17 +630,18 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
     dnnl::impl::status_t copy_from(const dnnl_primitive_attr &other) {
         using namespace dnnl::impl;
 
-        CHECK(output_scales_.copy_from(other.output_scales_));
-        CHECK(scales_.copy_from(other.scales_));
+        output_scales_ = other.output_scales_;
+        scales_ = other.scales_;
         zero_points_ = other.zero_points_;
         scratchpad_mode_ = other.scratchpad_mode_;
         fpmath_mode_ = other.fpmath_mode_;
-        CHECK(post_ops_.copy_from(other.post_ops_));
+        post_ops_.copy_from(other.post_ops_);
         rnn_data_qparams_ = other.rnn_data_qparams_;
         CHECK(rnn_weights_qparams_.copy_from(other.rnn_weights_qparams_));
         CHECK(rnn_weights_projection_qparams_.copy_from(
                 other.rnn_weights_projection_qparams_));
         CHECK(rnn_tparams_.copy_from(other.rnn_tparams_));
+        if (other.gpu_attr_) gpu_attr_ = other.gpu_attr_->clone();
 
         return status::success;
     }
@@ -635,7 +651,7 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
     enum class skip_mask_t : unsigned {
         none = 0,
         oscale = 1u << 0,
-        oscale_runtime = (unsigned)oscale | (1u << 1),
+        oscale_runtime = 1u << 1,
         scales = 1u << 2,
         scales_runtime = (unsigned)scales | (1u << 3),
         zero_points = 1u << 4,
@@ -645,7 +661,8 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
         rnn_weights_qparams = 1u << 8,
         rnn_tparams = 1u << 9,
         sum_dt = 1u << 10,
-        rnn_weights_projection_qparams = 1u << 11
+        rnn_weights_projection_qparams = 1u << 11,
+        gpu_attr = 1u << 12
     };
 
     /** Returns true if the attributes have default values.
@@ -667,7 +684,10 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
                 && rnn_weights_qparams_ == rhs.rnn_weights_qparams_
                 && rnn_weights_projection_qparams_
                         == rhs.rnn_weights_projection_qparams_
-                && rnn_tparams_ == rhs.rnn_tparams_;
+                && rnn_tparams_ == rhs.rnn_tparams_
+                && ((gpu_attr_ && rhs.gpu_attr_
+                            && gpu_attr_->is_equal(*rhs.gpu_attr_))
+                        || (!gpu_attr_ && !rhs.gpu_attr_));
         return ret;
     }
 
@@ -675,6 +695,8 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
     dnnl::impl::status_t set_scratchpad_mode(
             dnnl::impl::scratchpad_mode_t scratchpad_mode);
     dnnl::impl::status_t set_post_ops(const dnnl::impl::post_ops_t &post_ops);
+    dnnl::impl::status_t set_gpu_attr(
+            const dnnl::impl::primitive_attr_item_t &gpu_attr);
     dnnl::impl::status_t set_default_formats(
             const dnnl::impl::memory_desc_t *dst_md);
 
@@ -692,6 +714,8 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
                     return is_fpsubtype(data_type::bf16, dt_to);
                 case fpmath_mode::f16:
                     return is_fpsubtype(data_type::f16, dt_to);
+                case fpmath_mode::tf32:
+                    return is_fpsubtype(data_type::tf32, dt_to);
                 default: return false;
             }
         };
@@ -699,7 +723,7 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
     }
 
     // NOTE: make sure that the types below have overloaded comparison operator
-    dnnl::impl::scales_t output_scales_;
+    dnnl::impl::runtime_scales_t output_scales_;
     dnnl::impl::arg_scales_t scales_;
     dnnl::impl::zero_points_t zero_points_;
     dnnl::impl::scratchpad_mode_t scratchpad_mode_;
@@ -710,6 +734,8 @@ struct dnnl_primitive_attr : public dnnl::impl::c_compatible {
     dnnl::impl::scales_t rnn_weights_projection_qparams_;
     dnnl::impl::rnn_tparams_t rnn_tparams_;
 
+    std::unique_ptr<dnnl::impl::primitive_attr_item_t> gpu_attr_;
+
     dnnl_primitive_attr &operator=(const dnnl_primitive_attr &other) = delete;
 };
 
