@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -43,7 +43,16 @@ struct rnn_pd_t : public primitive_desc_t {
             case query::prop_kind:
                 *(prop_kind_t *)result = desc()->prop_kind;
                 break;
-            case query::rnn_d: *(const rnn_desc_t **)result = desc(); break;
+            case query::cell_kind:
+                *(alg_kind_t *)result = desc()->cell_kind;
+                break;
+            case query::activation_kind:
+                *(alg_kind_t *)result = desc()->activation_kind;
+                break;
+            case query::direction:
+                *(rnn_direction_t *)result = desc()->direction;
+                break;
+            case query::alpha_f32: *(float *)result = desc()->alpha; break;
             default: return primitive_desc_t::query(what, idx, result);
         }
         return status::success;
@@ -173,6 +182,10 @@ struct rnn_pd_t : public primitive_desc_t {
         return !memory_desc_wrapper(weights_projection_md_).is_zero();
     }
 
+    bool diff_weights_overwrite() const {
+        return desc_.flags & rnn_flags::diff_weights_overwrite;
+    }
+
     dnnl_rnn_direction_t direction() const { return desc_.direction; }
 
 protected:
