@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -42,22 +42,40 @@ enum class inner_blk_t {
     _,
     _4a,
     _4b,
+    _4c,
     _8a,
     _8b,
     _16a,
     _16b,
+    _16c,
+    _16d,
+    _16e,
+    _24a,
+    _24b,
+    _24c,
+    _24d,
+    _24e,
     _32a,
     _32b,
     _32c,
     _32d,
     _32e,
+    _48b,
+    _48c,
+    _64b,
+    _64c,
 
     _4a4b,
     _4b4a,
     _4b4c,
     _4c4b,
+    _8a2b,
     _8a8b,
     _8b8a,
+    _8b2c,
+    _8b16a,
+    _8b24a,
+    _8b32a,
     _8b8c,
     _8c8b,
     _16a16b,
@@ -72,22 +90,44 @@ enum class inner_blk_t {
     _32a32b,
     _16a2b,
     _16a4b,
+    _16b2a,
+    _16b4a,
     _16b2c,
     _16b4c,
+    _16c2b,
+    _16c4b,
+    _24a2b,
+    _24b2c,
     _32d4c,
     _32e2c,
     _32e4c,
+    _32b2a,
+    _32b4a,
+    _32c2b,
+    _32c4b,
     _64e2c,
     _64e4c,
     _32c2e,
+    _48c2b,
+    _48c4b,
+    _48b2a,
+    _48b4a,
+    _64b2a,
+    _64b4a,
+    _64c2b,
+    _64c4b,
 
     _2c8b4c,
     _8a16b2a,
+    _4b8a4b,
     _4b16a4b,
+    _4b24a4b,
     _4b32a4b,
     _4b64a4b,
     _2b8a4b,
+    _8b8a2b,
     _8b16a2b,
+    _8b24a2b,
     _8b32a2b,
     _8b64a2b,
     _8b16c2b,
@@ -127,6 +167,16 @@ enum class inner_blk_t {
     _16c32b4c,
     _16c48b4c,
     _16c64b4c,
+    _16b32c,
+    _16b48c,
+    _16b64c,
+    _16b32c2b,
+    _16b48c2b,
+    _16b64c2b,
+    _16b16c4b,
+    _16b32c4b,
+    _16b48c4b,
+    _16b64c4b,
 };
 
 /** returns the offset within the block for weights blocked over oc and ic */
@@ -135,12 +185,14 @@ constexpr int AB_or_BC_blk_off(int x0, int x1) {
     using ib = inner_blk_t;
     static_assert(
             utils::one_of(f, ib::_4a4b, ib::_4b4a, ib::_4b4c, ib::_4c4b,
-                    ib::_8a8b, ib::_8b8a, ib::_8b8c, ib::_8c8b, ib::_16a16b,
+                    ib::_8a2b, ib::_8b2c, ib::_8a8b, ib::_8b8a, ib::_8b16a,
+                    ib::_8b24a, ib::_8b32a, ib::_8b8c, ib::_8c8b, ib::_16a16b,
                     ib::_16b64a, ib::_16b32a, ib::_16b16a, ib::_16b16c,
                     ib::_16c16b, ib::_32a32b, ib::_16a2b, ib::_16a4b,
                     ib::_16b2c, ib::_16b4c, ib::_2c8b4c, ib::_8a16b2a,
-                    ib::_4b64a4b, ib::_4b32a4b, ib::_4b16a4b, ib::_2b8a4b,
-                    ib::_8b64a2b, ib::_8b32a2b, ib::_8b16a2b, ib::_8b16c2b,
+                    ib::_4b64a4b, ib::_4b32a4b, ib::_4b24a4b, ib::_4b16a4b,
+                    ib::_4b8a4b, ib::_2b8a4b, ib::_8b64a2b, ib::_8b32a2b,
+                    ib::_8b24a2b, ib::_8b16a2b, ib::_8b8a2b, ib::_8b16c2b,
                     ib::_4c16b4c, ib::_8c16b2c, ib::_2b4c2b, ib::_2c4b2c,
                     ib::_4b8c2b, ib::_4c8b2c, ib::_16a32b, ib::_16a48b,
                     ib::_16a64b, ib::_16a16b2a, ib::_16a32b2a, ib::_16a48b2a,
@@ -150,7 +202,10 @@ constexpr int AB_or_BC_blk_off(int x0, int x1) {
                     ib::_4a8b8a4b, ib::_4b8c8b4c, ib::_16b32a2b, ib::_16b48a2b,
                     ib::_16b64a2b, ib::_16b32a4b, ib::_16b48a4b, ib::_16b64a4b,
                     ib::_16c32b2c, ib::_16c48b2c, ib::_16c64b2c, ib::_16c32b4c,
-                    ib::_16c48b4c, ib::_16c64b4c),
+                    ib::_16c48b4c, ib::_16c64b4c, ib::_16b32c, ib::_16b48c,
+                    ib::_16b64c, ib::_16b32c2b, ib::_16b48c2b, ib::_16b64c2b,
+                    ib::_16b16c4b, ib::_16b32c4b, ib::_16b48c4b, ib::_16b64c4b,
+                    ib::_24a2b, ib::_24b2c),
             "unexpected inner_blk format");
 
     // clang-format off
@@ -161,28 +216,33 @@ constexpr int AB_or_BC_blk_off(int x0, int x1) {
         : (f == ib::_8b8a || f == ib::_8c8b) ? 8 * x1 + x0
         : (f == ib::_16a16b || f == ib::_16b16c) ? 16 * x0 + x1
         : (f == ib::_16b64a) ? 64 * x1 + x0
-        : (f == ib::_16b32a) ? 32 * x1 + x0
-        : (f == ib::_16b16a || f == ib::_16c16b) ? 16 * x1 + x0
-        : (f == ib::_16a2b || f == ib::_16b2c) ? 2 * x0 + x1
+        : (f == ib::_8b32a || f == ib::_16b32a) ? 32 * x1 + x0
+        : (f == ib::_8b24a) ? 24 * x1 + x0
+        : (f == ib::_8b16a || f == ib::_16b16a || f == ib::_16c16b) ? 16 * x1 + x0
+        : (f == ib::_8a2b || f == ib::_8b2c || f == ib::_16a2b || f == ib::_16b2c || f == ib::_24a2b || f == ib::_24b2c ) ? 2 * x0 + x1
         : (f == ib::_16a4b || f == ib::_16b4c) ? 4 * x0 + x1
-        : (f == ib::_32a32b || f == ib::_16a32b) ? 32 * x0 + x1
+        : (utils::one_of(f, ib::_32a32b, ib::_16a32b, ib::_16b32c)) ? 32 * x0 + x1
         : (utils::one_of(f, ib::_8a16b2a, ib::_8b16c2b, ib::_16a16b2a, ib::_16b16c2b)) ? (x0 / 2) * 32 + x1 * 2 + x0 % 2
-        : (f == ib::_16a48b) ? x0 * 48 + x1
-        : (f == ib::_16a64b) ? x0 * 64 + x1
-        : (f == ib::_16a32b2a) ? (x0 / 2) * 64 + x1 * 2 + x0 % 2
-        : (f == ib::_16a48b2a) ? (x0 / 2) * 96 + x1 * 2 + x0 % 2
-        : (f == ib::_16a64b2a) ? (x0 / 2) * 128 + x1 * 2 + x0 % 2
-        : (f == ib::_16a16b4a) ? (x0 / 4) * 64 + x1 * 4 + x0 % 4
-        : (f == ib::_16a32b4a) ? (x0 / 4) * 128 + x1 * 4 + x0 % 4
-        : (f == ib::_16a48b4a) ? (x0 / 4) * 192 + x1 * 4 + x0 % 4
-        : (f == ib::_16a64b4a) ? (x0 / 4) * 256 + x1 * 4 + x0 % 4
+        : (utils::one_of(f, ib::_16a48b, ib::_16b48c)) ? x0 * 48 + x1
+        : (utils::one_of(f, ib::_16a64b, ib::_16b64c)) ? x0 * 64 + x1
+        : (utils::one_of(f, ib::_16a32b2a, ib::_16b32c2b)) ? (x0 / 2) * 64 + x1 * 2 + x0 % 2
+        : (utils::one_of(f, ib::_16a48b2a, ib::_16b48c2b)) ? (x0 / 2) * 96 + x1 * 2 + x0 % 2
+        : (utils::one_of(f, ib::_16a64b2a, ib::_16b64c2b)) ? (x0 / 2) * 128 + x1 * 2 + x0 % 2
+        : (utils::one_of(f, ib::_16a16b4a, ib::_16b16c4b)) ? (x0 / 4) * 64 + x1 * 4 + x0 % 4
+        : (utils::one_of(f, ib::_16a32b4a, ib::_16b32c4b)) ? (x0 / 4) * 128 + x1 * 4 + x0 % 4
+        : (utils::one_of(f, ib::_16a48b4a, ib::_16b48c4b)) ? (x0 / 4) * 192 + x1 * 4 + x0 % 4
+        : (utils::one_of(f, ib::_16a64b4a, ib::_16b64c4b)) ? (x0 / 4) * 256 + x1 * 4 + x0 % 4
+        : (f == ib::_4b8a4b) ? (x1 / 4) * 32 + x0 * 4 + x1 % 4
         : (f == ib::_4b16a4b || f == ib::_4c16b4c) ? (x1 / 4) * 64 + x0 * 4 + x1 % 4
+        : (f == ib::_4b24a4b) ? (x1 / 4) * 96 + x0 * 4 + x1 % 4
         : (f == ib::_4b32a4b) ? (x1 / 4) * 128 + x0 * 4 + x1 % 4
         : (f == ib::_4b64a4b) ? (x1 / 4) * 256 + x0 * 4 + x1 % 4
         : (f == ib::_2b8a4b || f == ib::_2c8b4c) ? (x1 / 4) * 32 + x0 * 4 + x1 % 4
         : (f == ib::_16b16a2b || f == ib::_16c16b2c) ? (x1 / 2) * 32 + x0 * 2 + x1 % 2
         : (f == ib::_16b16a4b || f == ib::_16c16b4c) ? (x1 / 4) * 64 + x0 * 4 + x1 % 4
+        : (f == ib::_8b8a2b) ? (x1 / 2) * 16 + x0 * 2 + x1 % 2
         : (f == ib::_8b16a2b || f == ib::_8c16b2c) ? (x1 / 2) * 32 + x0 * 2 + x1 % 2
+        : (f == ib::_8b24a2b) ? (x1 / 2) * 48 + x0 * 2 + x1 % 2
         : (f == ib::_8b32a2b) ? (x1 / 2) * 64 + x0 * 2 + x1 % 2
         : (f == ib::_8b64a2b) ? (x1 / 2) * 128 + x0 * 2 + x1 % 2
         : (f == ib::_2b4c2b || f == ib::_2c4b2c) ? (x0 / 2) * 8 + x1 * 2 + x0 % 2
@@ -312,7 +372,9 @@ DECL_TRAITS(AB16b64a, _AB, _16b64a, 2);
 DECL_TRAITS(AB8b16a2b, _AB, _8b16a2b, 2);
 DECL_TRAITS(AB8b32a2b, _AB, _8b32a2b, 2);
 DECL_TRAITS(AB8b64a2b, _AB, _8b64a2b, 2);
+DECL_TRAITS(AB4b8a4b, _AB, _4b8a4b, 2);
 DECL_TRAITS(AB4b16a4b, _AB, _4b16a4b, 2);
+DECL_TRAITS(AB4b24a4b, _AB, _4b24a4b, 2);
 DECL_TRAITS(AB4b32a4b, _AB, _4b32a4b, 2);
 DECL_TRAITS(AB4b64a4b, _AB, _4b64a4b, 2);
 DECL_TRAITS(AB16b16a4b, _AB, _16b16a4b, 2);
@@ -336,6 +398,139 @@ DECL_TRAITS(BA16a16b4a, _AB, _16a16b4a, 2);
 DECL_TRAITS(BA16a32b4a, _AB, _16a32b4a, 2);
 DECL_TRAITS(BA16a48b4a, _AB, _16a48b4a, 2);
 DECL_TRAITS(BA16a64b4a, _AB, _16a64b4a, 2);
+DECL_TRAITS(aCB16b16c, _BC, _16b16c, 2);
+DECL_TRAITS(aCB16b32c, _BC, _16b32c, 2);
+DECL_TRAITS(aCB16b48c, _BC, _16b48c, 2);
+DECL_TRAITS(aCB16b64c, _BC, _16b64c, 2);
+DECL_TRAITS(aCB16b16c2b, _BC, _16b16c2b, 3);
+DECL_TRAITS(aCB16b32c2b, _BC, _16b32c2b, 3);
+DECL_TRAITS(aCB16b48c2b, _BC, _16b48c2b, 3);
+DECL_TRAITS(aCB16b64c2b, _BC, _16b64c2b, 3);
+DECL_TRAITS(aCB16b16c4b, _BC, _16b16c4b, 3);
+DECL_TRAITS(aCB16b32c4b, _BC, _16b32c4b, 3);
+DECL_TRAITS(aCB16b48c4b, _BC, _16b48c4b, 3);
+DECL_TRAITS(aCB16b64c4b, _BC, _16b64c4b, 3);
+
+DECL_TRAITS(aCdefB16b32c2b, _BC, _16b32c2b, 5);
+DECL_TRAITS(aCdefB16b32c4b, _BC, _16b32c4b, 5);
+DECL_TRAITS(aCdefB16b48c2b, _BC, _16b48c2b, 5);
+DECL_TRAITS(aCdefB16b48c4b, _BC, _16b48c4b, 5);
+DECL_TRAITS(aCdefB16b64c2b, _BC, _16b64c2b, 5);
+DECL_TRAITS(aCdefB16b64c4b, _BC, _16b64c4b, 5);
+DECL_TRAITS(BcdeA16a32b2a, _AB, _16a32b2a, 5);
+DECL_TRAITS(BcdeA16a32b4a, _AB, _16a32b4a, 5);
+DECL_TRAITS(BcdeA16a48b2a, _AB, _16a48b2a, 5);
+DECL_TRAITS(BcdeA16a48b4a, _AB, _16a48b4a, 5);
+DECL_TRAITS(BcdeA16a64b2a, _AB, _16a64b2a, 5);
+DECL_TRAITS(BcdeA16a64b4a, _AB, _16a64b4a, 5);
+DECL_TRAITS(aCdefb32c, _C, _32c, 5);
+DECL_TRAITS(aCdefB32c2b, _BC, _32c2b, 5);
+DECL_TRAITS(aCdefB32c4b, _BC, _32c4b, 5);
+DECL_TRAITS(aCdefb48c, _C, _48c, 5);
+DECL_TRAITS(aCdefB48c2b, _BC, _48c2b, 5);
+DECL_TRAITS(aCdefB48c4b, _BC, _48c4b, 5);
+DECL_TRAITS(aCdefb64c, _C, _64c, 5);
+DECL_TRAITS(aCdefB64c2b, _BC, _64c2b, 5);
+DECL_TRAITS(aCdefB64c4b, _BC, _64c4b, 5);
+DECL_TRAITS(Bcdea32b, _B, _32b, 5);
+DECL_TRAITS(BcdeA32b2a, _AB, _32b2a, 5);
+DECL_TRAITS(BcdeA32b4a, _AB, _32b4a, 5);
+DECL_TRAITS(Bcdea48b, _B, _48b, 5);
+DECL_TRAITS(BcdeA48b2a, _AB, _48b2a, 5);
+DECL_TRAITS(BcdeA48b4a, _AB, _48b4a, 5);
+DECL_TRAITS(Bcdea64b, _B, _64b, 5);
+DECL_TRAITS(BcdeA64b2a, _AB, _64b2a, 5);
+DECL_TRAITS(BcdeA64b4a, _AB, _64b4a, 5);
+DECL_TRAITS(Bca32b, _B, _32b, 3);
+DECL_TRAITS(BcA32b2a, _AB, _32b2a, 3);
+DECL_TRAITS(BcA32b4a, _AB, _32b4a, 3);
+DECL_TRAITS(Bca48b, _B, _48b, 3);
+DECL_TRAITS(BcA48b2a, _AB, _48b2a, 3);
+DECL_TRAITS(BcA48b4a, _AB, _48b4a, 3);
+DECL_TRAITS(Bca64b, _B, _64b, 3);
+DECL_TRAITS(BcA64b2a, _AB, _64b2a, 3);
+DECL_TRAITS(BcA64b4a, _AB, _64b4a, 3);
+DECL_TRAITS(aCdb32c, _C, _32c, 4);
+DECL_TRAITS(aCdB32c2b, _BC, _32c2b, 4);
+DECL_TRAITS(aCdB32c4b, _BC, _32c4b, 4);
+DECL_TRAITS(aCdb48c, _C, _48c, 4);
+DECL_TRAITS(aCdB48c2b, _BC, _48c2b, 4);
+DECL_TRAITS(aCdB48c4b, _BC, _48c4b, 4);
+DECL_TRAITS(aCdb64c, _C, _64c, 4);
+DECL_TRAITS(aCdB64c2b, _BC, _64c2b, 4);
+DECL_TRAITS(aCdB64c4b, _BC, _64c4b, 4);
+DECL_TRAITS(BcA16a16b2a, _AB, _16a16b2a, 3);
+DECL_TRAITS(BcA16a16b4a, _AB, _16a16b4a, 3);
+DECL_TRAITS(BcdA16a16b2a, _AB, _16a16b2a, 4);
+DECL_TRAITS(BcdA16a16b4a, _AB, _16a16b4a, 4);
+DECL_TRAITS(BcdeA16a16b2a, _AB, _16a16b2a, 5);
+DECL_TRAITS(BcdeA16a16b4a, _AB, _16a16b4a, 5);
+DECL_TRAITS(aCdB16b16c2b, _BC, _16b16c2b, 4);
+DECL_TRAITS(aCdB16b16c4b, _BC, _16b16c4b, 4);
+DECL_TRAITS(aCdeB16b16c2b, _BC, _16b16c2b, 5);
+DECL_TRAITS(aCdeB16b16c4b, _BC, _16b16c4b, 5);
+DECL_TRAITS(aCdefB16b16c2b, _BC, _16b16c2b, 5);
+DECL_TRAITS(aCdefB16b16c4b, _BC, _16b16c4b, 5);
+DECL_TRAITS(BcA16a32b2a, _AB, _16a32b2a, 3);
+DECL_TRAITS(BcA16a32b4a, _AB, _16a32b4a, 3);
+DECL_TRAITS(BcA16a48b2a, _AB, _16a48b2a, 3);
+DECL_TRAITS(BcA16a48b4a, _AB, _16a48b4a, 3);
+DECL_TRAITS(BcA16a64b2a, _AB, _16a64b2a, 3);
+DECL_TRAITS(BcA16a64b4a, _AB, _16a64b4a, 3);
+DECL_TRAITS(aCdB16b32c2b, _BC, _16b32c2b, 4);
+DECL_TRAITS(aCdB16b32c4b, _BC, _16b32c4b, 4);
+DECL_TRAITS(aCdB16b48c2b, _BC, _16b48c2b, 4);
+DECL_TRAITS(aCdB16b48c4b, _BC, _16b48c4b, 4);
+DECL_TRAITS(aCdB16b64c2b, _BC, _16b64c2b, 4);
+DECL_TRAITS(aCdB16b64c4b, _BC, _16b64c4b, 4);
+DECL_TRAITS(BcdA16a32b2a, _AB, _16a32b2a, 4);
+DECL_TRAITS(BcdA16a32b4a, _AB, _16a32b4a, 4);
+DECL_TRAITS(BcdA16a48b2a, _AB, _16a48b2a, 4);
+DECL_TRAITS(BcdA16a48b4a, _AB, _16a48b4a, 4);
+DECL_TRAITS(BcdA16a64b2a, _AB, _16a64b2a, 4);
+DECL_TRAITS(BcdA16a64b4a, _AB, _16a64b4a, 4);
+DECL_TRAITS(aCdeB16b32c2b, _BC, _16b32c2b, 5);
+DECL_TRAITS(aCdeB16b32c4b, _BC, _16b32c4b, 5);
+DECL_TRAITS(aCdeB16b48c2b, _BC, _16b48c2b, 5);
+DECL_TRAITS(aCdeB16b48c4b, _BC, _16b48c4b, 5);
+DECL_TRAITS(aCdeB16b64c2b, _BC, _16b64c2b, 5);
+DECL_TRAITS(aCdeB16b64c4b, _BC, _16b64c4b, 5);
+DECL_TRAITS(Bca16b, _B, _16b, 3);
+DECL_TRAITS(BcA16b2a, _AB, _16b2a, 3);
+DECL_TRAITS(BcA16b4a, _AB, _16b4a, 3);
+DECL_TRAITS(Bcda16b, _B, _16b, 4);
+DECL_TRAITS(BcdA16b2a, _AB, _16b2a, 4);
+DECL_TRAITS(BcdA16b4a, _AB, _16b4a, 4);
+DECL_TRAITS(Bcdea16b, _B, _16b, 5);
+DECL_TRAITS(BcdeA16b2a, _AB, _16b2a, 5);
+DECL_TRAITS(BcdeA16b4a, _AB, _16b4a, 5);
+DECL_TRAITS(aCdb16c, _C, _16c, 4);
+DECL_TRAITS(aCdB16c2b, _BC, _16c2b, 4);
+DECL_TRAITS(aCdB16c4b, _BC, _16c4b, 4);
+DECL_TRAITS(aCdeb16c, _C, _16c, 5);
+DECL_TRAITS(aCdeB16c2b, _BC, _16c2b, 5);
+DECL_TRAITS(aCdeB16c4b, _BC, _16c4b, 5);
+DECL_TRAITS(aCdefb16c, _C, _16c, 6);
+DECL_TRAITS(aCdefB16c2b, _BC, _16c2b, 6);
+DECL_TRAITS(aCdefB16c4b, _BC, _16c4b, 6);
+DECL_TRAITS(Bcda32b, _B, _32b, 4);
+DECL_TRAITS(BcdA32b2a, _AB, _32b2a, 4);
+DECL_TRAITS(BcdA32b4a, _AB, _32b4a, 4);
+DECL_TRAITS(Bcda48b, _B, _48b, 4);
+DECL_TRAITS(BcdA48b2a, _AB, _48b2a, 4);
+DECL_TRAITS(BcdA48b4a, _AB, _48b4a, 4);
+DECL_TRAITS(Bcda64b, _B, _64b, 4);
+DECL_TRAITS(BcdA64b2a, _AB, _64b2a, 4);
+DECL_TRAITS(BcdA64b4a, _AB, _64b4a, 4);
+DECL_TRAITS(aCdeb32c, _C, _32c, 5);
+DECL_TRAITS(aCdeB32c2b, _BC, _32c2b, 5);
+DECL_TRAITS(aCdeB32c4b, _BC, _32c4b, 5);
+DECL_TRAITS(aCdeb48c, _C, _48c, 5);
+DECL_TRAITS(aCdeB48c2b, _BC, _48c2b, 5);
+DECL_TRAITS(aCdeB48c4b, _BC, _48c4b, 5);
+DECL_TRAITS(aCdeb64c, _C, _64c, 5);
+DECL_TRAITS(aCdeB64c2b, _BC, _64c2b, 5);
+DECL_TRAITS(aCdeB64c4b, _BC, _64c4b, 5);
 
 DECL_TRAITS(Abc16a, _A, _16a, 3);
 DECL_TRAITS(ABc16a16b, _AB, _16a16b, 3);
@@ -441,30 +636,36 @@ DECL_TRAITS(aBdc16b, _B, _16b, 4);
 DECL_TRAITS(aBdC16b2c, _BC, _16b2c, 4);
 DECL_TRAITS(aBdC16b4c, _BC, _16b4c, 4);
 DECL_TRAITS(aBdc8b, _B, _8b, 4);
+DECL_TRAITS(aBdC8b2c, _BC, _8b2c, 4);
 DECL_TRAITS(aBdec16b, _B, _16b, 5);
 DECL_TRAITS(aBdeC16b2c, _BC, _16b2c, 5);
 DECL_TRAITS(aBdeC16b4c, _BC, _16b4c, 5);
 DECL_TRAITS(aBdec8b, _B, _8b, 5);
+DECL_TRAITS(aBdeC8b2c, _BC, _8b2c, 5);
 DECL_TRAITS(aBdefc16b, _B, _16b, 6);
 DECL_TRAITS(aBdefC16b2c, _BC, _16b2c, 6);
 DECL_TRAITS(aBdefC16b4c, _BC, _16b4c, 6);
 DECL_TRAITS(aBdefc8b, _B, _8b, 6);
+DECL_TRAITS(aBdefC8b2c, _BC, _8b2c, 6);
 DECL_TRAITS(aBdfec16b, _B, _16b, 6);
 DECL_TRAITS(aBedc16b, _B, _16b, 5);
 DECL_TRAITS(Acb16a, _A, _16a, 3);
 DECL_TRAITS(AcB16a2b, _AB, _16a2b, 3);
 DECL_TRAITS(AcB16a4b, _AB, _16a4b, 3);
 DECL_TRAITS(Acb8a, _A, _8a, 3);
+DECL_TRAITS(AcB8a2b, _AB, _8a2b, 3);
 DECL_TRAITS(aCBd16b16c, _BC, _16b16c, 4);
 DECL_TRAITS(aCBde16b16c, _BC, _16b16c, 5);
 DECL_TRAITS(Acdb16a, _A, _16a, 4);
 DECL_TRAITS(AcdB16a2b, _AB, _16a2b, 4);
 DECL_TRAITS(AcdB16a4b, _AB, _16a4b, 4);
 DECL_TRAITS(Acdb8a, _A, _8a, 4);
+DECL_TRAITS(AcdB8a2b, _AB, _8a2b, 4);
 DECL_TRAITS(Acdeb16a, _A, _16a, 5);
 DECL_TRAITS(AcdeB16a2b, _AB, _16a2b, 5);
 DECL_TRAITS(AcdeB16a4b, _AB, _16a4b, 5);
 DECL_TRAITS(Acdeb8a, _A, _8a, 5);
+DECL_TRAITS(AcdeB8a2b, _AB, _8a2b, 5);
 DECL_TRAITS(Acedb16a, _A, _16a, 5);
 DECL_TRAITS(Adcb16a, _A, _16a, 4);
 DECL_TRAITS(BAc16a16b, _AB, _16a16b, 3);
@@ -497,17 +698,26 @@ DECL_TRAITS(aBCdef4b8c2b, _BC, _4b8c2b, 6);
 DECL_TRAITS(aBCd4c8b2c, _BC, _4c8b2c, 4);
 DECL_TRAITS(aBCde4c8b2c, _BC, _4c8b2c, 5);
 DECL_TRAITS(aBCdef4c8b2c, _BC, _4c8b2c, 6);
+DECL_TRAITS(abDc16d, _D, _16d, 4);
 DECL_TRAITS(abDc32d, _D, _32d, 4);
 DECL_TRAITS(abDC32d4c, _CD, _32d4c, 4);
 DECL_TRAITS(abCd32c, _C, _32c, 4);
+DECL_TRAITS(abCde32c, _C, _32c, 5);
+DECL_TRAITS(abCdef32c, _C, _32c, 6);
+DECL_TRAITS(abCd4c, _C, _4c, 4);
+DECL_TRAITS(abCde4c, _C, _4c, 5);
+DECL_TRAITS(abCdef4c, _C, _4c, 6);
+DECL_TRAITS(abdEc16e, _E, _16e, 5);
 DECL_TRAITS(abdEc32e, _E, _32e, 5);
 DECL_TRAITS(abdEC32e2c, _CE, _32e2c, 5);
 DECL_TRAITS(abdEC32e4c, _CE, _32e4c, 5);
 DECL_TRAITS(abdEC64e2c, _CE, _64e2c, 5);
 DECL_TRAITS(abdEC64e4c, _CE, _64e4c, 5);
+DECL_TRAITS(abdCe16c, _C, _16c, 5);
 DECL_TRAITS(abdCe32c, _C, _32c, 5);
 DECL_TRAITS(abdCE32c2e, _CE, _32c2e, 5);
 DECL_TRAITS(decbA16a, _A, _16a, 5);
+DECL_TRAITS(decbA8a, _A, _8a, 5);
 
 DECL_TRAITS(aBdC16c16b2c, _BC, _16c16b2c, 4);
 DECL_TRAITS(aBdC16c16b4c, _BC, _16c16b4c, 4);
@@ -559,6 +769,41 @@ DECL_TRAITS(AcdeB16b48a4b, _AB, _16b48a4b, 5);
 DECL_TRAITS(AcdeB16b64a2b, _AB, _16b64a2b, 5);
 DECL_TRAITS(AcdeB16b64a4b, _AB, _16b64a4b, 5);
 
+DECL_TRAITS(Acb24a, _A, _24a, 3);
+DECL_TRAITS(Acdb24a, _A, _24a, 4);
+DECL_TRAITS(Acdeb24a, _A, _24a, 5);
+DECL_TRAITS(aBdc24b, _B, _24b, 4);
+DECL_TRAITS(aBdec24b, _B, _24b, 5);
+DECL_TRAITS(aBdefc24b, _B, _24b, 6);
+DECL_TRAITS(AcB24a2b, _AB, _24a2b, 3);
+DECL_TRAITS(AcdB24a2b, _AB, _24a2b, 4);
+DECL_TRAITS(AcdeB24a2b, _AB, _24a2b, 5);
+DECL_TRAITS(aBdC24b2c, _BC, _24b2c, 4);
+DECL_TRAITS(aBdeC24b2c, _BC, _24b2c, 5);
+DECL_TRAITS(aBdefC24b2c, _BC, _24b2c, 6);
+
+DECL_TRAITS(AB8b32a, _AB, _8b32a, 2);
+DECL_TRAITS(ABc8b32a, _AB, _8b32a, 3);
+DECL_TRAITS(ABcd8b32a, _AB, _8b32a, 4);
+DECL_TRAITS(ABcde8b32a, _AB, _8b32a, 5);
+DECL_TRAITS(AB8b24a, _AB, _8b24a, 2);
+DECL_TRAITS(ABc8b24a, _AB, _8b24a, 3);
+DECL_TRAITS(ABcd8b24a, _AB, _8b24a, 4);
+DECL_TRAITS(ABcde8b24a, _AB, _8b24a, 5);
+DECL_TRAITS(AB8b16a, _AB, _8b16a, 2);
+DECL_TRAITS(ABc8b16a, _AB, _8b16a, 3);
+DECL_TRAITS(ABcd8b16a, _AB, _8b16a, 4);
+DECL_TRAITS(ABcde8b16a, _AB, _8b16a, 5);
+DECL_TRAITS(AB8b8a, _AB, _8b8a, 2);
+
+DECL_TRAITS(AB8b8a2b, _AB, _8b8a2b, 2);
+DECL_TRAITS(ABc8b8a2b, _AB, _8b8a2b, 3);
+DECL_TRAITS(ABcd8b8a2b, _AB, _8b8a2b, 4);
+DECL_TRAITS(ABcde8b8a2b, _AB, _8b8a2b, 5);
+DECL_TRAITS(AB8b24a2b, _AB, _8b24a2b, 2);
+DECL_TRAITS(ABc8b24a2b, _AB, _8b24a2b, 3);
+DECL_TRAITS(ABcd8b24a2b, _AB, _8b24a2b, 4);
+DECL_TRAITS(ABcde8b24a2b, _AB, _8b24a2b, 5);
 } // namespace impl
 } // namespace dnnl
 
