@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2022 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -27,6 +27,7 @@
 #include "c_types_map.hpp"
 #include "dnnl_traits.hpp"
 #include "math_utils.hpp"
+#include "memory_desc.hpp"
 #include "nstl.hpp"
 #include "utils.hpp"
 
@@ -81,10 +82,12 @@ namespace types {
 
 inline size_t data_type_size(data_type_t data_type) {
     using namespace data_type;
-    switch (data_type) {
+    switch ((int)data_type) {
         case f16: return sizeof(prec_traits<f16>::type);
         case bf16: return sizeof(prec_traits<bf16>::type);
+        case tf32: // the tf32 type is an f32
         case f32: return sizeof(prec_traits<f32>::type);
+        case f64: return sizeof(prec_traits<f64>::type);
         case s32: return sizeof(prec_traits<s32>::type);
         case s8: return sizeof(prec_traits<s8>::type);
         case u8: return sizeof(prec_traits<u8>::type);
@@ -156,15 +159,28 @@ inline format_kind_t format_tag_to_kind(format_tag_t tag) {
     return format_kind::undef;
 }
 
+// Currently rnn_s8s8_compensation has common bits with rnn_u8s8_compensation
+// and scale_adjust constants so we have to perform additional checks to
+// separate these two cases
+inline bool extra_flag_rnn_s8s8_compensation_is_set(uint64_t flags) {
+    return ((flags & memory_extra_flags::rnn_s8s8_compensation)
+                   ^ memory_extra_flags::rnn_s8s8_compensation)
+            == 0;
+}
+
 inline bool memory_extra_desc_is_equal(
         const memory_extra_desc_t &lhs, const memory_extra_desc_t &rhs) {
     using namespace memory_extra_flags;
     return true && lhs.flags == rhs.flags
             && IMPLICATION(lhs.flags & compensation_conv_s8s8,
                     lhs.compensation_mask == rhs.compensation_mask)
-            && IMPLICATION(lhs.flags & rnn_u8s8_compensation,
+            && IMPLICATION((lhs.flags & rnn_u8s8_compensation)
+                            && !extra_flag_rnn_s8s8_compensation_is_set(
+                                    lhs.flags),
                     lhs.compensation_mask == rhs.compensation_mask)
-            && IMPLICATION(lhs.flags & scale_adjust,
+            && IMPLICATION((lhs.flags & scale_adjust)
+                            && !extra_flag_rnn_s8s8_compensation_is_set(
+                                    lhs.flags),
                     lhs.scale_adjust == rhs.scale_adjust)
             && IMPLICATION(lhs.flags & compensation_conv_asymmetric_src,
                     lhs.asymm_compensation_mask == rhs.asymm_compensation_mask);
@@ -217,6 +233,17 @@ inline bool rnn_packed_desc_is_equal(
     return ok;
 }
 
+inline bool sparse_desc_is_equal(
+        const sparse_desc_t &lhs, const sparse_desc_t &rhs) {
+    bool ok = lhs.encoding == rhs.encoding && lhs.nnz == rhs.nnz;
+    if (!ok) return false;
+
+    for (int i = 0; i < sparse_desc_t::max_metadata_types; i++)
+        ok = ok && lhs.metadata_types[i] == rhs.metadata_types[i];
+
+    return ok;
+}
+
 inline memory_desc_t zero_md() {
     auto zero = memory_desc_t();
     return zero;
@@ -227,15 +254,19 @@ inline bool is_zero_md(const memory_desc_t *md) {
 }
 
 inline data_type_t default_accum_data_type(
-        data_type_t src_dt, data_type_t dst_dt) {
+        data_type_t src_dt, data_type_t dst_dt, bool strict = true) {
     using namespace utils;
     using namespace data_type;
 
-    if (one_of(src_dt, s8, u8) && dst_dt != f32) return s32;
+    // we allow to use f32 accumulation type only when the
+    // accumulation chain is small. Otherwise, strict should be set to
+    // true
+    if (one_of(src_dt, s8, u8) && (dst_dt != f32 || strict)) return s32;
 
-    if (one_of(f16, src_dt, dst_dt)) return f16;
+    if (one_of(f16, src_dt, dst_dt)) return f32;
     if (one_of(bf16, src_dt, dst_dt)) return f32;
     if (one_of(f32, src_dt, dst_dt)) return f32;
+    if (one_of(f64, src_dt, dst_dt)) return f64;
     if (one_of(s32, src_dt, dst_dt)) return s32;
 
     if (one_of(s8, src_dt, dst_dt) || one_of(u8, src_dt, dst_dt)) return s32;
@@ -250,27 +281,79 @@ inline data_type_t default_accum_data_type(data_type_t src_dt,
     using namespace prop_kind;
 
     /* prop_kind doesn't matter */
-    if (everyone_is(f16, src_dt, wei_dt) && one_of(dst_dt, f16, f32, s8, u8))
-        return f16;
     if (everyone_is(f32, src_dt, wei_dt)) return f32;
+    if (everyone_is(f64, src_dt, wei_dt)) return f64;
 
     if (one_of(prop_kind, forward_training, forward_inference)) {
         if ((src_dt == u8 || src_dt == s8) && wei_dt == s8) return s32;
+        if (one_of(f16, src_dt, wei_dt)) return f32;
     } else if (prop_kind == backward_data) {
         if (one_of(src_dt, f32, s32, s8, u8) && wei_dt == s8
                 && one_of(dst_dt, s8, u8, s32))
             return s32;
-        if (everyone_is(f16, dst_dt, wei_dt) && one_of(src_dt, s8, u8))
-            return f16;
+        if (one_of(f16, dst_dt, wei_dt)) return f32;
         if (everyone_is(f32, dst_dt, wei_dt) && one_of(src_dt, s8, u8))
             return f32;
     }
 
     if (one_of(bf16, src_dt, wei_dt, dst_dt)) return f32;
+    if (one_of(f16, src_dt, wei_dt, dst_dt)) return f32;
 
     return data_type::undef;
 }
 
+inline bool is_integral_dt(data_type_t dt) {
+    using namespace data_type;
+    return utils::one_of(dt, s32, s8, u8);
+}
+
+template <typename data_t>
+inline void cvt_from_float(data_t *out, const float *inp, size_t nelems) {
+    assert(!"unimplemented");
+}
+
+template <typename data_t>
+inline void cvt_to_float(float *out, const data_t *inp, size_t nelems) {
+    assert(!"unimplemented");
+}
+
+template <>
+inline void cvt_from_float<bfloat16_t>(
+        bfloat16_t *out, const float *inp, size_t nelems) {
+    cvt_float_to_bfloat16(out, inp, nelems);
+}
+
+template <>
+inline void cvt_to_float<bfloat16_t>(
+        float *out, const bfloat16_t *inp, size_t nelems) {
+    cvt_bfloat16_to_float(out, inp, nelems);
+}
+
+template <>
+inline void cvt_from_float<float16_t>(
+        float16_t *out, const float *inp, size_t nelems) {
+    cvt_float_to_float16(out, inp, nelems);
+}
+
+template <>
+inline void cvt_to_float<float16_t>(
+        float *out, const float16_t *inp, size_t nelems) {
+    cvt_float16_to_float(out, inp, nelems);
+}
+
+inline void cvt_from_float(
+        data_type_t dt, void *out, const float *inp, size_t nelems) {
+    switch (dt) {
+        case data_type::bf16:
+            cvt_from_float((bfloat16_t *)out, inp, nelems);
+            break;
+        case data_type::f16:
+            cvt_from_float((float16_t *)out, inp, nelems);
+            break;
+        default: assert(!"unimplemented");
+    }
+}
+
 } // namespace types
 
 inline bool operator==(const memory_desc_t &lhs, const memory_desc_t &rhs) {
@@ -294,6 +377,9 @@ inline bool operator==(const memory_desc_t &lhs, const memory_desc_t &rhs) {
     else if (lhs.format_kind == format_kind::rnn_packed)
         return types::rnn_packed_desc_is_equal(lhs.format_desc.rnn_packed_desc,
                 rhs.format_desc.rnn_packed_desc);
+    else if (lhs.format_kind == format_kind::sparse)
+        return types::sparse_desc_is_equal(
+                lhs.format_desc.sparse_desc, rhs.format_desc.sparse_desc);
     return true;
 }
 
@@ -314,10 +400,12 @@ inline bool operator==(const batch_normalization_desc_t &lhs,
         const batch_normalization_desc_t &rhs) {
     bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
-            && COMPARE_DESC_MEMBERS(data_desc)
-            && COMPARE_DESC_MEMBERS(diff_data_desc)
-            && COMPARE_DESC_MEMBERS(data_scaleshift_desc)
-            && COMPARE_DESC_MEMBERS(diff_data_scaleshift_desc)
+            && COMPARE_DESC_MEMBERS(src_desc)
+            && COMPARE_DESC_MEMBERS(dst_desc)
+            && COMPARE_DESC_MEMBERS(diff_src_desc)
+            && COMPARE_DESC_MEMBERS(diff_dst_desc)
+            && COMPARE_DESC_MEMBERS(scaleshift_desc)
+            && COMPARE_DESC_MEMBERS(diff_scaleshift_desc)
             && COMPARE_DESC_MEMBERS(stat_desc)
             && COMPARE_FLOAT_DESC_MEMBERS(batch_norm_epsilon)
             && COMPARE_DESC_MEMBERS(flags);
@@ -342,7 +430,7 @@ inline bool operator==(const concat_desc_t &lhs, const concat_desc_t &rhs) {
     if (!ret) return ret;
 
     for (int i = 0; i < lhs.n; i++) {
-        ret = COMPARE_DESC_MEMBERS(src_mds[i]);
+        ret = *lhs.src_mds[i] == *rhs.src_mds[i];
         if (!ret) break;
     }
     return ret;
@@ -373,8 +461,10 @@ inline bool operator==(const eltwise_desc_t &lhs, const eltwise_desc_t &rhs) {
     bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
             && COMPARE_DESC_MEMBERS(alg_kind)
-            && COMPARE_DESC_MEMBERS(data_desc)
-            && COMPARE_DESC_MEMBERS(diff_data_desc)
+            && COMPARE_DESC_MEMBERS(src_desc)
+            && COMPARE_DESC_MEMBERS(dst_desc)
+            && COMPARE_DESC_MEMBERS(diff_src_desc)
+            && COMPARE_DESC_MEMBERS(diff_dst_desc)
             && COMPARE_FLOAT_DESC_MEMBERS(alpha)
             && COMPARE_FLOAT_DESC_MEMBERS(beta);
     return ret;
@@ -386,7 +476,9 @@ inline bool operator==(const gemm_desc_t &lhs, const gemm_desc_t &rhs) {
             && COMPARE_DESC_MEMBERS(b_desc)
             && COMPARE_DESC_MEMBERS(c_desc)
             && COMPARE_DESC_MEMBERS(bias_desc)
-            && COMPARE_DESC_MEMBERS(acc_type);
+            && COMPARE_DESC_MEMBERS(acc_type)
+            && COMPARE_DESC_MEMBERS(sum_ab)
+            && COMPARE_DESC_MEMBERS(sum_ab_type);
     return ret;
 }
 
@@ -406,26 +498,30 @@ inline bool operator==(
     return ret;
 }
 
-inline bool operator==(const layer_normalization_desc_t &lhs,
-        const layer_normalization_desc_t &rhs) {
+inline bool operator==(
+        const layer_normalization_desc_t &lhs, const layer_normalization_desc_t &rhs) {
     bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
-            && COMPARE_DESC_MEMBERS(data_desc)
-            && COMPARE_DESC_MEMBERS(diff_data_desc)
+            && COMPARE_DESC_MEMBERS(src_desc)
+            && COMPARE_DESC_MEMBERS(diff_src_desc)
             && COMPARE_DESC_MEMBERS(data_scaleshift_desc)
             && COMPARE_DESC_MEMBERS(diff_data_scaleshift_desc)
+            && COMPARE_DESC_MEMBERS(dst_desc)
+            && COMPARE_DESC_MEMBERS(diff_dst_desc)
             && COMPARE_DESC_MEMBERS(stat_desc)
             && COMPARE_FLOAT_DESC_MEMBERS(layer_norm_epsilon)
             && COMPARE_DESC_MEMBERS(flags);
-    return ret;
+     return ret;
 }
 
 inline bool operator==(const lrn_desc_t &lhs, const lrn_desc_t &rhs) {
     bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
             && COMPARE_DESC_MEMBERS(alg_kind)
-            && COMPARE_DESC_MEMBERS(data_desc)
-            && COMPARE_DESC_MEMBERS(diff_data_desc)
+            && COMPARE_DESC_MEMBERS(src_desc)
+            && COMPARE_DESC_MEMBERS(dst_desc)
+            && COMPARE_DESC_MEMBERS(diff_src_desc)
+            && COMPARE_DESC_MEMBERS(diff_dst_desc)
             && COMPARE_DESC_MEMBERS(local_size)
             && COMPARE_FLOAT_DESC_MEMBERS(lrn_alpha)
             && COMPARE_FLOAT_DESC_MEMBERS(lrn_beta)
@@ -443,7 +539,8 @@ inline bool operator==(const matmul_desc_t &lhs, const matmul_desc_t &rhs) {
     return ret;
 }
 
-inline bool operator==(const pooling_desc_t &lhs, const pooling_desc_t &rhs) {
+inline bool operator==(
+        const pooling_desc_t &lhs, const pooling_desc_t &rhs) {
     bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
             && COMPARE_DESC_MEMBERS(alg_kind)
@@ -455,27 +552,20 @@ inline bool operator==(const pooling_desc_t &lhs, const pooling_desc_t &rhs) {
             && COMPARE_DESC_ARRAY_MEMBERS(kernel, DNNL_MAX_NDIMS)
             && COMPARE_DESC_ARRAY_MEMBERS(padding[0], DNNL_MAX_NDIMS)
             && COMPARE_DESC_ARRAY_MEMBERS(padding[1], DNNL_MAX_NDIMS)
+            && COMPARE_DESC_ARRAY_MEMBERS(dilation, DNNL_MAX_NDIMS)
             && COMPARE_DESC_MEMBERS(accum_data_type);
-    return ret;
-}
-
-inline bool operator==(
-        const pooling_v2_desc_t &lhs, const pooling_v2_desc_t &rhs) {
-    const auto &v1_desc_lhs = *reinterpret_cast<const pooling_desc_t *>(&lhs);
-    const auto &v1_desc_rhs = *reinterpret_cast<const pooling_desc_t *>(&rhs);
-
-    bool ret = v1_desc_lhs == v1_desc_rhs
-            && COMPARE_DESC_ARRAY_MEMBERS(dilation, DNNL_MAX_NDIMS);
      return ret;
 }
 
 inline bool operator==(const prelu_desc_t &lhs, const prelu_desc_t &rhs) {
     const bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
-            && COMPARE_DESC_MEMBERS(data_desc)
+            && COMPARE_DESC_MEMBERS(src_desc)
             && COMPARE_DESC_MEMBERS(weights_desc)
-            && COMPARE_DESC_MEMBERS(diff_data_desc)
-            && COMPARE_DESC_MEMBERS(diff_weights_desc);
+            && COMPARE_DESC_MEMBERS(dst_desc)
+            && COMPARE_DESC_MEMBERS(diff_src_desc)
+            && COMPARE_DESC_MEMBERS(diff_weights_desc)
+            && COMPARE_DESC_MEMBERS(diff_dst_desc);
     return ret;
 }
 
@@ -549,30 +639,23 @@ inline bool operator==(const rnn_desc_t &lhs, const rnn_desc_t &rhs) {
 inline bool operator==(const shuffle_desc_t &lhs, const shuffle_desc_t &rhs) {
     bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
-            && COMPARE_DESC_MEMBERS(data_desc)
+            && COMPARE_DESC_MEMBERS(src_desc)
+            && COMPARE_DESC_MEMBERS(dst_desc)
             && COMPARE_DESC_MEMBERS(axis)
             && COMPARE_DESC_MEMBERS(group_size);
     return ret;
 }
 
-inline bool operator==(const softmax_desc_t &lhs, const softmax_desc_t &rhs) {
+inline bool operator==(
+        const softmax_desc_t &lhs, const softmax_desc_t &rhs) {
     bool ret = COMPARE_DESC_MEMBERS(primitive_kind)
             && COMPARE_DESC_MEMBERS(prop_kind)
-            && COMPARE_DESC_MEMBERS(data_desc)
-            && COMPARE_DESC_MEMBERS(diff_desc)
-            && COMPARE_DESC_MEMBERS(softmax_axis);
-    return ret;
-}
-
-inline bool operator==(
-        const softmax_v2_desc_t &lhs, const softmax_v2_desc_t &rhs) {
-    const auto &v1_desc_lhs = *reinterpret_cast<const softmax_desc_t *>(&lhs);
-    const auto &v1_desc_rhs = *reinterpret_cast<const softmax_desc_t *>(&rhs);
-
-    bool ret = v1_desc_lhs == v1_desc_rhs
             && COMPARE_DESC_MEMBERS(alg_kind)
+            && COMPARE_DESC_MEMBERS(src_desc)
+            && COMPARE_DESC_MEMBERS(diff_src_desc)
             && COMPARE_DESC_MEMBERS(dst_desc)
-            && COMPARE_DESC_MEMBERS(diff_dst_desc);
+            && COMPARE_DESC_MEMBERS(diff_dst_desc)
+            && COMPARE_DESC_MEMBERS(softmax_axis);
      return ret;
 }
 
@@ -583,7 +666,7 @@ inline bool operator==(const sum_desc_t &lhs, const sum_desc_t &rhs) {
     if (!ret) return ret;
 
     for (int i = 0; i < lhs.n; i++) {
-        ret = COMPARE_DESC_MEMBERS(src_mds[i]);
+        ret = *lhs.src_mds[i] == *rhs.src_mds[i];
         if (!ret) break;
     }
     if (!ret) return ret;
@@ -673,14 +756,14 @@ inline bool memory_desc_strides_check(
 
 inline status_t memory_desc_init_by_strides(
         memory_desc_t &md, const dims_t strides) {
-    return dnnl_memory_desc_init_by_strides(
-            &md, md.ndims, md.dims, md.data_type, strides);
+    return memory_desc_init_by_strides(
+            md, md.ndims, md.dims, md.data_type, strides);
 }
 
 inline status_t memory_desc_init_by_tag(
         memory_desc_t &md, format_tag_t tag, const dims_t strides = nullptr) {
-    status_t status = dnnl_memory_desc_init_by_tag(
-            &md, md.ndims, md.dims, md.data_type, tag);
+    status_t status
+            = memory_desc_init_by_tag(md, md.ndims, md.dims, md.data_type, tag);
     if (status != status::success || strides == nullptr) return status;
 
     if (!memory_desc_strides_check(md, strides))
@@ -742,7 +825,9 @@ inline status_t memory_desc_init_by_blocking_desc(
     for (int _d = ndims - 1; _d >= 0; --_d) {
         const int d = perm[_d];
         md.format_desc.blocking.strides[d] = stride;
-        stride *= md.padded_dims[d] / blocks[d];
+        if (md.padded_dims[d] != 0) { // Keep same stride for zero dim
+            stride *= md.padded_dims[d] / blocks[d];
+        }
     }
 
     md.extra = utils::zero<memory_extra_desc_t>();
@@ -763,9 +848,9 @@ inline status_t memory_desc_init_by_md_and_dt(memory_desc_t &md,
 /** returns true if memory desc @p md corresponds to the given format tag and
  * strides.
  * If strides are not passed (or passed as nullptr) the dense structure is
- * assumed (i.e. the one that dnnl_memory_desc_init_by_tag() returns).
+ * assumed (i.e. the one that memory_desc_init_by_tag() returns).
  * Strides might contain `0` value, indicating the stride must match the one
- * that dnnl_memory_desc_init_by_tag() returns.
+ * that memory_desc_init_by_tag() returns.
  * Strides might contain `-1` values, that would be ignored during the
  * comparison. For instance, this can be used if a stride along minibatch
  * doesn't matter. */
@@ -774,8 +859,8 @@ inline bool memory_desc_matches_tag(const memory_desc_t &md, format_tag_t tag,
     if (md.format_kind != types::format_tag_to_kind(tag)) return false;
 
     memory_desc_t md_gold;
-    status_t status = dnnl_memory_desc_init_by_tag(
-            &md_gold, md.ndims, md.dims, md.data_type, tag);
+    status_t status = memory_desc_init_by_tag(
+            md_gold, md.ndims, md.dims, md.data_type, tag);
     if (status != status::success) return false;
 
     if (md.format_kind != format_kind::blocked)
@@ -837,7 +922,7 @@ inline bool memory_desc_sanity_check(int ndims, const dims_t dims,
     if (ndims == 0) return true;
 
     bool ok = dims != nullptr && 0 < ndims && ndims <= DNNL_MAX_NDIMS
-            && utils::one_of(data_type, f16, bf16, f32, s32, s8, u8);
+            && utils::one_of(data_type, f16, bf16, f32, f64, s32, s8, u8);
     if (!ok) return false;
 
     bool has_runtime_dims = false;
@@ -854,11 +939,41 @@ inline bool memory_desc_sanity_check(int ndims, const dims_t dims,
     return true;
 }
 
-inline bool memory_desc_sanity_check(const memory_desc_t *md) {
-    if (md == nullptr) return false;
+inline bool memory_desc_sanity_check(const memory_desc_t &md) {
     return memory_desc_sanity_check(
-            md->ndims, md->dims, md->data_type, format_kind::undef);
+            md.ndims, md.dims, md.data_type, format_kind::undef);
 }
+
+inline void copy_c_op_desc(op_desc_t *dst, const op_desc_t *src) {
+#define CASE_OP_DESC(pkind) \
+    case primitive_kind::pkind: dst->pkind = src->pkind; break;
+
+    switch ((int)src->kind) {
+        CASE_OP_DESC(batch_normalization);
+        CASE_OP_DESC(binary);
+        CASE_OP_DESC(convolution);
+        CASE_OP_DESC(deconvolution);
+        CASE_OP_DESC(eltwise);
+        CASE_OP_DESC(gemm);
+        CASE_OP_DESC(inner_product);
+        CASE_OP_DESC(layer_normalization);
+        CASE_OP_DESC(lrn);
+        CASE_OP_DESC(matmul);
+        CASE_OP_DESC(pooling);
+        CASE_OP_DESC(prelu);
+        CASE_OP_DESC(reduction);
+        CASE_OP_DESC(resampling);
+        CASE_OP_DESC(rnn);
+        CASE_OP_DESC(shuffle);
+        CASE_OP_DESC(softmax);
+
+        // Internal descs
+        CASE_OP_DESC(zero_pad);
+        default: assert(!"unknown C primitive kind");
+    }
+#undef CASE_OP_DESC
+}
+
 } // namespace impl
 } // namespace dnnl
 
