@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -237,8 +237,10 @@ dnnl_status_t init_jit_profiling_jitdumpdir(
 std::string get_jit_profiling_jitdumpdir() {
     std::string jitdumpdir;
 #if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
-    if (!jit_profiling_jitdumpdir.initialized())
-        init_jit_profiling_jitdumpdir(nullptr, false);
+    if (!jit_profiling_jitdumpdir.initialized()) {
+        auto status = init_jit_profiling_jitdumpdir(nullptr, false);
+        if (status != status::success) return std::string();
+    }
     jitdumpdir = jit_profiling_jitdumpdir.get();
 #endif
     return jitdumpdir;
@@ -286,7 +288,7 @@ dnnl_status_t dnnl_set_max_cpu_isa(dnnl_cpu_isa_t isa) {
 }
 
 dnnl_cpu_isa_t dnnl_get_effective_cpu_isa() {
-    auto isa = dnnl_cpu_isa_all;
+    auto isa = dnnl_cpu_isa_default;
 #if DNNL_CPU_RUNTIME != DNNL_RUNTIME_NONE
     isa = dnnl::impl::cpu::platform::get_effective_cpu_isa();
 #endif
@@ -316,8 +318,7 @@ namespace impl {
 namespace threadpool_utils {
 
 namespace {
-static thread_local dnnl::threadpool_interop::threadpool_iface
-        *active_threadpool
+thread_local dnnl::threadpool_interop::threadpool_iface *active_threadpool
         = nullptr;
 }
 
