@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -110,6 +110,12 @@ int get_verbose() {
                 experimental::use_bnorm_stats_one_pass() ? "enabled"
                                                          : "disabled");
 #endif
+
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+        printf("onednn_verbose,info,experimental functionality for sparse "
+               "domain is enabled\n");
+#endif
+
         printf("onednn_verbose,info,prim_template:");
         printf("%soperation,engine,primitive,implementation,prop_"
                "kind,memory_descriptors,attributes,auxiliary,problem_desc,exec_"
@@ -201,13 +207,26 @@ std::ostream &operator<<(std::ostream &ss, format_kind_t format_kind) {
     return ss;
 }
 
-std::string flags2str(unsigned flags) {
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+std::ostream &operator<<(std::ostream &ss, sparse_encoding_t encoding) {
+    ss << dnnl_sparse_encoding2str(encoding);
+    return ss;
+}
+#endif
+
+std::string normalization_flags2str(unsigned flags) {
     std::string s;
-    if (flags & dnnl_use_global_stats) s += "G";
-    if (flags & dnnl_use_scaleshift) s += "S";
-    if (flags & dnnl_use_scale) s += "C";
-    if (flags & dnnl_use_shift) s += "H";
-    if (flags & dnnl_fuse_norm_relu) s += "R";
+    if (flags & normalization_flags::use_global_stats) s += "G";
+    if (flags & normalization_flags::use_scale) s += "C";
+    if (flags & normalization_flags::use_shift) s += "H";
+    if (flags & normalization_flags::fuse_norm_relu) s += "R";
+    if (flags & normalization_flags::fuse_norm_add_relu) s += "A";
+    return s;
+}
+
+std::string rnn_flags2str(unsigned flags) {
+    std::string s;
+    if (flags & rnn_flags::diff_weights_overwrite) s += "O";
     return s;
 }
 
@@ -224,7 +243,7 @@ std::ostream &operator<<(std::ostream &ss, const memory_extra_desc_t &extra) {
     return ss;
 }
 
-std::string md2fmt_tag_str(const dnnl_memory_desc_t *md) {
+std::string md2fmt_tag_str(const memory_desc_t *md) {
     memory_desc_wrapper mdw(md);
 
     const auto &blk = mdw.blocking_desc();
@@ -268,18 +287,21 @@ std::string md2fmt_tag_str(const dnnl_memory_desc_t *md) {
 
 // Forms a format string for a given memory descriptor.
 //
-// The format is defined as: 'dt:[p|o|0]:fmt_kind:fmt:extra'.
+// There are two formats:
+// - dense: defined as: 'dt:[p|o|0]:fmt_kind:fmt:extra'.
+// - sparse: defined as: 'dt:[p|o|0]:fmt_kind:encoding:extra'.
 // Here:
 //  - dt       -- data type
 //  - p        -- indicates there is non-trivial padding
 //  - o        -- indicates there is non-trivial padding offset
 //  - 0        -- indicates there is non-trivial offset0
 //  - fmt_kind -- format kind (blocked, wino, etc...)
+//  - encoding -- sparse encoding (csr, etc...)
 //  - fmt      -- extended format string (format_kind specific)
 //  - extra    -- shows extra fields (underspecified)
-std::string md2fmt_str(const dnnl_memory_desc_t *md) {
+std::string md2fmt_str(const memory_desc_t *md) {
     std::stringstream ss;
-    if (!md) {
+    if (!md || types::is_zero_md(md)) {
         ss << data_type::undef << "::" << format_kind::undef << "::";
         return ss.str();
     }
@@ -297,6 +319,7 @@ std::string md2fmt_str(const dnnl_memory_desc_t *md) {
     ss << (offset0 ? "0" : "") << ":" << mdw.format_kind() << ":";
 
     if (mdw.is_blocking_desc()) ss << md2fmt_tag_str(md);
+    if (mdw.is_sparse_desc()) ss << mdw.encoding();
 
     ss << mdw.extra();
 
@@ -319,7 +342,7 @@ static std::string get_val_str(T val) {
 
 // Returns string with dimensions from a given memory descriptor.
 // The format is defined as: dim0xdim1x...xdimN, with RT values signed as `*`.
-std::string md2dim_str(const dnnl_memory_desc_t *md) {
+std::string md2dim_str(const memory_desc_t *md) {
     if (md == nullptr || md->ndims == 0) return "";
 
     memory_desc_wrapper mdw(md);
@@ -351,14 +374,60 @@ std::string md2desc_str(const memory_desc_t *md) {
     return s;
 }
 
+std::ostream &operator<<(std::ostream &ss, const runtime_scales_t &oscale) {
+    ss << oscale.mask_;
+    return ss;
+}
+
 std::ostream &operator<<(std::ostream &ss, const scales_t &oscale) {
     ss << oscale.mask_;
     const float val = oscale.scales_[0];
+    // Can't use scientific flags since it breaks parsing on converter and
+    // benchdnn side.
     if (oscale.mask_ == 0 || is_runtime_value(val))
         ss << ":" << get_val_str(val);
     return ss;
 }
 
+namespace {
+int get_arg_index(int arg) {
+    if (arg & DNNL_ARG_MULTIPLE_SRC) return arg - DNNL_ARG_MULTIPLE_SRC;
+    switch (arg) {
+        case DNNL_ARG_SRC_0: return 0;
+        case DNNL_ARG_SRC_1: return 1;
+        default: return -1;
+    }
+    return -1;
+}
+
+std::string get_arg(int arg) {
+    if (arg & DNNL_ARG_MULTIPLE_SRC) return "msrc";
+
+    std::string s;
+    switch (arg) {
+        case DNNL_ARG_SRC: // DNNL_ARG_SRC_0
+        case DNNL_ARG_SRC_1: s = "src"; break;
+        case DNNL_ARG_DST: s = "dst"; break;
+        case DNNL_ARG_WEIGHTS: s = "wei"; break;
+        case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST:
+            s = "attr_post_op_dw_dst";
+            break;
+        case DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS:
+            s = "attr_post_op_dw_wei";
+            break;
+        default: assert(!"unsupported arg"); s = "unsupported arg";
+    }
+    return s;
+}
+} // namespace
+
+std::string arg2str(int arg) {
+    std::string s = get_arg(arg);
+    const int idx = get_arg_index(arg);
+    if (idx != -1) s += std::to_string(idx);
+    return s;
+}
+
 std::ostream &operator<<(std::ostream &ss, const primitive_attr_t *attr) {
     // scratchpad and fpmath mode are not a part of
     // has_default_values(). Check them first.
@@ -368,12 +437,12 @@ std::ostream &operator<<(std::ostream &ss, const primitive_attr_t *attr) {
     }
     const fpmath_mode_t &fpm = attr->fpmath_mode_;
     if (fpm != fpmath_mode_t::dnnl_fpmath_mode_strict) {
-        ss << "attr-fpmath_mode:" << dnnl_fpmath_mode2str(fpm) << " ";
+        ss << "attr-fpmath:" << dnnl_fpmath_mode2str(fpm) << " ";
     }
 
     if (attr->has_default_values()) return ss;
 
-    const scales_t &os = attr->output_scales_;
+    const runtime_scales_t &os = attr->output_scales_;
     if (!os.has_default_values()) { ss << "attr-oscale:" << os << " "; }
 
     std::string empty_delim, attr_delim = "+";
@@ -386,8 +455,8 @@ std::ostream &operator<<(std::ostream &ss, const primitive_attr_t *attr) {
             const auto &val = map_entry.second;
             if (val.has_default_values()) continue;
 
-            int idx = as.get_index_val(map_entry.first);
-            ss << delim << "src" << idx << ":" << val;
+            int arg = map_entry.first;
+            ss << delim << arg2str(arg) << ":" << val;
             delim = attr_delim;
         }
         ss << " ";
@@ -401,15 +470,9 @@ std::ostream &operator<<(std::ostream &ss, const primitive_attr_t *attr) {
             if (zp.has_default_values(arg)) continue;
 
             int mask = 0;
-            const int *zpp = nullptr;
-            zp.get(arg, nullptr, &mask, &zpp);
+            zp.get(arg, &mask);
 
-            ss << delim
-               << (arg == DNNL_ARG_SRC ? "src"
-                                       : arg == DNNL_ARG_DST ? "dst" : "wei")
-               << ":" << mask;
-            if (mask == 0 || is_runtime_value(*zpp))
-                ss << ":" << get_val_str(*zpp);
+            ss << delim << arg2str(arg) << ":" << mask;
             delim = attr_delim;
         }
         ss << " ";
@@ -439,10 +502,6 @@ std::ostream &operator<<(std::ostream &ss, const primitive_attr_t *attr) {
                        << c.padding;
                     if (c.wei_dt == s8 || c.dst_dt != f32)
                         ss << ":" << c.dst_dt;
-                    if (c.count > 0 && c.wei_dt == s8) {
-                        ss << ":" << c.mask;
-                        if (c.mask == 0) ss << ":" << c.scales[0];
-                    }
                 } break;
                 case primitive_kind::eltwise: {
                     const post_ops_t::entry_t::eltwise_t &ew = e.eltwise;
@@ -500,7 +559,7 @@ static std::string init_info_batch_normalization(
     ss << ",";
 
     ss << pd->attr() << ",";
-    ss << "flags:" << flags2str(pd->desc()->flags) << ",";
+    ss << "flags:" << normalization_flags2str(pd->desc()->flags) << ",";
     ss << md2desc_str(src_md);
 
     return ss.str();
@@ -649,17 +708,17 @@ static std::string init_info_layer_normalization(
     ss << e << "," << pd->kind() << "," << pd->name() << ","
        << pd->desc()->prop_kind << ",";
 
-    auto src_md = pd->src_md(0);
+    auto src_md = pd->src_md();
+    auto dst_md = pd->is_fwd() ? pd->dst_md() : pd->diff_dst_md();
     auto stats_md = pd->is_fwd() && !pd->stats_are_src() ? pd->dst_md(1)
                                                          : pd->src_md(1);
-    auto diff_src_md = pd->diff_src_md();
-    ss << "data_" << src_md;
+    ss << "src_" << src_md << " dst_" << dst_md;
     if (stats_md) ss << " stats_" << stats_md;
-    if (diff_src_md) ss << " diff_" << diff_src_md;
+    if (pd->is_bwd()) ss << " diff_src_" << pd->diff_src_md();
     ss << ",";
 
     ss << pd->attr() << ",";
-    ss << "flags:" << flags2str(pd->desc()->flags) << ",";
+    ss << "flags:" << normalization_flags2str(pd->desc()->flags) << ",";
     ss << md2dim_str(src_md);
 
     return ss.str();
@@ -885,7 +944,8 @@ static std::string init_info_rnn(const engine_t *e, const pd_t *pd) {
     ss << pd->attr() << ",";
     ss << "alg:" << pd->cell_kind()
        << " direction:" << dnnl_rnn_direction2str(pd->direction())
-       << " activation:" << pd->activation_kind() << ",";
+       << " activation:" << pd->activation_kind()
+       << " flags:" << rnn_flags2str(pd->desc()->flags) << ",";
 
     ss << "l" << pd->L() << "t" << pd->T() << "mb" << pd->MB() << "sic"
        << pd->SIC() << "slc" << pd->SLC() << "dhc" << pd->DHC() << "dic"
@@ -932,11 +992,6 @@ static std::string init_info_softmax(const engine_t *e, const pd_t *pd) {
     return ss.str();
 }
 
-template <typename pd_t>
-static std::string init_info_logsoftmax(const engine_t *e, const pd_t *pd) {
-    return init_info_softmax(e, pd);
-}
-
 template <typename pd_t>
 static std::string init_info_sum(const engine_t *e, const pd_t *pd) {
     std::stringstream ss;
@@ -962,7 +1017,6 @@ void pd_info_t::init(engine_t *engine, const primitive_desc_t *pd) {
     if (is_initialized_) return;
 
     std::call_once(initialization_flag_, [&] {
-        using logsoftmax_pd_t = softmax_pd_t;
 // clang-format off
 #define CASE(kind) \
     case primitive_kind::kind: \
@@ -979,9 +1033,7 @@ void pd_info_t::init(engine_t *engine, const primitive_desc_t *pd) {
             CASE(inner_product);
             CASE(layer_normalization);
             CASE(lrn);
-            CASE(logsoftmax);
             CASE(matmul);
-            case primitive_kind::pooling_v2:
             CASE(pooling);
             CASE(prelu);
             CASE(reduction);
@@ -989,7 +1041,6 @@ void pd_info_t::init(engine_t *engine, const primitive_desc_t *pd) {
             CASE(resampling);
             CASE(rnn);
             CASE(shuffle);
-            case primitive_kind::softmax_v2:
             CASE(softmax);
             CASE(sum);
             case primitive_kind::zero_pad: break;
