@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -44,134 +44,194 @@ using namespace dnnl::impl::prop_kind;
 const std::map<pk_dt_impl_key_t, std::vector<impl_list_item_t>> &impl_list_map() {
     static const std::map<pk_dt_impl_key_t, std::vector<impl_list_item_t>> the_map = REG_IP_P({
         {{forward, f32, f32, f32}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_bf16>) // bf32
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>) // bf32
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2>)
             CPU_INSTANCE_AARCH64_ACL(acl_inner_product_fwd_t)
             CPU_INSTANCE(gemm_inner_product_fwd_t<f32>)
             CPU_INSTANCE(ref_inner_product_fwd_t)
             nullptr,
         }},
         {{forward, bf16, bf16, f32}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_bf16>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_bf16>)
             CPU_INSTANCE_AVX512(gemm_bf16_inner_product_fwd_t<f32>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
             CPU_INSTANCE(ref_inner_product_fwd_t)
             nullptr,
         }},
         {{forward, bf16, bf16, bf16}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_bf16>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_bf16>)
             CPU_INSTANCE_AVX512(gemm_bf16_inner_product_fwd_t<bf16>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE(ref_inner_product_fwd_t)
+            nullptr,
+        }},
+        {{forward, f16, f16, f32}, {
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx_fp16>)
+            CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_fp16>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE(ref_inner_product_fwd_t)
+            nullptr,
+        }},
+        {{forward, f16, f16, f16}, {
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx_fp16>)
+            CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_fp16>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AARCH64_ACL(acl_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_fwd_t)
             nullptr,
         }},
         {{backward_data, f32, f32, f32}, REG_BWD_PK({
-            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_bf16_amx_bf16>) // bf32
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_amx>) // bf32
             CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_data_t<avx512_core>)
             CPU_INSTANCE(gemm_inner_product_bwd_data_t<f32>)
             CPU_INSTANCE(ref_inner_product_bwd_data_t)
             nullptr,
         })},
         {{backward_data, f32, bf16, bf16}, REG_BWD_PK({
-            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_bf16_amx_bf16>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_data_t<avx512_core_bf16>)
             CPU_INSTANCE_AVX512(gemm_bf16_inner_product_bwd_data_t<f32>)
             CPU_INSTANCE(ref_inner_product_bwd_data_t)
             nullptr,
         })},
         {{backward_data, bf16, bf16, bf16}, REG_BWD_PK({
-            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_bf16_amx_bf16>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_data_t<avx512_core_bf16>)
             CPU_INSTANCE_AVX512(gemm_bf16_inner_product_bwd_data_t<bf16>)
             CPU_INSTANCE(ref_inner_product_bwd_data_t)
             nullptr,
         })},
+        {{backward_data, f32, f16, f16}, REG_BWD_PK({
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_amx_fp16>)
+            CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_data_t<avx512_core_fp16>)
+            CPU_INSTANCE(ref_inner_product_bwd_data_t)
+            nullptr,
+        })},
+        {{backward_data, f16, f16, f16}, REG_BWD_PK({
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_data_t<avx512_core_amx_fp16>)
+            CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_data_t<avx512_core_fp16>)
+            CPU_INSTANCE(ref_inner_product_bwd_data_t)
+            nullptr,
+        })},
         {{backward_weights, f32, f32, f32}, REG_BWD_PK({
-            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_bf16_amx_bf16>) // bf32
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_amx>) // bf32
             CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_weights_t<avx512_core>)
             CPU_INSTANCE(gemm_inner_product_bwd_weights_t<f32>)
             CPU_INSTANCE(ref_inner_product_bwd_weights_t)
             nullptr,
         })},
         {{backward_weights, bf16, f32, bf16}, REG_BWD_PK({
-            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_bf16_amx_bf16>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_weights_t<avx512_core_bf16>)
             CPU_INSTANCE_AVX512(gemm_bf16_inner_product_bwd_weights_t<f32>)
             CPU_INSTANCE(ref_inner_product_bwd_weights_t)
             nullptr,
         })},
         {{backward_weights, bf16, bf16, bf16}, REG_BWD_PK({
-            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_bf16_amx_bf16>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_weights_t<avx512_core_bf16>)
             CPU_INSTANCE_AVX512(gemm_bf16_inner_product_bwd_weights_t<bf16>)
             CPU_INSTANCE(ref_inner_product_bwd_weights_t)
             nullptr,
         })},
+        {{backward_weights, f16, f32, f16}, REG_BWD_PK({
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_amx_fp16>)
+            CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_weights_t<avx512_core_fp16>)
+            CPU_INSTANCE(ref_inner_product_bwd_weights_t)
+            nullptr,
+        })},
+        {{backward_weights, f16, f16, f16}, REG_BWD_PK({
+            CPU_INSTANCE_AMX(brgemm_inner_product_bwd_weights_t<avx512_core_amx_fp16>)
+            CPU_INSTANCE_AVX512(brgemm_inner_product_bwd_weights_t<avx512_core_fp16>)
+            CPU_INSTANCE(ref_inner_product_bwd_weights_t)
+            nullptr,
+        })},
         {{forward, s8, s8, f32}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, s8, s8, s32}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, s8, s8, s8}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, s8, s8, u8}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, u8, s8, f32}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, u8, s8, s32}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, u8, s8, s8}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, u8, s8, u8}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni>)
             CPU_INSTANCE(gemm_x8s8s32x_inner_product_fwd_t)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, s8, s8, bf16}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
         {{forward, u8, s8, bf16}, {
-            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_bf16_amx_int8>)
+            CPU_INSTANCE_AMX(brgemm_inner_product_fwd_t<avx512_core_amx>)
             CPU_INSTANCE_AVX512(brgemm_inner_product_fwd_t<avx512_core_vnni>)
+            CPU_INSTANCE_AVX2(brgemm_inner_product_fwd_t<avx2_vnni_2>)
             CPU_INSTANCE(ref_inner_product_int8_fwd_t)
             nullptr,
         }},
