@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2022 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -42,16 +42,30 @@ inline status_t get_depthwise_conv_desc(convolution_desc_t &cd_dw,
     // Create new attributes with scales from depthwise post-op and copy
     // post-ops after depthwise post-op.
     auto &dw_po = attr_1x1.post_ops_.entry_[dw_po_index].depthwise_conv;
-    if (dw_po.wei_dt == data_type::s8 && dw_po.count) {
-        CHECK(attr_dw.output_scales_.set(
-                dw_po.count, dw_po.mask, dw_po.scales));
+
+    // erase 1x1 conv scales
+    for (auto arg : {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST}) {
+        auto &scale = attr_dw.scales_.get(arg);
+        if (!scale.has_default_values()) attr_dw.scales_.reset(arg);
     }
 
+    const auto &dw_src_scales = attr_1x1.scales_.get(DNNL_ARG_DST);
+    const auto &dw_wei_scales
+            = attr_1x1.scales_.get(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
+    const auto &dw_dst_scales
+            = attr_1x1.scales_.get(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST);
+    if (!dw_src_scales.has_default_values())
+        attr_dw.scales_.set(DNNL_ARG_SRC, dw_src_scales.mask_);
+    if (!dw_wei_scales.has_default_values())
+        attr_dw.scales_.set(DNNL_ARG_WEIGHTS, dw_wei_scales.mask_);
+    if (!dw_dst_scales.has_default_values())
+        attr_dw.scales_.set(DNNL_ARG_DST, dw_dst_scales.mask_);
+
     auto dw_po_len = attr_1x1.post_ops_.len() - (dw_po_index + 1);
     attr_dw.post_ops_.entry_.resize(dw_po_len);
     for (int i = 0; i < dw_po_len; ++i) {
-        CHECK(attr_dw.post_ops_.entry_[i].copy_from(
-                attr_1x1.post_ops_.entry_[i + dw_po_index + 1]));
+        attr_dw.post_ops_.entry_[i].copy_from(
+                attr_1x1.post_ops_.entry_[i + dw_po_index + 1]);
     }
 
     attr_dw.scratchpad_mode_ = attr_1x1.scratchpad_mode_;
@@ -90,18 +104,17 @@ inline status_t get_depthwise_conv_desc(convolution_desc_t &cd_dw,
     const auto data_tag
             = (src_dw_tag == format_tag::undef) ? format_tag::any : src_dw_tag;
 
-    dnnl_memory_desc_init_by_tag(
-            &src_md, ndims, src_dw_md.dims, src_dw_md.data_type, data_tag);
+    memory_desc_init_by_tag(
+            src_md, ndims, src_dw_md.dims, src_dw_md.data_type, data_tag);
 
-    dnnl_memory_desc_init_by_tag(
-            &weights_md, ndims + 1, weights_tz, dw_po.wei_dt, format_tag::any);
+    memory_desc_init_by_tag(
+            weights_md, ndims + 1, weights_tz, dw_po.wei_dt, format_tag::any);
 
     if (with_bias)
-        dnnl_memory_desc_init_by_tag(
-                &bias_md, 1, bias_tz, dw_po.bias_dt, format_tag::a);
+        memory_desc_init_by_tag(
+                bias_md, 1, bias_tz, dw_po.bias_dt, format_tag::a);
 
-    dnnl_memory_desc_init_by_tag(
-            &dst_md, ndims, dst_tz, dw_po.dst_dt, data_tag);
+    memory_desc_init_by_tag(dst_md, ndims, dst_tz, dw_po.dst_dt, data_tag);
 
     CHECK(conv_desc_init(&cd_dw, prop_kind::forward_inference,
             alg_kind::convolution_auto, &src_md, &weights_md,
