@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -33,6 +33,7 @@
 #include "cpu/binary_injector_utils.hpp"
 #include "cpu/matmul/gemm_x8s8s32x_matmul.hpp"
 #include "cpu/matmul/matmul_utils.hpp"
+#include "cpu/scale_utils.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -57,10 +58,15 @@ status_t gemm_x8s8s32x_matmul_t::pd_t::init(engine_t *engine) {
     using namespace utils;
     using namespace data_type;
 
-    auto check_attr_oscale = [&]() -> bool {
-        const auto &oscale = attr()->output_scales_;
-        return oscale.mask_ == 0
-                || (oscale.mask_ == (1 << (dst_md()->ndims - 1)));
+    auto check_attr_scales = [&]() -> bool {
+        bool ok = attr_scales_ok();
+        if (!attr()->scales_.get(DNNL_ARG_SRC).has_default_values()
+                && !attr()->scales_.get(DNNL_ARG_WEIGHTS).has_default_values()
+                && attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_ != 0) {
+            // This case requires scratchpad with unknown size
+            if (N() == DNNL_RUNTIME_DIM_VAL) ok = false;
+        }
+        return ok;
     };
 
     auto check_attr_zero_points
@@ -89,14 +95,15 @@ status_t gemm_x8s8s32x_matmul_t::pd_t::init(engine_t *engine) {
                                 *this));
     };
 
-    bool ok = one_of(src_md()->data_type, s8, u8)
+    bool ok = is_dense_data() && !has_zero_dim_memory()
+            && one_of(src_md()->data_type, s8, u8)
             && weights_md()->data_type == s8 && desc()->accum_data_type == s32
             && one_of(dst_md()->data_type, f32, s32, s8, u8)
             && IMPLICATION(with_bias(),
                     one_of(weights_md(1)->data_type, f32, s32, s8, u8)
                             && is_bias_1xN())
             && attr()->has_default_values(
-                    primitive_attr_t::skip_mask_t::oscale_runtime
+                    primitive_attr_t::skip_mask_t::scales_runtime
                             | primitive_attr_t::skip_mask_t::zero_points_runtime
                             | primitive_attr_t::skip_mask_t::post_ops
                             | primitive_attr_t::skip_mask_t::sum_dt,
@@ -104,7 +111,7 @@ status_t gemm_x8s8s32x_matmul_t::pd_t::init(engine_t *engine) {
             && attr_.post_ops_.check_sum_consistent_dt(dst_md()->data_type)
             // need to set up default formats first, so that latter checks can
             // be perfomed properly
-            && set_default_formats() && check_attr_oscale()
+            && set_default_formats() && check_attr_scales()
             && check_attr_zero_points() && check_attr_post_ops()
             && gemm_based::check_gemm_compatible_formats(*this)
             && attr_.set_default_formats(dst_md(0)) == status::success;
@@ -128,6 +135,8 @@ status_t gemm_x8s8s32x_matmul_t::pd_t::init(engine_t *engine) {
 
     nthr_ = dnnl_get_max_threads();
     gemm_based::book_acc_scratchpad(*this, params_, sizeof(int32_t), nthr_);
+    auto scratchpad = scratchpad_registry().registrar();
+    book_precomputed_scales(scratchpad, attr()->scales_, N());
 
     return status::success;
 }
@@ -170,14 +179,27 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
     const auto &po = this->pd()->attr()->post_ops_;
     const auto post_ops_binary_rhs_arg_vec = prepare_binary_args(po, ctx);
 
-    DEFINE_SCALES_BUFFER(scales);
+    const auto src_d = ctx.memory_mdw(DNNL_ARG_SRC, pd()->src_md());
+    const auto weights_d = ctx.memory_mdw(DNNL_ARG_WEIGHTS, pd()->weights_md());
+    const auto dst_d = ctx.memory_mdw(DNNL_ARG_DST, pd()->dst_md());
+
+    const int ndims = pd()->ndims();
+
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    auto &scratchpad = ctx.get_scratchpad_grantor();
+    const float *scales = precompute_scales(scratchpad, src_scales, wei_scales,
+            dst_d.dims()[ndims - 1], pd()->attr());
+
     DEFINE_ZERO_POINT_VALUE(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINT_VALUE(weights_zero_point, DNNL_ARG_WEIGHTS);
     DEFINE_ZERO_POINT_VALUE(dst_zero_point, DNNL_ARG_DST);
 
-    const auto src_d = ctx.memory_mdw(DNNL_ARG_SRC, pd()->src_md());
-    const auto weights_d = ctx.memory_mdw(DNNL_ARG_WEIGHTS, pd()->weights_md());
-    const auto dst_d = ctx.memory_mdw(DNNL_ARG_DST, pd()->dst_md());
+    if (src_d.has_zero_dim() || weights_d.has_zero_dim()
+            || dst_d.has_zero_dim())
+        return status::success;
 
     int8_t gemm_off_a_int8 = static_cast<int8_t>(src_zero_point);
     uint8_t gemm_off_a_uint8 = static_cast<uint8_t>(src_zero_point);
@@ -194,7 +216,6 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
     const float dst_zero_point_f32 = static_cast<float>(dst_zero_point);
 
     matmul_helper_t helper(src_d, weights_d, dst_d);
-    const int ndims = pd()->ndims();
     const int batch_ndims = ndims - 2;
     dim_t M = helper.M();
     const dim_t N = helper.N();
@@ -216,11 +237,9 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
     const int nthr = pd()->nthr_;
 
     const gemm_based::params_t &params = pd()->params();
-    const bool can_fuse_src_batch_dims = pd()->has_runtime_dims_or_strides()
-            ? helper.can_fuse_src_batch_dims()
-            : params.can_fuse_src_batch_dims_;
-    const dim_t acc_stride = gemm_based::get_scratchpad_size(
-            batch, M, N, can_fuse_src_batch_dims, nthr);
+    const bool use_single_gemm_call = pd()->has_runtime_dims_or_strides()
+            ? helper.use_single_gemm_call_optimization(po)
+            : params.use_single_gemm_call_optimization_;
     bool dst_is_acc = params.dst_is_acc_;
     int32_t *acc = dst_is_acc
             ? reinterpret_cast<int32_t *>(dst)
@@ -229,9 +248,9 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
     // case: dynamic sizes
     bool need_free_acc = false;
     if (acc == nullptr) {
-        acc = (int32_t *)malloc(sizeof(int32_t) * acc_stride
-                        * ((can_fuse_src_batch_dims || batch == 1) ? 1 : nthr),
-                64);
+        const size_t buf_elements = gemm_based::get_scratchpad_num_elements(
+                batch, M, N, use_single_gemm_call, nthr);
+        acc = (int32_t *)malloc(sizeof(int32_t) * buf_elements, 64);
 
         if (acc == nullptr) return status::out_of_memory;
         need_free_acc = true;
@@ -241,27 +260,11 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
     const float beta = params.gemm_beta_;
     const dim_t acc_ldc = dst_is_acc ? ldc : N;
     const int scale_idx_mult
-            = this->pd()->attr()->output_scales_.mask_ == (1 << (ndims - 1));
+            = this->pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_
+            == (1 << (ndims - 1));
 
     std::atomic<status_t> st(status::success);
-    // use parallel over batch when binary po with channel bcast
-    // (except batch == 1)
-    bool is_binary_po_per_oc = false;
-    bool is_binary_po_per_oc_sp = false;
-    bool is_binary_po_channel_bcast = false;
-    std::tie(is_binary_po_per_oc, is_binary_po_per_oc_sp,
-            is_binary_po_channel_bcast)
-            = bcast_strategies_present_tup(po.entry_, pd()->dst_md(),
-                    broadcasting_strategy_t::per_oc,
-                    broadcasting_strategy_t::per_oc_spatial,
-                    broadcasting_strategy_t::per_mb_spatial);
-    // if batched, parralel over batch for per_mb_sp and per_oc binary
-    // post-op broadcast
-    const bool can_use_po_with_fused_batch = !is_binary_po_channel_bcast
-            && IMPLICATION(
-                    is_binary_po_per_oc || is_binary_po_per_oc_sp, ndims == 2);
-    const bool parallel_over_batch = batch > 1 && !can_fuse_src_batch_dims;
-    if (IMPLICATION(can_use_po_with_fused_batch, parallel_over_batch)) {
+    if (!use_single_gemm_call) {
         const int src_mask
                 = utils::get_dims_mask(dst_d.dims(), src_d.dims(), ndims);
         const int wei_mask
@@ -272,6 +275,8 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
         const size_t dst_dt_size = types::data_type_size(dst_d.data_type());
         const size_t work_amount = (size_t)batch * M * N;
         const size_t work_per_batch = (size_t)M * N;
+        const dim_t acc_stride = gemm_based::get_scratchpad_block_elements(
+                batch, M, N, use_single_gemm_call, nthr);
 
         // NOTE: inside lambda, type cast variables captured by reference using
         // either c-like "(type)var" or functional "type(var)" notation in order
@@ -392,7 +397,7 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
                     const ptrdiff_t oc_off = i_work % N;
                     (*pp_kernel_)(curr_dst, curr_acc,
                             bias + oc_off * bia_dt_size,
-                            scales + oc_off * scale_idx_mult, 0,
+                            scales + oc_off * scale_idx_mult, dst_scales[0], 0,
                             dst_logical_off, dim1_off, gemm_M * gemm_N,
                             static_cast<size_t>(N), ldc, &dst_zero_point_f32,
                             post_ops_binary_rhs_arg_vec.data(), dst,
@@ -449,7 +454,7 @@ status_t gemm_x8s8s32x_matmul_t::execute_ref(const exec_ctx_t &ctx) const {
                     balance211((size_t)(M * N), nthr, ithr, start, end);
                     const size_t dst_logical_off = start;
                     const size_t dim1_off = start % N;
-                    (*pp_kernel_)(dst, acc, bias, scales, start,
+                    (*pp_kernel_)(dst, acc, bias, scales, dst_scales[0], start,
                             dst_logical_off, dim1_off, end, (size_t)N, ldc,
                             &dst_zero_point_f32,
                             post_ops_binary_rhs_arg_vec.data(), dst, 0, ctx,
