@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2022 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 * Copyright 2020 FUJITSU LIMITED
 * Copyright 2022 Arm Ltd. and affiliates
 *
@@ -16,6 +16,8 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <thread>
+
 #include "cpu/platform.hpp"
 
 #if DNNL_CPU_THREADING_RUNTIME == DNNL_RUNTIME_THREADPOOL
@@ -33,6 +35,8 @@
 #elif DNNL_AARCH64
 #include "cpu/aarch64/cpu_isa_traits.hpp"
 #if DNNL_AARCH64_USE_ACL
+// For checking if fp16 isa is supported on the platform
+#include "arm_compute/core/CPP/CPPTypes.h"
 // For setting the number of threads for ACL
 #include "src/common/cpuinfo/CpuInfo.h"
 #endif
@@ -65,7 +69,7 @@ dnnl_cpu_isa_t get_effective_cpu_isa() {
 #elif DNNL_AARCH64
     return aarch64::get_effective_cpu_isa();
 #else
-    return dnnl_cpu_isa_all;
+    return dnnl_cpu_isa_default;
 #endif
 }
 
@@ -103,6 +107,35 @@ bool prefer_ymm_requested() {
 }
 
 bool has_data_type_support(data_type_t data_type) {
+    // Notice: see notes in header
+    switch (data_type) {
+        case data_type::bf16:
+#if DNNL_X64
+            return x64::mayiuse(x64::avx512_core)
+                    || x64::mayiuse(x64::avx2_vnni_2);
+#elif DNNL_PPC64
+#if defined(USE_CBLAS) && defined(BLAS_HAS_SBGEMM) && defined(__MMA__)
+            return true;
+#endif
+#else
+            return false;
+#endif
+        case data_type::f16:
+#if DNNL_X64
+            return x64::mayiuse(x64::avx512_core_fp16)
+                    || x64::mayiuse(x64::avx2_vnni_2);
+#elif DNNL_AARCH64_USE_ACL
+            return arm_compute::CPUInfo::get().has_fp16();
+#else
+            return false;
+#endif
+        default: return true;
+    }
+}
+
+bool has_training_support(data_type_t data_type) {
+    // TODO: maybe return false for int8, but some primitives like prelu
+    // have training support
     switch (data_type) {
         case data_type::bf16:
 #if DNNL_X64
@@ -117,6 +150,8 @@ bool has_data_type_support(data_type_t data_type) {
         case data_type::f16:
 #if DNNL_X64
             return x64::mayiuse(x64::avx512_core_fp16);
+#elif DNNL_AARCH64_USE_ACL
+            return arm_compute::CPUInfo::get().has_fp16();
 #else
             return false;
 #endif
@@ -126,7 +161,9 @@ bool has_data_type_support(data_type_t data_type) {
 
 float s8s8_weights_scale_factor() {
 #if DNNL_X64
-    return x64::mayiuse(x64::avx512_core_vnni) ? 1.0f : 0.5f;
+    return x64::mayiuse(x64::avx512_core_vnni) || x64::mayiuse(x64::avx2_vnni)
+            ? 1.0f
+            : 0.5f;
 #else
     return 1.0f;
 #endif
