@@ -44,14 +44,16 @@ struct ref_layer_normalization_fwd_t : public primitive_t {
             using skip_mask_t = primitive_attr_t::skip_mask_t;
 
             bool ok = is_fwd()
-                    && utils::one_of(src_md()->data_type, f32, bf16, s8, u8)
-                    && utils::one_of(dst_md()->data_type, f32, bf16, s8, u8)
+                    && utils::one_of(
+                            src_md()->data_type, f32, bf16, f16, s8, u8)
+                    && utils::one_of(
+                            dst_md()->data_type, f32, bf16, f16, s8, u8)
                     && platform::has_data_type_support(src_md()->data_type)
                     && platform::has_data_type_support(dst_md()->data_type)
                     && stat_md()->data_type == f32
                     && check_scale_shift_data_type()
-                    && attr()->has_default_values(skip_mask_t::oscale_runtime)
-                    && attr_oscale_ok() && set_default_formats_common();
+                    && attr()->has_default_values(skip_mask_t::scales_runtime)
+                    && attr_scales_ok() && set_default_formats_common();
             if (!ok) return status::unimplemented;
 
             return status::success;
@@ -78,9 +80,10 @@ struct ref_layer_normalization_bwd_t : public primitive_t {
 
         status_t init(engine_t *engine) {
             using namespace data_type;
-            bool ok = is_bwd() && utils::one_of(src_md()->data_type, f32, bf16)
-                    && utils::one_of(diff_dst_md()->data_type, f32, bf16)
-                    && utils::one_of(diff_src_md()->data_type, f32, bf16)
+            bool ok = is_bwd()
+                    && utils::one_of(src_md()->data_type, f32, bf16, f16)
+                    && utils::one_of(diff_dst_md()->data_type, f32, bf16, f16)
+                    && utils::one_of(diff_src_md()->data_type, f32, bf16, f16)
                     && platform::has_data_type_support(src_md()->data_type)
                     && platform::has_data_type_support(diff_dst_md()->data_type)
                     && platform::has_data_type_support(diff_src_md()->data_type)
