@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -23,6 +23,7 @@
 #include "common/c_types_map.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/primitive.hpp"
+#include "common/reorder.hpp"
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 
@@ -44,7 +45,10 @@ template <typename gates_t, typename acc_t>
 // The loop body needs to be put in a function as some versions of icc have
 // an issue with lambdas & macros inside omp simd loops
 inline void body_loop(int i, int k, const gates_t *ws_gates, acc_t *diff_bias,
-        const rnn_utils::rnn_conf_t &rnn) {
+        const rnn_utils::rnn_conf_t &rnn,
+        rnn_utils::cell_position_t cell_position) {
+    if (rnn.diff_weights_overwrite && (cell_position & rnn_utils::last_iter))
+        diff_bias[i * rnn.dhc + k] = 0.0f;
     for (int j = 0; j < rnn.mb; j++)
         diff_bias[i * rnn.dhc + k]
                 += ws_gates[j * rnn.scratch_gates_ld + i * rnn.dhc + k];
@@ -52,20 +56,21 @@ inline void body_loop(int i, int k, const gates_t *ws_gates, acc_t *diff_bias,
 } // namespace
 
 template <typename gates_t, typename acc_t>
-void gates_reduction(const rnn_utils::rnn_conf_t &rnn, const gates_t *ws_gates_,
+void gates_reduction(const rnn_utils::rnn_conf_t &rnn,
+        rnn_utils::cell_position_t cell_position, const gates_t *ws_gates_,
         acc_t *diff_bias_) {
 
     // @todo block k on simd-width to enable vectorization in
     // parallel_nd path
 #if DNNL_CPU_RUNTIME == DNNL_RUNTIME_OMP && _OPENMP >= 201307 \
-        && __INTEL_COMPILER < 1910
+        && (!defined(__INTEL_COMPILER) || __INTEL_COMPILER < 1910)
 #pragma omp parallel for simd collapse(2)
     for (int i = 0; i < rnn.n_gates; i++)
         for (int k = 0; k < rnn.dhc; k++)
-            body_loop(i, k, ws_gates_, diff_bias_, rnn);
+            body_loop(i, k, ws_gates_, diff_bias_, rnn, cell_position);
 #else
     parallel_nd(rnn.n_gates, rnn.dhc, [&](dim_t i, dim_t k) {
-        body_loop(i, k, ws_gates_, diff_bias_, rnn);
+        body_loop(i, k, ws_gates_, diff_bias_, rnn, cell_position);
     });
 #endif
 }
@@ -96,6 +101,8 @@ struct _ref_rnn_common_t : public primitive_t {
 
     typedef rnn_cell_execution_sig((class_name::*cell_execution_f));
     typedef rnn_grid_execution_sig((class_name::*grid_execution_f));
+    typedef rnn_merged_layer_execution_sig(
+            (class_name::*merged_layer_execution_f));
 
     typedef rnn_gemm_sig((class_name::*gemm_t));
     typedef rnn_bias_prepare_sig((class_name::*bias_prepare_t));
@@ -110,7 +117,14 @@ struct _ref_rnn_common_t : public primitive_t {
         using base_pd_t::base_pd_t;
 
         const char *impl_name() const {
-            return rnn_.is_brgemm ? "brgemm" : "ref";
+#if DNNL_X64
+            using namespace dnnl::impl::cpu::x64;
+            return rnn_.is_brgemm
+                    ? JIT_IMPL_NAME_HELPER("brgemm:", rnn_.brgemm_isa, "")
+                    : "ref";
+#else
+            return "ref";
+#endif
         }
 
         DECLARE_COMMON_PD_T(impl_name(), class_name, USE_GLOBAL_SCRATCHPAD);
@@ -149,15 +163,15 @@ struct _ref_rnn_common_t : public primitive_t {
 
             rnn_ = zero<decltype(rnn_)>();
             rnn_.is_brgemm = false;
-            ok = init_conf<class_name>(rnn_, *this->desc(), this->src_md(0),
-                    this->src_md(1), this->src_md(2), this->weights_md(0),
-                    this->weights_md(1),
+            ok = init_conf<class_name>(rnn_, *this->desc(), *this->attr(),
+                    this->src_md(0), this->src_md(1), this->src_md(2),
+                    this->weights_md(0), this->weights_md(1),
                     this->arg_md(DNNL_ARG_WEIGHTS_PROJECTION), this->dst_md(0),
                     this->dst_md(1), this->dst_md(2),
                     this->arg_md(DNNL_ARG_BIAS));
             if (!ok) return status::unimplemented;
 
-            if (rnn_.is_bf16()) {
+            if (rnn_.is_bf16_conf()) {
                 if (!utils::one_of(
                             rnn_.bias_dt, data_type::bf16, data_type::f32)
                         || rnn_.src_iter_c_dt != rnn_.dst_iter_c_dt
@@ -171,7 +185,7 @@ struct _ref_rnn_common_t : public primitive_t {
                 return status::unimplemented;
 
             /* check that no data shift have been passed to s8s8 lstm */
-            if (!IMPLICATION(rnn_.is_signed_int8(),
+            if (!IMPLICATION(rnn_.is_signed_int8_conf(),
                         this->attr()->rnn_data_qparams_.shift_ == 0.f))
                 return status::unimplemented;
 
@@ -233,6 +247,7 @@ struct _ref_rnn_common_t : public primitive_t {
                     this->arg_md(DNNL_ARG_WEIGHTS_PROJECTION),
                     this->diff_weights_md(0), this->diff_weights_md(1),
                     this->arg_md(DNNL_ARG_DIFF_WEIGHTS_PROJECTION));
+            set_workspace_sizes<class_name>(rnn_, *this->desc());
             return status::success;
         }
 
@@ -253,15 +268,23 @@ struct _ref_rnn_common_t : public primitive_t {
                     = this->desc()->weights_layer_desc.data_type;
 
             bool ok = one_of(cell_kind, alg_kind::vanilla_rnn,
-                              alg_kind::vanilla_lstm)
+                              alg_kind::vanilla_lstm, alg_kind::vanilla_gru,
+                              alg_kind::vanilla_augru)
                     && IMPLICATION(aprop == prop_kind::forward,
                             one_of(this->desc()->prop_kind, forward_training,
                                     forward_inference))
                     && IMPLICATION(aprop == backward,
                             one_of(this->desc()->prop_kind, backward))
-                    && src_layer_dt == src_type
-                    && everyone_is(
-                            weights_type, weights_iter_dt, weights_layer_dt)
+                    // TODO: Enable diff_weights_overwrite support
+                    && IMPLICATION(aprop == backward,
+                            this->diff_weights_overwrite() == false)
+                    // cell_type (or src_type) and primitive data type should
+                    // match, except for the bf32 case.
+                    && IMPLICATION(
+                            this->attr()->fpmath_mode_ == fpmath_mode::strict,
+                            src_layer_dt == src_type
+                                    && everyone_is(weights_type,
+                                            weights_iter_dt, weights_layer_dt))
                     && this->set_default_params() == status::success
                     && this->with_bias();
 
@@ -269,9 +292,9 @@ struct _ref_rnn_common_t : public primitive_t {
 
             rnn_ = zero<decltype(rnn_)>();
             rnn_.is_brgemm = true;
-            ok = init_conf<class_name>(rnn_, *this->desc(), this->src_md(0),
-                    this->src_md(1), this->src_md(2), this->weights_md(0),
-                    this->weights_md(1),
+            ok = init_conf<class_name>(rnn_, *this->desc(), *this->attr(),
+                    this->src_md(0), this->src_md(1), this->src_md(2),
+                    this->weights_md(0), this->weights_md(1),
                     this->arg_md(DNNL_ARG_WEIGHTS_PROJECTION), this->dst_md(0),
                     this->dst_md(1), this->dst_md(2),
                     this->arg_md(DNNL_ARG_BIAS));
@@ -279,11 +302,20 @@ struct _ref_rnn_common_t : public primitive_t {
             ok = ok
                     && IMPLICATION(one_of(this->desc()->prop_kind,
                                            forward_training, backward),
-                            (rnn_.is_bf16() || rnn_.is_f32()));
+                            (rnn_.is_bf16_conf() || rnn_.is_f32_conf()));
 
             if (!ok) return status::unimplemented;
 
-            if (rnn_.is_f32()
+            // Support for GRU / AUGRU cell in BRGEMM-based implementation is
+            // limited by forward_inference pass for now, all_f32 is disabled
+            // due to performance degradation.
+            // TODO: Improve GRU / AUGRU coverage in BRGEMM-based implementation
+            ok = IMPLICATION(rnn_.is_orig_gru,
+                    this->desc()->prop_kind == forward_inference
+                            && !rnn_.is_cell_dt_f32());
+            if (!ok) return status::unimplemented;
+
+            if (rnn_.is_cell_dt_f32()
                     && utils::one_of(this->desc()->prop_kind, backward,
                             forward_training))
                 return status::unimplemented;
@@ -293,7 +325,7 @@ struct _ref_rnn_common_t : public primitive_t {
                         this->desc()->prop_kind == forward_inference)))
                 return status::unimplemented;
 
-            if (rnn_.is_bf16()) {
+            if (rnn_.is_bf16_conf()) {
                 if (!mayiuse(avx512_core_bf16)
                         || !utils::one_of(
                                 rnn_.bias_dt, data_type::bf16, data_type::f32)
@@ -306,16 +338,17 @@ struct _ref_rnn_common_t : public primitive_t {
                             data_type::f32)
                     || rnn_.src_iter_c_dt != rnn_.dst_iter_c_dt)
                 return status::unimplemented;
-
-            if (rnn_.is_signed_int8() && !mayiuse(avx512_core_bf16_amx_int8))
+            const auto isa = get_max_cpu_isa();
+            if (rnn_.is_signed_int8_conf()
+                    && !is_superset(isa, avx512_core_amx))
                 return status::unimplemented;
-            if (rnn_.is_int8() && !mayiuse(avx512_core_vnni))
+            if (rnn_.is_int8_conf() && !is_superset(isa, avx512_core_vnni))
                 return status::unimplemented;
-            if (rnn_.is_f32() && !mayiuse(avx512_core))
+            if (rnn_.is_f32_conf() && !is_superset(isa, avx2))
                 return status::unimplemented;
 
             /* check that no shift have been passed to s8s8 amx lstm */
-            if (!IMPLICATION(rnn_.is_signed_int8(),
+            if (!IMPLICATION(rnn_.is_signed_int8_conf(),
                         this->attr()->rnn_data_qparams_.shift_ == 0))
                 return status::unimplemented;
 
@@ -341,8 +374,11 @@ struct _ref_rnn_common_t : public primitive_t {
                     this->desc()->cell_kind, sizeof(src_layer_t),
                     sizeof(scratch_t)));
 
+            // must be called after configure_brgemm()
+            set_workspace_sizes<class_name>(rnn_, *this->desc());
+
             // Only AMX LSTM supports s8s8 now
-            if (rnn_.is_signed_int8() && !rnn_.is_int8_amx())
+            if (rnn_.is_signed_int8_conf() && !rnn_.is_cell_int8_amx())
                 return status::unimplemented;
 
             // Set weights descriptors to desired format
@@ -376,7 +412,7 @@ struct _ref_rnn_common_t : public primitive_t {
                     return status::unimplemented;
                 }
             }
-            if (rnn_.is_unsigned_int8()) {
+            if (rnn_.is_unsigned_int8_conf()) {
                 const memory_desc_wrapper &weights_layer_d(
                         this->weights_layer_md_);
                 const memory_desc_wrapper &weights_iter_d(
@@ -399,6 +435,30 @@ struct _ref_rnn_common_t : public primitive_t {
             }
             CHECK(this->check_layout_consistency(true /*is_brgemm*/));
 
+            if (rnn_.is_bf32()) {
+                const memory_desc_wrapper weights_layer_d(
+                        this->weights_layer_md_);
+                memory_desc_t weights_layer_md;
+                const memory_desc_wrapper weights_iter_d(
+                        this->weights_iter_md_);
+                memory_desc_t weights_iter_md;
+
+                const auto bf16_tag = rnn_.n_block == 64
+                        ? format_tag::ldgOI64o2i
+                        : format_tag::ldgOI32o2i;
+                memory_desc_init_by_tag(weights_layer_md,
+                        weights_layer_d.ndims(), weights_layer_d.dims(),
+                        data_type::bf16, bf16_tag);
+                CHECK(reorder_primitive_desc_create(bf32_wei_layer_reorder_pd_,
+                        engine, weights_layer_d.md_, &weights_layer_md,
+                        nullptr));
+
+                memory_desc_init_by_tag(weights_iter_md, weights_iter_d.ndims(),
+                        weights_iter_d.dims(), data_type::bf16, bf16_tag);
+                CHECK(reorder_primitive_desc_create(bf32_wei_iter_reorder_pd_,
+                        engine, weights_iter_d.md_, &weights_iter_md, nullptr));
+            }
+
             return status::success;
 #else
             return status::unimplemented;
@@ -419,7 +479,7 @@ struct _ref_rnn_common_t : public primitive_t {
                 // initialize the workspace if needed
                 if (rnn_.is_training) {
                     dims_t ws_dims = {(dim_t)ws_sz};
-                    dnnl_memory_desc_init_by_tag(&this->ws_md_, 1, ws_dims,
+                    memory_desc_init_by_tag(this->ws_md_, 1, ws_dims,
                             data_type::u8, format_tag::x);
                 }
             }
@@ -427,7 +487,10 @@ struct _ref_rnn_common_t : public primitive_t {
         }
 
         rnn_utils::rnn_conf_t rnn_;
-
+#if DNNL_X64
+        std::shared_ptr<primitive_desc_t> bf32_wei_layer_reorder_pd_;
+        std::shared_ptr<primitive_desc_t> bf32_wei_iter_reorder_pd_;
+#endif
     private:
         void init_scratchpad(size_t scratchpad_sz) {
             using namespace memory_tracking::names;
@@ -473,6 +536,12 @@ struct _ref_rnn_common_t : public primitive_t {
             if (rnn_.is_brgemm) {
                 ref_rnn_brgemm_t::init_scratchpad(rnn_, scratchpad,
                         sizeof(gemm_acc_t), alignof(gemm_acc_t));
+                if (rnn_.is_bf32()) {
+                    scratchpad.book(key_nested_multiple + 0,
+                            bf32_wei_layer_reorder_pd_->scratchpad_registry());
+                    scratchpad.book(key_nested_multiple + 1,
+                            bf32_wei_iter_reorder_pd_->scratchpad_registry());
+                }
             }
 #endif
         }
@@ -527,7 +596,9 @@ struct _ref_rnn_common_t : public primitive_t {
                 break;
             case alg_kind::vanilla_gru:
             case alg_kind::vanilla_augru:
-                cell_func = &class_name::cell_execution_gru;
+                cell_func = (pd()->rnn_.is_brgemm)
+                        ? &class_name::cell_execution_brgemm_fwd
+                        : &class_name::cell_execution_gru;
                 break;
             case alg_kind::lbr_augru:
             case alg_kind::lbr_gru:
@@ -536,6 +607,10 @@ struct _ref_rnn_common_t : public primitive_t {
             default: break;
         }
 
+        merged_layer_func = pd()->rnn_.is_brgemm && pd()->rnn_.merge_gemm_layer
+                        && aprop == prop_kind::forward
+                ? &class_name::merged_layer_brgemm_fwd
+                : &class_name::merged_layer_execution_ref;
         grid_computation = &class_name::linear_execution;
 
         size_t scratchpad_size, workspace_size;
@@ -548,28 +623,37 @@ struct _ref_rnn_common_t : public primitive_t {
                 scratch_cell_offset_, scratchpad_size, workspace_size);
 #if DNNL_X64
         const auto rnn = pd()->rnn_;
-        if (rnn.is_brgemm)
+        if (rnn.is_brgemm) {
+            if (rnn.is_bf32()) {
+
+                pd()->bf32_wei_layer_reorder_pd_->create_primitive(
+                        bf32_wei_layer_reorder_, engine);
+
+                pd()->bf32_wei_iter_reorder_pd_->create_primitive(
+                        bf32_wei_iter_reorder_, engine);
+            }
             return rnn_brgemm_.init_kernels(rnn, src_type, weights_type);
+        }
 #endif
         return status::success;
     }
 
     ~_ref_rnn_common_t() { delete rnn_postgemm_; }
 
-    status_t execute(const exec_ctx_t &ctx) const override {
-        execute_(ctx);
-        return status::success;
-    }
+    status_t execute(const exec_ctx_t &ctx) const override;
 
 private:
 #if DNNL_X64
     ref_rnn_brgemm_t rnn_brgemm_;
+    std::shared_ptr<primitive_t> bf32_wei_layer_reorder_;
+    std::shared_ptr<primitive_t> bf32_wei_iter_reorder_;
 #endif
-    void execute_(const exec_ctx_t &ctx) const;
 
     rnn_grid_execution_sig(linear_execution);
     rnn_cell_execution_sig(cell_execution_ref);
+    rnn_merged_layer_execution_sig(merged_layer_execution_ref);
     rnn_cell_execution_sig(cell_execution_brgemm_fwd);
+    rnn_merged_layer_execution_sig(merged_layer_brgemm_fwd);
     rnn_cell_execution_sig(cell_execution_brgemm_bwd);
 
     rnn_cell_execution_sig(cell_execution_gru);
@@ -583,9 +667,10 @@ private:
 
     float (*activation_func)(float s, float alpha, float cliping);
 
+    template <typename input_t>
     void copy_init_layer(const rnn_utils::rnn_conf_t &rnn,
             src_layer_t *ws_states_layer_, gemm_acc_t *ws_diff_states_layer_,
-            const src_layer_t *xt_, const gemm_acc_t *diff_dst_layer) const;
+            const input_t *xt_, const gemm_acc_t *diff_dst_layer) const;
 
     template <typename input_t>
     void copy_init_iter(const rnn_utils::rnn_conf_t &rnn,
@@ -631,6 +716,7 @@ private:
 
     grid_execution_f grid_computation;
     cell_execution_f cell_func;
+    merged_layer_execution_f merged_layer_func;
 
     bias_prepare_t bias_preparation_func;
     bias_finalize_t bias_finalization_func;
