@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2022 Intel Corporation
+* Copyright 2021-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,6 +32,8 @@ namespace dnnl {
 namespace impl {
 namespace cpu {
 namespace x64 {
+
+template <cpu_isa_t isa, typename Wmm>
 struct jit_brdgmm_kernel_base_t : public jit_generator {
     jit_brdgmm_kernel_base_t(const brgemm_t &abrd);
 
@@ -40,17 +42,21 @@ struct jit_brdgmm_kernel_base_t : public jit_generator {
     brgemm_t brg;
 
     static bool is_fast_vnni_int8(const brgemm_t &brg) {
-        return brg.is_dgmm && brg.is_int8 && brg.ldb_tail /*n_vlen_tail*/ == 0;
+        return brg.is_dgmm && brg.is_int8 && brg.isa_impl == avx512_core_vnni
+                && brg.ldb_tail /*n_vlen_tail*/ == 0;
     }
 
 private:
-    // using alias for Zmm and Ymm. useful for future avx2 support.
-    using Vmm = Xbyak::Zmm;
-    using Wmm =
-            typename utils::conditional<std::is_same<Vmm, Xbyak::Zmm>::value,
-                    Xbyak::Ymm, Xbyak::Xmm>::type;
-    std::unique_ptr<injector::jit_uni_postops_injector_t<avx512_core, Vmm>>
-            postops_injector_;
+    // note: this kernel doesn't yet support TMM's. We differentiate Wmm and Vmm
+    // just to follow same template style as brgemm_kernel.
+    using Vmm =
+            typename utils::conditional<std::is_same<Wmm, Xbyak::Tmm>::value,
+                    Xbyak::Zmm, Wmm>::type;
+    using Vmm_low_t = typename vreg_traits<Vmm>::Vmm_lower_t;
+    static constexpr cpu_isa_t po_isa_t = utils::map(isa, avx512_core, avx2,
+            avx2, avx2_vnni_2, avx2_vnni_2, avx512_core_fp16, avx512_core_fp16);
+    using po_injector_t = injector::jit_uni_postops_injector_t<po_isa_t, Vmm>;
+    std::unique_ptr<po_injector_t> postops_injector_;
     std::unique_ptr<bf16_emulation_t> bf16_emu_;
 
     Xbyak::Label permute_index_table;
@@ -83,6 +89,7 @@ private:
     const reg64_t reg_total_padding = reg_table_base;
     const reg64_t reg_aux_bias = reg_table_base;
     const reg64_t reg_aux_scales = reg_table_base;
+    const reg64_t reg_aux_dst_scales = reg_table_base;
     const reg64_t reg_binary_params = abi_param1; // default for binary ops
     const reg64_t reg_ptr_sum_scale = reg_aux_A_vpad_top;
     const reg64_t reg_ptr_sum_zp = reg_aux_A_vpad_bottom;
@@ -102,62 +109,83 @@ private:
     // note 2: zmm0 collides with vmm_permute, hence need to write this value
     // before every loop.
 
-    constexpr static int max_vmms_ = 32;
+    const int simd_w_;
+    const int max_vmms_;
     constexpr static int reg_batch0_addr_offs_ = 0;
     constexpr static int reg_bias_offs_ = 8;
     constexpr static int reg_scales_offs_ = 16;
     constexpr static int reg_A_offs_ = 24; // brgemm_strd
     constexpr static int reg_B_offs_ = 32; // brgemm_strd
     constexpr static int abi_param1_offs_ = 40;
-    constexpr static int stack_space_needed_ = 48;
+    constexpr static int reg_dst_scales_offs_ = 48;
+    constexpr static int stack_space_needed_ = 56;
 
     bool with_binary_non_scalar_bcast_ = false;
 
-    inline int M() { return brg.bcast_dim; };
-    inline int N() { return brg.load_dim; };
-    inline int m_vlen_blk() { return brg.bd_block; };
-    inline int nb_m_vlen_blk() { return brg.bdb; };
-    inline int m_vlen_tail() { return brg.bdb_tail; };
-    inline int m_blocking() { return brg.bd_block2; };
-    inline int nb_m_blocking() { return brg.bdb2; };
-    inline int m_blocking_tail() { return brg.bdb2_tail; };
+    inline int M() { return brg.bcast_dim; }
+    inline int N() { return brg.load_dim; }
+    inline int m_block1() { return brg.bd_block; }
+    inline int nb_m_block1() { return brg.bdb; }
+    inline int m_block1_tail() { return brg.bdb_tail; }
+    inline int m_block2() { return brg.bd_block2; }
+    inline int nb_m_block2() { return brg.bdb2; }
+    inline int m_block2_tail() { return brg.bdb2_tail; }
 
-    inline int n_vlen_blk() { return brg.ld_block; };
-    inline int nb_n_vlen_blk() { return brg.ldb; };
-    inline int n_vlen_tail() { return brg.ldb_tail; };
-    inline int n_blocking() { return brg.ld_block2; };
-    inline int nb_n_blocking() { return brg.ldb2; };
-    inline int n_blocking_tail() { return brg.ldb2_tail; };
+    inline int n_block1() { return brg.ld_block; }
+    inline int nb_n_block1() { return brg.ldb; }
+    inline int n_block1_tail() { return brg.ldb_tail; }
+    inline int n_block2() { return brg.ld_block2; }
+    inline int nb_n_block2() { return brg.ldb2; }
+    inline int n_block2_tail() { return brg.ldb2_tail; }
 
-    bool is_fma_embd() { return brg.is_f32; }
+    int tail_length() { return n_block1_tail() % simd_w_; }
+    bool is_fma_embd() { return brg.is_f32 && is_superset(isa, avx512_core); }
     bool is_fast_vnni_int8() { return is_fast_vnni_int8(brg); }
+    int vnni_substep() {
+        return brg.isa_impl == avx2_vnni_2 && (brg.is_bf16 || brg.is_f16) ? 2
+                                                                          : 1;
+    }
+    int get_substep_simd(int n_i, int v_i, bool has_n_tail) {
+        const int last_n_block_sz
+                = n_block2_tail() > 0 ? n_block2_tail() : n_block2();
+        if (has_n_tail && n_i + 1 == last_n_block_sz) {
+            return nstl::min(simd_w_, n_block1_tail() - v_i * simd_w_);
+        } else {
+            return simd_w_;
+        }
+    }
     Vmm vmm_permute() { return Vmm(0); } // used in fast_vnni_int8
     Vmm vmm_a() { return Vmm(is_fast_vnni_int8()); }
     Vmm vmm_b(int bi = 0) {
         return Vmm(is_fast_vnni_int8() + !is_fma_embd() + bi);
     }
-    Vmm accm(int m_blocks, int n_blocks, int m, int n) {
-        assert(m_blocks <= m_blocking() && m < m_blocks);
-        assert(n_blocks <= n_blocking() && n < n_blocks);
-        const int accm_start = max_vmms_ - m_blocks * n_blocks;
-        const int accm_rel_idx = n * m_blocks + m;
+    Vmm accm(int m_blocks, int n_blocks, int m, int n, int vnni_idx) {
+        assert(m_blocks <= m_block2() && m < m_blocks);
+        assert(n_blocks <= n_block2() && n < n_blocks);
+        const int accm_start = max_vmms_ - m_blocks * n_blocks * vnni_substep();
+        const int accm_rel_idx
+                = m * n_blocks * vnni_substep() + n * vnni_substep() + vnni_idx;
         const int idx = accm_start + accm_rel_idx;
         assert(idx < max_vmms_ && idx > vmm_b(0).getIdx());
         return Vmm(idx);
     }
     Vmm vmm_tmp(int i) {
-        const int idx = max_vmms_ - m_blocking() * n_blocking() - 1 - i;
+        const int idx
+                = max_vmms_ - m_block2() * n_block2() * vnni_substep() - 1 - i;
         assert(idx > (is_fast_vnni_int8() - 1));
         return Vmm(idx);
     }
-    Vmm vmm_mask(const Vmm vmm_in, bool mask_flag, bool store);
-    Wmm wmm_mask(const Wmm wmm_in, bool mask_flag, bool store);
 
+    template <typename U>
+    U maybe_mask(const U umm_in, bool mask_flag, bool store);
+    void init_masks();
     void read_params();
     void load_accumulators(int m_blocks, int n_blocks);
     void restore_A_B_matrices();
     void set_A_B_matrices();
     void advance_A_B_matrices();
+    void load_a(Vmm vmma, int m_i, int n_i, int v_i, bool has_n_tail);
+    void load_b(Vmm vmmb, int n_i, int v_i, bool has_n_tail);
     void brdgmm_microkernel(int m_blocks, int n_blocks, bool has_top_padding,
             bool has_bottom_padding, bool has_tail = false);
     void compute_loop();
@@ -165,6 +193,8 @@ private:
     void cvt2ps(data_type_t type_in, const Vmm vmm_in, const Xbyak::Operand &op,
             bool mask_flag, bool store);
     void apply_post_ops(int m_blocks, int n_blocks, bool has_n_tail);
+    void maybe_transpose_interleaved_vnni_to_plain(
+            int m_blocks, int n_blocks, bool has_n_tail);
     void store_accumulators(int m_blocks, int n_blocks, bool has_n_tail);
     void store_accumulators_without_post_ops(
             int m_blocks, int n_blocks, bool has_n_tail);
@@ -174,22 +204,24 @@ private:
     bool has_vpad() {
         return brg.brgattr.max_top_vpad > 0 || brg.brgattr.max_bottom_vpad > 0;
     }
-    bool check_effective_padding() { return has_vpad() && M() > m_blocking(); }
+    bool check_effective_padding() { return has_vpad() && M() > m_block2(); }
 
-    int oc_logical_offset(int n) { return n * n_vlen_blk(); }
+    int oc_logical_offset(int n) { return n * n_block1(); }
     int A_offset(int m, int n) {
-        return brg.typesize_A * (m * brg.LDA + n * n_vlen_blk());
+        return brg.typesize_A * (m * brg.LDA + n * n_block1());
     }
-    int B_offset(int n) { return brg.typesize_B * n * n_vlen_blk(); }
-    int C_offset(int m, int n) {
-        return brg.typesize_C * (m * brg.LDC + n * n_vlen_blk());
+    int B_offset(int n) { return brg.typesize_B * n * n_block1(); }
+    int C_offset(int m, int n, int v) {
+        return brg.typesize_C * (m * brg.LDC + n * n_block1() + v * simd_w_);
     }
-    int D_offset(int m, int n) {
-        return brg.typesize_D * (m * brg.LDD + n * n_vlen_blk());
+    int D_offset(int m, int n, int v) {
+        return brg.typesize_D * (m * brg.LDD + n * n_block1() + v * simd_w_);
     }
-    int bias_offset(int n) { return brg.typesize_bias * n * n_vlen_blk(); }
-    int scales_offset(int n) {
-        return sizeof(float) * brg.is_oc_scale * n * n_vlen_blk();
+    int bias_offset(int n, int v) {
+        return brg.typesize_bias * (n * n_block1() + v * simd_w_);
+    }
+    int scales_offset(int n, int v) {
+        return sizeof(float) * brg.is_oc_scale * (n * n_block1() + v * simd_w_);
     }
 
     void generate() override;
