@@ -435,7 +435,7 @@ void jit_avx2_kernel_sgemm_kern::generate() {
 }
 
 jit_avx2_kernel_sgemm_kern::jit_avx2_kernel_sgemm_kern(bool beta_zero)
-    : jit_generator(nullptr, 65536) {
+    : jit_generator(jit_name(), nullptr, 65536) {
 
     beta_zero_ = beta_zero;
 }
