@@ -64,7 +64,7 @@ struct xbyak_gemm_t : public jit_generator {
     xbyak_gemm_t(char isTransA, char isTransB, float beta, bool hasBias = false,
             void *code_ptr = nullptr,
             size_t code_size = 80 * Xbyak::DEFAULT_MAX_CODE_SIZE)
-        : jit_generator(code_ptr, code_size)
+        : jit_generator(jit_name(), code_ptr, code_size)
         , isTransA(isTransA)
         , isTransB(isTransB)
         , beta(beta)
@@ -1562,7 +1562,7 @@ xbyak_gemm_t *get_xbyak_gemm(
     };
 
     // Kernel table [isTransA][isTransB][hasBias][beta (0, 1, other)]
-    static std::unique_ptr<xbyak_gemm_t> kernel_table[2][2][2][3];
+    static maybe_unique_ptr<xbyak_gemm_t> kernel_table[2][2][2][3];
     static std::once_flag initialized;
     static std::atomic<dnnl_status_t> st(dnnl_success);
     std::call_once(initialized, [&] {
@@ -1640,9 +1640,10 @@ dnnl_status_t sgemm_nocopy_driver(const char *transa, const char *transb,
     float *ws = nullptr;
     bool use_heap_mem = BK > ker_b1->stack_k_capacity();
     if (use_heap_mem) {
-        // Kernel uses sizeK * unroll_m + 64 elements as workspace.
+        // Kernel uses sizeK * unroll_m + 64 + unroll_m elements as workspace.
         const dim_t max_sizeK = BK;
-        const size_t ws_size = sizeof *ws * (max_sizeK * UNROLL_M + 64);
+        const size_t ws_size
+                = sizeof *ws * (max_sizeK * UNROLL_M + 64 + UNROLL_M);
 
         ws = (float *)malloc(ws_size, PAGE_4K);
         if (!ws) return dnnl_out_of_memory;
