@@ -46,7 +46,10 @@ struct xbyak_gemm_smalln_tn_t : public jit_generator {
     xbyak_gemm_smalln_tn_t(int N, float beta, float alpha,
             void *code_ptr = nullptr,
             size_t code_size = 80 * Xbyak::DEFAULT_MAX_CODE_SIZE)
-        : jit_generator(code_ptr, code_size), N(N), beta(beta), alpha(alpha) {}
+        : jit_generator(jit_name(), code_ptr, code_size)
+        , N(N)
+        , beta(beta)
+        , alpha(alpha) {}
 
     void generate() override ATTRIBUTE_OPTIMIZE {
         using namespace Xbyak;
@@ -724,7 +727,7 @@ dnnl_status_t sgemm_smalln_tn(const dim_t m, const dim_t n, const dim_t k,
         const dim_t ldb, const float beta, float *C, const dim_t ldc) {
     using namespace avx512_core_gemm_smalln_tn_f32;
 
-    static std::unique_ptr<xbyak_gemm_smalln_tn_t> kernels[4][3][3];
+    static maybe_unique_ptr<xbyak_gemm_smalln_tn_t> kernels[4][3][3];
     static std::once_flag initialized;
 
     static dnnl_status_t st = dnnl_success;
