@@ -24,7 +24,7 @@ namespace cpu {
 namespace x64 {
 
 jit_avx2_u8_copy_sum_at_kern::jit_avx2_u8_copy_sum_at_kern()
-    : jit_generator(nullptr, U8_COPY_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name(), nullptr, U8_COPY_KERNEL_CODE_SIZE) {}
 
 void jit_avx2_u8_copy_sum_at_kern::generate() {
 
