@@ -25,7 +25,8 @@ namespace x64 {
 
 jit_avx512_core_u8_copy_sum_bt_kern::jit_avx512_core_u8_copy_sum_bt_kern(
         bool s8_case)
-    : jit_generator(nullptr, U8_COPY_KERNEL_CODE_SIZE), s8_case(s8_case) {}
+    : jit_generator(jit_name(), nullptr, U8_COPY_KERNEL_CODE_SIZE)
+    , s8_case(s8_case) {}
 
 void jit_avx512_core_u8_copy_sum_bt_kern::generate() {
 
