@@ -25,7 +25,7 @@ namespace x64 {
 
 jit_sse41_kernel_b0_b_gemm_s8u8s32_kern::
         jit_sse41_kernel_b0_b_gemm_s8u8s32_kern()
-    : jit_generator(nullptr, S8U8S32_COMPUTE_KERNEL_CODE_SIZE) {}
+    : jit_generator(jit_name(), nullptr, S8U8S32_COMPUTE_KERNEL_CODE_SIZE) {}
 
 void jit_sse41_kernel_b0_b_gemm_s8u8s32_kern::generate() {
 
