@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2022 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 * Copyright 2018 YANDEX LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -47,7 +47,7 @@ using namespace Xbyak;
 jit_avx2_1x1_conv_kernel_f32::jit_avx2_1x1_conv_kernel_f32(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, avx2)
+    : jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, avx2)
     , jcp(ajcp)
     , attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
@@ -59,7 +59,7 @@ jit_avx2_1x1_conv_kernel_f32::jit_avx2_1x1_conv_kernel_f32(
         const size_t tail_size = jcp.oc_without_padding % isa_simd_width_;
 
         rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx, r13, r14,
-                preserve_gpr, preserve_vmm,
+                r15, preserve_gpr, preserve_vmm,
                 GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size,
                 use_exact_tail_scalar_bcast};
@@ -72,7 +72,7 @@ jit_avx2_1x1_conv_kernel_f32::jit_avx2_1x1_conv_kernel_f32(
 }
 
 void jit_avx2_1x1_conv_kernel_f32::generate_bcast_loop(int load_loop_blk) {
-    mov(aux1_reg_bcast_data, reg_bcast_data);
+    mov(aux1_reg_bcast_data, ptr[rsp + reg_bcast_data_off]);
     mov(aux_reg_output_data, reg_output_data);
     mov(bcast_loop_iter, reg_bcast_loop_work);
 
@@ -163,7 +163,7 @@ void jit_avx2_1x1_conv_kernel_f32::apply_postops(
             iterate(load_loop_blk, ur, load_dim_tail,
                     [&](const bool mask_flag, const int i, const int j) {
                         const size_t aux_output_offset
-                                = (i * get_output_i_offset(jcp)
+                                = (i * get_output_i_offset(jcp, true)
                                           + j * get_output_j_offset(jcp))
                                 * sizeof(float);
                         const auto vmm_idx
@@ -181,9 +181,14 @@ void jit_avx2_1x1_conv_kernel_f32::apply_postops(
             rhs_arg_params.vmm_tail_idx_.clear();
 
             const injector_utils::register_preserve_guard_t register_guard(
-                    this, {abi_param1});
+                    this, {abi_param1, aux_reg_output_data});
             const size_t reg_guard_stack_occupied
                     = register_guard.stack_space_occupied();
+            if (jcp.with_dw_conv) {
+                add(aux_reg_output_data,
+                        ptr[rsp + reg_dw_binary_output_off
+                                + reg_guard_stack_occupied]);
+            }
             mov(abi_param1,
                     ptr[rsp + reg_abi_param1_backup
                             + reg_guard_stack_occupied]);
@@ -232,7 +237,8 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
     auto bcast_ptr = [=](int u, int j) {
         assert(j < jcp.ur);
         assert(u <= jcp.reduce_loop_unroll);
-        return ptr[aux_reg_bcast_data + get_bcast_offset(jcp, u, j)];
+        const size_t offset = get_bcast_offset(jcp, u, j);
+        return make_safe_addr(aux_reg_bcast_data, offset, reg_long_offt);
     };
 
     auto get_load_offset_bwd_w = [=](int u, int i) {
@@ -263,7 +269,6 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
     auto get_output_offset = [=](int i, int j) {
         switch (jcp.prop_kind) {
             case backward_weights: return sizeof(float) * jcp.oc_block * j;
-            case backward_data:
             default:
                 return (i * get_output_i_offset(jcp)
                                + j * get_output_j_offset(jcp))
@@ -278,12 +283,9 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
                         + (i ? reg_output_stride * i
                              : 0) // TODO: Xbyak should allow 0 scale
                         + sizeof(float) * jcp.oc_block * j];
-            case backward_data:
             default:
-                return ptr[aux_reg_output_data
-                        + (i * get_output_i_offset(jcp)
-                                  + j * get_output_j_offset(jcp))
-                                * sizeof(float)];
+                const size_t off = get_output_offset(i, j);
+                return make_safe_addr(aux_reg_output_data, off, reg_long_offt);
         }
     };
 
@@ -385,7 +387,6 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
         apply_postops(load_loop_blk, ur, load_dim_tail);
 
         if (jcp.prop_kind == backward_weights && load_dim_tail > 0) {
-            push(reg_bcast_data);
             push(aux_reg_bcast_data);
         }
 
@@ -435,7 +436,6 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
 
         if (jcp.prop_kind == backward_weights && load_dim_tail > 0) {
             pop(aux_reg_bcast_data);
-            pop(reg_bcast_data);
         }
     };
 
@@ -496,7 +496,7 @@ void jit_avx2_1x1_conv_kernel_f32::generate_reduce_loop(
     L(reduce_loop);
     {
         fma_block(false);
-        add(aux_reg_bcast_data, jcp.reduce_loop_bcast_step);
+        safe_add(aux_reg_bcast_data, jcp.reduce_loop_bcast_step, reg_long_offt);
         add(aux_reg_load_data, jcp.reduce_loop_load_step);
         sub(reduce_loop_iter, jcp.reduce_loop_unroll);
         jg(reduce_loop, T_NEAR);
@@ -567,17 +567,19 @@ void jit_avx2_1x1_conv_kernel_f32::generate_diff_bias_loop(int load_loop_blk) {
 void jit_avx2_1x1_conv_kernel_f32::generate() {
     preamble();
 
-    if (jcp.with_binary || (jcp.with_bias && jcp.prop_kind == backward_weights))
-        sub(rsp, stack_space_needed);
+    sub(rsp, stack_space_needed);
 
     if (jcp.with_binary) {
         const auto zeroed_reg = r15;
         xor_(zeroed_reg, zeroed_reg);
         mov(ptr[rsp + reg_binary_post_op_acc_off], zeroed_reg);
         mov(ptr[rsp + reg_abi_param1_backup], abi_param1);
+        if (jcp.with_dw_conv)
+            mov(ptr[rsp + reg_dw_binary_output_off], zeroed_reg);
     }
 
     mov(reg_bcast_data, ptr[param1 + GET_OFF(bcast_data)]);
+    mov(ptr[rsp + reg_bcast_data_off], reg_bcast_data);
     mov(reg_load_data, ptr[param1 + GET_OFF(load_data)]);
     mov(reg_output_data, ptr[param1 + GET_OFF(output_data)]);
     if (jcp.with_bias) {
@@ -598,24 +600,36 @@ void jit_avx2_1x1_conv_kernel_f32::generate() {
     auto generate_load_loop_body = [=](int load_loop_blk) {
         generate_bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
+        const size_t offst_with_dw_conv
+                = get_load_loop_output_fwd_offset(jcp, load_loop_blk);
+        const size_t offst_wo_dw_conv
+                = get_load_loop_output_fwd_offset(jcp, load_loop_blk, true);
         switch (jcp.prop_kind) {
             case forward_training:
             case forward_inference:
                 add(reg_bias_data,
                         load_loop_blk * jcp.oc_block * sizeof(float));
-                add(reg_output_data,
-                        get_load_loop_output_fwd_offset(jcp, load_loop_blk));
+                safe_add(reg_output_data, offst_with_dw_conv, reg_long_offt);
                 if (jcp.with_binary) {
                     mov(aux_reg_load_data,
                             ptr[rsp + reg_binary_post_op_acc_off]);
                     add(aux_reg_load_data, jcp.load_block * load_loop_blk);
                     mov(ptr[rsp + reg_binary_post_op_acc_off],
                             aux_reg_load_data);
+                    if (jcp.with_dw_conv) {
+                        mov(aux_reg_load_data,
+                                ptr[rsp + reg_dw_binary_output_off]);
+                        add(aux_reg_load_data,
+                                offst_wo_dw_conv - offst_with_dw_conv);
+                        mov(ptr[rsp + reg_dw_binary_output_off],
+                                aux_reg_load_data);
+                    }
                 }
                 break;
             case backward_data:
-                add(reg_output_data,
-                        get_load_loop_output_bwd_d_offset(jcp, load_loop_blk));
+                safe_add(reg_output_data,
+                        get_load_loop_output_bwd_d_offset(jcp, load_loop_blk),
+                        reg_long_offt);
                 break;
             case backward_weights:
                 for (int i = 0; i < load_loop_blk; i++)
@@ -671,8 +685,7 @@ void jit_avx2_1x1_conv_kernel_f32::generate() {
 
     L(load_loop_blk_end);
 
-    if (jcp.with_binary || (jcp.with_bias && jcp.prop_kind == backward_weights))
-        add(rsp, stack_space_needed);
+    add(rsp, stack_space_needed);
 
     postamble();
 
