@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -88,8 +88,9 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
         const void *post_ops_binary_rhs_arg_vec,
         const void *post_ops_binary_rhs_arg_vec_dw) const {
     const memory_desc_wrapper src_d(pd()->src_md());
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd()->dst_1x1_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
+    const memory_desc_wrapper dw_dst_d(pd()->dst_md());
     const memory_desc_wrapper dw_weights_d(
             pd()->arg_md(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS));
     const memory_desc_wrapper dw_bias_d(
@@ -128,6 +129,10 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
     const int nb_load_blocking_max = jcp.with_dw_conv
             ? jcp.nb_load_blocking
             : jcp.nb_load_blocking_max;
+    const bool is_dst_layout_nxc = utils::one_of(
+            jcp.dst_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
+    const bool is_src_layout_nxc = utils::one_of(
+            jcp.src_tag, format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
 
     // Begin: declare Variables needed for dw conv.
     memory_tracking::grantor_t dw_scratchpad(
@@ -183,8 +188,6 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
 
     auto ker_1x1 = [&](int ocb, int ocb_start, int icb, int n, int g, int od,
                            int oh, int ow, int id, int ih, int iw) {
-        const bool is_dst_layout_nxc = utils::one_of(jcp.dst_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int oc_off_idx = is_dst_layout_nxc
                 ? g * jcp.oc + ocb * jcp.oc_block
                 : g * nb_oc + ocb;
@@ -200,8 +203,6 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
         p.load_data
                 = &weights[pd()->with_groups() ? weights_d.blk_off(g, ocb, icb)
                                                : weights_d.blk_off(ocb, icb)];
-        const bool is_src_layout_nxc = utils::one_of(jcp.src_tag,
-                format_tag::nwc, format_tag::nhwc, format_tag::ndhwc);
         const int ic_off_idx = is_src_layout_nxc
                 ? g * jcp.ic + icb * jcp.ic_block
                 : g * nb_ic + icb;
@@ -220,7 +221,7 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
         p.dst_l_off = dst_off;
         p.oc_l_off = oc_off_idx * (is_dst_layout_nxc ? 1 : jcp.oc_block);
         p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        p.dst_orig = dst;
+        p.dst_orig = static_cast<const float *>(p.output_data) - dst_off;
 
         (*kernel_)(&p);
     };
@@ -322,8 +323,8 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
             addrs[i] = pbuf + ((oh_1x1++) % jcp_dw.kh) * row_offset;
 
         const auto ocb_end = ocb_start + load_step;
-        const auto wch_stride
-                = jcp_dw.iw * jcp_dw.nb_ch_blocking * jcp_dw.ch_block;
+        const auto wch_stride = (is_src_layout_nxc ? 1 : jcp_dw.iw)
+                * jcp_dw.nb_ch_blocking * jcp_dw.ch_block;
         const int dil_h = jcp_dw.dilate_h + 1;
         const int str_h = jcp_dw.stride_h;
         const int ch_num = jcp_dw.nb_ch_blocking;
@@ -347,7 +348,12 @@ void jit_avx512_common_1x1_convolution_fwd_t<src_type, wei_type,
             jit_conv_call_s par_conv_dw;
 
             par_conv_dw.src = addrs.data();
-            par_conv_dw.dst = &dst[dst_d.blk_off(n, ch, dw_oh, ow)];
+
+            const size_t ch_step = is_dst_layout_nxc
+                    ? jcp_dw.ch_block
+                    : dw_dst_d.blk_off(0, 1, 0, 0);
+            par_conv_dw.dst
+                    = &dst[dw_dst_d.blk_off(n, 0, dw_oh, ow) + ch * ch_step];
 
             par_conv_dw.filt
                     = &weights_dw[dw_weights_d.blk_off(ch, 0, 0, kh, kw)];
