@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -86,6 +86,10 @@ struct jit_avx512_common_1x1_convolution_fwd_t : public primitive_t {
             return status::success;
         }
 
+        const memory_desc_t *dst_1x1_md(int index = 0) const {
+            return cpu_convolution_fwd_pd_t::dst_md(index);
+        }
+
         const memory_desc_t *dst_md(int index = 0) const override {
             return jcp_.with_dw_conv ? dw_conv_pd_->dst_md(index) : &dst_md_;
         }
@@ -123,7 +127,23 @@ struct jit_avx512_common_1x1_convolution_fwd_t : public primitive_t {
         bool set_default_formats() {
             using namespace format_tag;
 
-            auto dat_tag = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const memory_desc_wrapper src_d(&src_md_);
+            const memory_desc_wrapper dst_d(&dst_md_);
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, nwc, nhwc, ndhwc);
+            const auto dat_tag_nCx16c
+                    = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const auto curr_src_tag
+                    = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+            const auto curr_dst_tag
+                    = dst_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+            const auto is_data_layout_nxc
+                    = IMPLICATION(curr_src_tag != dat_tag_nxc,
+                              src_d.format_kind() == format_kind::any)
+                    && IMPLICATION(curr_dst_tag != dat_tag_nxc,
+                            dst_d.format_kind() == format_kind::any)
+                    && utils::one_of(dat_tag_nxc, curr_src_tag, curr_dst_tag);
+            auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
             auto wei_tag = utils::pick(2 * ndims() - 6 + with_groups(),
                     OIw16i16o, gOIw16i16o, OIhw16i16o, gOIhw16i16o, OIdhw16i16o,
                     gOIdhw16i16o);
@@ -210,8 +230,14 @@ struct jit_avx512_common_1x1_convolution_fwd_t : public primitive_t {
 
             jcp_dw.dw_conv_buffer_oc
                     = jcp_1x1.nb_load_blocking * jcp_1x1.oc_block;
-            jcp_1x1.bcast_loop_output_step
-                    = jcp_1x1.ur * jcp_1x1.load_block * jcp_1x1.typesize_out;
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, format_tag::nwc,
+                    format_tag::nhwc, format_tag::ndhwc);
+            const bool is_data_nxc = utils::everyone_is(
+                    dat_tag_nxc, jcp_1x1.src_tag, jcp_1x1.dst_tag);
+            if (!is_data_nxc)
+                jcp_1x1.bcast_loop_output_step = jcp_1x1.ur * jcp_1x1.load_block
+                        * jcp_1x1.typesize_out;
 
             registrar_t scratchpad(scratchpad_registry_);
             registrar_t dw_scratchpad(scratchpad, names::prefix_fusion);
@@ -243,7 +269,7 @@ struct jit_avx512_common_1x1_convolution_fwd_t : public primitive_t {
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
                 new jit_avx512_common_1x1_conv_kernel(
-                        pd()->jcp_, *pd()->attr(), *pd()->dst_md(0))));
+                        pd()->jcp_, *pd()->attr(), *pd()->dst_1x1_md(0))));
         CHECK(kernel_->create_kernel());
 
         if (pd()->jcp_.with_dw_conv) {
@@ -331,7 +357,23 @@ struct jit_avx512_common_1x1_convolution_bwd_data_t : public primitive_t {
         bool set_default_formats() {
             using namespace format_tag;
 
-            auto dat_tag = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const memory_desc_wrapper diff_src_d(&diff_src_md_);
+            const memory_desc_wrapper diff_dst_d(&diff_dst_md_);
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, nwc, nhwc, ndhwc);
+            const auto dat_tag_nCx16c
+                    = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const auto curr_src_tag = diff_src_d.matches_one_of_tag(
+                    dat_tag_nxc, dat_tag_nCx16c);
+            const auto curr_dst_tag = diff_dst_d.matches_one_of_tag(
+                    dat_tag_nxc, dat_tag_nCx16c);
+            const auto is_data_layout_nxc
+                    = IMPLICATION(curr_src_tag != dat_tag_nxc,
+                              diff_src_d.format_kind() == format_kind::any)
+                    && IMPLICATION(curr_dst_tag != dat_tag_nxc,
+                            diff_dst_d.format_kind() == format_kind::any)
+                    && utils::one_of(dat_tag_nxc, curr_src_tag, curr_dst_tag);
+            auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
             auto wei_tag = utils::pick(2 * ndims() - 6 + with_groups(),
                     IOw16o16i, gIOw16o16i, IOhw16o16i, gIOhw16o16i, IOdhw16o16i,
                     gIOdhw16o16i);
@@ -428,7 +470,24 @@ struct jit_avx512_common_1x1_convolution_bwd_weights_t : public primitive_t {
         bool set_default_formats() {
             using namespace format_tag;
 
-            auto dat_tag = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const memory_desc_wrapper src_d(&src_md_);
+            const memory_desc_wrapper diff_dst_d(&diff_dst_md_);
+
+            const auto dat_tag_nxc = utils::pick(ndims() - 3, nwc, nhwc, ndhwc);
+            const auto dat_tag_nCx16c
+                    = utils::pick(ndims() - 3, nCw16c, nChw16c, nCdhw16c);
+            const auto curr_src_tag
+                    = src_d.matches_one_of_tag(dat_tag_nxc, dat_tag_nCx16c);
+            const auto curr_dst_tag = diff_dst_d.matches_one_of_tag(
+                    dat_tag_nxc, dat_tag_nCx16c);
+            const auto is_data_layout_nxc
+                    = IMPLICATION(curr_src_tag != dat_tag_nxc,
+                              src_d.format_kind() == format_kind::any)
+                    && IMPLICATION(curr_dst_tag != dat_tag_nxc,
+                            diff_dst_d.format_kind() == format_kind::any)
+                    && utils::one_of(dat_tag_nxc, curr_src_tag, curr_dst_tag);
+
+            auto dat_tag = is_data_layout_nxc ? dat_tag_nxc : dat_tag_nCx16c;
             auto wei_tag = utils::pick(2 * ndims() - 6 + with_groups(),
                     OIw16i16o, gOIw16i16o, OIhw16i16o, gOIhw16i16o, OIdhw16i16o,
                     gOIdhw16i16o);
