@@ -20,6 +20,7 @@
 #include "common/utils.hpp"
 
 #include "cpu/cpu_primitive.hpp"
+#include "cpu/scale_utils.hpp"
 
 #include "cpu/x64/jit_avx512_core_amx_conv_utils.hpp"
 #include "cpu/x64/jit_avx512_core_amx_convolution.hpp"
@@ -39,12 +40,12 @@ using namespace nstl;
     (pd()->with_groups() ? (d).blk_off((g), __VA_ARGS__) \
                          : (d).blk_off(__VA_ARGS__))
 
-#define mem_blk_off(md, n, c, d, h, w) \
+#define mem_blk_off(mdw, n, c, d, h, w) \
     (pd()->ndims() == 3 \
-                    ? (md).blk_off((n), (c), (w)) \
+                    ? (mdw).blk_off((n), (c), (w)) \
                     : (pd()->ndims() == 4 \
-                                    ? (md).blk_off((n), (c), (h), (w)) \
-                                    : (md).blk_off((n), (c), (d), (h), (w))))
+                                    ? (mdw).blk_off((n), (c), (h), (w)) \
+                                    : (mdw).blk_off((n), (c), (d), (h), (w))))
 
 template <typename T>
 static inline T accum_with_upper_bound(T ub, T lv, T uv) {
@@ -95,7 +96,12 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
     assert(jcp.is_relo);
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = precompute_scales(ctx.get_scratchpad_grantor(),
+            src_scales, wei_scales, pd()->OC(), pd()->attr());
 
     auto inp_p_buffer = ctx.get_scratchpad_grantor().template get<char>(
             key_conv_amx_inp_buffer); // fix the template
@@ -144,7 +150,6 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
     // Initialize the tile configuration in memory, so that each thread can
     // load this configuration from memory via `amx_tile_configure(tcfg)`.
     kernel_->tile_configure(tcfg);
-    const bool is_1d = pd()->ndims() == 3;
 
     // init zero_point padding buffer
     const bool req_zero_point_buffer = jcp.req_zero_point_buffer;
@@ -154,7 +159,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
     if (req_zero_point_buffer && zp_pbuff_outer_compute) {
         const size_t wei_oc_step = (size_t)jcp.kh * jcp.kw * jcp.ic_block_int_np
                 * jcp.nb_oc_blocking * jcp.oc_block;
-        const int sp_stride = dst_d.blk_off(0, 0, 0, 1);
+        const int sp_stride = mem_blk_off(dst_d, 0, 0, 0, 0, 1);
         const int dilate_h = jcp.dilate_h + 1;
         const int gen_kh = (jcp.kh - 1) * dilate_h + 1;
         const int oh_work = jcp.oh_pad;
@@ -214,7 +219,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
         amx_tile_configure(tcfg);
 
         const int oh_work = jcp.oh_pad;
-        const int sp_stride = dst_d.blk_off(0, 0, 0, 1);
+        const int sp_stride = mem_blk_off(dst_d, 0, 0, 0, 0, 1);
         const int dilate_h = jcp.dilate_h + 1;
         const int gen_kh = (jcp.kh - 1) * dilate_h + 1;
         const size_t wei_oc_step = (size_t)jcp.kh * jcp.kw * jcp.ic_block_int_np
@@ -317,8 +322,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                 if (!is_inp_buffer_relevant) {
                     // prepare padded input buffer
                     const int icb = g * jcp.ic;
-                    size_t inp_offset = is_1d ? src_d.blk_off(mb, icb, 0)
-                                              : src_d.blk_off(mb, icb, 0, 0);
+                    size_t inp_offset = src_d.blk_off(mb, icb);
                     const int iw_step = jcp.ngroups * jcp.ic_without_padding;
                     const char *psrc = src + src_dt_size * inp_offset;
                     // calculate overlap...
@@ -359,8 +363,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                 }
 
                 p.src = inp_buffer_oh;
-                size_t dst_offset = is_1d ? dst_d.blk_off(mb, ocb, ow)
-                                          : dst_d.blk_off(mb, ocb, oh, ow);
+                size_t dst_offset = mem_blk_off(dst_d, mb, ocb, 0, oh, ow);
                 p.dst = dst + dst_dt_size * dst_offset;
                 const size_t pbuff_offset
                         = zp_oh * jcp.ow_pad * sp_stride + ocb * oc_stride;
@@ -371,6 +374,7 @@ jit_avx512_core_amx_convolution_fwd_t::execute_forward_reduced_lowering(
                         + wei_dt_size * (g * oc_chunks + occ) * wei_oc_shift;
                 p.bias = bias_w;
                 p.scales = &oscales[jcp.is_oc_scale * oc];
+                p.dst_scale = &dst_scales[0];
 
                 p.acc_s32 = wsp + ithr * jcp.wsp_buffer_size;
 
@@ -447,7 +451,12 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
     const auto &jcp = pd()->jcp_;
     assert(jcp.nb_oc % jcp.nb_oc_blocking == 0);
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = precompute_scales(ctx.get_scratchpad_grantor(),
+            src_scales, wei_scales, pd()->OC(), pd()->attr());
 
     // TODO: use block offset instead of hand-calculated one
     //size_t wei_oc_shift = wht_blk_off(weights_d, 0, 1);
@@ -757,6 +766,7 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
                                 * wei_dt_size;
                 p.bias = bias_w;
                 p.scales = &oscales[jcp.is_oc_scale * oc];
+                p.dst_scale = &dst_scales[0];
 
                 p.acc_s32 = wsp + ithr * jcp.wsp_buffer_size;
                 if (req_zero_point_buffer
@@ -802,7 +812,7 @@ status_t jit_avx512_core_amx_convolution_fwd_t::execute_forward(
 
 template <data_type_t diff_src_type, data_type_t wei_type,
         data_type_t diff_dst_type>
-void jit_avx512_core_amx_convolution_bwd_data_t<diff_src_type, wei_type,
+status_t jit_avx512_core_amx_convolution_bwd_data_t<diff_src_type, wei_type,
         diff_dst_type>::execute_backward(const exec_ctx_t &ctx) const {
     const auto diff_dst = CTX_IN_MEM(const char *, DNNL_ARG_DIFF_DST);
     const auto weights = CTX_IN_MEM(const char *, DNNL_ARG_WEIGHTS);
@@ -814,12 +824,18 @@ void jit_avx512_core_amx_convolution_bwd_data_t<diff_src_type, wei_type,
 
     // unused in kernel for bf16, but attributes have scales buffer by default
     // and using it here simplifies the shared `execute_backward_loop`.
-    const float *oscales = pd()->attr()->output_scales_.scales_;
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = precompute_scales(ctx.get_scratchpad_grantor(),
+            src_scales, wei_scales, pd()->OC(), pd()->attr());
 
     amx_utils::execute_backward_convolution_body(ctx, pd()->jcp_, kernel_,
-            diff_dst, weights, nullptr /* no bias */, oscales, diff_src,
-            diff_dst_d, weights_d, memory_desc_wrapper(nullptr) /* no bias */,
-            diff_src_d);
+            diff_dst, weights, nullptr /* no bias */, oscales, dst_scales,
+            diff_src, diff_dst_d, weights_d,
+            memory_desc_wrapper(nullptr) /* no bias */, diff_src_d);
+    return status::success;
 }
 
 template struct jit_avx512_core_amx_convolution_bwd_data_t<data_type::bf16,
@@ -852,7 +868,7 @@ status_t jit_avx512_core_amx_convolution_bwd_weights_t::init(engine_t *engine) {
     if (j.transform_to_vnni) {
         CHECK(safe_ptr_assign(diff_wei_trans_kernel_,
                 new jit_diff_wei_trans_to_vnni_t(
-                        j.kd, j.kh, j.kw, j.ic_block, j.oc_block)));
+                        j.wei_dt, j.kd, j.kh, j.kw, j.ic_block, j.oc_block)));
         CHECK(diff_wei_trans_kernel_->create_kernel());
     }
     return status::success;
@@ -1745,6 +1761,28 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::compute_diff_weights(
     }
 }
 
+void jit_avx512_core_amx_convolution_bwd_weights_t::store_in_vnni_format(
+        const thread_info_t *ti) const {
+    const auto &jcp = kernel_->jcp;
+
+    for_(int g = ti->g_start; g < ti->g_end; g++)
+    for_(int oc_b = ti->oc_b_start; oc_b < ti->oc_b_end; oc_b++)
+    for_(int ic_b = ti->ic_b_start; ic_b < ti->ic_b_start + ti->ic_b_work;
+            ic_b += 2)
+    {
+        jit_conv_call_s p = jit_conv_call_s();
+
+        bfloat16_t *output = (bfloat16_t *)ti->diff_weights
+                + wei_offset_ext(g, oc_b, (ic_b / 2), 0);
+        float *input = ti->wei_bia_reduction + wei_offset_int(g, oc_b, ic_b, 0);
+
+        p.src = (void *)input;
+        p.dst = (void *)output;
+        p.last_ic_block = ((ic_b + 1) >= jcp.nb_ic) ? 1 : 0;
+        (*diff_wei_trans_kernel_)(&p);
+    }
+}
+
 void jit_avx512_core_amx_convolution_bwd_weights_t::
         reduce_and_convert_diff_weights_and_bias(
                 const thread_info_t *ti) const {
@@ -1757,31 +1795,11 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::
     const bool is_bf16_out = diff_weights_d.data_type() == data_type::bf16;
     const bool is_bf16_bias = jcp.with_bias && jcp.bia_dt == data_type::bf16;
 
-    auto store_in_vnni_format = [&]() {
-        for_(int g = ti->g_start; g < ti->g_end; g++)
-        for_(int oc_b = ti->oc_b_start; oc_b < ti->oc_b_end; oc_b++)
-        for_(int ic_b = ti->ic_b_start; ic_b < ti->ic_b_start + ti->ic_b_work;
-                ic_b += 2)
-        {
-            jit_conv_call_s p = jit_conv_call_s();
-
-            bfloat16_t *output = (bfloat16_t *)ti->diff_weights
-                    + wei_offset_ext(g, oc_b, (ic_b / 2), 0);
-            float *input
-                    = ti->wei_bia_reduction + wei_offset_int(g, oc_b, ic_b, 0);
-
-            p.src = (void *)input;
-            p.dst = (void *)output;
-            p.last_ic_block = ((ic_b + 1) >= jcp.nb_ic) ? 1 : 0;
-            (*diff_wei_trans_kernel_)(&p);
-        }
-    };
-
     if (nthr_mb_ == 1) {
         if (is_bf16_out) {
             // reduction is not required, only conversion
             if (jcp.transform_to_vnni) {
-                store_in_vnni_format();
+                store_in_vnni_format(ti);
             } else {
                 for_(int g = ti->g_start; g < ti->g_end; g++)
                 for (int oc_b = ti->oc_b_start; oc_b < ti->oc_b_end; oc_b++) {
@@ -1900,9 +1918,9 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::
         }
     }
 
-    if (jcp.transform_to_vnni) {
+    if (jcp.transform_to_vnni && jcp.global_transpose) {
         simple_barrier::barrier(ti->wei_bia_reduction_bctx, nthr_);
-        store_in_vnni_format();
+        store_in_vnni_format(ti);
     }
 }
 
@@ -2000,6 +2018,14 @@ void jit_avx512_core_amx_convolution_bwd_weights_t::execute_backward_weights(
         });
     }
 
+    if (jcp.transform_to_vnni && !jcp.global_transpose) {
+        parallel(nthr_, [&](const int ithr, const int nthr) {
+            assert(nthr_ == nthr);
+            thread_info_t thread_info(this, ctx, ithr);
+            store_in_vnni_format(&thread_info);
+        });
+    }
+
     if (pd()->with_bias() && (jcp.oc_without_padding % jcp.oc_block != 0)
             && jcp.bia_dt != data_type::bf16) {
         auto diff_bias = ctx.get_scratchpad_grantor().template get<const float>(
