@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -44,7 +44,7 @@ using namespace Xbyak;
 jit_avx512_core_bf16_1x1_conv_kernel::jit_avx512_core_bf16_1x1_conv_kernel(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, ker_code_size, true, avx512_core_bf16)
+    : jit_generator(jit_name(), nullptr, ker_code_size, true, avx512_core_bf16)
     , jcp(ajcp)
     , attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary) {
@@ -56,7 +56,7 @@ jit_avx512_core_bf16_1x1_conv_kernel::jit_avx512_core_bf16_1x1_conv_kernel(
         static constexpr bool use_exact_tail_scalar_bcast = true;
 
         const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
-                r14, r15, preserve_gpr, preserve_vmm,
+                r14, r15, r12, preserve_gpr, preserve_vmm,
                 GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size, k_load_dim_tail_mask,
                 use_exact_tail_scalar_bcast};
@@ -148,18 +148,16 @@ static int vreg_accum_idx(
 }
 
 Address jit_avx512_core_bf16_1x1_conv_kernel::output_ptr(
-        const int i_load, const int i_ur, const int scale) {
+        const int i_load, const int i_ur) {
     if (one_of(jcp.prop_kind, forward_training, forward_inference,
                 backward_data)) {
         const bool is_output_layout_nxc = is_out_layout_nxc();
-        assert(IMPLICATION(
-                is_output_layout_nxc, scale == 1 && !jcp.with_dw_conv));
         int i_load_shift = is_output_layout_nxc
                 ? jcp.load_block
                 : (jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim) * jcp.load_block;
         int i_ur_shift = is_output_layout_nxc ? jcp.load_dim : jcp.load_block;
         int offset = (i_load * i_load_shift + i_ur * i_ur_shift)
-                * jcp.typesize_out * scale;
+                * jcp.typesize_out;
         return EVEX_compress_addr(aux_reg_output_data, offset);
     } else
         return ptr[aux_reg_output_data
@@ -190,11 +188,15 @@ void jit_avx512_core_bf16_1x1_conv_kernel::apply_postops(
             binary_injector::rhs_arg_dynamic_params_t rhs_arg_params,
                     rhs_arg_params_tail;
             const auto mask_tail = jcp.oc_without_padding % isa_simd_width_;
+            if (jcp.with_dw_conv) {
+                add(aux_reg_output_data,
+                        EVEX_compress_addr(rsp, reg_dw_binary_output_off));
+            }
             iterate(load_loop_blk, ur, mask_tail,
                     [&](const bool mask_flag, const int i_load,
                             const int i_ur) {
                         const int aux_output_l_off
-                                = get_output_offset(i_load, i_ur);
+                                = get_output_offset(i_load, i_ur, true);
                         const auto vmm_idx
                                 = vreg_accum_idx(load_loop_blk, i_load, i_ur);
                         vmm_idxs.emplace(vmm_idx);
@@ -210,10 +212,15 @@ void jit_avx512_core_bf16_1x1_conv_kernel::apply_postops(
             rhs_arg_params.vmm_tail_idx_.clear();
 
             const injector_utils::register_preserve_guard_t register_guard(
-                    this, {reg_tmp});
+                    this, {reg_tmp, aux_reg_output_data});
             const size_t reg_guard_stack_occupied
                     = register_guard.stack_space_occupied();
-
+            if (jcp.with_dw_conv) {
+                add(aux_reg_output_data,
+                        EVEX_compress_addr(rsp,
+                                reg_dw_binary_output_off
+                                        + reg_guard_stack_occupied));
+            }
             mov(abi_param1,
                     EVEX_compress_addr(rsp,
                             reg_abi_param1_backup + reg_guard_stack_occupied));
@@ -988,6 +995,8 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
         xor_(zeroed_reg, zeroed_reg);
         mov(EVEX_compress_addr(rsp, reg_binary_post_op_acc_off), zeroed_reg);
         mov(EVEX_compress_addr(rsp, reg_abi_param1_backup), abi_param1);
+        if (jcp.with_dw_conv)
+            mov(EVEX_compress_addr(rsp, reg_dw_binary_output_off), zeroed_reg);
     }
 
     mov(reg_bcast_data, ptr[param1 + GET_OFF(bcast_data)]);
@@ -1059,18 +1068,19 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
         mov(reg_load_loop_work, ptr[rsp + reg_load_loop_work_off]);
 
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
+        const size_t off_with_dw_conv = load_loop_blk * jcp.load_block
+                * jcp.typesize_out
+                * (is_out_layout_nxc()
+                                ? 1
+                                : (jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim));
+        const size_t offst_wo_dw_conv = load_loop_blk * jcp.load_block
+                * jcp.typesize_out * (is_out_layout_nxc() ? 1 : jcp.bcast_dim);
         switch (jcp.prop_kind) {
             case forward_training:
             case forward_inference:
                 add(reg_bias_data,
                         load_loop_blk * jcp.load_block * jcp.typesize_bia);
-                add(reg_output_data,
-                        load_loop_blk * jcp.load_block * jcp.typesize_out
-                                * (is_out_layout_nxc()
-                                                ? 1
-                                                : (jcp.with_dw_conv
-                                                                ? jcp.ow
-                                                                : jcp.bcast_dim)));
+                add(reg_output_data, off_with_dw_conv);
                 add(reg_store_buf,
                         load_loop_blk * jcp.load_block * jcp.typesize_acc
                                 * (is_out_layout_nxc() ? 1 : jcp.bcast_dim));
@@ -1082,6 +1092,14 @@ void jit_avx512_core_bf16_1x1_conv_kernel::generate() {
                     add(oc_off_oprnd, jcp.load_block * load_loop_blk);
                     mov(EVEX_compress_addr(rsp, reg_binary_post_op_acc_off),
                             oc_off_oprnd);
+                    if (jcp.with_dw_conv) {
+                        mov(oc_off_oprnd,
+                                EVEX_compress_addr(
+                                        rsp, reg_dw_binary_output_off));
+                        add(oc_off_oprnd, offst_wo_dw_conv - off_with_dw_conv);
+                        mov(EVEX_compress_addr(rsp, reg_dw_binary_output_off),
+                                oc_off_oprnd);
+                    }
                 }
                 break;
             case backward_data:
