@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -48,7 +48,10 @@ _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::
         _jit_avx512_core_x8s8s32x_1x1_conv_kernel(
                 const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
                 const memory_desc_t &dst_md)
-    : jcp(ajcp), attr_(attr), postops_injector_(nullptr) {
+    : jit_generator(jit_name())
+    , jcp(ajcp)
+    , attr_(attr)
+    , postops_injector_(nullptr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -61,7 +64,7 @@ _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::
         static constexpr bool use_exact_tail_scalar_bcast = true;
 
         const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
-                r14, r15, preserve_gpr, preserve_vmm,
+                r14, r15, r13, preserve_gpr, preserve_vmm,
                 GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size, postops_mask,
                 use_exact_tail_scalar_bcast};
@@ -99,7 +102,7 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::bcast_loop(
         int num_substeps = jcp.bcast_block / jcp.ur;
         assert(num_substeps > 0 && num_substeps < 10);
         for (int i = 0; i < num_substeps; i++) {
-            reduce_loop(load_loop_blk, jcp.ur, i, false);
+            reduce_loop(load_loop_blk, jcp.ur, false);
             if (i < num_substeps - 1) {
                 add(aux1_reg_bcast_data, jcp.bcast_loop_bcast_substep);
                 add(aux_reg_output_data, jcp.bcast_loop_output_substep);
@@ -124,7 +127,7 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::bcast_loop(
         Label bcast_loop_tail_out;
         cmp(bcast_loop_iter, 0);
         jz(bcast_loop_tail_out, T_NEAR);
-        reduce_loop(load_loop_blk, jcp.ur_tail, 0, true);
+        reduce_loop(load_loop_blk, jcp.ur_tail, true);
         L(bcast_loop_tail_out);
     }
 }
@@ -238,9 +241,8 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::apply_postops(
             iterate(load_loop_blk, ur, mask_tail, oc_blk_is_smaller_than_vmm,
                     [&](const bool mask_flag, const int i_load,
                             const int i_ur) {
-                        const int ur_stride = jcp.with_dw_conv
-                                ? jcp.nb_load_blocking * jcp.oc_block * i_ur
-                                : jcp.oc_without_padding * jcp.ngroups * i_ur;
+                        const int ur_stride
+                                = jcp.oc_without_padding * jcp.ngroups * i_ur;
                         const size_t aux_output_l_off = jcp.typesize_out
                                 * (ur_stride + i_load * jcp.load_block);
                         const auto vmm_idx
@@ -289,13 +291,10 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::apply_postops(
 
 template <typename Vmm>
 void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
-        int load_loop_blk, int ur, int substep, bool wraparound) {
+        int load_loop_blk, int ur, bool wraparound) {
     auto vreg_load
             = [=](int i_load) { return Vmm(ur * load_loop_blk + i_load); };
 
-    auto vmm_bias_alpha = [=]() { return Vmm(ur * load_loop_blk); };
-
-    auto xmm_bias_alpha = [=]() { return Xmm(ur * load_loop_blk); };
     auto bias_ptr = [=](int i_load) {
         return EVEX_compress_addr(
                 reg_bias_data, jcp.typesize_bia * jcp.oc_block * i_load);
@@ -370,8 +369,6 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
         }
         if (jcp.signed_input && (!jcp.has_vnni)) {
             mov(reg_scratch, float2int(jcp.wei_adj_scale));
-            vmovq(xmm_bias_alpha(), reg_scratch);
-            vbroadcastss(vmm_bias_alpha(), xmm_bias_alpha());
         }
         if (jcp.src_zero_point) {
             mov(reg_zp_compensation,
@@ -384,12 +381,10 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
             auto vmm_bias = vmm_tmp;
             auto vmm_comp = vmm_bcast;
             if (jcp.with_bias) {
-                if (jcp.signed_input)
+                if (jcp.signed_input || jcp.dst_scale)
                     mov(reg_bias_data,
                             EVEX_compress_addr(rsp, reg_bias_data_off));
                 cvt2ps(jcp.bia_dt, vmm_bias, bias_ptr(i_load), mask_flag);
-                if (jcp.signed_input && (!jcp.has_vnni))
-                    vmulps(vmm_bias, vmm_bias, vmm_bias_alpha());
             }
             if (jcp.signed_input) {
                 mov(reg_comp_data, EVEX_compress_addr(rsp, reg_comp_data_off));
@@ -413,15 +408,33 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::reduce_loop(
                 vcvtdq2ps(r, r);
                 if (jcp.signed_input) vaddps(r, r, vmm_comp);
                 if (jcp.src_zero_point) vaddps(r, r, vmm_zp);
-                if (jcp.with_bias) vaddps(r, r, vmm_bias);
 
                 const Vmm mask_vmm = mask_flag ? r | k_load_dim_mask | T_z : r;
                 vmulps(mask_vmm, r, scale_ptr(i_load));
+
+                if (jcp.with_bias) vaddps(r, r, vmm_bias);
             }
         }
 
         apply_postops(load_loop_blk, ur, mask_flag_in, p_sum_scale, p_sum_zp);
 
+        if (jcp.dst_scale) {
+            mov(reg_ptr_dst_scale, EVEX_compress_addr(rsp, reg_dst_scale_off));
+            vmovups(vmm_dst_scale, EVEX_compress_addr(reg_ptr_dst_scale, 0));
+
+            /* Apply dst scale to accumulator */
+            for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
+                const bool mask_flag
+                        = mask_flag_in && i_load == load_loop_blk - 1;
+                for (int i_ur = 0; i_ur < ur; ++i_ur) {
+                    const auto r = vreg_accum(load_loop_blk, i_load, i_ur);
+                    const Vmm mask_vmm
+                            = mask_flag ? r | k_load_dim_mask | T_z : r;
+                    vmulps(mask_vmm, r, vmm_dst_scale);
+                }
+            }
+        }
+
         if (jcp.dst_zero_point) {
             mov(reg_dst_zero_point,
                     EVEX_compress_addr(rsp, reg_dst_zero_point_off));
@@ -641,6 +654,12 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::generate() {
         mov(EVEX_compress_addr(rsp, reg_src_zero_point_off),
                 reg_src_zero_point);
     }
+    if (jcp.dst_scale) {
+        if (!jcp.signed_input)
+            mov(EVEX_compress_addr(rsp, reg_bias_data_off), reg_bias_data);
+        mov(reg_ptr_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        mov(EVEX_compress_addr(rsp, reg_dst_scale_off), reg_ptr_dst_scale);
+    }
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
         mov(EVEX_compress_addr(rsp, reg_dst_zero_point_off),
@@ -712,11 +731,11 @@ void _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Vmm>::generate() {
         bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
         if (jcp.with_bias) {
-            if (jcp.signed_input)
+            if (jcp.signed_input || jcp.dst_scale)
                 mov(reg_bias_data, EVEX_compress_addr(rsp, reg_bias_data_off));
             add(reg_bias_data,
                     load_loop_blk * jcp.load_block * jcp.typesize_bia);
-            if (jcp.signed_input)
+            if (jcp.signed_input || jcp.dst_scale)
                 mov(EVEX_compress_addr(rsp, reg_bias_data_off), reg_bias_data);
         }
         if (jcp.with_binary) {
@@ -935,9 +954,9 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
             sum_requires_zp_zero});
     if (!post_ops_ok_) return status::unimplemented;
 
-    const int simd_w = (jcp.ic % 16 == 0 && jcp.oc % 16 == 0)
-            ? 16
-            : (jcp.ic % 8 == 0 && jcp.oc % 8 == 0) ? 8 : 4;
+    const int simd_w = (jcp.ic % 16 == 0 && jcp.oc % 16 == 0) ? 16
+            : (jcp.ic % 8 == 0 && jcp.oc % 8 == 0)            ? 8
+                                                              : 4;
 
     auto set_or_check_wei_format = [&]() -> bool {
         using namespace format_tag;
@@ -1190,12 +1209,10 @@ status_t jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_conf(
         jcp.nthr = nstl::min(jcp.nthr, nthr);
     }
 
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
-
-    // only common and per-oc-channel scales are supported
-    const bool oscales_ok = one_of(oscales.mask_, 0, 1 << 1);
-    if (!oscales_ok) return status::unimplemented;
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+    jcp.is_oc_scale = wei_scales.mask_ != 0;
+    jcp.dst_scale = !dst_scales.has_default_values();
 
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
@@ -1210,11 +1227,10 @@ void jit_avx512_core_x8s8s32x_1x1_conv_kernel::init_scratchpad(
         const jit_1x1_conv_conf_t &jcp, const primitive_attr_t &attr) {
     using namespace dnnl::impl::memory_tracking::names;
 
-    if (jcp.signed_input && (!jcp.has_vnni)) {
-        dim_t count = nstl::max<dim_t>(
-                attr.output_scales_.count_, (dim_t)jcp.ic_block);
-        scratchpad.book<float>(key_conv_adjusted_scales, count);
-    }
+    const int wei_mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const dim_t scales_count = wei_mask == 0 ? 1 : jcp.oc * jcp.ngroups;
+    const dim_t count = nstl::max<dim_t>(scales_count, (dim_t)jcp.ic_block);
+    scratchpad.book<float>(key_conv_adjusted_scales, count);
 }
 
 template struct _jit_avx512_core_x8s8s32x_1x1_conv_kernel<Xbyak::Zmm>;
