@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -52,50 +52,63 @@ status_t jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward(
                     pd()->jcp_.post_ops.entry_.size() + 1)
             : std::vector<const void *> {};
 
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    DEFINE_ARG_SCALES_BUFFER(
+            dw_wei_scales, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(
+            dw_dst_scales, DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST);
+
     DEFINE_ZERO_POINTS_BUFFER(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINTS_BUFFER(dst_zero_point, DNNL_ARG_DST);
 
     auto scratchpad = ctx.get_scratchpad_grantor();
 
-    if (pd()->jcp_.signed_input && (!pd()->jcp_.has_vnni)) {
-        auto local_scales
-                = scratchpad.template get<float>(key_conv_adjusted_scales);
-        auto scales = pd()->attr()->output_scales_.scales_;
-        size_t count = pd()->attr()->output_scales_.count_;
-        float factor = 1.f / pd()->jcp_.wei_adj_scale;
-        if (count == 1) {
-            utils::array_set(
-                    local_scales, scales[0] * factor, pd()->jcp_.ic_block);
-        } else {
-            for (size_t c = 0; c < count; c++)
-                local_scales[c] = scales[c] * factor;
-        }
+    auto local_scales
+            = scratchpad.template get<float>(key_conv_adjusted_scales);
+    // Src scale is always a single value
+    float src_scale = src_scales[0];
+    int wei_mask = pd()->attr()->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    float factor = (pd()->jcp_.signed_input && (!pd()->jcp_.has_vnni))
+            ? 1.f / pd()->jcp_.wei_adj_scale
+            : 1.f;
+    switch (wei_mask) {
+        case 0:
+            utils::array_set(local_scales, src_scale * wei_scales[0] * factor,
+                    pd()->jcp_.ic_block);
+            break;
+        default:
+            for (dim_t c = 0; c < pd()->OC(); c++)
+                local_scales[c] = src_scale * wei_scales[c] * factor;
     }
 
+    const float *dw_oscales = nullptr;
     if (pd()->jcp_.with_dw_conv) {
         auto jcp_dw = pd()->jcp_dw_;
-        if (jcp_dw->signed_input && (!jcp_dw->has_vnni)) {
-            memory_tracking::grantor_t dw_scratchpad(
-                    scratchpad, memory_tracking::names::prefix_fusion);
-            auto attr_dw = pd()->dw_conv_pd_->attr();
-
-            auto local_scales = dw_scratchpad.template get<float>(
-                    key_conv_adjusted_scales);
-            auto scales = attr_dw->output_scales_.scales_;
-            size_t count = attr_dw->output_scales_.count_;
-            float factor = 1.f / jcp_dw->wei_adj_scale;
-            if (count == 1) {
-                utils::array_set(
-                        local_scales, scales[0] * factor, pd()->jcp_.ic_block);
-            } else {
-                for (size_t c = 0; c < count; c++)
-                    local_scales[c] = scales[c] * factor;
-            }
+        memory_tracking::grantor_t dw_scratchpad(
+                scratchpad, memory_tracking::names::prefix_fusion);
+        auto dw_local_scales
+                = dw_scratchpad.template get<float>(key_conv_adjusted_scales);
+        auto attr_dw = pd()->dw_conv_pd_->attr();
+        int wei_mask = attr_dw->scales_.get(DNNL_ARG_WEIGHTS).mask_;
+        dim_t count = wei_mask == 0 ? 1 : pd()->dw_conv_pd_->OC();
+        float factor = 1.f / jcp_dw->wei_adj_scale;
+        if (count == 1) {
+            utils::array_set(dw_local_scales,
+                    dw_wei_scales[0] / dst_scales[0] * factor,
+                    pd()->jcp_.ic_block);
+        } else {
+            for (dim_t c = 0; c < count; c++)
+                dw_local_scales[c] = dw_wei_scales[c] / dst_scales[0] * factor;
         }
+        dw_oscales = dw_local_scales;
     }
     parallel(pd()->jcp_.nthr, [&](const int ithr, const int nthr) {
         execute_forward_thr(ithr, nthr, src, weights, bias, weights_dw, bias_dw,
-                dst, src_zero_point, dst_zero_point, scratchpad,
+                dst, local_scales, dst_scales, dw_oscales, dw_dst_scales,
+                src_zero_point, dst_zero_point, scratchpad,
                 post_ops_binary_rhs_arg_vec.data(),
                 post_ops_binary_rhs_arg_vec_dw.data());
     });
@@ -105,13 +118,16 @@ status_t jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward(
 void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
         const int ithr, const int nthr, const char *src, const char *weights,
         const char *bias, const char *weights_dw, const char *bias_dw,
-        char *dst, const int32_t *src_zero_point, const int32_t *dst_zero_point,
+        char *dst, const float *oscales, const float *dst_scales,
+        const float *dw_oscales, const float *dw_dst_scales,
+        const int32_t *src_zero_point, const int32_t *dst_zero_point,
         const memory_tracking::grantor_t &scratchpad,
         const void *post_ops_binary_rhs_arg_vec,
         const void *post_ops_binary_rhs_arg_vec_dw) const {
     const memory_desc_wrapper src_d(pd()->src_md());
-    const memory_desc_wrapper dst_d(pd()->dst_md());
+    const memory_desc_wrapper dst_d(pd()->dst_1x1_md());
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
+    const memory_desc_wrapper dw_dst_d(pd()->dst_md());
     const memory_desc_wrapper dw_weights_d(
             pd()->arg_md(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS));
 
@@ -127,8 +143,6 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
             ? scratchpad.get<char>(key_conv_rtus_space)
             : nullptr;
 
-    auto local_scales = scratchpad.get<float>(key_conv_adjusted_scales);
-
     const int work_amount = jcp.mb * jcp.ngroups * jcp.nb_bcast;
 
     const bool is_2d = pd()->ndims() == 4;
@@ -138,12 +152,6 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
     const int stride_h = pd()->KSH();
     const int stride_w = pd()->KSW();
 
-    float *oscales {nullptr};
-    if (jcp.signed_input && (!jcp.has_vnni))
-        oscales = scratchpad.get<float>(key_conv_adjusted_scales);
-    else
-        oscales = pd()->attr()->output_scales_.scales_;
-
     auto offset = weights_d.size() - weights_d.additional_buffer_size();
     char *w = const_cast<char *>(weights);
     const int32_t *compensation = (jcp.signed_input)
@@ -177,7 +185,6 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
             scratchpad, memory_tracking::names::prefix_fusion);
 
     size_t dw_bia_dt_size = 0;
-    float *dw_oscales {nullptr};
     int32_t *compensation_dw {nullptr};
     if (jcp.with_dw_conv && jcp_dw) {
         if (jcp_dw->with_bias)
@@ -189,10 +196,7 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
         compensation_dw = (jcp_dw->signed_input)
                 ? reinterpret_cast<int32_t *>(w + offset)
                 : nullptr;
-        if (jcp_dw->signed_input && (!jcp_dw->has_vnni))
-            dw_oscales = dw_scratchpad.get<float>(key_conv_adjusted_scales);
-        else
-            dw_oscales = dw_pd->attr()->output_scales_.scales_;
+        dw_oscales = dw_scratchpad.get<float>(key_conv_adjusted_scales);
     }
 
     char *pbuf {nullptr};
@@ -272,9 +276,8 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
                 : nullptr;
         p.src_zero_point = jcp.src_zero_point ? src_zero_point : nullptr;
         p.dst_zero_point = jcp.dst_zero_point ? dst_zero_point : nullptr;
-        p.scales = (jcp.signed_input && (!jcp.has_vnni))
-                ? &local_scales[jcp.is_oc_scale * _ocb * jcp.oc_block]
-                : &oscales[jcp.is_oc_scale * _ocb * jcp.oc_block];
+        p.scales = &oscales[jcp.is_oc_scale * _ocb * jcp.oc_block];
+        p.dst_scale = dst_scales;
         const size_t src_off = is_3d
                 ? src_d.blk_off(n, _icb * jcp.ic_block, id, ih, iw)
                 : is_2d ? src_d.blk_off(n, _icb * jcp.ic_block, ih, iw)
@@ -295,7 +298,8 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
         p.dst_l_off = dst_off;
         p.oc_l_off = _ocb * jcp.oc_block;
         p.post_ops_binary_rhs_arg_vec = post_ops_binary_rhs_arg_vec;
-        p.dst_orig = dst;
+        p.dst_orig = static_cast<const char *>(p.output_data)
+                - dst_off * dst_dt_size;
 
         (*kernel_)(&p);
     };
@@ -416,6 +420,7 @@ void jit_avx512_core_x8s8s32x_1x1_convolution_fwd_t::execute_forward_thr(
             par_conv_dw.scales = dw_oscales
                     ? &dw_oscales[jcp_dw->is_oc_scale * ocb * jcp_dw->ch_block]
                     : nullptr;
+            par_conv_dw.dst_scale = dw_dst_scales;
 
             par_conv_dw.oc_l_off = ocb * jcp_dw->ch_block;
             par_conv_dw.post_ops_binary_rhs_arg_vec
