@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2022 Intel Corporation
+* Copyright 2016-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -56,7 +56,10 @@ template <typename Vmm>
 _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::_jit_avx512_core_x8s8s32x_fwd_kernel(
         const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jcp(ajcp), attr_(attr), postops_injector_(nullptr) {
+    : jit_generator(jit_name())
+    , jcp(ajcp)
+    , attr_(attr)
+    , postops_injector_(nullptr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -69,7 +72,7 @@ _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::_jit_avx512_core_x8s8s32x_fwd_kernel(
         static constexpr bool use_exact_tail_scalar_bcast = false;
 
         const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
-                r14, r15, preserve_gpr, preserve_vmm,
+                r14, r15, r13, preserve_gpr, preserve_vmm,
                 GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size, postops_mask,
                 use_exact_tail_scalar_bcast};
@@ -259,13 +262,6 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
         p_sum_zp = &p_entry.sum.zero_point;
     }
 
-    if (jcp.signed_input && (!jcp.has_vnni)) {
-        /* put 'wei_adj_scale = 0.5' for bias calculation */
-        mov(reg_bias_alpha, float2int(jcp.wei_adj_scale));
-        vmovq(xmm_bias_alpha(), reg_bias_alpha);
-        vbroadcastss(vmm_bias_alpha(), xmm_bias_alpha());
-    }
-
     for (int k = 0; k < nb_oc_block; k++) {
         const bool mask_flag = last_oc_block_flag && k == nb_oc_block - 1;
         int scale_offset = jcp.is_oc_scale * (sizeof(float) * k * oc_block);
@@ -274,8 +270,6 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
             auto bias_addr = EVEX_compress_addr(reg_bias, bias_offset);
 
             cvt2ps(jcp.bia_dt, vmm_bias, bias_addr, mask_flag);
-            if (jcp.signed_input && (!jcp.has_vnni)) /* bias *= 0.5 */
-                vmulps(vmm_bias, vmm_bias, vmm_bias_alpha());
         }
         if (jcp.signed_input) {
             int comp_offset = sizeof(int32_t) * k * oc_block;
@@ -305,17 +299,32 @@ void _jit_avx512_core_x8s8s32x_fwd_kernel<Vmm>::store_output(
             if (jcp.src_zero_point) vpaddd(vmm, vmm, vmm_zp);
             vcvtdq2ps(vmm, vmm);
 
-            if (jcp.with_bias) vaddps(vmm, vmm, vmm_bias);
-
             const Vmm vmm_k = vmm_mask(vmm, mask_flag);
             vmulps(vmm_k, vmm,
                     EVEX_compress_addr(reg_ptr_scales, scale_offset));
+
+            if (jcp.with_bias) vaddps(vmm, vmm, vmm_bias);
         }
     }
 
     apply_postops(ur_w, last_oc_block_flag, nb_oc_block, oc_block, p_sum_scale,
             p_sum_zp);
 
+    if (jcp.dst_scale) {
+        mov(reg_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        vmovups(vmm_dst_scale, EVEX_compress_addr(reg_dst_scale, 0));
+
+        /* Apply dst scale to accumulator */
+        for (int k = 0; k < nb_oc_block; k++) {
+            const bool mask_flag = last_oc_block_flag && k == nb_oc_block - 1;
+            for (int j = 0; j < ur_w; j++) {
+                Vmm vmm = vmm_out(j, k);
+                const Vmm vmm_k = vmm_mask(vmm, mask_flag);
+                vmulps(vmm_k, vmm, vmm_dst_scale);
+            }
+        }
+    }
+
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
         vcvtdq2ps(vmm_zp, EVEX_compress_addr(reg_dst_zero_point, 0, true));
@@ -1458,6 +1467,11 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
     if ((jcp.dst_zero_point || jcp.src_zero_point) && jcp.is_fused_conv)
         return status::unimplemented;
 
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+    jcp.is_oc_scale = wei_scales.mask_ != 0;
+    jcp.dst_scale = !dst_scales.has_default_values();
+
     jcp.has_vnni = mayiuse(avx512_core_vnni);
     const bool bf16_req_extra_regs = cd.dst_desc.data_type == data_type::bf16
             && !isa_has_bf16(jcp.isa);
@@ -1721,13 +1735,6 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
 
     jcp.nb_ic_L2 = jcp.nb_ic;
 
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
-
-    // only common and per-oc-channel scales are supported
-    const bool oscales_ok = one_of(oscales.mask_, 0, 1 << 1);
-    if (!oscales_ok) return status::unimplemented;
-
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
             ? weights_d.extra().scale_adjust
@@ -1739,12 +1746,10 @@ status_t jit_avx512_core_x8s8s32x_fwd_kernel::init_conf(jit_conv_conf_t &jcp,
 void jit_avx512_core_x8s8s32x_fwd_kernel::init_scratchpad(
         memory_tracking::registrar_t &scratchpad, const jit_conv_conf_t &jcp,
         const primitive_attr_t &attr) {
-    if (jcp.signed_input && (!jcp.has_vnni)) {
-        dim_t count = attr.output_scales_.count_ == 1
-                ? (dim_t)16
-                : attr.output_scales_.count_;
-        scratchpad.book<float>(key_conv_adjusted_scales, count);
-    }
+    const int wei_mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const dim_t scales_count = wei_mask == 0 ? 1 : jcp.oc * jcp.ngroups;
+    dim_t count = wei_mask == 0 ? (dim_t)16 : scales_count;
+    scratchpad.book<float>(key_conv_adjusted_scales, count);
 }
 
 template struct _jit_avx512_core_x8s8s32x_fwd_kernel<Zmm>;
