@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2022 Intel Corporation
+* Copyright 2021-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 #include "common/utils.hpp"
 
 #include "cpu/cpu_primitive.hpp"
+#include "cpu/scale_utils.hpp"
 
 #include "cpu/x64/amx_tile_configure.hpp"
 #include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
@@ -49,25 +50,24 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
     const auto src_type = src_md(0)->data_type;
     const auto wei_type = weights_md(0)->data_type;
     const auto dst_type = dst_md(0)->data_type;
+    const bool is_int8 = one_of(src_type, u8, s8);
 
     using skip_mask_t = primitive_attr_t::skip_mask_t;
     auto skip_mask = skip_mask_t::post_ops | skip_mask_t::sum_dt
             | skip_mask_t::zero_points_runtime;
-    if (one_of(src_type, u8, s8)) skip_mask |= skip_mask_t::oscale;
+    if (one_of(src_type, u8, s8)) skip_mask |= skip_mask_t::scales_runtime;
 
     bool ok = is_fwd() && set_default_alg_kind(alg_kind::convolution_direct)
             && expect_data_types(src_type, wei_type, data_type::undef, dst_type,
                     data_type::undef)
-            && IMPLICATION(with_bias(),
-                    ((one_of(src_type, u8, s8)
-                             && one_of(bias_md_.data_type, f32, s32, s8, u8))
-                            || (one_of(src_type, bf16)
-                                    && one_of(bias_md_.data_type, f32, bf16))
-                            || (one_of(src_type, f32)
-                                    && one_of(bias_md_.data_type, f32))))
+            && IMPLICATION(is_int8,
+                    one_of(bias_md_.data_type, data_type::undef, f32, s32, s8,
+                            u8))
+            && IMPLICATION(!is_int8,
+                    one_of(bias_md_.data_type, data_type::undef, f32, src_type))
             && attr()->has_default_values(skip_mask, dst_type)
             && attr()->post_ops_.check_sum_consistent_dt(dst_type)
-            && !has_zero_dim_memory() && zero_points_ok();
+            && !has_zero_dim_memory() && zero_points_ok() && arg_scales_ok();
     if (!ok) return status::unimplemented;
 
     CHECK(brgemm_convolution_utils::init_1x1_conf(jcp_, isa, *desc(), src_md_,
@@ -83,10 +83,18 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
     with_sum = (sum_idx != -1);
     sum_scale = with_sum ? p.entry_[sum_idx].sum.scale : 0.0;
 
-    for_(int i_init = 0; i_init < 2; i_init++)
+    ic_chunks = div_up(jcp_.nb_ic, jcp_.nb_ic_blocking);
+    need_postwork = jcp_.with_bias || jcp_.with_eltwise || jcp_.with_binary
+            || (one_of(src_type, u8, s8) && wei_type == s8) // oscales needed
+            || (jcp_.dst_dt != jcp_.acc_dt) || jcp_.with_sum;
+
+    int i_init_begin = (ic_chunks == 1) ? 1 : 0;
+    int i_init_end = 2;
+
     for_(int i_M = 0; i_M < 2; i_M++)
     for_(int i_N = 0; i_N < 2; i_N++)
-    for (int i_K = 0; i_K < 2; i_K++) {
+    for_(int i_K = 0; i_K < 2; i_K++)
+    for (int i_init = i_init_begin; i_init < i_init_end; i_init++) {
         auto vbeta = (i_init) ? 0 : beta;
         auto vM = (i_M) ? jcp_.M_tail : jcp_.M;
         auto vN = (i_N) ? jcp_.N_tail : jcp_.N;
@@ -104,25 +112,42 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::pd_t::init(engine_t *engine) {
 
         brgemm_attr_t brgattr;
         brgattr.max_bs = jcp_.gemm_batch_size;
+        brgattr.hint_innermost_loop = jcp_.brgemm_bd_loop_innermost
+                ? brgemm_bd_loop_innermost
+                : brgemm_innermost_undef;
         brgattr.max_top_vpad = jcp_.max_vpad;
         brgattr.max_bottom_vpad = jcp_.max_vpad;
-        brgattr.hint_expected_A_size = 0;
-        brgattr.hint_expected_B_size = brgattr.max_bs * vK * vN;
-        brgattr.hint_expected_C_size = 0;
+
+        // assuming 2x2 decomposition in amx brgemm kernel
+        const auto bd_blocking = 2 * jcp_.amx_h;
+        brgattr.hint_expected_A_size = bd_blocking * vK;
+        brgattr.hint_expected_B_size = vN * vK;
+        brgattr.hint_expected_C_size = bd_blocking * vN;
+
         brgattr.wary_tail_read = false;
-        const bool is_small_mb = jcp_.mb == 1;
-        brgattr.use_uker = jcp_.use_uker && !is_small_mb;
-        brgattr.use_interleave_stores = brgattr.use_uker;
+        brgattr.use_uker = jcp_.use_uker;
+        brgattr.use_interleave_stores = jcp_.use_interleave_stores;
         brgattr.hint_prefetching = jcp_.hint_prefetching;
+        brgattr.fpmath_mode = attr()->fpmath_mode_;
+        // if post-ops are required and there are no intermediate calculations
+        // (like ic_chunks > 1) then we don't need code without post-ops in
+        // brgemm kernel
+        if (need_postwork && ic_chunks == 1) brgattr.postops_only = true;
+
         CHECK(brgemm_desc_set_attr(&brg, brgattr));
         auto LDD = jcp_.oc_without_padding;
         brg.with_sum = with_sum;
         CHECK(brgemm_desc_set_postops(
                 &brg, attr(), &dst_md_, LDD, jcp_.bia_dt));
+        jcp_.amx_buf_size_per_thread = nstl::max(
+                brg.get_wsp_buffer_size(), jcp_.amx_buf_size_per_thread);
     }
 
+    brgemm_convolution_utils::set_amx_wsp_per_thread(jcp_);
     auto scratchpad = scratchpad_registry().registrar();
     brgemm_convolution_utils::init_scratchpad(scratchpad, jcp_);
+    if (jcp_.with_scales)
+        book_precomputed_scales(scratchpad, attr()->scales_, OC());
 
     return status::success;
 }
@@ -151,8 +176,6 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::init(engine_t *engine) {
     src_dsz = jcp.src_dsz;
     wei_dsz = jcp.wei_dsz;
 
-    ic_chunks = div_up(jcp.nb_ic, jcp.nb_ic_blocking);
-
     // const variables used for address calculations
     src_w_sz = (dim_t)IW * jcp.ngroups * jcp.ic_without_padding;
     src_h_sz = IH * src_w_sz;
@@ -162,21 +185,16 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::init(engine_t *engine) {
     dst_d_sz = OD * dst_h_sz;
 
     const auto src_type = pd()->src_md(0)->data_type;
-    const auto wei_type = pd()->weights_md(0)->data_type;
 
-    const auto last_ic_block
-            = (src_type == f32) ? 1 : ((src_type == bf16) ? 2 : 4);
+    const auto last_ic_block = (isa == avx512_core_fp16 && src_type == f16)
+            ? 1
+            : data_type_vnni_granularity(src_type);
 
-    wei_oc_sz = jcp.wei_plain ? jcp.oc : jcp.oc_block;
-    wei_ic_sz = jcp.wei_plain
-            ? (dim_t)rnd_up(jcp.ic, last_ic_block) * jcp.oc
+    wei_ic_stride = jcp.wei_plain ? jcp.oc_without_padding : jcp.oc_block;
+    wei_ocb_stride = jcp.wei_plain
+            ? jcp.oc_block
             : (dim_t)rnd_up(jcp.ic, last_ic_block) * jcp.oc_block;
-    wei_ocb_sz = jcp.wei_plain ? jcp.oc_block * last_ic_block
-                               : jcp.nb_oc * wei_ic_sz;
-
-    need_postwork = jcp.with_bias || jcp.with_eltwise || jcp.with_binary
-            || (one_of(src_type, u8, s8) && wei_type == s8) // oscales needed
-            || (jcp.dst_dt != jcp.acc_dt) || jcp.with_sum;
+    wei_g_stride = jcp.wei_plain ? jcp.oc : jcp.nb_oc * wei_ocb_stride;
 
     for (int i = 0; i < 16; i++)
         brg_kernels_[i] = nullptr;
@@ -187,12 +205,14 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::init(engine_t *engine) {
                         jit_avx512_core_brgemm_conv_rtus_kernel_t(jcp)));
         CHECK(rtus_kernel_->create_kernel());
     }
+    int i_init_begin = (pd()->ic_chunks == 1) ? 1 : 0;
+    int i_init_end = 2;
 
     const bool is_amx = brgemm_convolution_utils::is_amx(isa);
     for_(int i_M = 0; i_M < 2; i_M++)
     for_(int i_N = 0; i_N < 2; i_N++)
     for_(int i_K = 0; i_K < 2; i_K++)
-    for (int i_init = 0; i_init < 2; i_init++) {
+    for (int i_init = i_init_begin; i_init < i_init_end; i_init++) {
         auto brg_idx = get_brg_idx(i_init, i_M, i_N, i_K);
         auto &brg = pd()->brgs_[brg_idx];
         if (brg.bcast_dim > 0 && brg.load_dim > 0 && brg.reduce_dim > 0
@@ -301,8 +321,9 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
         brgemm_batch_element_t *const __restrict brg_batch,
         char *const c_buffer, const char *inp_buffer, int g, int n, int ocb,
         int od, int oh, int ow, int icc, int *last_palette_idx,
-        int32_t src_zp_vals, int32_t *src_zp_comp, int32_t *dst_zp_vals,
-        int32_t *s8s8_compensation) const {
+        const float *oscales, int32_t src_zp_vals, int32_t *src_zp_comp,
+        int32_t *dst_zp_vals, int32_t *s8s8_compensation,
+        const float *dst_scales) const {
 
     const memory_desc_wrapper src_d(pd()->src_md());
     const memory_desc_wrapper weights_d(pd()->weights_md());
@@ -318,14 +339,13 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
     const std::vector<const void *> &post_ops_binary_rhs_arg_vec
             = brgemm_ctx.post_ops_binary_rhs_arg_vec;
 
-    const float *oscales = pd()->attr()->output_scales_.scales_;
-
     const auto &jcp = pd()->jcp_;
     auto ndims = pd()->ndims();
 
     const bool is_amx = brgemm_convolution_utils::is_amx(isa);
-    char *const wsp_tile
-            = is_amx ? brgemm_ctx.wsp_tile + ithr * 4 * 1024 : nullptr;
+    char *const wsp_tile = is_amx
+            ? brgemm_ctx.wsp_tile + ithr * jcp.amx_buf_size_per_thread
+            : nullptr;
 
     const int id = ndims_pick(od * SD, 0, 0);
     const int ih = ndims_pick(oh * SH, oh * SH, 0);
@@ -345,15 +365,14 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
     const bool is_os_tail = jcp.is_os_blocking ? (jcp.os - os < jcp.os_block)
                                                : (OW - ow < jcp.ow_block);
     const bool is_oc_tail = (jcp.oc - oc < jcp.oc_block);
-    const bool is_ic_tail
-            = (icc == ic_chunks - 1 && ((jcp.ic - ic) % jcp.ic_block != 0));
+    const bool is_ic_tail = (icc == pd()->ic_chunks - 1
+            && ((jcp.ic - ic) % jcp.ic_block != 0));
 
     const auto src_offset = n * src_d_sz + id * src_h_sz + ih * src_w_sz
             + iw * jcp.ngroups * jcp.ic_without_padding + g_ic;
     const auto src_base
             = jcp.is_rtus ? inp_buffer : src + src_dt_size * src_offset;
-    const auto wei_offset = jcp.wei_plain ? g * wei_ic_sz + ocb * wei_ocb_sz
-                                          : g * wei_ocb_sz + ocb * wei_ic_sz;
+    const auto wei_offset = g * wei_g_stride + ocb * wei_ocb_stride;
     const auto wei_base = weights + wei_dt_size * wei_offset;
     const auto ptr_D = dst
             + dst_dt_size
@@ -367,38 +386,44 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
             - (is_ic_tail ? 1 : 0);
 
     const auto comp_offset = (g * jcp.nb_oc + ocb) * jcp.oc_block;
-    int32_t *src_zp_comp_ptr = (jcp.src_zero_point && icc == ic_chunks - 1)
+    int32_t *src_zp_comp_ptr
+            = (jcp.src_zero_point && icc == pd()->ic_chunks - 1)
             ? &src_zp_comp[comp_offset]
             : nullptr;
-    int32_t *s8s8_comp_ptr = (jcp.s8s8_avx512 && icc == ic_chunks - 1)
+    int32_t *s8s8_comp_ptr = (jcp.s8s8_avx512 && icc == pd()->ic_chunks - 1)
             ? &s8s8_compensation[comp_offset]
             : nullptr;
 
     const auto call_brgemm = [=](int brg_idx, int ic_block_s, int n_ic_blocks,
                                      bool do_postops) {
-        for (int k = 0; k < n_ic_blocks; k++) {
-            const auto ic_off = (ic_block_s + k) * jcp.ic_block;
-            const auto src_ic = ic_off;
-            const auto wei_ic = ic + ic_off;
-            const auto ptr_A = src_base + src_dt_size * src_ic;
-            const auto ptr_B = wei_base + wei_dt_size * wei_ic * wei_oc_sz;
-            brg_batch[k].ptr.A = ptr_A;
-            brg_batch[k].ptr.B = ptr_B;
-            brg_batch[k].vvpad.top = 0;
-            brg_batch[k].vvpad.bottom = 0;
-        }
-
         // NOTE: avoid some costly tile reconfigurations here by keeping track
         //       of the previous brg kernel tile configuration palette
         // TODO: adjust harness to avoid even more tile reconfigurations
         if (is_amx) {
             const int curr_palette_idx = brg_kernel_palette_idx_[brg_idx];
             if (curr_palette_idx != *last_palette_idx) {
-                amx_tile_configure(brg_kernel_palette_[curr_palette_idx].p);
+                if (*last_palette_idx == -1
+                        || std::memcmp(brg_kernel_palette_[curr_palette_idx].p,
+                                   brg_kernel_palette_[*last_palette_idx].p,
+                                   AMX_PALETTE_SIZE)
+                                != 0)
+                    amx_tile_configure(brg_kernel_palette_[curr_palette_idx].p);
                 *last_palette_idx = curr_palette_idx;
             }
         }
 
+        for (int k = 0; k < n_ic_blocks; k++) {
+            const auto ic_off = (ic_block_s + k) * jcp.ic_block;
+            const auto src_ic = ic_off;
+            const auto wei_ic = ic + ic_off;
+            const auto ptr_A = src_base + src_dt_size * src_ic;
+            const auto ptr_B = wei_base + wei_dt_size * wei_ic * wei_ic_stride;
+            brg_batch[k].ptr.A = ptr_A;
+            brg_batch[k].ptr.B = ptr_B;
+            brg_batch[k].vvpad.top = 0;
+            brg_batch[k].vvpad.bottom = 0;
+        }
+
         const brgemm_kernel_t *brg_ker = brg_kernels_[brg_idx].get();
         if (do_postops) {
             const brgemm_post_ops_data_t post_ops_data {
@@ -407,7 +432,8 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
                     post_ops_binary_rhs_arg_vec.data(),
                     static_cast<size_t>(g_oc), 0, dst, 0,
                     static_cast<void *>(src_zp_comp_ptr), nullptr,
-                    static_cast<void *>(dst_zp_vals), false, src_zp_vals};
+                    static_cast<void *>(dst_zp_vals), false, src_zp_vals, false,
+                    false, dst_scales};
 
             void *scratch = is_amx ? static_cast<void *>(wsp_tile)
                                    : static_cast<void *>(s8s8_comp_ptr);
@@ -421,8 +447,8 @@ void brgemm_1x1_convolution_fwd_t<isa>::exec_ker(
         }
     };
 
-    const auto do_post_work
-            = (need_postwork || jcp.use_buffer) && icc == ic_chunks - 1;
+    const auto do_post_work = (pd()->need_postwork || jcp.use_buffer)
+            && icc == pd()->ic_chunks - 1;
 
     if (nb_ic_b > 0) {
         const auto brg_idx
@@ -450,6 +476,13 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
     const bool is_amx = brgemm_convolution_utils::is_amx(isa);
     const memory_desc_wrapper weights_d(pd()->weights_md(0));
 
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = precompute_scales(ctx.get_scratchpad_grantor(),
+            src_scales, wei_scales, pd()->OC(), pd()->attr());
+
     DEFINE_ZERO_POINT_VALUE(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINT_VALUE(dst_zero_point, DNNL_ARG_DST);
 
@@ -519,14 +552,15 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
                 char *inp_buffer_sp = (jcp.is_rtus) \
                         ? inp_buffer + src_dsz * os * jcp.LDA \
                         : nullptr; \
-                for (int icc = 0; icc < ic_chunks; icc++) { \
+                for (int icc = 0; icc < pd()->ic_chunks; icc++) { \
                     if (jcp.is_rtus) \
                         maybe_rtus(ithr, brgemm_ctx.src, inp_buffer_sp, \
                                 inp_buffer_mask, g, n, icc, od, oh, ow); \
                     exec_ker(brgemm_ctx, ithr, brg_batch, c_buffer, \
                             inp_buffer_sp, g, n, ocb, od, oh, ow, icc, \
-                            &last_palette_idx, src_zero_point, \
-                            zp_compensation, dst_zp_vals, s8s8_compensation); \
+                            &last_palette_idx, oscales, src_zero_point, \
+                            zp_compensation, dst_zp_vals, s8s8_compensation, \
+                            dst_scales); \
                 } \
             } \
             last_n = n; \
@@ -563,12 +597,12 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
         int n {0}, g {0}, ocb {0}, od {0}, oh {0}, owb {0}; \
         nd_iterator_init(start, __VA_ARGS__); \
         for (auto work = start; work < end; work++) { \
-            for (int icc = 0; icc < ic_chunks; icc++) { \
+            for (int icc = 0; icc < pd()->ic_chunks; icc++) { \
                 const int ow = owb * jcp.ow_block; \
                 exec_ker(brgemm_ctx, ithr, brg_batch, c_buffer, nullptr, g, n, \
-                        ocb, od, oh, ow, icc, &last_palette_idx, \
+                        ocb, od, oh, ow, icc, &last_palette_idx, oscales, \
                         src_zero_point, zp_compensation, dst_zp_vals, \
-                        s8s8_compensation); \
+                        s8s8_compensation, dst_scales); \
             } \
             nd_iterator_step(__VA_ARGS__); \
         } \
@@ -590,11 +624,14 @@ status_t brgemm_1x1_convolution_fwd_t<isa>::execute_forward_all(
     return status::success;
 }
 
+template struct brgemm_1x1_convolution_fwd_t<avx2>;
+template struct brgemm_1x1_convolution_fwd_t<avx2_vnni_2>;
 template struct brgemm_1x1_convolution_fwd_t<avx512_core>;
 template struct brgemm_1x1_convolution_fwd_t<avx512_core_vnni>;
 template struct brgemm_1x1_convolution_fwd_t<avx512_core_bf16>;
-template struct brgemm_1x1_convolution_fwd_t<avx512_core_bf16_amx_int8>;
-template struct brgemm_1x1_convolution_fwd_t<avx512_core_bf16_amx_bf16>;
+template struct brgemm_1x1_convolution_fwd_t<avx512_core_fp16>;
+template struct brgemm_1x1_convolution_fwd_t<avx512_core_amx>;
+template struct brgemm_1x1_convolution_fwd_t<avx512_core_amx_fp16>;
 
 } // namespace x64
 } // namespace cpu
