@@ -19,8 +19,9 @@
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 #include "cpu/cpu_primitive.hpp"
-#include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
+#include "cpu/scale_utils.hpp"
 
+#include "cpu/x64/injectors/jit_uni_binary_injector.hpp"
 #include "cpu/x64/jit_brgemm_conv.hpp"
 
 namespace dnnl {
@@ -50,23 +51,32 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::pd_t::init(
     const auto src_type = src_md(0)->data_type;
     const auto wei_type = weights_md(0)->data_type;
     const auto dst_type = dst_md(0)->data_type;
+    const bool is_int8 = one_of(src_type, u8, s8);
+
+    // The following check will detect if this implementation is being
+    // executed through a BWD_D Convolution call and prevent the primitive from
+    // executing 'use_inversion == true' as FWD. This can only work if the
+    // diff_src_desc and diff_dst_desc are defined in the aforementioned.
+    const convolution_desc_t &cd = *desc();
+    if (use_inversion
+            && one_of(true, types::is_zero_md(&cd.diff_src_desc),
+                    types::is_zero_md(&cd.diff_dst_desc)))
+        return status::unimplemented;
 
     using skip_mask_t = primitive_attr_t::skip_mask_t;
     auto skip_mask = skip_mask_t::post_ops | skip_mask_t::sum_dt
             | skip_mask_t::zero_points_runtime;
-    if (one_of(src_type, u8, s8)) skip_mask |= skip_mask_t::oscale_runtime;
+    if (is_int8) skip_mask |= skip_mask_t::scales_runtime;
 
     bool ok = is_fwd() && set_default_alg_kind(alg_kind::convolution_direct)
-            && IMPLICATION(with_bias(),
-                    ((one_of(src_type, u8, s8)
-                             && one_of(bias_md_.data_type, f32, s32, s8, u8))
-                            || (one_of(src_type, bf16)
-                                    && one_of(bias_md_.data_type, f32, bf16))
-                            || (one_of(src_type, f32)
-                                    && one_of(bias_md_.data_type, f32))))
+            && IMPLICATION(is_int8,
+                    one_of(bias_md_.data_type, data_type::undef, f32, s32, s8,
+                            u8))
+            && IMPLICATION(!is_int8,
+                    one_of(bias_md_.data_type, data_type::undef, f32, src_type))
             && attr()->has_default_values(skip_mask, dst_type)
             && attr()->post_ops_.check_sum_consistent_dt(dst_type)
-            && !has_zero_dim_memory() && zero_points_ok();
+            && !has_zero_dim_memory() && zero_points_ok() && arg_scales_ok();
     if (!ok) return status::unimplemented;
     const auto is_amx = brgemm_convolution_utils::is_amx(isa);
 
@@ -75,10 +85,12 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::pd_t::init(
 
     const auto adj_M = nstl::max(jcp_.M, jcp_.M_tail);
 
-    // 1. Use unrolled kernel for exec_trans only to avoid creation a lot of
-    //    kernels for each kw range
+    // 1. The unrolled kernel can be used for exec_trans and exec_base and for
+    // amx only. For exec_base it makes sense to use unrolled kernel only if
+    // there is no padding by width.
     // 2. For exec_trans block by kw is always KW
-    assert(IMPLICATION(jcp_.use_uker, is_amx && jcp_.exec_type == exec_trans));
+    assert(IMPLICATION(jcp_.use_uker,
+            is_amx && one_of(jcp_.exec_type, exec_base, exec_trans)));
     assert(IMPLICATION(jcp_.use_interleave_stores, jcp_.use_uker));
 
     batchsizes.resize(jcp_.max_batch + 1);
@@ -87,20 +99,25 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::pd_t::init(
 
     first_bs = 0;
     bs_c = 0;
-    if (jcp_.use_uker) {
-        const auto SD = jcp_.stride_d;
-        const auto FP = jcp_.f_pad;
-        const auto DD = jcp_.dilate_d + 1;
-        const auto KD = jcp_.kd;
-        const auto ID = jcp_.id;
 
-        const auto SH = jcp_.stride_h;
-        const auto TP = jcp_.t_pad;
-        const auto DH = jcp_.dilate_h + 1;
-        const auto KH = jcp_.kh;
-        const auto IH = jcp_.ih;
-        const auto KD_BLOCK = jcp_.kd_block;
-        const auto KH_BLOCK = jcp_.kh_block;
+    const auto SD = jcp_.stride_d;
+    const auto FP = jcp_.f_pad;
+    const auto DD = jcp_.dilate_d + 1;
+    const auto KD = jcp_.kd;
+    const auto ID = jcp_.id;
+
+    const auto SH = jcp_.stride_h;
+    const auto TP = jcp_.t_pad;
+    const auto DH = jcp_.dilate_h + 1;
+    const auto KH = jcp_.kh;
+    const auto KW = jcp_.kw;
+    const auto IH = jcp_.ih;
+
+    const auto KD_BLOCK = jcp_.kd_block;
+    const auto KH_BLOCK = jcp_.kh_block;
+    const auto KW_BLOCK = jcp_.kw_block;
+
+    if (jcp_.use_uker) {
 
         assert(KD % KD_BLOCK == 0);
         assert(KH % KH_BLOCK == 0);
@@ -184,18 +201,37 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::pd_t::init(
         brgattr.bd_mask = bd_mask;
     };
 
+    ic_chunks = div_up(jcp_.nb_ic, jcp_.nb_ic_blocking);
+    need_postwork = jcp_.with_bias || jcp_.with_eltwise || jcp_.with_binary
+            || (one_of(src_type, u8, s8) && wei_type == s8) // oscales needed
+            || (jcp_.dst_dt != jcp_.acc_dt) || jcp_.with_sum || jcp_.use_M_mask
+            || jcp_.src_zero_point || jcp_.dst_zero_point;
+
     const auto M_end = nstl::max(jcp_.M, jcp_.M_tail);
+
+    int K_begin = 0;
+    int K_end = (jcp_.K_tail == 0) ? 1 : 2;
+
+    int i_init_begin = (jcp_.K_tail == 0 && jcp_.exec_type == exec_trans
+                               && div_up(jcp_.nb_ic, jcp_.nb_ic_blocking) == 1
+                               && KD_BLOCK == KD && KH_BLOCK == KH)
+            ? 1
+            : 0;
+    int i_init_end = 2;
+
     for (int i = 0; i < M_end; i++) {
         auto vM = i + 1;
         // init only needed brgemm descriptors
-        if (one_of(jcp_.exec_type, exec_trans, exec_vpad) && vM != jcp_.M
-                && vM != jcp_.M_tail)
+        if ((one_of(jcp_.exec_type, exec_trans, exec_vpad)
+                    || (jcp_.exec_type == exec_base && jcp_.l_pad == 0
+                            && jcp_.r_pad == 0))
+                && vM != jcp_.M && vM != jcp_.M_tail)
             continue;
         for (int bs = 0; bs <= jcp_.max_batch; bs++) {
             if (batchsizes[bs] == -1) continue;
-            for_(int i_init = 0; i_init < 2; i_init++)
+            for_(int i_init = i_init_begin; i_init < i_init_end; i_init++)
             for_(int i_N = 0; i_N < 2; i_N++)
-            for (int i_K = 0; i_K < 2; i_K++) {
+            for (int i_K = K_begin; i_K < K_end; i_K++) {
                 auto vbeta = (i_init) ? 0 : beta;
                 auto vN = (i_N) ? jcp_.N_tail : jcp_.N;
                 auto vK = (i_K) ? jcp_.K_tail : jcp_.K;
@@ -228,7 +264,7 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::pd_t::init(
                 brgattr.max_bs = bs;
                 brgattr.hint_innermost_loop = jcp_.brgemm_bd_loop_innermost
                         ? brgemm_bd_loop_innermost
-                        : brgemm_ld_loop_innermost;
+                        : brgemm_innermost_undef;
                 if (jcp_.amx_tile_load_xx) {
                     // assuming 2x2 decomposition in amx brgemm kernel
                     // and overlap of input by kw
@@ -257,6 +293,15 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::pd_t::init(
                     brgattr.max_bottom_vpad = jcp_.max_vpad;
                 }
                 brgattr.fpmath_mode = attr()->fpmath_mode_;
+                brgattr.K_koef = (float)bs / KW;
+
+                // if need post_ops and there are no intermediate calculations
+                // (like ic_chunks > 1 or blocking by kernel) we don't need
+                // code without post-ops in brgemm kernel
+                if (need_postwork && ic_chunks == 1 && KD_BLOCK == KD
+                        && KH_BLOCK == KH && KW_BLOCK == KW)
+                    brgattr.postops_only = true;
+
                 CHECK(brgemm_desc_set_attr(brg, brgattr));
 
                 auto LDD = jcp_.oc_without_padding;
@@ -273,6 +318,8 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::pd_t::init(
     brgemm_convolution_utils::set_amx_wsp_per_thread(jcp_);
     auto scratchpad = scratchpad_registry().registrar();
     brgemm_convolution_utils::init_scratchpad(scratchpad, jcp_);
+    if (jcp_.with_scales)
+        book_precomputed_scales(scratchpad, attr()->scales_, OC());
 
     return status::success;
 }
@@ -382,14 +429,14 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::add_po_kernel(
     bcfg->alpha = !is_init && IMPLICATION(jcp.with_sum, jcp.use_buffer);
     bcfg->beta = is_init ? 0 : 1;
     CHECK(safe_ptr_assign(kernels_po_[ker_idx],
-            new jit_brgemm_kernel_post_ops(jcp, *bcfg, *_pd->attr())));
+            new jit_brgemm_kernel_post_ops<isa>(jcp, *bcfg, *_pd->attr())));
     kernels_po_[ker_idx]->create_kernel();
     return status::success;
 }
 
 template <cpu_isa_t isa, bool use_inversion>
 void brgemm_convolution_fwd_t<isa, use_inversion>::add_po_kernels(
-        int i_N, int init_bcast_dim, int po_bcast_dim, bool need_postwork) {
+        int i_N, int init_bcast_dim, int po_bcast_dim) {
     const auto _pd = pd();
     const auto &jcp = _pd->jcp_;
     const auto &brgs = _pd->brgs_;
@@ -411,7 +458,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::add_po_kernels(
         }
     }
 
-    if ((need_postwork || jcp.use_buffer) && po_bcast_dim > 0) {
+    if ((_pd->need_postwork || jcp.use_buffer) && po_bcast_dim > 0) {
         auto brg_idx = _pd->get_brg_idx(
                 _pd->first_bs, po_bcast_dim - 1, 0, i_N, i_K);
         if (brgs[brg_idx]) {
@@ -510,8 +557,6 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
     DH = ndims_pick(jcp.dilate_h, jcp.dilate_h, 0) + 1;
     DW = jcp.dilate_w + 1;
 
-    ic_chunks = div_up(jcp.nb_ic, jcp.nb_ic_blocking);
-
     // const variables used for address calculations
     src_w_sz = static_cast<dim_t>(IW) * jcp.ngroups * jcp.ic_without_padding;
     src_h_sz = IH * src_w_sz;
@@ -520,13 +565,13 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
     dst_h_sz = OH * dst_w_sz;
     dst_d_sz = OD * dst_h_sz;
 
-    const auto src_type = pd()->src_md(0)->data_type;
-    const auto wei_type = pd()->weights_md(0)->data_type;
-    wei_ic_sz = static_cast<dim_t>(jcp.icp) * jcp.oc_block;
-    wei_kw_sz = KW * wei_ic_sz;
-    wei_kh_sz = KH * wei_kw_sz;
-    wei_kd_sz = KD * wei_kh_sz;
-    wei_ocb_sz = jcp.nb_oc * wei_kd_sz;
+    wei_kw_stride = static_cast<dim_t>(jcp.icp)
+            * (jcp.wei_plain ? jcp.oc_without_padding : jcp.oc_block);
+    wei_kh_stride = KW * wei_kw_stride;
+    wei_kd_stride = KH * wei_kh_stride;
+    wei_ocb_stride = jcp.wei_plain ? jcp.oc_block : KD * wei_kd_stride;
+    wei_g_stride = jcp.wei_plain ? jcp.oc : jcp.nb_oc * wei_ocb_stride;
+    wei_ic_stride = jcp.wei_plain ? jcp.oc_without_padding : jcp.oc_block;
 
     comp_kw_sz = static_cast<dim_t>(jcp.oc_block);
     comp_ker_sz = jcp.ker_ranges_size * comp_kw_sz;
@@ -534,10 +579,6 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
 
     need_compensation
             = (jcp.src_zero_point || jcp.s8s8_avx512) && !jcp.req_brg_comp_pad;
-    need_postwork = jcp.with_bias || jcp.with_eltwise || jcp.with_binary
-            || (one_of(src_type, u8, s8) && wei_type == s8) // oscales needed
-            || (jcp.dst_dt != jcp.acc_dt) || jcp.with_sum || jcp.use_M_mask
-            || jcp.src_zero_point || jcp.dst_zero_point;
 
     // ---- Initialize arrays ---------------------
     brg_kernels_.resize(_pd->brgs_sz_);
@@ -554,10 +595,13 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
             kernels_po_[get_ker_po_idx(i, i_init, i_N)] = nullptr;
     }
 
-    CHECK(safe_ptr_assign(copy_to_pbuffer_,
-            new jit_avx512_core_brgemm_conv_trans_kernel_t(jcp)));
-    CHECK(copy_to_pbuffer_->create_kernel());
+    if (jcp.exec_type == exec_trans) {
+        CHECK(safe_ptr_assign(copy_to_pbuffer_,
+                new jit_avx512_core_brgemm_conv_trans_kernel_t(jcp)));
+        CHECK(copy_to_pbuffer_->create_kernel());
+    }
     if (jcp.copy_block_only) {
+        assert(jcp.exec_type == exec_trans && "Missing copy kernel");
         const auto iw_block = copy_to_pbuffer_->dst_w(jcp.ow_block);
         const auto ih_block = get_inp_size(IHP, jcp.oh_block, KH, SH, DH - 1);
         const auto id_block = get_inp_size(IDP, jcp.od_block, KD, SD, DD - 1);
@@ -572,9 +616,11 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
         pbuf_d_sz = pbuf_h_sz * jcp.idp;
     }
 
-    CHECK(safe_ptr_assign(comp_vpad_pbuffer_,
-            new jit_avx512_core_brgemm_conv_comp_pad_kernel_t(jcp)));
-    CHECK(comp_vpad_pbuffer_->create_kernel());
+    if (jcp.req_cal_comp_pad) {
+        CHECK(safe_ptr_assign(comp_vpad_pbuffer_,
+                new jit_avx512_core_brgemm_conv_comp_pad_kernel_t(jcp)));
+        CHECK(comp_vpad_pbuffer_->create_kernel());
+    }
 
     is_amx = brgemm_convolution_utils::is_amx(isa);
 
@@ -584,8 +630,9 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
     int N_begin = 0;
     int N_end = (jcp.N_tail == jcp.N) ? 1 : 2;
     int K_begin = 0;
-    int K_end = (jcp.K_tail == jcp.K) ? 1 : 2;
-    int i_init_begin = (div_up(jcp.nb_ic, jcp.nb_ic_blocking) == 1
+    int K_end = (jcp.K_tail == 0) ? 1 : 2;
+    int i_init_begin = (jcp.K_tail == 0 && jcp.exec_type == exec_trans
+                               && div_up(jcp.nb_ic, jcp.nb_ic_blocking) == 1
                                && KD_BLOCK == KD && KH_BLOCK == KH)
             ? 1
             : 0;
@@ -607,12 +654,19 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
     for_(int i_N = N_begin; i_N < N_end; i_N++)
     for (int i_M = M_begin; i_M < M_end; i_M++) {
         // init "init" and "po" kernels for cases then we never call brgemm kernels
-        // e.g. for d/h padded areas
+        // e.g. for d/h padded and dilated filter areas
+        const bool filter_in_padding = jcp.f_pad >= EXT_KD
+                || jcp.back_pad >= EXT_KD || jcp.t_pad >= EXT_KH
+                || jcp.b_pad >= EXT_KH;
+        // note: overly simplistic condition. Ideally, the condition would
+        // only detect cases where there is strictly no overlap between the
+        // input and filter.
+        const bool dilate_no_overlap
+                = jcp.dilate_d >= jcp.id || jcp.dilate_h >= jcp.ih;
         if (IMPLICATION(jcp.exec_type == exec_trans,
-                    jcp.f_pad > EXT_KD || jcp.back_pad > EXT_KD
-                            || jcp.t_pad > EXT_KH || jcp.b_pad > EXT_KH)) {
+                    filter_in_padding || dilate_no_overlap)) {
             auto M = (i_M) ? jcp.M_tail : jcp.M;
-            add_po_kernels(i_N, M, M, need_postwork);
+            add_po_kernels(i_N, M, M);
         }
     }
 
@@ -650,8 +704,7 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
                 get_ow_range(ow, kw_f - 1, ow_s, ow_f);
                 const auto po_bcast_dim
                         = (i_side == 0) ? (ow_s - ow) : (ow + M - ow_f);
-                add_po_kernels(
-                        i_N, init_bcast_dim, po_bcast_dim, need_postwork);
+                add_po_kernels(i_N, init_bcast_dim, po_bcast_dim);
             }
 
             if (kw_f == jcp.kw && kw_s == 0) break;
@@ -688,8 +741,7 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::init(engine_t *engine) {
                 get_ow_range(ow, kw_f - 1, ow_s, ow_f);
                 const auto po_bcast_dim
                         = (i_side == 0) ? (ow_s - ow) : (ow + M - ow_f);
-                add_po_kernels(
-                        i_N, init_bcast_dim, po_bcast_dim, need_postwork);
+                add_po_kernels(i_N, init_bcast_dim, po_bcast_dim);
             }
 
             if (kw_f == jcp.kw && kw_s == 0) break;
@@ -824,7 +876,7 @@ struct brgemm_convolution_fwd_t<isa, use_inversion>::brgemm_thread_ctx_t {
     brgemm_batch_element_t *__restrict brg_batch;
     char *c_buffer;
     char *wsp_tile;
-    S_t cur_palette;
+    int cur_brg_idx = -1;
     int g, n, ocb;
     int od, odb, oh, ohb, owb;
     int icc;
@@ -833,6 +885,22 @@ struct brgemm_convolution_fwd_t<isa, use_inversion>::brgemm_thread_ctx_t {
     int32_t *src_zp_comp_ptr;
     int32_t *dst_zp_vals;
     int32_t *s8s8_comp_ptr;
+    const float *dst_scales {nullptr};
+    void maybe_tile_configure(bool is_amx,
+            const std::vector<brgemm_convolution_fwd_t::S_t>
+                    &brg_kernel_palettes,
+            int brg_ker_idx) {
+        // TODO: avoid costly tile reconfigurations
+        if (!is_amx) return;
+        if (brg_ker_idx == cur_brg_idx) return;
+        // TODO: more accurately estimate the costs of memcmp and tile configuration
+        if (cur_brg_idx == -1
+                || std::memcmp(brg_kernel_palettes[brg_ker_idx].a,
+                           brg_kernel_palettes[cur_brg_idx].a, AMX_PALETTE_SIZE)
+                        != 0)
+            amx_tile_configure(brg_kernel_palettes[brg_ker_idx].a);
+        cur_brg_idx = brg_ker_idx;
+    }
 };
 
 template <cpu_isa_t isa, bool use_inversion>
@@ -844,7 +912,12 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::execute(
     DEFINE_ZERO_POINT_VALUE(src_zero_point, DNNL_ARG_SRC);
     DEFINE_ZERO_POINT_VALUE(dst_zero_point, DNNL_ARG_DST);
 
-    DEFINE_SCALES_BUFFER(oscales);
+    DEFINE_ARG_SCALES_BUFFER(src_scales, DNNL_ARG_SRC);
+    DEFINE_ARG_SCALES_BUFFER(wei_scales, DNNL_ARG_WEIGHTS);
+    DEFINE_ARG_SCALES_BUFFER(dst_scales, DNNL_ARG_DST);
+
+    const float *oscales = precompute_scales(ctx.get_scratchpad_grantor(),
+            src_scales, wei_scales, _pd->OC(), _pd->attr());
 
     brgemm_exec_ctx_t brgemm_ctx(ctx, _pd);
 
@@ -949,7 +1022,6 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::execute(
 
         brgemm_thread_ctx_t btc(
                 brgemm_ctx, ithr, brg_batch, c_buffer, wsp_tile);
-        std::memset(btc.cur_palette.a, 0, AMX_PALETTE_SIZE);
 
         int last_n = -1;
         int last_g = -1;
@@ -970,6 +1042,7 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::execute(
             btc.src_zp_comp_ptr
                     = jcp.src_zero_point ? src_zp_comp_base : nullptr;
             btc.s8s8_comp_ptr = jcp.s8s8_avx512 ? s8s8_comp_base : nullptr;
+            btc.dst_scales = dst_scales;
 
             if (jcp.exec_type == exec_trans && (last_n != n || last_g != g)) {
                 if (!jcp.copy_block_only)
@@ -986,7 +1059,7 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::execute(
                     : nstl::min(OH, oh_begin + jcp.oh_block);
             for_(int od = od_begin; od < od_end; od++)
             for (int oh = oh_begin; oh < oh_end; oh++) {
-                for (int icc = 0; icc < ic_chunks; icc++) {
+                for (int icc = 0; icc < _pd->ic_chunks; icc++) {
                     btc.od = od;
                     btc.oh = oh;
                     btc.icc = icc;
@@ -1063,8 +1136,9 @@ status_t brgemm_convolution_fwd_t<isa, use_inversion>::cal_compensation(
 
             const auto buffer_offs
                     = g * comp_ocb_sz + ocb * comp_ker_sz + k * comp_kw_sz;
-            const auto wei_offs = (g * jcp.nb_oc + ocb) * wei_kd_sz
-                    + kd_b * wei_kh_sz + kh_b * wei_kw_sz + kw_b * wei_ic_sz;
+            const auto wei_offs = g * wei_g_stride + ocb * wei_ocb_stride
+                    + kd_b * wei_kd_stride + kh_b * wei_kh_stride
+                    + kw_b * wei_kw_stride;
 
             jit_brgemm_conv_comp_pad_call_s p;
 
@@ -1091,7 +1165,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::perform_outwork(
         int kd_l, int kh_l, const void *post_ops_binary_rhs_arg_vec,
         const float *oscales, int32_t src_zp_vals, int32_t *src_zp_ptr,
         int32_t *dst_zp_ptr, int32_t *s8s8_compensation, bool maybe_do_init,
-        bool do_postwork, bool do_post_comp) const {
+        bool do_postwork, bool do_post_comp, const float *dst_scales) const {
 
     const auto _pd = pd();
     const auto &jcp = _pd->jcp_;
@@ -1118,6 +1192,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::perform_outwork(
         p.dst_orig = dst;
         p.c_zp_values = dst_zp_ptr;
         p.a_comp_val = src_zp_vals;
+        p.ptr_dst_scales = (void *)dst_scales;
     }
 
     auto call_outwork_ker = [&](bool is_postwork, bool has_postcomp,
@@ -1182,17 +1257,6 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::call_brgemm_kernel(
     const auto brg_ker = brg_kernels_[brg_idx].get();
     assert(brg_ker != nullptr);
 
-    // TODO: avoid costly tile reconfigurations
-    if (is_amx) {
-        if (std::memcmp(btc.cur_palette.a, brg_kernel_palettes_[brg_idx].a,
-                    AMX_PALETTE_SIZE)
-                != 0) {
-            amx_tile_configure(brg_kernel_palettes_[brg_idx].a);
-            std::memcpy(btc.cur_palette.a, brg_kernel_palettes_[brg_idx].a,
-                    AMX_PALETTE_SIZE);
-        }
-    }
-
     const auto do_only_pass_comp = !do_postops && jcp.src_zero_point
             && (jcp.req_brg_comp_pad || jcp.max_vpad > 0);
     const auto maybe_do_postops
@@ -1204,7 +1268,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::call_brgemm_kernel(
                 static_cast<size_t>(g_oc), 0, btc.brgemm_ctx.dst, 0,
                 static_cast<void *>(src_zp_ptr), nullptr,
                 static_cast<void *>(dst_zp_ptr), false, src_zp_vals,
-                do_only_comp, do_only_pass_comp};
+                do_only_comp, do_only_pass_comp, btc.dst_scales};
 
         void *scratch = is_amx ? static_cast<void *>(btc.wsp_tile)
                                : static_cast<void *>(s8s8_comp);
@@ -1390,7 +1454,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::maybe_conv_inp(int ithr,
     const auto kh_f = ndims_pick(kh_f_, kh_f_, 1); \
     const auto kh_l = kh_f - kh_s; \
     const bool is_oc_tail = (jcp.oc - oc < jcp.oc_block); \
-    const bool is_ic_tail = (btc.icc == ic_chunks - 1 \
+    const bool is_ic_tail = (btc.icc == _pd->ic_chunks - 1 \
             && ((jcp.ic - ic) % jcp.ic_block != 0)); \
     const bool is_ow_tail = (OW - ow < jcp.ow_block); \
     const bool is_oh_tail = (OH - oh < jcp.oh_block); \
@@ -1421,13 +1485,14 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_base(
     get_kw_range(ow, kw_s, kw_full_s, kw_full_f, kw_f);
 
     const auto src_base = src + src_dsz * (btc.n * src_d_sz + g_ic);
-    const auto wei_base
-            = weights + wei_dsz * (btc.g * wei_ocb_sz + btc.ocb * wei_kd_sz);
+    const auto wei_base = weights
+            + wei_dsz * (btc.g * wei_g_stride + btc.ocb * wei_ocb_stride);
 
     const auto call_brgemm = [&](int brg_idx, int ic_block_s, int n_ic_blocks,
                                      int32_t *src_zp, int32_t *s8s8_comp,
                                      bool do_postops, bool do_only_comp) {
         if (k_l <= 0) return;
+        btc.maybe_tile_configure(is_amx, brg_kernel_palettes_, brg_idx);
 
         for (int i_icb = 0; i_icb < n_ic_blocks; i_icb++) {
             const auto ic_off = (ic_block_s + i_icb) * jcp.ic_block;
@@ -1435,20 +1500,21 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_base(
             const auto wei_ic = ic + ic_off;
             const auto n_icb_off = i_icb * k_l;
             const auto src_base_ic = src_base + src_dsz * src_ic;
-            const auto wei_base_ic = wei_base + wei_dsz * wei_ic * jcp.oc_block;
+            const auto wei_base_ic
+                    = wei_base + wei_dsz * wei_ic * wei_ic_stride;
 
             auto k = 0;
             for (int kd = kd_b; kd < kd_e; kd++) {
                 const auto id = iid + kd * DD;
                 const auto src_base_kd = src_base_ic + src_dsz * id * src_h_sz;
                 const auto wei_base_kd = wei_base_ic
-                        + wei_dsz * maybe_invert(kd, KD) * wei_kh_sz;
+                        + wei_dsz * maybe_invert(kd, KD) * wei_kd_stride;
                 for (int kh = kh_b; kh < kh_e; kh++) {
                     const auto ih = iih + kh * DH;
                     const auto src_base_kh
                             = src_base_kd + src_dsz * ih * src_w_sz;
                     const auto wei_base_kh = wei_base_kd
-                            + wei_dsz * maybe_invert(kh, KH) * wei_kw_sz;
+                            + wei_dsz * maybe_invert(kh, KH) * wei_kh_stride;
                     for (int kw = kw_b; kw < kw_e; kw++) {
                         const auto iw = iiw_b + kw * DW;
                         btc.brg_batch[n_icb_off + k].ptr.A = src_base_kh
@@ -1458,7 +1524,8 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_base(
                         btc.brg_batch[n_icb_off + k].vvpad.bottom = 0;
                         // general wei layout is gOdhwI<block_o><block_i>
                         btc.brg_batch[n_icb_off + k].ptr.B = wei_base_kh
-                                + wei_dsz * maybe_invert(kw, KW) * wei_ic_sz;
+                                + wei_dsz * maybe_invert(kw, KW)
+                                        * wei_kw_stride;
                         k++;
                     }
                 }
@@ -1477,10 +1544,12 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_base(
 
         const auto do_init
                 = btc.icc == 0 && kd_b == kd_s && kh_b == kh_s && kw_b == kw_s;
-        const auto do_postwork = need_postwork && btc.icc == (ic_chunks - 1)
-                && kd_e == kd_f && kh_e == kh_f && kw_e == kw_f;
+        const auto do_postwork = _pd->need_postwork
+                && btc.icc == (_pd->ic_chunks - 1) && kd_e == kd_f
+                && kh_e == kh_f && kw_e == kw_f;
         const auto do_only_comp = need_compensation && kd_e == kd_f
-                && kh_e == kh_f && kw_e != kw_f && btc.icc == (ic_chunks - 1);
+                && kh_e == kh_f && kw_e != kw_f
+                && btc.icc == (_pd->ic_chunks - 1);
         if (ow_e - ow_b <= 0 && !do_init && !do_postwork) return;
 
         k_l = (kd_e - kd_b) * (kh_e - kh_b) * (kw_e - kw_b);
@@ -1537,7 +1606,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_base(
                 g_oc, is_oc_tail, ow_b, ow_e, kd_l, kh_l,
                 post_ops_binary_rhs_arg_vec.data(), btc.oscales,
                 btc.src_zp_vals, btc.src_zp_comp_ptr, btc.dst_zp_vals,
-                btc.s8s8_comp_ptr, do_init, do_postwork, false);
+                btc.s8s8_comp_ptr, do_init, do_postwork, false, btc.dst_scales);
     };
 
     if (kd_f > kd_s && kh_f > kh_s && kw_f > kw_s) {
@@ -1586,12 +1655,13 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_base(
         }
     } else {
         const auto do_init = btc.icc == 0;
-        const auto do_postwork = need_postwork && btc.icc == (ic_chunks - 1);
+        const auto do_postwork
+                = _pd->need_postwork && btc.icc == (_pd->ic_chunks - 1);
         perform_outwork(dst_base, dst, btc.c_buffer, bias_w, btc.od, btc.oh, ow,
                 g_oc, is_oc_tail, ow, ow, kd_l, kh_l,
                 post_ops_binary_rhs_arg_vec.data(), btc.oscales,
                 btc.src_zp_vals, btc.src_zp_comp_ptr, btc.dst_zp_vals,
-                btc.s8s8_comp_ptr, do_init, do_postwork, false);
+                btc.s8s8_comp_ptr, do_init, do_postwork, false, btc.dst_scales);
     }
 }
 
@@ -1607,8 +1677,8 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
     MAYBE_UNUSED(g_ic);
     MAYBE_UNUSED(src);
 
-    const auto wei_base
-            = weights + wei_dsz * (btc.g * wei_ocb_sz + btc.ocb * wei_kd_sz);
+    const auto wei_base = weights
+            + wei_dsz * (btc.g * wei_g_stride + btc.ocb * wei_ocb_stride);
     const int ow_b {ow},
             ow_e {ow + (is_ow_tail ? jcp.ow % jcp.ow_block : jcp.ow_block)};
     const int oh_b {oh},
@@ -1631,6 +1701,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
     const auto call_brgemm = [&](int brg_idx, int ic_block_s, int n_ic_blocks,
                                      bool do_postops) {
         if (k_l <= 0) return;
+        btc.maybe_tile_configure(is_amx, brg_kernel_palettes_, brg_idx);
 
         const auto kh_ee = jcp.kh_sets > 1 ? kh_b + 1 : kh_e;
         const auto kw_e = jcp.kw_sets > 1 ? 1 : KW;
@@ -1655,7 +1726,8 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
             const auto pbuf_base_ic = pbuf_base
                     + src_dsz
                             * ((jcp.copy_block_only ? 0 : (i_icb * pbuf_d_sz)));
-            const auto wei_base_ic = wei_base + wei_dsz * wei_ic * jcp.oc_block;
+            const auto wei_base_ic
+                    = wei_base + wei_dsz * wei_ic * wei_ic_stride;
 
             auto k = 0;
             for (int kd = kd_b; kd < kd_e; kd++) {
@@ -1663,7 +1735,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
                 const auto pbuf_base_kd
                         = pbuf_base_ic + src_dsz * id * pbuf_h_sz;
                 const auto wei_base_kd = wei_base_ic
-                        + wei_dsz * maybe_invert(kd, KD) * wei_kh_sz;
+                        + wei_dsz * maybe_invert(kd, KD) * wei_kd_stride;
                 for (int kh = kh_b; kh < kh_ee; kh++) {
                     const auto ih = jcp.kh_sets > 1
                             ? (iih + 2 * TP)
@@ -1674,7 +1746,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
                             + wei_dsz
                                     * ((jcp.kh_sets > 1 ? 0
                                                         : maybe_invert(kh, KH))
-                                            * wei_kw_sz);
+                                            * wei_kh_stride);
                     for (int kw = 0; kw < kw_e; kw++) {
                         const auto iw = iiw_b - iiw_shift + kw * DW + LP;
                         // inp_buffer layout is Cdhw<ic_block>c
@@ -1685,7 +1757,8 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
                         btc.brg_batch[n_icb_off + k].vvpad.bottom = 0;
                         // general wei layout is gOdhwI<block_o><block_i>
                         btc.brg_batch[n_icb_off + k].ptr.B = wei_base_kh
-                                + wei_dsz * maybe_invert(kw, KW) * wei_ic_sz;
+                                + wei_dsz * maybe_invert(kw, KW)
+                                        * wei_kw_stride;
                         k++;
                     }
                 }
@@ -1700,8 +1773,9 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
 
     const auto kdhw_loop = [&]() {
         const auto do_init = btc.icc == 0 && kd_b == kd_s && kh_b == kh_s;
-        const auto do_postwork = need_postwork && btc.icc == (ic_chunks - 1)
-                && kd_e == kd_f && kh_e == kh_f;
+        const auto do_postwork = _pd->need_postwork
+                && btc.icc == (_pd->ic_chunks - 1) && kd_e == kd_f
+                && kh_e == kh_f;
         if (ow_e - ow_b <= 0 && !do_init && !do_postwork) return;
 
         k_l = (kd_e - kd_b) * (jcp.kh_sets > 1 ? 1 : (kh_e - kh_b))
@@ -1740,12 +1814,13 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_trans(
         }
     } else {
         const auto do_init = btc.icc == 0;
-        const auto do_postwork = need_postwork && btc.icc == (ic_chunks - 1);
+        const auto do_postwork
+                = _pd->need_postwork && btc.icc == (_pd->ic_chunks - 1);
         perform_outwork(dst_base, dst, btc.c_buffer, bias_w, btc.od, btc.oh, ow,
                 g_oc, is_oc_tail, ow, ow, kd_l, kh_l,
                 post_ops_binary_rhs_arg_vec.data(), btc.oscales,
                 btc.src_zp_vals, btc.src_zp_comp_ptr, btc.dst_zp_vals,
-                btc.s8s8_comp_ptr, do_init, do_postwork, false);
+                btc.s8s8_comp_ptr, do_init, do_postwork, false, btc.dst_scales);
     }
 }
 
@@ -1763,8 +1838,8 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_vpad(
     const char *const __restrict src_base
             = src + src_dsz * (btc.n * src_d_sz + g_ic);
 
-    const char *const __restrict wei_base
-            = weights + wei_dsz * (btc.g * wei_ocb_sz + btc.ocb * wei_kd_sz);
+    const char *const __restrict wei_base = weights
+            + wei_dsz * (btc.g * wei_g_stride + btc.ocb * wei_ocb_stride);
 
     const int ow_b {ow}, ow_e {ow + (is_ow_tail ? jcp.M_tail : jcp.M)};
     iiw_b = ow_b * SW - LP;
@@ -1786,6 +1861,8 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_vpad(
     const auto call_brgemm = [&](int brg_idx, int ic_block_s, int n_ic_blocks,
                                      int32_t *src_zp, int32_t *s8s8_comp,
                                      bool do_postops) {
+        btc.maybe_tile_configure(is_amx, brg_kernel_palettes_, brg_idx);
+
         for (int i_icb = 0; i_icb < n_ic_blocks; i_icb++) {
             const auto ic_off = (ic_block_s + i_icb) * jcp.ic_block;
             const auto src_ic = ic_off;
@@ -1794,7 +1871,7 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_vpad(
             const char *const __restrict src_base_ic
                     = src_base + src_dsz * src_ic;
             const char *const __restrict wei_base_ic
-                    = wei_base + wei_dsz * wei_ic * jcp.oc_block;
+                    = wei_base + wei_dsz * wei_ic * wei_ic_stride;
             brgemm_batch_element_t *const __restrict icb_batch
                     = btc.brg_batch + n_icb_off;
 
@@ -1804,13 +1881,13 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_vpad(
                 const char *const __restrict src_base_kd
                         = src_base_ic + src_dsz * id * src_h_sz;
                 const char *const __restrict wei_base_kd = wei_base_ic
-                        + wei_dsz * maybe_invert(kd, KD) * wei_kh_sz;
+                        + wei_dsz * maybe_invert(kd, KD) * wei_kd_stride;
                 for (int kh = kh_b; kh < kh_e; kh++) {
                     const auto ih = iih + kh * DH;
                     const char *const __restrict src_base_kh
                             = src_base_kd + src_dsz * ih * src_w_sz;
                     const char *const __restrict wei_base_kh = wei_base_kd
-                            + wei_dsz * maybe_invert(kh, KH) * wei_kw_sz;
+                            + wei_dsz * maybe_invert(kh, KH) * wei_kh_stride;
                     for (int kw = 0; kw < KW; kw++) {
                         const auto iw = iiw_b + kw * DW;
                         const auto ptr_A = src_base_kh
@@ -1822,7 +1899,8 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_vpad(
                         }
                         // general wei layout is gOdhwI<block_o><block_i>
                         const auto ptr_B = wei_base_kh
-                                + wei_dsz * maybe_invert(kw, KW) * wei_ic_sz;
+                                + wei_dsz * maybe_invert(kw, KW)
+                                        * wei_kw_stride;
 
                         icb_batch[k].ptr.A = ptr_A;
                         icb_batch[k].ptr.B = ptr_B;
@@ -1840,8 +1918,9 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_vpad(
 
     const auto kdhw_loop = [&]() {
         const auto do_init = btc.icc == 0 && kd_b == kd_s && kh_b == kh_s;
-        const auto do_postwork = need_postwork && btc.icc == (ic_chunks - 1)
-                && kd_e == kd_f && kh_e == kh_f;
+        const auto do_postwork = _pd->need_postwork
+                && btc.icc == (_pd->ic_chunks - 1) && kd_e == kd_f
+                && kh_e == kh_f;
 
         if (ow_e - ow_b <= 0 && !do_init && !do_postwork) return;
 
@@ -1890,25 +1969,32 @@ void brgemm_convolution_fwd_t<isa, use_inversion>::ker_vpad(
         }
     } else {
         const auto do_init = btc.icc == 0;
-        const auto do_postwork = need_postwork && btc.icc == (ic_chunks - 1);
+        const auto do_postwork
+                = _pd->need_postwork && btc.icc == (_pd->ic_chunks - 1);
         perform_outwork(dst_base, dst, btc.c_buffer, bias_w, btc.od, btc.oh, ow,
                 g_oc, is_oc_tail, ow, ow, kd_l, kh_l,
                 post_ops_binary_rhs_arg_vec.data(), btc.oscales,
                 btc.src_zp_vals, btc.src_zp_comp_ptr, btc.dst_zp_vals,
-                btc.s8s8_comp_ptr, do_init, do_postwork, false);
+                btc.s8s8_comp_ptr, do_init, do_postwork, false, btc.dst_scales);
     }
 }
 
 #undef BRGEMM_CONV_KER_HEADER
 
+template struct brgemm_convolution_fwd_t<avx2>;
+template struct brgemm_convolution_fwd_t<avx2_vnni_2>;
+template struct brgemm_convolution_fwd_t<avx2_vnni_2, true>;
 template struct brgemm_convolution_fwd_t<avx512_core>;
 template struct brgemm_convolution_fwd_t<avx512_core, true>;
 template struct brgemm_convolution_fwd_t<avx512_core_vnni>;
 template struct brgemm_convolution_fwd_t<avx512_core_bf16>;
 template struct brgemm_convolution_fwd_t<avx512_core_bf16, true>;
-template struct brgemm_convolution_fwd_t<avx512_core_bf16_amx_int8>;
-template struct brgemm_convolution_fwd_t<avx512_core_bf16_amx_bf16>;
-template struct brgemm_convolution_fwd_t<avx512_core_bf16_amx_bf16, true>;
+template struct brgemm_convolution_fwd_t<avx512_core_fp16>;
+template struct brgemm_convolution_fwd_t<avx512_core_fp16, true>;
+template struct brgemm_convolution_fwd_t<avx512_core_amx>;
+template struct brgemm_convolution_fwd_t<avx512_core_amx, true>;
+template struct brgemm_convolution_fwd_t<avx512_core_amx_fp16>;
+template struct brgemm_convolution_fwd_t<avx512_core_amx_fp16, true>;
 
 } // namespace x64
 
