@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2022 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -56,27 +56,22 @@ struct brgemm_inner_product_fwd_t : public primitive_t {
             auto src_dt = invariant_src_md()->data_type;
             auto dst_dt = invariant_dst_md()->data_type;
             auto wei_dt = invariant_wei_md()->data_type;
+            const bool is_int8 = one_of(src_dt, u8, s8);
 
             using skip_mask_t = primitive_attr_t::skip_mask_t;
             auto skip_mask = skip_mask_t::post_ops;
-            if (one_of(src_dt, u8, s8))
-                skip_mask |= skip_mask_t::oscale | skip_mask_t::oscale_runtime;
+            if (is_int8) skip_mask |= skip_mask_t::scales_runtime;
 
             bool ok = is_fwd() && mayiuse(isa)
                     && expect_data_types(src_dt, wei_dt, data_type::undef,
                             dst_dt, data_type::undef)
-                    && IMPLICATION(with_bias(),
-                            ((one_of(src_dt, u8, s8)
-                                     && one_of(bias_md_.data_type, f32, bf16,
-                                             s32, s8, u8))
-                                    || (one_of(src_dt, bf16)
-                                            && one_of(bias_md_.data_type, f32,
-                                                    bf16))
-                                    || (one_of(src_dt, f32)
-                                            && one_of(
-                                                    bias_md_.data_type, f32))))
-                    && attr()->has_default_values(skip_mask)
-                    && !has_zero_dim_memory();
+                    && IMPLICATION(with_bias() && is_int8,
+                            one_of(bias_md_.data_type, f32, bf16, s32, s8, u8))
+                    && IMPLICATION(with_bias() && !is_int8,
+                            one_of(bias_md_.data_type, f32, src_dt))
+                    && attr()->has_default_values(skip_mask, dst_dt)
+                    && attr()->post_ops_.check_sum_consistent_dt(dst_dt)
+                    && !has_zero_dim_memory() && arg_scales_ok();
             if (!ok) return status::unimplemented;
 
             CHECK(brgemm_inner_product_utils::init_ip_conf(isa, jbgp_, *desc(),
@@ -86,7 +81,7 @@ struct brgemm_inner_product_fwd_t : public primitive_t {
             bool are_post_ops_applicable = one_of(true, jbgp_.with_sum,
                     jbgp_.with_bias, jbgp_.with_scales, jbgp_.with_eltwise,
                     jbgp_.with_binary, jbgp_.acc_dt != jbgp_.dst_dt,
-                    jbgp_.signed_input);
+                    jbgp_.req_s8s8_compensation, jbgp_.with_dst_scales);
 
             const float alpha = 1.0;
             const float beta = 1.0;
@@ -118,29 +113,40 @@ struct brgemm_inner_product_fwd_t : public primitive_t {
                     brgattr.generate_skip_accumulation = true;
                     CHECK(brgemm_desc_set_attr(&brg, brgattr));
                 }
-                if (isa == avx512_core_bf16_amx_int8
-                        || isa == avx512_core_bf16_amx_bf16) {
+                if (jbgp_.is_amx) {
                     brgemm_attr_t brgattr;
                     brgattr.max_bs = bs;
                     brgattr.wary_tail_read = false;
                     brgattr.hint_expected_A_size = jbgp_.mb * jbgp_.ic;
                     brgattr.hint_expected_B_size = jbgp_.oc * jbgp_.ic;
                     brgattr.hint_expected_C_size = jbgp_.mb * jbgp_.oc;
-                    brgattr.hint_innermost_loop = brgemm_ld_loop_innermost;
+                    brgattr.hint_innermost_loop = brgemm_innermost_undef;
                     brgattr.use_uker = jbgp_.use_uker;
                     brgattr.use_interleave_stores = jbgp_.use_interleave_stores;
                     brgattr.hint_prefetching = jbgp_.hint_prefetching;
+                    brgattr.fpmath_mode = attr()->fpmath_mode_;
 
                     CHECK(brgemm_desc_set_attr(&brg, brgattr));
+                    jbgp_.amx_buf_size_per_thread
+                            = nstl::max(brg.get_wsp_buffer_size(),
+                                    jbgp_.amx_buf_size_per_thread);
                 }
             }
 
             auto scratchpad = scratchpad_registry().registrar();
             brgemm_inner_product_utils::init_scratchpad(scratchpad, jbgp_);
+            if (jbgp_.with_scales)
+                book_precomputed_scales(scratchpad, attr()->scales_, OC());
 
             return status::success;
         }
 
+        bool arg_scales_ok() const {
+            std::vector<int> supported_args
+                    = {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST};
+            return attr_scales_ok(supported_args);
+        }
+
         int get_brg_kernel_idx(bool is_bs_tail, bool do_initialization,
                 bool is_M_tail, bool is_N_tail, bool is_K_tail, int bs) const {
             auto vM = (is_M_tail) ? jbgp_.M_tail : jbgp_.M;
@@ -161,8 +167,7 @@ struct brgemm_inner_product_fwd_t : public primitive_t {
                     : jbgp_.ic;
             auto bs = (is_K_tail)
                     ? 1
-                    : ((is_bs_tail) ? (adj_ic / jbgp_.ic_block)
-                                            % jbgp_.gemm_batch_size
+                    : ((is_bs_tail) ? (adj_ic / jbgp_.K) % jbgp_.gemm_batch_size
                                     : jbgp_.gemm_batch_size);
             return bs;
         }
@@ -186,8 +191,7 @@ struct brgemm_inner_product_fwd_t : public primitive_t {
             brgemm_kernel_t *ker = nullptr;
             CHECK(brgemm_kernel_create(&ker, pd()->brg_descs_[idx]));
             CHECK(safe_ptr_assign(brg_kernels_[idx], ker));
-            if (isa == avx512_core_bf16_amx_int8
-                    || isa == avx512_core_bf16_amx_bf16)
+            if (pd()->jbgp_.is_amx)
                 CHECK(brgemm_init_tiles(
                         pd()->brg_descs_[idx], &brg_kernel_palettes_[idx][0]));
         }
@@ -235,12 +239,10 @@ struct brgemm_inner_product_bwd_data_t : public primitive_t {
 
             bool ok = true && desc()->prop_kind == prop_kind::backward_data
                     && !has_zero_dim_memory() && mayiuse(isa)
-                    && ((diff_dst_dt == data_type::bf16
-                                && wei_dt == data_type::bf16
-                                && utils::one_of(diff_src_dt, data_type::bf16,
-                                        data_type::f32))
-                            || utils::everyone_is(data_type::f32, diff_dst_dt,
-                                    wei_dt, diff_src_dt))
+                    && utils::one_of(diff_dst_dt, data_type::f32,
+                            data_type::bf16, data_type::f16)
+                    && wei_dt == diff_dst_dt
+                    && utils::one_of(diff_src_dt, data_type::f32, diff_dst_dt)
                     && attr()->has_default_values(
                             primitive_attr_t::skip_mask_t::post_ops);
             if (!ok) return status::unimplemented;
@@ -253,6 +255,9 @@ struct brgemm_inner_product_bwd_data_t : public primitive_t {
             const float alpha = 1.0;
             const float beta = 1.0;
             const float beta_init = 0.0;
+            const auto dt_b = isa == avx512_core_fp16 && jbgp_.use_buffer_b
+                    ? data_type::f32
+                    : wei_dt;
 
             for_(int i_bs = 0; i_bs < 2; i_bs++)
             for_(int i_init = 0; i_init < 2; i_init++)
@@ -269,26 +274,29 @@ struct brgemm_inner_product_bwd_data_t : public primitive_t {
 
                 brgemm_t &brg = brg_descs_[idx];
                 CHECK(brgemm_desc_init(&brg, isa, jbgp_.brg_type, diff_dst_dt,
-                        wei_dt, false, false, brgemm_row_major, alpha, vbeta,
+                        dt_b, false, false, brgemm_row_major, alpha, vbeta,
                         jbgp_.LDA, jbgp_.LDB, jbgp_.LDC, vM, vN, vK));
 
                 auto LDD = jbgp_.ic_without_padding;
                 CHECK(brgemm_desc_set_postops(
                         &brg, attr(), &diff_src_md_, LDD, jbgp_.bia_dt));
-
-                if (isa == avx512_core_bf16_amx_bf16) {
+                if (jbgp_.is_amx) {
                     brgemm_attr_t brgattr;
                     brgattr.max_bs = bs;
                     brgattr.wary_tail_read = false;
                     brgattr.hint_expected_A_size = jbgp_.mb * jbgp_.oc;
                     brgattr.hint_expected_B_size = jbgp_.oc * jbgp_.ic;
                     brgattr.hint_expected_C_size = jbgp_.mb * jbgp_.ic;
-                    brgattr.hint_innermost_loop = brgemm_ld_loop_innermost;
+                    brgattr.hint_innermost_loop = brgemm_innermost_undef;
                     brgattr.use_uker = jbgp_.use_uker;
                     brgattr.use_interleave_stores = jbgp_.use_interleave_stores;
                     brgattr.hint_prefetching = jbgp_.hint_prefetching;
+                    brgattr.fpmath_mode = attr()->fpmath_mode_;
 
                     CHECK(brgemm_desc_set_attr(&brg, brgattr));
+                    jbgp_.amx_buf_size_per_thread
+                            = nstl::max(brg.get_wsp_buffer_size(),
+                                    jbgp_.amx_buf_size_per_thread);
                 }
             }
 
@@ -344,7 +352,7 @@ struct brgemm_inner_product_bwd_data_t : public primitive_t {
             brgemm_kernel_t *ker = nullptr;
             CHECK(brgemm_kernel_create(&ker, pd()->brg_descs_[idx]));
             CHECK(safe_ptr_assign(brg_kernels_[idx], ker));
-            if (isa == avx512_core_bf16_amx_bf16)
+            if (jbgp.is_amx)
                 CHECK(brgemm_init_tiles(
                         pd()->brg_descs_[idx], &brg_kernel_palettes_[idx][0]));
         }
@@ -400,12 +408,10 @@ struct brgemm_inner_product_bwd_weights_t : public primitive_t {
 
             bool ok = true && desc()->prop_kind == prop_kind::backward_weights
                     && !has_zero_dim_memory() && mayiuse(isa)
-                    && ((src_dt == data_type::bf16
-                                && diff_dst_type == data_type::bf16
-                                && utils::one_of(diff_wei_type, data_type::bf16,
-                                        data_type::f32))
-                            || utils::everyone_is(data_type::f32, src_dt,
-                                    diff_dst_type, diff_wei_type))
+                    && utils::one_of(src_dt, data_type::f32, data_type::bf16,
+                            data_type::f16)
+                    && diff_dst_type == src_dt
+                    && utils::one_of(diff_wei_type, data_type::f32, src_dt)
                     && attr()->has_default_values(
                             primitive_attr_t::skip_mask_t::post_ops);
             if (!ok) return status::unimplemented;
@@ -417,6 +423,12 @@ struct brgemm_inner_product_bwd_weights_t : public primitive_t {
             const float alpha = 1.0;
             const float beta = 1.0;
             const float beta_init = 0.0;
+            const auto dt_a = isa == avx512_core_fp16 && jbgp_.use_buffer_a
+                    ? data_type::f32
+                    : jbgp_.src_dt;
+            const auto dt_b = isa == avx512_core_fp16 && jbgp_.use_buffer_b
+                    ? data_type::f32
+                    : jbgp_.dst_dt;
 
             for_(int i_bs = 0; i_bs < 2; i_bs++)
             for_(int i_init = 0; i_init < 2; i_init++)
@@ -431,22 +443,26 @@ struct brgemm_inner_product_bwd_weights_t : public primitive_t {
                 int idx = get_brg_kernel_idx(i_bs, i_init, i_M, i_N, i_K, bs);
                 if (idx < 0) continue;
                 brgemm_t &brg = brg_descs_[idx];
-                CHECK(brgemm_desc_init(&brg, isa, jbgp_.brg_type, jbgp_.src_dt,
-                        jbgp_.dst_dt, false, false, brgemm_row_major, alpha,
-                        vbeta, jbgp_.LDA, jbgp_.LDB, jbgp_.LDC, vM, vN, vK));
-                if (isa == avx512_core_bf16_amx_bf16) {
+                CHECK(brgemm_desc_init(&brg, isa, jbgp_.brg_type, dt_a, dt_b,
+                        false, false, brgemm_row_major, alpha, vbeta, jbgp_.LDA,
+                        jbgp_.LDB, jbgp_.LDC, vM, vN, vK));
+                if (jbgp_.is_amx) {
                     brgemm_attr_t brgattr;
                     brgattr.max_bs = bs;
                     brgattr.wary_tail_read = false;
                     brgattr.hint_expected_A_size = jbgp_.mb * jbgp_.ic;
                     brgattr.hint_expected_B_size = jbgp_.mb * jbgp_.oc;
                     brgattr.hint_expected_C_size = jbgp_.ic * jbgp_.oc;
-                    brgattr.hint_innermost_loop = brgemm_ld_loop_innermost;
+                    brgattr.hint_innermost_loop = brgemm_innermost_undef;
                     brgattr.use_uker = jbgp_.use_uker;
                     brgattr.use_interleave_stores = jbgp_.use_interleave_stores;
                     brgattr.hint_prefetching = jbgp_.hint_prefetching;
+                    brgattr.fpmath_mode = attr()->fpmath_mode_;
 
                     CHECK(brgemm_desc_set_attr(&brg, brgattr));
+                    jbgp_.amx_buf_size_per_thread
+                            = nstl::max(brg.get_wsp_buffer_size(),
+                                    jbgp_.amx_buf_size_per_thread);
                 }
             }
 
@@ -498,7 +514,7 @@ struct brgemm_inner_product_bwd_weights_t : public primitive_t {
             brgemm_kernel_t *ker = nullptr;
             CHECK(brgemm_kernel_create(&ker, pd()->brg_descs_[idx]));
             CHECK(safe_ptr_assign(brg_kernels_[idx], ker));
-            if (isa == avx512_core_bf16_amx_bf16)
+            if (jbgp.is_amx)
                 CHECK(brgemm_init_tiles(
                         pd()->brg_descs_[idx], &brg_kernel_palettes_[idx][0]));
 
@@ -513,7 +529,7 @@ struct brgemm_inner_product_bwd_weights_t : public primitive_t {
                 }
             }
         }
-        if (isa == avx512_core_bf16_amx_bf16) {
+        if (jbgp.is_amx) {
             ext_ic_block_ = jbgp.ic_block_ext;
             ext_oc_block_ = jbgp.oc_block_ext;
         }
@@ -523,12 +539,13 @@ struct brgemm_inner_product_bwd_weights_t : public primitive_t {
             CHECK(create_brgemm_trans_to_vnni(trans_B_kernel_, &pd()->jbgp_,
                     jit_brgemm_trans_to_vnni_t::matrix_to_transform::matrix_B));
 
-        if (isa != avx512_core_bf16_amx_bf16) {
+        if (!jbgp.is_amx) {
             if (jbgp.wei_dt != jbgp.acc_dt)
                 CHECK(create_brgemm_trans_to_vnni(trans_C_kernel_, &pd()->jbgp_,
                         jit_brgemm_trans_to_vnni_t::matrix_to_transform::
                                 matrix_C));
-        } else if (jbgp.wei_dt == dnnl::impl::data_type::bf16) {
+        } else if (utils::one_of(
+                           jbgp.wei_dt, data_type::bf16, data_type::f16)) {
             CHECK(create_brgemm_amx_ip_trans_wei(diff_wei_trans_kernel_,
                     &pd()->jbgp_, ext_ic_block_, ext_oc_block_));
         }
@@ -547,6 +564,7 @@ struct brgemm_inner_product_bwd_weights_t : public primitive_t {
     }
 
 private:
+    enum loop_order_t { osc_icc_occ, osc_occ_icc, occ_icc_osc };
     struct thread_info_t;
     std::unique_ptr<jit_brgemm_kernel_diff_bias_t> kernels_db_[2][2];
     std::unique_ptr<brgemm_kernel_t>
