@@ -187,6 +187,9 @@ struct jit_conv_conf_t {
     bool zp_src_is_common; // common, otherwise (TODO) per-channel
     bool req_zero_point_buffer; // used for calculating padding compensation
     bool zp_pbuff_outer_compute; // indicates if zp_bbuff is computed in
+
+    bool dst_scale;
+
     // a separate parallel region
     int ow_pad, oh_pad, od_pad; // output elements with padding & filter overlap
 
@@ -417,6 +420,7 @@ struct jit_conv_call_s {
     const int32_t *dst_zero_point;
     const void *tile_cfg;
     const void *tile_cfg_tail;
+    const void *dst_scale;
 
     // ptr to table of void * elements that are pointers to
     // post_op binary src1 tensors
@@ -482,6 +486,7 @@ struct jit_deconv_call_s {
     const void *filt; /* hack, non-const for backward_weights */
     const void *bias; /* hack, non-const for backward_bias */
     const void *scales;
+    const void *dst_scale;
     const void *compensation;
     const int32_t *zp_src_pad_str_compensation;
     const int32_t *zp_compensation;
@@ -599,6 +604,8 @@ struct jit_1x1_conv_conf_t {
     bool dst_zero_point;
     bool zp_src_is_common; // common, otherwise (TODO) per-channel
 
+    bool dst_scale;
+
     cpu_isa_t isa;
     bool uses_permw_transposition;
 };
@@ -615,6 +622,7 @@ struct jit_1x1_conv_call_s {
     const int32_t *zp_compensation;
     const int32_t *src_zero_point;
     const int32_t *dst_zero_point;
+    const void *dst_scale;
 
     // ptr to table of void * elements that are pointers to
     // post_op binary src1 tensors
@@ -675,7 +683,7 @@ struct jit_pool_conf_t {
     bool with_eltwise;
     bool with_binary;
     int nthr;
-    memory_desc_t *tmp_md = nullptr;
+    memory_desc_t tmp_md;
 };
 
 struct jit_pool_call_s {
@@ -736,7 +744,7 @@ struct jit_resampling_conf_t {
     jit_memory_tag_kind_t tag_kind = jit_memory_tag_kind_t::undef;
     alg_kind_t alg = alg_kind::undef;
 
-    cpu_isa_t isa = isa_any;
+    cpu_isa_t isa = isa_undef;
 
     post_ops_t post_ops = post_ops_t();
     bool with_postops = false;
@@ -786,6 +794,7 @@ struct jit_brdgmm_conv_conf_t {
 
     bool with_bias;
     bool with_post_ops;
+    bool with_scale;
     bool is_oc_scale;
 
     data_type_t src_dt;
@@ -820,19 +829,21 @@ struct jit_brgemm_conv_conf_t {
     prop_kind_t prop_kind;
     conv_brgemm_loop_order_t loop_order;
     conv_harness_t harness;
-    int simd_w, amx_w, amx_h;
+    int simd_w, acc_simd_w, amx_w, amx_h;
     int ndims;
     int mb;
     int ngroups, ic, oc, oc_without_padding, ic_without_padding;
 
-    int od_block, oh_block, nb_od, nb_oh,
-            ih_block; // blocking  - included in parallelization
+    int od_block, oh_block, nb_od,
+            nb_oh; // blocking  - included in parallelization
+    int id_block, ih_block, nb_id, nb_ih;
     dim_t inp_buffer_size, inp_buffer_mask_size;
     conv_brgemm_exec_type_t exec_type;
 
-    int id, ih, iw, od, oh, ow, os, idp, ihp, iwp, icp;
+    int id, ih, iw, od, oh, ow, os, is, idp, ihp, iwp, icp, odp, ohp, owp, ocp;
     int f_pad, l_pad, t_pad;
     int back_pad, r_pad, b_pad;
+    int l_ovf, r_ovf, t_ovf, b_ovf, f_ovf, back_ovf;
     int kd, kh, kw;
     int ext_kd, ext_kh, ext_kw;
     int kd_block, kh_block, kw_block, kd_block_pad, kh_block_pad, kw_block_pad;
@@ -845,15 +856,18 @@ struct jit_brgemm_conv_conf_t {
     bool with_binary;
 
     bool is_fused_conv;
+    bool is_is_blocking;
     bool is_os_blocking;
     bool is_rtus;
     int nb_ic, ic_block;
     int nb_oc, oc_block;
-    int nb_iw, iw_block;
+    int nb_iw, iw_block, iw_tail;
     int nb_ow, ow_block, ow_tail;
+    int nb_is, is_block;
     int nb_os, os_block;
     int nb_oc_blocking;
     int nb_ic_blocking;
+    int nb_is_blocking;
     int nb_os_blocking;
 
     data_type_t src_dt;
@@ -873,7 +887,8 @@ struct jit_brgemm_conv_conf_t {
     dim_t comp_a_buffer_size;
     dim_t s8s8_comp_buffer_size;
 
-    int is_oc_scale;
+    bool with_scales;
+    int is_ic_scale, is_oc_scale;
 
     int LDA, LDB, LDC, LDD;
     int M, N, K, M_tail, N_tail, K_tail;
@@ -890,12 +905,12 @@ struct jit_brgemm_conv_conf_t {
     int amx_buf_size_per_thread;
 
     bool wei_plain;
-    bool is_ic_padded;
+    bool is_ic_padded, is_oc_padded;
     int kw_sets, kh_sets;
     bool copy_block_only;
     bool amx_tile_load_xx;
     int use_M_mask;
-    int oskip;
+    int oskip, iskip;
     bool brgemm_bd_loop_innermost;
 
     bool use_uker;
@@ -948,7 +963,7 @@ struct jit_shuffle_conf_t {
     dim_t c_split_size = 0;
     dim_t sp_split_size = 0;
 
-    cpu_isa_t isa = isa_any;
+    cpu_isa_t isa = isa_undef;
 };
 
 struct jit_shuffle_call_s {
@@ -992,7 +1007,7 @@ struct jit_binary_conf_t {
     dim_t outer_dims = 1;
     int src1_stride = 1;
     int not_bcasted_sp_dims = 0;
-    cpu_isa_t isa = isa_any;
+    cpu_isa_t isa = isa_undef;
 
     data_type_t src0_type = data_type::undef;
     data_type_t src1_type = data_type::undef;
@@ -1019,7 +1034,7 @@ struct jit_reduction_conf_t {
     std::size_t acc_dt_size = 0;
 
     alg_kind_t alg = alg_kind::undef;
-    cpu_isa_t isa = isa_any;
+    cpu_isa_t isa = isa_undef;
 
     dim_t idle_size = 0;
     dim_t reduce_size = 0;
