@@ -36,7 +36,7 @@ using namespace Xbyak;
 struct jit_trans_iw_ic_int16_t : public jit_trans_src_t, public jit_generator {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_trans_iw_ic_int16_t)
     jit_trans_iw_ic_int16_t(const jit_conv_conf_t *conf)
-        : jit_trans_src_t(conf) {}
+        : jit_trans_src_t(conf), jit_generator(jit_name()) {}
 
     void operator()(ctx_t *ctx) override { jit_generator::operator()(ctx); }
 
@@ -406,7 +406,9 @@ void jit_trans_iw_ic_int16_t::generate() {
     vmovdqa32(vidx5, (const int32_t *)idx5);
 
     // Data for every strided case is placed consecutively
-    for (int s = 0; s < str_w; s++) {
+    // For 1x1 convolutions with strides we transpose only needed elements
+    const auto str_w_end = (conf_->kw == 1) ? 1 : str_w;
+    for (int s = 0; s < str_w_end; s++) {
         const int left_pad = div_up(conf_->l_pad - s, str_w);
         const int iw1 = iw + conf_->l_pad;
         const int iw_s = (s < (iw1 % str_w) ? div_up(iw1, str_w) : iw1 / str_w)
@@ -473,7 +475,8 @@ void jit_trans_iw_ic_int16_t::generate() {
 
 struct jit_trans_ow_oc_t : public jit_trans_dst_t, public jit_generator {
     DECLARE_CPU_JIT_AUX_FUNCTIONS(jit_trans_ow_oc_t)
-    jit_trans_ow_oc_t(const jit_conv_conf_t *conf) : jit_trans_dst_t(conf) {}
+    jit_trans_ow_oc_t(const jit_conv_conf_t *conf)
+        : jit_trans_dst_t(conf), jit_generator(jit_name()) {}
 
     void operator()(ctx_t *ctx) override { jit_generator::operator()(ctx); }
 
@@ -510,12 +513,15 @@ private:
     reg32_t regw_tmp = r14d;
     reg64_t imm_addr64 = rbx;
 
-    void transpose(int nrows, int l_pad, int r_pad, bool nontemporal_stores);
+    void transpose(int nrows, int l_pad, int r_pad, bool nontemporal_stores,
+            bool do_convert = true);
     void generate() override;
 };
 
-void jit_trans_ow_oc_t::transpose(
-        int nrows, int l_pad, int r_pad, bool nontemporal_stores) {
+// do_convert (default is 'true') is a flag that determines when to do the
+// transformation of the input data and when to simply zero out the output data
+void jit_trans_ow_oc_t::transpose(int nrows, int l_pad, int r_pad,
+        bool nontemporal_stores, bool do_convert) {
     assert(nrows >= 0 && nrows <= transpose_size);
     static_assert(transpose_size == 16, "Unsupported transpose size");
     if (!nrows) return;
@@ -555,16 +561,24 @@ void jit_trans_ow_oc_t::transpose(
         // TODO: adopt for nhwc?
         for (int i = 0; i < nrows / 2; i++) {
             auto zmm_src0 = src_zmm(i);
-            vmovdqu16(
-                    zmm_src0, EVEX_compress_addr(reg_src, 2 * i * src_stride));
-            vpermw(zmm_src0, vidx2, zmm_src0);
+            if (do_convert) {
+                vmovdqu16(zmm_src0,
+                        EVEX_compress_addr(reg_src, 2 * i * src_stride));
+                vpermw(zmm_src0, vidx2, zmm_src0);
+            } else {
+                vpxord(zmm_src0, zmm_src0, zmm_src0);
+            }
             store(zmm_src0, 2 * i);
         }
         if (r_pad > 0) {
             auto zmm_src0 = src_zmm(29);
-            vmovdqu16(zmm_src0 | mask_lo | T_z,
-                    EVEX_compress_addr(reg_src, (nrows - 1) * src_stride));
-            vpermw(zmm_src0, vidx2, zmm_src0);
+            if (do_convert) {
+                vmovdqu16(zmm_src0 | mask_lo | T_z,
+                        EVEX_compress_addr(reg_src, (nrows - 1) * src_stride));
+                vpermw(zmm_src0, vidx2, zmm_src0);
+            } else {
+                vpxord(zmm_src0, zmm_src0, zmm_src0);
+            }
             store(zmm_src0, nrows - 1);
         }
     } else {
@@ -572,36 +586,46 @@ void jit_trans_ow_oc_t::transpose(
             auto src0 = src_ymm(2 * i);
             auto src1 = src_ymm(2 * i + 1);
             auto zmm_src0 = src_zmm(2 * i);
-            load_ymm(2 * i);
-            if (is_layout_nxc && conf_->oc_tail) {
-                load_ymm(2 * i + 1);
-                auto ymm_tmp = Ymm(30);
-                vpunpcklwd(ymm_tmp, src0, src1);
-                vpunpckhwd(src0, src0, src1);
-                vinserti64x4(zmm_src0, zmm_src0, ymm_tmp, 1);
+            if (do_convert) {
+                load_ymm(2 * i);
+                if (is_layout_nxc && conf_->oc_tail) {
+                    load_ymm(2 * i + 1);
+                    auto ymm_tmp = Ymm(30);
+                    vpunpcklwd(ymm_tmp, src0, src1);
+                    vpunpckhwd(src0, src0, src1);
+                    vinserti64x4(zmm_src0, zmm_src0, ymm_tmp, 1);
+                } else {
+                    vpunpcklwd(src1, src0,
+                            EVEX_compress_addr(
+                                    reg_src, (2 * i + 1) * src_stride));
+                    vpunpckhwd(src0, src0,
+                            EVEX_compress_addr(
+                                    reg_src, (2 * i + 1) * src_stride));
+                    vinserti64x4(zmm_src0, zmm_src0, src1, 1);
+                }
+                vpermpd(zmm_src0 | kFF, vidx1, zmm_src0);
             } else {
-                vpunpcklwd(src1, src0,
-                        EVEX_compress_addr(reg_src, (2 * i + 1) * src_stride));
-                vpunpckhwd(src0, src0,
-                        EVEX_compress_addr(reg_src, (2 * i + 1) * src_stride));
-                vinserti64x4(zmm_src0, zmm_src0, src1, 1);
+                vpxord(zmm_src0, zmm_src0, zmm_src0);
             }
-            vpermpd(zmm_src0 | kFF, vidx1, zmm_src0);
             store(zmm_src0, 2 * i);
         }
         if (r_pad > 0) {
             auto src0 = src_ymm(nrows - 1);
             auto src1 = src_ymm(nrows);
             auto zmm_src0 = src_zmm(30);
-            load_ymm(nrows - 1);
+            if (do_convert) {
+                load_ymm(nrows - 1);
 
-            vpxor(src1, src1, src1);
-            vpunpckhwd(src1, src0, src1);
-            vinserti64x4(zmm_src0, zmm_src0, src1, 0);
-            vpxor(src1, src1, src1);
-            vpunpcklwd(src0, src0, src1);
-            vinserti64x4(zmm_src0, zmm_src0, src0, 1);
-            vpermpd(zmm_src0 | kFF, vidx1, zmm_src0);
+                vpxor(src1, src1, src1);
+                vpunpckhwd(src1, src0, src1);
+                vinserti64x4(zmm_src0, zmm_src0, src1, 0);
+                vpxor(src1, src1, src1);
+                vpunpcklwd(src0, src0, src1);
+                vinserti64x4(zmm_src0, zmm_src0, src0, 1);
+                vpermpd(zmm_src0 | kFF, vidx1, zmm_src0);
+            } else {
+                vpxord(zmm_src0, zmm_src0, zmm_src0);
+            }
             store(zmm_src0, nrows - 1);
         }
     }
@@ -636,6 +660,8 @@ void jit_trans_ow_oc_t::generate() {
     const size_t tr_src_step = (size_t)oc_block * transpose_size * typesize;
     const int right_pad = ow % 2;
 
+    const auto zero_tr_ow = nstl::max(0, conf_->tr_ow - ow - right_pad);
+
     mov(reg_src, ptr[param1 + GET_OFF(src)]);
     mov(reg_tr_src, ptr[param1 + GET_OFF(tr_src)]);
     mov(reg_src_prf, ptr[param1 + GET_OFF(src_prf)]);
@@ -684,6 +710,33 @@ void jit_trans_ow_oc_t::generate() {
         }
     }
     transpose(tail, 0, right_pad, nontemporal_stores);
+    if (zero_tr_ow) {
+        const auto zero_transposes = utils::div_up(zero_tr_ow, transpose_size);
+        const auto zero_loop_iters = nstl::max(0, zero_transposes - 1);
+        const auto zero_tail = zero_tr_ow - zero_loop_iters * transpose_size;
+        const auto zero_right_pad = zero_tr_ow % 2;
+
+        // shift over tail
+        auto tr_src_tail_step
+                = (size_t)oc_block * (tail + right_pad) * typesize;
+        add(reg_tr_src, tr_src_tail_step);
+        add(reg_tr_src_prf, tr_src_tail_step);
+
+        // zero the tr_ow - ow
+        if (zero_loop_iters) {
+            mov(reg_loop, zero_loop_iters);
+            Label zero_loop;
+            L(zero_loop);
+            {
+                transpose(transpose_size, 0, 0, nontemporal_stores, false);
+                add(reg_tr_src, tr_src_step);
+                add(reg_tr_src_prf, tr_src_step);
+                sub(reg_loop, 1);
+                jnz(zero_loop);
+            }
+        }
+        transpose(zero_tail, 0, zero_right_pad, nontemporal_stores, false);
+    }
 
     postamble();
 }
@@ -969,7 +1022,7 @@ void jit_diff_wei_trans_to_vnni_t::generate() {
                         for (int ic = 0; ic < ic_block_ / 2; ic++) {
                             auto zmm_src_0 = get_zmm_src_0(ic);
                             auto zmm_src_1 = get_zmm_src_1(ic);
-                            auto zmm_bf16 = get_zmm_bf16(ic);
+                            auto zmm_out = get_zmm_bf16(ic);
 
                             vmovups(zmm_src_0,
                                     ptr[reg_input
@@ -981,15 +1034,23 @@ void jit_diff_wei_trans_to_vnni_t::generate() {
                                             + typesize_acc
                                                     * ((2 * ic + 1) * oc_block_
                                                             + ocb)]);
-
-                            vcvtne2ps2bf16(zmm_bf16, zmm_src_1, zmm_src_0);
-                            vpermw(zmm_bf16, zmm_idx, zmm_bf16);
+                            if (out_dt_ == data_type::bf16) {
+                                vcvtne2ps2bf16(zmm_out, zmm_src_1, zmm_src_0);
+                            } else if (out_dt_ == data_type::f16) {
+                                vcvtps2phx(Ymm(zmm_src_0.getIdx()), zmm_src_0);
+                                vcvtps2phx(Ymm(zmm_src_1.getIdx()), zmm_src_1);
+                                vinsertf32x8(zmm_out, zmm_src_0,
+                                        Ymm(zmm_src_1.getIdx()), 1);
+                            } else {
+                                assert(!"unsupported data type");
+                            }
+                            vpermw(zmm_out, zmm_idx, zmm_out);
 
                             vmovups(ptr[reg_output + out_offset
                                             + typesize_out
                                                     * (ic_count * oc_block_ * 2
                                                             + ocb * 2)],
-                                    zmm_bf16);
+                                    zmm_out);
                             ic_count++;
                         }
                     }
