@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@
 #include "common/dnnl_thread.hpp"
 #include "common/memory_tracking.hpp"
 #include "common/nstl.hpp"
-#include "common/primitive_iterator.hpp"
+#include "common/primitive_desc_iterator.hpp"
 #include "common/type_helpers.hpp"
 #include "common/utils.hpp"
 
@@ -169,7 +169,7 @@ struct rtus_driver_t : public jit_generator {
     rtus_driver_t(int iw, int stride_w, int src_step_h, int src_step_icb,
             int ws_step_icb, bool src_to_ws, size_t typesize, int ic,
             bool is_nspc = false)
-        : jit_generator(nullptr, MAX_CODE_SIZE, true, isa)
+        : jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, isa)
         , iw_(iw)
         , stride_w_(stride_w)
         , src_step_h_(src_step_h)
@@ -375,7 +375,8 @@ struct rtus_driver_t : public jit_generator {
         const size_t w_step_factor = ic_ * typesize_;
         const size_t max_load_store_bytes = isa == sse41
                 ? typesize_ == 4 ? 16 : 8
-                : typesize_ == 4 ? 32 : 16;
+                : typesize_ == 4 ? 32
+                                 : 16;
         const size_t load_store_size
                 = isa == avx512_core ? vlen_ : max_load_store_bytes;
         size_t load_store_tail_size = (typesize_ == 1 ? max_load_store_bytes
@@ -675,11 +676,13 @@ inline size_t get_load_bwd_w_offset(const jcp_t &jcp, int i, int u0) {
     }
 }
 
-inline size_t get_output_i_offset(const jcp_t &jcp) {
+inline size_t get_output_i_offset(
+        const jcp_t &jcp, bool ignore_dw_conv = false) {
     if (is_out_layout_nxc(jcp)) {
         return jcp.load_block;
     } else {
-        return (jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim) * jcp.load_block;
+        return (!ignore_dw_conv && jcp.with_dw_conv ? jcp.ow : jcp.bcast_dim)
+                * jcp.load_block;
     }
 }
 
@@ -688,10 +691,10 @@ inline size_t get_output_j_offset(const jcp_t &jcp) {
 }
 
 inline size_t get_load_loop_output_fwd_offset(
-        const jcp_t &jcp, int load_loop_blk) {
+        const jcp_t &jcp, int load_loop_blk, bool ignore_dw_conv = false) {
     size_t offset = load_loop_blk * jcp.oc_block * sizeof(float);
     if (!is_out_layout_nxc(jcp)) {
-        offset *= jcp.with_dw_conv ? jcp.ow : jcp.os;
+        offset *= !ignore_dw_conv && jcp.with_dw_conv ? jcp.ow : jcp.os;
     }
     return offset;
 }
