@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 * Copyright 2018 YANDEX LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -17,6 +17,8 @@
 
 #include <bitset>
 
+#include "common/dnnl_thread.hpp"
+
 #include "cpu/cpu_pooling_pd.hpp"
 #include "cpu/x64/jit_avx512_core_bf16cvt.hpp"
 #include "cpu/x64/jit_uni_pool_kernel.hpp"
@@ -42,10 +44,10 @@ jit_uni_pool_kernel<isa>::~jit_uni_pool_kernel() = default;
 template <cpu_isa_t isa>
 jit_uni_pool_kernel<isa>::jit_uni_pool_kernel(
         const jit_pool_conf_t &ajpp, const memory_desc_t *dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, isa)
+    : jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, isa)
     , jpp(ajpp)
     , bf16_emu_(nullptr) {
-    if (jpp.is_bf16 && !isa_has_bf16(jpp.isa))
+    if (use_bf16_emulation())
         bf16_emu_ = utils::make_unique<bf16_emulation_t>(this,
                 bf16_emu_reserv_1, bf16_emu_reserv_2, bf16_emu_reserv_3,
                 bf16_emu_reserv_4, bf16_emu_reserv_5);
@@ -63,10 +65,10 @@ jit_uni_pool_kernel<isa>::jit_uni_pool_kernel(
 
         const binary_injector::rhs_arg_static_params_t rhs_sp {
                 static_cast<std::size_t>(this->xmm4.getIdx()), this->r14,
-                this->r15, preserve_gpr, preserve_vmm,
+                this->r15, this->r13, preserve_gpr, preserve_vmm,
                 GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(jpp.tag_kind == jit_memory_tag_kind_t::ncsp
-                                ? *(jpp.tmp_md)
+                                ? jpp.tmp_md
                                 : *dst_md),
                 postop_tail, k_c_tail_mask, use_exact_tail_scalar_bcast};
 
@@ -127,15 +129,22 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
     jpp.ow = dst_d.dims()[ndims - 1];
     jpp.oh = (ndims == 3) ? 1 : dst_d.dims()[ndims - 2];
 
-    const bool is_avx512 = isa == avx512_core;
+    const bool is_avx512 = is_superset(isa, avx512_core);
     jpp.ndims = ndims;
     jpp.mb = src_d.dims()[0];
     jpp.c_without_padding = src_d.dims()[1];
     jpp.c_block = is_avx512 ? 16 : 8;
 
     jpp.alg = pd.alg_kind;
+    jpp.tmp_md = memory_desc_t();
+
+    jpp.is_bf16 = (src_d.data_type() == data_type::bf16
+            && dst_d.data_type() == data_type::bf16);
+    jpp.is_f16 = (src_d.data_type() == data_type::f16
+            && dst_d.data_type() == data_type::f16);
 
     using namespace format_tag;
+
     const auto blocked_fmt_tag = is_avx512
             ? utils::pick(ndims - 3, nCw16c, nChw16c, nCdhw16c)
             : utils::pick(ndims - 3, nCw8c, nChw8c, nCdhw8c);
@@ -153,12 +162,14 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
             && jpp.c_without_padding > 3
             && ((jpp.ih > 1 && jpp.iw > 1
                         && block_size <= L3_cache_size_per_core)
-                    || src_d.data_type() == data_type::bf16);
+                    || utils::one_of(src_d.data_type(), data_type::bf16,
+                            data_type::f16));
 
     const bool backward_ncsp_allowed = jpp.is_backward
             && ((jpp.ih > 1 && jpp.iw > 1 && jpp.c_without_padding > 1
                         && block_size <= L3_cache_size_per_core)
-                    || (src_d.data_type() == data_type::bf16
+                    || (utils::one_of(src_d.data_type(), data_type::bf16,
+                                data_type::f16)
                             && !(jpp.alg == pooling_max
                                     && block_size > L3_cache_size_per_core)));
 
@@ -182,17 +193,20 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
         // transform input to blocked f32, call f32 jit, transform result to
         // plain output
         jpp.is_bf16 = false;
+        jpp.is_f16 = false;
         jpp.dt_size = types::data_type_size(data_type::f32);
         jpp.tag_kind = jit_memory_tag_kind_t::ncsp;
 
         // used to initialize binary post-ops
         if (ppd->is_fwd() && jpp.with_binary) {
-            CHECK(dnnl_memory_desc_init_by_tag(jpp.tmp_md, ndims,
-                    dst_d.md_->dims, data_type::f32, blocked_fmt_tag));
+            CHECK(memory_desc_init_by_tag(jpp.tmp_md, ndims, dst_d.md_->dims,
+                    data_type::f32, blocked_fmt_tag));
         }
     } else {
         jpp.is_bf16 = (src_d.data_type() == data_type::bf16
                 && dst_d.data_type() == data_type::bf16);
+        jpp.is_f16 = (src_d.data_type() == data_type::f16
+                && dst_d.data_type() == data_type::f16);
         jpp.dt_size = types::data_type_size(src_d.data_type());
         jpp.tag_kind = (fmt_tag == nspc_fmt_tag)
                 ? jit_memory_tag_kind_t::nspc
@@ -201,7 +215,7 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
 
     if (ppd->is_fwd() && jpp.with_binary) {
         CHECK(set_binary_postops_formats(attr.post_ops_,
-                jpp.tag_kind == jit_memory_tag_kind_t::ncsp ? jpp.tmp_md
+                jpp.tag_kind == jit_memory_tag_kind_t::ncsp ? &jpp.tmp_md
                                                             : dst_d.md_));
     }
 
@@ -209,11 +223,26 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
                                                          : isa;
 
     const bool args_ok = true && mayiuse(isa) && (fmt_tag != format_tag::undef)
-            && IMPLICATION(jpp.is_bf16, mayiuse(avx512_core))
+            && IMPLICATION(jpp.is_bf16,
+                    utils::one_of(jpp.isa, avx512_core_bf16, avx512_core,
+                            avx2_vnni_2))
+            && IMPLICATION(jpp.is_f16,
+                    utils::one_of(jpp.isa, avx512_core_fp16, avx2_vnni_2))
             && utils::one_of(pd.alg_kind, pooling_max,
                     pooling_avg_include_padding, pooling_avg_exclude_padding);
     if (!args_ok) return status::unimplemented;
 
+    const bool is_xf16_avx2_vnni_2
+            = (jpp.is_bf16 || jpp.is_f16) && isa == avx2_vnni_2;
+    // note: avx2_vnni_2 only supports nxc format
+    if (!IMPLICATION(is_xf16_avx2_vnni_2,
+                jpp.tag_kind == jit_memory_tag_kind_t::nspc))
+        return status::unimplemented;
+
+    // note: avx2_vnni_2 only supports FWD direction
+    if (!IMPLICATION(is_xf16_avx2_vnni_2, !jpp.is_backward))
+        return status::unimplemented;
+
     jpp.c = jpp.tag_kind == jit_memory_tag_kind_t::blocked
             ? utils::rnd_up(jpp.c_without_padding, jpp.c_block)
             : jpp.c_without_padding;
@@ -257,7 +286,7 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
     if (jpp.alg == pooling_max) {
         jpp.ur = is_avx512 ? 16 : 4;
 
-        if ((isa == avx || isa == avx2) && jpp.c_tail > 0)
+        if (utils::one_of(isa, avx, avx2, avx2_vnni_2) && jpp.c_tail > 0)
             // Additional register needed for tail mask
             jpp.ur -= 1;
 
@@ -271,10 +300,10 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
         else
             jpp.ur = is_avx512 ? 24 : 12;
     }
-    if (jpp.is_bf16) {
+    if ((jpp.is_bf16 || jpp.is_f16) && isa != avx2_vnni_2) {
         jpp.ur = (!isa_has_bf16(jpp.isa))
                 ? jpp.ur - 4 // Free registers for AVX512 emulation
-                : jpp.ur - 1; // Free register for cvt from bf16 to f32
+                : jpp.ur - 1; // Free register for cvt from bf16/f16 to f32
     }
 
     // select jpp.ur_bc
@@ -298,7 +327,7 @@ status_t jit_uni_pool_kernel<isa>::init_conf(jit_pool_conf_t &jpp,
                 best_eff = eff;
                 jpp.ur_bc = ur_bc;
             }
-            if (eff > 0.9) break; // Heuristic threshold
+            if (eff > 0.9f) break; // Heuristic threshold
         }
 
         //take into account cache re-usage after zeroing on backward
@@ -347,12 +376,21 @@ inline void jit_uni_pool_kernel<isa>::prepare_tail_mask() {
         size_t c_tail_mask = (1ULL << jpp.c_tail) - 1ULL;
         mov(tmp_gpr.cvt32(), c_tail_mask);
         kmovw(k_c_tail_mask, tmp_gpr.cvt32());
-    } else if (isa == avx || isa == avx2) {
+    } else if (utils::one_of(isa, avx, avx2, avx2_vnni_2)) {
+        constexpr int max_words_in_ymm = 8;
+
+        // for 'avx2_vnni_2' mask works with 2 x xf16 elements,
+        // in case of 'c_tail % 2 != 0' load/store an additional word
+        // for the remaining element.
+        auto dt_elem_div = isa == avx2_vnni_2 ? 2 : 1;
+        auto mask_offset = max_words_in_ymm - (jpp.c_tail / dt_elem_div);
+        auto mask_register
+                = isa == avx2_vnni_2 ? xmm_c_tail_mask : vmm_c_tail_mask;
         static const uint32_t mask[16] = {0xffffffff, 0xffffffff, 0xffffffff,
                 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0xffffffff, 0,
                 0, 0, 0, 0, 0, 0, 0};
-        mov(tmp_gpr, reinterpret_cast<size_t>(&mask[8 - jpp.c_tail]));
-        vmovups(vmm_c_tail_mask, ptr[tmp_gpr]);
+        mov(tmp_gpr, reinterpret_cast<size_t>(&mask[mask_offset]));
+        vmovups(mask_register, ptr[tmp_gpr]);
     }
 }
 
@@ -365,7 +403,7 @@ inline void jit_uni_pool_kernel<isa>::put_one_in_vmm() {
 template <cpu_isa_t isa>
 inline void jit_uni_pool_kernel<isa>::uni_broadcast_reg_val(
         const int reg_idx, const int vmm_idx) {
-    movq(Xmm(vmm_idx), reg64_t(reg_idx));
+    uni_vmovq(Xmm(vmm_idx), reg64_t(reg_idx));
     uni_vpbroadcastd(Vmm(vmm_idx), Xmm(vmm_idx));
 }
 
@@ -391,23 +429,21 @@ inline void jit_uni_pool_kernel<isa>::load(const int idx,
         /*TODO: maybe use vpmovzxwd + vpslld,
              * in order to free up vmm_idx() register */
         if (is_c_tail_proccessing && !jpp.is_c_padded) {
-            Vmm vmm_to_load = is_c_tail_proccessing
-                    ? Vmm(idx) | k_c_tail_mask | T_z
-                    : Vmm(idx);
+            Vmm vmm_to_load = Vmm(idx) | k_c_tail_mask | T_z;
             vpmovzxwd(vmm_to_load, ptr[reg_ptr + offset]);
             vpslld(vmm_to_load, vmm_to_load, 16);
         } else {
             vmovups(Ymm(idx), ptr[reg_ptr + offset]);
             vpermw(Vmm(idx) | k_mask_cvt | T_z, vmm_idx(), Vmm(idx));
         }
+    } else if (jpp.is_f16) {
+        Vmm vmm_to_load = is_c_tail_proccessing && !jpp.is_c_padded
+                ? Vmm(idx) | k_c_tail_mask | T_z
+                : Vmm(idx);
+        vcvtph2psx(vmm_to_load, ptr[reg_ptr + offset]);
     } else {
         if (is_c_tail_proccessing && !jpp.is_c_padded) {
-            if (isa == sse41) {
-                for (int i = 0; i < jpp.c_tail % (jpp.c_block / 2); i++) {
-                    pinsrd(Xmm(idx), ptr[reg_ptr + offset + i * jpp.dt_size],
-                            i);
-                }
-            } else if (isa == avx || isa == avx2) {
+            if (isa == avx || isa == avx2) {
                 vmaskmovps(Vmm(idx), vmm_c_tail_mask, ptr[reg_ptr + offset]);
             } else {
                 vmovups(Zmm(idx) | k_c_tail_mask | T_z, ptr[reg_ptr + offset]);
@@ -418,11 +454,50 @@ inline void jit_uni_pool_kernel<isa>::load(const int idx,
     }
 }
 
+template <>
+inline void jit_uni_pool_kernel<avx2_vnni_2>::load(const int idx,
+        const reg64_t &reg_ptr, const int offset,
+        const bool is_c_tail_proccessing) {
+    if (is_c_tail_proccessing) {
+        vmaskmovps(Xmm(idx), xmm_c_tail_mask, ptr[reg_ptr + offset]);
+        if (jpp.c_tail % 2 != 0) {
+            const int tail_pos = jpp.c_tail - 1;
+            auto word_addr
+                    = ptr[reg_ptr + offset + tail_pos * sizeof(bfloat16_t)];
+            vpinsrw(Xmm(idx), Xmm(idx), word_addr, tail_pos);
+        }
+    }
+    if (jpp.is_bf16) {
+        if (is_c_tail_proccessing)
+            vpmovzxwd(Ymm(idx), Xmm(idx));
+        else
+            vpmovzxwd(Ymm(idx), ptr[reg_ptr + offset]);
+        vpslld(Ymm(idx), Ymm(idx), 16);
+    } else if (jpp.is_f16) {
+        if (is_c_tail_proccessing)
+            vcvtph2ps(Ymm(idx), Xmm(idx));
+        else
+            vcvtph2ps(Ymm(idx), ptr[reg_ptr + offset]);
+    } else
+        assert(!"invalid data type");
+}
+
+template <>
+inline void jit_uni_pool_kernel<sse41>::load(const int idx,
+        const reg64_t &reg_ptr, const int offset,
+        const bool is_c_tail_proccessing) {
+    if (is_c_tail_proccessing && !jpp.is_c_padded) {
+        for (int i = 0; i < jpp.c_tail % (jpp.c_block / 2); i++)
+            pinsrd(Xmm(idx), ptr[reg_ptr + offset + i * jpp.dt_size], i);
+    } else
+        uni_vmovups(Vmm(idx), ptr[reg_ptr + offset]);
+}
+
 template <cpu_isa_t isa>
 inline void jit_uni_pool_kernel<isa>::store(const int idx,
         const reg64_t &reg_ptr, const int offset,
         const bool is_c_tail_proccessing) {
-    if (jpp.is_bf16) {
+    if (jpp.is_bf16 || jpp.is_f16) {
         if (is_c_tail_proccessing) {
             if (jpp.is_c_padded) {
                 vmovdqu16(Ymm(idx) | k_c_tail_mask | T_z, Ymm(idx));
@@ -434,42 +509,14 @@ inline void jit_uni_pool_kernel<isa>::store(const int idx,
     } else {
         if (is_c_tail_proccessing) {
             if (!jpp.is_c_padded) {
-                if (isa == sse41)
-                    for (int i = 0; i < jpp.c_tail % (jpp.c_block / 2); i++)
-                        pextrd(ptr[reg_ptr + offset + i * jpp.dt_size],
-                                Xmm(idx), i);
-                else if (isa == avx || isa == avx2)
+                if (isa == avx || isa == avx2)
                     vmaskmovps(
                             ptr[reg_ptr + offset], vmm_c_tail_mask, Vmm(idx));
                 else
                     vmovups(ptr[reg_ptr + offset] | k_c_tail_mask, Zmm(idx));
             } else {
                 if (jpp.with_postops) {
-                    if (isa == sse41) {
-                        static constexpr auto xmm_half = 4;
-                        const auto tail_size
-                                = (jpp.c_without_padding > jpp.c_block)
-                                ? jpp.c_without_padding % (jpp.c - jpp.c_block)
-                                : jpp.c_without_padding;
-                        const auto tail_size_real = (tail_size >= xmm_half)
-                                ? tail_size - xmm_half
-                                : tail_size;
-                        uni_vxorps(xmm_tmp_1, xmm_tmp_1, xmm_tmp_1);
-                        if (tail_size <= xmm_half && sse_high_half) {
-                            // just zero out upper half padding and don't write anything else
-                            uni_vmovups(vmmword[reg_ptr + offset], xmm_tmp_1);
-                            return;
-                        }
-
-                        if ((tail_size < xmm_half && !sse_high_half)
-                                || (tail_size > xmm_half && sse_high_half)) {
-                            std::bitset<8> tail_mask((1 << tail_size_real) - 1);
-                            tail_mask.flip();
-                            uni_vblendps(Vmm(idx), Vmm(idx), xmm_tmp_1,
-                                    tail_mask.to_ulong());
-                        }
-
-                    } else if (isa == avx || isa == avx2) {
+                    if (isa == avx || isa == avx2) {
                         uni_vxorps(ymm_tmp_1, ymm_tmp_1, ymm_tmp_1);
                         uni_vblendvps(
                                 Vmm(idx), ymm_tmp_1, Vmm(idx), vmm_c_tail_mask);
@@ -483,6 +530,62 @@ inline void jit_uni_pool_kernel<isa>::store(const int idx,
     }
 }
 
+template <>
+inline void jit_uni_pool_kernel<avx2_vnni_2>::store(const int idx,
+        const reg64_t &reg_ptr, const int offset,
+        const bool is_c_tail_proccessing) {
+    if (jpp.is_bf16 || jpp.is_f16) {
+        if (is_c_tail_proccessing) {
+            vmaskmovps(ptr[reg_ptr + offset], xmm_c_tail_mask, Xmm(idx));
+            if (jpp.c_tail % 2 != 0) {
+                const int tail_pos = jpp.c_tail - 1;
+                auto word_addr = ptr[reg_ptr + offset + tail_pos * 2];
+                vpextrw(word_addr, Xmm(idx), tail_pos);
+            }
+        } else
+            vmovups(xword[reg_ptr + offset], Xmm(idx));
+    } else
+        assert(!"datatype not supported");
+}
+
+template <>
+inline void jit_uni_pool_kernel<sse41>::store(const int idx,
+        const reg64_t &reg_ptr, const int offset,
+        const bool is_c_tail_proccessing) {
+    if (is_c_tail_proccessing) {
+        if (!jpp.is_c_padded) {
+            for (int i = 0; i < jpp.c_tail % (jpp.c_block / 2); i++)
+                pextrd(ptr[reg_ptr + offset + i * jpp.dt_size], Xmm(idx), i);
+        } else {
+            if (jpp.with_postops) {
+                static constexpr auto xmm_half = 4;
+                const auto tail_size = (jpp.c_without_padding > jpp.c_block)
+                        ? jpp.c_without_padding % (jpp.c - jpp.c_block)
+                        : jpp.c_without_padding;
+                const auto tail_size_real = (tail_size >= xmm_half)
+                        ? tail_size - xmm_half
+                        : tail_size;
+                uni_vxorps(xmm_tmp_1, xmm_tmp_1, xmm_tmp_1);
+                if (tail_size <= xmm_half && sse_high_half) {
+                    // just zero out upper half padding and don't write anything else
+                    uni_vmovups(vmmword[reg_ptr + offset], xmm_tmp_1);
+                    return;
+                }
+
+                if ((tail_size < xmm_half && !sse_high_half)
+                        || (tail_size > xmm_half && sse_high_half)) {
+                    std::bitset<8> tail_mask((1 << tail_size_real) - 1);
+                    tail_mask.flip();
+                    uni_vblendps(Vmm(idx), Vmm(idx), xmm_tmp_1,
+                            tail_mask.to_ulong());
+                }
+            }
+            uni_vmovups(vmmword[reg_ptr + offset], Vmm(idx));
+        }
+    } else
+        uni_vmovups(vmmword[reg_ptr + offset], Vmm(idx));
+}
+
 template <cpu_isa_t isa>
 bool jit_uni_pool_kernel<isa>::post_ops_ok(jit_pool_conf_t &jpp,
         const primitive_attr_t &attr, const memory_desc_wrapper &dst_d) {
@@ -498,9 +601,13 @@ bool jit_uni_pool_kernel<isa>::post_ops_ok(jit_pool_conf_t &jpp,
                 const auto alg = entry.eltwise.alg;
                 jpp.with_eltwise = eltwise_injector::is_supported(isa, alg);
             } else if (entry.is_binary()) {
-                if (isa != avx512_core
-                        && entry.binary.src1_desc.data_type == data_type::bf16)
-                    return false;
+                const bool is_bf16_ok = IMPLICATION(
+                        entry.binary.src1_desc.data_type == data_type::bf16,
+                        utils::one_of(isa, avx512_core, avx2_vnni_2));
+                const bool is_f16_ok = IMPLICATION(
+                        entry.binary.src1_desc.data_type == data_type::f16,
+                        utils::one_of(isa, avx512_core_fp16, avx2_vnni_2));
+                if (!(is_bf16_ok && is_f16_ok)) return false;
 
                 jpp.with_binary = true;
             } else
@@ -522,7 +629,7 @@ void jit_uni_pool_kernel<isa>::apply_postops(int ur_bc, int ur_w, int c_block,
     const int start_idx = end_idx - (ur_bc * ur_w);
     const bool sse41_postops_disabled
             = isa == sse41 && disable_postops_when_sse_high_half_processed_;
-
+    if (end_idx - start_idx == 0) return;
     if (jpp.with_binary && !sse41_postops_disabled) {
 
         const int c_off = (jpp.tag_kind == jit_memory_tag_kind_t::nspc)
@@ -573,15 +680,17 @@ inline void jit_uni_pool_kernel<isa>::maybe_recalculate_divisor(
 
         if (non_zero_kw != prev_kw) {
             mov(tmp_gpr, float2int((float)non_zero_kw));
-            movq(xmm_tmp, tmp_gpr);
+            uni_vmovq(xmm_tmp, tmp_gpr);
             uni_vbroadcastss(vmm_tmp, xmm_tmp);
-            if (with_c_tail_proccessing && (isa == avx || isa == avx2)) {
+            if (with_c_tail_proccessing
+                    && (utils::one_of(isa, avx, avx2, avx2_vnni_2))) {
                 push_vmm_val(vmm_c_tail_mask.getIdx());
                 uni_broadcast_reg_val(
                         reg_ker_area_h.getIdx(), vmm_ker_area_h.getIdx());
             }
             uni_vmulps(vmm_tmp, vmm_tmp, vmm_ker_area_h);
-            if (with_c_tail_proccessing && (isa == avx || isa == avx2)) {
+            if (with_c_tail_proccessing
+                    && (utils::one_of(isa, avx, avx2, avx2_vnni_2))) {
                 pop_vmm_val(vmm_c_tail_mask.getIdx());
             }
             prev_kw = non_zero_kw;
@@ -670,11 +779,13 @@ inline void jit_uni_pool_kernel<isa>::avg_step(int ur_w, int ur_bc, int pad_l,
                             bf16_emu_->vcvtneps2bf16(inpyr, zreg(inpr_i));
                         else
                             vcvtneps2bf16(inpyr, inpvr);
+                    } else if (jpp.is_f16) {
+                        vcvtps2ph(inpyr, inpvr, _op_mxcsr);
                     }
                     store(reg_idx(inpr_i), aux_reg_input, input_offset,
                             is_tail_processing(bci));
                 } else {
-                    if (jpp.is_bf16 || is_tail_processing(bci)
+                    if (jpp.is_bf16 || jpp.is_f16 || is_tail_processing(bci)
                             || (isa == sse41
                                     && c_off % (jpp.c_block / 2) != 0)) {
                         load(vmm_tmp_1.getIdx(), aux_reg_input, input_offset,
@@ -723,13 +834,24 @@ inline void jit_uni_pool_kernel<isa>::avg_step(int ur_w, int ur_bc, int pad_l,
                 const auto accvr = vreg(accr_i);
                 const auto output_offset
                         = dt_size * (jj * c_off + bci * c_block);
+                const auto accyr = yreg(accr_i);
                 if (jpp.is_bf16) {
-                    const auto acczr = zreg(accr_i);
-                    const auto accyr = yreg(accr_i);
-                    if (!isa_has_bf16(jpp.isa))
-                        bf16_emu_->vcvtneps2bf16(accyr, acczr);
-                    else
-                        vcvtneps2bf16(accyr, accvr);
+                    if (isa == avx2_vnni_2) {
+                        auto accxr = xreg(accr_i);
+                        vcvtneps2bf16(accxr, accyr, Xbyak::VexEncoding);
+                    } else {
+                        const auto acczr = zreg(accr_i);
+                        if (!isa_has_bf16(jpp.isa))
+                            bf16_emu_->vcvtneps2bf16(accyr, acczr);
+                        else
+                            vcvtneps2bf16(accyr, accvr);
+                    }
+                } else if (jpp.is_f16) {
+                    if (isa == avx2_vnni_2) {
+                        auto accxr = xreg(accr_i);
+                        vcvtps2ph(accxr, accyr, _op_mxcsr);
+                    } else
+                        vcvtps2ph(accyr, accvr, _op_mxcsr);
                 }
                 store(reg_idx(accr_i), reg_output, output_offset,
                         is_tail_processing(bci));
@@ -760,7 +882,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
     };
 
     mov(tmp_gpr, float2int(nstl::numeric_limits<float>::lowest()));
-    movq(xmm_tmp, tmp_gpr);
+    uni_vmovq(xmm_tmp, tmp_gpr);
     uni_vbroadcastss(vmm_tmp, xmm_tmp);
 
     for_(int jj = 0; jj < ur_w; jj++)
@@ -773,7 +895,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
         }
     }
     if (jpp.is_training) {
-        movq(xmm_tmp, reg_k_shift);
+        uni_vmovq(xmm_tmp, reg_k_shift);
         uni_vpbroadcastd(vmm_k_offset, xmm_tmp);
     }
     if (jpp.ndims == 5) {
@@ -812,7 +934,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
                     cmpps(vmm_mask, inpvr, _cmp_lt_os);
                     blendvps(accvr, inpvr);
                     if (jpp.is_training) blendvps(indvr, vmm_k_offset);
-                } else if (isa == avx || isa == avx2) {
+                } else if (utils::one_of(isa, avx, avx2, avx2_vnni_2)) {
                     vcmpps(cvtvr, accvr, inpvr, _cmp_lt_os);
                     vblendvps(accvr, accvr, inpvr, cvtvr);
                     if (jpp.is_training)
@@ -825,7 +947,8 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
                 }
             }
             if (jpp.is_training) {
-                if (with_c_tail_proccessing && (isa == avx || isa == avx2)) {
+                if (with_c_tail_proccessing
+                        && (utils::one_of(isa, avx, avx2, avx2_vnni_2))) {
                     push_vmm_val(vmm_c_tail_mask.getIdx());
                     put_one_in_vmm();
                 }
@@ -835,7 +958,8 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
                 else
                     uni_vpaddd(vmm_k_offset, vmm_k_offset, vmm_one);
 
-                if (with_c_tail_proccessing && (isa == avx || isa == avx2))
+                if (with_c_tail_proccessing
+                        && (utils::one_of(isa, avx, avx2, avx2_vnni_2)))
                     pop_vmm_val(vmm_c_tail_mask.getIdx());
             }
         }
@@ -849,7 +973,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
         add(aux_reg_input_d, jpp.dt_size * jpp.ih * iw * c_off);
         if (jpp.is_training) {
             mov(tmp_gpr, ptr[reg_param + GET_OFF(kd_padding_shift)]);
-            movq(xmm_tmp, tmp_gpr);
+            uni_vmovq(xmm_tmp, tmp_gpr);
             uni_vpbroadcastd(vmm_tmp, xmm_tmp);
             if (isa == avx && !mayiuse(avx2)) {
                 Xmm t(vmm_mask.getIdx());
@@ -877,13 +1001,24 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
         const auto accr_i = reg_ind(0, bci, jj, ur_bc, ur_w);
         const auto accvr = vreg(accr_i);
         const auto output_offset = jpp.dt_size * (jj * c_off + bci * c_block);
+        auto accyr = yreg(accr_i);
         if (jpp.is_bf16) {
-            auto acczr = zreg(accr_i);
-            auto accyr = yreg(accr_i);
-            if (!isa_has_bf16(jpp.isa))
-                bf16_emu_->vcvtneps2bf16(accyr, acczr);
-            else
-                vcvtneps2bf16(accyr, accvr);
+            if (isa == avx2_vnni_2) {
+                auto accxr = xreg(accr_i);
+                vcvtneps2bf16(accxr, accyr, Xbyak::VexEncoding);
+            } else {
+                auto acczr = zreg(accr_i);
+                if (!isa_has_bf16(jpp.isa))
+                    bf16_emu_->vcvtneps2bf16(accyr, acczr);
+                else
+                    vcvtneps2bf16(accyr, accvr);
+            }
+        } else if (jpp.is_f16) {
+            if (isa == avx2_vnni_2) {
+                auto accxr = xreg(accr_i);
+                vcvtps2ph(accxr, accyr, _op_mxcsr);
+            } else
+                vcvtps2ph(accyr, accvr, _op_mxcsr);
         }
         store(reg_idx(accr_i), reg_output, output_offset,
                 is_tail_processing(bci));
@@ -913,7 +1048,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_fwd(int ur_w, int ur_bc,
                             pextrb(ptr[reg_index + step_index + i], xr, 4 * i);
                         }
                     }
-                } else if (isa == avx || isa == avx2) {
+                } else if (utils::one_of(isa, avx, avx2, avx2_vnni_2)) {
                     auto yr = yreg(indr_i);
                     if (is_tail_processing(bci) && !jpp.is_c_padded) {
                         const int max_nr_of_vals
@@ -1061,7 +1196,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_bwd(int ur_w, int ur_bc,
                     is_tail_processing(bci));
         }
     }
-    movq(xmm_tmp, reg_k_shift);
+    uni_vmovq(xmm_tmp, reg_k_shift);
     uni_vpbroadcastd(vmm_k_offset, xmm_tmp);
 
     if (jpp.simple_alg && jpp.ndims == 5) {
@@ -1144,6 +1279,8 @@ inline void jit_uni_pool_kernel<isa>::max_step_bwd(int ur_w, int ur_bc,
                             bf16_emu_->vcvtneps2bf16(indyr, indzr);
                         else
                             vcvtneps2bf16(indyr, inpvr);
+                    } else if (jpp.is_f16) {
+                        vcvtps2ph(indyr, inpvr, _op_mxcsr);
                     }
                     store(inpvr.getIdx(), aux_reg_input, inp_offset,
                             is_tail_processing(bci));
@@ -1173,7 +1310,7 @@ inline void jit_uni_pool_kernel<isa>::max_step_bwd(int ur_w, int ur_bc,
         add(aux_reg_input_d, jpp.dt_size * jpp.ih * iw * c_off);
 
         mov(tmp_gpr, reg_kd_pad_shift);
-        movq(xmm_tmp, tmp_gpr);
+        uni_vmovq(xmm_tmp, tmp_gpr);
         uni_vpbroadcastd(vmm_tmp, xmm_tmp);
         if (isa == avx && !mayiuse(avx2)) {
             Xmm t(vmm_mask.getIdx());
@@ -1298,7 +1435,7 @@ void jit_uni_pool_kernel<isa>::generate() {
     xor_(rcx, rdi);
     xor_(rdi, rcx);
 #endif
-    if (!isa_has_bf16(jpp.isa) && jpp.is_bf16) bf16_emu_->init_vcvtneps2bf16();
+    if (use_bf16_emulation()) bf16_emu_->init_vcvtneps2bf16();
 
     mov(reg_input, ptr[reg_param + GET_OFF(src)]);
     mov(reg_output, ptr[reg_param + GET_OFF(dst)]);
@@ -1309,7 +1446,7 @@ void jit_uni_pool_kernel<isa>::generate() {
     mov(reg_ker_area_h, ptr[reg_param + GET_OFF(ker_area_h)]);
     mov(reg_nbc, ptr[reg_param + GET_OFF(ur_bc)]);
 
-    if (jpp.is_bf16) {
+    if ((jpp.is_bf16 || jpp.is_f16) && isa != avx2_vnni_2) {
         mov(tmp_gpr.cvt32(), 0xAAAAAAAA);
         kmovd(k_mask_cvt, tmp_gpr.cvt32());
 
@@ -1366,7 +1503,8 @@ void jit_uni_pool_kernel<isa>::generate() {
             zero_diff_src(ur_bc, with_c_tail_processing);
 
         if (jpp.alg == pooling_avg_exclude_padding
-                && (!with_c_tail_processing || (isa != avx && isa != avx2))) {
+                && (!with_c_tail_processing
+                        || (!utils::one_of(isa, avx, avx2, avx2_vnni_2)))) {
             // vmm_ker_area_h and vmm_c_tail_mask are stored in one register
             // so when vmm_c_tail_mask is used we need to load vmm_ker_area_h
             // exactly where this information is needed with the
@@ -1377,17 +1515,20 @@ void jit_uni_pool_kernel<isa>::generate() {
 
         if (jpp.alg == pooling_avg_include_padding) {
             mov(tmp_gpr, float2int((float)(kw * kh * jpp.kd)));
-            movq(xmm_tmp, tmp_gpr);
+            uni_vmovq(xmm_tmp, tmp_gpr);
             uni_vpbroadcastd(vmm_tmp, xmm_tmp);
         }
 
         if (jpp.alg == pooling_max && (jpp.is_training || jpp.is_backward)) {
-            if (!with_c_tail_processing || (isa != avx && isa != avx2)) {
+            if (!with_c_tail_processing
+                    || (!utils::one_of(isa, avx, avx2, avx2_vnni_2))) {
                 // The same situation as above(vmm_ker_area_h).
                 put_one_in_vmm();
             }
 
-            if (isa == avx || isa == avx2) { mov(reg_shuf_mask, 0x0c080400); }
+            if (utils::one_of(isa, avx, avx2, avx2_vnni_2)) {
+                mov(reg_shuf_mask, 0x0c080400);
+            }
         }
 
         const int ur_w = nstl::min(jpp.ow, jpp.ur / jpp.ur_bc);
@@ -1482,7 +1623,7 @@ void jit_uni_pool_kernel<isa>::generate() {
     if (jpp.with_eltwise && postops_injector_)
         postops_injector_->prepare_table();
 
-    if (jpp.is_bf16) {
+    if ((jpp.is_bf16 || jpp.is_f16) && isa != avx2_vnni_2) {
         align(64);
         L(idx_table);
         const uint16_t _idx[] = {0, 0, 1, 1, 2, 2, 3, 3, 4, 4, 5, 5, 6, 6, 7, 7,
@@ -1495,7 +1636,9 @@ void jit_uni_pool_kernel<isa>::generate() {
 template struct jit_uni_pool_kernel<sse41>;
 template struct jit_uni_pool_kernel<avx>;
 template struct jit_uni_pool_kernel<avx2>;
+template struct jit_uni_pool_kernel<avx2_vnni_2>;
 template struct jit_uni_pool_kernel<avx512_core>;
+template struct jit_uni_pool_kernel<avx512_core_fp16>;
 
 } // namespace x64
 } // namespace cpu
