@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2023 Intel Corporation
+* Copyright 2017-2022 Intel Corporation
 * Copyright 2018 YANDEX LLC
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -70,9 +70,9 @@ private:
     Zmm zreg(int idx) const noexcept { return Zmm(reg_idx(idx)); }
     Vmm vreg(int idx) const noexcept { return Vmm(reg_idx(idx)); }
 
-    const Xbyak::AddressFrame &vmmword = (isa == sse41) ? xword
-            : (isa == avx || isa == avx2)               ? yword
-                                                        : zword;
+    const Xbyak::AddressFrame &vmmword = (isa == sse41)  ? xword
+            : utils::one_of(isa, avx, avx2, avx2_vnni_2) ? yword
+                                                         : zword;
 
     Xmm vmm_mask = Xmm(0);
     Xmm xmm_tmp_1 = Xmm(0);
@@ -81,9 +81,8 @@ private:
 
     // Used only for avx and if c tail is present
     Vmm vmm_c_tail_mask = Vmm(2);
+    Xmm xmm_c_tail_mask = Xmm(2);
 
-    Xmm xmm_ker_area_h = Xmm(2);
-    Xmm xmm_one = Xmm(2);
     Xmm xmm_tmp = Xmm(3);
 
     Vmm vmm_ker_area_h = Vmm(2);
@@ -251,6 +250,10 @@ private:
     static bool post_ops_ok(jit_pool_conf_t &jpp, const primitive_attr_t &attr,
             const memory_desc_wrapper &dst_d);
 
+    inline bool use_bf16_emulation() const {
+        return jpp.is_bf16 && !isa_has_bf16(jpp.isa) && isa != avx2_vnni_2;
+    }
+
     std::unique_ptr<bf16_emulation_t> bf16_emu_;
     std::unique_ptr<injector::jit_uni_postops_injector_t<isa>>
             postops_injector_;
