@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -47,12 +47,14 @@ template <cpu_isa_t isa, typename Vmm>
 _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::_jit_uni_x8s8s32x_1x1_conv_kernel(
         const jit_1x1_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, isa), jcp(ajcp), attr_(attr) {
+    : jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, isa)
+    , jcp(ajcp)
+    , attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
         static constexpr bool preserve_vmm = true;
-        rhs_arg_static_params_t rhs_arg_static_params {15, r13, r14,
+        rhs_arg_static_params_t rhs_arg_static_params {15, r13, r14, r15,
                 preserve_gpr, preserve_vmm,
                 GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md)};
@@ -89,7 +91,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::bcast_loop(
     L(bcast_loop);
     {
         assert(jcp.bcast_block == jcp.ur);
-        reduce_loop(load_loop_blk, jcp.ur, 0, false);
+        reduce_loop(load_loop_blk, jcp.ur, false);
         add(aux1_reg_bcast_data, jcp.bcast_loop_bcast_step);
         add(aux_reg_output_data, jcp.bcast_loop_output_step);
 
@@ -103,7 +105,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::bcast_loop(
         Label bcast_loop_tail_out;
         cmp(reg_bcast_loop_iter, 0);
         jz(bcast_loop_tail_out, T_NEAR);
-        reduce_loop(load_loop_blk, jcp.ur_tail, 0, true);
+        reduce_loop(load_loop_blk, jcp.ur_tail, true);
         L(bcast_loop_tail_out);
     }
 }
@@ -195,9 +197,8 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_postops(const int ur,
         vmm_index_set_t vmm_idxs;
         if (jcp.with_binary) {
             iterate(ur, load_loop_blk, [&](const int i_ur, const int i_load) {
-                const int ur_stride = jcp.with_dw_conv
-                        ? jcp.nb_load_blocking * jcp.oc_block * i_ur
-                        : jcp.oc_without_padding * jcp.ngroups * i_ur;
+                const int ur_stride
+                        = jcp.oc_without_padding * jcp.ngroups * i_ur;
                 const size_t aux_output_offset = jcp.typesize_out
                         * (ur_stride + i_load * jcp.load_block);
                 const auto vmm_idx
@@ -224,7 +225,7 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::apply_postops(const int ur,
 
 template <cpu_isa_t isa, typename Vmm>
 void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
-        int load_loop_blk, int ur, int substep, bool wraparound) {
+        int load_loop_blk, int ur, bool wraparound) {
 
     // use 0x10001 to represent 2 words of 0x1
     // and avoid using uni_vpbroadcastb that is missing in jit generator
@@ -241,14 +242,6 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
         return Vmm(15 - vmm_idx);
     };
 
-    auto vmm_bias_alpha = [&]() {
-        const int vmm_idx = ur * load_loop_blk;
-        assert(vmm_idx < ker_max_reg_idx);
-        return Vmm(15 - vmm_idx);
-    };
-
-    auto xmm_bias_alpha = [&]() { return Xmm(vmm_bias_alpha().getIdx()); };
-
     auto bcast_ptr = [&](int i_reduce, int i_ur) {
         assert(i_ur < jcp.ur);
         assert(i_reduce <= jcp.reduce_loop_unroll);
@@ -299,11 +292,6 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
             mov(ptr[rsp + reg_load_data_off], reg_load_data);
             mov(reg_ptr_sum_scale, reinterpret_cast<size_t>(p_sum_scale));
         }
-        if (jcp.signed_input && (!jcp.has_vnni)) {
-            mov(reg_store_bcast, float2int(jcp.wei_adj_scale));
-            uni_vmovq(xmm_bias_alpha(), reg_store_bcast);
-            uni_vbroadcastss(vmm_bias_alpha(), xmm_bias_alpha());
-        }
         if (jcp.src_zero_point) {
             mov(reg_zp_compensation, ptr[rsp + reg_zp_compensation_off]);
             mov(reg_src_zero_point, ptr[rsp + reg_src_zero_point_off]);
@@ -317,12 +305,10 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
             const auto ptr_scales_offset
                     = jcp.is_oc_scale * (sizeof(float) * jcp.oc_block * i_load);
             if (jcp.with_bias) {
-                if (jcp.signed_input)
+                if (jcp.signed_input || jcp.dst_scale)
                     mov(reg_bias_data, ptr[rsp + reg_bias_data_off]);
                 cvt2ps(jcp.bia_dt, vmm_bias, reg_bias_data,
                         jcp.typesize_bia * jcp.oc_block * i_load, load_size);
-                if (jcp.signed_input && (!jcp.has_vnni))
-                    uni_vmulps(vmm_bias, vmm_bias, vmm_bias_alpha());
             }
             if (jcp.signed_input) {
                 mov(reg_comp_data, ptr[rsp + reg_comp_data_off]);
@@ -352,14 +338,28 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::reduce_loop(
                 uni_vcvtdq2ps(r, r);
                 if (jcp.signed_input) uni_vaddps(r, r, vmm_comp);
                 if (jcp.src_zero_point) uni_vaddps(r, r, vmm_zp_comp);
-                if (jcp.with_bias) uni_vaddps(r, r, vmm_bias);
 
                 uni_vmulps(r, r, vmm_scale);
+
+                if (jcp.with_bias) uni_vaddps(r, r, vmm_bias);
             }
         }
 
         apply_postops(ur, load_loop_blk, mask_flag_in, p_sum_scale, p_sum_zp);
 
+        if (jcp.dst_scale) {
+            mov(reg_ptr_dst_scale, ptr[rsp + reg_dst_scale_off]);
+            uni_vmovups(vmm_dst_scale, ptr[reg_ptr_dst_scale]);
+
+            /* Apply dst scale to accumulator */
+            for (int i_load = 0; i_load < load_loop_blk; ++i_load) {
+                for (int i_ur = 0; i_ur < ur; ++i_ur) {
+                    const auto r = vreg_accum(load_loop_blk, i_load, i_ur);
+                    uni_vmulps(r, r, vmm_dst_scale);
+                }
+            }
+        }
+
         if (jcp.dst_zero_point) {
             mov(reg_dst_zero_point, ptr[rsp + reg_dst_zero_point_off]);
             uni_vpbroadcastd(vmm_zp, ptr[reg_dst_zero_point]);
@@ -518,6 +518,11 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
         mov(reg_src_zero_point, ptr[param1 + GET_OFF(src_zero_point)]);
         mov(ptr[rsp + reg_src_zero_point_off], reg_src_zero_point);
     }
+    if (jcp.dst_scale) {
+        if (!jcp.signed_input) mov(ptr[rsp + reg_bias_data_off], reg_bias_data);
+        mov(reg_ptr_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        mov(ptr[rsp + reg_dst_scale_off], reg_ptr_dst_scale);
+    }
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
         mov(ptr[rsp + reg_dst_zero_point_off], reg_dst_zero_point);
@@ -538,11 +543,11 @@ void _jit_uni_x8s8s32x_1x1_conv_kernel<isa, Vmm>::generate() {
         bcast_loop(load_loop_blk);
         add(reg_load_data, load_loop_blk * jcp.load_loop_load_step);
         if (jcp.with_bias) {
-            if (jcp.signed_input)
+            if (jcp.signed_input || jcp.dst_scale)
                 mov(reg_bias_data, ptr[rsp + reg_bias_data_off]);
             add(reg_bias_data,
                     load_loop_blk * jcp.load_block * jcp.typesize_bia);
-            if (jcp.signed_input)
+            if (jcp.signed_input || jcp.dst_scale)
                 mov(ptr[rsp + reg_bias_data_off], reg_bias_data);
         }
         if (jcp.with_binary) {
@@ -900,12 +905,10 @@ status_t jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_conf(
     // miniumum size of load dim chunk for work distribution within threads
     jcp.nb_load_chunk = 1;
 
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
-
-    // only common and per-oc-channel scales are supported
-    const bool oscales_ok = one_of(oscales.mask_, 0, 1 << 1);
-    if (!oscales_ok) return status::unimplemented;
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+    jcp.is_oc_scale = wei_scales.mask_ != 0;
+    jcp.dst_scale = !dst_scales.has_default_values();
 
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
@@ -921,10 +924,10 @@ void jit_uni_x8s8s32x_1x1_conv_kernel<isa>::init_scratchpad(
         const jit_1x1_conv_conf_t &jcp, const primitive_attr_t &attr) {
     using namespace dnnl::impl::memory_tracking::names;
 
-    if (jcp.signed_input && (!jcp.has_vnni)) {
-        dim_t count = nstl::max<dim_t>(attr.output_scales_.count_, 8);
-        scratchpad.book<float>(key_conv_adjusted_scales, count);
-    }
+    const int wei_mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const dim_t scales_count = wei_mask == 0 ? 1 : jcp.oc * jcp.ngroups;
+    const dim_t count = nstl::max<dim_t>(scales_count, 8);
+    scratchpad.book<float>(key_conv_adjusted_scales, count);
 }
 
 template struct _jit_uni_x8s8s32x_1x1_conv_kernel<avx2, Ymm>;
