@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -69,13 +69,18 @@ struct jit_uni_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
                                     weights_md(1)->data_type, f32, s32, s8, u8))
                     && utils::one_of(dst_md(0)->data_type, f32, s32, s8, u8)
                     && desc()->accum_data_type == s32
-                    && attr()->has_default_values(smask_t::oscale_runtime
+                    && attr()->has_default_values(smask_t::scales_runtime
                                     | smask_t::zero_points_runtime
                                     | smask_t::post_ops | smask_t::sum_dt,
                             dst_md(0)->data_type)
+                    && attr()->scales_.has_default_values(
+                            {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST,
+                                    DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS,
+                                    DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST})
                     && attr()->post_ops_.check_sum_consistent_dt(
                             dst_md(0)->data_type)
-                    && !has_zero_dim_memory() && zero_points_ok()
+                    && !has_zero_dim_memory() && attr_scales_ok()
+                    && zero_points_ok()
                     && set_default_formats_common(
                             dat_tag(), format_tag::any, dat_tag())
                     && set_or_check_wei_format()
@@ -101,6 +106,10 @@ struct jit_uni_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
             return status::success;
         }
 
+        const memory_desc_t *dst_1x1_md(int index = 0) const {
+            return cpu_convolution_fwd_pd_t::dst_md(index);
+        }
+
         const memory_desc_t *dst_md(int index = 0) const override {
             return jcp_.with_dw_conv ? dw_conv_pd_->dst_md(index) : &dst_md_;
         }
@@ -126,6 +135,10 @@ struct jit_uni_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
                     && attr_post_op_dw_inputs() > 1)
                 return arg_usage_t::input;
 
+            if (arg == (DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_ATTR_OUTPUT_SCALES)
+                    && jcp_.with_dw_conv)
+                return arg_usage_t::input;
+
             return convolution_fwd_pd_t::arg_usage(arg);
         }
 
@@ -139,8 +152,8 @@ struct jit_uni_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
         bool zero_points_ok() const {
             // Only common zero points are supported -> mask should only be 0
             int mask_src = 0, mask_dst = 0;
-            attr()->zero_points_.get(DNNL_ARG_SRC, nullptr, &mask_src, nullptr);
-            attr()->zero_points_.get(DNNL_ARG_DST, nullptr, &mask_dst, nullptr);
+            attr()->zero_points_.get(DNNL_ARG_SRC, &mask_src);
+            attr()->zero_points_.get(DNNL_ARG_DST, &mask_dst);
             return attr()->zero_points_.has_default_values(DNNL_ARG_WEIGHTS)
                     && mask_src == 0 && mask_dst == 0;
         }
@@ -323,7 +336,7 @@ struct jit_uni_x8s8s32x_1x1_convolution_fwd_t : public primitive_t {
     status_t init(engine_t *engine) override {
         CHECK(safe_ptr_assign(kernel_,
                 new jit_uni_x8s8s32x_1x1_conv_kernel<isa>(
-                        pd()->jcp_, *pd()->attr(), *pd()->dst_md())));
+                        pd()->jcp_, *pd()->attr(), *pd()->dst_1x1_md())));
         CHECK(kernel_->create_kernel());
 
         if (pd()->jcp_.with_dw_conv) {
@@ -347,7 +360,8 @@ private:
     void execute_forward_thr(const int ithr, const int nthr, const char *src,
             const char *weights, const char *bias, const char *weights_dw,
             const char *bias_dw, char *dst, const float *oscales,
-            const float *dw_oscales, const int32_t *src_zero_point,
+            const float *dst_scales, const float *dw_oscales,
+            const float *dw_dst_scales, const int32_t *src_zero_point,
             const int32_t *dst_zero_point,
             const memory_tracking::grantor_t &scratchpad,
             const void *post_ops_binary_rhs_arg_vec,
