@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -56,7 +56,9 @@ template <cpu_isa_t isa, typename Vmm>
 _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::_jit_uni_x8s8s32x_fwd_kernel(
         const jit_conv_conf_t &ajcp, const primitive_attr_t &attr,
         const memory_desc_t &dst_md)
-    : jit_generator(nullptr, MAX_CODE_SIZE, true, isa), jcp(ajcp), attr_(attr) {
+    : jit_generator(jit_name(), nullptr, MAX_CODE_SIZE, true, isa)
+    , jcp(ajcp)
+    , attr_(attr) {
     if (jcp.with_eltwise || jcp.with_binary || jcp.with_sum) {
         using namespace binary_injector;
         static constexpr bool preserve_gpr = true;
@@ -68,7 +70,7 @@ _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::_jit_uni_x8s8s32x_fwd_kernel(
                 : jcp.oc_without_padding % isa_simd_width_;
 
         const rhs_arg_static_params_t rhs_arg_static_params {helper_vmm_idx,
-                r13, r14, preserve_gpr, preserve_vmm,
+                r13, r14, r15, preserve_gpr, preserve_vmm,
                 GET_OFF(post_ops_binary_rhs_arg_vec), GET_OFF(dst_orig),
                 memory_desc_wrapper(dst_md), tail_size, true};
         const static_params_t static_params {
@@ -240,23 +242,14 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::store_output(
         p_sum_scale = &p_entry.sum.scale;
         p_sum_zp = &p_entry.sum.zero_point;
     }
-    if (jcp.signed_input && (!jcp.has_vnni)) {
-        /* put 'wei_adj_scale = 0.5' for bias calculation */
-        mov(reg_bias_alpha, float2int(jcp.wei_adj_scale));
-        uni_vmovq(xmm_bias_alpha(), reg_bias_alpha);
-        uni_vbroadcastss(vmm_bias_alpha(), xmm_bias_alpha());
-    }
 
     for (int k = 0; k < nb_oc_block; ++k) {
         const bool mask_flag = last_oc_block_flag && k == nb_oc_block - 1;
         const int load_size = mask_flag ? get_tail_size() : get_blocking_size();
         int scale_offset = jcp.is_oc_scale * (sizeof(float) * k * oc_block);
         if (jcp.with_bias) {
-
             int bias_offset = jcp.typesize_bia * k * oc_block;
             cvt2ps(jcp.bia_dt, vmm_bias, reg_bias, bias_offset, load_size);
-            if (jcp.signed_input && (!jcp.has_vnni)) /* bias *= 0.5 */
-                uni_vmulps(vmm_bias, vmm_bias, vmm_bias_alpha());
         }
         if (jcp.signed_input) {
             const int comp_offset = sizeof(int32_t) * k * oc_block;
@@ -288,15 +281,28 @@ void _jit_uni_x8s8s32x_fwd_kernel<isa, Vmm>::store_output(
             if (jcp.src_zero_point) uni_vpaddd(vmm, vmm, vmm_zp_comp);
             uni_vcvtdq2ps(vmm, vmm);
 
-            if (jcp.with_bias) uni_vaddps(vmm, vmm, vmm_bias);
-
             uni_vmulps(vmm, vmm, vmm_scale);
+
+            if (jcp.with_bias) uni_vaddps(vmm, vmm, vmm_bias);
         }
     }
 
     apply_postops(nb_oc_block, ur_w, last_oc_block_flag, oc_block, p_sum_scale,
             p_sum_zp);
 
+    if (jcp.dst_scale) {
+        mov(reg_dst_scale, ptr[param1 + GET_OFF(dst_scale)]);
+        uni_vmovups(vmm_dst_scale, ptr[reg_dst_scale]);
+
+        /* Apply dst scale to accumulator */
+        for (int k = 0; k < nb_oc_block; k++) {
+            for (int j = 0; j < ur_w; j++) {
+                const Vmm vmm = vmm_out(j, k);
+                uni_vmulps(vmm, vmm, vmm_dst_scale);
+            }
+        }
+    }
+
     if (jcp.dst_zero_point) {
         mov(reg_dst_zero_point, ptr[param1 + GET_OFF(dst_zero_point)]);
         uni_vpbroadcastd(vmm_zp, ptr[reg_dst_zero_point]);
@@ -1297,6 +1303,12 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
     jcp.signed_input = src_d.data_type() == data_type::s8;
     jcp.is_depthwise = true && with_groups && everyone_is(1, jcp.ic, jcp.oc);
 
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+
+    jcp.is_oc_scale = wei_scales.mask_ != 0;
+    jcp.dst_scale = !dst_scales.has_default_values();
+
     const auto zp = attr.zero_points_;
     jcp.dst_zero_point = !zp.has_default_values(DNNL_ARG_DST);
     jcp.src_zero_point = !zp.has_default_values(DNNL_ARG_SRC);
@@ -1573,13 +1585,6 @@ status_t jit_uni_x8s8s32x_fwd_kernel<isa>::init_conf(jit_conv_conf_t &jcp,
 
     jcp.nb_ic_L2 = jcp.nb_ic;
 
-    const auto &oscales = attr.output_scales_;
-    jcp.is_oc_scale = oscales.mask_ == 1 << 1;
-
-    // only common and per-oc-channel scales are supported
-    const bool oscales_ok = one_of(oscales.mask_, 0, 1 << 1);
-    if (!oscales_ok) return status::unimplemented;
-
     jcp.wei_adj_scale
             = (weights_d.extra().flags & memory_extra_flags::scale_adjust)
             ? weights_d.extra().scale_adjust
@@ -1593,12 +1598,10 @@ void jit_uni_x8s8s32x_fwd_kernel<isa>::init_scratchpad(
         memory_tracking::registrar_t &scratchpad, const jit_conv_conf_t &jcp,
         const primitive_attr_t &attr) {
 
-    if (jcp.signed_input && (!jcp.has_vnni)) {
-        dim_t count = attr.output_scales_.count_ == 1
-                ? (dim_t)8
-                : attr.output_scales_.count_;
-        scratchpad.book<float>(key_conv_adjusted_scales, count);
-    }
+    const int mask = attr.scales_.get(DNNL_ARG_WEIGHTS).mask_;
+    const dim_t scales_count = mask == 0 ? 1 : jcp.oc * jcp.ngroups;
+    dim_t count = scales_count == 1 ? (dim_t)8 : scales_count;
+    scratchpad.book<float>(key_conv_adjusted_scales, count);
 }
 
 template struct _jit_uni_x8s8s32x_fwd_kernel<avx2, Ymm>;
