@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020 Intel Corporation
+* Copyright 2020-2022 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -31,7 +31,7 @@ jit_avx512_common_lrn_kernel_bwd_nhwc_t<
         float alpha, float beta, int local_size, void *code_ptr,
         size_t code_size)
     : jit_avx512_common_lrn_kernel_bwd_t<d_type>(
-            alpha, beta, local_size, code_ptr, code_size)
+            alpha, beta, local_size, code_ptr, code_size, jit_name())
     , tmp_mask_prev_ {[this]() {
         std::vector<int> v(this->local_size_ / 2);
         std::iota(v.begin(), v.end(), this->zdiffsrc_ + 2);
@@ -55,6 +55,7 @@ void jit_avx512_common_lrn_kernel_bwd_nhwc_t<d_type>::generate() {
     static const auto stack_space = zmm_size_ * 9;
 
     this->preamble();
+    if (this->bf16_emu_) this->bf16_emu_->init_vcvtneps2bf16();
     if (C_tail) reserve_stack_space(stack_space);
     this->set_up_ker_params();
     this->execute_compute_loop(num_full_16c_blocks, C_tail);
@@ -294,7 +295,7 @@ void jit_avx512_common_lrn_kernel_bwd_nhwc_t<d_type>::load_compute_data(
 
     const int loop_size = loop_size_param;
     static constexpr int mask_shift = sizeof(int32_t);
-    static constexpr int acc_size = d_type == bf16 ? 2 : 4;
+    static constexpr int acc_size = utils::one_of(d_type, bf16, f16) ? 2 : 4;
     const auto load_shifted_padded_with_zeros
             = [this](int dstIdx, int srcIdx, int maskTmpIdx, int offset) {
                   this->uni_vpxor(this->zreg(0, dstIdx), this->zreg(0, dstIdx),
@@ -318,7 +319,8 @@ void jit_avx512_common_lrn_kernel_bwd_nhwc_t<d_type>::load_compute_data(
                                     + offset),
                     true);
 
-        if (d_type == bf16 || tail_proc != tail_mode::NoTail) {
+        if (utils::one_of(d_type, bf16, f16)
+                || tail_proc != tail_mode::NoTail) {
             if (tail_proc == tail_mode::NoTail) {
                 IRB_LOOP(this->load_data(this->zreg(irb, this->z_tmp_),
                         this->EVEX_compress_addr(
@@ -433,6 +435,7 @@ void jit_avx512_common_lrn_kernel_bwd_nhwc_t<d_type>::store_compute_data(
 
 template class jit_avx512_common_lrn_kernel_bwd_nhwc_t<f32>;
 template class jit_avx512_common_lrn_kernel_bwd_nhwc_t<bf16>;
+template class jit_avx512_common_lrn_kernel_bwd_nhwc_t<f16>;
 
 } // namespace lrn
 } // namespace x64
