@@ -16,6 +16,7 @@
 
 #include <unordered_set>
 
+#include "common/dnnl_thread.hpp"
 #include "cpu/platform.hpp"
 #include "cpu/x64/injectors/jit_uni_postops_injector.hpp"
 #include "cpu/x64/matmul/brgemm_matmul_utils.hpp"
@@ -42,15 +43,27 @@ int get_default_n_block(format_tag_t matrix_b_tag) {
     // Note: consider using weights mem_descriptor 'inner_blks' to
     // return B's inner block for non-default cases.
     switch (matrix_b_tag) {
+        case aCB16b64c:
+        case aCB16b64c2b:
+        case aCB16b64c4b:
         case BA16a64b4a:
         case BA16a64b2a:
         case BA16a64b: return 64;
+        case aCB16b48c:
+        case aCB16b48c2b:
+        case aCB16b48c4b:
         case BA16a48b:
         case BA16a48b2a:
         case BA16a48b4a: return 48;
+        case aCB16b32c:
+        case aCB16b32c2b:
+        case aCB16b32c4b:
         case BA16a32b:
         case BA16a32b2a:
         case BA16a32b4a: return 32;
+        case aCB16b16c:
+        case aCB16b16c2b:
+        case aCB16b16c4b:
         case BA16a16b:
         case BA16a16b2a:
         case BA16a16b4a: return 16;
@@ -106,11 +119,14 @@ status_t check_isa_with_datatype(
     const bool ok = IMPLICATION(bm_conf_utils.is_f32(),
                             isa == avx512_core || bm_conf_utils.is_bf32())
             && IMPLICATION(bm_conf_utils.is_int8(),
-                    one_of(isa, avx512_core_bf16_amx_int8, avx512_core_vnni))
+                    one_of(isa, avx512_core_amx, avx512_core_vnni, avx2_vnni_2,
+                            avx2_vnni))
             && IMPLICATION(bm_conf_utils.is_bf16(),
-                    one_of(isa, avx512_core_bf16_amx_bf16, avx512_core_bf16))
+                    one_of(isa, avx512_core_amx, avx512_core_bf16))
+            && IMPLICATION(bm_conf_utils.is_f16(),
+                    one_of(isa, avx512_core_amx_fp16, avx512_core_fp16))
             && IMPLICATION(bm_conf_utils.is_int8_with_bf16_dst(),
-                    mayiuse(avx512_core_vnni));
+                    is_superset(isa, avx512_core_vnni) || isa == avx2_vnni_2);
     return ok ? status::success : status::unimplemented;
 }
 
@@ -122,10 +138,12 @@ brgemm_matmul_conf_utils_t::brgemm_matmul_conf_utils_t(
     , f32_dt(utils::everyone_is(f32, bgmmc.src_dt, bgmmc.wei_dt, bgmmc.dst_dt))
     , bf16_dt(utils::everyone_is(bf16, bgmmc.src_dt, bgmmc.wei_dt)
               && one_of(bgmmc.dst_dt, bf16, f32))
+    , f16_dt(utils::everyone_is(f16, bgmmc.src_dt, bgmmc.wei_dt)
+              && one_of(bgmmc.dst_dt, f16, f32))
     , int8_dt(utils::one_of(bgmmc.src_dt, u8, s8) && bgmmc.wei_dt == s8
               && one_of(bgmmc.dst_dt, u8, s8, s32, f32, bf16))
     , bf32_dt(f32_dt && attr.fpmath_mode_ == fpmath_mode::bf16
-              && isa == avx512_core_bf16_amx_bf16)
+              && isa == avx512_core_amx)
     , A_any_layout(A_any_layout)
     , B_any_layout(B_any_layout)
     , C_any_layout(C_any_layout)
@@ -143,8 +161,9 @@ brgemm_matmul_conf_utils_t::brgemm_matmul_conf_utils_t(
     , blocked_B_layouts_allowed(!utils::one_of(format_tag::undef,
               blocked_64n_B_layout_tag, blocked_48n_B_layout_tag,
               blocked_32n_B_layout_tag, blocked_16n_B_layout_tag))
-    , n_blk_fixed((!B_any_layout) && blocked_B_layouts_allowed) {
-    assert(int8_dt || bf16_dt || f32_dt || bf32_dt);
+    , n_blk_fixed((!B_any_layout) && blocked_B_layouts_allowed)
+    , isa_(isa) {
+    assert(int8_dt || bf16_dt || f16_dt || f32_dt || bf32_dt);
 }
 
 status_t brgemm_matmul_conf_utils_t::set_or_check_B_tag(
@@ -183,11 +202,10 @@ status_t brgemm_matmul_conf_utils_t::set_or_check_B_tag(
         if (transposed_tensor_layout_tag == bgmmc.wei_tag) {
             memory_desc_t B_md_plain;
             const status_t status
-                    = dnnl_memory_desc_init_by_tag(&B_md_plain, B_md.ndims,
-                            B_md.dims, B_md.data_type, plain_tensor_layout_tag);
+                    = memory_desc_init_by_tag(B_md_plain, B_md.ndims, B_md.dims,
+                            B_md.data_type, plain_tensor_layout_tag);
             if (status != status::success) return status;
-            if (status == status::success && B_md_plain == B_md)
-                bgmmc.wei_tag = plain_tensor_layout_tag;
+            if (B_md_plain == B_md) bgmmc.wei_tag = plain_tensor_layout_tag;
         }
 
         if (format_tag::undef == bgmmc.wei_tag) return status::unimplemented;
@@ -214,7 +232,8 @@ status_t brgemm_matmul_conf_utils_t::set_or_check_tags(memory_desc_t &A_md,
         CHECK(memory_desc_init_by_tag(A_md, desired_A_tag));
         bgmmc.src_tag = desired_A_tag;
     } else {
-        bgmmc.src_tag = (this->is_bf16() || this->is_f32() || this->is_bf32())
+        bgmmc.src_tag = (this->is_bf16() || this->is_f32() || this->is_bf32()
+                                || this->is_f16())
                 ? memory_desc_matches_one_of_tag(A_md, plain_tensor_layout_tag,
                         transposed_tensor_layout_tag, acbd, adbc)
                 : memory_desc_matches_one_of_tag(
@@ -242,14 +261,17 @@ status_t brgemm_matmul_conf_utils_t::set_or_check_tags(memory_desc_t &A_md,
 status_t brgemm_matmul_conf_utils_t::set_B_flags(memory_desc_t &B_md) const {
 
     memory_desc_t want_B_md = B_md;
+    // Set bits for all dimensions except k dimension
+    const int compensation_mask
+            = ((1 << bgmmc.ndims) - 1 - (1 << (bgmmc.ndims - 2)));
     if (bgmmc.s8s8_compensation_required && bgmmc.blocked_B) {
         want_B_md.extra.flags |= memory_extra_flags::compensation_conv_s8s8;
-        want_B_md.extra.compensation_mask = (1 << 1);
+        want_B_md.extra.compensation_mask = compensation_mask;
     }
     if (bgmmc.src_zp_type != brgemm_broadcast_t::none && bgmmc.blocked_B) {
         want_B_md.extra.flags
                 |= memory_extra_flags::compensation_conv_asymmetric_src;
-        want_B_md.extra.asymm_compensation_mask = (1 << 1);
+        want_B_md.extra.asymm_compensation_mask = compensation_mask;
     }
 
     if (B_any_layout) {
@@ -262,27 +284,31 @@ status_t brgemm_matmul_conf_utils_t::set_B_flags(memory_desc_t &B_md) const {
 
 format_tag_t brgemm_matmul_conf_utils_t::pick_blocked_B_layout(
         int n_blk) const {
-    if (bgmmc.ndims > 2) return format_tag::undef;
+
+    if (bgmmc.ndims > 3) return format_tag::undef;
     if (this->is_int8()) switch (n_blk) {
-            case 64: return BA16a64b4a;
-            case 48: return BA16a48b4a;
-            case 32: return BA16a32b4a;
-            case 16: return BA16a16b4a;
+            case 64: return bgmmc.ndims == 3 ? aCB16b64c4b : BA16a64b4a;
+            case 48: return bgmmc.ndims == 3 ? aCB16b48c4b : BA16a48b4a;
+            case 32: return bgmmc.ndims == 3 ? aCB16b32c4b : BA16a32b4a;
+            case 16: return bgmmc.ndims == 3 ? aCB16b16c4b : BA16a16b4a;
             default: return format_tag::undef;
         }
-    if (this->is_bf16()) switch (n_blk) {
-            case 64: return BA16a64b2a;
-            case 48: return BA16a48b2a;
-            case 32: return BA16a32b2a;
-            case 16: return BA16a16b2a;
+
+    if (this->is_bf16()
+            || (this->is_f16() && bgmmc.isa == avx512_core_amx_fp16))
+        switch (n_blk) {
+            case 64: return bgmmc.ndims == 3 ? aCB16b64c2b : BA16a64b2a;
+            case 48: return bgmmc.ndims == 3 ? aCB16b48c2b : BA16a48b2a;
+            case 32: return bgmmc.ndims == 3 ? aCB16b32c2b : BA16a32b2a;
+            case 16: return bgmmc.ndims == 3 ? aCB16b16c2b : BA16a16b2a;
             default: return format_tag::undef;
         }
     // Note: bf32 assumes f32 blocking
-    if (this->is_f32() || this->is_bf32()) switch (n_blk) {
-            case 64: return BA16a64b;
-            case 48: return BA16a48b;
-            case 32: return BA16a32b;
-            case 16: return BA16a16b;
+    if (this->is_f32() || this->is_bf32() || this->is_f16()) switch (n_blk) {
+            case 64: return bgmmc.ndims == 3 ? aCB16b64c : BA16a64b;
+            case 48: return bgmmc.ndims == 3 ? aCB16b48c : BA16a48b;
+            case 32: return bgmmc.ndims == 3 ? aCB16b32c : BA16a32b;
+            case 16: return bgmmc.ndims == 3 ? aCB16b16c : BA16a16b;
             default: return format_tag::undef;
         }
     return format_tag::undef;
@@ -509,7 +535,7 @@ struct matmul_avx512_blocking_params_t {
                 bgmmc.acc_dt, bgmmc.dst_dt, bgmmc.with_sum);
         bgmmc.LDA = (bgmmc.src_tag == acbd && !bgmmc.use_buffer_a
                         ? bgmmc.A_strides[1] / bgmmc.a_dt_sz
-                        : get_actual_lda(bgmmc.use_buffer_a, bgmmc.a_dt_sz));
+                        : get_actual_lda(bgmmc.use_buffer_a, bgmmc.tr_a_dt_sz));
     }
 };
 
@@ -526,10 +552,12 @@ void compute_blocking_heuristic_amx(const brgemm_matmul_conf_t &bgmmc,
     const int min_k_per_thread = 1024;
     const int max_k_parallel_work
             = div_up(static_cast<int>(bgmmc.K), min_k_per_thread);
-    const bool is_amx_bf16
-            = bgmmc.isa == avx512_core_bf16_amx_bf16; // note: also bf32
-    const bool is_amx_int8 = bgmmc.isa == avx512_core_bf16_amx_int8;
-    const int max_nthr_k = is_amx_bf16 && bgmmc.batch == 1
+    const bool is_amx_xf16 = bgmmc.is_amx
+            && (bm_conf_utils.is_bf16() || bm_conf_utils.is_f16()
+                    || bm_conf_utils.is_bf32());
+    const bool is_amx_int8 = bgmmc.is_amx && bm_conf_utils.is_int8();
+
+    const int max_nthr_k = is_amx_xf16 && bgmmc.batch == 1
             ? nstl::min(saturate(1, 7, bgmmc.nthr / 8), max_k_parallel_work)
             : 1;
     int iter = 0;
@@ -550,7 +578,7 @@ void compute_blocking_heuristic_amx(const brgemm_matmul_conf_t &bgmmc,
                 = (maybe_low_blocking || low_parallelism) && bgmmc.M_blk > 32
                 ? div_up(bgmmc.M_blk, 2)
                 : bgmmc.M_blk;
-        const int min_N_blk = low_parallelism && is_amx_bf16
+        const int min_N_blk = low_parallelism && is_amx_xf16
                         && !bm_conf_utils.check_n_blk_fixed()
                         && bgmmc.N_blk > 32
                 ? 32
@@ -676,6 +704,63 @@ float compute_blocking_heuristic_avx512(brgemm_matmul_conf_t &bgmmc,
     return best_imbalance;
 }
 
+float compute_blocking_heuristic_avx2(brgemm_matmul_conf_t &bgmmc,
+        const brgemm_matmul_conf_utils_t &bm_conf_utils,
+        const matmul_avx512_blocking_params_t::matmul_params_t &matmul,
+        matmul_avx512_blocking_params_t &best_blocking) {
+
+    const int nthr = bgmmc.nthr;
+
+    const int max_m_blk = nstl::min(/*64*/ 256, matmul.M);
+    int min_m_blk = nstl::min(32, matmul.M); // max_m_blk
+
+    int n_blk = bgmmc.N_blk;
+    const int n_chunks = div_up(matmul.N, n_blk);
+    const int max_n_chunks = bgmmc.use_buffer_a ? 16 : 1;
+    const int n_chunks_start = nstl::min(max_n_chunks, n_chunks);
+
+    int default_k_blk = 1024;
+    int k_blk = nstl::min(matmul.K, default_k_blk);
+    int start_nthr_k = 1;
+
+    // for cases with low parallel work, reduce 'min_m_blk' to
+    // increase potential parallelization balance.
+    const size_t max_parallel = matmul.batch * n_chunks;
+    const bool low_parallel_work = static_cast<size_t>(nthr) > max_parallel;
+    if (low_parallel_work) {
+
+        min_m_blk = nstl::min(matmul.M, 16);
+
+        bool low_spatial_work = matmul.M <= 40;
+        if (low_spatial_work) {
+
+            // Reduce n_blk size to increase parallel space
+            // note: over reduction of n_blk size on 2d shapes when n_chunks == 1
+            // showed significant performance degradation
+            if (!bm_conf_utils.check_n_blk_fixed()
+                    && IMPLICATION(n_chunks == 1, bgmmc.batch_ndims > 0))
+                n_blk = nstl::min(matmul.N, 32);
+        }
+    }
+
+    float best_imbalance = 1.f; // reduce
+    for_(int nthr_k = start_nthr_k; nthr_k >= 1; --nthr_k)
+    for_(int n_chunk_size = n_chunks_start; n_chunk_size >= 1; --n_chunk_size)
+    for (int m_blk = max_m_blk; m_blk >= min_m_blk; --m_blk) {
+
+        matmul_avx512_blocking_params_t cur_params(matmul, nthr);
+        cur_params.update_params(
+                1, m_blk, n_chunk_size, n_blk, 1, k_blk, nthr_k);
+
+        float cur_imbalance = cur_params.get_imbalance();
+        if (cur_imbalance < best_imbalance) {
+            best_imbalance = cur_imbalance;
+            best_blocking = cur_params;
+        }
+    }
+    return best_imbalance;
+}
+
 status_t compute_blocking_heuristic(brgemm_matmul_conf_t &bgmmc,
         const brgemm_matmul_conf_utils_t &bm_conf_utils) {
 
@@ -717,7 +802,7 @@ status_t compute_blocking_heuristic(brgemm_matmul_conf_t &bgmmc,
 
         best_blocking.update_configuration(bgmmc);
 
-    } else {
+    } else if (is_superset(bm_conf_utils.get_isa(), avx512_core)) {
         // TODO:
         // *) adjust K_BLK using 'rnd_up(bgmmc.K, bgmmc.required_k_granularity)'
         //    for non-f32 datatypes.
@@ -765,6 +850,20 @@ status_t compute_blocking_heuristic(brgemm_matmul_conf_t &bgmmc,
 
         if (best_imbalance == 1.f) return status::unimplemented;
 
+        best_blocking.update_configuration(bgmmc);
+    } else {
+        assert(one_of(bm_conf_utils.get_isa(), avx2_vnni, avx2_vnni_2));
+
+        const matmul_avx512_blocking_params_t::matmul_params_t matmul(
+                bgmmc.M, bgmmc.N, bgmmc.K, bgmmc.batch);
+
+        matmul_avx512_blocking_params_t best_blocking(matmul, bgmmc.nthr);
+
+        const float best_imbalance = compute_blocking_heuristic_avx2(
+                bgmmc, bm_conf_utils, matmul, best_blocking);
+
+        if (best_imbalance == 1.f) return status::unimplemented;
+
         best_blocking.update_configuration(bgmmc);
     }
 
@@ -791,7 +890,7 @@ status_t init_brgemm_matmul_conf(cpu_isa_t isa, brgemm_matmul_conf_t &bgmmc,
     bgmmc.with_bias = mmd.bias_desc.format_kind != format_kind::undef;
     bgmmc.bia_dt = bgmmc.with_bias ? mmd.bias_desc.data_type : data_type::undef;
     bgmmc.s8s8_compensation_required
-            = isa == avx512_core_vnni && bgmmc.src_dt == s8;
+            = bgmmc.src_dt == s8 && one_of(isa, avx512_core_vnni, avx2_vnni);
     bgmmc.ndims = dst_d.ndims();
 
     brgemm_matmul_conf_utils_t bm_conf_utils(bgmmc, isa, attr,
@@ -802,11 +901,7 @@ status_t init_brgemm_matmul_conf(cpu_isa_t isa, brgemm_matmul_conf_t &bgmmc,
 
     CHECK(check_isa_with_datatype(isa, bm_conf_utils));
 
-    const bool is_amx_int8 = isa == avx512_core_bf16_amx_int8;
-    const bool is_amx_bf16 = isa == avx512_core_bf16_amx_bf16;
-    const bool is_amx_bf32 = bm_conf_utils.is_bf32();
-    bgmmc.is_amx = is_amx_int8 || is_amx_bf16 || is_amx_bf32;
-
+    bgmmc.is_amx = is_superset(isa, avx512_core_amx);
     bgmmc.a_dt_sz = bgmmc.tr_a_dt_sz = types::data_type_size(bgmmc.src_dt);
     bgmmc.b_dt_sz = bgmmc.tr_b_dt_sz = types::data_type_size(bgmmc.wei_dt);
 
@@ -819,6 +914,12 @@ status_t init_brgemm_matmul_conf(cpu_isa_t isa, brgemm_matmul_conf_t &bgmmc,
         bgmmc.wei_dt = bf16;
         bgmmc.tr_a_dt_sz = types::data_type_size(bf16);
         bgmmc.tr_b_dt_sz = types::data_type_size(bf16);
+    } else if (bm_conf_utils.is_f16() && bgmmc.isa == avx512_core_fp16) {
+        // Similar to bf32, convert input data before compute
+        bgmmc.src_dt = f32;
+        bgmmc.wei_dt = f32;
+        bgmmc.tr_a_dt_sz = types::data_type_size(f32);
+        bgmmc.tr_b_dt_sz = types::data_type_size(f32);
     }
 
     bgmmc.acc_dt = bm_conf_utils.is_int8() ? s32 : f32;
@@ -827,16 +928,24 @@ status_t init_brgemm_matmul_conf(cpu_isa_t isa, brgemm_matmul_conf_t &bgmmc,
     bgmmc.acc_dt_sz = types::data_type_size(bgmmc.acc_dt);
     if (bgmmc.with_bias) bgmmc.bias_dt_sz = types::data_type_size(bgmmc.bia_dt);
 
-    bgmmc.with_scales = !attr.output_scales_.has_default_values();
+    const auto &src_scales = attr.scales_.get(DNNL_ARG_SRC);
+    const auto &wei_scales = attr.scales_.get(DNNL_ARG_WEIGHTS);
+    bgmmc.with_scales = !src_scales.has_default_values()
+            || !wei_scales.has_default_values();
     if (bgmmc.with_scales) {
-        const auto &oscales = attr.output_scales_;
-        bgmmc.is_oscale_per_n = oscales.mask_ == 1 << (bgmmc.ndims - 1);
+        bgmmc.is_oscale_per_n = wei_scales.mask_ == 1 << (bgmmc.ndims - 1);
 
         // only common and per-oc-channel scales are supported
-        const bool oscales_ok = oscales.mask_ == 0 || bgmmc.is_oscale_per_n;
+        const bool oscales_ok = wei_scales.mask_ == 0 || bgmmc.is_oscale_per_n;
         if (!oscales_ok) return status::unimplemented;
     }
 
+    const auto &dst_scales = attr.scales_.get(DNNL_ARG_DST);
+    bgmmc.with_dst_scales = !dst_scales.has_default_values();
+    // only common scales are supported
+    if (bgmmc.with_dst_scales && dst_scales.mask_ != 0)
+        return status::unimplemented;
+
     const auto &p = attr.post_ops_;
     bgmmc.with_sum = p.find(primitive_kind::sum) != -1;
     const int eltwise_ind = p.find(primitive_kind::eltwise);
@@ -897,12 +1006,16 @@ status_t init_brgemm_matmul_conf(cpu_isa_t isa, brgemm_matmul_conf_t &bgmmc,
 
     bgmmc.transposed_A = (bm_conf_utils.check_is_transposed(bgmmc.src_tag)
             || bgmmc.src_tag == adbc);
-    const bool lda_is_big_2pow = bm_conf_utils.is_bf16() && !bgmmc.transposed_A
-            && math::is_pow2(bgmmc.K) && bgmmc.K >= 4096 && bgmmc.M >= 1024;
+    const bool lda_is_big_2pow
+            = (bm_conf_utils.is_bf16()
+                      || (bgmmc.is_amx && bm_conf_utils.is_f16()))
+            && !bgmmc.transposed_A && math::is_pow2(bgmmc.K) && bgmmc.K >= 4096
+            && bgmmc.M >= 1024;
     const bool is_copy_a_required
             = (bgmmc.is_amx
                       && ((bgmmc.K % bgmmc.required_k_granularity != 0)
                               || bm_conf_utils.is_bf32()))
+            || (bm_conf_utils.is_f16() && isa == avx512_core_fp16)
             || bgmmc.wei_zp_type != brgemm_broadcast_t::none
             || bgmmc.transposed_A || lda_is_big_2pow;
     bgmmc.use_buffer_a = is_copy_a_required;
@@ -997,8 +1110,8 @@ void init_aux_values(brgemm_matmul_conf_t &bgmmc,
     bgmmc.buffer_a_per_thread_sz
             = bgmmc.buffer_a_chunk_shift_along_m * bgmmc.M_chunk_size;
 
-    bgmmc.buffer_b_chunk_sz
-            = bgmmc.b_dt_sz * bgmmc.LDB * rnd_up(bgmmc.K_blk, bgmmc.wei_k_blk);
+    bgmmc.buffer_b_chunk_sz = bgmmc.tr_b_dt_sz * bgmmc.LDB
+            * rnd_up(bgmmc.K_blk, bgmmc.wei_k_blk);
     bgmmc.buffer_b_per_thread_sz
             = bgmmc.buffer_b_chunk_sz * bgmmc.brgemm_batch_size;
 
@@ -1054,7 +1167,7 @@ void init_aux_values(brgemm_matmul_conf_t &bgmmc,
             bgmmc.with_scales, bgmmc.with_eltwise, bgmmc.with_binary,
             bgmmc.acc_dt != bgmmc.dst_dt, bgmmc.s8s8_compensation_required,
             bgmmc.has_zero_point_a, bgmmc.has_zero_point_b,
-            bgmmc.has_zero_point_c);
+            bgmmc.has_zero_point_c, bgmmc.with_dst_scales);
 
     bgmmc.zp_a_comp_shift_n = bgmmc.wei_n_blk;
     bgmmc.zp_a_comp_elems_per_thr
@@ -1109,7 +1222,7 @@ void init_scratchpad(memory_tracking::registrar_t &scratchpad,
                 bgmmc.nthr * bgmmc.zp_b_comp_elems_per_thr,
                 types::data_type_size(s32));
 
-    if (one_of(bgmmc.isa, avx512_core_bf16_amx_int8, avx512_core_bf16_amx_bf16))
+    if (is_superset(bgmmc.isa, avx512_core_amx))
         scratchpad.book(key_conv_amx_tile_buffer,
                 static_cast<size_t>(bgmmc.nthr) * bgmmc.wsp_tile_per_thr_bytes,
                 default_data_align);
