@@ -15,6 +15,7 @@
 *******************************************************************************/
 #include <tuple>
 #include <utility>
+#include "common/dnnl_thread.hpp"
 #include "cpu/rnn/rnn_utils.hpp"
 #include "cpu/x64/rnn/rnn_brgemm_utils.hpp"
 
@@ -35,7 +36,7 @@ std::pair<dim_t, dim_t> brgemm_calc_k_block_amx(
         dim_t K1, dim_t K2, bool is_int8);
 std::pair<dim_t, dim_t> brgemm_calc_k_block_vanilla_rnn(dim_t K1, dim_t K2,
         dim_t M, dim_t n_block, dim_t src_layer_type_size, dim_t As, dim_t Bs,
-        dim_t Cs, dim_t l2_cache_size, bool is_bf16);
+        dim_t Cs, dim_t l2_cache_size, bool is_bf16, x64::cpu_isa_t isa);
 
 dim_t brgemm_calc_m_block(alg_kind_t cell_kind, prop_kind_t aprop, dim_t nthr,
         dim_t M, dim_t N_blocks, bool is_f32, bool is_int8_amx,
@@ -51,10 +52,8 @@ dim_t adjust_m_block_lstm(dim_t nthr, dim_t M, dim_t N_blocks, bool is_int8_amx,
         bool is_bf16_amx);
 
 x64::cpu_isa_t brgemm_calc_isa(dim_t K1, dim_t K2, bool is_int8, bool is_bf16) {
-    const bool is_amx_int8
-            = is_int8 && x64::mayiuse(x64::avx512_core_bf16_amx_int8);
-    const bool is_amx_bf16
-            = is_bf16 && x64::mayiuse(x64::avx512_core_bf16_amx_bf16);
+    const bool is_amx_int8 = is_int8 && x64::mayiuse(x64::avx512_core_amx);
+    const bool is_amx_bf16 = is_bf16 && x64::mayiuse(x64::avx512_core_amx);
 
     if (is_amx_int8 || is_amx_bf16) {
         const dim_t padding = (is_int8 ? 4 : (is_bf16 ? 2 : 1));
@@ -67,41 +66,38 @@ x64::cpu_isa_t brgemm_calc_isa(dim_t K1, dim_t K2, bool is_int8, bool is_bf16) {
                 || k2_block_tail % padding || k1_block_amx % padding
                 || k2_block_amx % padding;
 
-        if (!amx_block_invalid) {
-            return is_amx_int8 ? x64::avx512_core_bf16_amx_int8
-                               : x64::avx512_core_bf16_amx_bf16;
-        }
+        if (!amx_block_invalid) return x64::avx512_core_amx;
     }
 
     if (is_int8) {
         return x64::avx512_core_vnni;
     } else if (is_bf16) {
         return x64::avx512_core_bf16;
+    } else { // f32
+        return utils::map(true, x64::isa_undef, mayiuse(avx512_core),
+                avx512_core, mayiuse(avx2), avx2);
     }
-
-    return x64::isa_any;
 }
 
 std::pair<dim_t, dim_t> brgemm_calc_k_block(dim_t K1, dim_t K2, dim_t M,
         dim_t n_block, alg_kind_t cell_kind, dim_t src_layer_type_size,
         dim_t As, dim_t Bs, dim_t Cs, dim_t l2_cache_size, x64::cpu_isa_t isa,
         bool is_int8, bool is_bf16) {
-    const bool is_amx_int8 = is_int8 && isa == x64::avx512_core_bf16_amx_int8;
-    const bool is_amx_bf16 = is_bf16 && isa == x64::avx512_core_bf16_amx_bf16;
+    const bool is_amx_int8 = is_int8 && isa == x64::avx512_core_amx;
+    const bool is_amx_bf16 = is_bf16 && isa == x64::avx512_core_amx;
 
     if (is_amx_int8 || is_amx_bf16)
         return brgemm_calc_k_block_amx(K1, K2, is_int8);
     else if (cell_kind == alg_kind::vanilla_rnn)
         return brgemm_calc_k_block_vanilla_rnn(K1, K2, M, n_block,
-                src_layer_type_size, As, Bs, Cs, l2_cache_size, is_bf16);
+                src_layer_type_size, As, Bs, Cs, l2_cache_size, is_bf16, isa);
 
     return std::make_pair(K1, K2);
 }
 
 std::pair<dim_t, dim_t> brgemm_calc_k_block_amx(
         dim_t K1, dim_t K2, bool is_int8) {
-    const bool is_amx_int8
-            = is_int8 && x64::mayiuse(x64::avx512_core_bf16_amx_int8);
+    const bool is_amx_int8 = is_int8 && x64::mayiuse(x64::avx512_core_amx);
     const dim_t max_row_width = is_amx_int8 ? 64 : 32;
 
     dim_t k1_block = nstl::min(K1, max_row_width);
@@ -117,16 +113,20 @@ std::pair<dim_t, dim_t> brgemm_calc_k_block_amx(
 
 std::pair<dim_t, dim_t> brgemm_calc_k_block_vanilla_rnn(dim_t K1, dim_t K2,
         dim_t M, dim_t n_block, dim_t src_layer_type_size, dim_t As, dim_t Bs,
-        dim_t Cs, dim_t l2_cache_size, bool is_bf16) {
+        dim_t Cs, dim_t l2_cache_size, bool is_bf16, x64::cpu_isa_t isa) {
 
     //Heuristics experimentally selected.
+    // For avx2 (i.e., non-avx512_core) the value is chosen while tuning
+    // deepspeech model. It might be applicable for avx512_core as well (TODO)
+    const float l2_occupancy
+            = is_superset(isa, x64::avx512_core) ? 0.25f : 0.75f;
     const bool should_adjust_by_l2 = static_cast<float>(As + Bs + Cs)
-            >= 0.25 * static_cast<float>(l2_cache_size);
+            >= l2_occupancy * static_cast<float>(l2_cache_size);
     dim_t k1_block = K1;
     dim_t k2_block = K2;
 
     if (should_adjust_by_l2) {
-        int block_size = (l2_cache_size * 0.25f)
+        int block_size = (l2_cache_size * l2_occupancy)
                 / ((M + n_block) * src_layer_type_size);
 
         if (is_bf16) {
@@ -260,8 +260,7 @@ x64::cpu_isa_t adjust_isa_by_m_block(
      * throughput.
      */
     if (is_int8_amx && m_block < 4) {
-        if (x64::mayiuse(x64::avx512_core_bf16_amx_int8))
-            return x64::avx512_core_bf16_amx_int8;
+        if (x64::mayiuse(x64::avx512_core_amx)) return x64::avx512_core_amx;
     }
 
     return current_isa;
@@ -294,12 +293,12 @@ void rnn_brgemm_base_t::init_scratchpad(const cpu::rnn_utils::rnn_conf_t &rnn,
         memory_desc_t wei_layer_desc;
         const auto tag = rnn.n_block == 64 ? format_tag::ldgOI64o2i
                                            : format_tag::ldgOI32o2i;
-        dnnl_memory_desc_init_by_tag(
-                &wei_layer_desc, 5, wei_layer_dims, data_type::bf16, tag);
+        memory_desc_init_by_tag(
+                wei_layer_desc, 5, wei_layer_dims, data_type::bf16, tag);
 
         memory_desc_t wei_iter_desc;
-        dnnl_memory_desc_init_by_tag(
-                &wei_iter_desc, 5, wei_iter_dims, data_type::bf16, tag);
+        memory_desc_init_by_tag(
+                wei_iter_desc, 5, wei_iter_dims, data_type::bf16, tag);
 
         scratchpad.book(key_rnn_bf32_wei_layer_trans,
                 memory_desc_wrapper(wei_layer_desc).size(), 64);
@@ -336,10 +335,11 @@ status_t rnn_brgemm_t<prop_kind::forward>::configure_brgemm(
     rnn.K2padded = utils::rnd_up(rnn.K2, padding);
 
     rnn.brgemm_isa = brgemm_calc_isa(rnn.K1, rnn.K2, is_int8, is_bf16);
+    if (rnn.brgemm_isa == isa_undef) return status::unimplemented;
     const int bf32_reduction_dim_threshold = 128;
     const bool is_shape_ok_for_bf32 = rnn.K1 >= bf32_reduction_dim_threshold
             && rnn.K2 >= bf32_reduction_dim_threshold;
-    const bool is_bf32 = is_bf16 && rnn.brgemm_isa == avx512_core_bf16_amx_bf16
+    const bool is_bf32 = is_bf16 && rnn.brgemm_isa == avx512_core_amx
             && rnn.dt_conf == all_f32
             // workspace data type and layouts can differ between fwd and bwd
             // implementations during training.
@@ -354,7 +354,8 @@ status_t rnn_brgemm_t<prop_kind::forward>::configure_brgemm(
             = rnn.is_cell_int8_amx() || rnn.is_cell_bf16_amx();
     const bool can_use_block64
             = is_amx_isa_selected && rnn.N % 64 == 0 && !rnn.is_lstm_projection;
-    rnn.n_block = can_use_block64 ? 64 : 32;
+    const int simd_w = isa_max_vlen(rnn.brgemm_isa) / sizeof(float);
+    rnn.n_block = can_use_block64 ? 64 : 2 * simd_w;
     rnn.N_blocks = utils::div_up(rnn.N, rnn.n_block);
     rnn.n_tail = rnn.N % rnn.n_block;
 
@@ -447,9 +448,7 @@ status_t rnn_brgemm_t<prop_kind::forward>::configure_brgemm(
                 rnn.brgemm_isa = rnn.is_cell_dt_int8() ? x64::avx512_core_vnni
                                                        : x64::avx512_core_bf16;
             } else {
-                rnn.brgemm_isa = rnn.is_cell_dt_int8()
-                        ? x64::avx512_core_bf16_amx_int8
-                        : x64::avx512_core_bf16_amx_bf16;
+                rnn.brgemm_isa = x64::avx512_core_amx;
             }
         } else {
             rnn.kproj_block = rnn.Kproj;
@@ -866,7 +865,7 @@ status_t rnn_brgemm_t<prop_kind::backward>::configure_brgemm(
     diff_src_conf.isa
             = brgemm_calc_isa(diff_src_conf.K, diff_src_conf.K, false, is_bf16);
     const bool is_bf16_amx
-            = is_bf16 && diff_src_conf.isa == x64::avx512_core_bf16_amx_bf16;
+            = is_bf16 && diff_src_conf.isa == x64::avx512_core_amx;
     const bool split_gates_computation = is_bf16_amx && diff_src_conf.K >= 1024
             && diff_src_conf.n_tail == 0;
     diff_src_conf.gates_block = split_gates_computation ? 1 : rnn.n_gates;
@@ -916,7 +915,7 @@ status_t rnn_brgemm_t<prop_kind::backward>::configure_brgemm(
             = brgemm_calc_isa(diff_wei_conf.K, diff_wei_conf.K, false, is_bf16);
 
     const bool is_wei_bf16_amx = rnn.is_cell_dt_bf16()
-            && diff_wei_conf.isa == x64::avx512_core_bf16_amx_bf16;
+            && diff_wei_conf.isa == x64::avx512_core_amx;
     const bool diff_wei_can_use_nblock64 = is_wei_bf16_amx
             && diff_wei_conf.N % 64 == 0 && !rnn.is_lstm_peephole;
     diff_wei_conf.n_block = diff_wei_can_use_nblock64 ? 64 : 32;
@@ -986,20 +985,20 @@ status_t rnn_brgemm_t<prop_kind::backward>::configure_brgemm(
     rnn.K2padded = rnn.K1padded;
     rnn.unfused_post_gemm = true;
 
-    if (utils::one_of(x64::avx512_core_bf16_amx_bf16, diff_wei_conf.isa,
-                diff_src_conf.isa)) {
-        rnn.brgemm_isa = x64::avx512_core_bf16_amx_bf16;
+    if (utils::one_of(
+                x64::avx512_core_amx, diff_wei_conf.isa, diff_src_conf.isa)) {
+        rnn.brgemm_isa = x64::avx512_core_amx;
     } else {
         rnn.brgemm_isa = diff_wei_conf.isa;
     }
 
     if (!rnn.is_orig_gru) {
         rnn.diff_src_brgemm.loop_order
-                = is_bf16 && diff_src_conf.isa == x64::avx512_core_bf16_amx_bf16
+                = is_bf16 && diff_src_conf.isa == x64::avx512_core_amx
                 ? brgemm_rnn_execute_loop_order_t::mblk_nblk
                 : brgemm_rnn_execute_loop_order_t::nblk_mblk;
         rnn.diff_wei_brgemm.loop_order
-                = is_bf16 && diff_wei_conf.isa == x64::avx512_core_bf16_amx_bf16
+                = is_bf16 && diff_wei_conf.isa == x64::avx512_core_amx
                 ? brgemm_rnn_execute_loop_order_t::mblk_nblk
                 : brgemm_rnn_execute_loop_order_t::nblk_mblk;
     }
@@ -1167,7 +1166,7 @@ static status_t init_kernels_diff_src(rnn_diff_src_brgemm_t &diff_src,
     }
 
     const bool is_bf16_amx = rnn.is_cell_dt_bf16()
-            && diff_src_conf.isa == x64::avx512_core_bf16_amx_bf16;
+            && diff_src_conf.isa == x64::avx512_core_amx;
 
     if (is_bf16_amx) {
         CHECK(brgemm_init_tiles(diff_src.desc_iter_layer_beta0_,
@@ -1277,7 +1276,7 @@ static status_t init_kernels_diff_wei(rnn_diff_wei_brgemm_t &diff_wei,
     }
 
     const bool is_bf16_amx_wei = rnn.is_cell_dt_bf16()
-            && diff_wei_conf.isa == x64::avx512_core_bf16_amx_bf16;
+            && diff_wei_conf.isa == x64::avx512_core_amx;
 
     if (is_bf16_amx_wei) {
         CHECK(brgemm_init_tiles(
