@@ -179,7 +179,17 @@ private:
             if (is_last_dim) step = rem_target_base_blk;
         }
 
-        int ret = utils::rnd_dn(target_dim, step);
+        int dimension_bound = [&]() {
+            // Prefer powers of 2 for small dimensions as they generally
+            // result in better load instructions.
+            int bound = utils::rnd_up_pow2(dim);
+            if (bound % step == 0 && (double)dim / bound >= target_eff)
+                return bound;
+            return utils::rnd_up(dim, step);
+        }();
+        int target_bound = utils::rnd_dn(target_dim, step);
+        int ret = std::min(dimension_bound, target_bound);
+
         while (ret >= step) {
             bool ok = true;
             if (require_pow_2 && !math::is_pow2(ret)) ok = false;
@@ -203,8 +213,8 @@ private:
         return ret;
     }
 
-    bool is_thr_unlimited() const {
-        if (level_ != tile_level_t::thr) return false;
+    bool is_loop_unlimited() const {
+        if (level_ != tile_level_t::loop) return false;
         if (rem_bmnk_dim_.is_unlimited()) return false;
         if (bmnk_dim_.tg_dim() != 1) return false;
         return true;
@@ -227,7 +237,7 @@ private:
     bool try_assign_multi_blocks() {
         // Check restrictions to apply the heuristics.
         if (!allow_fuse()) return false;
-        if (!is_thr_unlimited() && !is_iter_full_match()) return false;
+        if (!is_loop_unlimited() && !is_iter_full_match()) return false;
 
         int nprb_dims = (int)prb_dims_.size();
         std::vector<int> rem_dims(nprb_dims, 1);
@@ -311,17 +321,26 @@ private:
 
 void block_helper_t::compute() {
     is_frozen_ = true;
-
     ir_assert(vectorize_by_b() || vectorize_by_n());
 
     init_bmnk_dims();
     init_bmnk_blocks();
     init_prb_blocks();
 
+    // XXX: Fix up loop dims for when K grid slicing is disabled.
+    if (!allow_k_grid_slicing_) {
+        for (auto &kv : dims_) {
+            auto &d = kv.second;
+            if (d.bmnk() != 'K') continue;
+            d.set_loop_dim(1);
+            d.set_loop_dim(d.grid_dim());
+        }
+    }
+
 #ifdef GEN_CONV_DEBUG
     for (auto &kv : dims_) {
         auto &d = kv.second;
-        const char *tags[] = {"iter", "thr", "tg"};
+        const char *tags[] = {"iter", "tg", "loop"};
         for (int i = min_tile_level_idx; i <= max_tile_level_idx; i++) {
             auto level = (tile_level_t)i;
             std::string env_name
@@ -352,13 +371,6 @@ void block_helper_t::compute() {
         }
         ir_assert(iter_blk % bmnk_dim(bmnk).base_iter_block() == 0);
     }
-
-    // Initialize padded dim and iter block sizes.
-    for (auto &kv : dims_) {
-        auto &d = kv.second;
-        padded_dim_sizes_.emplace(d.name(), d.padded_size());
-        iter_blocks_.emplace(d.name(), d.iter_dim());
-    }
 }
 
 void block_helper_t::init_bmnk_blocks() {
@@ -386,7 +398,7 @@ void block_helper_t::init_bmnk_blocks() {
         case fma_kind_t::mad: {
             int max_m_iter_dim = prb_max_dim('M', tile_level_t::iter);
             m_inst_blk = std::min(8, utils::rnd_down_pow2(max_m_iter_dim));
-            bn_inst_blk = hw_cfg_.vec_size();
+            bn_inst_blk = vec_size_;
             k_inst_blk = 1;
             bool use_small_m_block = hw_cfg_.hw() <= ngen::HW::XeHP
                     && m_dim().base_iter_block() == 1;
@@ -396,7 +408,7 @@ void block_helper_t::init_bmnk_blocks() {
                     && !m_dim().pref_tg_block();
             if (!m_dim().pref_tg_block())
                 m_dim().set_max_dim(tile_level_t::tg, small_m_tg ? 1 : 4);
-            bn_blk = hw_cfg_.vec_size();
+            bn_blk = vec_size_;
             k_blk = compute_mad_k_block();
             if (!allow_k_grid_slicing_ && !allow_k_tg_slicing_) {
                 do {
@@ -417,7 +429,9 @@ void block_helper_t::init_bmnk_blocks() {
             ir_assert(vectorize_by_n())
                     << "dpas can support N vectorization only.";
             int max_iter_dim = prb_max_dim('M', tile_level_t::iter);
-            int target_m_blk = reduce_m_block ? 16 : 32;
+            int target_m_blk = reduce_m_block ? 16
+                    : expand_m_block_hint_    ? 64
+                                              : 32;
             if (max_iter_dim % target_m_blk != 0 && max_iter_dim > 32) {
                 float max_utilization_rate = 0.;
                 for (int i
@@ -519,7 +533,7 @@ void block_helper_t::init_bmnk_blocks() {
     // Set thread group blocks.
     bool with_k_tg_slicing = (k_dim().tg_dim() > 1);
     if (!with_k_tg_slicing) {
-        int target_tg_size = hw_cfg_.max_tg_size();
+        int target_tg_size = max_tg_size_;
         int est_threads = 1;
         for (char bmnk : {'B', 'M', 'N'})
             est_threads *= bmnk_dim(bmnk).grid_dim();
@@ -528,29 +542,33 @@ void block_helper_t::init_bmnk_blocks() {
             target_tg_size = std::min(target_tg_size, 16);
         }
 
+        auto init_tg_dim = [&](dim_info_t &d) -> int {
+            int i_max_tg_dim = min(target_tg_size, d.max_dim(tile_level_t::tg));
+            int target_blk = i_max_tg_dim * d.iter_dim();
+            int base_tg_dim = compute_block(d.size(), target_blk, d.iter_dim())
+                    / d.iter_dim();
+            //restrict maximum single tg dim as max_tg size is reduced
+            return std::min(utils::rnd_down_pow2(base_tg_dim),
+                    max_tg_overridden_ ? target_tg_size
+                                    / (hw_cfg_.hw() >= ngen::HW::XeHPC ? 2 : 4)
+                                       : simd_size_);
+        };
+
         // Compute max thread group blocks, independently for each dimension.
         std::vector<char> tg_bmnks = {vectorize_by_b() ? 'B' : 'N', 'M'};
         std::vector<int> tg_dims(tg_bmnks.size(), 1);
-        bool any_pref_dim = false;
-        int *split_dim_idx;
+        bool any_pref_dim = any_pref_tg_block();
+        int *split_dim_idx, *pref_dim_idx;
+        char split_dim_bmnk;
         for (size_t i = 0; i < tg_bmnks.size(); i++) {
             auto &d = bmnk_dim(tg_bmnks[i]);
-            int i_max_tg_dim = min(target_tg_size, d.max_dim(tile_level_t::tg));
-            int target_blk = i_max_tg_dim * d.iter_dim();
-            int tg_dim = compute_block(d.size(), target_blk, d.iter_dim())
-                    / d.iter_dim();
-            //restrict maximum single tg dim as max_tg size is reduced
-            tg_dim = std::min(utils::rnd_down_pow2(tg_dim),
-                    hw_cfg_.max_tg_overriden() ? target_tg_size
-                                    / (hw_cfg_.hw() >= ngen::HW::XeHPC ? 2 : 4)
-                                               : hw_cfg_.simd_size());
+            int tg_dim = init_tg_dim(d);
             tg_dims[i] = tg_dim;
-            if (d.pref_tg_block()) {
-                //only one preferred dim allowed
-                assert(!any_pref_dim);
-                any_pref_dim = true;
-            } else {
+            if (!d.pref_tg_block()) {
                 split_dim_idx = &tg_dims.at(i);
+                split_dim_bmnk = tg_bmnks[i];
+            } else {
+                pref_dim_idx = &tg_dims.at(i);
             }
         }
 
@@ -559,7 +577,20 @@ void block_helper_t::init_bmnk_blocks() {
                     tg_dims.begin(), tg_dims.end(), 1, std::multiplies<int>());
         };
         auto max_tg_dim = [&]() -> int & {
-            if (any_pref_dim && *split_dim_idx > 1) return *split_dim_idx;
+            if (any_pref_dim) {
+                auto split_dim = bmnk_dim(split_dim_bmnk);
+                int split_rem_grid = split_dim.size() / split_dim.iter_dim();
+                // Special case preserve non-zero tg dim for non-pref bmnk
+                bool preserve_min_dim
+                        = (split_rem_grid > 1 && *split_dim_idx < 4
+                                && *pref_dim_idx > *split_dim_idx);
+                if (*split_dim_idx == 1
+                        || (preserve_min_dim
+                                && (*pref_dim_idx > *split_dim_idx)))
+                    return *pref_dim_idx;
+                else
+                    return *split_dim_idx;
+            }
             return *std::max_element(tg_dims.begin(), tg_dims.end());
         };
 
@@ -579,7 +610,7 @@ void block_helper_t::init_k_blocking() {
     // Thread and thread group dims must not be set yet.
     for (char bmnk : {'B', 'M', 'N', 'K'}) {
         auto &d = bmnk_dim(bmnk);
-        ir_assert(d.thr_dim() == 1);
+        ir_assert(d.loop_dim() == 1);
         ir_assert(d.tg_dim() == 1);
     }
 
@@ -587,35 +618,35 @@ void block_helper_t::init_k_blocking() {
         int est_threads = 1;
         for (char bmnk : {'B', 'M', 'N', 'K'})
             est_threads *= bmnk_dim(bmnk).grid_dim();
-        int def_k_thr_dim = utils::div_up(est_threads, 2 * hw_cfg_.eu_count());
-        def_k_thr_dim = std::min(100, def_k_thr_dim);
-        def_k_thr_dim = std::max(1, def_k_thr_dim);
-        int k_thr_dim = def_k_thr_dim;
+        int def_k_loop_dim = utils::div_up(est_threads, 2 * hw_cfg_.eu_count());
+        def_k_loop_dim = std::min(100, def_k_loop_dim);
+        def_k_loop_dim = std::max(1, def_k_loop_dim);
+        int k_loop_dim = def_k_loop_dim;
 #ifdef GEN_CONV_DEBUG
-        k_thr_dim = getenv_int("k_thr_dim", k_thr_dim);
+        k_loop_dim = getenv_int("k_loop_dim", k_loop_dim);
 #endif
-        k_dim().set_thr_dim(k_thr_dim);
+        k_dim().set_loop_dim(k_loop_dim);
         return;
     }
 
     if (!enable_k_tg_slicing()) {
-        k_dim().set_thr_dim(dim_value_t::unlimited());
+        k_dim().set_loop_dim(dim_value_t::unlimited());
         return;
     }
 
     int k_nblks = utils::div_up(
             prb_blocked_dim('K').size(), k_dim().base_iter_block());
-    int tg_dim0 = min(hw_cfg_.max_tg_size(), k_dim().max_dim(tile_level_t::tg));
+    int tg_dim0 = min(max_tg_size_, k_dim().max_dim(tile_level_t::tg));
     for (int tg_dim = tg_dim0; tg_dim >= 1; tg_dim /= 2) {
         if (k_nblks % tg_dim == 0) {
-            k_dim().set_thr_dim(k_nblks / tg_dim);
+            k_dim().set_loop_dim(k_nblks / tg_dim);
             k_dim().set_tg_dim(tg_dim);
             return;
         }
     }
 
     // Couldn't enable TG slicing.
-    k_dim().set_thr_dim(dim_value_t::unlimited());
+    k_dim().set_loop_dim(dim_value_t::unlimited());
 }
 
 bool block_helper_t::enable_k_tg_slicing() const {
