@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022 Intel Corporation
+* Copyright 2022-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -30,11 +30,11 @@
 #include "common/math_utils.hpp"
 #include "common/utils.hpp"
 #include "gpu/compute/device_info.hpp"
-#include "gpu/jit/conv/fma_support.hpp"
-#include "gpu/jit/conv/hw_config.hpp"
-#include "gpu/jit/conv/ir_core.hpp"
-#include "gpu/jit/conv/utils.hpp"
+#include "gpu/jit/ir/core.hpp"
+#include "gpu/jit/ir/fma.hpp"
+#include "gpu/jit/ir/hw_config.hpp"
 #include "gpu/jit/ngen/ngen.hpp"
+#include "gpu/jit/utils/utils.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -45,14 +45,14 @@ namespace jit {
 enum class tile_level_t {
     unknown,
     iter, // Number of elements per iteration.
-    thr, // Number of iterations per thread.
     tg, // Number of threads per thread group.
+    loop, // Number of iterations per loop.
     _last,
 };
 
 // Min/max integer indices of tile levels.
 const int min_tile_level_idx = (int)tile_level_t::iter;
-const int max_tile_level_idx = (int)tile_level_t::tg;
+const int max_tile_level_idx = (int)tile_level_t::loop;
 
 // Describes the dimension value, contains either an integer or a special
 // "unlimited" value which behaves like infinity when mixing in operations with
@@ -167,7 +167,7 @@ public:
         return ir_utils::safe_divide(padded_size(), tg_blk());
     }
     dim_value_t tg_dim() const { return dim(tile_level_t::tg); }
-    dim_value_t thr_dim() const { return dim(tile_level_t::thr); }
+    dim_value_t loop_dim() const { return dim(tile_level_t::loop); }
     dim_value_t iter_dim() const { return dim(tile_level_t::iter); }
     dim_value_t dim(tile_level_t level) const {
         int idx = (int)level;
@@ -182,7 +182,7 @@ public:
     bool pref_tg_block() { return pref_tg_block_; }
     void set_pref_tg_block(bool value = true) { pref_tg_block_ = value; }
     void set_tg_dim(dim_value_t value) { set_dim(tile_level_t::tg, value); }
-    void set_thr_dim(dim_value_t value) { set_dim(tile_level_t::thr, value); }
+    void set_loop_dim(dim_value_t value) { set_dim(tile_level_t::loop, value); }
     void set_iter_dim(dim_value_t value) { set_dim(tile_level_t::iter, value); }
     void set_dim(tile_level_t level, dim_value_t value) {
         int idx = (int)level;
@@ -196,13 +196,16 @@ public:
         max_tile_dims_[idx] = value;
     }
 
+    int inner_dims() const { return inner_dims_; }
+    void incr_inner_dims() { inner_dims_++; }
+
     int iter_blk() const { return iter_dim(); }
-    int thr_blk() const { return thr_dim() * iter_blk(); }
-    int tg_blk() const { return tg_dim() * thr_blk(); }
+    int loop_blk() const { return loop_dim() * iter_blk(); }
+    int tg_blk() const { return tg_dim() * loop_blk(); }
 
     bool has_any_blocking() const {
         if (iter_dim() != 1) return true;
-        if (thr_dim() != 1) return true;
+        if (loop_dim() != 1) return true;
         if (tg_dim() != 1) return true;
         return false;
     }
@@ -219,7 +222,7 @@ public:
         oss << "  Allow split:       " << to_string(allow_split_) << std::endl;
         oss << "  Order key:         " << order_key_ << std::endl;
 
-        const char *tags[] = {"Iteration", "Thread", "Thread group", nullptr};
+        const char *tags[] = {"Iteration", "Thread group", "Loop", nullptr};
         int level_id = min_tile_level_idx;
         for (const char **tag = tags; *tag; tag++) {
             tile_level_t level = (tile_level_t)level_id++;
@@ -238,8 +241,8 @@ public:
         std::ostringstream oss;
         oss << "Dimension " << name_ << pad_str(":", -18 + (int)name_.length());
         oss << "(grid:" << pad_int(grid_dim(), 5) << ") x ";
+        oss << "(loop:" << pad_int(loop_dim(), 5) << ") x ";
         oss << "(tg:" << pad_int(tg_dim(), 5) << ") x ";
-        oss << "(thr:" << pad_int(thr_dim(), 5) << ") x ";
         oss << "(iter:" << pad_int(iter_dim(), 5) << ")";
         return oss.str();
     }
@@ -274,6 +277,9 @@ private:
     // Dimension kind in terms of BMNK notation.
     char bmnk_ = ' ';
 
+    // Number of prb dims for a BMNK dim
+    int inner_dims_ = 0;
+
     // Whether the dimension can be blocked. Dimensions without blocking are
     // implicitly tiled on the grid level (handle one element per thread
     // group).
@@ -309,10 +315,16 @@ private:
 //   - Maximal block sizes (e.g. to limit GRF usage)
 //   - Fuse/split settings - to get the desired blocking decomposition
 //   - K-slicing settings (grid or thread group sliciing)
-// - Computing block sizes for each tile: per iteration, per thread, per thread
+// - Computing block sizes for each tile: per iteration, per loop, per thread
 //   group
 class block_helper_t {
 public:
+    bool is_frozen() const { return is_frozen_; }
+
+    const std::unordered_map<std::string, dim_info_t> &dims() const {
+        return dims_;
+    }
+
     dim_info_t &dim(const std::string &name) {
         ir_assert(dims_.count(name) != 0) << "Dimension not found: " << name;
         return dims_.at(name);
@@ -328,6 +340,26 @@ public:
         fma_kind_ = fma_kind;
     }
 
+    void set_simd_size(int simd_size) {
+        check_if_can_set();
+        simd_size_ = simd_size;
+    }
+
+    void set_vec_size(int vec_size) {
+        check_if_can_set();
+        vec_size_ = vec_size;
+    }
+
+    void set_max_tg_size(int max_tg_size) {
+        check_if_can_set();
+        max_tg_size_ = max_tg_size;
+    }
+
+    void set_max_tg_overridden(bool max_tg_overridden) {
+        check_if_can_set();
+        max_tg_overridden_ = max_tg_overridden;
+    }
+
     void set_abc_types(
             data_type_t a_type, data_type_t b_type, data_type_t c_type) {
         check_if_can_set();
@@ -336,6 +368,11 @@ public:
         c_type_ = c_type;
     }
 
+    void set_use_2d_send(bool use_a_2d_send, bool use_b_2d_send) {
+        use_a_2d_send_ = use_a_2d_send;
+        use_b_2d_send_ = use_b_2d_send;
+    }
+
     void set_max_m_tg_dim(int value) {
         m_dim().set_max_dim(tile_level_t::tg, value);
     }
@@ -384,8 +421,8 @@ public:
         }
     }
 
-    void set_thr_dim(const std::string &name, int value) {
-        dim(name).set_thr_dim(value);
+    void set_loop_dim(const std::string &name, int value) {
+        dim(name).set_loop_dim(value);
     }
 
     void set_tg_dim(const std::string &name, int value) {
@@ -396,13 +433,14 @@ public:
         dim(name).set_max_dim(tile_level_t::iter, value);
     }
 
-    void set_max_thr_dim(const std::string &name, int value) {
-        dim(name).set_max_dim(tile_level_t::thr, value);
+    void set_max_loop_dim(const std::string &name, int value) {
+        dim(name).set_max_dim(tile_level_t::loop, value);
     }
 
     void set_max_tg_dim(const std::string &name, int value) {
         dim(name).set_max_dim(tile_level_t::tg, value);
     }
+
     void set_pref_tg_block(const std::string &name, bool value = true) {
         dim(name).set_pref_tg_block(value);
     }
@@ -415,6 +453,14 @@ public:
         return false;
     }
 
+    void set_expand_m_block_hint() { expand_m_block_hint_ = true; }
+    bool expand_m_block_hint() { return expand_m_block_hint_; }
+
+    void set_reduce_m_block_hint(bool value = true) {
+        reduce_m_block_hint_ = value;
+        reduce_m_block_hint_set_ = true;
+    }
+
     void allow_fuse(std::initializer_list<std::string> names) {
         check_if_can_set();
         for (auto &name : names) {
@@ -436,8 +482,7 @@ public:
     void set_vector_dim(const std::string &name) {
         check_if_can_set();
         auto &d = dim(name);
-        d.set_base_iter_block(
-                math::lcm(hw_cfg_.vec_size(), d.base_iter_block()));
+        d.set_base_iter_block(math::lcm(vec_size_, d.base_iter_block()));
         vector_bmnk_ = d.bmnk();
     }
 
@@ -463,13 +508,17 @@ public:
 
     void compute();
 
+    bool has_dim(const std::string &name) const {
+        return dims_.count(name) != 0;
+    }
+
     int iter_dim(const std::string &name) const { return dim(name).iter_dim(); }
-    int thr_dim(const std::string &name) const { return dim(name).thr_dim(); }
+    int loop_dim(const std::string &name) const { return dim(name).loop_dim(); }
     int tg_dim(const std::string &name) const { return dim(name).tg_dim(); }
     int grid_dim(const std::string &name) const { return dim(name).grid_dim(); }
 
     int iter_blk(const std::string &name) const { return dim(name).iter_blk(); }
-    int thr_blk(const std::string &name) const { return dim(name).thr_blk(); }
+    int loop_blk(const std::string &name) const { return dim(name).loop_blk(); }
     int tg_blk(const std::string &name) const { return dim(name).tg_blk(); }
 
     dim_value_t max_iter_dim(const std::string &name) const {
@@ -480,14 +529,6 @@ public:
         return dim(name).padded_size();
     }
 
-    const std::unordered_map<std::string, int> &padded_dim_sizes() const {
-        return padded_dim_sizes_;
-    }
-
-    const std::unordered_map<std::string, int> &iter_dims() const {
-        return iter_blocks_;
-    }
-
     std::string str() const {
         std::ostringstream oss;
         for (auto &kv : dims_) {
@@ -592,6 +633,10 @@ private:
         if (c_type_ != data_type::s32) return false;
         return true;
     }
+    bool is_tf32() const {
+        return a_type_ == data_type::tf32 && b_type_ == data_type::tf32
+                && c_type_ == data_type::f32;
+    }
 
     bool vectorize_by_b() const { return vectorize_by_bmnk('B'); }
     bool vectorize_by_m() const { return vectorize_by_bmnk('M'); }
@@ -620,6 +665,7 @@ private:
             bmnk_d.set_size(bmnk_d.size() * d.size());
             bmnk_d.set_base_iter_block(
                     bmnk_d.base_iter_block() * d.base_iter_block());
+            if (d.is_blocked() && d.size() != 1) bmnk_d.incr_inner_dims();
         }
     }
 
@@ -650,10 +696,17 @@ private:
     // General information about HW and computation.
     hw_config_t hw_cfg_;
     fma_kind_t fma_kind_ = fma_kind_t::unknown;
+    int vec_size_ = -1;
+    int simd_size_ = -1;
+    int max_tg_size_ = 0;
+    bool max_tg_overridden_ = false;
     data_type_t a_type_ = data_type::undef;
     data_type_t b_type_ = data_type::undef;
     data_type_t c_type_ = data_type::undef;
 
+    bool use_a_2d_send_ = false;
+    bool use_b_2d_send_ = false;
+
     // Whether K computation can be split across threads in thread group.
     bool allow_k_tg_slicing_ = false;
 
@@ -663,15 +716,14 @@ private:
     // Problem dimensions.
     std::unordered_map<std::string, dim_info_t> dims_;
 
-    // Padded dimension sizes according to block sizes and padding blocks.
-    std::unordered_map<std::string, int> padded_dim_sizes_;
-
-    // Iteration block sizes.
-    std::unordered_map<std::string, int> iter_blocks_;
-
     // BMNK dimensions.
     static const int bmnk_length = 4;
     dim_info_t bmnk_dims_[bmnk_length];
+
+    bool expand_m_block_hint_ = false;
+
+    bool reduce_m_block_hint_;
+    bool reduce_m_block_hint_set_ = false;
 };
 
 } // namespace jit
