@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022 Intel Corporation
+* Copyright 2022-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,10 +16,11 @@
 
 #include "gpu/jit/conv/epilogue.hpp"
 
-#include "gpu/jit/conv/builder_utils.hpp"
-#include "gpu/jit/conv/message_support.hpp"
-#include "gpu/jit/conv/reduce_support.hpp"
-#include "gpu/jit/conv/reorder_support.hpp"
+#include "gpu/jit/ir/message.hpp"
+#include "gpu/jit/ir/mul_add.hpp"
+#include "gpu/jit/ir/reduce.hpp"
+#include "gpu/jit/ir/reorder.hpp"
+#include "gpu/jit/utils/trace.hpp"
 
 namespace dnnl {
 namespace impl {
@@ -53,8 +54,8 @@ public:
     stmt_t build_stmt(const layout_t &reg_layout, const expr_t &reg_buf) const {
         ir_assert(mem_view_.nvdims() == reg_layout.ndims())
                 << "Incompatible view/layout.";
-        int max_step = std::min(16,
-                2 * ir_ctx_->hw_cfg().grf_size() / reg_layout.type().size());
+        int max_step = std::min(
+                16, 2 * ir_ctx_->grf_size() / reg_layout.type().size());
         auto base_tile = reg_layout.split_into_max_tile(
                 max_step, /*is_dense_tile=*/true);
         stmt_t stmt;
@@ -292,6 +293,30 @@ public:
         return ret;
     }
 
+    stmt_t build_post_load_op_stmt() {
+        switch (info_.post_load_op()) {
+            case post_load_op_kind_t::inv: {
+                int elems = reg_layout_.elems();
+                auto &type = reg_layout_.type();
+                stmt_t ret;
+                int max_step = 2 * ir_ctx_->grf_size() / type.size();
+                for (int i = 0; i < elems;) {
+                    int step = std::min(max_step, elems - i);
+                    int off = i * type.size();
+                    auto l = load_t::make(type.with_elems(step), reg_buf_, off);
+                    auto s = store_t::make(reg_buf_, off,
+                            shuffle_t::make_broadcast(1.0f, step) / l);
+                    ret = ret.append(s);
+                    i += step;
+                }
+                return ret;
+            }
+            case post_load_op_kind_t::none: return stmt_t();
+            default: ir_error_not_expected();
+        }
+        return stmt_t();
+    }
+
     stmt_t build_zero_out_stmt() const {
         ir_assert(needs_store());
         return create_zero_out_stmt(*ir_ctx_, reg_buf_, reg_layout_.size());
@@ -638,7 +663,7 @@ public:
         restore_zero_padding_ = post_op_ctx_.need_to_restore_zero_padding();
 
         for (auto &po : post_op_ctx_.post_ops()) {
-            post_op_builders_.emplace_back(cfg.hw(), po);
+            post_op_builders_.emplace_back(ir_ctx_.hw(), po);
         }
 
         // Estimate buffer sizes required to load the full tensor, do not do
@@ -668,9 +693,9 @@ private:
 
     // Represents a GRF buffer and layout to store C tensor.
     struct c_stage_t {
-        c_stage_t(const layout_t &layout, const expr_t &buf,
+        c_stage_t(const layout_t &layout, int buf_size, const expr_t &buf,
                 const stmt_t &stmt = stmt_t())
-            : layout(layout), buf(buf), stmt(stmt) {}
+            : layout(layout), buf_size(buf_size), buf(buf), stmt(stmt) {}
 
         void set_next(
                 ir_context_t &ir_ctx, c_stage_t *next, bool force_reorder) {
@@ -704,10 +729,10 @@ private:
             return buf.as<ptr_t>().base;
         }
 
-        int buf_size() const {
+        int get_buf_size() const {
             ir_assert(buf.is_same(buf_base()))
                     << "Size must be queried from another stage.";
-            return int(layout.size());
+            return (buf_size == 0) ? int(layout.size()) : buf_size;
         }
 
         void prepend_stmt(const stmt_t &stmt) {
@@ -715,6 +740,7 @@ private:
         }
 
         layout_t layout;
+        int buf_size;
         expr_t buf;
         stmt_t stmt; // Statement to emit after the stage.
     };
@@ -733,7 +759,7 @@ private:
         }
 
         // Generate prefetch statements.
-        if (cfg_.is_ge_xe_hpc()) {
+        if (ir_ctx_.hw() >= ngen::HW::XeHPC) {
             for (auto &t : post_op_tensors_) {
                 if (!t.needs_load()) continue;
                 if (t.do_preload()) continue;
@@ -748,6 +774,11 @@ private:
             stmt_ = stmt_.append(t.build_convert_stmt());
         }
 
+        // Generate post-load statements.
+        for (auto &t : post_op_tensors_) {
+            stmt_ = stmt_.append(t.build_post_load_op_stmt());
+        }
+
         // Initialize buffers for output post-op tensors.
         for (auto &t : post_op_tensors_) {
             if (!t.needs_store()) continue;
@@ -830,12 +861,14 @@ private:
         }
 
         // S_y -> GMEM.
-        auto send_op = cfg_.do_atomic_update ? send_op_t::atomic_fadd
-                                             : send_op_t::store;
-        auto send_hint = get_send_hint(cfg_.hw_cfg, send_op, abc_kind_t::c,
+        auto send_op = gemm_schedule_.with_kernel_grid_k_slicing()
+                ? send_op_t::atomic_fadd
+                : send_op_t::store;
+        auto send_params = get_send_params(ir_ctx_.exec_cfg(), send_op,
+                send_address_t::a64, fma_kind_t::unknown, abc_kind_t::c,
                 c_mem_tile_view, gemm_schedule_);
-        auto r2g = make_access_builder(ir_ctx_, c_mem_tile_view, c_mem_buf_,
-                tmp_reg_buf, send_op, send_address_t::a64, send_hint);
+        auto r2g = make_access_builder(
+                ir_ctx_, c_mem_tile_view, c_mem_buf_, tmp_reg_buf, send_params);
 
         // Initialize C stages.
         std::vector<c_stage_t> c_stages;
@@ -847,19 +880,20 @@ private:
         int c_f32_stage_idx = -1;
         int c_zero_pad_stage_idx = -1;
 
-        c_stages.emplace_back(c_tile_layout, c_reg_buf); // M_x
+        c_stages.emplace_back(c_tile_layout, 0, c_reg_buf); // M_x
         if (with_post_ops) {
             c_f32_stage_idx = int(c_stages.size());
-            c_stages.emplace_back(c_fx_layout, make_c_tmp_buffer()); // R_f32
+            c_stages.emplace_back(c_fx_layout, 0, make_c_tmp_buffer()); // R_f32
         }
         if (restore_zero_padding_) {
             c_zero_pad_stage_idx = int(c_stages.size());
-            c_stages.emplace_back(c_fx_layout, make_c_tmp_buffer()); // Z_f32
+            c_stages.emplace_back(c_fx_layout, 0, make_c_tmp_buffer()); // Z_f32
         }
-        c_stages.emplace_back(r2g.reg_layout(), tmp_reg_buf, r2g.stmt()); // S_y
+        c_stages.emplace_back(r2g.reg_layout(), r2g.reg_buf_size(), tmp_reg_buf,
+                r2g.stmt()); // S_y
 
         int nstages = int(c_stages.size());
-        bool is_dpasw = (cfg_.fma_kind == fma_kind_t::dpasw);
+        bool is_dpasw = (cfg_.fma_kind() == fma_kind_t::dpasw);
 
         // Generate reorders between C stages if needed.
         for (int i = 0; i < nstages; i++) {
@@ -912,7 +946,7 @@ private:
             auto &buf = s.buf_base();
             auto ret = seen.insert(buf);
             if (i == 0 || !ret.second) continue;
-            int size = utils::rnd_up(s.buf_size(), cfg_.hw_cfg.grf_size());
+            int size = utils::rnd_up(s.get_buf_size(), ir_ctx_.grf_size());
             tile_stmt = alloc_t::make(buf, size, alloc_kind_t::grf, tile_stmt);
         }
 
@@ -1007,11 +1041,11 @@ private:
 };
 
 int get_post_op_mem_usage(const post_op_tensor_info_t &info, int c_elems,
-        int max_elems_per_dim = 64) {
+        const view_t &c_mem_view, int max_elems_per_dim = 64) {
     int po_elems = 1;
     for (int i = 0; i < info.view().nvdims(); i++) {
         if ((info.mask() & (1 << i)) == 0) continue;
-        po_elems *= std::min(max_elems_per_dim, (int)info.view().vdims()[i]);
+        po_elems *= std::min(max_elems_per_dim, (int)c_mem_view.vdims()[i]);
     }
     po_elems = std::min(po_elems, c_elems);
     int type_size = info.view().type().size();
@@ -1020,7 +1054,7 @@ int get_post_op_mem_usage(const post_op_tensor_info_t &info, int c_elems,
     return load_size + cvt_size;
 }
 
-int find_tile_size(const hw_config_t &hw_cfg,
+int find_tile_size(const exec_config_t &exec_cfg,
         const post_op_context_t &post_op_ctx, const view_t &c_mem_view,
         const layout_t &c_reg_layout, int preload_max_size, int post_op_blk) {
     bool with_post_ops = !post_op_ctx.post_ops().empty();
@@ -1041,15 +1075,15 @@ int find_tile_size(const hw_config_t &hw_cfg,
             for (int j = i; j < std::min(npost_ops, i + post_op_blk); j++) {
                 auto &t = infos[j];
                 if (!t.is_input() || !t.buf().type().is_ptr()) continue;
-                po_batch_size += get_post_op_mem_usage(t, elems);
+                po_batch_size += get_post_op_mem_usage(t, elems, c_mem_view);
             }
             po_size = std::max(po_size, po_batch_size);
         }
 
-        int total_size = c_size + po_size;
-        int available_size
-                = hw_cfg.regs() * hw_cfg.grf_size() - (int)c_reg_layout.size();
-        if (total_size <= available_size * 0.7) return tile_size;
+        int total_size = c_size + preload_max_size + po_size;
+        int available_size = exec_cfg.regs() * exec_cfg.grf_size()
+                - (int)c_reg_layout.size();
+        if (total_size <= available_size * 0.8) return tile_size;
     }
     ir_error_not_expected();
     return -1;
@@ -1068,7 +1102,7 @@ stmt_t create_epilogue_stmt(const conv_config_t &cfg, ir_context_t &ir_ctx,
     int post_op_blk = 8;
     // Tile size in bytes. All post-ops are applied to a single tile, then to
     // the next tile, etc.
-    int tile_size = find_tile_size(cfg.hw_cfg, post_op_ctx, c_mem_view,
+    int tile_size = find_tile_size(cfg.exec_cfg(), post_op_ctx, c_mem_view,
             c_reg_layout, preload_max_size, post_op_blk);
 
     ir_trace() << "Creating epilogue with parameters"
