@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -16,6 +16,7 @@
 
 #include <cctype>
 
+#include "common/gemm_utils.hpp"
 #include "common/impl_registration.hpp"
 #include "gpu/compute/device_info.hpp"
 #include "gpu/jit/gemm/gen_gemm_kernel.hpp"
@@ -58,8 +59,20 @@ status_t gen_gemm_kernel_desc_t::finalize() {
     strategy_.unroll[LoopM] = entry_->driverInfo.unroll[LoopM];
     strategy_.unroll[LoopN] = entry_->driverInfo.unroll[LoopN];
     parseStrategy(entry_->strategy, hw_, problem_, strategy_);
+    if (isPacked(problem_.A.layout) || isPacked(problem_.B.layout))
+        strategy_.panelCheck |= (hw_ >= ngen::HW::XeHPC);
     adjustStrategy(hw_, problem_, strategy_);
 
+    // Disable global k parallelization if it wouldn't be used.
+    if (strategy_.kParallel && k_ >= 0) {
+        auto k_min = aux_params_.k0;
+        if (strategy_.kParallelLocal) k_min *= strategy_.wg[LoopK];
+        if (k_ <= k_min) {
+            strategy_.kParallel = false;
+            strategy_.C.atomic = false;
+        }
+    }
+
     // Always use variable beta for global k-parallel kernels.
     if (strategy_.kParallel) problem_.beta_real = Scalar<double>();
 
@@ -112,9 +125,72 @@ void gen_gemm_kernel_desc_t::update_driver_info() {
 #undef ARCH_DISPATCH
 }
 
+status_t gen_gemm_kernel_desc_t::transfer_post_ops(
+        const post_ops_t &post_ops, bool swap_ab) {
+    if (post_ops.len() > 0) {
+        problem_.postOps = post_ops;
+        int po_count = post_ops.len();
+        problem_.Tbinary.reserve(po_count);
+        problem_.binary.reserve(po_count);
+        problem_.binaryRow.reserve(po_count);
+        problem_.binaryCol.reserve(po_count);
+        problem_.binaryBatch.reserve(po_count);
+
+        if (problem_.Ta == Type::f16) problem_.Ts = Type::f32;
+
+        for (int i = 0; i < po_count; i++) {
+            const auto &entry = post_ops.entry_[i];
+            if (entry.kind != primitive_kind::binary) {
+                problem_.Tbinary.push_back(Type::invalid);
+                problem_.binaryRow.push_back(false);
+                problem_.binaryCol.push_back(false);
+                problem_.binaryBatch.push_back(false);
+                problem_.binary.push_back(MatrixAddressing {});
+                continue;
+            }
+
+            const auto &src_md = entry.binary.src1_desc;
+            memory_desc_wrapper src_mdw(src_md);
+
+            int ndims = src_mdw.ndims();
+            auto T = convert_dnnl_to_kernel_type(src_mdw.data_type());
+            int nr = (ndims >= 1) ? src_mdw.dims()[ndims - 1] : 1;
+            int nc = (ndims >= 2) ? src_mdw.dims()[ndims - 2] : 1;
+            bool trans = false;
+
+            if (src_mdw.ndims() >= 2) {
+                if (src_md.format_kind != format_kind::blocked
+                        || !is_md_gemm_compatible_plain_format(&src_md, false))
+                    return status::unimplemented;
+                trans = (src_md.format_desc.blocking.strides[ndims - 1] > 1);
+            }
+
+            if (swap_ab) {
+                trans = !trans;
+                std::swap(nr, nc);
+            }
+
+            problem_.Tbinary.push_back(T);
+            problem_.binaryRow.push_back(nr > 1);
+            problem_.binaryCol.push_back(nc > 1);
+            problem_.binaryBatch.push_back(ndims >= 3);
+
+            MatrixAddressing atype;
+            atype.layout = trans ? MatrixLayout::T : MatrixLayout::N;
+            atype.crosspack = 1;
+            atype.setAlignment(T.size());
+
+            problem_.binary.push_back(atype);
+        }
+    }
+
+    return status::success;
+}
+
 status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
         int stepping, int eu_count, compute_mode mode, int batch_dims,
-        bool trans_a, bool trans_b, bool ab_offset, bool c_offset, bool bias,
+        bool trans_a, bool trans_b, bool trans_co, bool swap_ab, bool a_offset,
+        bool b_offset, bool c_offset, bool bias, sum_ab_t reduce_ab,
         float alpha, float beta, const post_ops_t &post_ops, data_type_t a_type,
         data_type_t b_type, data_type_t c_type, data_type_t co_type,
         data_type_t acc_type, int align_a, int align_b, int align_c, dim_t m,
@@ -129,6 +205,8 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
     n_ = n;
     k_ = k;
     eu_count_ = eu_count;
+    a_offset_ = a_offset;
+    b_offset_ = b_offset;
 
     align_a = nstl::max(align_a, int(types::data_type_size(a_type)));
     align_b = nstl::max(align_b, int(types::data_type_size(b_type)));
@@ -153,24 +231,28 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
         problem_.batch = BatchMode::Strided;
         problem_.batchDims = batch_dims;
     }
-    if (ab_offset) problem_.abOffset = ABOffset::Calc;
+    if (a_offset || b_offset) problem_.abOffset = ABOffset::Calc;
 
     if (problem_.Ta.isInteger()) problem_.Ts = Type::f32;
 
     if (alpha == 1.0f) problem_.alpha_real = alpha;
     if (beta == 0.0f || beta == 1.0f) problem_.beta_real = beta;
 
-    if (post_ops.len() > 0) {
-        problem_.post_ops = post_ops;
-        if (a_type == data_type::f16) problem_.Ts = Type::f32;
-    }
-    if (c_offset || bias) {
+    auto status = transfer_post_ops(post_ops, swap_ab);
+    if (status != status::success) return status;
+
+    if (c_offset || bias || reduce_ab != sum_ab::sum_none) {
         assert(!(c_offset && bias));
-        problem_.cOffset = bias ? COffset::Pre : COffset::Post;
+        if (bias) problem_.cOffset = COffset::Pre;
+        if (c_offset) problem_.cOffset = COffset::Post;
         problem_.CO.crosspack = 1;
         problem_.CO.alignment = problem_.C.alignment;
+        problem_.CO.layout = trans_co ? MatrixLayout::T : MatrixLayout::N;
     }
 
+    problem_.sumA = (reduce_ab == sum_ab::sum_b_col);
+    problem_.sumB = (reduce_ab == sum_ab::sum_a_row);
+
     // Select a kernel from the catalog.
     MatchParams match_params[3];
     int npatterns = 1;
@@ -183,11 +265,12 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
     match_params[0].sizes.batch = batch;
     match_params[0].stepping = stepping;
 
-    std::string tags(match_params[0].tags);
-    if (lda * problem_.Ta >= 64) tags += kcatalog::ReqBlock2DA;
-    if (ldb * problem_.Tb >= 64) tags += kcatalog::ReqBlock2DB;
-    if (ldc * problem_.Tc >= 64) tags += kcatalog::ReqBlock2DC;
-    match_params[0].tags = tags.c_str();
+    auto tags = const_cast<char *>(match_params[0].tags);
+    while (*tags)
+        tags++;
+    if (lda * problem_.Ta >= 64) *tags++ = kcatalog::ReqBlock2DA;
+    if (ldb * problem_.Tb >= 64) *tags++ = kcatalog::ReqBlock2DB;
+    if (ldc * problem_.Tc >= 64) *tags++ = kcatalog::ReqBlock2DC;
 
     if ((mode & mode_tf32)
             && utils::everyone_is(Type::f32, problem_.Ta, problem_.Tb)) {
@@ -208,7 +291,7 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
     EvaluateParams eval_params;
 
     eval_params.sizes = match_params[0].sizes;
-    eval_params.beta = beta;
+    eval_params.beta = (post_ops.len() > 0) ? 0.0f : beta;
     eval_params.euCount = eu_count;
 
     entry_ = select(
@@ -236,21 +319,24 @@ status_t gen_gemm_nocopy_kernel_desc_t::select_kernel(compute::gpu_arch_t arch,
 }
 
 status_t gen_gemm_xe_systolic_kernel_desc_t::select_kernel(
-        compute::gpu_arch_t arch, int eu_count, int batch_dims, bool packed_c,
-        offset_t a_offset, offset_t b_offset, offset_t c_offset, offset_t bias,
-        float alpha, float beta, const post_ops_t &post_ops, data_type_t a_type,
-        data_type_t b_type, data_type_t c_type, data_type_t co_type,
-        data_type_t acc_type, dim_t m, dim_t n, dim_t k, dim_t batch,
-        int unroll_m, int unroll_n, bool alt) {
+        compute::gpu_arch_t arch, int stepping, int eu_count, int batch_dims,
+        bool packed_c, bool trans_co, bool a_offset, bool b_offset,
+        bool c_offset, bool bias, float alpha, float beta,
+        const post_ops_t &post_ops, data_type_t a_type, data_type_t b_type,
+        data_type_t c_type, data_type_t co_type, data_type_t acc_type, dim_t m,
+        dim_t n, dim_t k, dim_t batch, int unroll_m, int unroll_n, bool alt) {
     using namespace ngen;
     using namespace kcatalog;
 
     arch_ = arch;
     hw_ = convert_dnnl_arch_to_hw(arch);
+    stepping_ = stepping;
     m_ = m;
     n_ = n;
     k_ = k;
     eu_count_ = eu_count;
+    a_offset_ = a_offset;
+    b_offset_ = b_offset;
 
     if (!utils::one_of(hw_, HW::XeHP, HW::XeHPG, HW::XeHPC))
         return status::unimplemented;
@@ -288,26 +374,20 @@ status_t gen_gemm_xe_systolic_kernel_desc_t::select_kernel(
         problem_.batch = BatchMode::Strided;
         problem_.batchDims = batch_dims;
     }
-    if (a_offset == offset_t::fixed && b_offset == offset_t::fixed)
-        problem_.abOffset = ABOffset::Load;
-    else if (a_offset != offset_t::none || b_offset != offset_t::none)
-        return status::unimplemented;
+    if (a_offset || b_offset) problem_.abOffset = ABOffset::Load;
     if (alpha == 1.0f) problem_.alpha_real = alpha;
     if (beta == 0.0f || beta == 1.0f) problem_.beta_real = beta;
-    if (post_ops.len() > 0) {
-        problem_.post_ops = post_ops;
-        problem_.Ts = Type::f32;
-    }
-    if (c_offset == offset_t::runtime)
-        problem_.cOffset = COffset::Post;
-    else if (c_offset != offset_t::none)
-        return status::unimplemented;
 
-    if (bias == offset_t::runtime) {
+    auto status = transfer_post_ops(post_ops);
+    if (status != status::success) return status;
+
+    if (c_offset) problem_.cOffset = COffset::Post;
+
+    if (bias) {
         if (problem_.cOffset != COffset::None) return status::unimplemented;
         problem_.cOffset = COffset::Pre;
-    } else if (bias != offset_t::none)
-        return status::unimplemented;
+        problem_.CO.layout = trans_co ? MatrixLayout::T : MatrixLayout::N;
+    }
 
     if (problem_.cOffset != COffset::None) {
         problem_.CO.crosspack = 1;
@@ -325,7 +405,7 @@ status_t gen_gemm_xe_systolic_kernel_desc_t::select_kernel(
     match_params.unroll[LoopN] = unroll_n;
 
     const char alt_tag[2] = {kcatalog::ReqCustom1, '\0'};
-    if (alt) match_params.tags = &alt_tag[0];
+    if (alt) match_params.lateTags = match_params.tags = &alt_tag[0];
 
     EvaluateParams eval_params;
 
@@ -401,11 +481,31 @@ void gen_gemm_kernel_t::init_interface() {
     interface_.newArgument("k", DataType::d);
     interface_.newArgument("alpha_real", s_type_ngen);
     interface_.newArgument("beta_real", s_type_ngen);
-    if (problem.abOffset != ABOffset::None)
-        interface_.newArgument("abo", DataType::ud);
-    if (problem.cOffset != COffset::None) {
+    if (problem.abOffset != ABOffset::None) {
+        if (!desc()->a_offset_ && !desc()->b_offset_)
+            interface_.newArgument("abo", DataType::ud);
+        else {
+            if (desc()->a_offset_)
+                interface_.newArgument(
+                        "ao_ptr", ExternalArgumentType::GlobalPtr);
+            if (desc()->b_offset_)
+                interface_.newArgument(
+                        "bo_ptr", ExternalArgumentType::GlobalPtr);
+        }
+    }
+    if (problem.cOffset != COffset::None || problem.sumA || problem.sumB) {
         interface_.newArgument("CO", ExternalArgumentType::GlobalPtr);
         interface_.newArgument("offset_CO", DataType::d);
+        if (problem.cOffset == COffset::Pre)
+            interface_.newArgument("ldco", DataType::d);
+    }
+    for (int i = 0; i < problem.postOps.len(); i++) {
+        if (problem.postOps.entry_[i].kind != primitive_kind::binary) continue;
+        auto bname = "binary" + std::to_string(i);
+        interface_.newArgument(bname, ExternalArgumentType::GlobalPtr);
+        interface_.newArgument("offset_" + bname, DataType::d);
+        if (problem.binaryRow[i] && problem.binaryCol[i])
+            interface_.newArgument("ld" + bname, DataType::d);
     }
     interface_.newArgument("flags", DataType::ud);
     if (strategy.kParallel || strategy.kParallelLocal)
@@ -415,10 +515,20 @@ void gen_gemm_kernel_t::init_interface() {
             interface_.newArgument("stride_A1", DataType::d);
             interface_.newArgument("stride_B1", DataType::d);
             interface_.newArgument("stride_C1", DataType::d);
+            for (int i = 0; i < problem.postOps.len(); i++)
+                if (problem.postOps.entry_[i].kind == primitive_kind::binary
+                        && problem.binaryBatch[i])
+                    interface_.newArgument(
+                            "stride1_binary" + std::to_string(i), DataType::d);
         }
         interface_.newArgument("stride_A", DataType::d);
         interface_.newArgument("stride_B", DataType::d);
         interface_.newArgument("stride_C", DataType::d);
+        for (int i = 0; i < problem.postOps.len(); i++)
+            if (problem.postOps.entry_[i].kind == primitive_kind::binary
+                    && problem.binaryBatch[i])
+                interface_.newArgument(
+                        "stride_binary" + std::to_string(i), DataType::d);
         if (problem.batchDims > 1) {
             interface_.newArgument("batch_size1", DataType::ud);
             interface_.newArgument("recip_batch_size1", DataType::ud);
