@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2022 Intel Corporation
+* Copyright 2022-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -44,6 +44,15 @@ AccessType getAccessType(char c) {
     }
 }
 
+char downgradeBlock2D(char c) {
+    switch (std::tolower(c)) {
+        case 'm':
+        case 'v': return 'b';
+        case 't': return 's';
+        default: return c;
+    }
+}
+
 AddressBase getAddressBase(char c) {
     switch (c) {
         case 'a': return AddressBase::createA64(true);
@@ -96,16 +105,31 @@ void getCaching(std::stringstream &s, MatrixAddressingStrategy &astrategy) {
     }
 }
 
+static void getTiling(std::stringstream &s, bool doR, bool doC,
+        MatrixAddressingStrategy &astrategy) {
+    if (s.peek() == '#') {
+        char eat;
+        int in;
+        if (doR) s >> eat >> in, astrategy.tileR = in;
+        if (doC) s >> eat >> in, astrategy.tileC = in;
+    }
+}
+
 void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
         GEMMStrategy &strategy) {
     std::stringstream s(str);
     bool overrideFusedLoop = false;
+    bool gotSR = false;
 
     char eat, asA, asB, asC, accessA, accessB, accessC;
+    char accessAUnaligned = '\0', accessBUnaligned = '\0';
     char accessAPrefetch = 's', accessBPrefetch = 's', accessCPrefetch = 's';
 
-    s >> std::ws >> asA >> accessA >> strategy.ka_load;
+    s >> std::ws >> asA >> accessA;
+    if (s.peek() == '/') s >> eat >> accessAUnaligned;
+    s >> strategy.ka_load;
     if (s.peek() == '/') s >> eat >> strategy.ka_load_masked;
+    getTiling(s, true, false, strategy.A);
     if (s.peek() == 'x') s >> eat >> strategy.A_copies;
     getCaching(s, strategy.A);
     if (s.peek() == '+') {
@@ -119,8 +143,11 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
             strategy.prefetchAMasked = strategy.prefetchA;
         getCaching(s, strategy.A_prefetch);
     }
-    s >> std::ws >> asB >> accessB >> strategy.kb_load;
+    s >> std::ws >> asB >> accessB;
+    if (s.peek() == '/') s >> eat >> accessBUnaligned;
+    s >> strategy.kb_load;
     if (s.peek() == '/') s >> eat >> strategy.kb_load_masked;
+    getTiling(s, false, true, strategy.B);
     if (s.peek() == 'x') s >> eat >> strategy.B_copies;
     getCaching(s, strategy.B);
     if (s.peek() == '+') {
@@ -135,6 +162,7 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
         getCaching(s, strategy.B_prefetch);
     }
     s >> std::ws >> asC >> accessC;
+    getTiling(s, true, true, strategy.C);
     getCaching(s, strategy.C);
     if (s.peek() == '+') {
         strategy.prefetchC = 1;
@@ -143,6 +171,9 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
         getCaching(s, strategy.C_prefetch);
     }
 
+    if (!accessAUnaligned) accessAUnaligned = downgradeBlock2D(accessA);
+    if (!accessBUnaligned) accessBUnaligned = downgradeBlock2D(accessB);
+
     strategy.A.base = strategy.A_prefetch.base = getAddressBase(asA);
     strategy.B.base = strategy.B_prefetch.base = getAddressBase(asB);
     strategy.C.base = strategy.C_prefetch.base = getAddressBase(asC);
@@ -151,9 +182,12 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
     strategy.A.newDP = bool(std::isupper(accessA));
     strategy.B.newDP = bool(std::isupper(accessB));
     strategy.C.newDP = bool(std::isupper(accessC));
+    strategy.CO.newDP = strategy.C.newDP;
     strategy.A.accessType = getAccessType(accessA);
     strategy.B.accessType = getAccessType(accessB);
     strategy.C.accessType = getAccessType(accessC);
+    strategy.unalignedAccA = getAccessType(accessAUnaligned);
+    strategy.unalignedAccB = getAccessType(accessBUnaligned);
     strategy.A.cachingW = CacheSettingsLSC::Default;
     strategy.B.cachingW = CacheSettingsLSC::Default;
     strategy.A_prefetch.prefetch = true;
@@ -201,18 +235,31 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
             strategy.GRFs = std::stoi(mod);
         } else if (mod == "sys")
             strategy.systolic = true;
+        else if (mod == "dw")
+            strategy.dpasw = true;
         else if (mod == "fs")
             strategy.fixedSystolic = strategy.systolic = true;
         else if (mod == "ar")
             strategy.altCRemainder = true;
-        else if (mod == "sr")
+        else if (mod == "sr") {
             strategy.altCRemainder = false;
+            gotSR = true;
+        } else if (mod == "br")
+            strategy.block2DCRemainder = true;
+        else if (mod == "bf")
+            strategy.block2DCFull = strategy.block2DCRemainder = true;
         else if (mod == "ac")
             strategy.cAccumulators = true;
         else if (mod == "el")
             strategy.cLoadAhead = true;
         else if (mod == "di")
             strategy.delayABInc = true;
+        else if (mod == "bf")
+            strategy.loadBFirst = true;
+        else if (mod == "dm")
+            strategy.doubleMasking = true;
+        else if (mod == "kd")
+            strategy.kDescRem = true;
         else if (mod == "sc")
             strategy.splitCopy = true;
         else if (mod == "sm")
@@ -253,6 +300,8 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
             strategy.B.padded = strategy.B_prefetch.padded = true;
         } else if (mod == "pc")
             strategy.C.padded = strategy.C_prefetch.padded = true;
+        else if (mod == "up")
+            strategy.panelCheck = true;
         else if (mod == "mnk") {
             strategy.loopOrder[0] = LoopM;
             strategy.loopOrder[1] = LoopN;
@@ -271,26 +320,24 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
             strategy.reverse[LoopM] = true;
         else if (mod == "rn")
             strategy.reverse[LoopN] = true;
-        else if (mod == "ql")
-            strategy.skewLocalIDs = true;
         else if (mod == "kb") {
             strategy.kParallel = true;
             strategy.C.atomic = true;
+            strategy.CO.atomic = problem.sumA || problem.sumB;
+            if (strategy.CO.atomic)
+                strategy.CO.base = AddressBase::createA64(true);
         } else if (mod == "kr")
             strategy.kParallelLocal = true;
         else if (mod == "au")
             strategy.C.atomic = true;
-        else if (mod == "xp")
-            strategy.xParallel = true;
         else if (mod == "ff")
             strategy.forceWGUpdate = WGFixed;
         else if (mod == "wg") {
             char x;
             s >> strategy.wg[LoopM];
-            s >> std::ws >> x;
+            s >> x;
             s >> strategy.wg[LoopN];
             strategy.wg[LoopK] = 0;
-            s >> std::ws;
             if (s.peek() == 'x') s >> x >> strategy.wg[LoopK];
         } else if (mod == "nb") {
             char x;
@@ -307,6 +354,11 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
             strategy.A_prefetch.prefetch = false;
             strategy.B_prefetch.prefetch = false;
             strategy.C_prefetch.prefetch = false;
+        } else if (mod == "nq") {
+            strategy.A.noExtraPad = strategy.A_prefetch.noExtraPad = true;
+            strategy.B.noExtraPad = strategy.B_prefetch.noExtraPad = true;
+            strategy.C.noExtraPad = strategy.C_prefetch.noExtraPad = true;
+            strategy.CO.noExtraPad = true;
         } else if (mod.length() >= 2) {
             if (mod.substr(0, 2) == "ms")
                 strategy.mSplitThresh = stoi(mod.substr(2));
@@ -323,6 +375,8 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
             } else if (mod.substr(0, 2) == "sb") {
                 strategy.barrierFreq = stoi(mod.substr(2));
                 strategy.splitBarrier = true;
+            } else if (mod.substr(0, 2) == "ql") {
+                strategy.skewLocalIDs = true;
             } else
                 switch (mod[0]) {
                     case 'b':
@@ -394,6 +448,16 @@ void parseStrategy(const char *str, HW hw, const GEMMProblem &problem,
 
     if (strategy.ka_pfStride == 0) strategy.ka_pfStride = strategy.ka_prefetch;
     if (strategy.kb_pfStride == 0) strategy.kb_pfStride = strategy.kb_prefetch;
+
+    if (strategy.block2DCRemainder && !gotSR) strategy.altCRemainder = true;
+
+    int poCount = problem.postOps.len();
+    strategy.binary.resize(poCount);
+    for (auto &astrategy : strategy.binary) {
+        astrategy.base = (hw >= HW::XeHPC) ? AddressBase::createA64(true)
+                                           : AddressBase::createBTS(0);
+        astrategy.newDP = strategy.C.newDP;
+    }
 }
 
 void adjustStrategy(HW hw, const GEMMProblem &problem, GEMMStrategy &strategy) {
@@ -434,6 +498,20 @@ void adjustStrategy(HW hw, const GEMMProblem &problem, GEMMStrategy &strategy) {
     if (strategy.prefetchC && !isBlock2D(strategy.C_prefetch.accessType))
         strategy.remHandling[LoopM] = strategy.remHandling[LoopN]
                 = RemainderHandling::Split;
+
+    // ... and always use split remainder handling on later GPUs when panel checks are active.
+    if (strategy.panelCheck && strategy.lateExit() && hw >= HW::XeHP) {
+        if (isPacked(problem.A.layout))
+            strategy.remHandling[LoopM] = RemainderHandling::Split;
+        if (isPacked(problem.B.layout))
+            strategy.remHandling[LoopN] = RemainderHandling::Split;
+    }
+
+    // Finally, no remainder handling needed for GEMV-type kernels.
+    if (strategy.unroll[LoopM] * strategy.wg[LoopM] == 1)
+        strategy.remHandling[LoopM] = RemainderHandling::Ignore;
+    if (strategy.unroll[LoopN] * strategy.wg[LoopN] == 1)
+        strategy.remHandling[LoopN] = RemainderHandling::Ignore;
 }
 
 } // namespace jit
