@@ -22,7 +22,7 @@
 #include "common/impl_registration.hpp"
 #include "common/type_helpers.hpp"
 #include "gpu/jit/gemm/gemm_walk_orders.hpp"
-#include "gpu/jit/ngen_type_bridge.hpp"
+#include "gpu/jit/utils/ngen_type_bridge.hpp"
 #include "gpu/ocl/gemm/xe_systolic_gemm_copy_kernel.hpp"
 
 namespace dnnl {
@@ -57,21 +57,8 @@ status_t xe_hp_systolic_gemm_t::pd_t::init(engine_t *engine) {
             && utils::one_of(d->c_type(), s32, f32, s8, u8, f16));
 
     if (dt_int_ok) {
-        if (attr()->zero_points_.defined(DNNL_ARG_SRC)) {
-            const int *ao_i32 = nullptr;
-            attr()->zero_points_.get(DNNL_ARG_SRC, nullptr, nullptr, &ao_i32);
-            a_zp_ = (*ao_i32 != 0);
-        } else if (!attr()->zero_points_.has_default_values(DNNL_ARG_SRC))
-            return status::unimplemented;
-
-        if (attr()->zero_points_.defined(DNNL_ARG_WEIGHTS)) {
-            const int *bo_i32 = nullptr;
-            attr()->zero_points_.get(
-                    DNNL_ARG_WEIGHTS, nullptr, nullptr, &bo_i32);
-            b_zp_ = (*bo_i32 != 0);
-        } else if (!attr()->zero_points_.has_default_values(DNNL_ARG_WEIGHTS))
-            return status::unimplemented;
-
+        a_zp_ = !attr()->zero_points_.has_default_values(DNNL_ARG_SRC);
+        b_zp_ = !attr()->zero_points_.has_default_values(DNNL_ARG_WEIGHTS);
         c_zp_ = !attr()->zero_points_.has_default_values(DNNL_ARG_DST);
     }
 
@@ -96,49 +83,41 @@ status_t xe_hp_systolic_gemm_t::pd_t::init(engine_t *engine) {
     if (!packed_c())
         limits_ok = limits_ok && (d->ldc() != DNNL_RUNTIME_DIM_VAL);
 
-    auto attr_skip_mask = smask_t::oscale | smask_t::post_ops;
+    auto attr_skip_mask = smask_t::scales_runtime | smask_t::post_ops;
 
     if (dt_int_ok) attr_skip_mask |= smask_t::zero_points_runtime;
 
     bool arch_ok = utils::one_of(
             arch, arch_t::xe_hp, arch_t::xe_hpg, arch_t::xe_hpc);
 
-    ok = true && limits_ok && (dt_float_ok || dt_int_ok) && arch_ok
+    ok = ok && limits_ok && (dt_float_ok || dt_int_ok) && arch_ok
             && compute_engine->mayiuse(compute::device_ext_t::
                             intel_subgroup_split_matrix_multiply_accumulate)
             && attr()->has_default_values(attr_skip_mask)
-            && attr()->output_scales_.mask_ == 0 && attr()->post_ops_.len() <= 2
+            && desc()->sum_ab == sum_ab::sum_none
             && IMPLICATION(with_bias(),
-                    (dt_float_ok || dt_int_ok)
-                            && utils::one_of(d->bias_type(), d->a_type(), f32)
-                            && utils::one_of(bias_cmask(), 0, 1 << 0, 1 << 1));
+                    utils::one_of(d->bias_type(), d->a_type(), f32)
+                            && utils::one_of(bias_cmask(), 0, 1, 2, 3));
 
-    // check if there is sum post op and only at first place
-    ok &= IMPLICATION(attr()->post_ops_.find(sum) != -1,
-            attr()->post_ops_.find(sum) == 0
-                    && attr()->post_ops_.find(sum, 1) == -1);
-
-    // check if post ops are supported
-    ok &= IMPLICATION(attr()->post_ops_.len() > 0,
-            jit_post_op_injector_is_supported(attr()->post_ops_, true));
+    auto status = init_post_ops();
+    if (status != status::success) return status;
 
     if (dt_int_ok) {
-        ok &= IMPLICATION(a_zp_, !packed_b()) && IMPLICATION(b_zp_, !packed_a())
-                && IMPLICATION(
-                        c_zp_, !attr()->zero_points_.defined(DNNL_ARG_DST));
+        ok &= IMPLICATION(a_zp_, !packed_b())
+                && IMPLICATION(b_zp_, !packed_a());
 
         int cmask_a = 0, cmask_b = 0, cmask_c = 0;
-        attr()->zero_points_.get(DNNL_ARG_WEIGHTS, nullptr, &cmask_b, nullptr);
-        attr()->zero_points_.get(DNNL_ARG_SRC, nullptr, &cmask_a, nullptr);
-        attr()->zero_points_.get(DNNL_ARG_DST, nullptr, &cmask_c, nullptr);
+        CHECK(attr()->zero_points_.get(DNNL_ARG_WEIGHTS, &cmask_b));
+        CHECK(attr()->zero_points_.get(DNNL_ARG_SRC, &cmask_a));
+        CHECK(attr()->zero_points_.get(DNNL_ARG_DST, &cmask_c));
         ok &= (cmask_a == 0) && (cmask_b == 0)
                 && utils::one_of(cmask_c, 0, 1 << 0, 1 << 1);
-        ok &= IMPLICATION(utils::one_of(d->c_type(), f32, s8, u8, f16),
-                (attr()->post_ops_.len() == 0) && use_new_kernels());
     }
 
     if (!ok) return status::unimplemented;
 
+    init_scratchpad();
+
     return status::success;
 }
 
@@ -151,40 +130,36 @@ struct nocopy_table_t {
 };
 
 const nocopy_table_t xe_hp_f16_nocopy_table[] = {
-        // NN    NT     TN   TT
-        {{{1280, 768}, {512, 384}}, {{512, 768}, {1024, 512}}}};
-
-const nocopy_table_t xe_hp_bf16_nocopy_table[] = {
-        // NN   NT     TN   TT
-        {{{512, 256}, {768, 512}}, {{512, 256}, {384, 384}}}};
+        // NN    NT     TN    TT
+        {{{2880, 512}, {4096, 1024}}, {{0, 0}, {0, 0}}}};
 
 const nocopy_table_t xe_hp_x8x8s32_nocopy_table[] = {
+        // NN    NT     TN    TT
+        {{{1344, 576}, {4800, 384}}, {{0, 0}, {0, 0}}}};
+
+const nocopy_table_t xe_hp_f16_nocopy_bad_ld_table[] = {
         // NN   NT     TN   TT
-        {{{384, 384}, {384, 384}}, {{384, 512}, {384, 256}}}};
+        {{{288, 320}, {288, 288}}, {{288, 320}, {288, 288}}}};
+
+const nocopy_table_t xe_hp_x8x8s32_nocopy_bad_ld_table[] = {
+        // NN   NT     TN   TT
+        {{{656, 528}, {352, 384}}, {{656, 528}, {352, 384}}}};
 
 const nocopy_table_t xe_hpc_f16_nocopy_table[] = {
         // NN    NT      TN    TT
         {{{8192, 8192}, {8192, 8192}}, {{0, 0}, {0, 0}}}};
 
-const nocopy_table_t xe_hpc_bf16_nocopy_table[] = {
-        // NN    NT      TN    TT
-        {{{8192, 8192}, {8192, 8192}}, {{0, 0}, {0, 0}}}};
-
 const nocopy_table_t xe_hpc_x8x8s32_nocopy_table[] = {
         // NN    NT      TN    TT
-        {{{1024, 1024}, {1024, 1024}}, {{0, 0}, {0, 0}}}};
+        {{{2049, 3000}, {1088, 1024}}, {{0, 0}, {0, 0}}}};
 
 const nocopy_table_t xe_hpc_f16_nocopy_bad_ld_table[] = {
         // NN    NT      TN    TT
         {{{1024, 1024}, {1024, 1024}}, {{0, 0}, {0, 0}}}};
 
-const nocopy_table_t xe_hpc_bf16_nocopy_bad_ld_table[] = {
-        // NN    NT      TN    TT
-        {{{1024, 1024}, {1024, 1024}}, {{0, 0}, {0, 0}}}};
-
 const nocopy_table_t xe_hpc_x8x8s32_nocopy_bad_ld_table[] = {
-        // NN    NT      TN    TT
-        {{{1024, 1024}, {1024, 1024}}, {{0, 0}, {0, 0}}}};
+        // NN    NT    TN   TT
+        {{{624, 624}, {480, 624}}, {{0, 0}, {0, 0}}}};
 } // namespace
 
 bool xe_hp_systolic_gemm_t::pd_t::use_nocopy() {
@@ -195,8 +170,8 @@ bool xe_hp_systolic_gemm_t::pd_t::use_nocopy() {
 
     if (any_prepacked_ || (packed_a_ && packed_b_)) return false;
 
-    // Use no-copy for gemv cases.
-    if (d->m() <= 1 || d->n() <= 1) return true;
+    // Use no-copy for gemv/ger cases.
+    if (d->m() <= 1 || d->n() <= 1 || d->k() <= 1) return true;
 
     // Use no-copy implementation if one matrix is very small.
     if (d->m() < 32 && d->n() < 32) return true;
@@ -207,10 +182,10 @@ bool xe_hp_systolic_gemm_t::pd_t::use_nocopy() {
     if (utils::one_of(d->a_type(), bf16, f16, s8, u8)) {
         // clang-format off
         const nocopy_table_t *all_tables[2][2][3] = {
-            {{xe_hp_f16_nocopy_table, xe_hp_bf16_nocopy_table, xe_hp_x8x8s32_nocopy_table},
-             {xe_hp_f16_nocopy_table, xe_hp_bf16_nocopy_table, xe_hp_x8x8s32_nocopy_table}},
-            {{xe_hpc_f16_nocopy_table, xe_hpc_bf16_nocopy_table, xe_hpc_x8x8s32_nocopy_table},
-             {xe_hpc_f16_nocopy_bad_ld_table, xe_hpc_bf16_nocopy_bad_ld_table, xe_hpc_x8x8s32_nocopy_bad_ld_table}}
+            {{xe_hp_f16_nocopy_table, xe_hp_f16_nocopy_table, xe_hp_x8x8s32_nocopy_table},
+             {xe_hp_f16_nocopy_bad_ld_table, xe_hp_f16_nocopy_bad_ld_table, xe_hp_x8x8s32_nocopy_bad_ld_table}},
+            {{xe_hpc_f16_nocopy_table, xe_hpc_f16_nocopy_table, xe_hpc_x8x8s32_nocopy_table},
+             {xe_hpc_f16_nocopy_bad_ld_table, xe_hpc_f16_nocopy_bad_ld_table, xe_hpc_x8x8s32_nocopy_bad_ld_table}}
         };
         // clang-format on
         int type_idx = (d->a_type() == f16) ? 0 : (d->a_type() == bf16) ? 1 : 2;
@@ -308,9 +283,6 @@ bool xe_hp_systolic_gemm_t::pd_t::set_default_formats(data_type_t dt) {
     if (bc_prepacked_32) unroll_n_ = 32;
     if (bc_prepacked_48) unroll_n_ = 48;
 
-    use_new_kernels_ = !with_ab_zero_points() && (d->k() >= 64);
-    use_new_kernels_ |= (arch >= compute::gpu_arch_t::xe_hpc);
-
     new_kd_t::choose_unrolls(arch, dev_info_->eu_count(), d->a_type(),
             d->b_type(), d->c_type(), d->m(), d->n(), d->k(), d->batch(),
             unroll_m_, unroll_n_, alt_);
@@ -391,12 +363,42 @@ bool xe_hp_systolic_gemm_t::pd_t::set_default_formats(data_type_t dt) {
     if ((!packed_a_ && unroll_m_ == 16) || (!packed_b_ && unroll_n_ == 16))
         return false;
 
-    // Zero-padding bug in old kernel driver for packed cases.
-    use_new_kernels_ |= (packed_a_ || packed_b_);
-
     return gpu_gemm_pd_t::set_default_formats();
 }
 
+void xe_hp_systolic_gemm_t::pd_t::init_scratchpad() {
+    if (packed_a() && packed_b()) return;
+
+    auto a_type = desc()->a_type();
+    auto b_type = desc()->b_type();
+
+    auto m = desc()->m();
+    auto n = desc()->n();
+    auto k = desc()->k();
+
+    int64_t align_m = unroll_m_ * 8; // TODO: this should not be hardcoded
+    int64_t align_n = unroll_n_ * 8; // instead read from DriverInfo
+
+    auto m_aligned = utils::rnd_up(m, align_m);
+    auto n_aligned = utils::rnd_up(n, align_n);
+
+    auto max_ldab_packed = max_ld_packed(k);
+
+    auto scratchpad = scratchpad_registry().registrar();
+
+    if (!packed_a()) {
+        scratchpad.book(memory_tracking::names::key_gemm_blocked_a,
+                m_aligned * max_ldab_packed, types::data_type_size(a_type), 64,
+                65536);
+    }
+
+    if (!packed_b()) {
+        scratchpad.book(memory_tracking::names::key_gemm_blocked_b,
+                n_aligned * max_ldab_packed, types::data_type_size(b_type), 64,
+                65536);
+    }
+}
+
 status_t xe_hp_systolic_gemm_t::init(engine_t *engine) {
     arch_ = pd()->dev_info_->gpu_arch();
     eu_count_ = pd()->dev_info_->eu_count();
@@ -407,7 +409,7 @@ status_t xe_hp_systolic_gemm_t::init(engine_t *engine) {
     int cmask = -1;
 
     if (pd()->with_c_zero_points())
-        pd()->attr()->zero_points_.get(DNNL_ARG_DST, nullptr, &cmask, nullptr);
+        CHECK(pd()->attr()->zero_points_.get(DNNL_ARG_DST, &cmask));
     else if (pd()->with_bias())
         cmask = pd()->bias_cmask();
 
@@ -415,14 +417,14 @@ status_t xe_hp_systolic_gemm_t::init(engine_t *engine) {
         case 0: co_kind_ = 'F'; break;
         case (1 << 1): co_kind_ = 'R'; break;
         case (1 << 0): co_kind_ = 'C'; break;
+        case 3: co_kind_ = 'M'; break;
         case -1:
         default: co_kind_ = 'N'; break;
     }
 
     // Initialize compute kernels (assembly)
     {
-        auto status = pd()->use_new_kernels() ? init_compute_new(engine)
-                                              : init_compute_old(engine);
+        auto status = init_compute(engine);
         if (status != status::success) return status;
     }
 
@@ -442,146 +444,27 @@ status_t xe_hp_systolic_gemm_t::init(engine_t *engine) {
                     pd()->with_ab_zero_points(), clear_sum);
             if (status != status::success) return status;
 
-            create_kernel(engine, &copy_kernel_[copy_b][clear_sum],
-                    copy_kernel_t::name(arch_), kernel_ctx);
+            CHECK(create_kernel(engine, &copy_kernel_[copy_b][clear_sum],
+                    copy_kernel_t::name(arch_), kernel_ctx));
             if (!copy_kernel_[copy_b][clear_sum]) return status::runtime_error;
         }
     }
 
     if (get_verbose() >= 2) {
-        printf("onednn_verbose,info,gpu,gemm,kernel:%dx%d,%dx%dx%d,new:%c\n",
+        printf("onednn_verbose,info,gpu,gemm,kernel:%dx%d,%dx%dx%d\n",
                 pd()->unroll_m(), pd()->unroll_n(), compute_info_.wg[LoopM],
-                compute_info_.wg[LoopN], compute_info_.wg[LoopK],
-                pd()->use_new_kernels() ? 'Y' : 'N');
+                compute_info_.wg[LoopN], compute_info_.wg[LoopK]);
     }
 
     return status::success;
 }
 
-status_t xe_hp_systolic_gemm_t::init_compute_old(engine_t *engine) {
-    using kernel_t = xehp_systolic_gemm_kernel_t<gpu_xe_hp>;
-    using arch_t = compute::gpu_arch_t;
-
-    kernel_t::config_t cfg;
-
-    auto a_type = pd()->desc()->a_type();
-    auto b_type = pd()->desc()->b_type();
-    auto c_type = pd()->desc()->c_type();
-    auto acc_type = pd()->impl_acc_type();
-
-    cfg.a_type = convert_dnnl_type_to_ngen(a_type);
-    cfg.b_type = convert_dnnl_type_to_ngen(b_type);
-    cfg.c_type = convert_dnnl_type_to_ngen(c_type);
-    cfg.acc_type = convert_dnnl_type_to_ngen(acc_type);
-    cfg.alpha1 = (pd()->alpha() == 1.0f);
-    cfg.beta0 = (pd()->beta() == 0.0f);
-    cfg.beta1 = (pd()->beta() == 1.0f);
-    cfg.post_ops = pd()->attr()->post_ops_;
-    cfg.a_bias = cfg.b_bias = pd()->with_ab_zero_points();
-    cfg.c_packed = pd()->packed_c();
-    cfg.batch = pd()->with_batch();
-    walk_n_first_ = cfg.walk_n_first
-            = (pd()->desc()->m() >= 2 * pd()->desc()->n());
-    cfg.tile_m = pd()->unroll_m();
-    cfg.tile_n = pd()->unroll_n();
-    cfg.global_3x_buf = (cfg.tile_n == 32);
-
-    auto compute_engine = utils::downcast<compute::compute_engine_t *>(engine);
-    cfg.eu_count = compute_engine->device_info()->eu_count();
-
-    if (pd()->with_c_zero_points())
-        cfg.co_type = cfg.c_type;
-    else if (pd()->with_bias()) {
-        cfg.early_c_bias = true;
-        cfg.co_type = convert_dnnl_type_to_ngen(pd()->desc()->bias_type());
-    }
-
-    switch (co_kind_) {
-        case 'F': cfg.c_bias = kernel_t::bias_t::fixed; break;
-        case 'R': cfg.c_bias = kernel_t::bias_t::row; break;
-        case 'C': cfg.c_bias = kernel_t::bias_t::column; break;
-        case 'N':
-        default: cfg.c_bias = kernel_t::bias_t::none; break;
-    }
-
-    bool may_k_block = (pd()->desc()->k() > kernel_t::min_block_k(a_type));
-    bool got_info = false;
-    UNUSED(got_info); // If none of supported archs were selected by user.
-
-    for (bool first_k_block : {false, true}) {
-        for (bool last_k_block : {false, true}) {
-            if ((!first_k_block || !last_k_block) && !may_k_block) continue;
-            if (may_k_block && last_k_block
-                    && (cfg.c_bias == kernel_t::bias_t::none)
-                    && !cfg.have_post_op())
-                kernel_[first_k_block][last_k_block]
-                        = kernel_[first_k_block][false];
-            else if (may_k_block && first_k_block && cfg.beta1)
-                kernel_[first_k_block][last_k_block]
-                        = kernel_[false][last_k_block];
-            else {
-                auto cfg_copy = cfg;
-                if (!first_k_block) {
-                    cfg_copy.beta0 = false;
-                    cfg_copy.beta1 = true;
-                }
-                if (!last_k_block) {
-                    cfg_copy.c_bias = kernel_t::bias_t::none;
-                    cfg_copy.post_ops = post_ops_t();
-                }
-
-                switch (arch_) {
-                    case arch_t::xe_hp:
-                        REG_XEHP_ISA({
-                            auto kernel = kernel_t(cfg_copy);
-
-                            create_kernel(engine,
-                                    &kernel_[first_k_block][last_k_block],
-                                    &kernel);
-
-                            if (!got_info) {
-                                compute_info_ = kernel.driver_info(eu_count_);
-                                got_info = true;
-                            }
-                        });
-                        break;
-                    case arch_t::xe_hpg:
-                        REG_XEHPG_ISA({
-                            using kernel_xe_hpg_t
-                                    = xehp_systolic_gemm_kernel_t<gpu_xe_hpg>;
-                            cfg_copy.emulate64 = true;
-                            auto kernel = kernel_xe_hpg_t(
-                                    cfg_copy.cast<kernel_xe_hpg_t::config_t>());
-
-                            create_kernel(engine,
-                                    &kernel_[first_k_block][last_k_block],
-                                    &kernel);
-
-                            if (!got_info) {
-                                compute_info_ = kernel.driver_info(eu_count_);
-                                got_info = true;
-                            }
-                        });
-                        break;
-                    default:
-                        assert(!"Unsupported GPU architecture.");
-                        return status::unimplemented;
-                        break;
-                }
-
-                if (!kernel_[first_k_block][last_k_block])
-                    return status::runtime_error;
-            }
-        }
-    }
-
-    return status::success;
-}
-
-status_t xe_hp_systolic_gemm_t::init_compute_new(engine_t *engine) {
+status_t xe_hp_systolic_gemm_t::init_compute(engine_t *engine) {
     using kernel_t = gen_gemm_kernel_t;
     using kd_t = gen_gemm_xe_systolic_kernel_desc_t;
-    using offset_t = kd_t::offset_t;
+
+    auto *compute_engine = utils::downcast<compute::compute_engine_t *>(engine);
+    int stepping = compute_engine->device_info()->stepping_id();
 
     const auto d = pd()->desc();
 
@@ -590,25 +473,35 @@ status_t xe_hp_systolic_gemm_t::init_compute_new(engine_t *engine) {
     auto c_type = d->c_type();
     auto co_type = pd()->impl_co_type();
     auto acc_type = pd()->impl_acc_type();
+    bool trans_co = pd()->with_bias() && (d->trans_bias() == dnnl_trans);
 
-    offset_t ab_offset
-            = pd()->with_ab_zero_points() ? offset_t::fixed : offset_t::none;
-    offset_t c_offset
-            = pd()->with_c_zero_points() ? offset_t::runtime : offset_t::none;
-    offset_t bias_offset
-            = pd()->with_bias() ? offset_t::runtime : offset_t::none;
-
-    bool may_k_block = (d->k() > kd_t::min_block_k(a_type));
+    bool may_k_block
+            = (d->k() > kd_t::min_block_k(a_type)) && pd()->allow_k_blocking();
     bool got_info = false;
 
-    bool with_eltwise
-            = (pd()->attr()->post_ops_.find(primitive_kind::eltwise) != -1);
+    auto post_ops = pd()->post_ops();
+    bool with_post_ops = (post_ops->find(primitive_kind::eltwise) != -1)
+            || (post_ops->find(primitive_kind::binary) != -1);
+
+    kd_t kd_full;
+
+    auto status = kd_full.select_kernel(arch_, stepping, eu_count_,
+            pd()->with_batch(), pd()->packed_c(), trans_co,
+            pd()->with_a_zero_points(), pd()->with_b_zero_points(),
+            pd()->with_c_zero_points(), pd()->with_bias(), pd()->alpha(),
+            pd()->beta(), *post_ops, a_type, b_type, c_type, co_type, acc_type,
+            d->m(), d->n(), d->k(), d->batch(), pd()->unroll_m(),
+            pd()->unroll_n(), pd()->alt());
+
+    if (status != status::success) return status;
+
+    problem_ = std::move(*kd_full.problem());
 
     for (bool first_k_block : {false, true}) {
         for (bool last_k_block : {false, true}) {
             if ((!first_k_block || !last_k_block) && !may_k_block) continue;
-            if (may_k_block && last_k_block && (c_offset == offset_t::none)
-                    && !with_eltwise)
+            if (may_k_block && last_k_block && !pd()->with_c_zero_points()
+                    && !with_post_ops)
                 kernel_[first_k_block][last_k_block]
                         = kernel_[first_k_block][false];
             else if (may_k_block && first_k_block && pd()->beta() == 1.0f)
@@ -616,21 +509,22 @@ status_t xe_hp_systolic_gemm_t::init_compute_new(engine_t *engine) {
                         = kernel_[false][last_k_block];
             else {
                 auto this_beta = pd()->beta();
-                auto this_c_offset = c_offset;
-                auto *this_post_ops = &pd()->attr()->post_ops_;
+                bool this_c_offset = pd()->with_c_zero_points();
+                auto *this_post_ops = pd()->post_ops();
                 post_ops_t no_post_ops;
 
                 if (!first_k_block) this_beta = 1.0f;
                 if (!last_k_block) {
-                    this_c_offset = offset_t::none;
+                    this_c_offset = false;
                     this_post_ops = &no_post_ops;
                 }
 
                 kd_t kd;
 
-                auto status = kd.select_kernel(arch_, eu_count_,
-                        pd()->with_batch(), pd()->packed_c(), ab_offset,
-                        ab_offset, this_c_offset, bias_offset, pd()->alpha(),
+                auto status = kd.select_kernel(arch_, stepping, eu_count_,
+                        pd()->with_batch(), pd()->packed_c(), trans_co,
+                        pd()->with_a_zero_points(), pd()->with_b_zero_points(),
+                        this_c_offset, pd()->with_bias(), pd()->alpha(),
                         this_beta, *this_post_ops, a_type, b_type, c_type,
                         co_type, acc_type, d->m(), d->n(), d->k(), d->batch(),
                         pd()->unroll_m(), pd()->unroll_n(), pd()->alt());
@@ -644,8 +538,8 @@ status_t xe_hp_systolic_gemm_t::init_compute_new(engine_t *engine) {
 
                 kernel_t kernel(kd);
 
-                create_kernel(
-                        engine, &kernel_[first_k_block][last_k_block], &kernel);
+                CHECK(create_kernel(engine,
+                        &kernel_[first_k_block][last_k_block], &kernel));
 
                 if (!kernel_[first_k_block][last_k_block])
                     return status::runtime_error;
@@ -656,46 +550,6 @@ status_t xe_hp_systolic_gemm_t::init_compute_new(engine_t *engine) {
     return status::success;
 }
 
-status_t xe_hp_systolic_gemm_t::init_res_storage(
-        engine_t *engine, gpu_resource_t *r) const {
-    auto a_type = pd()->desc()->a_type();
-    auto b_type = pd()->desc()->b_type();
-
-    auto m = pd()->desc()->m();
-    auto n = pd()->desc()->n();
-    auto k = pd()->desc()->k();
-
-    int64_t align_m = compute_info_.wgTile(LoopM);
-    int64_t align_n = compute_info_.wgTile(LoopN);
-
-    auto m_aligned = utils::rnd_up(m, align_m);
-    auto n_aligned = utils::rnd_up(n, align_n);
-
-    auto max_ldab_packed = max_ld_packed(k);
-
-    if (!pd()->packed_a()) {
-        memory_storage_t *a_packed_ptr;
-        engine->create_memory_storage(&a_packed_ptr,
-                m_aligned * max_ldab_packed * types::data_type_size(a_type));
-        if (!a_packed_ptr) return status::runtime_error;
-
-        std::unique_ptr<memory_storage_t> a_packed(a_packed_ptr);
-        r->add_memory_storage(A_PACKED_, std::move(a_packed));
-    }
-
-    if (!pd()->packed_b()) {
-        memory_storage_t *b_packed_ptr;
-        engine->create_memory_storage(&b_packed_ptr,
-                n_aligned * max_ldab_packed * types::data_type_size(b_type));
-        if (!b_packed_ptr) return status::runtime_error;
-
-        std::unique_ptr<memory_storage_t> b_packed(b_packed_ptr);
-        r->add_memory_storage(B_PACKED_, std::move(b_packed));
-    }
-
-    return status::success;
-}
-
 bool xe_hp_systolic_gemm_t::enable_mn_blocking() const {
     return (pd()->desc()->m() >= 8192) && (pd()->desc()->n() >= 8192);
 }
@@ -746,9 +600,7 @@ xe_hp_systolic_gemm_t::get_blocking() const {
     nblock_k = nstl::max<int64_t>(nblock_k, 1);
     block_k = utils::div_up(k, nblock_k);
     block_k = nstl::max<dim_t>(block_k, 1);
-    block_k = utils::rnd_up(
-            (pd()->desc()->acc_type != pd()->desc()->c_type()) ? k : block_k,
-            unroll_k);
+    block_k = utils::rnd_up(pd()->allow_k_blocking() ? block_k : k, unroll_k);
     block_k = nstl::max<dim_t>(block_k, 1);
 
     return std::make_tuple(block_m, block_n, block_k);
@@ -858,8 +710,10 @@ status_t xe_hp_systolic_gemm_t::launch_compute(const gemm_exec_ctx_t &ctx,
         int32_t m, int32_t n, int32_t k, const memory_storage_t &ap,
         int64_t offset_a, int32_t lda, const memory_storage_t &bp,
         int64_t offset_b, int32_t ldb, const memory_storage_t &c,
-        int64_t offset_c, int32_t ldc, float alpha, float beta, int16_t ao,
-        int16_t bo, const memory_storage_t &co, int32_t offset_co,
+        int64_t offset_c, int32_t ldc, float alpha, float beta,
+        const memory_storage_t *ao, const memory_storage_t *bo,
+        const memory_storage_t &co, int32_t offset_co, int po_count,
+        const memory_storage_t **po_srcs, int32_t *offset_po_src,
         bool first_k_block, bool last_k_block, int32_t batch, int32_t stride_a,
         int32_t stride_b, int32_t stride_c) const {
 
@@ -893,26 +747,41 @@ status_t xe_hp_systolic_gemm_t::launch_compute(const gemm_exec_ctx_t &ctx,
     arg_list.set(argn++, k);
     arg_list.set(argn++, alpha);
     arg_list.set(argn++, beta);
-    if (pd()->with_ab_zero_points()) {
-        uint32_t abo = (uint16_t(ao) | (uint16_t(bo) << 16));
-        arg_list.set(argn++, abo);
-    }
+
+    if (pd()->with_a_zero_points()) arg_list.set(argn++, *ao);
+    if (pd()->with_b_zero_points()) arg_list.set(argn++, *bo);
     if ((pd()->with_bias() || pd()->with_c_zero_points())) {
         arg_list.set(argn++, co);
         arg_list.set(argn++, offset_co);
+        if (pd()->with_bias()) {
+            int32_t ldco = pd()->desc()->ld_bias();
+            arg_list.set(argn++, ldco);
+        }
     }
-    if (pd()->use_new_kernels()) {
-        uint32_t flags = 0;
-        if (co_kind_ == 'R') flags |= FlagCORow;
-        if (co_kind_ == 'C') flags |= FlagCOColumn;
-        if (!first_k_block) flags |= FlagNoninitialKBlock;
-        if (!last_k_block) flags |= FlagNonfinalKBlock;
-        arg_list.set(argn++, flags);
+    for (int i = 0; i < po_count; i++) {
+        if (!po_srcs[i]) continue;
+        arg_list.set(argn++, *po_srcs[i]);
+        arg_list.set(argn++, offset_po_src[i]);
+
+        if (problem_.binaryRow[i] && problem_.binaryCol[i])
+            arg_list.set(argn++, int32_t(pd()->ld_binary(i)));
     }
+
+    uint32_t flags = 0;
+    if (co_kind_ == 'R') flags |= FlagCORow;
+    if (co_kind_ == 'C') flags |= FlagCOColumn;
+    if (co_kind_ == 'M') flags |= FlagCORow | FlagCOColumn;
+    if (!first_k_block) flags |= FlagNoninitialKBlock;
+    if (!last_k_block) flags |= FlagNonfinalKBlock;
+    arg_list.set(argn++, flags);
+
     if (pd()->with_batch()) {
         arg_list.set(argn++, stride_a);
         arg_list.set(argn++, stride_b);
         arg_list.set(argn++, stride_c);
+        for (int i = 0; i < po_count; i++)
+            if (problem_.binaryBatch[i])
+                arg_list.set(argn++, int32_t(pd()->stride_binary(i, 0)));
     }
 
     auto thread_m = utils::div_up(m, pd()->unroll_m() * tg_m) * tg_m;
@@ -956,6 +825,7 @@ status_t xe_hp_systolic_gemm_t::execute(const gemm_exec_ctx_t &ctx) const {
     auto lda = packed_a ? 0 : pd()->desc()->lda();
     auto ldb = packed_b ? 0 : pd()->desc()->ldb();
     auto ldc = packed_c ? pd()->ldc_packed() : pd()->desc()->ldc();
+    auto ldco = pd()->with_bias() ? pd()->desc()->ld_bias() : 0;
 
     auto stride_a = pd()->desc()->stride_a();
     auto stride_b = pd()->desc()->stride_b();
@@ -970,11 +840,57 @@ status_t xe_hp_systolic_gemm_t::execute(const gemm_exec_ctx_t &ctx) const {
     auto &c_zp = GEMM_CTX_ARG_STORAGE(c_zero_point);
     auto &bias = GEMM_CTX_ARG_STORAGE(bias);
     auto *co = &c_zp;
+    memory_storage_t *ao = nullptr, *bo = nullptr;
 
-    auto &a_packed = packed_a ? a : CTX_GPU_RES_STORAGE(A_PACKED_);
-    auto &b_packed = packed_b ? b : CTX_GPU_RES_STORAGE(B_PACKED_);
+    std::unique_ptr<memory_storage_t> a_packed_temp, b_packed_temp;
 
-    int32_t ao = 0, bo = 0;
+    if (!packed_a)
+        a_packed_temp = ctx.get_scratchpad_grantor().get_memory_storage(
+                memory_tracking::names::key_gemm_blocked_a);
+    if (!packed_b)
+        b_packed_temp = ctx.get_scratchpad_grantor().get_memory_storage(
+                memory_tracking::names::key_gemm_blocked_b);
+
+    auto &a_packed = packed_a ? a : *a_packed_temp;
+    auto &b_packed = packed_b ? b : *b_packed_temp;
+
+    const memory_storage_t *po_srcs[GEMM_MAX_PO];
+
+    int po_count = pd()->post_ops()->len();
+    assert(po_count <= GEMM_MAX_PO);
+
+    for (int i = 0; i < po_count; i++) {
+        auto &src = pd()->binary_srcs()[i];
+        switch (src.type) {
+            case pd_t::binary_src_t::binary:
+                po_srcs[i]
+                        = ctx.args()
+                                  .exec_args
+                                  .at(DNNL_ARG_ATTR_MULTIPLE_POST_OP(src.index)
+                                          | DNNL_ARG_SRC_1)
+                                  .mem->memory_storage();
+                break;
+            case pd_t::binary_src_t::bias: po_srcs[i] = &bias; break;
+            case pd_t::binary_src_t::scales:
+                switch (src.index) {
+                    case DNNL_ARG_WEIGHTS:
+                        po_srcs[i] = &GEMM_CTX_ARG_STORAGE(a_scales);
+                        break;
+                    case DNNL_ARG_SRC:
+                        po_srcs[i] = &GEMM_CTX_ARG_STORAGE(b_scales);
+                        break;
+                    case DNNL_ARG_DST:
+                        po_srcs[i] = &GEMM_CTX_ARG_STORAGE(c_scales);
+                        break;
+                    default:
+                        po_srcs[i] = nullptr;
+                        assert(!"invalid scale type");
+                        break;
+                }
+                break;
+            default: po_srcs[i] = nullptr; break;
+        }
+    }
 
     size_t off_a0
             = a.offset() / types::data_type_size(a_type) + pd()->dyn_offset_a;
@@ -984,14 +900,14 @@ status_t xe_hp_systolic_gemm_t::execute(const gemm_exec_ctx_t &ctx) const {
             = c.offset() / types::data_type_size(c_type) + pd()->dyn_offset_c;
     size_t off_co0 = 0;
 
+    int32_t po_offsets0[GEMM_MAX_PO] = {0}, po_offsets[GEMM_MAX_PO] = {0};
+    for (int i = 0; i < po_count; i++)
+        if (po_srcs[i])
+            po_offsets0[i] = po_srcs[i]->offset() / problem_.Tbinary[i];
+
     if (pd()->with_ab_zero_points()) {
-        const int *ao_i32 = nullptr;
-        const int *bo_i32 = nullptr;
-        pd()->attr()->zero_points_.get(DNNL_ARG_SRC, nullptr, nullptr, &ao_i32);
-        pd()->attr()->zero_points_.get(
-                DNNL_ARG_WEIGHTS, nullptr, nullptr, &bo_i32);
-        ao = -*ao_i32;
-        bo = -*bo_i32;
+        ao = &GEMM_CTX_ARG_STORAGE(a_zero_point);
+        bo = &GEMM_CTX_ARG_STORAGE(b_zero_point);
     }
 
     if (pd()->with_bias()) {
@@ -1002,9 +918,8 @@ status_t xe_hp_systolic_gemm_t::execute(const gemm_exec_ctx_t &ctx) const {
     int64_t block_m = 0, block_n = 0, block_k = 0;
     std::tie(block_m, block_n, block_k) = get_blocking();
 
-    auto ld_packed = get_ld_packed(k);
-    auto lda_packed = packed_a ? pd()->lda_packed() : ld_packed;
-    auto ldb_packed = packed_b ? pd()->ldb_packed() : ld_packed;
+    auto lda_packed = pd()->lda_packed(k);
+    auto ldb_packed = pd()->ldb_packed(k);
 
     status_t status;
 
@@ -1047,15 +962,36 @@ status_t xe_hp_systolic_gemm_t::execute(const gemm_exec_ctx_t &ctx) const {
                 switch (co_kind_) {
                     case 'R': off_co += Bm; break;
                     case 'C': off_co += Bn; break;
+                    case 'M':
+                        off_co += isColMajor(problem_.CO.layout)
+                                ? (Bn * ldco + Bm)
+                                : (Bm * ldco + Bn);
+                        break;
                     default: break;
                 }
 
+                for (int i = 0; i < po_count; i++) {
+                    po_offsets[i] = po_offsets0[i];
+                    bool row = problem_.binaryRow[i],
+                         col = problem_.binaryCol[i];
+                    if (row && col) {
+                        auto ld = pd()->ld_binary(i);
+                        po_offsets[i] += isColMajor(problem_.binary[i].layout)
+                                ? (Bn * ld + Bm)
+                                : (Bm * ld + Bn);
+                    } else if (row)
+                        po_offsets[i] += Bm;
+                    else if (col)
+                        po_offsets[i] += Bn;
+                }
+
                 float this_beta = first_k_block ? beta : 1.0f;
                 status = launch_compute(ctx, size_m, size_n, size_k, a_packed,
                         off_a_packed, lda_packed, b_packed, off_b_packed,
                         ldb_packed, c, off_c, ldc, alpha, this_beta, ao, bo,
-                        *co, off_co, first_k_block, last_k_block, batch,
-                        stride_a, stride_b, stride_c);
+                        *co, off_co, po_count, po_srcs, po_offsets,
+                        first_k_block, last_k_block, batch, stride_a, stride_b,
+                        stride_c);
                 if (status) return status;
             }
         }
