@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -31,8 +31,9 @@ class ELFCodeGenerator : public BinaryCodeGenerator<hw>
 public:
     inline std::vector<uint8_t> getBinary();
     static inline HW getBinaryArch(const std::vector<uint8_t> &binary);
-    static inline void getBinaryHWInfo(const std::vector<uint8_t> &binary, HW &outHW, int &outStepping);
+    static inline void getBinaryHWInfo(const std::vector<uint8_t> &binary, HW &outHW, Product &outProduct);
 
+    explicit ELFCodeGenerator(Product product_)  : BinaryCodeGenerator<hw>(product_) {}
     explicit ELFCodeGenerator(int stepping_ = 0) : BinaryCodeGenerator<hw>(stepping_) {}
 
 protected:
@@ -98,6 +99,8 @@ protected:
         BinaryCodeGenerator<hw>::epilogue(GRFCount, hasSLM, r0_info);
     }
 
+    inline std::vector<uint8_t> getBinary(const std::vector<uint8_t> &code);
+
 private:
     using BinaryCodeGenerator<hw>::labelManager;
     using BinaryCodeGenerator<hw>::rootStream;
@@ -163,11 +166,12 @@ private:
             uint64_t entrySize = 0;
         } sectionHeaders[5];
         struct Note {
-            uint32_t nameSize = 0;
+            uint32_t nameSize = 8;
             uint32_t descSize = 4;
             enum Type : uint32_t {
                 ProductFamily = 1, GfxCoreFamily = 2, TargetMetadata = 3
             } type = Type::GfxCoreFamily;
+            const char name[8] = "IntelGT";
             uint32_t payload;
         } noteGfxCore;
         struct StringTable {
@@ -188,18 +192,7 @@ private:
             fileHeader.sectionTableOff = offsetof(ZebinELF, sectionHeaders);
             fileHeader.sectionCount = sizeof(sectionHeaders) / sizeof(SectionHeader);
 
-#ifdef NGEN_OLD_ZEBIN_ELF
-            fileHeader.type = ZebinExec;
-            fileHeader.sectionCount--;
-#endif
-
             fileHeader.flags.all = 0;
-            fileHeader.flags.parts.useGfxCoreFamily = 1;
-            fileHeader.machine = static_cast<uint16_t>(npack::encodeGfxCoreFamily(hw));
-            if (hw == HW::XeHPG) {
-                fileHeader.flags.parts.useGfxCoreFamily = 0;
-                fileHeader.machine = static_cast<uint16_t>(npack::ProductFamily::DG2);
-            }
 
             sectionHeaders[0].name = 0;
             sectionHeaders[0].type = SectionHeader::Type::Null;
@@ -323,14 +316,17 @@ template <typename... Targs> void requireBarriers(Targs&&... args) { ngen::ELFCo
 
 template <HW hw>
 std::vector<uint8_t> ELFCodeGenerator<hw>::getBinary()
+{
+    return getBinary(this->getCode());
+}
+
+template <HW hw>
+std::vector<uint8_t> ELFCodeGenerator<hw>::getBinary(const std::vector<uint8_t> &kernel)
 {
     using super = BinaryCodeGenerator<hw>;
     std::vector<uint8_t> binary;
     std::string metadata;
 
-    // Get raw kernel, also fixing up labels.
-    auto kernel = super::getCode();
-
     // Locate entrypoints for XeHP+.
     if (hw >= HW::XeHP) {
         auto idPerThread = super::_labelLocalIDsLoaded.getID(labelManager);
@@ -354,9 +350,9 @@ std::vector<uint8_t> ELFCodeGenerator<hw>::getBinary()
     binary.resize(szELF + szMetadata + szKernel);
 
     (void) new(binary.data()) ZebinELF(szKernelName, szMetadata, szKernel);
-    std::memcpy(&binary[ZebinELF::kernelNameOffset()], interface_.getExternalName().c_str(), szKernelName);
-    std::memcpy(&binary[szELF], metadata.c_str(), metadata.size());
-    std::memcpy(&binary[szELF + szMetadata], kernel.data(), kernel.size());
+    utils::copy_into(binary, ZebinELF::kernelNameOffset(), interface_.getExternalName());
+    utils::copy_into(binary, szELF, metadata);
+    utils::copy_into(binary, szELF + szMetadata, kernel);
 
     return binary;
 }
@@ -365,20 +361,21 @@ template <HW hw>
 inline HW ELFCodeGenerator<hw>::getBinaryArch(const std::vector<uint8_t> &binary)
 {
     HW outHW;
-    int outStepping;
+    Product outProduct;
 
-    getBinaryHWInfo(binary, outHW, outStepping);
+    getBinaryHWInfo(binary, outHW, outProduct);
 
     return outHW;
 }
 
 template <HW hw>
-inline void ELFCodeGenerator<hw>::getBinaryHWInfo(const std::vector<uint8_t> &binary, HW &outHW, int &outStepping)
+inline void ELFCodeGenerator<hw>::getBinaryHWInfo(const std::vector<uint8_t> &binary, HW &outHW, Product &outProduct)
 {
     using Note = typename ZebinELF::Note;
 
     outHW = HW::Unknown;
-    outStepping = 0;
+    outProduct.family = ProductFamily::Unknown;
+    outProduct.stepping = 0;
 
     auto zebinELF = reinterpret_cast<const ZebinELF *>(binary.data());
     if (zebinELF->valid()) {
@@ -390,13 +387,13 @@ inline void ELFCodeGenerator<hw>::getBinaryHWInfo(const std::vector<uint8_t> &bi
                 auto rstart = reinterpret_cast<const uint8_t *>(start);
                 if (start->descSize == sizeof(start->payload)) {
                     auto *actualPayload = reinterpret_cast<const uint32_t *>(
-                        rstart + offsetof(Note, payload) + utils::alignup_pow2(start->nameSize, 4)
+                        rstart + offsetof(Note, payload) - sizeof(Note::name) + utils::alignup_pow2(start->nameSize, 4)
                     );
                     switch (start->type) {
                         case Note::Type::ProductFamily: {
-                            auto decodedHW = npack::decodeProductFamily(static_cast<npack::ProductFamily>(*actualPayload));
-                            if (decodedHW != HW::Unknown)
-                                outHW = decodedHW;
+                            auto decodedFamily = npack::decodeProductFamily(static_cast<npack::ProductFamily>(*actualPayload));
+                            if (decodedFamily != ProductFamily::Unknown)
+                                outProduct.family = decodedFamily;
                             break;
                         }
                         case Note::Type::GfxCoreFamily:
@@ -406,13 +403,13 @@ inline void ELFCodeGenerator<hw>::getBinaryHWInfo(const std::vector<uint8_t> &bi
                         case Note::Type::TargetMetadata: {
                             typename ZebinELF::TargetMetadata metadata;
                             metadata.all = *actualPayload;
-                            outStepping = metadata.parts.minHWRevision;
+                            outProduct.stepping = metadata.parts.minHWRevision;
                         }
                         default: break;
                     }
                 }
                 start = reinterpret_cast<const Note *>(
-                    rstart + offsetof(Note, payload)
+                    rstart + offsetof(Note, name)
                            + utils::alignup_pow2(start->nameSize, 4)
                            + utils::alignup_pow2(start->descSize, 4)
                 );
@@ -421,11 +418,16 @@ inline void ELFCodeGenerator<hw>::getBinaryHWInfo(const std::vector<uint8_t> &bi
             if (zebinELF->fileHeader.flags.parts.useGfxCoreFamily)
                 outHW = npack::decodeGfxCoreFamily(static_cast<npack::GfxCoreFamily>(zebinELF->fileHeader.machine));
             else
-                outHW = npack::decodeProductFamily(static_cast<npack::ProductFamily>(zebinELF->fileHeader.machine));
-            outStepping = zebinELF->fileHeader.flags.parts.minHWRevision;
+                outProduct.family = npack::decodeProductFamily(static_cast<npack::ProductFamily>(zebinELF->fileHeader.machine));
+            outProduct.stepping = zebinELF->fileHeader.flags.parts.minHWRevision;
         }
     } else
-        npack::getBinaryHWInfo(binary, outHW, outStepping);
+        npack::getBinaryHWInfo(binary, outHW, outProduct);
+
+    if (outHW != HW::Unknown && outProduct.family == ProductFamily::Unknown)
+        outProduct.family = genericProductFamily(outHW);
+    else if (outHW == HW::Unknown && outProduct.family != ProductFamily::Unknown)
+        outHW = getCore(outProduct.family);
 }
 
 } /* namespace ngen */
