@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -113,6 +113,9 @@ public:
     void setSkipPerThreadOffset(int32_t offset)          { offsetSkipPerThread = offset; }
     void setSkipCrossThreadOffset(int32_t offset)        { offsetSkipCrossThread = offset; }
 
+    inline GRF getCrossthreadBase(bool effective = true) const;
+    inline GRF getArgLoadBase() const;
+
     inline void finalize();
 
     template <typename CodeGenerator>
@@ -169,10 +172,11 @@ protected:
     int walkOrder[3] = {-1, -1, -1};
     size_t wg[3] = {0, 0, 0};
 
+    int crossthreadBytes = 0;
     int crossthreadGRFs = 0;
     int inlineGRFs = 0;
     inline int getCrossthreadGRFs() const;
-    inline GRF getCrossthreadBase(bool effective = true) const;
+    inline int getCrossthreadBytes() const;
     int grfsPerLID() const { return (simd > 16 && GRF::bytes(hw) < 64) ? 2 : 1; }
 
     static inline int defaultInlineGRFs(HW hw);
@@ -327,7 +331,7 @@ void InterfaceHandler::generateDummyCL(std::ostream &stream) const
 
     if (needLocalID)        stream << "    (void) ____[get_local_id(0)];\n";
     if (needLocalSize)      stream << "    (void) ____[get_enqueued_local_size(0)];\n";
-    if (barrierCount > 0)   stream << "    barrier(CLK_GLOBAL_MEM_FENCE);\n";
+    if (barrierCount > 0)   stream << "    __asm__ volatile(\"barrier\");\n";
     for (int i = 1; i < barrierCount; i++) {
         stream << "    local NamedBarrier_t *bar" << i << ";\n"
                   "    bar" << i << " = named_barrier_init(1);\n"
@@ -349,6 +353,12 @@ void InterfaceHandler::generateDummyCL(std::ostream &stream) const
             stream << "    __asm__ volatile(\"\" :: \"rw.u\"(" << assignment.name << "));\n";
     }
 
+    for (const auto &assignment : assignments) {
+        // Force IGC to assume surface accesses could occur if necessary.
+        if (assignment.exttype == ExternalArgumentType::GlobalPtr && assignment.globalSurfaceAccess())
+            stream << "    { volatile uchar __load = ((global uchar *) " << assignment.name << ")[get_local_id(0)];}\n";
+    }
+
     stream << "}\n";
 }
 
@@ -434,7 +444,9 @@ void InterfaceHandler::finalize()
 
     assignArgsOfType(ExternalArgumentType::Hidden);
 
-    crossthreadGRFs = base.getBase() - getCrossthreadBase().getBase() + 1;
+    crossthreadBytes = (base.getBase() - getCrossthreadBase().getBase()) * GRF::bytes(hw)
+                     + ((offset + 31) & -32);
+    crossthreadGRFs = GRF::bytesToGRFs(hw, crossthreadBytes);
 
     // Manually add regular local size arguments.
     if (needLocalSize && !needNonuniformWGs)
@@ -455,6 +467,19 @@ GRF InterfaceHandler::getCrossthreadBase(bool effective) const
         return GRF(1 + 3 * grfsPerLID());
 }
 
+GRF InterfaceHandler::getArgLoadBase() const
+{
+    return getCrossthreadBase().advance(inlineGRFs);
+}
+
+int InterfaceHandler::getCrossthreadBytes() const
+{
+#ifdef NGEN_SAFE
+    if (!finalized) throw interface_not_finalized();
+#endif
+    return crossthreadBytes;
+}
+
 int InterfaceHandler::getCrossthreadGRFs() const
 {
 #ifdef NGEN_SAFE
@@ -473,17 +498,10 @@ int InterfaceHandler::defaultInlineGRFs(HW hw)
 template <typename CodeGenerator>
 void InterfaceHandler::generatePrologue(CodeGenerator &generator, const GRF &temp) const
 {
-#ifdef NGEN_INTERFACE_OLD_PROLOGUE
     if (needLocalID)
-        generator.loadlid(getCrossthreadGRFs(), needLocalID, simd, temp, 8*16);
-    if (getCrossthreadGRFs() > 1)
-        generator.loadargs(getCrossthreadBase(), getCrossthreadGRFs(), temp);
-#else
-    if (needLocalID)
-        generator.loadlid(getCrossthreadGRFs(), needLocalID, simd, temp, 12*16);
+        generator.loadlid(getCrossthreadBytes(), needLocalID, simd, temp, -1);
     if (getCrossthreadGRFs() > inlineGRFs)
-        generator.loadargs(getCrossthreadBase().advance(inlineGRFs), getCrossthreadGRFs() - inlineGRFs, temp);
-#endif
+        generator.loadargs(getArgLoadBase(), getCrossthreadGRFs() - inlineGRFs, temp);
 }
 
 std::string InterfaceHandler::generateZeInfo() const
@@ -494,7 +512,7 @@ std::string InterfaceHandler::generateZeInfo() const
 
     std::stringstream md;
 
-    md << "version: 1.0\n"
+    md << "version: 1.8\n"
           "kernels: \n"
           "  - name: \"" << kernelName << "\"\n"
           "    execution_env: \n"
@@ -514,11 +532,10 @@ std::string InterfaceHandler::generateZeInfo() const
            << "        - " << walkOrder[1] << "\n"
            << "        - " << std::max(walkOrder[2], 0) << "\n";
     }
-    md << "      actual_kernel_start_offset: " << offsetSkipCrossThread << '\n';
     if (offsetSkipPerThread > 0)
         md << "      offset_to_skip_per_thread_data_load: " << offsetSkipPerThread << '\n';
     if (barrierCount > 0)
-        md << "      barrier_count: " << barrierCount << '\n';
+        md << "      barrier_count: " << utils::roundup_pow2(barrierCount) << '\n';
     if (allow64BitBuffers)
         md << "      has_4gb_buffers: true\n";
     if (needDPAS)
@@ -531,8 +548,12 @@ std::string InterfaceHandler::generateZeInfo() const
         md << "      has_no_stateless_write: true\n";
     if (needNoPreemption)
         md << "      disable_mid_thread_preemption: true\n";
-    md << "\n";
-    md << "    payload_arguments: \n";
+    if (inlineGRFs > 0)
+        md << "      inline_data_payload_size: " << inlineGRFs * GRF::bytes(hw) << "\n";
+    if (!assignments.empty()) {
+        md << "\n"
+              "    payload_arguments: \n";
+    }
     for (auto &assignment : assignments) {
         uint32_t size = 0;
         bool skipArg = false;
@@ -554,14 +575,15 @@ std::string InterfaceHandler::generateZeInfo() const
                 md << "      - arg_type: arg_byvalue\n";
                 size = (assignment.reg.getDwords() << 2);
                 break;
-            case ExternalArgumentType::LocalPtr:
             case ExternalArgumentType::GlobalPtr:
-                if (!assignment.globalStatelessAccess())
+                if (!assignment.globalStatelessAccess()) {
                     skipArg = true;
-                else {
-                    md << "      - arg_type: arg_bypointer\n";
-                    size = (assignment.reg.getDwords() << 2);
+                    break;
                 }
+                // fall through
+            case ExternalArgumentType::LocalPtr:
+                md << "      - arg_type: arg_bypointer\n";
+                size = (assignment.reg.getDwords() << 2);
                 break;
             case ExternalArgumentType::Hidden: {
                 explicitArg = false;
@@ -599,28 +621,31 @@ std::string InterfaceHandler::generateZeInfo() const
         md << "\n";
     }
 
-    md << "\n";
-    md << "    binding_table_indices: \n";
-
+    bool firstSurface = true;
     for (auto &assignment : assignments) {
         if (assignment.globalSurfaceAccess()) {
+            if (firstSurface) {
+                md << "\n"
+                      "    binding_table_indices: \n";
+                firstSurface = false;
+            }
             md << "      - bti_value: " << assignment.surface << "\n"
                   "        arg_index: " << assignment.index << "\n"
                   " \n";
         }
     }
 
-    md << "\n";
-    md << "    per_thread_payload_arguments: \n";
-
     if (needLocalID) {
+        md << "\n"
+              "    per_thread_payload_arguments: \n";
+
         if (simd == 1) {
             md << "      - arg_type: packed_local_ids\n"
                   "        offset: 0\n"
                   "        size: 6\n"
                   "  \n";
         } else {
-            auto localIDBytes = grfsPerLID() * 32;
+            auto localIDBytes = grfsPerLID() * GRF::bytes(hw);
             localIDBytes *= 3; // runtime currently supports 0 or 3 localId channels in per thread data
             md << "      - arg_type: local_id\n"
                   "        offset: 0\n"
