@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,6 +21,7 @@
 
 #include <CL/cl.h>
 
+#include <atomic>
 #include <sstream>
 
 #include "ngen_elf.hpp"
@@ -48,11 +49,15 @@ template <HW hw>
 class OpenCLCodeGenerator : public ELFCodeGenerator<hw>
 {
 public:
+    explicit OpenCLCodeGenerator(Product product_)  : ELFCodeGenerator<hw>(product_) {}
     explicit OpenCLCodeGenerator(int stepping_ = 0) : ELFCodeGenerator<hw>(stepping_) {}
 
     inline std::vector<uint8_t> getBinary(cl_context context, cl_device_id device, const std::string &options = "-cl-std=CL2.0");
     inline cl_kernel getKernel(cl_context context, cl_device_id device, const std::string &options = "-cl-std=CL2.0");
     static inline HW detectHW(cl_context context, cl_device_id device);
+    static inline void detectHWInfo(cl_context context, cl_device_id device, HW &outHW, Product &outProduct);
+
+    /* Deprecated. Use the Product-based API instead. */
     static inline void detectHWInfo(cl_context context, cl_device_id device, HW &outHW, int &outStepping);
 
 private:
@@ -80,8 +85,8 @@ static inline std::vector<uint8_t> getOpenCLCProgramBinary(cl_context context, c
         throw opencl_error();
 
     detail::handleCL(clBuildProgram(program, 1, &device, options, nullptr, nullptr));
-    size_t nDevices = 0;
-    detail::handleCL(clGetProgramInfo(program, CL_PROGRAM_NUM_DEVICES, sizeof(size_t), &nDevices, nullptr));
+    cl_uint nDevices = 0;
+    detail::handleCL(clGetProgramInfo(program, CL_PROGRAM_NUM_DEVICES, sizeof(cl_uint), &nDevices, nullptr));
     std::vector<cl_device_id> devices(nDevices);
     detail::handleCL(clGetProgramInfo(program, CL_PROGRAM_DEVICES, sizeof(cl_device_id) * nDevices, devices.data(), nullptr));
     size_t deviceIdx = std::distance(devices.begin(), std::find(devices.begin(), devices.end(), device));
@@ -105,11 +110,12 @@ static inline std::vector<uint8_t> getOpenCLCProgramBinary(cl_context context, c
     return binaries[deviceIdx];
 }
 
-inline bool tryZebinFirst(cl_device_id device)
+inline bool tryZebinFirst(cl_device_id device, bool setDefault = false, bool newDefault = false)
 {
-    // Future: return true for newer OpenCL runtimes
-    //         with known zebin support.
-    return false;
+    static std::atomic<bool> hint(false);
+    if (setDefault) hint = newDefault;
+
+    return hint;
 }
 
 }; /* namespace detail */
@@ -149,6 +155,7 @@ std::vector<uint8_t> OpenCLCodeGenerator<hw>::getBinary(cl_context context, cl_d
             try {
                 return getPatchTokenBinary(context, device, &code, options);
             } catch (...) {
+                (void) detail::tryZebinFirst(device, true, true);
                 continue;
             }
         } else
@@ -193,9 +200,10 @@ cl_kernel OpenCLCodeGenerator<hw>::getKernel(cl_context context, cl_device_id de
         status = clBuildProgram(program, 1, &device, options.c_str(), nullptr, nullptr);
 
         good = (status == CL_SUCCESS);
-        if (good)
+        if (good) {
+            (void) detail::tryZebinFirst(device, true, !legacy);
             break;
-        else
+        } else
             detail::handleCL(clReleaseProgram(program));
     }
 
@@ -216,9 +224,9 @@ template <HW hw>
 HW OpenCLCodeGenerator<hw>::detectHW(cl_context context, cl_device_id device)
 {
     HW outHW;
-    int outStepping;
+    Product outProduct;
 
-    detectHWInfo(context, device, outHW, outStepping);
+    detectHWInfo(context, device, outHW, outProduct);
 
     return outHW;
 }
@@ -226,12 +234,20 @@ HW OpenCLCodeGenerator<hw>::detectHW(cl_context context, cl_device_id device)
 template <HW hw>
 void OpenCLCodeGenerator<hw>::detectHWInfo(cl_context context, cl_device_id device, HW &outHW, int &outStepping)
 {
-    const char *dummyCL = "kernel void _(){}";
+    Product outProduct;
+    detectHWInfo(context, device, outHW, outProduct);
+    outStepping = outProduct.stepping;
+}
+
+template <HW hw>
+void OpenCLCodeGenerator<hw>::detectHWInfo(cl_context context, cl_device_id device, HW &outHW, Product &outProduct)
+{
+    const char *dummyCL = "kernel void _ngen_hw_detect(){}";
     const char *dummyOptions = "";
 
     auto binary = detail::getOpenCLCProgramBinary(context, device, dummyCL, dummyOptions);
 
-    ELFCodeGenerator<hw>::getBinaryHWInfo(binary, outHW, outStepping);
+    ELFCodeGenerator<hw>::getBinaryHWInfo(binary, outHW, outProduct);
 }
 
 } /* namespace ngen */
