@@ -63,8 +63,8 @@ struct cudnn_conv_inner_product_fwd_t : public cudnn_inner_product_fwd_t {
         status_t init(engine_t *engine) {
             using namespace data_type;
             using namespace prop_kind;
-            const auto attr_skip_mask = primitive_attr_t::skip_mask_t::oscale
-                    | primitive_attr_t::skip_mask_t::post_ops;
+            using sm_t = primitive_attr_t::skip_mask_t;
+            const auto attr_skip_mask = sm_t::scales_runtime | sm_t::post_ops;
             // Flag for checking if the fused routine can be used for the
             // blocked format case. If set to true, that implies ReLU and
             // blocking are used.
@@ -79,10 +79,10 @@ struct cudnn_conv_inner_product_fwd_t : public cudnn_inner_product_fwd_t {
                     && blocking_ok(with_eltwise(), use_fused_path_for_blocking)
                     && IMPLICATION(with_bias(), memory_format_ok(weights_md(1)))
                     && attr()->has_default_values(attr_skip_mask)
-                    && post_ops_ok(attr())
-                    && IMPLICATION(!attr()->output_scales_.has_default_values(),
+                    && attr_post_ops_ok(attr())
+                    && IMPLICATION(!attr()->scales_.has_default_values(),
                             utils::one_of(src_md_.data_type, s8)
-                                    && attr()->output_scales_.mask_ == 0);
+                                    && attr_scales_ok());
             if (!ok) return status::unimplemented;
             if (has_zero_dim_memory()) return status::success;
 
@@ -93,22 +93,6 @@ struct cudnn_conv_inner_product_fwd_t : public cudnn_inner_product_fwd_t {
                     with_eltwise(), with_sum(), use_fused_path_for_blocking);
             return st;
         }
-        bool post_ops_ok(const primitive_attr_t *attr) const {
-            const auto &p = attr->post_ops_;
-
-            auto is_eltwise
-                    = [&](int idx) { return p.entry_[idx].is_eltwise(false); };
-            auto is_sum = [&](int idx) { return p.entry_[idx].is_sum(false); };
-
-            switch (p.len()) {
-                case 0: return true; // no post_ops
-                case 1: return is_eltwise(0) || is_sum(0); // sum OR eltwise
-                case 2: return is_sum(0) && is_eltwise(1); // sum -> eltwise
-                default: return false;
-            }
-
-            return false;
-        }
         bool with_eltwise() const {
             return attr()->post_ops_.find(primitive_kind::eltwise) != -1;
         }
