@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 * Copyright 2020 Codeplay Software Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -30,6 +30,12 @@
 #include "gpu/nvidia/sycl_cuda_stream.hpp"
 #include "gpu/nvidia/sycl_cuda_utils.hpp"
 
+// `CUDNN_FMA_MATH` is available starting from cuDNN v8.
+// The behavior is consistent with `CUDNN_DEFAULT_MATH` for v7.
+#if defined(CUDNN_MAJOR) && (CUDNN_MAJOR < 8)
+#define CUDNN_FMA_MATH CUDNN_DEFAULT_MATH
+#endif
+
 namespace dnnl {
 namespace impl {
 namespace gpu {
@@ -52,7 +58,6 @@ protected:
     cudnnTensorFormat_t formats[NUM_IO];
     bool filter_needs_transform = false;
     cudnnFilterDescriptor_t weights_desc;
-    float alpha = 0.f;
     float beta = 0.f;
     int group_count = 1;
     bool with_groups = false;
@@ -60,7 +65,6 @@ protected:
     bool with_bias = false;
 
     bool do_scaling = false;
-    float output_scaling = 1.0f;
     bool use_temp_dst_ = false;
     cudnnDataType_t computation_data_type = CUDNN_DATA_FLOAT;
     cudnnDataType_t reorder_type = CUDNN_DATA_INT8;
@@ -128,10 +132,8 @@ public:
         CHECK(set_padding_and_dilation(pd));
         with_groups = pd->with_groups();
         with_bias = pd->with_bias();
-        alpha = 1.0f;
         beta = 0.0f;
-        output_scaling = pd->attr()->output_scales_.scales_[0];
-        do_scaling = output_scaling != 1.f;
+        do_scaling = !pd->attr()->scales_.has_default_values();
         dnnl_descs[x] = *pd->invariant_src_md();
         dnnl_descs[weights] = *pd->invariant_wei_md();
         dnnl_descs[y] = *pd->invariant_dst_md();
@@ -213,10 +215,36 @@ public:
     };
 
     status_t create_and_set_convolution_desc(const convolution_pd_t *pd) {
+
+        cudnnMathType_t cudnn_math_type = CUDNN_TENSOR_OP_MATH;
+
+        if (computation_data_type == CUDNN_DATA_FLOAT) {
+
+            dnnl_fpmath_mode_t dnnl_fpmath_mode = pd->attr()->fpmath_mode_;
+
+            switch (dnnl_fpmath_mode) {
+                case dnnl_fpmath_mode_strict:
+                    cudnn_math_type = CUDNN_FMA_MATH;
+                    break;
+                case dnnl_fpmath_mode_bf16:
+                    cudnn_math_type = CUDNN_TENSOR_OP_MATH;
+                    break;
+                case dnnl_fpmath_mode_f16:
+                    cudnn_math_type = CUDNN_TENSOR_OP_MATH_ALLOW_CONVERSION;
+                    break;
+                case dnnl_fpmath_mode_tf32:
+                    cudnn_math_type = CUDNN_DEFAULT_MATH;
+                    break;
+                case dnnl_fpmath_mode_any:
+                    cudnn_math_type = CUDNN_TENSOR_OP_MATH_ALLOW_CONVERSION;
+                    break;
+            }
+        }
+
         CUDNN_EXECUTE_FUNC_V(cudnnCreateConvolutionDescriptor, &conv_desc);
         // Allow cuDNN to dispatch into Tensor Core implementations
         CHECK(CUDNN_EXECUTE_FUNC_S(
-                cudnnSetConvolutionMathType, conv_desc, CUDNN_TENSOR_OP_MATH));
+                cudnnSetConvolutionMathType, conv_desc, cudnn_math_type));
         CUDNN_EXECUTE_FUNC_V(cudnnSetConvolutionNdDescriptor, conv_desc,
                 ndims[x] - 2, padding, filter_strides, dilation,
                 cudnnConvolutionMode_t::CUDNN_CROSS_CORRELATION,
@@ -277,13 +305,6 @@ public:
                 &beta, descs[io::y], y);
     }
 
-    void execute_scale(cudnnHandle_t handle, void *y) const {
-        if (do_scaling) {
-            CUDNN_EXECUTE_FUNC_V(
-                    cudnnScaleTensor, handle, descs[io::y], y, &output_scaling);
-        }
-    }
-
     void execute_set_weights_bias(
             cudnnHandle_t handle, void *weights, void *bias, float value) {
         CUDNN_EXECUTE_FUNC_V(
@@ -470,16 +491,35 @@ public:
             const std::vector<void *> &args) const override {
         auto x = args[0], weights = args[1], y = args[2], bias = args[3],
              scratchpad = args[4], post_op_scratch = args[6],
-             post_op_reorder = args[7];
+             post_op_reorder = args[7], src_scale = args[8],
+             wei_scale = args[9], dst_scale = args[10];
         void *output = use_temp_dst_ ? post_op_scratch : y;
         if (using_transformed_filter()) {
             auto w_scratch = args[5];
             transform_filter(handle, weights, w_scratch);
             weights = w_scratch;
         }
+
         bool fused = conv_bias || conv_bias_eltwise;
+
+        float scale = 1.0f;
+        if (src_scale || wei_scale) {
+            if (src_scale) {
+                float host_src_scale = 1.0f;
+                CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_src_scale,
+                        (CUdeviceptr)src_scale, sizeof(float));
+                scale *= host_src_scale;
+            }
+            if (wei_scale) {
+                float host_wei_scale = 1.0f;
+                CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_wei_scale,
+                        (CUdeviceptr)wei_scale, sizeof(float));
+                scale *= host_wei_scale;
+            }
+        }
+
         if (fused) {
-            auto err = cudnnConvolutionBiasActivationForward(handle, &alpha,
+            auto err = cudnnConvolutionBiasActivationForward(handle, &scale,
                     descs[io::x], x, weights_desc, weights, conv_desc,
                     fwd_alg_kind, scratchpad, scratchpad_size, &beta,
                     descs[io::y], output, descs[io::bias], bias,
@@ -496,7 +536,7 @@ public:
         if (!fused) {
             const float bias_alpha = 1.0f;
             const float bias_beta = 1.0f;
-            CUDNN_EXECUTE_FUNC_V(cudnnConvolutionForward, handle, &alpha,
+            CUDNN_EXECUTE_FUNC_V(cudnnConvolutionForward, handle, &scale,
                     descs[io::x], x, weights_desc, weights, conv_desc,
                     fwd_alg_kind, scratchpad, scratchpad_size, &beta,
                     descs[io::y], output);
@@ -506,7 +546,6 @@ public:
                         output);
             }
         }
-        execute_scale(handle, output);
         // skip first eltwise in case it is fused into convolution
         const int post_ops_start_pos = fused && conv_bias_eltwise;
         for (int i = post_ops_start_pos; i < num_post_ops; i++) {
@@ -541,6 +580,15 @@ public:
         if (need_reorder) {
             execute_reorder(handle, post_op_scratch, y, false);
         }
+
+        if (dst_scale) {
+            float host_dst_scale = 1.0f;
+            CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_dst_scale,
+                    (CUdeviceptr)dst_scale, sizeof(float));
+            float inv_scale = 1.0f / host_dst_scale;
+            CUDNN_EXECUTE_FUNC(
+                    cudnnScaleTensor, handle, descs[io::y], y, &inv_scale);
+        }
     }
     status_t init_scratchpad(engine_t *engine, convolution_pd_t *pd) override {
         auto &sycl_engine = *utils::downcast<sycl_cuda_engine_t *>(engine);
@@ -579,6 +627,8 @@ public:
                 descs[x], weights_desc, conv_desc, descs[y],
                 requested_algo_count, &returned_algo_count, perf.data()));
 
+        dnnl_fpmath_mode_t dnnl_fpmath_mode = pd->attr()->fpmath_mode_;
+
         auto submit_status = CUDNN_STATUS_NOT_SUPPORTED;
         for (size_t i = 0; i < returned_algo_count; i++) {
             submit_status = perf[i].status;
@@ -620,6 +670,10 @@ public:
                         break;
                     default: return status::unimplemented;
                 }
+                if ((computation_data_type == CUDNN_DATA_FLOAT)
+                        && (dnnl_fpmath_mode == dnnl_fpmath_mode_strict)
+                        && (perf[i].mathType != CUDNN_FMA_MATH))
+                    continue;
                 fwd_alg_kind = perf[i].algo;
                 CHECK(CUDNN_EXECUTE_FUNC_S(cudnnSetConvolutionMathType,
                         conv_desc, perf[i].mathType));
@@ -656,9 +710,6 @@ public:
             case alg_kind::eltwise_logistic:
                 act_mode = CUDNN_ACTIVATION_SIGMOID;
                 break;
-            case alg_kind::eltwise_bounded_relu:
-                act_mode = CUDNN_ACTIVATION_CLIPPED_RELU;
-                break;
             default: return status::unimplemented;
         }
         CHECK(CUDNN_EXECUTE_FUNC_S(cudnnSetActivationDescriptor, eltwise_desc,
@@ -707,6 +758,9 @@ protected:
         CHECK(CUDNN_EXECUTE_FUNC_S(cudnnFindConvolutionBackwardDataAlgorithm,
                 handle, weights_desc, descs[y], conv_desc, descs[x],
                 requested_algo_count, &returned_algo_count, perf.data()));
+
+        dnnl_fpmath_mode_t dnnl_fpmath_mode = pd->attr()->fpmath_mode_;
+
         for (size_t i = 0; i < returned_algo_count; i++) {
             if (perf[i].status == CUDNN_STATUS_SUCCESS) {
                 switch (pd->desc()->alg_kind) {
@@ -737,6 +791,10 @@ protected:
                         break;
                     default: return status::unimplemented;
                 }
+                if ((computation_data_type == CUDNN_DATA_FLOAT)
+                        && (dnnl_fpmath_mode == dnnl_fpmath_mode_strict)
+                        && (perf[i].mathType != CUDNN_FMA_MATH))
+                    continue;
                 bwd_algo = perf[i].algo;
                 CHECK(CUDNN_EXECUTE_FUNC_S(cudnnSetConvolutionMathType,
                         conv_desc, perf[i].mathType));
@@ -778,6 +836,7 @@ protected:
             transform_filter(handle, weights, w_scratch);
             weights = w_scratch;
         }
+        const float alpha = 1.0f;
         const float bias_alpha = 1.0f;
         const float bias_beta = 1.0f;
         CUDNN_EXECUTE_FUNC_V(cudnnConvolutionBackwardData, handle, &alpha,
@@ -849,6 +908,9 @@ public:
         CHECK(CUDNN_EXECUTE_FUNC_S(cudnnFindConvolutionBackwardFilterAlgorithm,
                 handle, descs[x], descs[y], conv_desc, weights_desc,
                 requested_algo_count, &returned_algo_count, perf.data()));
+
+        dnnl_fpmath_mode_t dnnl_fpmath_mode = pd->attr()->fpmath_mode_;
+
         for (size_t i = 0; i < returned_algo_count; i++) {
             if (perf[i].status == CUDNN_STATUS_SUCCESS) {
                 switch (pd->desc()->alg_kind) {
@@ -881,6 +943,10 @@ public:
                         break;
                     default: return status::unimplemented;
                 }
+                if ((computation_data_type == CUDNN_DATA_FLOAT)
+                        && (dnnl_fpmath_mode == dnnl_fpmath_mode_strict)
+                        && (perf[i].mathType != CUDNN_FMA_MATH))
+                    continue;
                 bwd_filter_algo = perf[i].algo;
                 CHECK(CUDNN_EXECUTE_FUNC_S(cudnnSetConvolutionMathType,
                         conv_desc, perf[i].mathType));
@@ -924,6 +990,7 @@ public:
             transform_filter(handle, weights, w_scratch);
             filter = w_scratch;
         }
+        const float alpha = 1.0f;
         const float bias_alpha = 1.0f;
         const float bias_beta = 0.0f;
         CUDNN_EXECUTE_FUNC_V(cudnnConvolutionBackwardFilter, handle, &alpha,
