@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2022 Intel Corporation
 * Copyright 2020 Codeplay Software Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,7 +22,7 @@
 
 #include "common/c_types_map.hpp"
 #include "common/deconvolution_pd.hpp"
-#include "common/primitive_iterator.hpp"
+#include "common/primitive_desc_iterator.hpp"
 #include "gpu/nvidia/cudnn_convolution.hpp"
 #include "gpu/nvidia/cudnn_deconvolution_impl.hpp"
 
@@ -132,7 +132,7 @@ struct cudnn_deconvolution_fwd_t : public primitive_t {
             convolution_desc_t cd;
             CHECK(conv_descr_create(desc(), &cd));
             primitive_attr_t conv_attr = *attr();
-            dnnl_primitive_desc_iterator it(
+            primitive_desc_iterator_t it(
                     engine, (op_desc_t *)&cd, &conv_attr, nullptr);
             while (++it != it.end()) {
                 conv_pd_ = *it;
@@ -261,7 +261,7 @@ struct cudnn_deconvolution_bwd_data_t : public primitive_t {
             convolution_desc_t cd;
             CHECK(conv_descr_create(desc(), &cd));
             primitive_attr_t conv_attr = *attr();
-            dnnl_primitive_desc_iterator it(
+            primitive_desc_iterator_t it(
                     engine, (op_desc_t *)&cd, &conv_attr, nullptr);
             while (++it != it.end()) {
                 conv_pd_ = *it;
@@ -361,7 +361,7 @@ struct cudnn_deconvolution_bwd_weights_t : public primitive_t {
             convolution_desc_t cd;
             CHECK(conv_descr_create(desc(), &cd));
             primitive_attr_t conv_attr = *attr();
-            dnnl_primitive_desc_iterator it(
+            primitive_desc_iterator_t it(
                     engine, (op_desc_t *)&cd, &conv_attr, nullptr);
             while (++it != it.end()) {
                 conv_pd_ = *it;
