@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2021 Intel Corporation
+* Copyright 2020-2023 Intel Corporation
 * Copyright 2020 Codeplay Software Limited
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
@@ -77,9 +77,6 @@ struct cudnn_matmul_impl_t {
             case alg_kind::eltwise_relu:
                 mode = cudnnActivationMode_t::CUDNN_ACTIVATION_RELU;
                 break;
-            case alg_kind::eltwise_bounded_relu:
-                mode = cudnnActivationMode_t::CUDNN_ACTIVATION_CLIPPED_RELU;
-                break;
             case alg_kind::eltwise_tanh:
                 mode = cudnnActivationMode_t::CUDNN_ACTIVATION_TANH;
                 break;
@@ -115,6 +112,8 @@ struct cudnn_matmul_impl_t {
         memory_desc_wrapper weights_d = memory_desc_wrapper(pd->weights_md());
         memory_desc_wrapper dst_d = memory_desc_wrapper(pd->dst_md());
 
+        with_dst_scale_
+                = !pd->attr()->scales_.get(DNNL_ARG_DST).has_default_values();
         with_bias_ = pd->with_bias();
         if ((with_bias_)
                 && (pd->weights_md(1)->data_type != pd->dst_md()->data_type)) {
@@ -137,7 +136,7 @@ struct cudnn_matmul_impl_t {
 
         if (with_eltwise(0, pd) || with_eltwise(1, pd)) {
             with_eltwise_ = true;
-            create_and_set_op_descriptor(pd);
+            CHECK(create_and_set_op_descriptor(pd));
         }
 
         // Set parameter when post-op sum is specified
@@ -190,21 +189,24 @@ struct cudnn_matmul_impl_t {
         }
     }
 
+    int get_ld(const memory_desc_wrapper desc, cublasOperation_t trans) {
+        const int ndims = desc.ndims();
+        const auto *strides = &desc.blocking_desc().strides[ndims - 2];
+        const int ld = strides[trans == cublasOperation_t::CUBLAS_OP_N ? 0 : 1];
+        return ld;
+    }
+
+    int get_batch_stride(const memory_desc_wrapper desc) {
+        auto dims = desc.dims();
+        auto strides = desc.blocking_desc().strides;
+        return dims[0] == 1 ? 0 : strides[0];
+    }
+
     status_t init_gemm_parameters(const memory_desc_wrapper src_d,
             const memory_desc_wrapper weights_d,
             const memory_desc_wrapper dst_d) {
-        const auto &dst_bd = dst_d.blocking_desc();
-
-        if (isbatched_) {
-            // Check that user didn't provide broadcast case. See pd_t::init()
-            // for details.
-            const auto src_batch = src_d.dims()[0];
-            const auto wei_batch = weights_d.dims()[0];
-            if (src_batch != wei_batch) return status::runtime_error;
-
-            batch_count_ = dst_d.dims()[0];
-        }
 
+        if (isbatched_) batch_count_ = dst_d.dims()[0];
         const dim_t M = dst_d.dims()[isbatched_ + 1];
         const dim_t N = dst_d.dims()[isbatched_ + 0];
         const dim_t K = src_d.dims()[isbatched_ + 1];
@@ -213,6 +215,7 @@ struct cudnn_matmul_impl_t {
         N_ = (int)N;
         K_ = (int)K;
 
+        const auto &dst_strides = &dst_d.blocking_desc().strides[isbatched_];
         const auto &src_strides = &src_d.blocking_desc().strides[isbatched_];
         const auto &weights_strides
                 = &weights_d.blocking_desc().strides[isbatched_];
@@ -226,21 +229,26 @@ struct cudnn_matmul_impl_t {
         transB_ = src_strides[1] == 1 && src_d.dims()[isbatched_ + 0] > 1
                 ? cublasOperation_t::CUBLAS_OP_N
                 : cublasOperation_t::CUBLAS_OP_T;
+        // C matrix is the dst
+        transC_ = dst_strides[1] == 1 && dst_d.dims()[isbatched_ + 0] > 1
+                ? cublasOperation_t::CUBLAS_OP_N
+                : cublasOperation_t::CUBLAS_OP_T;
 
-        lda_ = (int)
-                weights_strides[transA_ == cublasOperation_t::CUBLAS_OP_N ? 0
-                                                                          : 1];
-        ldb_ = (int)
-                src_strides[transB_ == cublasOperation_t::CUBLAS_OP_N ? 0 : 1];
-        ldc_ = (int)dst_bd.strides[isbatched_ + 0];
+        lda_ = get_ld(weights_d, transA_);
+        ldb_ = get_ld(src_d, transB_);
+        ldc_ = get_ld(dst_d, transC_);
 
         if (isbatched_) {
             // These parameters are required for cublasGemmStridedBatchedEx()
-            stride_a_ = (transA_ == cublasOperation_t::CUBLAS_OP_N) ? lda_ * K_
-                                                                    : lda_ * M_;
-            stride_b_ = (transB_ == cublasOperation_t::CUBLAS_OP_N) ? ldb_ * N_
-                                                                    : ldb_ * K_;
-            stride_c_ = ldc_ * N_;
+            stride_a_ = get_batch_stride(weights_d);
+            stride_b_ = get_batch_stride(src_d);
+            stride_c_ = get_batch_stride(dst_d);
+
+            // Enable broadcast semantics.
+            if (src_d.dims()[0] > weights_d.dims()[0])
+                stride_a_ = 0;
+            else if (src_d.dims()[0] < weights_d.dims()[0])
+                stride_b_ = 0;
         }
 
         return status::success;
@@ -253,7 +261,8 @@ struct cudnn_matmul_impl_t {
         // We need to initialize them in the execute function.
         CHECK(init_gemm_parameters(src_d, weights_d, dst_d));
 
-        if (with_bias_ || reorder_required_ || with_eltwise_) {
+        if (with_bias_ || reorder_required_ || with_eltwise_
+                || with_dst_scale_) {
             // Initialise cuDNN descriptors
             cudnnDataType_t data_types[NUM_IO];
             int ndims = dst_d.ndims() < 4 ? 4 : dst_d.ndims();
@@ -298,7 +307,7 @@ struct cudnn_matmul_impl_t {
 
     void execute(cublasHandle_t cublas_handle, cudnnHandle_t cudnn_handle,
             void *a, void *b, void *c, void *bias, void *scratch,
-            const float scales) {
+            void *src_scale, void *wei_scale, void *dst_scale) {
         float gemm_beta = 0;
         if (!bias_dt_mismatch_ && !reorder_required_) {
             // Case where no reorder is required, scratchpad points to dst (c)
@@ -306,24 +315,62 @@ struct cudnn_matmul_impl_t {
             temp_mem_desc_ = tensor_descs_[io::dst];
             gemm_beta = post_op_sum_;
         }
+        auto flip_op = [](cublasOperation_t op) {
+            return (op == cublasOperation_t::CUBLAS_OP_T)
+                    ? cublasOperation_t::CUBLAS_OP_N
+                    : cublasOperation_t::CUBLAS_OP_T;
+        };
+
+        float scale = 1.0f;
+        if (src_scale || wei_scale) {
+            if (src_scale) {
+                float host_src_scale = 1.0f;
+                CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_src_scale,
+                        (CUdeviceptr)src_scale, sizeof(float));
+                scale *= host_src_scale;
+            }
+            if (wei_scale) {
+                float host_wei_scale = 1.0f;
+                CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_wei_scale,
+                        (CUdeviceptr)wei_scale, sizeof(float));
+                scale *= host_wei_scale;
+            }
+        }
+
         if (isbatched_) {
             // Calls cublasGemmStridedBatchedEx()
-            CUBLAS_EXECUTE_FUNC(cublasGemmStridedBatchedEx, cublas_handle,
-                    transA_, transB_, M_, N_, K_, &scales, a, weights_type_,
-                    lda_, stride_a_, b, src_type_, ldb_, stride_b_, &gemm_beta,
-                    scratch, dst_type_, ldc_, stride_c_, batch_count_,
-                    acc_type_, gemm_algo_);
+            if (transC_ == cublasOperation_t::CUBLAS_OP_T) {
+                CUBLAS_EXECUTE_FUNC(cublasGemmStridedBatchedEx, cublas_handle,
+                        flip_op(transB_), flip_op(transA_), N_, M_, K_, &scale,
+                        b, src_type_, ldb_, stride_b_, a, weights_type_, lda_,
+                        stride_a_, &gemm_beta, scratch, dst_type_, ldc_,
+                        stride_c_, batch_count_, acc_type_, gemm_algo_);
+
+            } else {
+                CUBLAS_EXECUTE_FUNC(cublasGemmStridedBatchedEx, cublas_handle,
+                        transA_, transB_, M_, N_, K_, &scale, a, weights_type_,
+                        lda_, stride_a_, b, src_type_, ldb_, stride_b_,
+                        &gemm_beta, scratch, dst_type_, ldc_, stride_c_,
+                        batch_count_, acc_type_, gemm_algo_);
+            }
         } else {
             // Calls cublasGemmEx()
-            CUBLAS_EXECUTE_FUNC(cublasGemmEx, cublas_handle, transA_, transB_,
-                    M_, N_, K_, &scales, a, weights_type_, lda_, b, src_type_,
-                    ldb_, &gemm_beta, scratch, dst_type_, ldc_, acc_type_,
-                    gemm_algo_);
+            if (transC_ == cublasOperation_t::CUBLAS_OP_T) {
+                CUBLAS_EXECUTE_FUNC(cublasGemmEx, cublas_handle,
+                        flip_op(transB_), flip_op(transA_), N_, M_, K_, &scale,
+                        b, src_type_, ldb_, a, weights_type_, lda_, &gemm_beta,
+                        scratch, dst_type_, ldc_, acc_type_, gemm_algo_);
+            } else {
+                CUBLAS_EXECUTE_FUNC(cublasGemmEx, cublas_handle, transA_,
+                        transB_, M_, N_, K_, &scale, a, weights_type_, lda_, b,
+                        src_type_, ldb_, &gemm_beta, scratch, dst_type_, ldc_,
+                        acc_type_, gemm_algo_);
+            }
         }
         if (with_bias_) {
             // When bias is specified call cudnnAddTensor()
             float bias_beta = 1;
-            CUDNN_EXECUTE_FUNC(cudnnAddTensor, cudnn_handle, &scales,
+            CUDNN_EXECUTE_FUNC(cudnnAddTensor, cudnn_handle, &scale,
                     tensor_descs_[io::bias], bias, &bias_beta, temp_mem_desc_,
                     scratch);
         }
@@ -342,6 +389,15 @@ struct cudnn_matmul_impl_t {
                     &reorder_alpha, temp_mem_desc_, scratch, &post_op_sum_,
                     tensor_descs_[io::dst], c);
         }
+
+        if (dst_scale) {
+            float host_dst_scale = 1.0f;
+            CUDA_EXECUTE_FUNC(cuMemcpy, (CUdeviceptr)&host_dst_scale,
+                    (CUdeviceptr)dst_scale, sizeof(float));
+            float inv_scale = 1.0f / host_dst_scale;
+            CUDNN_EXECUTE_FUNC(cudnnScaleTensor, cudnn_handle,
+                    tensor_descs_[io::dst], c, &inv_scale);
+        }
     }
 
     ~cudnn_matmul_impl_t() { cleanup(); }
@@ -384,10 +440,12 @@ private:
     }
     cublasOperation_t transA_;
     cublasOperation_t transB_;
+    cublasOperation_t transC_;
     int M_, N_, K_;
     int lda_, ldb_, ldc_;
     long long int stride_a_, stride_b_, stride_c_;
-    bool isbatched_ = false, with_bias_ = false, bias_dt_mismatch_ = false;
+    bool isbatched_ = false, with_bias_ = false, bias_dt_mismatch_ = false,
+         with_dst_scale_ = false;
     bool reorder_required_ = false, with_eltwise_ = false;
     bool with_scratchpad_ = false, has_runtime_params_ = false;
     dnnl_data_type_t scratchpad_type_;
