@@ -29,25 +29,6 @@ namespace gpu {
 namespace nvidia {
 namespace compat {
 
-#if DNNL_USE_SYCL121_API
-using interop_handle = ::sycl::interop_handler;
-template <typename T, typename U>
-T get_native_mem(const interop_handle &ih, U acc) {
-    return reinterpret_cast<T>(ih.get_mem<::sycl::backend::cuda>(acc));
-}
-
-template <typename T>
-void host_task(::sycl::handler &cgh, const T &task) {
-    cgh.interop_task(task);
-}
-
-template <typename native_object_t, typename sycl_object_t>
-native_object_t get_native(const sycl_object_t &sycl_object) {
-    auto handle = sycl_object.template get_native<::sycl::backend::cuda>();
-    return reinterpret_cast<native_object_t>(handle);
-}
-
-#else
 using interop_handle = ::sycl::interop_handle;
 template <typename T, typename U>
 T get_native_mem(const interop_handle &ih, U acc) {
@@ -66,7 +47,6 @@ native_object_t get_native(const sycl_object_t &sycl_object) {
             = ::sycl::get_native<::sycl::backend::ext_oneapi_cuda>(sycl_object);
     return reinterpret_cast<native_object_t>(handle);
 }
-#endif
 
 } // namespace compat
 } // namespace nvidia
