@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2022 Intel Corporation
+* Copyright 2021-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -50,8 +50,7 @@ static status_t init_conf_common(pool_conf_t &conf, offsets_t &off,
     set_default_pool_conf(conf, *pd->desc(), *pd->invariant_src_md(),
             *pd->invariant_dst_md(), *pd->attr());
 
-    if (conf.id != conf.kd || conf.iw != conf.kw || conf.ih != conf.kh
-            || conf.od * conf.ow * conf.oh != 1)
+    if (conf.iw != conf.kw || conf.ih != conf.kh || conf.ow * conf.oh != 1)
         return status::unimplemented;
 
     const memory_desc_wrapper src_mdw(pd->invariant_src_md());
@@ -63,10 +62,14 @@ static status_t init_conf_common(pool_conf_t &conf, offsets_t &off,
             || utils::array_product(padded_dst_dims + 2, conf.ndims - 2)
                     != conf.od * conf.oh * conf.ow)
         return status::unimplemented;
+
+    using namespace dnnl::impl::alg_kind;
     if (!conf.is_backward) {
-        // gen9_global_pooling_fwd doesn't support zero padding.
-        if (conf.mb != conf.mb_padded || conf.c != conf.c_padded)
-            return status::unimplemented;
+        if (conf.alg == pooling_max) {
+            // gen9_global_pooling_fwd doesn't support zero padding.
+            if (conf.mb != conf.mb_padded || conf.c != conf.c_padded)
+                return status::unimplemented;
+        }
         // heuristics: for small shapes, gen9_pooling_fwd provides better perf.
         if (conf.kd * conf.kh * conf.kw < 128) return status::unimplemented;
     }
@@ -121,7 +124,7 @@ static status_t init_kernel_ctx_common(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int("NEED_ZERO_PADDING",
             (conf.mb != conf.mb_padded || conf.c != conf.c_padded));
 
-    def_attr_info(kernel_ctx, conf.attr_info, post_ops);
+    CHECK(def_attr_info(kernel_ctx, conf.attr_info, post_ops));
 
     def_offsets(off.src_off, kernel_ctx, "SRC", conf.ndims);
     def_offsets(off.dst_off, kernel_ctx, "DST", conf.ndims);
@@ -143,20 +146,46 @@ status_t gen9_global_pooling_fwd_t::pd_t::init_kernel_ctx(
     return init_kernel_ctx_common(kernel_ctx, conf, off, attr()->post_ops_);
 }
 
+void gen9_global_pooling_fwd_t::pd_t::init_scratchpad() {
+    auto scratchpad = scratchpad_registry().registrar();
+    size_t size = utils::array_product(
+            conf.dst_md_info.padded_dims, conf.dst_md_info.ndims);
+    scratchpad.book(memory_tracking::names::key_pool_reduction, size,
+            types::data_type_size(data_type::f32), OCL_BUFFER_ALIGNMENT);
+
+    scratchpad.book(memory_tracking::names::key_nested,
+            reduction_pd_->scratchpad_registry());
+}
+
 status_t gen9_global_pooling_fwd_t::execute_forward(
         const exec_ctx_t &ctx) const {
-    auto &src = CTX_IN_STORAGE(DNNL_ARG_SRC);
-    auto &dst = CTX_OUT_STORAGE(DNNL_ARG_DST);
-    auto &ws = CTX_OUT_STORAGE(DNNL_ARG_WORKSPACE);
+    if (!reduction_p_) {
+        auto &src = CTX_IN_STORAGE(DNNL_ARG_SRC);
+        auto &dst = CTX_OUT_STORAGE(DNNL_ARG_DST);
+        auto &ws = CTX_OUT_STORAGE(DNNL_ARG_WORKSPACE);
 
-    compute::kernel_arg_list_t arg_list;
-    arg_list.set(0, src);
-    arg_list.set(1, ws);
-    arg_list.set(2, dst);
+        compute::kernel_arg_list_t arg_list;
+        arg_list.set(0, src);
+        arg_list.set(1, ws);
+        arg_list.set(2, dst);
 
-    auto nd_range = pd()->conf.dispatch.nd_range();
+        auto nd_range = pd()->conf.dispatch.nd_range();
 
-    return parallel_for(ctx, nd_range, kernel_, arg_list);
+        return parallel_for(ctx, nd_range, kernel_, arg_list);
+    } else {
+
+        exec_args_t reduction_args;
+        reduction_args[DNNL_ARG_SRC] = ctx.args().at(DNNL_ARG_SRC);
+        reduction_args[DNNL_ARG_DST] = ctx.args().at(DNNL_ARG_DST);
+        exec_ctx_t reduction_ctx(ctx, std::move(reduction_args));
+
+        nested_scratchpad_t ns(
+                ctx, memory_tracking::names::key_nested, reduction_p_);
+        reduction_ctx.set_scratchpad_grantor(ns.grantor());
+
+        // Executing the reduction kernel
+        return reduction_p_->execute(reduction_ctx);
+    }
 }
 
 status_t gen9_global_pooling_bwd_t::pd_t::init_conf(engine_t *engine) {
