@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2022 Intel Corporation
+* Copyright 2021-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,6 +32,7 @@ namespace impl {
 namespace gpu {
 namespace ocl {
 
+// Extract N and C block sizes from blk.inner_blks
 std::pair<int, int> get_n_c_block_sizes(const memory_desc_wrapper &mdw) {
     int n_block_size = 1;
     int c_block_size = 1;
@@ -84,15 +85,118 @@ std::pair<int, int> get_initial_n_split(const int n, const bool is_n_reduced) {
     return std::make_pair(initial_n_chunk_size, initial_n_chunks_num);
 }
 
+// Used by other implementations to check if the given problem can use the
+// gen9 implementation without creating a new pd
+bool gen9_reduction_t::is_compatible(const memory_desc_wrapper &src_mdw,
+        const memory_desc_wrapper &dst_mdw, reduction_conf_t *conf) {
+    const int ndims = src_mdw.ndims();
+    const dnnl_dim_t *src_dims = src_mdw.md_->dims;
+    const dnnl_dim_t *dst_dims = dst_mdw.md_->dims;
+
+    using namespace dnnl::impl::format_tag;
+    const bool is_nhwc = (src_mdw.matches_one_of_tag(nwc, nhwc, ndhwc)
+            != format_tag::undef);
+
+    // Last blocked dim is C and it has blockSize size
+    auto is_c_blocked_by
+            = [](const memory_desc_wrapper &mdw, const int blockSize) {
+                  const blocking_desc_t &blk = mdw.blocking_desc();
+                  if (blk.inner_nblks == 0) return false;
+                  return (blk.inner_idxs[blk.inner_nblks - 1] == 1)
+                          && (blk.inner_blks[blk.inner_nblks - 1] == blockSize);
+              };
+
+    // plain layouts: NHWC, src C must be divisible by 16.
+    if (is_nhwc) {
+        int c = src_dims[1];
+        if (c % 16 != 0) { return false; }
+        if (src_dims[0] != dst_dims[0]) { return false; }
+        if (src_dims[1] != dst_dims[1]) { return false; }
+    } else {
+        // blocked layouts: IC/OC must have blocks of 16 or 32
+        if (!(is_c_blocked_by(src_mdw, 16) || is_c_blocked_by(src_mdw, 32)))
+            return false;
+        if (!(is_c_blocked_by(dst_mdw, 16) || is_c_blocked_by(dst_mdw, 32)))
+            return false;
+    }
+
+    int src_n_block_size, src_c_block_size;
+    int dst_n_block_size, dst_c_block_size;
+    std::tie(src_n_block_size, src_c_block_size) = get_n_c_block_sizes(src_mdw);
+    std::tie(dst_n_block_size, dst_c_block_size) = get_n_c_block_sizes(dst_mdw);
+
+    // src/dst blocking must match
+    if (src_n_block_size != dst_n_block_size
+            || src_c_block_size != dst_c_block_size
+            || src_mdw.blocking_desc().inner_nblks
+                    != dst_mdw.blocking_desc().inner_nblks)
+        return false;
+
+    // Either 0th/1st dims blocked or just 1st dim blocked
+    if ((src_n_block_size == 1 && src_mdw.blocking_desc().inner_nblks > 1)
+            || src_mdw.blocking_desc().inner_nblks > 2) {
+        return false;
+    }
+
+    int div = 1;
+    int hwd_size = 1;
+    int hwd_reduction_size = 1;
+    dims_t reduce_dims;
+    bool is_reduction_dim[MAX_NDIMS];
+    for (int d = 0; d < ndims; d++) {
+        reduce_dims[d] = dim_t {1};
+        is_reduction_dim[d] = src_dims[d] != dst_dims[d];
+
+        if (is_reduction_dim[d]) {
+            reduce_dims[d] = src_dims[d];
+            div *= reduce_dims[d];
+        }
+        if (d >= 2) {
+            hwd_size *= src_dims[d];
+            hwd_reduction_size *= reduce_dims[d];
+        }
+    }
+
+    // If any spatial dims are reduced, they all have to be.
+    if (hwd_size != hwd_reduction_size && hwd_reduction_size > 1) {
+        return false;
+    }
+
+    // full padded C must be a multiple of 16 -- redundant due to blocking?
+    int sub_group_size = 16;
+    const auto &src_padded_dims = src_mdw.padded_dims();
+    if (src_padded_dims[1] % sub_group_size != 0) return false;
+
+    // If supplied with conf, save side effects/relevant variables
+    if (conf != nullptr) {
+        conf->n_block_size = src_n_block_size;
+        conf->c_block_size = is_nhwc ? src_dims[1] : src_c_block_size;
+        conf->div = div;
+        conf->hwd_reduction_size = hwd_reduction_size;
+        conf->hwd_size = hwd_size;
+        conf->sub_group_size = sub_group_size;
+        for (int d = 0; d < ndims; d++) {
+            conf->src_dims[d] = src_dims[d];
+            conf->dst_dims[d] = dst_dims[d];
+            conf->is_reduction_dim[d] = is_reduction_dim[d];
+            conf->reduce_dims[d] = reduce_dims[d];
+        }
+    }
+
+    return true;
+}
+
 status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
     const reduction_pd_t *pd = this;
 
     const memory_desc_wrapper src_mdw(pd->src_md());
     const memory_desc_wrapper dst_mdw(pd->dst_md());
 
+    if (!is_compatible(src_mdw, dst_mdw, &conf)) {
+        return status::unimplemented;
+    }
+
     const int ndims = src_mdw.ndims();
-    const dnnl_dim_t *src_dims = src_mdw.md_->dims;
-    const dnnl_dim_t *dst_dims = dst_mdw.md_->dims;
     const compute::compute_engine_t *compute_engine
             = utils::downcast<compute::compute_engine_t *>(engine);
     const int num_threads = compute_engine->device_info()->hw_threads();
@@ -106,67 +210,12 @@ status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
     conf.power = pd->desc()->p;
     conf.eps = pd->desc()->eps;
     conf.dispatch = compute_engine->create_dispatch(src_mdw.md_);
-    conf.finilize_dispatch = compute_engine->create_dispatch();
 
-    auto is_c_blocked_by
-            = [](const memory_desc_wrapper &mdw, const int blockSize) {
-                  const blocking_desc_t &blk = mdw.blocking_desc();
-                  if (blk.inner_nblks == 0) return false;
-                  return (blk.inner_idxs[blk.inner_nblks - 1] == 1)
-                          && (blk.inner_blks[blk.inner_nblks - 1] == blockSize);
-              };
-
-    if (!(is_c_blocked_by(src_mdw, 16) || is_c_blocked_by(src_mdw, 32)))
-        return status::unimplemented;
-
-    int src_n_block_size, src_c_block_size;
-    int dst_n_block_size, dst_c_block_size;
-    std::tie(src_n_block_size, src_c_block_size) = get_n_c_block_sizes(src_mdw);
-    std::tie(dst_n_block_size, dst_c_block_size) = get_n_c_block_sizes(dst_mdw);
-    if (src_n_block_size != dst_n_block_size
-            || src_c_block_size != dst_c_block_size
-            || src_mdw.blocking_desc().inner_nblks
-                    != dst_mdw.blocking_desc().inner_nblks)
-        return status::unimplemented;
-
-    conf.n_block_size = src_n_block_size;
-    conf.c_block_size = src_c_block_size;
-    if ((conf.n_block_size == 1 && src_mdw.blocking_desc().inner_nblks > 1)
-            || src_mdw.blocking_desc().inner_nblks > 2) {
-        return status::unimplemented;
-    }
-
-    conf.div = 1;
-    int hwd_size = 1;
-    int hwd_reduction_size = 1;
-    for (int d = 0; d < ndims; d++) {
-        conf.src_dims[d] = src_dims[d];
-        conf.reduce_dims[d] = conf.dst_dims[d] = dim_t {1};
-        conf.is_reduction_dim[d] = conf.src_dims[d] != dst_dims[d];
-
-        if (conf.is_reduction_dim[d]) {
-            conf.reduce_dims[d] = conf.src_dims[d];
-            conf.div *= conf.reduce_dims[d];
-        } else {
-            conf.dst_dims[d] = conf.src_dims[d];
-        }
-        if (d >= 2) {
-            hwd_size *= conf.src_dims[d];
-            hwd_reduction_size *= conf.reduce_dims[d];
-        }
-    }
-    if (hwd_size != hwd_reduction_size && hwd_reduction_size > 1) {
-        return status::unimplemented;
-    }
-
-    conf.sub_group_size = 16;
-    if (conf.src_dims[1] % conf.sub_group_size != 0) {
-        return status_t::dnnl_unimplemented;
-    }
+    // number of C chunks in dim 1
     conf.initial_c_chunks
-            = std::min(static_cast<int>(conf.src_dims[1]) / conf.sub_group_size,
-                    conf.c_block_size / conf.sub_group_size);
+            = std::min(conf.c_block_size / conf.sub_group_size, 8);
 
+    // Split N chunks/chunk size according to heuristic
     std::tie(conf.initial_n_chunk_size, conf.initial_n_chunks)
             = get_initial_n_split(conf.src_dims[0], conf.is_reduction_dim[0]);
 
@@ -174,22 +223,25 @@ status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
         return conf.initial_n_chunk_size * conf.initial_c_chunks
                 * conf.initial_hwd_chunk_size;
     };
+
+    // Number of chunks of hwd to reduce
     const auto get_wi_per_hwd = [this]() {
         return ceil(static_cast<float>(conf.initial_hwd_dim)
                 / conf.initial_hwd_chunk_size);
     };
+
     const auto get_used_threads_num = [this, get_wi_per_hwd]() {
         return conf.initial_n_chunks * conf.src_dims[1]
                 / (conf.sub_group_size * conf.initial_c_chunks)
                 * get_wi_per_hwd();
     };
 
-    if (hwd_reduction_size == 1) {
+    if (conf.hwd_reduction_size == 1) {
         conf.initial_hwd_chunk_size = 1;
         // If there is no HWD reduction use vectors only to read whole C block
         conf.vector_size = conf.initial_c_chunks;
-        conf.initial_hwd_dim = hwd_size;
-        conf.final_hwd_dim = hwd_size;
+        conf.initial_hwd_dim = conf.hwd_size;
+        conf.final_hwd_dim = conf.hwd_size;
         conf.final_hwd_chunk_size = 1;
     } else {
         // Start with such constant and try to adjust that with heuristics
@@ -199,7 +251,7 @@ status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
         } else {
             conf.vector_size = 8;
         }
-        conf.initial_hwd_dim = hwd_reduction_size;
+        conf.initial_hwd_dim = conf.hwd_reduction_size;
 
         // Experimentally selected values
         constexpr int min_elems_per_wi = 64;
@@ -211,6 +263,7 @@ status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
                 && get_wi_per_hwd() < max_wi_per_hwd) {
             conf.initial_hwd_chunk_size /= 2;
         }
+
         while ((get_used_threads_num() > min_threads
                        && get_reduction_elems_per_wi() < min_elems_per_wi)
                 || get_wi_per_hwd() > max_wi_per_hwd) {
@@ -224,11 +277,12 @@ status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
         conf.final_hwd_chunk_size = conf.final_hwd_dim;
     }
 
+    const auto &src_padded_dims = src_mdw.padded_dims();
     conf.final_c_dim = conf.is_reduction_dim[1]
-            ? conf.src_dims[1] / (conf.sub_group_size * conf.initial_c_chunks)
+            ? src_padded_dims[1] / (conf.sub_group_size * conf.initial_c_chunks)
             : conf.src_dims[1];
     conf.final_c_chunk_size = conf.is_reduction_dim[1]
-            ? conf.src_dims[1] / (conf.sub_group_size * conf.initial_c_chunks)
+            ? src_padded_dims[1] / (conf.sub_group_size * conf.initial_c_chunks)
             : 1;
 
     conf.final_n_dim = conf.is_reduction_dim[0] ? conf.initial_n_chunks
@@ -237,6 +291,7 @@ status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
             = conf.is_reduction_dim[0] ? conf.initial_n_chunks : 1;
 
     int initial_n_chunks_padded, initial_c_padded;
+
     if (conf.final_c_chunk_size == 1 && conf.final_n_chunk_size == 1
             && conf.final_hwd_chunk_size == 1) {
         conf.skip_final_phase = true;
@@ -256,50 +311,64 @@ status_t gen9_reduction_t::pd_t::init_conf(engine_t *engine) {
             initial_c_padded, conf.initial_c_chunks);
     conf.dispatch.define_dim("INITIAL_HWD_CHUNK_ID", std::min(ndims - 1, 2),
             conf.final_hwd_dim, 1);
+
+    // Each initial kernel will handle 16 C channels
+    // Requires INITIAL_C (initial_c_padded) to be a multiple of sub_group_size
     CHECK(conf.dispatch.vectorize_dim("INITIAL_C", conf.sub_group_size));
     conf.dispatch.set_kernel_attr_suffix("INITIAL");
     conf.dispatch.generate();
+    conf.attr_info = attr_info_t::create(pd->attr());
 
-    const int final_n_padded
-            = utils::rnd_up(conf.final_n_dim, conf.n_block_size);
-    const int final_n_chunks_padded
-            = utils::div_up(final_n_padded, conf.final_n_chunk_size);
-    conf.finilize_dispatch.define_dim("FINAL_N", 0, final_n_chunks_padded);
-    const int final_c_padded
-            = utils::rnd_up(conf.final_c_dim, conf.c_block_size);
-    const int final_c_chunks_padded
-            = utils::div_up(final_c_padded, conf.final_c_chunk_size);
-    conf.finilize_dispatch.define_dim(
-            "FINAL_C", std::min(ndims - 1, 1), final_c_chunks_padded);
-    conf.finilize_dispatch.define_dim("FINAL_HWD", std::min(ndims - 1, 2),
-            conf.final_hwd_dim / conf.final_hwd_chunk_size);
-    conf.finilize_dispatch.set_kernel_attr_suffix("FINAL");
-    conf.finilize_dispatch.generate();
+    if (!conf.skip_final_phase) {
+        conf.finalize_dispatch = compute_engine->create_dispatch();
+        const int final_n_padded
+                = utils::rnd_up(conf.final_n_dim, conf.n_block_size);
+        const int final_n_chunks_padded
+                = utils::div_up(final_n_padded, conf.final_n_chunk_size);
+        conf.finalize_dispatch.define_dim("FINAL_N", 0, final_n_chunks_padded);
+        const int final_c_padded
+                = utils::rnd_up(conf.final_c_dim, conf.c_block_size);
+        const int final_c_chunks_padded
+                = utils::div_up(final_c_padded, conf.final_c_chunk_size);
+        conf.finalize_dispatch.define_dim(
+                "FINAL_C", std::min(ndims - 1, 1), final_c_chunks_padded);
+        conf.finalize_dispatch.define_dim("FINAL_HWD", std::min(ndims - 1, 2),
+                conf.final_hwd_dim / conf.final_hwd_chunk_size);
+        conf.finalize_dispatch.set_kernel_attr_suffix("FINAL");
+        conf.finalize_dispatch.generate();
+    }
 
     return status::success;
 }
 
-static status_t init_kernel_ctx_common(
-        compute::kernel_ctx_t &kernel_ctx, const reduction_conf_t &conf) {
+static status_t init_kernel_ctx_common(compute::kernel_ctx_t &kernel_ctx,
+        const reduction_conf_t &conf, const post_ops_t &post_ops) {
     using namespace alg_kind;
 
     kernel_ctx.set_data_type(conf.src_type);
 
+    // N shape descriptors
+    kernel_ctx.define_int("IS_N_REDUCED", conf.is_reduction_dim[0]);
     kernel_ctx.define_int("INITIAL_N", conf.src_dims[0]);
+    kernel_ctx.define_int("INITIAL_N_CHUNKS", conf.initial_n_chunks);
+    kernel_ctx.define_int("INITIAL_N_CHUNK_SIZE", conf.initial_n_chunk_size);
+    kernel_ctx.define_int("N_BLOCK_SIZE", conf.n_block_size);
+
+    // C shape descriptors
+    kernel_ctx.define_int("IS_C_REDUCED", conf.is_reduction_dim[1]);
     kernel_ctx.define_int("INITIAL_C", conf.src_dims[1]);
     kernel_ctx.define_int("INITIAL_C_CHUNKS", conf.initial_c_chunks);
-    kernel_ctx.define_int("INITIAL_N_CHUNKS", conf.initial_n_chunks);
-    kernel_ctx.define_int("SKIP_FINAL_PHASE", conf.skip_final_phase);
-    kernel_ctx.define_int("FINAL_N_DIM", conf.final_n_dim);
-    kernel_ctx.define_int("FINAL_N_CHUNK_SIZE", conf.final_n_chunk_size);
-    kernel_ctx.define_int("INITIAL_N_CHUNK_SIZE", conf.initial_n_chunk_size);
-    kernel_ctx.define_int("FINAL_C_DIM", conf.final_c_dim);
-    kernel_ctx.define_int("FINAL_C_CHUNK_SIZE", conf.final_c_chunk_size);
+    // No INITIAL_C_CHUNK_SIZE variable -- equal to SUB_GROUP_SIZE
+    kernel_ctx.define_int("C_BLOCK_SIZE", conf.c_block_size);
+
+    // Spatial shape descriptors
     kernel_ctx.define_int("INITIAL_HWD_DIM", conf.initial_hwd_dim);
-    kernel_ctx.define_int("FINAL_HWD_DIM", conf.final_hwd_dim);
     kernel_ctx.define_int(
             "INITIAL_HWD_CHUNK_SIZE", conf.initial_hwd_chunk_size);
-    kernel_ctx.define_int("FINAL_HWD_CHUNK_SIZE", conf.final_hwd_chunk_size);
+    kernel_ctx.define_int(
+            "IS_HWD_REDUCED", conf.final_hwd_dim < conf.initial_hwd_dim);
+
+    // DST shape descriptors
     kernel_ctx.define_int("DST_N", conf.dst_dims[0]);
     kernel_ctx.define_int("DST_C", conf.dst_dims[1]);
     kernel_ctx.define_int(
@@ -307,19 +376,30 @@ static status_t init_kernel_ctx_common(
     kernel_ctx.define_int(
             "DST_C_PADDED", utils::rnd_up(conf.dst_dims[1], conf.c_block_size));
 
+    // General problem descriptors
     kernel_ctx.define_int("SUB_GROUP_SIZE", conf.sub_group_size);
-    kernel_ctx.define_int("C_BLOCK_SIZE", conf.c_block_size);
-    kernel_ctx.define_int("N_BLOCK_SIZE", conf.n_block_size);
     kernel_ctx.define_int("VECT_DT_N", conf.vector_size);
     kernel_ctx.define_int("REDUCTION_SIZE", conf.div);
     kernel_ctx.define_int("NDIMS", conf.ndims);
     kernel_ctx.define_int("POWER", conf.power);
     kernel_ctx.define_float("EPS", conf.eps);
 
-    kernel_ctx.define_int("IS_N_REDUCED", conf.is_reduction_dim[0]);
-    kernel_ctx.define_int("IS_C_REDUCED", conf.is_reduction_dim[1]);
-    kernel_ctx.define_int(
-            "IS_HWD_REDUCED", conf.final_hwd_dim < conf.initial_hwd_dim);
+    kernel_ctx.define_int("SKIP_FINAL_PHASE", conf.skip_final_phase);
+
+    // Final kernel variables
+    kernel_ctx.define_int("FINAL_N_DIM", conf.final_n_dim);
+    kernel_ctx.define_int("FINAL_N_CHUNK_SIZE", conf.final_n_chunk_size);
+    kernel_ctx.define_int("FINAL_C_DIM", conf.final_c_dim);
+    kernel_ctx.define_int("FINAL_C_CHUNK_SIZE", conf.final_c_chunk_size);
+    kernel_ctx.define_int("FINAL_HWD_DIM", conf.final_hwd_dim);
+    kernel_ctx.define_int("FINAL_HWD_CHUNK_SIZE", conf.final_hwd_chunk_size);
+
+    // Define H/W/D dimensions for use in binary post ops
+    std::string dim_names[3] = {"D", "H", "W"};
+    for (int i = 2; i < 5; i++) {
+        dim_t dim = (i < conf.ndims) ? conf.dst_dims[i] : 1;
+        kernel_ctx.define_int("DST_" + dim_names[i - 2] + "_DIM", dim);
+    }
 
     switch (conf.alg) {
         case reduction_max: kernel_ctx.define_int("IS_MAX", 1); break;
@@ -345,15 +425,18 @@ static status_t init_kernel_ctx_common(
     def_memory_desc_info(kernel_ctx, conf.src_md_info, "SRC");
     def_memory_desc_info(kernel_ctx, conf.dst_md_info, "DST");
 
+    CHECK(def_attr_info(kernel_ctx, conf.attr_info, post_ops));
+
     def_dispatch(kernel_ctx, conf.dispatch);
-    def_dispatch(kernel_ctx, conf.finilize_dispatch);
+    if (!conf.skip_final_phase)
+        def_dispatch(kernel_ctx, conf.finalize_dispatch);
 
     return status::success;
 }
 
 status_t gen9_reduction_t::pd_t::init_kernel_ctx(
         compute::kernel_ctx_t &kernel_ctx) const {
-    return init_kernel_ctx_common(kernel_ctx, conf);
+    return init_kernel_ctx_common(kernel_ctx, conf, attr()->post_ops_);
 }
 
 void gen9_reduction_t::pd_t::init_scratchpad() {
@@ -375,24 +458,29 @@ status_t gen9_reduction_t::execute_gen9(const exec_ctx_t &ctx) const {
                     memory_tracking::names::key_reduction);
     const auto &conf = pd()->conf;
 
+    // Kick off the initial reduction phase
     compute::kernel_arg_list_t reduction_arg_list;
     reduction_arg_list.set(0, src);
     reduction_arg_list.set(1, conf.skip_final_phase ? dst : *temp_reduce);
+    if (conf.skip_final_phase) {
+        append_post_ops_to_arg_list(
+                ctx, reduction_arg_list, 2, pd()->attr()->post_ops_);
+    }
     auto initial_nd_range = conf.dispatch.nd_range();
     status_t status = parallel_for(
             ctx, initial_nd_range, initial_kernel, reduction_arg_list);
 
-    if (!conf.skip_final_phase) {
-        if (status != status::success) return status;
-        compute::kernel_arg_list_t final_reduction_arg_list;
-        final_reduction_arg_list.set(0, *temp_reduce);
-        final_reduction_arg_list.set(1, dst);
-        auto final_nd_range = conf.finilize_dispatch.nd_range();
-        return parallel_for(
-                ctx, final_nd_range, final_kernel, final_reduction_arg_list);
-    } else {
-        return status;
-    }
+    if (conf.skip_final_phase || status != status::success) return status;
+
+    // Continue with final reduction phase
+    compute::kernel_arg_list_t final_reduction_arg_list;
+    final_reduction_arg_list.set(0, *temp_reduce);
+    final_reduction_arg_list.set(1, dst);
+    append_post_ops_to_arg_list(
+            ctx, final_reduction_arg_list, 2, pd()->attr()->post_ops_);
+    auto final_nd_range = conf.finalize_dispatch.nd_range();
+    return parallel_for(
+            ctx, final_nd_range, final_kernel, final_reduction_arg_list);
 }
 
 } // namespace ocl
