@@ -369,7 +369,8 @@ status_t gen9_wino_convolution_fwd_t::pd_t::init_kernel_ctx(
     dst_dims[2] = conf.ndims > 4 ? conf.od : conf.oh;
     dst_dims[3] = conf.ndims > 4 ? conf.oh : conf.ow;
     dst_dims[4] = conf.ow;
-    def_attr_info(kernel_ctx, conf.attr_info, attr()->post_ops_, &dst_dims);
+    CHECK(def_attr_info(
+            kernel_ctx, conf.attr_info, attr()->post_ops_, &dst_dims));
 
     kernel_ctx.print_options();
     return status::success;
@@ -440,7 +441,7 @@ status_t gen9_wino_convolution_fwd_t::execute_forward(
             && !gpu_eltwise_fwd_pd_t::eltwise_preserves_zero(
                     attr_info.eltwise_alg, attr_info.eltwise_alpha,
                     attr_info.eltwise_beta)) {
-        ctx.zero_pad_output(DNNL_ARG_DST);
+        CHECK(ctx.zero_pad_output(DNNL_ARG_DST));
     }
     return status;
 }
