@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2021-2022 Intel Corporation
+* Copyright 2021-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -34,7 +34,7 @@
   To fill cache line kernel must load 16 consecutive items (16c) and
   store 16 consecutive items (16a). So it needs to operate on a matrix of
   16a x 16c.
-  It will load 16 non-adjacent (strided by A) sets of 16 adjacent data 
+  It will load 16 non-adjacent (strided by A) sets of 16 adjacent data
   (strided by C, src' innermost dimension), perform internal transposition,
    then store 16 non-adjacent (strided by C) sets of 16 adjacent data (strided
    by A, dst's innermost dimension).
@@ -57,102 +57,401 @@ namespace ocl {
 using namespace dnnl::impl::memory_tracking::names;
 
 struct dimension_t {
-    int size = 0;
-    int step = 0;
-    int idx = 0;
+    dim_t size;
+    dim_t step;
+    int idx;
 };
 
 using dimensions_t = std::vector<dimension_t>;
 
-bool is_power_of_2(int n) {
-    return ((n & (n - 1)) == 0);
-}
+// Return a description of dimensions sorted by stride, i.e., nesting order.
+dimensions_t dims_by_stride(const memory_desc_wrapper &mdw) {
+    const auto &desc = mdw.blocking_desc();
+    const auto &strides = desc.strides;
 
-using stride_t = struct {
-    int idx;
-    int stride;
-    int size;
-};
+    // Sort blocks by stride.
+    const auto cmp = [&](const dimension_t &a, const dimension_t &b) {
+        // Order by stride. Ties mean that we have at least one dim of size 1.
+        // We don't care about the order of those dims, just that that the dim
+        // with size > 1 is sorted last.
+        const auto a_stride = strides[a.idx];
+        const auto b_stride = strides[b.idx];
+        return a_stride < b_stride || (a_stride == b_stride && a.size < b.size);
+    };
 
-// Stride sorter. Smaller stride = inner dim, bigger stride = outer dim.
-// Dimensions of size 1 are considered outermost regardless of strides and
-// they are sorted by index.
-bool stride_less(const stride_t &a, const stride_t &b) {
-    if (a.size == 1 && b.size == 1) {
-        return a.idx > b.idx;
-    } else if (a.size != 1 && b.size == 1) {
-        return true;
-    } else if (a.size == 1 && b.size != 1) {
-        return false;
-    } else {
-        return a.stride < b.stride;
+    const int ndims = mdw.ndims();
+    dimensions_t dims(ndims);
+    for (int d = 0; d < ndims; ++d) {
+        auto &blk = dims[d];
+        blk.idx = d;
+        blk.size = mdw.padded_dims()[d];
     }
+    std::sort(dims.begin(), dims.end(), cmp);
+    return dims;
 }
 
 // Returns description of blocks and dimensions that constitute the format tag
 // of tensor, starting from innermost. Blocks, if exist, take precedence before
 // dimensions. Order of dimensions is determined by sorting strides; smallest
-// stride is innermost dimension. Dimensions of size 1 are ignored (treated as
-// outermost). Dimensions are treated like blocks, that is they don't report
-// whole tensor size across given axis but rather number of underlying blocks
-// for given dimension.
+// stride is innermost dimension. Dimensions of size 1 are ignored. This may
+// lead to illegal tensor tags, where the innermost dim is the same as the
+// outermost block:
+//     8x8x1x1 aBcd8b becomes cdaB8b (note: "B8b").
+// In such cases, this function combines the last dim with the first block(s),
+// so xB8b becomes xb. Dimensions are treated like blocks, that is they don't
+// report whole tensor size across given axis but rather number of underlying
+// blocks for given dimension.
 // Example: ABcd8b8a2b 32x48x1x7 will return description that amounts to...
 // outermost-> 1c:4a:3b:7d:8b:8a:2b <-innermost
-dimensions_t query_dims_and_blocks(
-        const memory_desc_wrapper &md, int distance = 0) {
-    const int nblks = md.blocking_desc().inner_nblks;
-    const int ndims = md.ndims();
+dimensions_t query_dims_and_blocks(const memory_desc_wrapper &mdw) {
+    auto blocks = dims_by_stride(mdw);
+    const int ndims = mdw.ndims();
+    const auto &desc = mdw.blocking_desc();
+    const int nblks = desc.inner_nblks;
 
-    std::vector<stride_t> strides(ndims);
-    for (int d = 0; d < ndims; ++d) {
-        strides[d].idx = d;
-        strides[d].stride = md.blocking_desc().strides[d];
-        strides[d].size = md.padded_dims()[d];
+    // Calculate info for inner blocks
+    dimensions_t inner_blks(nblks);
+    std::vector<int> steps(ndims, 1);
+    dim_t blks_size = 1;
+    for (int i = nblks - 1; i >= 0; --i) {
+        auto &blk = inner_blks[i];
+        blk.idx = desc.inner_idxs[i];
+        blk.size = desc.inner_blks[i];
+        blk.step = steps[blk.idx];
+        // steps increase in reverse order of how blocks are listed
+        steps[blk.idx] *= blk.size;
+        blks_size *= blk.size;
     }
-    std::sort(strides.begin(), strides.end(), stride_less);
-    for (int i = 0; i < nblks; i++) {
-        stride_t blk;
-        blk.idx = md.blocking_desc().inner_idxs[i];
-        blk.size = md.blocking_desc().inner_blks[i];
-        if (i == 0 && blk.idx == strides[0].idx) { continue; }
-        strides.insert(strides.begin(), blk);
+
+    // Divide dim by its step to get block size
+    for (auto &blk : blocks) {
+        blk.step = steps[blk.idx];
+        blk.size = utils::div_up(blk.size, blk.step);
     }
-    // calculate step sizes
-    int steps[MAX_NDIMS] = {1, 1, 1, 1, 1, 1};
-    dimensions_t dims(strides.size());
-    for (size_t i = 0; i < strides.size(); i++) {
-        dims[i].idx = strides[i].idx;
-        dims[i].size = strides[i].size;
-        dims[i].step = steps[dims[i].idx];
-        steps[strides[i].idx] *= strides[i].size;
-    }
-    // divide last instance of given dim (it's true dim, not a block)
-    // by its step
-    bool idx_done[MAX_NDIMS] = {false, false, false, false, false, false};
-    for (int i = (int)(dims.size() - 1); i >= 0; i--) {
-        if (!idx_done[dims[i].idx]) {
-            dims[i].size = utils::div_up(dims[i].size, dims[i].step);
-            idx_done[dims[i].idx] = true;
+
+    // If we have any dims with block size 1, we ignore them.
+    const auto size_1 = [](const dimension_t &b) { return b.size == 1; };
+    const auto end = blocks.end();
+    blocks.erase(std::remove_if(blocks.begin(), end, size_1), end);
+
+    dim_t stride = blocks.empty() ? 1 : desc.strides[blocks[0].idx];
+    for (auto &blk : inner_blks) {
+        if (blk.size == 1) continue; // Can safely ignore blocks of size 1
+        if (blocks.empty() || blocks[0].idx != blk.idx || blks_size != stride) {
+            blocks.insert(blocks.begin(), blk);
+        } else {
+            // Combine blocks with repeated index if there is no extra padding
+            blk.size *= blocks[0].size;
+            blocks[0] = blk;
         }
+        blks_size /= blk.size;
+        stride = blks_size;
     }
-    return dims;
+
+    if (blocks.empty() && ndims > 0) {
+        dimension_t blk;
+        blk.idx = 0;
+        blk.size = 1;
+        blk.step = 1;
+        blocks.push_back(blk);
+    }
+    return blocks;
 }
 
-dimensions_t query_dims_and_blocks(const memory_desc_t &m, int distance = 0) {
-    memory_desc_wrapper mdw(m);
-    return query_dims_and_blocks(mdw, distance);
+dimensions_t query_dims_and_blocks(const memory_desc_t &md) {
+    const memory_desc_wrapper mdw(md);
+    return query_dims_and_blocks(mdw);
+}
+
+bool is_generic_faster_than_ref(
+        const memory_desc_t &src_md, const memory_desc_t &dst_md) {
+    const dim_t max_1d_ref_nelems = 512;
+    const dim_t max_nd_ref_nelems = 512 * 512;
+    auto nelems
+            = std::max(utils::array_product(src_md.padded_dims, src_md.ndims),
+                    utils::array_product(dst_md.padded_dims, dst_md.ndims));
+    if (src_md.ndims == 1 && dst_md.ndims == 1)
+        return nelems > max_1d_ref_nelems;
+    auto src_blks = query_dims_and_blocks(src_md);
+    auto dst_blks = query_dims_and_blocks(dst_md);
+    if (src_blks.empty() || dst_blks.empty()) return false;
+    auto src_inner_idx = src_blks[0].idx;
+    auto dst_inner_idx = dst_blks[0].idx;
+    auto scale = (src_inner_idx != dst_inner_idx) ? 2 : 1;
+    return nelems > scale * max_nd_ref_nelems;
+}
+
+using dim_pair_t = std::array<dimension_t, 2>;
+
+// Return whether the two blocks represent an equal part of the same dimension.
+bool equal_blocks(const dim_pair_t &a, const dim_pair_t &b) {
+    return (a[0].size == b[0].size && a[1].size == b[1].size);
+}
+
+// Combine dimension j into dimension i.
+void combine(memory_desc_t &md, int i, int j) {
+    const int new_ndims = md.ndims - 1;
+    if (new_ndims == 0) return; // Don't delete the only dimension.
+    auto &desc = md.format_desc.blocking;
+    auto &strides = desc.strides;
+    const int outer = strides[i] < strides[j] ? j : i;
+    const int inner = strides[i] < strides[j] ? i : j;
+
+    const auto outer_stride = strides[outer];
+    const auto outer_size = md.padded_dims[outer];
+    md.offset0 += strides[outer] * md.padded_offsets[outer];
+    md.dims[i] = md.dims[outer] * md.padded_dims[inner];
+    md.padded_dims[i] = md.padded_dims[i] * md.padded_dims[j];
+    md.padded_offsets[i] = md.padded_offsets[inner];
+    strides[i] = strides[inner];
+    for (int k = j; k < new_ndims; ++k) {
+        md.dims[k] = md.dims[k + 1];
+        md.padded_dims[k] = md.padded_dims[k + 1];
+        md.padded_offsets[k] = md.padded_offsets[k + 1];
+        strides[k] = strides[k + 1];
+    }
+    md.dims[new_ndims] = 0;
+    md.padded_dims[new_ndims] = 0;
+    md.padded_offsets[new_ndims] = 0;
+    strides[new_ndims] = 0;
+
+    auto &idxs = desc.inner_idxs;
+    auto &blks = desc.inner_blks;
+    int nblks = desc.inner_nblks;
+    auto blks_size = utils::array_product(blks, nblks);
+    int count = 0;
+    bool last_is_combined = false;
+    dim_t blocks = 1;
+    for (int k = 0; k < nblks; ++k) {
+        if (idxs[k] == i || idxs[k] == j) {
+            blocks *= blks[k];
+            // Combine the innermost dim and outermost block when they have the
+            // same index and no extra padding, e.g., ...A8a... -> ...a...
+            if (count == 0 && strides[i] == blks_size) {
+                md.dims[i] = md.padded_dims[i];
+                strides[i] /= blks[k];
+                blks_size /= blks[k];
+            } else if (last_is_combined) {
+                blks[count - 1] *= blks[k];
+            } else {
+                last_is_combined = true;
+                blks[count] = blks[k];
+                idxs[count] = i;
+                count++;
+            }
+            continue;
+        }
+        last_is_combined = false;
+        blks[count] = blks[k];
+        idxs[count] = (idxs[k] > j ? idxs[k] - 1 : idxs[k]);
+        count++;
+    }
+    // We've changed Nx1x...x1xM to 1x...x1xNM by combining dims, now fix the
+    // strides of the size-1 dims by multiplying by the step of the size-N dim.
+    auto outer_step = utils::div_up(outer_size, blocks);
+    for (int k = 0; k < new_ndims; ++k) {
+        if (strides[k] == outer_stride) strides[k] *= outer_step;
+    }
+    desc.inner_nblks = count;
+    md.ndims = new_ndims;
+}
+
+void remove_bit(int &mask, int bit) {
+    const int lower_bits = (1 << bit) - 1;
+    mask = (mask & lower_bits) | ((mask >> 1) & ~lower_bits);
+}
+
+// For each dimension, determine if the inner dimensions do not account for its
+// stride. We cannot combine a dimension that does not align with the stride of
+// the next outer dimension.
+int extended_dims(const memory_desc_t &md) {
+    int mask = 0;
+    const int ndims = md.ndims;
+    const auto &blkg = md.format_desc.blocking;
+    const int nblks = blkg.inner_nblks;
+
+    auto dims = dims_by_stride(md);
+    std::vector<dim_t> blocks(ndims, 1);
+    dim_t expected_stride = 1;
+    for (int i = 0; i < nblks; ++i) {
+        auto idx = blkg.inner_idxs[i];
+        auto blks = blkg.inner_blks[i];
+        blocks[idx] *= blks;
+        expected_stride *= blks;
+    }
+
+    for (int i = 0; i < ndims; ++i) {
+        const auto &dim = dims[i];
+        auto stride = blkg.strides[dim.idx];
+        auto step = utils::div_up(dim.size, blocks[dim.idx]);
+        if (stride != expected_stride) {
+            mask |= (1 << dim.idx);
+            expected_stride = stride;
+        }
+        expected_stride *= step;
+    }
+    return mask;
+}
+
+struct pair_filter_t {
+public:
+    using value_type = dim_pair_t;
+
+private:
+    using const_dim_iterator_t = typename dimensions_t::const_iterator;
+    using predicate_t = std::function<bool(const value_type &)>;
+
+public:
+    struct iterator_t {
+        bool operator==(const iterator_t &o) const { return it == o.it; }
+        bool operator!=(const iterator_t &o) const { return it != o.it; }
+        value_type operator*() const { return {*it, *(it + 1)}; }
+        iterator_t &operator++() {
+            advance();
+            return *this;
+        }
+        iterator_t operator++(int) {
+            auto cpy = *this;
+            advance();
+            return cpy;
+        }
+        iterator_t(const_dim_iterator_t it, const_dim_iterator_t end,
+                predicate_t pred)
+            : it(it), end(end), pred(std::move(pred)) {
+            advance(true);
+        }
+
+    private:
+        void advance(bool check_first = false) {
+            if (it == end || (check_first && pred(operator*()))) return;
+            while (++it != end && !pred(operator*())) {}
+        }
+
+        const_dim_iterator_t it, end;
+        predicate_t pred;
+    };
+
+    iterator_t begin() const { return {begin_, end_ - 1, pred}; }
+    iterator_t end() const { return {end_ - 1, end_ - 1, pred}; }
+    bool empty() const { return begin() == end(); }
+
+    pair_filter_t(const dimensions_t &iter, const predicate_t &pred)
+        : begin_(iter.begin()), end_(iter.end()), pred(pred) {}
+
+private:
+    const_dim_iterator_t begin_, end_;
+    predicate_t pred;
+};
+
+#define NO_IDX (-1)
+// Find the index of the dimension that always and only follows the dimension
+// with index idx. If none exists, return NO_IDX. If no dimension with index idx
+// is present in the given block representation, return idx to delete the
+// dimension
+int successor(const dimensions_t &a, int idx) {
+    int succ;
+    auto match_idx = [&](const dim_pair_t &p) { return p[0].idx == idx; };
+    auto match_xor = [&](const dim_pair_t &p) {
+        return match_idx(p) ^ (p[1].idx == succ);
+    };
+    // idx is the index of outermost dim; it has no successor
+    if (a.back().idx == idx) return NO_IDX;
+    auto filtered = pair_filter_t(a, match_idx);
+    // no dim with index idx appears in block representation; delete it
+    if (filtered.empty()) return idx;
+    succ = (*filtered.begin())[1].idx;
+    // succ is the index of the innermost dim; it has no predecessor
+    if (a.front().idx == succ) return NO_IDX;
+    if (!pair_filter_t(a, match_xor).empty()) return NO_IDX;
+    return succ;
+}
+
+// Find the index of the dimension that ALWAYS follows dimension `idx` in the
+// given block representations. The successor dimension will be combined with
+// the given dimension, or, in the case that the given dimension does not appear
+// in the block representation, it will be deleted.
+int successor(const dimensions_t &a, const dimensions_t &b, int idx) {
+    auto succ = successor(a, idx);
+    if (succ == NO_IDX || succ != successor(b, idx)) return NO_IDX;
+
+    auto pred = [&](const dim_pair_t &p) { return p[0].idx == idx; };
+    pair_filter_t iter_a(a, pred);
+    pair_filter_t iter_b(b, pred);
+
+    auto it_a = iter_a.begin();
+    auto it_b = iter_b.begin();
+    const auto end_a = iter_a.end();
+    const auto end_b = iter_b.end();
+
+    for (; it_a != end_a && it_b != end_b; ++it_a, ++it_b) {
+        if (!equal_blocks(*it_a, *it_b)) return NO_IDX;
+    }
+    return (it_a != end_a || it_b != end_b) ? NO_IDX : succ;
+}
+
+bool can_be_combined(int idx, int mask) {
+    return !(idx == NO_IDX || (mask & (1 << idx)));
+}
+
+void compress(memory_desc_t &a, memory_desc_t &b, int &a_mask, int &b_mask) {
+    const auto blks_a = query_dims_and_blocks(a);
+    const auto blks_b = query_dims_and_blocks(b);
+    const int skip_mask = a_mask | b_mask | extended_dims(a) | extended_dims(b);
+
+    const int ndims = a.ndims;
+    std::vector<int> successors(ndims, NO_IDX);
+    std::vector<int> aliases(ndims);
+    for (int i = 0; i < ndims; ++i) {
+        aliases[i] = i;
+        if ((a_mask | b_mask) & (1 << i)) continue;
+        auto succ = successor(blks_a, blks_b, i);
+        if (!can_be_combined(succ, skip_mask)) continue;
+        successors[i] = succ;
+    }
+
+    for (int i = ndims - 1; i >= 0; --i) {
+        int succ = successors[i];
+        if (succ == NO_IDX) continue;
+        while (succ != aliases[succ])
+            succ = aliases[succ];
+        int from = std::max(i, succ);
+        int into = std::min(i, succ);
+        combine(a, into, from);
+        combine(b, into, from);
+        remove_bit(a_mask, from);
+        remove_bit(b_mask, from);
+        aliases[from] = into;
+    }
+}
+#undef NO_IDX
+
+void fix_steps(dimensions_t &blk, dimensions_t pkt) {
+    int steps[MAX_NDIMS] = {1, 1, 1, 1, 1, 1};
+    for (size_t i = 0; i < pkt.size(); i++) {
+        steps[pkt[i].idx] *= pkt[i].size;
+    }
+    for (size_t i = 0; i < blk.size(); i++) {
+        blk[i].step = steps[blk[i].idx];
+        steps[blk[i].idx] *= blk[i].size;
+    }
 }
 
 // Returns vector of blocks that were present in a but missing from b
-dimensions_t find_missing_blocks(dimensions_t all, dimensions_t subset) {
+dimensions_t find_missing_blocks(
+        dimensions_t all, dimensions_t subset, bool round_up) {
     dimensions_t ret;
     for (size_t ia = 0; ia < all.size(); ia++) {
         dimension_t from_a = all[ia];
         for (size_t ib = 0; ib < subset.size(); ib++) {
             if (subset[ib].idx == from_a.idx) {
                 auto smaller = std::min(from_a.size, subset[ib].size);
-                from_a.size /= smaller;
-                subset[ib].size /= smaller;
+                if (round_up) {
+                    from_a.size = utils::div_up(from_a.size, smaller);
+                    subset[ib].size = utils::div_up(subset[ib].size, smaller);
+                } else {
+                    from_a.size /= smaller;
+                    subset[ib].size /= smaller;
+                }
             }
         }
         if (from_a.size > 1) { ret.push_back(from_a); }
@@ -187,49 +486,28 @@ dimensions_t remainder(dimensions_t all, dimensions_t subset) {
 // Examples:
 // For 1024x1024 ab, it will be (16b)
 // for 16x16x16 ABc2a2b, it will be (4c2a2b)
-bool fill_to_vect(int simd_size, dimensions_t all, dimensions_t &subset) {
-    int need = simd_size;
-    int current_size = 1;
+bool fill_to_vect(
+        int simd_size, const dimensions_t &all, dimensions_t &subset) {
+    const int min_full_vecs = 5; // TODO: tune me
+    dim_t current_size = 1;
     subset.clear();
-    for (size_t i = 0; i < all.size(); i++) {
-        if (i == 0 && all[i].size > 3 * simd_size) {
-            // Vectorize innermost dim. Since it's not divisible by simd size,
-            // it will need to be padded. And for that the vectorised dim
+    for (auto &dim : all) {
+        dim_t next_size = current_size * dim.size;
+        int next_full_vecs = next_size / simd_size;
+        if (next_full_vecs >= min_full_vecs || next_size % simd_size == 0) {
+            // Vectorize innermost dim(s). If it's not divisible by simd size,
+            // they will need to be padded. And for that the vectorised dim(s)
             // should be large enough because otherwise the padding would be
             // too significant fraction of tensor and it would hurt perf.
-            dimension_t tmp = all[i];
-            tmp.size = simd_size;
+            dimension_t tmp = dim;
+            tmp.size = simd_size / current_size;
             subset.push_back(tmp);
             return true;
         }
-        if (all[i].size % 2 != 0) {
-            // can't fill packet to exactly 16
-            return false;
-        }
-        if (all[i].size <= need) {
-            subset.push_back(all[i]);
-            current_size *= all[i].size;
-            need /= all[i].size;
-        } else if ((all[i].size / need) * need == all[i].size) {
-            dimension_t tmp = all[i];
-            tmp.size = need;
-            subset.push_back(tmp);
-            current_size *= tmp.size;
-            need /= tmp.size;
-        } else {
-            // can't fill packet exactly to requested value; either size or
-            // simd was not power-of-2?
-            // TODO: cases where innermost dim is small and odd are not
-            // supported by this kernel because there's no logic to construct
-            // 16-item packet in that case. However, it should be possible to
-            // support cases like ...16a3b: declare packet=5.3a3b, burst=3a,
-            // then on opencl side add logic that will decode it into integer
-            // a,b coordinates. Such change would almost eliminate use of slow,
-            // reference reorder.
-            return false;
-        }
-        if (need == 1) { return true; }
-        if (current_size == 1) { return false; }
+        // No hope of properly filling the vector.
+        if (simd_size % next_size != 0) return false;
+        current_size = next_size;
+        subset.push_back(dim);
     }
     // there was not enough data in tensor to fill even a single packet
     return false;
@@ -322,8 +600,8 @@ size_t check_burst_length(dimensions_t all, dimensions_t subset) {
 // burst size = 32 (2b*4c*4a)
 bool increase_burst(dimensions_t all, dimensions_t &subset, dimensions_t &other,
         size_t itemlimit, size_t current_size, size_t optimal_size) {
-    const int space_coeff = (int)(itemlimit / check_size(subset));
-    const int request_coeff = (int)(utils::div_up(optimal_size, current_size));
+    const dim_t space_coeff = itemlimit / check_size(subset);
+    const dim_t request_coeff = utils::div_up(optimal_size, current_size);
     dimensions_t subset_copy = subset;
     if (space_coeff < 2) { return false; }
     for (size_t i = 0; i < all.size(); i++) {
@@ -336,7 +614,7 @@ bool increase_burst(dimensions_t all, dimensions_t &subset, dimensions_t &other,
         }
         if (all[i].size != 1) {
             // add to subset new item or enlarge last item, if it was the same dim
-            int incr = std::min(space_coeff, all[i].size);
+            auto incr = std::min(space_coeff, all[i].size);
             incr = std::min(incr, request_coeff);
             all[i].size = incr;
             bool success = add_to_vector(subset, all[i]);
@@ -366,8 +644,8 @@ bool split_into_blocks_and_packets(size_t vect, size_t optimal_burst_bytes,
     dimensions_t dremainder = remainder(dst, dst_packet);
     // 3. The same amount of data will be read and written. So, every dimension
     // that's in src packet and not in dst packet must be in dst block.
-    src_block = find_missing_blocks(dst_packet, src_packet);
-    dst_block = find_missing_blocks(src_packet, dst_packet);
+    src_block = find_missing_blocks(dst_packet, src_packet, true);
+    dst_block = find_missing_blocks(src_packet, dst_packet, false);
     // 4a. Check how much continuous data will be read/written...
     size_t burst_size_src
             = vect * sizeof_src * check_burst_length(sremainder, src_block);
@@ -399,6 +677,8 @@ bool split_into_blocks_and_packets(size_t vect, size_t optimal_burst_bytes,
     // possible continuous memory accesses.
     src_block = fix_order_to(src_block, sremainder);
     dst_block = fix_order_to(dst_block, dremainder);
+    fix_steps(src_block, src_packet);
+    fix_steps(dst_block, dst_packet);
     return true;
 }
 
@@ -460,12 +740,12 @@ bool fill_conf_vld(const memory_desc_wrapper &src,
         cfg.dst_blk[i].blk_size = dst_block[i].size;
         cfg.dst_blk[i].step_size = dst_block[i].step;
     }
-    cfg.vector_dim = src_packet[0].idx;
-    vect_dim = src_packet[0].idx;
+    cfg.vector_dim = dst_packet[0].idx;
+    vect_dim = dst_packet[0].idx;
     vect_size = 16;
     for (int i = 0; i < LOOP_NEST_LEVEL; i++) {
-        if (cfg.src_blk[i].blk_size != 1) {
-            blocks[cfg.src_blk[i].dim_idx] *= cfg.src_blk[i].blk_size;
+        if (cfg.dst_blk[i].blk_size != 1) {
+            blocks[cfg.dst_blk[i].dim_idx] *= cfg.dst_blk[i].blk_size;
         }
     }
     // Multiply by 16 the size of the dimension that will be vectorized.
@@ -480,280 +760,50 @@ bool fill_conf_vld(const memory_desc_wrapper &src,
     cfg.rescale_coeff = 16;
 
     for (int i = 0; i < LOOP_NEST_LEVEL; i++) {
-        auto sb = cfg.src_vct[i];
-        blocks[sb.dim_idx] *= sb.blk_size;
+        auto db = cfg.dst_vct[i];
+        blocks[db.dim_idx] *= db.blk_size;
     }
 
     return true;
 }
 
-#define IRRELEVANT_DIM 7
-
-bool is_relevant(const dimension_t dim, int idxa, int idxb) {
-    return (dim.idx == idxa || dim.idx == idxb) && (dim.size > 1);
-}
-
-dimension_t get_idx(
-        const dimensions_t &dims, int idxa, int idxb, size_t &iter) {
-    dimension_t ret;
-    if (iter >= dims.size()) {
-        ret.idx = IRRELEVANT_DIM;
-        ret.size = 0;
-        return ret;
-    }
-    // found relevant dim? advance iterator and return that dim
-    if (is_relevant(dims[iter], idxa, idxb)) {
-        ret = dims[iter];
-        iter++;
-        return ret;
-    } else {
-        // found irrelevant dim? consume all subsequent irrelevant dims
-        ret.idx = IRRELEVANT_DIM;
-        ret.size = 0;
-        iter++;
-        while (iter < dims.size() && !is_relevant(dims[iter], idxa, idxb)) {
-            iter++;
-        }
-        return ret;
-    }
-}
-
-bool doesnt_fit(dimension_t a, dimension_t b) {
-    return (a.idx != b.idx) || (a.size != b.size);
-}
-
-// Mode of finding adjacent dimensions to be combined together.
-// adjacent = Every instance of given dim must be a neighbor of the other dim,
-//            this combination doesn't block any other valid combination.
-// any      = Dims don't have to be adjacent. Any such combination may prevent
-//            other 'any' combination, they are mutually exclusive. Need to
-//            write logic to choose best from such possible combinations.
-//            NOT IMPLEMENTED.
-enum mode_t { adjacent, any };
-
-bool check_adjacent(int dim, int &prev_dim, bool &adjacent) {
-    if (prev_dim != IRRELEVANT_DIM && dim != IRRELEVANT_DIM) {
-        adjacent = true;
-    } else if (adjacent == true && dim == IRRELEVANT_DIM) {
-        adjacent = false;
-    } else if (adjacent == false && prev_dim != IRRELEVANT_DIM
-            && dim == IRRELEVANT_DIM) {
-        return false;
-    }
-    return true;
-}
-
-bool compare_formats(const dimensions_t &dims_a, const dimensions_t &dims_b,
-        int dim1, int dim2, mode_t mode) {
-    size_t iter_a = 0;
-    size_t iter_b = 0;
-    dimension_t dim_a;
-    dimension_t dim_b;
-    int prev_dim_a = IRRELEVANT_DIM;
-    int prev_dim_b = IRRELEVANT_DIM;
-    bool a_adj = false;
-    bool b_adj = false;
-    do {
-        dim_a = get_idx(dims_a, dim1, dim2, iter_a);
-        dim_b = get_idx(dims_b, dim1, dim2, iter_b);
-        if (doesnt_fit(dim_a, dim_b)) { return false; }
-        if (mode == mode_t::adjacent) {
-            if (!check_adjacent(dim_a.idx, prev_dim_a, a_adj)) { return false; }
-            if (!check_adjacent(dim_b.idx, prev_dim_b, b_adj)) { return false; }
-        }
-        prev_dim_a = dim_a.idx;
-        prev_dim_b = dim_b.idx;
-    } while (iter_a < dims_a.size() && iter_b < dims_b.size());
-    if (mode == mode_t::adjacent) {
-        if (!check_adjacent(IRRELEVANT_DIM, prev_dim_a, a_adj)) {
-            return false;
-        }
-        if (!check_adjacent(IRRELEVANT_DIM, prev_dim_b, b_adj)) {
-            return false;
-        }
-    }
-    return true;
-}
-
-// Takes memory descriptor and information which two dimension to combine.
-// Returns new memory descriptor with those two dimensions combined into one.
-// TODO: it doesn't understand padded_offsets
-void combine_dims(memory_desc_t &new_a, int dim1, int dim2) {
-    // shorter names
-    dnnl_dim_t *str = &new_a.format_desc.blocking.strides[0];
-    dnnl_dim_t *blks = &new_a.format_desc.blocking.inner_blks[0];
-    dnnl_dim_t *idxs = &new_a.format_desc.blocking.inner_idxs[0];
-
-    for (int i = 0; i < new_a.ndims; i++) {
-        if (i == dim1) {
-            new_a.dims[i] = new_a.padded_dims[i];
-        } else if (i == dim2) {
-            new_a.dims[dim1] *= new_a.padded_dims[i];
-        } else if (i < dim2) {
-            new_a.dims[i] = new_a.dims[i];
-        } else {
-            new_a.dims[i - 1] = new_a.dims[i];
-        }
-    }
-    for (int i = new_a.ndims - 1; i < MAX_NDIMS; i++) {
-        new_a.dims[i] = 0;
-    }
-
-    for (int i = 0; i < new_a.ndims; i++) {
-        if (i == dim1) {
-            new_a.padded_dims[i] = new_a.padded_dims[i];
-        } else if (i == dim2) {
-            new_a.padded_dims[dim1] *= new_a.padded_dims[i];
-        } else if (i < dim2) {
-            new_a.padded_dims[i] = new_a.padded_dims[i];
-        } else {
-            new_a.padded_dims[i - 1] = new_a.padded_dims[i];
-        }
-    }
-    for (int i = new_a.ndims - 1; i < MAX_NDIMS; i++) {
-        new_a.padded_dims[i] = 0;
-    }
-
-    for (int i = 0; i < new_a.ndims; i++) {
-        if (i == dim1) {
-            str[i] = std::min(str[dim1], str[dim2]);
-        } else if (i == dim2) { // already accounted for in the line above
-        } else if (i < dim2) { // do nothing: str[i] = str[i];
-        } else {
-            str[i - 1] = str[i];
-        }
-    }
-    for (int i = new_a.ndims - 1; i < MAX_NDIMS; i++) {
-        str[i] = 0;
-    }
-
-    bool last_is_combined = false;
-    int out_iter = 0;
-    for (int i = 0; i < new_a.format_desc.blocking.inner_nblks; i++) {
-        if (idxs[i] == dim1 || idxs[i] == dim2) {
-            if (last_is_combined) {
-                blks[out_iter - 1] *= blks[i];
-            } else {
-                last_is_combined = true;
-                blks[out_iter] = blks[i];
-                idxs[out_iter] = dim1;
-                out_iter++;
-            }
-        } else {
-            last_is_combined = false;
-            blks[out_iter] = blks[i];
-            idxs[out_iter] = (idxs[i] > dim2 ? idxs[i] - 1 : idxs[i]);
-            out_iter++;
-        }
-    }
-    new_a.format_desc.blocking.inner_nblks = out_iter;
-    new_a.ndims = new_a.ndims - 1;
-}
-
-int shift_mask(int &mask, int dim) {
-    int ret = 0;
-    for (int i = 0; i < dim; i++) {
-        ret |= (mask & (1 << i));
-    }
-    for (int i = dim; i < MAX_NDIMS - 1; i++) {
-        ret |= ((mask & (1 << i)) >> 1);
-    }
-    return ret;
-}
-
-// Returns bit mask with each bit corresponding to dimension that has strides
-// larger than what could be expected according to dimensions sizes.
-int is_padded_by_strides(const memory_desc_wrapper &md) {
-    int ret = 0;
-    const int nblks = md.blocking_desc().inner_nblks;
-    const int ndims = md.ndims();
-
-    std::vector<stride_t> strides(ndims);
-    for (int d = 0; d < ndims; ++d) {
-        strides[d].idx = d;
-        strides[d].stride = md.blocking_desc().strides[d];
-        strides[d].size = md.padded_dims()[d];
-    }
-    std::sort(strides.begin(), strides.end(), stride_less);
-    int blocks_size = 1;
-    for (int i = 0; i < nblks; i++) {
-        blocks_size *= md.blocking_desc().inner_blks[i];
-    }
-    int expected_stride = blocks_size;
-    for (size_t i = 0; i < strides.size(); i++) {
-        if (strides[i].stride != expected_stride) {
-            ret |= 0x1 << strides[i].idx;
-            expected_stride = strides[i].stride * strides[i].size;
-        } else {
-            expected_stride *= strides[i].size;
-        }
-    }
-    return ret;
-}
-
-// Given description of src and dst tensors, try to find a subset of dimensions
-// that are not reordered between src and dst, then try to treat that subset as
-// a single dmension.
-// Example: abcd -> acdb 128x3x7x7; the CD dimensions stay in the same order
-// relative to each other, so operation can be redefined as abz -> azb 128x3x49
-bool try_combine_dims(
-        memory_desc_t &a, memory_desc_t &b, int &mask, mode_t mode) {
-    const dimensions_t a_dims = query_dims_and_blocks(a);
-    const dimensions_t b_dims = query_dims_and_blocks(b);
-    int padded_strides = is_padded_by_strides(a) | is_padded_by_strides(b);
-    for (int i = 0; i < a.ndims; i++) {
-        for (int j = i + 1; j < b.ndims; j++) {
-            if ((mask >> i) & 0x1) { continue; }
-            if ((mask >> j) & 0x1) { continue; }
-            if ((padded_strides >> i) & 0x1) { continue; }
-            if ((padded_strides >> j) & 0x1) { continue; }
-            if (i == j) { continue; }
-            if (compare_formats(a_dims, b_dims, i, j, mode)) {
-                combine_dims(a, i, j);
-                combine_dims(b, i, j);
-                mask = shift_mask(mask, j);
-                return true;
-            }
-        }
-    }
-    return false;
-}
-
 status_t generic_reorder_t::pd_t::init_conf(engine_t *engine) {
     using namespace format_tag;
 
     size_t memlimit_bytes;
     size_t optimal_burst_bytes;
 
-    conf.scale_quant = attr()->output_scales_.mask_ != 0;
-    conf.scale_mask = conf.scale_quant ? attr()->output_scales_.mask_ : 0;
-    conf.scales_num = conf.scale_quant ? attr()->output_scales_.count_ : 0;
+    const memory_desc_wrapper original_src_mdw(src_md());
+    const memory_desc_wrapper original_dst_mdw(dst_md());
+    quantization_t src_quant(attr(), original_src_mdw, DNNL_ARG_SRC);
+    quantization_t dst_quant(attr(), original_dst_mdw, DNNL_ARG_DST);
+
+    auto src_mask = src_quant.scale_mask();
+    auto dst_mask = dst_quant.scale_mask();
 
     memory_desc_t new_a;
     memory_desc_t new_b;
+    primitive_attr_t attr_copy = *attr();
     memcpy(&new_a, src_md(), sizeof(new_a));
     memcpy(&new_b, dst_md(), sizeof(new_b));
+    compress(new_a, new_b, src_mask, dst_mask);
+    if (src_mask) CHECK(attr_copy.scales_.set(DNNL_ARG_SRC, src_mask));
+    if (dst_mask) CHECK(attr_copy.scales_.set(DNNL_ARG_DST, dst_mask));
 
-#ifndef PROBLEM_WITH_CONCAT_FIXED
-    // In some configurations concat primitive is implemented as series of
-    // reorders. When this kernel is used, results are wrong.
-    if (new_a.offset0 != 0 || new_b.offset0 != 0) {
-        return status::unimplemented;
-    }
-#endif
-
-    while (try_combine_dims(new_a, new_b, conf.scale_mask, adjacent)) {}
+    if (!is_generic_faster_than_ref(new_a, new_b)) return status::unimplemented;
 
     const memory_desc_wrapper src_mdw(new_a);
     const memory_desc_wrapper dst_mdw(new_b);
     conf.src_md_info = memory_desc_info_t::create(src_mdw);
     conf.dst_md_info = memory_desc_info_t::create(dst_mdw);
 
+    conf.src_quant = {&attr_copy, src_mdw, DNNL_ARG_SRC};
+    conf.dst_quant = {&attr_copy, dst_mdw, DNNL_ARG_DST};
+    conf.sum_quant = {&attr_copy};
+
     status_t status = status::success;
 
     const auto &padded_dims = dst_mdw.padded_dims();
-    conf.with_sum_ab = (alpha() != 1.f || beta() != 0.f);
-    conf.with_sum_a = conf.with_sum_ab && beta() == 0.f;
     conf.has_padding = !src_mdw.is_dense() || !dst_mdw.is_dense();
     conf.ndims = src_mdw.ndims();
     conf.nelems = utils::array_product(padded_dims, conf.ndims);
@@ -773,7 +823,7 @@ status_t generic_reorder_t::pd_t::init_conf(engine_t *engine) {
     int vect_size = 1;
     int vect_dim = 0;
 
-    if (!fill_conf_vld(src_mdw, dst_mdw, conf.scale_mask, memlimit_bytes,
+    if (!fill_conf_vld(src_mdw, dst_mdw, src_mask | dst_mask, memlimit_bytes,
                 optimal_burst_bytes, conf.aux_data.vld, vect_dim, vect_size,
                 &blocks[0])) {
         return status::unimplemented;
@@ -823,15 +873,9 @@ status_t generic_reorder_t::pd_t::init_kernel_ctx(
     kernel_ctx.define_int("NDIMS", conf.ndims);
     kernel_ctx.add_option("-cl-std=CL2.0");
 
-    if (conf.with_sum_a)
-        kernel_ctx.define_int("WITH_SUM_A", 1);
-    else if (conf.with_sum_ab)
-        kernel_ctx.define_int("WITH_SUM_AB", 1);
-
-    if (conf.scale_quant) {
-        kernel_ctx.define_int("SCALE_QUANT", 1);
-        kernel_ctx.define_int("SCALE_MASK", conf.scale_mask);
-    }
+    conf.src_quant.define_macros(kernel_ctx, "SRC");
+    conf.dst_quant.define_macros(kernel_ctx, "DST");
+    conf.sum_quant.define_macros(kernel_ctx, "SUM");
 
     def_dispatch(kernel_ctx, conf.dispatch);
 
@@ -910,10 +954,17 @@ status_t generic_reorder_t::pd_t::init_kernel_ctx(
 }
 
 void generic_reorder_t::pd_t::init_scratchpad() {
-    if (conf.scales_num > 0) {
+    if (conf.src_quant.with_scale()) {
         auto scratchpad = scratchpad_registry().registrar();
-        scratchpad.book(memory_tracking::names::key_reorder_scales,
-                conf.scales_num, sizeof(float), OCL_BUFFER_ALIGNMENT);
+        scratchpad.book(memory_tracking::names::key_reorder_src_scales,
+                conf.src_quant.num_scales(), sizeof(float),
+                OCL_BUFFER_ALIGNMENT);
+    }
+    if (conf.dst_quant.with_scale()) {
+        auto scratchpad = scratchpad_registry().registrar();
+        scratchpad.book(memory_tracking::names::key_reorder_dst_scales,
+                conf.dst_quant.num_scales(), sizeof(float),
+                OCL_BUFFER_ALIGNMENT);
     }
 }
 
@@ -927,31 +978,18 @@ status_t generic_reorder_t::execute(const exec_ctx_t &ctx) const {
 
     const auto &conf = pd()->conf;
     if (conf.nelems == 0) { return status::success; }
-    const float alpha = pd()->alpha();
-    const float beta = pd()->beta();
-
-    std::unique_ptr<memory_storage_t> scales;
-    if (conf.scale_quant) {
-        scales = ctx.get_scratchpad_grantor().get_memory_storage(
-                key_reorder_scales);
-
-        void *tmp_ptr = nullptr;
-        status = scales->map_data(&tmp_ptr, ctx.stream(),
-                sizeof(float) * pd()->attr()->output_scales_.count_);
-        if (status != status::success) { return status; }
-        utils::array_copy((float *)tmp_ptr,
-                pd()->attr()->output_scales_.scales_,
-                pd()->attr()->output_scales_.count_);
-        status = scales->unmap_data(tmp_ptr, ctx.stream());
-        if (status != status::success) { return status; }
-    }
 
     compute::kernel_arg_list_t arg_list;
     arg_list.set(0, src);
     arg_list.set(1, dst);
-    arg_list.set(2, alpha);
-    arg_list.set(3, beta);
-    arg_list.set(4, scales ? *scales : memory_storage_t::empty_storage());
+
+    arg_list.set(2, conf.src_quant.scales(ctx));
+    arg_list.set(3, conf.src_quant.zero_points(ctx));
+    arg_list.set(4, conf.dst_quant.scales(ctx));
+    arg_list.set(5, conf.dst_quant.zero_points(ctx));
+
+    arg_list.set(6, conf.sum_quant.scales());
+    arg_list.set(7, conf.sum_quant.zero_points());
 
     auto nd_range = conf.dispatch.nd_range();
 
