@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2020-2023 Intel Corporation
+* Copyright 2020-2022 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -52,35 +52,16 @@ float linear_bwd(float dd, float alpha) {
     return dd * alpha;
 }
 
-float bounded_relu_fwd(float s, float alpha) {
-    s = s > 0 ? s : 0;
-    return s > alpha ? alpha : s;
+float soft_relu_fwd(float s, float alpha) {
+    s = alpha * s;
+    float v = (s < log((float)DATA_MAX) ? log1p(exp(s)) : s);
+    return v / alpha;
 }
-float bounded_relu_bwd(float dd, float s, float alpha) {
-    return dd * (0 < s && s <= alpha ? 1 : 0);
-}
-
-float soft_relu_fwd(float s) {
-    return s < log((float)DATA_MAX) ? log1p(exp(s)) : s;
-}
-float soft_relu_bwd(float dd, float s) {
+float soft_relu_bwd(float dd, float s, float alpha) {
+    s = alpha * s;
     return dd / (1 + exp(-s));
 }
 
-float soft_relu_v2_fwd(float s, float alpha) {
-    return soft_relu_fwd(s * alpha) / alpha;
-}
-float soft_relu_v2_bwd(float dd, float s, float alpha) {
-    return soft_relu_bwd(dd, s * alpha);
-}
-
-float logsigmoid_fwd(float s) {
-    return -soft_relu_fwd(-s);
-}
-float logsigmoid_bwd(float dd, float s) {
-    return soft_relu_bwd(dd, -s);
-}
-
 float logistic_fwd(float s) {
     return 1.0f / (1.0f + exp(-s));
 }
@@ -128,11 +109,11 @@ float tanh_bwd_use_dst(float dd, float d) {
 }
 
 float mish_fwd(float s) {
-    return s * tanh_fwd(soft_relu_fwd(s));
+    return s * tanh_fwd(soft_relu_fwd(s, 1.f));
 }
 float mish_bwd(float dd, float s) {
-    const float tanh = tanh_fwd(soft_relu_fwd(s));
-    const float srelu_bwd = soft_relu_bwd(1.0f, s);
+    const float tanh = tanh_fwd(soft_relu_fwd(s, 1.f));
+    const float srelu_bwd = soft_relu_bwd(1.f, s, 1.f);
     const float derivative = tanh + s * srelu_bwd * (1 - pow(tanh, 2.0f));
     return dd * derivative;
 }
@@ -243,13 +224,13 @@ float hardsigmoid_bwd(float dd, float s, float alpha, float beta) {
     return v <= 0.f ? 0.f : v >= 1.f ? 0.f : dd * alpha;
 }
 
-float hardswish_fwd(float s) {
-    return (s / 6.f) * bounded_relu_fwd(s + 3.f, 6.f);
+float hardswish_fwd(float s, float alpha, float beta) {
+    return s * hardsigmoid_fwd(s, alpha, beta);
 }
-float hardswish_bwd(float dd, float s) {
-    return (s < 3.f && s > -3.f ? dd * (2 * s + 3.f) / 6.f
-                    : s >= 3.f  ? dd
-                                : 0.f);
+float hardswish_bwd(float dd, float s, float alpha, float beta) {
+    float v = alpha * s + beta;
+    float w = 2.f * alpha * s + beta;
+    return (v <= 0.f ? 0.f : v >= 1.f ? dd : dd * w);
 }
 
 float fwd_eltwise_common(
@@ -257,10 +238,7 @@ float fwd_eltwise_common(
     switch (eltwise_alg) {
         case RELU: return scale_ * relu_fwd(x, alpha_); break;
         case LINEAR: return scale_ * linear_fwd(x, alpha_, beta_); break;
-        case BOUNDED_RELU: return scale_ * bounded_relu_fwd(x, alpha_); break;
-        case SOFT_RELU: return scale_ * soft_relu_fwd(x); break;
-        case SOFT_RELU_V2: return scale_ * soft_relu_v2_fwd(x, alpha_); break;
-        case LOGSIGMOID: return scale_ * logsigmoid_fwd(x); break;
+        case SOFT_RELU: return scale_ * soft_relu_fwd(x, alpha_); break;
         case MISH: return scale_ * mish_fwd(x); break;
         case LOGISTIC: return scale_ * logistic_fwd(x); break;
         case TANH: return scale_ * tanh_fwd(x); break;
@@ -277,7 +255,7 @@ float fwd_eltwise_common(
         case POW: return scale_ * pow_fwd(x, alpha_, beta_); break;
         case GELU_ERF: return scale_ * gelu_erf_fwd(x); break;
         case ROUND: return scale_ * round_fwd(x); break;
-        case HARDSWISH: return scale_ * hardswish_fwd(x); break;
+        case HARDSWISH: return scale_ * hardswish_fwd(x, alpha_, beta_); break;
         case HARDSIGMOID:
             return scale_ * hardsigmoid_fwd(x, alpha_, beta_);
             break;
@@ -306,10 +284,7 @@ float bwd_eltwise(float x, float y, float alpha_, float beta_) {
     switch (ELTWISE_ALG) {
         case RELU: return relu_bwd(x, y, alpha_); break;
         case LINEAR: return linear_bwd(x, alpha_); break;
-        case BOUNDED_RELU: return bounded_relu_bwd(x, y, alpha_); break;
-        case SOFT_RELU: return soft_relu_bwd(x, y); break;
-        case SOFT_RELU_V2: return soft_relu_v2_bwd(x, y, alpha_); break;
-        case LOGSIGMOID: return logsigmoid_bwd(x, y); break;
+        case SOFT_RELU: return soft_relu_bwd(x, y, alpha_); break;
         case MISH: return mish_bwd(x, y); break;
         case LOGISTIC: return logistic_bwd(x, y); break;
         case TANH: return tanh_bwd(x, y); break;
@@ -325,7 +300,7 @@ float bwd_eltwise(float x, float y, float alpha_, float beta_) {
         case CLIP_V2: return clip_v2_bwd(x, y, alpha_, beta_); break;
         case POW: return pow_bwd(x, y, alpha_, beta_); break;
         case GELU_ERF: return gelu_erf_bwd(x, y); break;
-        case HARDSWISH: return hardswish_bwd(x, y); break;
+        case HARDSWISH: return hardswish_bwd(x, y, alpha_, beta_); break;
         case HARDSIGMOID: return hardsigmoid_bwd(x, y, alpha_, beta_); break;
 
         case RELU_DST: return relu_bwd_use_dst(x, y, alpha_); break;
