@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -25,13 +25,10 @@
 #include "common/utils.hpp"
 #include "gpu/compute/compute.hpp"
 #include "gpu/gpu_impl_list.hpp"
+#include "gpu/ocl/ocl_gpu_engine_id.hpp"
 #include "gpu/ocl/ocl_gpu_kernel.hpp"
 #include "gpu/ocl/ocl_utils.hpp"
 
-#ifdef DNNL_USE_RT_OBJECTS_IN_PRIMITIVE_CACHE
-#include "gpu/ocl/ocl_gpu_engine_id.hpp"
-#endif
-
 namespace dnnl {
 namespace impl {
 namespace gpu {
@@ -69,8 +66,6 @@ public:
             const char *source_string,
             const compute::kernel_ctx_t &kernel_ctx) const override;
 
-    std::function<void(void *)> get_program_list_deleter() const override;
-
     const impl_list_item_t *get_concat_implementation_list() const override {
         return gpu_impl_list_t::get_concat_implementation_list();
     }
@@ -108,15 +103,12 @@ public:
         return device_info_->get_cache_blob(size, cache_blob);
     }
 
-#ifdef DNNL_USE_RT_OBJECTS_IN_PRIMITIVE_CACHE
     engine_id_t engine_id() const override {
         return engine_id_t(new ocl_gpu_engine_id_impl_t(
                 device(), context(), kind(), runtime_kind(), index()));
     }
 
 protected:
-#endif
-
     ~ocl_gpu_engine_t() override {
         if (device_) { clReleaseDevice(device_); }
         if (context_) { clReleaseContext(context_); }
