@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -19,6 +19,9 @@
 
 #include "gpu/ocl/ocl_math_utils.h"
 
+#define auto __auto_type
+#define typeof(x) __typeof__(x)
+
 #define unroll_for __attribute__((opencl_unroll_hint)) for
 
 #define for_ for
@@ -33,6 +36,12 @@
 #pragma OPENCL EXTENSION cl_khr_fp16 : enable
 #endif
 
+#if DT_F64
+#pragma OPENCL EXTENSION cl_khr_fp64 : enable
+#endif
+
+#define dim_t long // 64 bit per the OpenCL specification
+
 #if DT_F32 == 1
 #define DATA_T float
 #define DATA2_T float2
@@ -49,7 +58,10 @@
 #define DEF_ACC_DATA8_T float8
 #define POST_OP_DATA_T float
 #define TO_DATA_T(v) (float)(v)
-#define TO_DEF_ACC_DATA_T(v) (float)(v)
+#define TO_DEF_ACC_DATA_T convert_float
+#define TO_DEF_ACC_DATA2_T convert_float2
+#define TO_DEF_ACC_DATA4_T convert_float4
+#define TO_DEF_ACC_DATA8_T convert_float8
 #define DATA_TO_REF convert_float
 #define CONVERT_DATA_T convert_float
 #define CONVERT_DATA2_T convert_float2
@@ -94,6 +106,38 @@
 #define MMAD_ACC_DATA4_T float4
 #define MMAD_ACC_DATA8_T float8
 
+#elif DT_F64 == 1
+#define DATA_T double
+#define DATA2_T double2
+#define DATA4_T double4
+#define DATA8_T double8
+#define DATA16_T double16
+#define DATA_MAX DBL_MAX
+#define DATA_MIN -DATA_MAX
+#define DATA_ZERO 0.0d
+#define DATA_ONE 1.0d
+#define DEF_ACC_DATA_T double
+#define DEF_ACC_DATA2_T double2
+#define DEF_ACC_DATA4_T double4
+#define DEF_ACC_DATA8_T double8
+#define POST_OP_DATA_T double
+#define TO_DATA_T(v) (double)(v)
+#define TO_DEF_ACC_DATA_T(v) (double)(v)
+#define DATA_TO_REF convert_float
+#define CONVERT_DATA_T convert_double
+#define CONVERT_DATA2_T convert_double2
+#define CONVERT_DATA4_T convert_double4
+#define CONVERT_DATA8_T convert_double8
+#define CONVERT_FLOAT_T convert_float
+#define CONVERT_FLOAT2_T convert_float2
+#define CONVERT_FLOAT4_T convert_float4
+#define CONVERT_FLOAT8_T convert_float8
+
+#define AS_DATA_T as_double
+#define AS_DATA2_T as_double2
+#define AS_DATA4_T as_double4
+#define AS_DATA8_T as_double8
+
 #elif DT_F16 == 1
 
 #define DATA_T half
@@ -106,13 +150,16 @@
 #define DATA_MIN -DATA_MAX
 #define DATA_ZERO 0.0h
 #define DATA_ONE 1.0h
-#define DEF_ACC_DATA_T half
-#define DEF_ACC_DATA2_T half2
-#define DEF_ACC_DATA4_T half4
-#define DEF_ACC_DATA8_T half8
-#define POST_OP_DATA_T half
-#define TO_DATA_T(v) (half)(v)
-#define TO_DEF_ACC_DATA_T(v) (half)(v)
+#define DEF_ACC_DATA_T float
+#define DEF_ACC_DATA2_T float2
+#define DEF_ACC_DATA4_T float4
+#define DEF_ACC_DATA8_T float8
+#define POST_OP_DATA_T float
+#define TO_DATA_T convert_half
+#define TO_DEF_ACC_DATA_T convert_float
+#define TO_DEF_ACC_DATA2_T convert_float2
+#define TO_DEF_ACC_DATA4_T convert_float4
+#define TO_DEF_ACC_DATA8_T convert_float8
 #define DATA_TO_REF convert_half
 #define CONVERT_DATA_T convert_half
 #define CONVERT_DATA2_T convert_half2
@@ -163,21 +210,24 @@
 #define DATA4_T ushort4
 #define DATA8_T ushort8
 #define DATA16_T ushort16
-#define DATA_MAX as_float(0x7f7f0000)
-#define DATA_MIN (-DATA_MAX)
-#define DATA_ZERO 0.0f
-#define DATA_ONE 1.0f
+#define DATA_MAX (ushort)0x7F7F
+#define DATA_MIN (ushort)0xFF7F
+#define DATA_ZERO (ushort)0x0000
+#define DATA_ONE (ushort)0x3F80
 #define DEF_ACC_DATA_T float
 #define DEF_ACC_DATA2_T float2
 #define DEF_ACC_DATA4_T float4
 #define DEF_ACC_DATA8_T float8
-#define TO_DATA_T(v) cvt_f32_to_bf16(v)
-#define TO_DEF_ACC_DATA_T(v) cvt_bf16_to_f32(v)
+#define TO_DATA_T cvt_f32_to_bf16
+#define TO_DEF_ACC_DATA_T cvt_bf16_to_f32
+#define TO_DEF_ACC_DATA2_T cvt_bf16_to_f32
+#define TO_DEF_ACC_DATA4_T cvt_bf16_to_f32
+#define TO_DEF_ACC_DATA8_T cvt_bf16_to_f32
 #define DATA_TO_REF cvt_bf16_to_f32
-#define CONVERT_DATA_T cvt_f32_to_bf16
-#define CONVERT_DATA2_T cvt_f32_to_bf16
-#define CONVERT_DATA4_T cvt_f32_to_bf16
-#define CONVERT_DATA8_T cvt_f32_to_bf16
+#define CONVERT_DATA_T(v) cvt_f32_to_bf16(convert_float(v))
+#define CONVERT_DATA2_T(v) cvt_f32_to_bf16(convert_float2(v))
+#define CONVERT_DATA4_T(v) cvt_f32_to_bf16(convert_float4(v))
+#define CONVERT_DATA8_T(v) cvt_f32_to_bf16(convert_float8(v))
 #define CONVERT_FLOAT_T cvt_bf16_to_f32
 #define CONVERT_FLOAT2_T cvt_bf16_to_f32
 #define CONVERT_FLOAT4_T cvt_bf16_to_f32
@@ -232,7 +282,10 @@
 #define DEF_ACC_DATA8_T int8
 #define POST_OP_DATA_T float
 #define TO_DATA_T(v) convert_char_sat_rte(v)
-#define TO_DEF_ACC_DATA_T(v) (float)(v)
+#define TO_DEF_ACC_DATA_T convert_int_sat_rte
+#define TO_DEF_ACC_DATA2_T convert_int2_sat_rte
+#define TO_DEF_ACC_DATA4_T convert_int4_sat_rte
+#define TO_DEF_ACC_DATA8_T convert_int8_sat_rte
 #define DATA_TO_REF convert_float
 #define CONVERT_DATA_T convert_char_sat_rte
 #define CONVERT_DATA2_T convert_char2_sat_rte
@@ -294,7 +347,10 @@
 #define DEF_ACC_DATA8_T int8
 #define POST_OP_DATA_T float
 #define TO_DATA_T(v) convert_uchar_sat_rte(v)
-#define TO_DEF_ACC_DATA_T(v) (float)(v)
+#define TO_DEF_ACC_DATA_T convert_int_sat_rte
+#define TO_DEF_ACC_DATA2_T convert_int2_sat_rte
+#define TO_DEF_ACC_DATA4_T convert_int4_sat_rte
+#define TO_DEF_ACC_DATA8_T convert_int8_sat_rte
 #define DATA_TO_REF convert_float
 #define CONVERT_DATA_T convert_uchar_sat_rte
 #define CONVERT_DATA2_T convert_uchar2_sat_rte
@@ -348,7 +404,13 @@
 #define DATA4_T int4
 #define DATA8_T int8
 #define DATA16_T int16
+#define DATA_MAX INT_MAX
+#define DATA_MIN INT_MIN
+#define DATA_ZERO 0
+#define DATA_ONE 1
 #define DATA_TO_REF convert_float
+#define TO_DATA_T(v) convert_int_sat_rte
+#define TO_DATA8_T(v) convert_int8_sat_rte
 #define CONVERT_DATA_T convert_int_sat_rte
 #define CONVERT_DATA2_T convert_int2_sat_rte
 #define CONVERT_DATA4_T convert_int4_sat_rte
@@ -357,6 +419,15 @@
 #define CONVERT_FLOAT2_T convert_float2
 #define CONVERT_FLOAT4_T convert_float4
 #define CONVERT_FLOAT8_T convert_float8
+#define DEF_ACC_DATA_T int
+#define DEF_ACC_DATA2_T int2
+#define DEF_ACC_DATA4_T int4
+#define DEF_ACC_DATA8_T int8
+#define TO_DATA_T(v) convert_int_sat_rte(v)
+#define TO_DEF_ACC_DATA_T convert_int_sat_rte
+#define TO_DEF_ACC_DATA2_T convert_int2_sat_rte
+#define TO_DEF_ACC_DATA4_T convert_int4_sat_rte
+#define TO_DEF_ACC_DATA8_T convert_int8_sat_rte
 #define POST_OP_DATA_T float
 #define DATA_MIN INT_MIN
 #define DATA_MAX INT_MAX
@@ -397,6 +468,7 @@
 #if VECT_DT_N == 1
 #define VECT_DATA_T DATA_T
 #define VECT_DEF_ACC_DATA_T DEF_ACC_DATA_T
+#define AS_VECT_DEF_ACC_DATA_T TO_DEF_ACC_DATA_T
 #define AS_VECT_DATA_T AS_DATA_T
 #define VECT_BLOCK_READ BLOCK_READ
 #define VECT_BLOCK_WRITE BLOCK_WRITE
@@ -408,6 +480,8 @@
 #define AS_VECT_BLOCK_DATA_T AS_BLOCK_DATA_T
 #define CONVERT_VECT_FLOAT_T CONVERT_FLOAT_T
 #define CONVERT_VECTOR_DATA_T CONVERT_DATA_T
+#define CONVERT_VECT_CHAR_T convert_char
+#define CONVERT_VECT_INT_T convert_int
 #define VECT_INT_T int
 #define VECT_UINT_T uint
 #define VECT_FLOAT_T float
@@ -416,9 +490,11 @@
 #define AS_VECT_UINT_T as_uint
 #define AS_VECT_FLOAT_T as_float
 #define AS_VECT_CHAR_T as_char
+#define AS_VECT_UCHAR_T as_uchar
 #elif VECT_DT_N == 2
 #define VECT_DATA_T DATA2_T
 #define VECT_DEF_ACC_DATA_T DEF_ACC_DATA2_T
+#define AS_VECT_DEF_ACC_DATA_T TO_DEF_ACC_DATA2_T
 #define AS_VECT_DATA_T AS_DATA2_T
 #define VECT_BLOCK_READ BLOCK_READ2
 #define VECT_BLOCK_WRITE BLOCK_WRITE2
@@ -430,6 +506,8 @@
 #define AS_VECT_BLOCK_DATA_T AS_BLOCK_DATA2_T
 #define CONVERT_VECT_FLOAT_T CONVERT_FLOAT2_T
 #define CONVERT_VECTOR_DATA_T CONVERT_DATA2_T
+#define CONVERT_VECT_CHAR_T convert_char2
+#define CONVERT_VECT_INT_T convert_int2
 #define VECT_INT_T int2
 #define VECT_UINT_T uint2
 #define VECT_FLOAT_T float2
@@ -438,9 +516,11 @@
 #define AS_VECT_UINT_T as_uint2
 #define AS_VECT_FLOAT_T as_float2
 #define AS_VECT_CHAR_T as_char2
+#define AS_VECT_UCHAR_T as_uchar2
 #elif VECT_DT_N == 4
 #define VECT_DATA_T DATA4_T
 #define VECT_DEF_ACC_DATA_T DEF_ACC_DATA4_T
+#define AS_VECT_DEF_ACC_DATA_T TO_DEF_ACC_DATA4_T
 #define AS_VECT_DATA_T AS_DATA4_T
 #define VECT_BLOCK_READ BLOCK_READ4
 #define VECT_BLOCK_WRITE BLOCK_WRITE4
@@ -452,6 +532,8 @@
 #define AS_VECT_BLOCK_DATA_T AS_BLOCK_DATA4_T
 #define CONVERT_VECT_FLOAT_T CONVERT_FLOAT4_T
 #define CONVERT_VECTOR_DATA_T CONVERT_DATA4_T
+#define CONVERT_VECT_CHAR_T convert_char4
+#define CONVERT_VECT_INT_T convert_int4
 #define VECT_INT_T int4
 #define VECT_UINT_T uint4
 #define VECT_FLOAT_T float4
@@ -460,9 +542,11 @@
 #define AS_VECT_UINT_T as_uint4
 #define AS_VECT_FLOAT_T as_float4
 #define AS_VECT_CHAR_T as_char4
+#define AS_VECT_UCHAR_T as_uchar4
 #elif VECT_DT_N == 8
 #define VECT_DATA_T DATA8_T
 #define VECT_DEF_ACC_DATA_T DEF_ACC_DATA8_T
+#define AS_VECT_DEF_ACC_DATA_T TO_DEF_ACC_DATA8_T
 #define AS_VECT_DATA_T AS_DATA8_T
 #define VECT_BLOCK_READ BLOCK_READ8
 #define VECT_BLOCK_WRITE BLOCK_WRITE8
@@ -474,6 +558,8 @@
 #define AS_VECT_BLOCK_DATA_T AS_BLOCK_DATA8_T
 #define CONVERT_VECT_FLOAT_T CONVERT_FLOAT8_T
 #define CONVERT_VECTOR_DATA_T CONVERT_DATA8_T
+#define CONVERT_VECT_CHAR_T convert_char8
+#define CONVERT_VECT_INT_T convert_int8
 #define VECT_INT_T int8
 #define VECT_UINT_T uint8
 #define VECT_FLOAT_T float8
@@ -482,6 +568,7 @@
 #define AS_VECT_UINT_T as_uint8
 #define AS_VECT_FLOAT_T as_float8
 #define AS_VECT_CHAR_T as_char8
+#define AS_VECT_UCHAR_T as_uchar8
 #endif
 
 #define AS_MMAD_DATA_T CONCAT2(as_, MMAD_DATA_T)
@@ -877,6 +964,11 @@
 #define TO_DST2(x) convert_float2(x)
 #define TO_DST4(x) convert_float4(x)
 #define TO_DST8(x) convert_float8(x)
+#elif DST_DT_F64
+#define TO_DST(x) convert_double(x)
+#define TO_DST2(x) convert_double2(x)
+#define TO_DST4(x) convert_double4(x)
+#define TO_DST8(x) convert_double8(x)
 #else
 #error "Not expected"
 #endif
@@ -913,6 +1005,9 @@
 #elif C_DT_F32
 #define TO_C(x) convert_float(x)
 #define TO_C8(x) convert_float8(x)
+#elif C_DT_F64
+#define TO_C(x) convert_double(x)
+#define TO_C8(x) convert_double8(x)
 #else
 #error "Not expected"
 #endif
@@ -923,6 +1018,8 @@
 #define TO_ACC(x) convert_half(x)
 #elif ACC_DT_F32
 #define TO_ACC(x) convert_float(x)
+#elif ACC_DT_F64
+#define TO_ACC(x) convert_double(x)
 #elif ACC_DT_S32
 #define TO_ACC(x) convert_int(x)
 #else
@@ -1226,6 +1323,20 @@
 #define GWS2_GET_BLOCK4() GWS2_BLOCK4
 #define GWS2_GET_BLOCK5() GWS2_BLOCK5
 
+#define GWS3_GET_ID0() GWS3_OP0((get_global_id(GWS3_IDX0) / GWS3_STRIDE0), ROUND_UP(GWS3_DIM0, GWS3_BLOCK0)) / GWS3_VEC_SIZE0 * GWS3_VEC_SIZE0 * GWS3_BLOCK0
+#define GWS3_GET_ID1() GWS3_OP1((get_global_id(GWS3_IDX1) / GWS3_STRIDE1), ROUND_UP(GWS3_DIM1, GWS3_BLOCK1)) / GWS3_VEC_SIZE1 * GWS3_VEC_SIZE1 * GWS3_BLOCK1
+#define GWS3_GET_ID2() GWS3_OP2((get_global_id(GWS3_IDX2) / GWS3_STRIDE2), ROUND_UP(GWS3_DIM2, GWS3_BLOCK2)) / GWS3_VEC_SIZE2 * GWS3_VEC_SIZE2 * GWS3_BLOCK2
+#define GWS3_GET_ID3() GWS3_OP3((get_global_id(GWS3_IDX3) / GWS3_STRIDE3), ROUND_UP(GWS3_DIM3, GWS3_BLOCK3)) / GWS3_VEC_SIZE3 * GWS3_VEC_SIZE3 * GWS3_BLOCK3
+#define GWS3_GET_ID4() GWS3_OP4((get_global_id(GWS3_IDX4) / GWS3_STRIDE4), ROUND_UP(GWS3_DIM4, GWS3_BLOCK4)) / GWS3_VEC_SIZE4 * GWS3_VEC_SIZE4 * GWS3_BLOCK4
+#define GWS3_GET_ID5() GWS3_OP5((get_global_id(GWS3_IDX5) / GWS3_STRIDE5), ROUND_UP(GWS3_DIM5, GWS3_BLOCK5)) / GWS3_VEC_SIZE5 * GWS3_VEC_SIZE5 * GWS3_BLOCK5
+
+#define GWS3_GET_BLOCK0() GWS3_BLOCK0
+#define GWS3_GET_BLOCK1() GWS3_BLOCK1
+#define GWS3_GET_BLOCK2() GWS3_BLOCK2
+#define GWS3_GET_BLOCK3() GWS3_BLOCK3
+#define GWS3_GET_BLOCK4() GWS3_BLOCK4
+#define GWS3_GET_BLOCK5() GWS3_BLOCK5
+
 // clang-format on
 
 // With work-group qualifier, without sub-group qualifier.
@@ -1270,3 +1381,106 @@
     } while (0)
 
 #endif
+
+#if SRC_DT_U8 == 1
+#define SRC_DT_ALIAS UCHAR
+#elif SRC_DT_S8 == 1
+#define SRC_DT_ALIAS CHAR
+#elif SRC_DT_F16 == 1
+#define SRC_DT_ALIAS HALF
+#elif SRC_DT_BF16 == 1
+#define SRC_DT_ALIAS BFLOAT
+#elif SRC_DT_F32 == 1
+#define SRC_DT_ALIAS FLOAT
+#endif
+
+#if DST_DT_U8 == 1
+#define DST_DT_ALIAS UCHAR
+#elif DST_DT_S8 == 1
+#define DST_DT_ALIAS CHAR
+#elif DST_DT_F16 == 1
+#define DST_DT_ALIAS HALF
+#elif DST_DT_BF16 == 1
+#define DST_DT_ALIAS BFLOAT
+#elif DST_DT_F32 == 1
+#define DST_DT_ALIAS FLOAT
+#endif
+
+#define ALIAS(prefix) CONCAT2(prefix, _DT_ALIAS)
+
+// BLOCK types
+#define BLOCK1_T uchar
+#define BLOCK2_T ushort
+#define BLOCK4_T uint
+#define BLOCK8_T ulong
+#define BLOCK_T(alias) CONCAT3(BLOCK, SIZEOF(alias), _T)
+
+#define BLOCK1_ALIAS UCHAR
+#define BLOCK2_ALIAS USHORT
+#define BLOCK4_ALIAS UINT
+#define BLOCK_ALIAS(prefix) CONCAT3(BLOCK, SIZEOF(ALIAS(prefix)), _ALIAS)
+
+#define SIZEOF_UCHAR 1
+#define SIZEOF_CHAR 1
+#define SIZEOF_BFLOAT 2
+#define SIZEOF_HALF 2
+#define SIZEOF_FLOAT 4
+#define SIZEOF(alias) CONCAT2(SIZEOF_, alias)
+
+#define READ_UCHAR8 intel_sub_group_block_read_uc8
+#define READ_USHORT8 intel_sub_group_block_read_us8
+#define READ_UINT8 intel_sub_group_block_read8
+#define READ_BLOCK8(prefix, ptr) READ_BLOCK_N(prefix, 8)(ptr)
+#define READ_BLOCK_N(prefix, n) CONCAT3(READ_, BLOCK_ALIAS(prefix), n)
+
+#define WRITE_UCHAR8 intel_sub_group_block_write_uc8
+#define WRITE_USHORT8 intel_sub_group_block_write_us8
+#define WRITE_UINT8 intel_sub_group_block_write8
+#define WRITE_BLOCK8(prefix, ptr, val) WRITE_BLOCK_N(prefix, 8)(ptr, val)
+#define WRITE_BLOCK_N(prefix, n) CONCAT3(WRITE_, BLOCK_ALIAS(prefix), n)
+
+#define AS_UCHAR8 as_uchar8
+#define AS_CHAR8 as_char8
+#define AS_HALF8 as_half8
+#define AS_USHORT8 as_ushort8
+#define AS_BFLOAT8 as_ushort8
+#define AS_FLOAT8 as_float8
+#define AS_INT8 as_int8
+#define AS_UINT8 as_uint8
+
+#define BLOCK_TO_DATA8(prefix, val) BLOCK_TO_DATA_N(prefix, 8)(val)
+#define BLOCK_TO_DATA_N(prefix, n) CONCAT3(AS_, ALIAS(prefix), n)
+
+#define DATA_TO_BLOCK8(prefix, val) DATA_TO_BLOCK_N(prefix, 8)(val)
+#define DATA_TO_BLOCK_N(prefix, n) CONCAT3(AS_, BLOCK_ALIAS(prefix), n)
+
+#define UCHAR_TO_FLOAT1 convert_float
+#define UCHAR_TO_FLOAT8 convert_float8
+#define FLOAT_TO_UCHAR1 convert_uchar_sat_rte
+#define FLOAT_TO_UCHAR8 convert_uchar8_sat_rte
+
+#define CHAR_TO_FLOAT1 convert_float
+#define CHAR_TO_FLOAT8 convert_float8
+#define FLOAT_TO_CHAR1 convert_char_sat_rte
+#define FLOAT_TO_CHAR8 convert_char8_sat_rte
+
+#define HALF_TO_FLOAT1 convert_float
+#define HALF_TO_FLOAT8 convert_float8
+#define FLOAT_TO_HALF1 convert_half
+#define FLOAT_TO_HALF8 convert_half8
+
+#define BFLOAT_TO_FLOAT1 cvt_bf16_to_f32
+#define BFLOAT_TO_FLOAT8 cvt_bf16_to_f32
+#define FLOAT_TO_BFLOAT1 cvt_f32_to_bf16
+#define FLOAT_TO_BFLOAT8 cvt_f32_to_bf16
+
+#define FLOAT_TO_FLOAT1 convert_float
+#define FLOAT_TO_FLOAT8 convert_float8
+
+#define DATA_TO_FLOAT(prefix, val) DATA_TO_FLOAT_N(prefix, 1)(val)
+#define DATA_TO_FLOAT8(prefix, val) DATA_TO_FLOAT_N(prefix, 8)(val)
+#define DATA_TO_FLOAT_N(prefix, n) CONCAT3(ALIAS(prefix), _TO_FLOAT, n)
+
+#define FLOAT_TO_DATA(prefix, val) FLOAT_TO_DATA_N(prefix, 1)(val)
+#define FLOAT_TO_DATA8(prefix, val) FLOAT_TO_DATA_N(prefix, 8)(val)
+#define FLOAT_TO_DATA_N(prefix, n) CONCAT3(FLOAT_TO_, ALIAS(prefix), n)
