@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -20,6 +20,7 @@
 #include <mutex>
 #include <CL/cl_ext.h>
 
+#include "gpu/ocl/ocl_gpu_engine.hpp"
 #include "gpu/ocl/ocl_gpu_kernel.hpp"
 #include "gpu/ocl/ocl_utils.hpp"
 
@@ -194,28 +195,42 @@ status_t get_ocl_device_index(size_t *index, cl_device_id device) {
     }
 }
 
+cl_platform_id get_ocl_platform(cl_device_id device) {
+    cl_platform_id platform;
+    cl_int err = clGetDeviceInfo(
+            device, CL_DEVICE_PLATFORM, sizeof(platform), &platform, nullptr);
+    if (err != CL_SUCCESS) return nullptr;
+    return platform;
+}
+
+cl_platform_id get_ocl_platform(engine_t *engine) {
+    return utils::downcast<ocl_gpu_engine_t *>(engine)->platform();
+}
+
 status_t get_ocl_kernel_arg_type(compute::scalar_type_t *type,
         cl_kernel ocl_kernel, cl_uint idx, bool allow_undef) {
     char s_type[16];
-    OCL_CHECK(clGetKernelArgInfo(ocl_kernel, idx, CL_KERNEL_ARG_TYPE_NAME,
-            sizeof(s_type), s_type, nullptr));
+    auto cl_status = clGetKernelArgInfo(ocl_kernel, idx,
+            CL_KERNEL_ARG_TYPE_NAME, sizeof(s_type), s_type, nullptr);
+    if (cl_status == CL_SUCCESS) {
 #define CASE(x) \
     if (!strcmp(STRINGIFY(x), s_type)) { \
         *type = compute::scalar_type_t::_##x; \
         return status::success; \
     }
-    CASE(char)
-    CASE(float)
-    CASE(half)
-    CASE(int)
-    CASE(long)
-    CASE(short)
-    CASE(uchar)
-    CASE(uint)
-    CASE(ulong)
-    CASE(ushort)
-    CASE(zero_pad_mask_t)
+        CASE(char)
+        CASE(float)
+        CASE(half)
+        CASE(int)
+        CASE(long)
+        CASE(short)
+        CASE(uchar)
+        CASE(uint)
+        CASE(ulong)
+        CASE(ushort)
+        CASE(zero_pad_mask_t)
 #undef CASE
+    }
 
     if (allow_undef) {
         *type = compute::scalar_type_t::undef;
@@ -231,16 +246,41 @@ cl_mem clCreateBuffer_wrapper(cl_context context, cl_mem_flags flags,
     return clCreateBuffer(context, flags, size, host_ptr, errcode_ret);
 }
 
-status_t get_ocl_program_binary(cl_program program, cl_device_id device,
-        std::shared_ptr<compute::binary_t> &binary) {
-
-    size_t n_devices = 0;
+static status_t get_number_devices(cl_program program, size_t *n_devices) {
     cl_int err = clGetProgramInfo(program, CL_PROGRAM_NUM_DEVICES,
-            sizeof(size_t), &n_devices, nullptr);
+            sizeof(size_t), n_devices, nullptr);
+    OCL_CHECK(err);
+    return status::success;
+}
+
+status_t get_ocl_program_binary_size(
+        cl_kernel kernel, cl_device_id device, size_t *size) {
+    size_t device_idx = 0;
+    CHECK(get_ocl_device_index(&device_idx, device));
+
+    cl_program program;
+    cl_int err = clGetKernelInfo(
+            kernel, CL_KERNEL_PROGRAM, sizeof(program), &program, nullptr);
     OCL_CHECK(err);
 
-    std::vector<size_t> binarySize(n_devices);
+    size_t n_devices = 0;
+    CHECK(get_number_devices(program, &n_devices));
+
+    std::vector<size_t> binary_sizes(n_devices);
     err = clGetProgramInfo(program, CL_PROGRAM_BINARY_SIZES,
+            sizeof(size_t) * n_devices, binary_sizes.data(), nullptr);
+    OCL_CHECK(err);
+    (*size) = binary_sizes[device_idx];
+    return status::success;
+}
+
+status_t get_ocl_program_binary(
+        cl_program program, cl_device_id device, compute::binary_t &binary) {
+    size_t n_devices = 0;
+    CHECK(get_number_devices(program, &n_devices));
+
+    std::vector<size_t> binarySize(n_devices);
+    cl_int err = clGetProgramInfo(program, CL_PROGRAM_BINARY_SIZES,
             sizeof(size_t) * n_devices, binarySize.data(), nullptr);
     OCL_CHECK(err);
 
@@ -252,22 +292,21 @@ status_t get_ocl_program_binary(cl_program program, cl_device_id device,
     size_t device_idx = std::distance(
             devices.begin(), std::find(devices.begin(), devices.end(), device));
     std::vector<uint8_t *> binary_pointers(n_devices);
-    std::vector<std::shared_ptr<compute::binary_t>> binaries(n_devices);
+    std::vector<compute::binary_t> binaries(n_devices);
     for (size_t i = 0; i < n_devices; ++i) {
-        binaries[i] = std::make_shared<compute::binary_t>(binarySize[i]);
-        binary_pointers[i] = binaries[i]->data();
+        binaries[i] = compute::binary_t(binarySize[i]);
+        binary_pointers[i] = binaries[i].data();
     }
 
     err = clGetProgramInfo(program, CL_PROGRAM_BINARIES,
             sizeof(uint8_t *) * n_devices, binary_pointers.data(), nullptr);
     OCL_CHECK(err);
     binary = binaries[device_idx];
-
     return status::success;
 }
 
-status_t get_ocl_program_binary(cl_kernel kernel, cl_device_id device,
-        std::shared_ptr<compute::binary_t> &binary) {
+status_t get_ocl_program_binary(
+        cl_kernel kernel, cl_device_id device, compute::binary_t &binary) {
     cl_int err;
 
     cl_program program;
@@ -320,18 +359,11 @@ void dump_kernel_binary(cl_kernel ocl_kernel) {
 }
 
 void dump_kernel_binary(
-        const engine_t *engine, const compute::kernel_t &binary_kernel) {
+        const engine_t *engine, const compute::kernel_t &kernel) {
     if (!get_jit_dump()) return;
-
-    compute::kernel_t realized_kernel;
-    auto status = binary_kernel.realize(&realized_kernel, engine, nullptr);
-
-    // Ignore error.
-    if (status != status::success) return;
-
-    auto *kernel
-            = utils::downcast<const ocl_gpu_kernel_t *>(realized_kernel.impl());
-    dump_kernel_binary(kernel->ocl_kernel());
+    auto *kernel_impl
+            = utils::downcast<const ocl_gpu_kernel_t *>(kernel.impl());
+    dump_kernel_binary(kernel_impl->ocl_kernel());
 }
 #else
 void dump_kernel_binary(const engine_t *, const compute::kernel_t &) {}
@@ -411,6 +443,23 @@ status_t clone_kernel(cl_kernel kernel, cl_kernel *cloned_kernel) {
     return status::success;
 }
 
+status_t create_ocl_program(gpu::ocl::ocl_wrapper_t<cl_program> &ocl_program,
+        cl_device_id dev, cl_context ctx,
+        const gpu::compute::binary_t *binary) {
+    cl_int err;
+    const unsigned char *binary_buffer = binary->data();
+    size_t binary_size = binary->size();
+    assert(binary_size > 0);
+
+    ocl_program = clCreateProgramWithBinary(
+            ctx, 1, &dev, &binary_size, &binary_buffer, nullptr, &err);
+    OCL_CHECK(err);
+    err = clBuildProgram(ocl_program, 1, &dev, nullptr, nullptr, nullptr);
+    OCL_CHECK(err);
+
+    return status::success;
+}
+
 } // namespace ocl
 } // namespace gpu
 } // namespace impl
