@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -46,7 +46,7 @@ struct ref_convolution_fwd_t : public gpu_primitive_t {
                     = utils::downcast<compute::compute_engine_t *>(engine);
 
             const auto attr_skip_mask
-                    = primitive_attr_t::skip_mask_t::oscale_runtime
+                    = primitive_attr_t::skip_mask_t::scales_runtime
                     | primitive_attr_t::skip_mask_t::zero_points_runtime
                     | primitive_attr_t::skip_mask_t::post_ops
                     | primitive_attr_t::skip_mask_t::sum_dt;
@@ -76,12 +76,9 @@ struct ref_convolution_fwd_t : public gpu_primitive_t {
                     && attr_.set_default_formats(dst_md(0)) == status::success
                     && post_ops_with_binary_ok(
                             attr(), dst_md()->data_type, 5, 0xffff)
-                    && zero_points_ok(attr())
-                    && IMPLICATION(!attr()->output_scales_.has_default_values(),
-                            utils::one_of(src_md_.data_type, s8, u8)
-                                    && utils::one_of(
-                                            attr()->output_scales_.mask_, 0,
-                                            1 << 1));
+                    && attr_scales_ok() && zero_points_ok(attr())
+                    && IMPLICATION(!attr()->scales_.has_default_values(),
+                            utils::one_of(src_md_.data_type, s8, u8));
             if (!ok) return status::unimplemented;
 
             return init_conf(engine);
@@ -90,8 +87,6 @@ struct ref_convolution_fwd_t : public gpu_primitive_t {
         status_t init_conf(engine_t *engine);
         status_t init_kernel_ctx(compute::kernel_ctx_t &kernel_ctx) const;
 
-        const memory_desc_t *scales_md() const { return &scales_md_; }
-
         conv_conf_t conf;
 
     private:
@@ -103,20 +98,6 @@ struct ref_convolution_fwd_t : public gpu_primitive_t {
                     : utils::pick(ndims() - 3, oiw, oihw, oidhw);
             return set_default_formats_common(dat_tag, wei_tag, dat_tag);
         }
-
-        status_t init_scales_md() {
-            if (conf.attr_info.with_per_oc_oscales
-                    && !conf.attr_info.with_runtime_oscales) {
-                scales_md_.data_type = data_type::f32;
-                scales_md_.ndims = 1;
-                scales_md_.dims[0] = attr()->output_scales_.count_;
-                return memory_desc_init_by_tag(scales_md_, format_tag::x);
-            }
-
-            return status::success;
-        }
-
-        memory_desc_t scales_md_;
     };
 
     status_t init(engine_t *engine) override {
@@ -125,7 +106,8 @@ struct ref_convolution_fwd_t : public gpu_primitive_t {
         auto status = pd()->init_kernel_ctx(kernel_ctx);
         if (status != status::success) return status;
 
-        create_kernel(engine, &kernel_, "ref_convolution_fwd", kernel_ctx);
+        CHECK(create_kernel(
+                engine, &kernel_, "ref_convolution_fwd", kernel_ctx));
         if (!kernel_) return status::runtime_error;
 
         return status::success;
@@ -135,34 +117,10 @@ struct ref_convolution_fwd_t : public gpu_primitive_t {
         return execute_forward(ctx);
     }
 
-protected:
-    status_t init_res_storage(
-            engine_t *engine, gpu_resource_t *r) const override {
-        if (!pd()->conf.attr_info.with_per_oc_oscales
-                || pd()->conf.attr_info.with_runtime_oscales)
-            return status::success;
-        memory_desc_wrapper scales_mdw(pd()->scales_md());
-        memory_storage_t *tmp_mem_storage_ptr;
-        CHECK(engine->create_memory_storage(
-                &tmp_mem_storage_ptr, scales_mdw.nelems() * sizeof(float)));
-
-        std::unique_ptr<memory_storage_t> tmp_mem_storage(tmp_mem_storage_ptr);
-        void *scales_ptr = nullptr;
-        CHECK(tmp_mem_storage->map_data(&scales_ptr, nullptr,
-                sizeof(float) * pd()->attr()->output_scales_.count_));
-        utils::array_copy((float *)scales_ptr,
-                pd()->attr()->output_scales_.scales_,
-                pd()->attr()->output_scales_.count_);
-        CHECK(tmp_mem_storage->unmap_data(scales_ptr, nullptr));
-        r->add_memory_storage(SCALES_, std::move(tmp_mem_storage));
-        return status::success;
-    }
-
 private:
     status_t execute_forward(const exec_ctx_t &ctx) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
     compute::kernel_t kernel_;
-    enum { SCALES_ = 0 };
 };
 
 struct ref_convolution_bwd_data_t : public gpu_primitive_t {
@@ -174,7 +132,7 @@ struct ref_convolution_bwd_data_t : public gpu_primitive_t {
 
         status_t init(engine_t *engine) {
             using sm = primitive_attr_t::skip_mask_t;
-            const auto attr_skip_mask = sm::post_ops | sm::oscale_runtime
+            const auto attr_skip_mask = sm::post_ops | sm::scales_runtime
                     | sm::zero_points_runtime;
             using namespace data_type;
             const auto *compute_engine
@@ -187,12 +145,7 @@ struct ref_convolution_bwd_data_t : public gpu_primitive_t {
                     && attr()->has_default_values(attr_skip_mask)
                     && post_ops_with_binary_ok(
                             attr(), dst_md()->data_type, ndims())
-                    && zero_points_ok(attr())
-                    && IMPLICATION(!attr()->output_scales_.has_default_values(),
-                            utils::one_of(diff_dst_md()->data_type, u8, s8)
-                                    && utils::one_of(
-                                            attr()->output_scales_.mask_, 0,
-                                            1 << 1))
+                    && attr_scales_ok() && zero_points_ok(attr())
                     && IMPLICATION(utils::one_of(f64, diff_src_md()->data_type,
                                            dst_md()->data_type),
                             compute_engine->mayiuse(
@@ -208,8 +161,6 @@ struct ref_convolution_bwd_data_t : public gpu_primitive_t {
         status_t init_conf(engine_t *engine);
         status_t init_kernel_ctx(compute::kernel_ctx_t &kernel_ctx) const;
 
-        const memory_desc_t *scales_md() const { return &scales_md_; }
-
         conv_conf_t conf;
 
     private:
@@ -221,20 +172,6 @@ struct ref_convolution_bwd_data_t : public gpu_primitive_t {
                     : utils::pick(ndims() - 3, oiw, oihw, oidhw);
             return set_default_formats_common(dat_tag, wei_tag, dat_tag);
         }
-
-        status_t init_scales_md() {
-            if (conf.attr_info.with_per_oc_oscales
-                    && !conf.attr_info.with_runtime_oscales) {
-                scales_md_.data_type = data_type::f32;
-                scales_md_.ndims = 1;
-                scales_md_.dims[0] = attr()->output_scales_.count_;
-                return memory_desc_init_by_tag(scales_md_, format_tag::x);
-            }
-
-            return status::success;
-        }
-
-        memory_desc_t scales_md_;
     };
 
     status_t init(engine_t *engine) override {
@@ -243,7 +180,8 @@ struct ref_convolution_bwd_data_t : public gpu_primitive_t {
         auto status = pd()->init_kernel_ctx(kernel_ctx);
         if (status != status::success) return status;
 
-        create_kernel(engine, &kernel_, "ref_convolution_bwd_data", kernel_ctx);
+        CHECK(create_kernel(
+                engine, &kernel_, "ref_convolution_bwd_data", kernel_ctx));
         if (!kernel_) return status::runtime_error;
 
         return status::success;
@@ -253,34 +191,10 @@ struct ref_convolution_bwd_data_t : public gpu_primitive_t {
         return execute_backward_data(ctx);
     }
 
-protected:
-    status_t init_res_storage(
-            engine_t *engine, gpu_resource_t *r) const override {
-        if (!pd()->conf.attr_info.with_per_oc_oscales
-                || pd()->conf.attr_info.with_runtime_oscales)
-            return status::success;
-        memory_desc_wrapper scales_mdw(pd()->scales_md());
-        memory_storage_t *tmp_mem_storage_ptr;
-        CHECK(engine->create_memory_storage(
-                &tmp_mem_storage_ptr, scales_mdw.nelems() * sizeof(float)));
-
-        std::unique_ptr<memory_storage_t> tmp_mem_storage(tmp_mem_storage_ptr);
-        void *scales_ptr = nullptr;
-        CHECK(tmp_mem_storage->map_data(&scales_ptr, nullptr,
-                sizeof(float) * pd()->attr()->output_scales_.count_));
-        utils::array_copy((float *)scales_ptr,
-                pd()->attr()->output_scales_.scales_,
-                pd()->attr()->output_scales_.count_);
-        CHECK(tmp_mem_storage->unmap_data(scales_ptr, nullptr));
-        r->add_memory_storage(SCALES_, std::move(tmp_mem_storage));
-        return status::success;
-    }
-
 private:
     status_t execute_backward_data(const exec_ctx_t &ctx) const;
     const pd_t *pd() const { return (const pd_t *)primitive_t::pd().get(); }
     compute::kernel_t kernel_;
-    enum { SCALES_ = 0 };
 };
 
 struct ref_convolution_bwd_weights_t : public gpu_primitive_t {
@@ -340,8 +254,8 @@ struct ref_convolution_bwd_weights_t : public gpu_primitive_t {
         auto status = pd()->init_kernel_ctx(kernel_ctx);
         if (status != status::success) return status;
 
-        create_kernel(
-                engine, &kernel_, "ref_convolution_bwd_weights", kernel_ctx);
+        CHECK(create_kernel(
+                engine, &kernel_, "ref_convolution_bwd_weights", kernel_ctx));
         if (!kernel_) return status::runtime_error;
 
         return status::success;
