@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -34,7 +34,6 @@
 #include "common/math_utils.hpp"
 #include "common/type_helpers.hpp"
 #include "gpu/gemm/gpu_gemm.hpp"
-#include "gpu/gemm/gpu_gemm_utils.hpp"
 
 #define DEBUGPRINT 0
 #if DEBUGPRINT
@@ -195,7 +194,8 @@ static status_t init_conf(rnn_conf_t &conf, const rnn_pd_t *rnn_pd,
 }
 
 static status_t init_kernel_ctx(compute::kernel_ctx_t &kernel_ctx,
-        const rnn_conf_t &conf, const rnn_offsets_t &off) {
+        const rnn_conf_t &conf, const rnn_offsets_t &off, int subgroup_size,
+        bool use_subgroup_reduction) {
 
     kernel_ctx.define_int("IS_FWD", conf.is_fwd);
     kernel_ctx.define_int("IS_TRAINING", conf.is_training);
@@ -226,7 +226,6 @@ static status_t init_kernel_ctx(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int("R2L", dnnl_unidirectional_right2left);
     kernel_ctx.define_int("CONCAT", dnnl_bidirectional_concat);
     kernel_ctx.define_int("SUM", dnnl_bidirectional_sum);
-    kernel_ctx.define_int("UNIDEF", dnnl_unidirectional);
     kernel_ctx.define_int("DIRECTION_KIND", conf.direction_kind);
 
     kernel_ctx.define_int("BATCH", conf.batch);
@@ -246,6 +245,9 @@ static status_t init_kernel_ctx(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int("N_PARTS_WEI_ST", conf.n_parts_weights_iter);
     kernel_ctx.define_int("N_PARTS_WEI_I", conf.n_parts_weights_layer);
 
+    kernel_ctx.define_int("SUBGROUP_SIZE", subgroup_size);
+    kernel_ctx.define_int("USE_SUBGROUP_REDUCTION", use_subgroup_reduction);
+
     def_offsets(off.src_layer_off, kernel_ctx, "SRC_L", conf.src_layer_ndims);
     def_offsets(off.src_iter_off, kernel_ctx, "SRC_I", conf.src_iter_ndims);
     if (conf.with_src_iter_c)
@@ -386,13 +388,17 @@ status_t _ref_rnn_common_t<prop_kind::backward>::pd_t::set_default_params() {
     int arch_ld = is_xe_hpc ? 128 : 64;
     if (src_layer_md_.format_kind == format_kind::any)
         CHECK(memory_desc_init_by_tag(src_layer_md_, tnc));
-    if (weights_layer_md_.format_kind == format_kind::any)
+    if (weights_layer_md_.format_kind == format_kind::any) {
         CHECK(memory_desc_init_by_tag(weights_layer_md_, ldgoi));
+        CHECK(rnn_utils::set_good_strides(arch_ld, weights_layer_md_, ldgoi));
+    }
     if (dst_layer_md_.format_kind == format_kind::any)
         CHECK(memory_desc_init_by_tag(dst_layer_md_, tnc));
 
-    if (weights_iter_md_.format_kind == format_kind::any)
+    if (weights_iter_md_.format_kind == format_kind::any) {
         CHECK(memory_desc_init_by_tag(weights_iter_md_, ldgoi));
+        CHECK(rnn_utils::set_good_strides(arch_ld, weights_iter_md_, ldgoi));
+    }
 
     if (diff_src_layer_md_.format_kind == format_kind::any)
         CHECK(memory_desc_init_by_tag(diff_src_layer_md_, tnc));
@@ -457,6 +463,8 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
             = utils::downcast<const compute::compute_engine_t *>(engine);
 
     is_xe_hpc = compute_engine->is_xe_hpc();
+    subgroup_size = compute_engine->device_info()->max_subgroup_size();
+    max_eus_per_wg = compute_engine->device_info()->max_eus_per_wg();
 
     const alg_kind_t cell_kind = this->desc()->cell_kind;
 
@@ -533,11 +541,6 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
         case (prop_kind::forward): break;
         case (prop_kind::backward):
             ok = ok && utils::one_of(this->desc()->prop_kind, backward);
-            ok = ok
-                    && memory_desc_matches_one_of_tag(
-                            this->weights_layer_md_, ldgoi)
-                    && memory_desc_matches_one_of_tag(
-                            this->weights_iter_md_, ldgoi);
             break;
         default: ok = false;
     }
@@ -584,8 +587,7 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
     // initialize the workspace_pd if needed
     if (rnn_conf.use_workspace) {
         dims_t ws_dims = {(dim_t)ws_sz};
-        dnnl_memory_desc_init_by_tag(
-                &this->ws_md_, 1, ws_dims, data_type::u8, x);
+        memory_desc_init_by_tag(this->ws_md_, 1, ws_dims, data_type::u8, x);
     }
 
     rnn_conf.acc_data_type = acc_data_t;
@@ -593,6 +595,9 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
     status_t status = init_conf<aprop>(conf, rnn_conf, this, this->off);
     if (status != status::success) { return status; }
 
+    auto fpmath_mode = this->attr()->fpmath_mode_;
+    use_subgroup_reduction = conf.batch >= subgroup_size;
+
     // The inputs of create_gemm_pd describe a gemm in column major.
     // Below, we have to transpose the a and b descriptor to describe
     // the GEMM as a row major problem.
@@ -612,7 +617,8 @@ status_t _ref_rnn_common_t<aprop>::pd_t::init(engine_t *engine) {
 
         primitive_attr_t attr;
         attr.post_ops_.append_sum(beta);
-        dnnl_primitive_desc_iterator it(
+        CHECK(attr.set_fpmath_mode(fpmath_mode));
+        primitive_desc_iterator_t it(
                 engine, (op_desc_t *)&gemm_desc, &attr, nullptr);
         if (!it.is_initialized()) return status::out_of_memory;
 
@@ -803,7 +809,8 @@ status_t _ref_rnn_common_t<aprop>::init(engine_t *engine) {
             = (size_t *)malloc(sizeof(size_t) * wei_offsets_iter_sz, 64);
 
     compute::kernel_ctx_t kernel_ctx;
-    status_t status = init_kernel_ctx(kernel_ctx, pd()->conf, pd()->off);
+    status_t status = init_kernel_ctx(kernel_ctx, pd()->conf, pd()->off,
+            pd()->subgroup_size, pd()->use_subgroup_reduction);
     CHECK(status);
 
     std::vector<const char *> kernel_names
@@ -927,7 +934,6 @@ status_t _ref_rnn_common_t<aprop>::init_res_storage(
 
 template <prop_kind_t aprop>
 gemm_sig((_ref_rnn_common_t<aprop>::gemm_primitive)) {
-    using namespace gemm_utils;
 
     // FIXME: This should be created once per execute() instead of creating
     // memory before each gemm call. Each cell type (+prop kind) might have
@@ -1097,7 +1103,7 @@ gemm_sig((_ref_rnn_common_t<aprop>::gemm_primitive)) {
 
 template <prop_kind_t aprop>
 void _ref_rnn_common_t<aprop>::gates_reduction(const exec_ctx_t &ctx, int dir,
-        int lay, int iter, int n_gates, int dhc, int batch,
+        int lay, int iter, int n_bias, int dhc, int batch,
         const memory_storage_t &scratch_gates,
         const memory_storage_t &scratch_cell,
         const memory_storage_t &diff_bias) const {
@@ -1110,7 +1116,9 @@ void _ref_rnn_common_t<aprop>::gates_reduction(const exec_ctx_t &ctx, int dir,
     arg_list.set(4, scratch_gates);
     arg_list.set(5, scratch_cell);
 
-    auto nd_range = compute::nd_range_t({n_gates, dhc});
+    auto nd_range = pd()->use_subgroup_reduction
+            ? get_nd_range({pd()->subgroup_size, dhc, n_bias})
+            : get_nd_range({dhc, n_bias});
 
     parallel_for(ctx, nd_range, gates_reduction_kernel_, arg_list);
 }
@@ -1124,6 +1132,21 @@ grid_execution_sig((_ref_rnn_common_t<aprop>::linear_execution)) {
     int n_dir = rnn.n_dir;
     int n_iter = rnn.n_iter;
 
+    if (aprop == prop_kind::backward && pd()->diff_weights_overwrite()) {
+        compute::compute_stream_t *compute_stream
+                = utils::downcast<compute::compute_stream_t *>(ctx.stream());
+        auto zero = [&](const memory_storage_t &data, int arg_id) {
+            auto mdw = memory_desc_wrapper(pd()->arg_md(arg_id));
+            return compute_stream->fill(data, 0, mdw.size(),
+                    compute_stream->ctx().get_deps(),
+                    compute_stream->ctx().get_deps());
+        };
+
+        CHECK(zero(diff_bias, DNNL_ARG_DIFF_BIAS));
+        CHECK(zero(diff_weights_layer, DNNL_ARG_DIFF_WEIGHTS_LAYER));
+        CHECK(zero(diff_weights_iter, DNNL_ARG_DIFF_WEIGHTS_ITER));
+    }
+
     // Grid Computation for RNN with a cell execution call
     for (int dir = 0; dir < n_dir; dir++) {
         for (int j = 0; j < n_layer; j++) {
@@ -1176,6 +1199,7 @@ grid_execution_sig((_ref_rnn_common_t<aprop>::linear_execution)) {
             }
         }
     }
+    return status::success;
 }
 //********* GRID computations strategy: utility functions **********//
 
@@ -1218,7 +1242,8 @@ void _ref_rnn_common_t<aprop>::copy_init_layer(const exec_ctx_t &ctx,
         arg_list.set(3, (cl_int)lr);
         arg_list.set(4, (cl_int)rl);
 
-        parallel_for(ctx, compute::nd_range_t({slc, batch, n_iter}),
+        parallel_for(ctx,
+                compute::nd_range_t(get_nd_range({slc, batch, n_iter})),
                 copy_init_layer_kernel_, arg_list);
     } else {
         compute::kernel_arg_list_t arg_list;
@@ -1228,7 +1253,7 @@ void _ref_rnn_common_t<aprop>::copy_init_layer(const exec_ctx_t &ctx,
         arg_list.set(3, (cl_int)0);
         arg_list.set(4, (cl_int)0);
 
-        parallel_for(ctx, compute::nd_range_t({batch, n_iter}),
+        parallel_for(ctx, compute::nd_range_t(get_nd_range({batch, n_iter})),
                 copy_init_layer_kernel_, arg_list);
     }
 }
@@ -1286,7 +1311,7 @@ void _ref_rnn_common_t<aprop>::copy_res_layer(const exec_ctx_t &ctx,
         arg_list.set(5, shift);
         arg_list.set(6, scale);
         arg_list.set(7, (int)dequantize);
-        parallel_for(ctx, compute::nd_range_t({dhc, batch, n_iter}),
+        parallel_for(ctx, get_nd_range({dhc, batch, n_iter}),
                 copy_res_layer_kernel_, arg_list);
     } else {
         compute::kernel_arg_list_t arg_list;
@@ -1295,7 +1320,7 @@ void _ref_rnn_common_t<aprop>::copy_res_layer(const exec_ctx_t &ctx,
         arg_list.set(2, scratch_diff_states);
         arg_list.set(3, (cl_int)lr);
         arg_list.set(4, (cl_int)rl);
-        parallel_for(ctx, compute::nd_range_t({slc, batch, n_iter}),
+        parallel_for(ctx, get_nd_range({slc, batch, n_iter}),
                 copy_res_layer_kernel_, arg_list);
     }
 }
