@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -51,13 +51,13 @@ status_t xe_lp_x8s8x_convolution_fwd_t::pd_t::init_conf() {
     // XXX: try reduce number of spatial dims when iw/ow/kw=1,
     // memory tags will be selected based on the number of input dimensions
     bool use_reshaped_mem = ndims > 3;
-    if (dnnl_memory_desc_reshape(&r_src, src, src->ndims - 1, src->dims)
+    if (memory_desc_reshape(r_src, *src, src->ndims - 1, src->dims)
             != status::success)
         use_reshaped_mem = false;
-    if (dnnl_memory_desc_reshape(&r_dst, dst, dst->ndims - 1, dst->dims)
+    if (memory_desc_reshape(r_dst, *dst, dst->ndims - 1, dst->dims)
             != status::success)
         use_reshaped_mem = false;
-    if (dnnl_memory_desc_reshape(&r_wei, wei, wei->ndims - 1, wei->dims)
+    if (memory_desc_reshape(r_wei, *wei, wei->ndims - 1, wei->dims)
             != status::success)
         use_reshaped_mem = false;
 
@@ -457,8 +457,6 @@ status_t xe_lp_x8s8x_convolution_fwd_t::pd_t::init_kernel_ctx(
     kernel_ctx.define_int("OWB", ow_nchunk);
     kernel_ctx.define_int("OWX", owx);
 
-    kernel_ctx.define_int("DISABLE_DPAS", disable_dpas);
-
     if (conf.is_depthwise)
         kernel_ctx.define_int("WEI_32G", 1);
     else
@@ -474,8 +472,8 @@ status_t xe_lp_x8s8x_convolution_fwd_t::pd_t::init_kernel_ctx(
                     : conf.attr_info.sum_data_type,
             "SUM");
 
-    def_attr_info(
-            kernel_ctx, conf.attr_info, attr()->post_ops_, &(dst_md()->dims));
+    CHECK(def_attr_info(
+            kernel_ctx, conf.attr_info, attr()->post_ops_, &(dst_md()->dims)));
 
     kernel_ctx.add_option("-Dcl_intel_subgroups_char");
     kernel_ctx.add_option("-Dcl_intel_subgroups_long");
@@ -542,18 +540,9 @@ status_t xe_lp_x8s8x_convolution_fwd_t::execute_forward(
     unsigned arg_idx = append_post_ops_to_arg_list(
             ctx, arg_list, 4, pd()->attr()->post_ops_);
 
-    if (conf.attr_info.common_oscales) {
-        float scales = pd()->attr()->output_scales_.scales_[0];
-        arg_list.set(arg_idx++, scales);
-    } else {
-        arg_list.set(arg_idx++, 1.0f);
-    }
-
-    if (conf.attr_info.with_per_oc_oscales) {
-        if (conf.attr_info.with_runtime_oscales)
-            arg_list.set(arg_idx++, oscales);
-        else
-            arg_list.set(arg_idx++, CTX_GPU_RES_STORAGE(SCALES_));
+    if (conf.attr_info.with_common_oscales
+            || conf.attr_info.with_per_oc_oscales) {
+        arg_list.set(arg_idx++, oscales);
     } else {
         arg_list.set(arg_idx++, memory_storage_t::empty_storage());
     }
@@ -578,7 +567,7 @@ status_t xe_lp_x8s8x_convolution_fwd_t::execute_forward(
     status_t status = parallel_for(ctx, nd_range, kernel_, arg_list);
 
     if (!post_ops_preserves_zeroes(ctx, pd()->attr()->post_ops_)) {
-        ctx.zero_pad_output(DNNL_ARG_DST);
+        CHECK(ctx.zero_pad_output(DNNL_ARG_DST));
     }
     return status;
 }
@@ -748,8 +737,6 @@ status_t xe_lp_x8s8x_convolution_bwd_data_t::pd_t::init_kernel_ctx(
 
     kernel_ctx.define_int("IS_NHWC", conf.is_nhwc);
 
-    kernel_ctx.define_int("DISABLE_DPAS", disable_dpas);
-
     kernel_ctx.set_data_type(conf.dst_data_type);
     def_data_type(kernel_ctx, conf.src_data_type, "SRC");
     def_data_type(kernel_ctx, conf.dst_data_type, "DST");
