@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -35,6 +35,27 @@ namespace gpu {
 #define MAX_NDIMS 6
 #define MAX_POST_OPS_SUPPORTED 32
 
+inline bool memory_desc_ndims_ok(const memory_desc_t *md) {
+    return md->ndims > MAX_NDIMS;
+}
+
+template <typename T, typename... Rest>
+bool memory_desc_ndims_ok(const T *first, const Rest *...rest) {
+    return memory_desc_ndims_ok(first) || memory_desc_ndims_ok(rest...);
+}
+
+inline dim_t get_attr_oscales_count(int mask, const memory_desc_wrapper &md) {
+    dim_t count = 1;
+    if (mask == 0) return count;
+
+    for (int d = 0; d < md.ndims(); d++) {
+        const int dim_mask = 1 << d;
+        if (dim_mask & mask) count *= md.dims()[d];
+    }
+
+    return count;
+}
+
 struct memory_desc_info_t {
     // Max levels of blocking
     static const int max_nlevels = 3;
@@ -42,13 +63,13 @@ struct memory_desc_info_t {
     int ndims;
     data_type_t data_type;
 
-    int offset0;
-    int dims[MAX_NDIMS];
-    int padded_dims[MAX_NDIMS];
+    dim_t offset0;
+    dim_t dims[MAX_NDIMS];
+    dim_t padded_dims[MAX_NDIMS];
 
     int nlevels;
-    int blocks[MAX_NDIMS][max_nlevels + 1];
-    int strides[MAX_NDIMS][max_nlevels + 1];
+    dim_t blocks[MAX_NDIMS][max_nlevels + 1];
+    dim_t strides[MAX_NDIMS][max_nlevels + 1];
 
     static memory_desc_info_t create(const memory_desc_wrapper &mdw) {
         using namespace format_tag;
@@ -127,15 +148,12 @@ struct attr_info_t {
                 = (attr_info.sum_idx != -1) && (attr_info.sum_scale != 0.0f);
 
         // Output scales
-        attr_info.with_oscales = !attr->output_scales_.has_default_values();
+        attr_info.with_oscales
+                = !attr->scales_.get(DNNL_ARG_WEIGHTS).has_default_values();
 
-        const auto &scales_mask = attr->output_scales_.mask_;
+        const auto &scales_mask = attr->scales_.get(DNNL_ARG_WEIGHTS).mask_;
         attr_info.with_common_oscales
                 = attr_info.with_oscales && (scales_mask == 0);
-        attr_info.common_oscales = (attr_info.with_common_oscales
-                        ? attr->output_scales_.scales_[0]
-                        : 1.0f);
-
         attr_info.with_per_oc_oscales
                 = attr_info.with_oscales && (scales_mask == (1 << 1));
 
@@ -143,32 +161,35 @@ struct attr_info_t {
 
         const auto &src0_scales = attr->scales_.get(DNNL_ARG_SRC_0);
         attr_info.with_src0_scale = !src0_scales.has_default_values();
-        attr_info.src0_scale = *src0_scales.scales_;
         assert(src0_scales.mask_ == 0);
 
         const auto &src1_scales = attr->scales_.get(DNNL_ARG_SRC_1);
         attr_info.with_src1_scale = !src1_scales.has_default_values();
-        attr_info.src1_scale = *src1_scales.scales_;
         assert(src1_scales.mask_ == 0);
 
+        const auto &src_scales = attr->scales_.get(DNNL_ARG_SRC);
+        attr_info.with_src_scales = !src_scales.has_default_values();
+        assert(src_scales.mask_ == 0);
+
+        const auto &wei_scales = attr->scales_.get(DNNL_ARG_WEIGHTS);
+        attr_info.with_wei_scales = !wei_scales.has_default_values();
+        attr_info.wei_scales_mask = wei_scales.mask_;
+
+        const auto &dst_scales = attr->scales_.get(DNNL_ARG_DST);
+        attr_info.with_dst_scales = !dst_scales.has_default_values();
+        assert(dst_scales.mask_ == 0);
+
         // zero points
         const auto &zp = attr->zero_points_;
         attr_info.with_src_zpoints = !zp.has_default_values(DNNL_ARG_SRC);
+        attr_info.with_wei_zpoints = !zp.has_default_values(DNNL_ARG_WEIGHTS);
         attr_info.with_dst_zpoints = !zp.has_default_values(DNNL_ARG_DST);
 
         attr_info.with_per_ic_src_zpoints = attr_info.with_src_zpoints
                 && !zp.defined(DNNL_ARG_SRC) && !zp.common(DNNL_ARG_SRC);
-        attr_info.common_src_zpoint
-                = attr_info.with_src_zpoints && zp.defined(DNNL_ARG_SRC)
-                ? *zp.get(DNNL_ARG_SRC)
-                : 0;
 
         attr_info.with_per_oc_dst_zpoints = attr_info.with_dst_zpoints
                 && !zp.defined(DNNL_ARG_DST) && !zp.common(DNNL_ARG_DST);
-        attr_info.common_dst_zpoint
-                = attr_info.with_dst_zpoints && zp.defined(DNNL_ARG_DST)
-                ? *zp.get(DNNL_ARG_DST)
-                : 0;
 
         attr_info.initialized = true;
         return attr_info;
@@ -192,50 +213,49 @@ struct attr_info_t {
 
     bool with_oscales;
     bool with_common_oscales;
-    float common_oscales;
     bool with_per_oc_oscales;
     bool with_runtime_oscales;
 
     bool with_src0_scale;
-    float src0_scale;
-
     bool with_src1_scale;
-    float src1_scale;
+    bool with_src_scales;
+    bool with_wei_scales;
+    bool with_dst_scales;
+    bool wei_scales_mask;
 
     bool with_src_zpoints;
+    bool with_wei_zpoints;
     bool with_dst_zpoints;
     bool with_per_ic_src_zpoints;
     bool with_per_oc_dst_zpoints;
-    int common_src_zpoint;
-    int common_dst_zpoint;
 };
 
 struct offsets_t {
-    int src_off[4][MAX_NDIMS];
-    int wei_off[4][MAX_NDIMS];
-    int dst_off[4][MAX_NDIMS];
+    dim_t src_off[4][MAX_NDIMS];
+    dim_t wei_off[4][MAX_NDIMS];
+    dim_t dst_off[4][MAX_NDIMS];
 };
 
 struct rnn_offsets_t {
-    int src_layer_off[4][MAX_NDIMS];
-    int src_iter_off[4][MAX_NDIMS];
-    int src_iter_c_off[4][MAX_NDIMS];
-    int weights_layer_off[4][MAX_NDIMS];
-    int weights_iter_off[4][MAX_NDIMS];
-    int bias_off[4][MAX_NDIMS];
-    int dst_layer_off[4][MAX_NDIMS];
-    int dst_iter_off[4][MAX_NDIMS];
-    int dst_iter_c_off[4][MAX_NDIMS];
-    int diff_src_layer_off[4][MAX_NDIMS];
-    int diff_src_iter_off[4][MAX_NDIMS];
-    int diff_src_iter_c_off[4][MAX_NDIMS];
-    int diff_weights_layer_off[4][MAX_NDIMS];
-    int diff_weights_iter_off[4][MAX_NDIMS];
-    int diff_bias_off[4][MAX_NDIMS];
-    int diff_dst_layer_off[4][MAX_NDIMS];
-    int diff_dst_iter_off[4][MAX_NDIMS];
-    int diff_dst_iter_c_off[4][MAX_NDIMS];
-    int ws_off[4][MAX_NDIMS];
+    dim_t src_layer_off[4][MAX_NDIMS];
+    dim_t src_iter_off[4][MAX_NDIMS];
+    dim_t src_iter_c_off[4][MAX_NDIMS];
+    dim_t weights_layer_off[4][MAX_NDIMS];
+    dim_t weights_iter_off[4][MAX_NDIMS];
+    dim_t bias_off[4][MAX_NDIMS];
+    dim_t dst_layer_off[4][MAX_NDIMS];
+    dim_t dst_iter_off[4][MAX_NDIMS];
+    dim_t dst_iter_c_off[4][MAX_NDIMS];
+    dim_t diff_src_layer_off[4][MAX_NDIMS];
+    dim_t diff_src_iter_off[4][MAX_NDIMS];
+    dim_t diff_src_iter_c_off[4][MAX_NDIMS];
+    dim_t diff_weights_layer_off[4][MAX_NDIMS];
+    dim_t diff_weights_iter_off[4][MAX_NDIMS];
+    dim_t diff_bias_off[4][MAX_NDIMS];
+    dim_t diff_dst_layer_off[4][MAX_NDIMS];
+    dim_t diff_dst_iter_off[4][MAX_NDIMS];
+    dim_t diff_dst_iter_c_off[4][MAX_NDIMS];
+    dim_t ws_off[4][MAX_NDIMS];
 };
 
 // Convolution
@@ -277,25 +297,12 @@ struct conv_conf_t {
     int oc_block, ic_block, nchunk;
     int omb;
     int odb, ohb, owb;
-    size_t wei_block;
     int icb;
     int ocb;
-    int osp_chunk, mb_chunk, mb_block, slm_ic;
+    int osp_chunk, mb_chunk, mb_block;
     int iw_tail;
-    int mb_blk_wg;
-    int max_blk_wg, ic_blk_wg, oc_blk_wg;
-    int ic_blk_sg, oc_blk_sg;
-    int k_blocks, k_block_tail;
     size_t wei_slm_size, src_slm_size, dst_slm_size;
     int sub_group_size;
-    int workgroups_along_k;
-    int num_buffers;
-    int calc_block;
-    int ic_split;
-
-    int oc_group;
-    int ow_group;
-    int sp_group;
 
     size_t gws_d[3], lws_d[3];
     // Original global work sizes, before applying rounding in case when
@@ -304,12 +311,11 @@ struct conv_conf_t {
     compute::dispatch_t dispatch;
 
     bool with_bias, with_groups;
-    bool use_split_barrier;
 
     attr_info_t attr_info;
 
     bool is_depthwise;
-    bool is_nhwc, use_dpasw;
+    bool is_nhwc;
     bool reorder_wei = false;
     bool reorder_bias = false;
     int ver;
@@ -318,7 +324,6 @@ struct conv_conf_t {
     bool is_src_nchw, is_src_nhwc;
     bool is_dst_nhwc;
 
-    bool use_256grf_per_thread;
     int tile_size;
     int wino_m;
     int wino_r;
@@ -362,13 +367,15 @@ struct pool_conf_t {
     bool is_plain;
     bool is_training, is_backward;
     bool use_mb_c_block, use_only_c_block;
+    bool unroll_mb = false;
     int chunks_per_c_block, chunks_per_mb_block;
     int vect_dt_n;
     int nvect;
     compute::dispatch_t dispatch;
-    compute::gpu_arch_t gpu_arch;
     int sub_group_size;
     int global_pool_spatial_chunk;
+    int num_batches = 1;
+    int mb_block_size = 16;
 
     attr_info_t attr_info;
     memory_desc_info_t src_md_info;
@@ -452,6 +459,7 @@ struct rnn_conf_t {
     int dhc;
     int dlc;
     int wic;
+    int arch_ld;
     int n_parts_weights_iter, n_parts_weights_layer;
     int src_layer_ndims;
     int src_iter_ndims;
@@ -512,22 +520,44 @@ struct bnorm_conf_t {
     int id, ih, iw;
     int nn, sp, sp_tail, vect_size;
     int stat_sp_nblocks, stat_sp_tail, stat_sp_block;
+    int update_sp_nblocks, update_sp_tail, update_sp_block;
     int reduce_stat_nblocks;
     bool with_relu, use_16mb_unroll, use_nhwc;
     int stat_ic;
     bool is_forward, is_backward;
-    bool use_scaleshift, use_scale, use_shift, save_stats, is_training;
-    bool fuse_norm_relu, calculate_stats, calculate_diff_stats;
-    bool diff_scaleshift, diff_scale, diff_shift;
+    bool use_scale, use_shift, save_stats, is_training;
+    bool calculate_stats, calculate_diff_stats;
+    bool fuse_norm_relu, fuse_norm_add_relu;
+    bool diff_scale, diff_shift;
     float relu_negative_slope, eps;
     int sub_group_size;
     bool vectorize_calc_stats;
     bool skip_reduce_stat;
     bool use_stats_one_pass;
+    bool nhwc_optimized;
+    int calc_stat_ic;
+    bool use_fused_atomics_reduction;
+    bool use_workaround;
+    int update_sp_unroll;
+    int max_vect_size;
+
+    std::string flags;
+    bool bn_tuning;
+    bool is_blocked_16c;
+    bool is_blocked_16n16c;
+    bool is_blocked_32n16c;
+    bool is_nhwc;
+    bool is_overrided_use_fused_atomics_reduction;
+    bool is_overrided_ic_block;
+    bool is_overrided_max_vect_size;
+    bool is_overrided_stat_sp_block;
+    bool is_overrided_update_sp_block;
+    bool is_overrided_update_sp_unroll;
 
     compute::dispatch_t dispatch_calc_stat;
     compute::dispatch_t dispatch_reduce_stat;
     compute::dispatch_t dispatch;
+    compute::dispatch_t dispatch_reduce_aux;
 };
 
 // Layer Normalization
@@ -542,7 +572,6 @@ struct lnorm_conf_t {
     memory_desc_info_t dst_md_info;
     memory_desc_info_t stat_md_info;
 
-    bool use_scaleshift;
     bool use_scale;
     bool use_shift;
     bool calculate_stats;
@@ -567,8 +596,9 @@ struct lnorm_conf_t {
 struct binary_conf_t {
     int ndims, nvect;
     bool use_unroll_16b, src0_unroll_16b;
-    bool is_ncX_layout;
+    bool is_plain_layout;
     bool plain_to_ABcd4a4b;
+    bool isXa16b;
     data_type_t src0_data_type;
     data_type_t src1_data_type;
     data_type_t dst_data_type;
@@ -586,6 +616,7 @@ struct binary_conf_t {
     bool is_ne;
     bool is_tensor_op;
     compute::dispatch_t dispatch;
+    int mb_block;
     int dim0[MAX_NDIMS];
     int src0_bcast_dims[MAX_NDIMS];
     int src1_bcast_dims[MAX_NDIMS];
@@ -602,29 +633,42 @@ struct binary_conf_t {
 };
 
 // Reduction
+struct reduction_phase_t {
+    data_type_t src_type, dst_type;
+    compute::nd_range_t nd_range;
+    compute::kernel_t kernel;
+    dim_t outer_dim_size, reduction_size, inner_dim_size;
+    bool is_final, is_first;
+};
+
 struct reduction_conf_t {
+    // Used by reference implementation
+    alg_kind_t alg;
     int ndims, power, div;
     float eps;
     dim_t src_dims[MAX_NDIMS], reduce_dims[MAX_NDIMS], dst_dims[MAX_NDIMS];
     bool is_reduction_dim[MAX_NDIMS];
+    int hwd_reduction_size, hwd_size;
+    data_type_t src_type, dst_type;
+    memory_desc_info_t src_md_info, dst_md_info;
+    compute::dispatch_t dispatch;
+    offsets_t off;
+    attr_info_t attr_info;
 
+    // Used by gen9 implementation
     int initial_hwd_dim, initial_hwd_chunk_size;
     int final_hwd_dim, final_hwd_chunk_size;
     int initial_c_chunks, final_c_dim, final_c_chunk_size;
     int initial_n_chunk_size, initial_n_chunks;
     int final_n_dim, final_n_chunk_size;
-
     bool skip_final_phase;
     int c_block_size, n_block_size;
     int vector_size;
     int sub_group_size;
-    data_type_t src_type, dst_type;
-    alg_kind_t alg;
-    compute::dispatch_t dispatch;
-    compute::dispatch_t finilize_dispatch;
-    memory_desc_info_t src_md_info, dst_md_info;
-    offsets_t off;
-    attr_info_t attr_info;
+    compute::dispatch_t finalize_dispatch;
+
+    // Used by combined implementation
+    std::vector<reduction_phase_t> phases;
 };
 
 // Reorder
@@ -650,6 +694,23 @@ enum reorder_kernel_t {
     xb_to_xab_xba
 };
 
+// Resampling
+struct resampling_conf_t {
+    dim_t ndims;
+    offsets_t off;
+    dim_t MB, C;
+    dim_t ID, IH, IW;
+    dim_t OD, OH, OW;
+    float FD, FH, FW;
+    dim_t vect_size;
+    dims_t padded_strides;
+    size_t lws[3], gws[3];
+    int sub_group_size;
+    dim_t padded_c;
+    attr_info_t attr_info;
+    compute::dispatch_t dispatch;
+};
+
 struct block_desc_t {
     int dim_idx;
     int blk_size;
@@ -694,9 +755,140 @@ union reorder_implementation {
     xb_to_xab_xba_t ab;
     vectorize_last_dim_t vld;
 };
+
+class scales_query_t {
+public:
+    bool has_default_values() const { return scales_.has_default_values(); }
+    int get_mask() const { return scales_.mask_; }
+    size_t get_count() const { return count_; }
+    memory_storage_t &get_scales(const exec_ctx_t &ctx) const {
+        return CTX_IN_STORAGE(DNNL_ARG_ATTR_SCALES | arg_);
+    }
+
+    scales_query_t() = default;
+    scales_query_t(const primitive_attr_t *attr, const memory_desc_wrapper &mdw,
+            int arg)
+        : arg_(arg) {
+        scales_ = attr->scales_.get(arg);
+        count_ = get_attr_oscales_count(scales_.mask_, mdw);
+    }
+
+private:
+    runtime_scales_t scales_;
+    dim_t count_ = 0;
+    int arg_ = 0;
+};
+
+class zero_points_query_t {
+public:
+    bool has_default_values() const { return zps_.has_default_values(arg_); }
+    int get_mask() const {
+        int mask;
+        zps_.get(arg_, &mask);
+        return mask;
+    }
+    size_t get_count() const { return count_; }
+    memory_storage_t &get_zero_points(const exec_ctx_t &ctx) const {
+        return CTX_IN_STORAGE(DNNL_ARG_ATTR_ZERO_POINTS | arg_);
+    }
+
+    zero_points_query_t() = default;
+    zero_points_query_t(const primitive_attr_t *attr,
+            const memory_desc_wrapper &mdw, int arg)
+        : arg_(arg) {
+        zps_ = attr->zero_points_;
+        int mask;
+        zps_.get(arg, &mask);
+        count_ = get_attr_oscales_count(mask, mdw);
+    }
+
+private:
+    zero_points_t zps_;
+    dim_t count_;
+    int arg_;
+};
+
+struct quantization_t {
+public:
+    bool with_scale() const { return !scale_.has_default_values(); }
+    int scale_mask() const { return scale_.get_mask(); }
+    size_t num_scales() const { return scale_.get_count(); }
+    memory_storage_t &scales(const exec_ctx_t &ctx) const {
+        return scale_.get_scales(ctx);
+    }
+
+    bool with_zp() const { return !zp_.has_default_values(); }
+    int zp_mask() const { return zp_.get_mask(); }
+    size_t num_zps() const { return zp_.get_count(); }
+    memory_storage_t &zero_points(const exec_ctx_t &ctx) const {
+        return zp_.get_zero_points(ctx);
+    }
+
+    void define_macros(
+            compute::kernel_ctx_t &kernel_ctx, const std::string &name) const {
+        if (with_scale()) {
+            kernel_ctx.define_int("WITH_" + name + "_SCALE", 1);
+            kernel_ctx.define_int(name + "_SCALE_MASK", scale_mask());
+            kernel_ctx.define_int(name + "_NUM_SCALES", num_scales());
+        }
+
+        if (with_zp()) {
+            kernel_ctx.define_int("WITH_" + name + "_ZPOINT", 1);
+            kernel_ctx.define_int(name + "_ZPOINT_MASK", zp_mask());
+            kernel_ctx.define_int(name + "_NUM_ZPOINTS", num_zps());
+        }
+    }
+
+    quantization_t(const primitive_attr_t *attr, const memory_desc_wrapper &mdw,
+            int arg)
+        : scale_(attr, mdw, arg), zp_(attr, mdw, arg) {}
+    quantization_t() = default;
+
+private:
+    scales_query_t scale_;
+    zero_points_query_t zp_;
+};
+
+struct sum_quantization_t {
+public:
+    bool with_scale() const { return scale_ != 0; }
+    int scale_mask() const { return 0; }
+    size_t num_scales() const { return (size_t)(with_scale()); }
+    float scales() const { return scale_; }
+
+    bool with_zp() const { return zp_ != 0; }
+    int zp_mask() const { return 0; }
+    size_t num_zps() const { return (size_t)(with_zp()); }
+    int zero_points() const { return zp_; }
+
+    void define_macros(
+            compute::kernel_ctx_t &kernel_ctx, const std::string &name) const {
+        if (with_scale()) kernel_ctx.define_int("WITH_" + name + "_SCALE", 1);
+        if (with_zp()) kernel_ctx.define_int("WITH_" + name + "_ZPOINT", 1);
+    }
+
+    sum_quantization_t(const primitive_attr_t *attr) {
+        const auto &post_ops = attr->post_ops_;
+        const int sum_idx = post_ops.find(primitive_kind::sum);
+        if (sum_idx != -1) {
+            const auto &sum = post_ops.entry_[sum_idx].sum;
+            scale_ = sum.scale;
+            zp_ = sum.zero_point;
+        }
+    }
+    sum_quantization_t() = default;
+
+private:
+    float scale_ = 0;
+    int zp_ = 0;
+};
+
 struct reorder_conf_t {
     bool has_padding;
-    bool scale_quant, with_sum_ab, with_sum_a;
+
+    quantization_t src_quant, dst_quant;
+    sum_quantization_t sum_quant;
+
     reorder_kernel_t implementation;
     int ndims;
     size_t nelems;
@@ -704,9 +896,6 @@ struct reorder_conf_t {
     compute::dispatch_t dispatch;
 
     int sub_group_size;
-    int scale_mask;
-    size_t scales_num;
-
     memory_desc_info_t src_md_info;
     memory_desc_info_t dst_md_info;
 
@@ -716,9 +905,10 @@ struct reorder_conf_t {
 // Concat
 struct concat_conf_t {
     dim_t dst_extern_dim_size;
-    dim_t src_extern_dim_sizes[30];
-    dim_t offset[30];
+    dim_t src_extern_dim_sizes[64];
+    dim_t offset[64];
     dim_t inner_axis;
+    dim_t dst_offset0;
     int block;
     int n;
     int simd;
@@ -728,11 +918,13 @@ struct concat_conf_t {
     data_type_t src_type, dst_type;
     compute::dispatch_t dispatch;
     int ndims;
-    memory_desc_info_t src_md_infos[30];
+    memory_desc_info_t src_md_infos[64];
     memory_desc_info_t dst_md_info;
     int concat_axis;
     int sub_group_size;
     int iter_dim_idx, iter_dim_chunk;
+    scales_query_t scale_src[64];
+    uint64_t scales_mask;
 };
 
 // Elementwise
@@ -743,6 +935,7 @@ struct eltwise_conf_t {
     data_type_t data_type;
     alg_kind_t alg;
     bool is_forward;
+    int work_group_size;
     int sub_group_size;
     compute::dispatch_t dispatch;
     memory_desc_info_t data_md_info;
@@ -762,9 +955,9 @@ struct shuffle_conf_t {
     memory_desc_info_t dst_md_info;
 };
 
-inline void set_default_pool_conf(pool_conf_t &conf,
-        const pooling_v2_desc_t &desc, const memory_desc_t &src_md,
-        const memory_desc_t &dst_md, const primitive_attr_t &attr) {
+inline void set_default_pool_conf(pool_conf_t &conf, const pooling_desc_t &desc,
+        const memory_desc_t &src_md, const memory_desc_t &dst_md,
+        const primitive_attr_t &attr) {
     const memory_desc_wrapper src_mdw(src_md);
     const memory_desc_wrapper dst_mdw(dst_md);
 
@@ -793,13 +986,9 @@ inline void set_default_pool_conf(pool_conf_t &conf,
     conf.kh = (ndims == 3) ? 1 : desc.kernel[ndims - 4];
     conf.kw = desc.kernel[ndims - 3];
 
-    if (desc.primitive_kind != dnnl_pooling_v2) {
-        conf.dd = conf.dh = conf.dw = 0;
-    } else {
-        conf.dd = (ndims == 5) ? desc.dilation[0] : 0;
-        conf.dh = (ndims == 3) ? 0 : desc.dilation[ndims - 4];
-        conf.dw = desc.dilation[ndims - 3];
-    }
+    conf.dd = (ndims == 5) ? desc.dilation[0] : 0;
+    conf.dh = (ndims == 3) ? 0 : desc.dilation[ndims - 4];
+    conf.dw = desc.dilation[ndims - 3];
 
     conf.f_pad = (ndims == 5) ? desc.padding[0][0] : 0;
     conf.t_pad = (ndims == 3) ? 0 : desc.padding[0][ndims - 4];
@@ -897,7 +1086,7 @@ inline void set_offsets(compute::kernel_ctx_t &kernel_ctx,
     md.compute_strides_compat(strides_compat);
 
     for (int d = 0; d < MAX_NDIMS; ++d) {
-        const int block = block_dims[d];
+        const dim_t block = block_dims[d];
 
         kernel_ctx.define_int(
                 utils::format("%s_B%d", str, d), (d < md.ndims()) ? block : 1);
@@ -910,7 +1099,8 @@ inline void set_offsets(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int(utils::format("%s_OFFSET_PAD", str), md.md_->offset0);
 }
 
-inline void set_offsets(const memory_desc_wrapper &md, int offs[4][MAX_NDIMS]) {
+inline void set_offsets(
+        const memory_desc_wrapper &md, dim_t offs[4][MAX_NDIMS]) {
     dim_t block_dims[DNNL_MAX_NDIMS];
     dim_t strides_compat[2][DNNL_MAX_NDIMS];
 
@@ -919,7 +1109,7 @@ inline void set_offsets(const memory_desc_wrapper &md, int offs[4][MAX_NDIMS]) {
     const dims_t &dims = md.dims();
 
     for (int d = 0; d < md.ndims(); ++d) {
-        const int block = block_dims[d];
+        const dim_t block = block_dims[d];
 
         offs[0][d] = block;
         offs[1][d] = strides_compat[0][d];
@@ -928,7 +1118,7 @@ inline void set_offsets(const memory_desc_wrapper &md, int offs[4][MAX_NDIMS]) {
     }
 }
 
-inline void def_offsets(const int offs[4][MAX_NDIMS],
+inline void def_offsets(const dim_t offs[4][MAX_NDIMS],
         compute::kernel_ctx_t &kernel_ctx, const char *str, const int ndims) {
 
     for (int d = 0; d < MAX_NDIMS; d++) {
@@ -958,6 +1148,10 @@ inline void def_data_type(
             kernel_ctx.add_option(
                     utils::format("-D%s_DATA_T=float -D%s_DT_F32", str, str));
             break;
+        case data_type::f64:
+            kernel_ctx.add_option(
+                    utils::format("-D%s_DATA_T=double -D%s_DT_F64", str, str));
+            break;
         case data_type::s8:
             kernel_ctx.add_option(
                     utils::format("-D%s_DATA_T=char -D%s_DT_S8", str, str));
@@ -984,14 +1178,14 @@ inline void def_memory_desc_info(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int(utils::format("%s_NLEVELS", prefix), md_info.nlevels);
 
     for (int d = 0; d < MAX_NDIMS; ++d) {
-        int dim = (d < md_info.ndims) ? md_info.dims[d] : 1;
-        int padded_dim = (d < md_info.ndims) ? md_info.padded_dims[d] : 1;
+        dim_t dim = (d < md_info.ndims) ? md_info.dims[d] : 1;
+        dim_t padded_dim = (d < md_info.ndims) ? md_info.padded_dims[d] : 1;
         kernel_ctx.define_int(utils::format("%s_D%d", prefix, d), dim);
         kernel_ctx.define_int(utils::format("%s_PD%d", prefix, d), padded_dim);
 
         for (int l = 0; l < md_info.nlevels + 1; ++l) {
-            int block = (d < md_info.ndims) ? md_info.blocks[d][l] : 1;
-            int stride = (d < md_info.ndims) ? md_info.strides[d][l] : 0;
+            dim_t block = (d < md_info.ndims) ? md_info.blocks[d][l] : 1;
+            dim_t stride = (d < md_info.ndims) ? md_info.strides[d][l] : 0;
             kernel_ctx.define_int(
                     utils::format("%s_B%d_%d", prefix, d, l), block);
             kernel_ctx.define_int(
@@ -1018,9 +1212,7 @@ inline void def_binary_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
 inline void def_eltwise_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
     kernel_ctx.define_int("RELU", alg_kind::eltwise_relu);
     kernel_ctx.define_int("LINEAR", alg_kind::eltwise_linear);
-    kernel_ctx.define_int("BOUNDED_RELU", alg_kind::eltwise_bounded_relu);
     kernel_ctx.define_int("SOFT_RELU", alg_kind::eltwise_soft_relu);
-    kernel_ctx.define_int("LOGSIGMOID", alg_kind::eltwise_logsigmoid);
     kernel_ctx.define_int("MISH", alg_kind::eltwise_mish);
     kernel_ctx.define_int("LOGISTIC", alg_kind::eltwise_logistic);
     kernel_ctx.define_int("TANH", alg_kind::eltwise_tanh);
@@ -1038,6 +1230,7 @@ inline void def_eltwise_alg_kinds(compute::kernel_ctx_t &kernel_ctx) {
     kernel_ctx.define_int("GELU_ERF", alg_kind::eltwise_gelu_erf);
     kernel_ctx.define_int("ROUND", alg_kind::eltwise_round);
     kernel_ctx.define_int("HARDSWISH", alg_kind::eltwise_hardswish);
+    kernel_ctx.define_int("HARDSIGMOID", alg_kind::eltwise_hardsigmoid);
 
     kernel_ctx.define_int("RELU_DST", alg_kind::eltwise_relu_use_dst_for_bwd);
     kernel_ctx.define_int(
@@ -1093,7 +1286,7 @@ inline bool post_ops_with_binary_ok(const primitive_attr_t *attr,
     return is_po_ok;
 }
 
-inline void def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
+inline status_t def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
         const post_ops_t &post_ops, const dnnl_dims_t *dst_dims) {
     const int po_nop_id = 0;
     const int po_binary_id = 1;
@@ -1163,8 +1356,8 @@ inline void def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
                         break;
                 }
             }
-            dnnl_memory_desc_init_by_tag(&weight_mem_desc, weight_ndims,
-                    weight_dims, data_type_t::dnnl_f32, weights_tag);
+            CHECK(memory_desc_init_by_tag(weight_mem_desc, weight_ndims,
+                    weight_dims, data_type_t::dnnl_f32, weights_tag));
             const memory_desc_wrapper weight_mdw(weight_mem_desc);
             const auto mdi = memory_desc_info_t::create(weight_mdw);
             def_memory_desc_info(kernel_ctx, mdi, bin_arg_name.c_str());
@@ -1173,10 +1366,10 @@ inline void def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
             kernel_ctx.define_int(
                     "PO_" + std::to_string(idx) + "_BIN_ARG_DT_IS_BF16", 0);
         } else {
-            dnnl_memory_desc_t empty_mem_desc;
+            memory_desc_t empty_mem_desc;
             dnnl_dims_t empty_dims = {1, 1, 1, 1};
-            dnnl_memory_desc_init_by_tag(&empty_mem_desc, 4, empty_dims,
-                    data_type_t::dnnl_s8, format_tag_t::dnnl_nchw);
+            CHECK(memory_desc_init_by_tag(empty_mem_desc, 4, empty_dims,
+                    data_type_t::dnnl_s8, format_tag_t::dnnl_nchw));
             const memory_desc_wrapper src1_mdw(empty_mem_desc);
             const auto mdi = memory_desc_info_t::create(src1_mdw);
             def_memory_desc_info(kernel_ctx, mdi, bin_arg_name.c_str());
@@ -1232,12 +1425,13 @@ inline void def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
         }
         po_kernel_args += ", const __global PO_" + std::to_string(idx)
                 + "_BIN_ARG_DATA_T *po_" + std::to_string(idx) + "_binary_arg";
+        return status::success;
     };
 
     for (int idx = 0; idx < post_ops.len(); ++idx, ++nof_supported_post_ops) {
         const std::string bin_arg_name
                 = "PO_" + std::to_string(idx) + "_BIN_ARG";
-        add_po_defines(bin_arg_name, post_ops.entry_[idx], idx);
+        CHECK(add_po_defines(bin_arg_name, post_ops.entry_[idx], idx));
     }
 
     kernel_ctx.define_int("POST_OP_CHAIN_LENGTH", nof_supported_post_ops);
@@ -1248,6 +1442,7 @@ inline void def_post_ops_cfg(compute::kernel_ctx_t &kernel_ctx,
     }
     po_kernel_args += "\"";
     kernel_ctx.add_option(po_kernel_args);
+    return status::success;
 }
 
 inline int append_post_ops_to_arg_list_base(const exec_args_t &args,
@@ -1314,7 +1509,7 @@ inline bool post_ops_preserves_zeroes(
     return preserve_zeroes;
 }
 
-inline void def_attr_info(compute::kernel_ctx_t &kernel_ctx,
+inline status_t def_attr_info(compute::kernel_ctx_t &kernel_ctx,
         const attr_info_t &attr_info, const post_ops_t &post_ops,
         const dnnl_dims_t *dst_dims = nullptr) {
     assert(attr_info.initialized);
@@ -1339,10 +1534,14 @@ inline void def_attr_info(compute::kernel_ctx_t &kernel_ctx,
     kernel_ctx.define_int("SCALES_PER_OC", attr_info.with_per_oc_oscales);
     kernel_ctx.define_int("SCALES_COMMON", attr_info.with_common_oscales);
 
+    kernel_ctx.define_int("WITH_SRC_SCALES", attr_info.with_src_scales);
+    kernel_ctx.define_int("WITH_WEI_SCALES", attr_info.with_wei_scales);
+    kernel_ctx.define_int("WITH_DST_SCALES", attr_info.with_dst_scales);
+    kernel_ctx.define_int("WEI_SCALES_MASK", attr_info.wei_scales_mask);
+
     kernel_ctx.define_int("WITH_SRC_ZPOINTS", attr_info.with_src_zpoints);
+    kernel_ctx.define_int("WITH_WEI_ZPOINTS", attr_info.with_wei_zpoints);
     kernel_ctx.define_int("WITH_DST_ZPOINTS", attr_info.with_dst_zpoints);
-    kernel_ctx.define_int("SRC_ZPOINT_COMMON", attr_info.common_src_zpoint);
-    kernel_ctx.define_int("DST_ZPOINT_COMMON", attr_info.common_dst_zpoint);
     kernel_ctx.define_int(
             "WITH_SRC_ZPOINTS_PER_IC", attr_info.with_per_ic_src_zpoints);
     kernel_ctx.define_int(
@@ -1351,7 +1550,7 @@ inline void def_attr_info(compute::kernel_ctx_t &kernel_ctx,
     def_binary_alg_kinds(kernel_ctx);
     def_eltwise_alg_kinds(kernel_ctx);
 
-    def_post_ops_cfg(kernel_ctx, post_ops, dst_dims);
+    return def_post_ops_cfg(kernel_ctx, post_ops, dst_dims);
 }
 
 inline void def_dispatch(compute::kernel_ctx_t &kernel_ctx,
