@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -25,12 +25,12 @@
 #include <utility>
 #include <vector>
 
-#include "dnnl.h"
+#include "oneapi/dnnl/dnnl.h"
 
 #include "common.hpp"
 #include "utils/parser.hpp"
 
-#include "tests/test_thread.hpp"
+#include "utils/parallel.hpp"
 
 // BENCHDNN_MEMORY_CHECK macro enables guarding mechanism for memory allocation:
 // memory block is allocated on a page boundary and the page after the block is
@@ -53,6 +53,8 @@ bench_mode_t PERF {0x4}; // Performance mode. May be combined with CORR.
 bench_mode_t LIST {0x8}; // Listing mode. Standalone mode to only create prb.
 bench_mode_t PROF {
         0x10}; // Profiling-based performance mode (may be only combined with performance mode).
+bench_mode_t INIT {
+        0x20}; // Initialization mode. Standalone mode to only create primitive.
 
 bool is_bench_mode(bench_mode_t user_mode) {
     return !(bench_mode & user_mode).none();
@@ -68,8 +70,11 @@ const char *state2str(res_state_t state) {
     CASE(SKIPPED);
     CASE(MISTRUSTED);
     CASE(UNIMPLEMENTED);
+    CASE(INVALID_ARGUMENTS);
     CASE(FAILED);
     CASE(LISTED);
+    CASE(EXECUTED);
+    CASE(INITIALIZED);
 #undef CASE
     assert(!"unknown res state");
     return "STATE_UNDEF";
@@ -81,7 +86,6 @@ const char *skip_reason2str(skip_reason_t skip_reason) {
     CASE(CASE_NOT_SUPPORTED);
     CASE(DATA_TYPE_NOT_SUPPORTED);
     CASE(INVALID_CASE);
-    CASE(KNOWN_LIMITATION);
     CASE(NOT_ENOUGH_RAM);
     CASE(SKIP_IMPL_HIT);
     CASE(SKIP_START);
@@ -89,66 +93,65 @@ const char *skip_reason2str(skip_reason_t skip_reason) {
     return "SKIP_UNKNOWN";
 }
 
-void parse_result(
-        res_t &res, bool &want_perf_report, int status, const char *pstr) {
+void parse_result(res_t &res, const char *pstr) {
     auto &bs = benchdnn_stat;
     const char *state = state2str(res.state);
-    want_perf_report = false;
 
     switch (res.state) {
         case UNTESTED:
-            if (is_bench_mode(PERF)) {
-                want_perf_report = true;
-                if (status == FAIL)
-                    bs.failed++;
-                else
-                    bs.passed++;
-                break;
-            } else if (is_bench_mode(RUN)) {
-                assert(status == OK);
-                BENCHDNN_PRINT(0, "%d:EXECUTED __REPRO: %s\n", bs.tests, pstr);
-                break;
-            }
+            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            bs.failed++;
+            break;
+        case EXECUTED:
+            if (is_bench_mode(RUN))
+                BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            bs.passed++;
+            break;
         case FAILED:
-            assert(status == FAIL);
             bs.failed++;
             BENCHDNN_PRINT(0, "%d:%s (errors:%lu total:%lu) __REPRO: %s\n",
                     bs.tests, state, (unsigned long)res.errors,
                     (unsigned long)res.total, pstr);
             break;
         case SKIPPED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s (%s) __REPRO: %s\n", bs.tests, state,
                     skip_reason2str(res.reason), pstr);
             bs.skipped++;
             break;
         case UNIMPLEMENTED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
             bs.unimplemented++;
             bs.failed++;
             break;
+        case INVALID_ARGUMENTS:
+            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            bs.invalid_arguments++;
+            bs.failed++;
+            break;
         case MISTRUSTED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            want_perf_report = true;
             bs.mistrusted++;
             break;
         case PASSED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
-            want_perf_report = true;
             bs.passed++;
             break;
         case LISTED:
-            assert(status == OK);
             BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
             bs.listed++;
             break;
+        case INITIALIZED:
+            BENCHDNN_PRINT(0, "%d:%s __REPRO: %s\n", bs.tests, state, pstr);
+            bs.passed++;
+            break;
         default: assert(!"unknown state"); SAFE_V(FAIL);
     }
 
-    if (want_perf_report && is_bench_mode(PERF)) {
+    bs.tests++;
+    assert(bs.tests
+            == bs.passed + bs.skipped + bs.mistrusted + bs.failed + bs.listed);
+
+    if (is_bench_mode(PERF)) {
         using bt = timer::timer_t;
         const auto &t = res.timer_map.perf_timer();
         for (int mode = 0; mode < (int)bt::n_modes; ++mode)
@@ -169,7 +172,7 @@ static void *zmalloc_protect(size_t size) {
     const size_t page_sz = getpagesize();
 
     const size_t block_sz = size + 3 * sizeof(void *);
-    const size_t total_sz = div_up(block_sz, page_sz) * page_sz + page_sz;
+    const size_t total_sz = rnd_up(block_sz, page_sz) + page_sz;
 
     void *mem_ptr;
     int rc = ::posix_memalign(&mem_ptr, page_sz, total_sz);
@@ -344,10 +347,10 @@ int readlink(const char *path, char *buf, size_t buf_max) {
 #include <unistd.h>
 #endif /* _WIN32 */
 
-std::string locate_batch_file(const std::string &fname) {
+std::string locate_file(const std::string &fname) {
     SAFE_V(fname.length() < PATH_MAX ? OK : FAIL);
 
-    const int max_paths = 4;
+    const int max_paths = 30;
 
     static int n_paths = 0;
     static std::string search_paths[max_paths];
@@ -367,7 +370,10 @@ std::string locate_batch_file(const std::string &fname) {
             break;
         }
     if (!dir_found) {
-        SAFE_V(n_paths < max_paths ? OK : FAIL);
+        if (n_paths >= max_paths) {
+            BENCHDNN_PRINT(0, "%s%d\n",
+                    "Warning: Number of searched paths exceeded ", max_paths);
+        }
         search_paths[n_paths++] = std::move(fdir);
     }
 
@@ -378,7 +384,7 @@ std::string locate_batch_file(const std::string &fname) {
         std::string fullname = search_paths[n] + "/" + fname;
         ifs.open(fullname);
         if (ifs.is_open()) {
-            BENCHDNN_PRINT(50, "batch file used: %s\n", fullname.c_str());
+            BENCHDNN_PRINT(50, "file used: %s\n", fullname.c_str());
             ifs.close();
             return fullname;
         }
@@ -391,19 +397,18 @@ std::string locate_batch_file(const std::string &fname) {
     fdir.resize(PATH_MAX);
     int length = readlink("/proc/self/exe", &fdir[0], PATH_MAX);
     if (length) {
-        std::string s_fdir = std::string(dirname(&fdir[0]));
+        std::string s_fdir = dirname(&fdir[0]);
         for (int i_try = 0; i_try < 2; ++i_try) {
-            fdir = s_fdir + std::string("/inputs/") + std::string(driver_name);
-            if (i_try == 1) { // Windows has different folder structure
-                fdir = s_fdir + std::string("/../inputs/")
-                        + std::string(driver_name);
-            }
-            // NOLINTNEXTLINE(performance-inefficient-string-concatenation)
-            std::string fullname = fdir + "/" + fname;
+            fdir = s_fdir;
+            fdir.append(i_try == 1 ? "/../inputs/" : "/inputs/");
+            assert(!driver_name.empty());
+            fdir.append(driver_name);
+            std::string fullname = fdir + "/";
+            fullname += fname;
             ifs.open(fullname);
             if (ifs.is_open()) {
                 search_paths[n_paths++] = std::move(fdir);
-                BENCHDNN_PRINT(50, "batch file used: %s\n", fullname.c_str());
+                BENCHDNN_PRINT(50, "file used: %s\n", fullname.c_str());
                 ifs.close();
                 return fullname;
             }
@@ -416,7 +421,7 @@ std::string locate_batch_file(const std::string &fname) {
 }
 
 int batch(const char *fname, bench_f bench) {
-    std::ifstream ifs(locate_batch_file(std::string(fname)));
+    std::ifstream ifs(locate_file(std::string(fname)));
     SAFE(ifs.is_open() ? OK : FAIL, CRIT);
 
     std::vector<std::string> opts;
@@ -469,6 +474,11 @@ int64_t div_up(const int64_t a, const int64_t b) {
     return (a + b - 1) / b;
 }
 
+int64_t rnd_up(const int64_t a, const int64_t b) {
+    SAFE_V(b != 0 ? OK : FAIL);
+    return div_up(a, b) * b;
+}
+
 int64_t next_pow2(int64_t a) {
     assert(a > 0 && a <= ((int64_t)1 << 62));
     if (a > 1) a--;
@@ -528,7 +538,7 @@ void gemm(const char *layout, const char *transa, const char *transb, int64_t m,
         return b[j * ldb + i];
     };
 
-    dnnl::impl::parallel_nd(m, n, [&](int64_t i, int64_t j) {
+    benchdnn_parallel_nd(m, n, [&](int64_t i, int64_t j) {
         float ab = 0.0f;
         for (int64_t _k = 0; _k < k; ++_k)
             ab += a_accessor(i, _k) * b_accessor(_k, j);
