@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -93,6 +93,7 @@ extern bool canonical;
 extern bool mem_check;
 extern bool attr_same_pd_check;
 extern std::string skip_impl; /* empty or "" means skip nothing */
+extern std::string driver_name;
 
 #define BENCHDNN_PRINT(v, fmt, ...) \
     do { \
@@ -108,9 +109,8 @@ extern std::string skip_impl; /* empty or "" means skip nothing */
     T &operator=(const T &) = delete;
 
 using bench_mode_t = std::bitset<8>;
-extern bench_mode_t RUN, CORR, PERF, LIST, PROF; // pre-defined modes
+extern bench_mode_t RUN, CORR, PERF, LIST, PROF, INIT; // pre-defined modes
 extern bench_mode_t bench_mode; // user mode
-extern const char *driver_name;
 
 bool is_bench_mode(bench_mode_t user_mode);
 
@@ -131,6 +131,7 @@ struct stat_t {
     int skipped;
     int mistrusted;
     int unimplemented;
+    int invalid_arguments;
     int listed;
     double ms[timer::timer_t::mode_t::n_modes];
 };
@@ -143,8 +144,11 @@ enum res_state_t {
     SKIPPED,
     MISTRUSTED,
     UNIMPLEMENTED,
+    INVALID_ARGUMENTS,
     FAILED,
-    LISTED
+    LISTED,
+    INITIALIZED,
+    EXECUTED,
 };
 const char *state2str(res_state_t state);
 
@@ -153,7 +157,6 @@ enum skip_reason_t {
     CASE_NOT_SUPPORTED,
     DATA_TYPE_NOT_SUPPORTED,
     INVALID_CASE,
-    KNOWN_LIMITATION,
     NOT_ENOUGH_RAM,
     SKIP_IMPL_HIT,
     SKIP_START,
@@ -169,8 +172,7 @@ struct res_t {
     size_t ibytes, obytes;
 };
 
-void parse_result(
-        res_t &res, bool &want_perf_report, int status, const char *pstr);
+void parse_result(res_t &res, const char *pstr);
 
 /* misc */
 void init_fp_mode();
@@ -186,12 +188,14 @@ bool match_regex(const char *str, const char *pattern);
 bool maybe_skip(const std::string &impl_str);
 
 typedef int (*bench_f)(int argc, char **argv);
+std::string locate_file(const std::string &fname);
 int batch(const char *fname, bench_f bench);
 
 /* returns 1 with given probability */
 int flip_coin(ptrdiff_t seed, float probability);
 
 int64_t div_up(const int64_t a, const int64_t b);
+int64_t rnd_up(const int64_t a, const int64_t b);
 int64_t next_pow2(int64_t a);
 int mxcsr_cvt(float f);
 
