@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2021 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,9 +14,9 @@
 * limitations under the License.
 *******************************************************************************/
 
-#include "tests/test_thread.hpp"
+#include "utils/parallel.hpp"
 
-#include "conv/conv_common.hpp"
+#include "conv/ref_conv.hpp"
 
 namespace conv {
 
@@ -25,6 +25,34 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
     const dnn_mem_t &wei_m = args.find(DNNL_ARG_WEIGHTS);
     const dnn_mem_t &bia_m = args.find(DNNL_ARG_BIAS);
     const dnn_mem_t &dst_m = args.find(DNNL_ARG_DST);
+    const dnn_mem_t &src_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &dst_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
+    const dnn_mem_t &src_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const dnn_mem_t &dst_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
+
+    const bool has_src_scale = !prb->attr.scales.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_scale = !prb->attr.scales.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_scale = !prb->attr.scales.get(DNNL_ARG_DST).is_def();
+    assert(IMPLICATION(has_src_scale, src_scales.nelems() == 1));
+    assert(IMPLICATION(has_dst_scale, dst_scales.nelems() == 1));
+    float src_scale = has_src_scale ? src_scales.get_elem(0) : 1.f;
+    float dst_scale = has_dst_scale ? 1.f / dst_scales.get_elem(0) : 1.f;
+    const int wei_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_WEIGHTS, dnnl_convolution, prb->has_groups);
+
+    const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
+    const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
+    const int src_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_SRC).policy);
+    const int dst_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_DST).policy);
+
     /* help compiler optimize the code */
     const int64_t MB = prb->mb, G = prb->g, OC = prb->oc, IC = prb->ic;
     const int64_t OCG = OC / G, ICG = IC / G;
@@ -57,10 +85,13 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
                     for (int64_t ic = 0; ic < ICG; ++ic) {
                         int64_t src_off = ((ic * ID + id) * IH + ih) * IW + iw;
                         int64_t wei_off = ((ic * KD + kd) * KH + kh) * KW + kw;
-                        float s = src_loc[src_off];
-                        maybe_zero_point(prb->attr, s, prb->src_zp,
-                                g * ICG + ic, DNNL_ARG_SRC);
-                        d += s * wei_loc[wei_off];
+                        int src_zp = has_src_zp ? src_zps.get_elem(
+                                             src_zp_mask > 0 ? g * ICG + ic : 0)
+                                                : 0;
+                        const float s = src_loc[src_off];
+                        const float w = wei_loc[wei_off];
+                        const float d_tmp = (s - src_zp) * w;
+                        d += d_tmp;
                     }
                 }
             }
@@ -68,7 +99,7 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
     };
 
     auto v_po_masks = prb->attr.post_ops.get_po_masks();
-    dnnl::impl::parallel_nd(G, MB, OCG, OD, OH, OW,
+    benchdnn_parallel_nd(G, MB, OCG, OD, OH, OW,
             [&](int64_t g, int64_t mb, int64_t oc, int64_t od, int64_t oh,
                     int64_t ow) {
                 const size_t dst_off = dst_off_f(prb, mb, g, oc, od, oh, ow);
@@ -77,22 +108,29 @@ void compute_ref_direct_fwd(const prb_t *prb, const args_t &args) {
                 float conv_res = 0;
                 ker(conv_res, g, mb, oc, od, oh, ow);
 
+                // apply scale as:
+                //    dst = src_scale * wei_scale * conv(src - zp_src, wei)
+                float wei_scale = 1.f;
+                if (has_wei_scale)
+                    wei_scale = wei_scales.get_elem(
+                            wei_scale_mask > 0 ? g * OCG + oc : 0);
+                const float scale = src_scale * wei_scale;
+                conv_res *= scale;
+
                 if (prb->dir & FLAG_BIA) {
                     const size_t bia_off = bia_off_f(prb, g, oc);
                     conv_res += ((float *)bia_m)[bia_off];
                 }
 
-                maybe_oscale(prb->attr, conv_res, prb->scales, g * OCG + oc);
-
                 const auto v_po_vals
                         = prepare_po_vals(dst_m, args, v_po_masks, dst_off);
 
                 maybe_post_ops(prb->attr, conv_res, dst, v_po_vals);
 
-                maybe_zero_point(prb->attr, conv_res, prb->dst_zp, g * OCG + oc,
-                        DNNL_ARG_DST, true);
-
-                dst = conv_res;
+                int dst_zp = has_dst_zp
+                        ? dst_zps.get_elem(dst_zp_mask > 0 ? g * OCG + oc : 0)
+                        : 0;
+                dst = conv_res * dst_scale + dst_zp;
             });
 }
 
@@ -101,6 +139,34 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
     const dnn_mem_t &wei_m = args.find(DNNL_ARG_WEIGHTS);
     const dnn_mem_t &bia_m = args.find(DNNL_ARG_BIAS);
     const dnn_mem_t &diff_dst_m = args.find(DNNL_ARG_DIFF_DST);
+    const dnn_mem_t &src_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_SRC);
+    const dnn_mem_t &wei_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_WEIGHTS);
+    const dnn_mem_t &dst_scales
+            = args.find(DNNL_ARG_ATTR_SCALES | DNNL_ARG_DST);
+    const dnn_mem_t &src_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC);
+    const dnn_mem_t &dst_zps
+            = args.find(DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_DST);
+
+    const bool has_src_scale = !prb->attr.scales.get(DNNL_ARG_SRC).is_def();
+    const bool has_wei_scale = !prb->attr.scales.get(DNNL_ARG_WEIGHTS).is_def();
+    const bool has_dst_scale = !prb->attr.scales.get(DNNL_ARG_DST).is_def();
+    assert(IMPLICATION(has_src_scale, src_scales.nelems() == 1));
+    assert(IMPLICATION(has_dst_scale, dst_scales.nelems() == 1));
+    float src_scale = has_src_scale ? src_scales.get_elem(0) : 1.f;
+    float dst_scale = has_dst_scale ? 1.f / dst_scales.get_elem(0) : 1.f;
+    const int wei_scale_mask = prb->attr.scales.get_mask(
+            DNNL_ARG_WEIGHTS, dnnl_convolution, prb->has_groups);
+
+    const bool has_src_zp = !prb->attr.zero_points.get(DNNL_ARG_SRC).is_def();
+    const bool has_dst_zp = !prb->attr.zero_points.get(DNNL_ARG_DST).is_def();
+    const int src_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_SRC).policy);
+    const int dst_zp_mask = attr_t::get_default_mask(
+            prb->attr.zero_points.get(DNNL_ARG_DST).policy);
+
     /* help compiler optimize the code */
     const int64_t MB = prb->mb, G = prb->g, OC = prb->oc, IC = prb->ic;
     const int64_t OCG = OC / G, ICG = IC / G;
@@ -116,18 +182,6 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
     enum { precompute_size = 16 };
     const bool fast = MAX3(KD, KH, KW) <= precompute_size;
 
-    // from bwd pov zp src from fwd is zp diff dst and
-    // zp dst is zp dst is zp diff_src
-    const auto map_arg_to_zp_arg = [](int num) {
-        switch (num) {
-            case DNNL_ARG_DIFF_DST: return DNNL_ARG_SRC;
-            case DNNL_ARG_DIFF_SRC: return DNNL_ARG_DST;
-            default: assert(false && "map_arg_to_zp_arg unsupported arg");
-        }
-
-        return -1;
-    };
-
     /* pre-computes arrays of oh(ow) and kh(kw) for traversing in kernel */
     auto precompute_ok
             = [](int64_t i, int64_t O, int64_t K, int64_t S, int64_t P,
@@ -167,10 +221,18 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
                         = ((oc * OD + od[d]) * OH + oh[h]) * OW + ow[w];
                 const int64_t wei_off
                         = ((oc * ICG * KD + kd[d]) * KH + kh[h]) * KW + kw[w];
-                float diff_dst_val = diff_dst_loc[diff_dst_off];
-                maybe_zero_point(prb->attr, diff_dst_val, prb->src_zp,
-                        g * OCG + oc, map_arg_to_zp_arg(DNNL_ARG_DIFF_DST));
-                ds += diff_dst_val * wei_loc[wei_off];
+                int src_zp = has_src_zp
+                        ? src_zps.get_elem(src_zp_mask > 0 ? g * OCG + oc : 0)
+                        : 0;
+                float diff_dst_val
+                        = (diff_dst_loc[diff_dst_off] - src_zp) * src_scale;
+
+                float wei_scale = 1.f;
+                if (has_wei_scale)
+                    wei_scale = wei_scales.get_elem(
+                            wei_scale_mask > 0 ? g * ICG + ic : 0);
+                float wei_val = wei_loc[wei_off] * wei_scale;
+                ds += diff_dst_val * wei_val;
             }
         }
     };
@@ -199,12 +261,19 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
                                 = ((oc * OD + od) * OH + oh) * OW + ow;
                         const int64_t wei_off
                                 = ((oc * ICG * KD + kd) * KH + kh) * KW + kw;
-                        float diff_dst_val = diff_dst_loc[diff_dst_off];
-                        maybe_zero_point(prb->attr, diff_dst_val, prb->src_zp,
-                                g * OCG + oc,
-                                map_arg_to_zp_arg(DNNL_ARG_DIFF_DST));
+                        int src_zp = has_src_zp ? src_zps.get_elem(
+                                             src_zp_mask > 0 ? g * OCG + oc : 0)
+                                                : 0;
+                        float diff_dst_val
+                                = (diff_dst_loc[diff_dst_off] - src_zp)
+                                * src_scale;
 
-                        ds += diff_dst_val * wei_loc[wei_off];
+                        float wei_scale = 1.f;
+                        if (has_wei_scale)
+                            wei_scale = wei_scales.get_elem(
+                                    wei_scale_mask > 0 ? g * ICG + ic : 0);
+                        float wei_val = wei_loc[wei_off] * wei_scale;
+                        ds += diff_dst_val * wei_val;
                     }
                 }
             }
@@ -212,7 +281,7 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
     };
 
     auto v_po_masks = prb->attr.post_ops.get_po_masks();
-    dnnl::impl::parallel_nd(G, MB, ICG, ID, IH, IW,
+    benchdnn_parallel_nd(G, MB, ICG, ID, IH, IW,
             [&](int64_t g, int64_t mb, int64_t ic, int64_t id, int64_t ih,
                     int64_t iw) {
                 size_t src_off = src_off_f(prb, mb, g, ic, id, ih, iw);
@@ -227,16 +296,16 @@ void compute_ref_direct_bwd_d(const prb_t *prb, const args_t &args) {
                     const size_t bia_off = (size_t)g * ICG + ic;
                     conv_res += ((float *)bia_m)[bia_off];
                 }
-                maybe_oscale(prb->attr, conv_res, prb->scales, g * ICG + ic);
 
                 const auto v_po_vals = prepare_po_vals(
                         diff_src_m, args, v_po_masks, src_off);
 
                 maybe_post_ops(prb->attr, conv_res, ds, v_po_vals);
-                maybe_zero_point(prb->attr, conv_res, prb->dst_zp, g * ICG + ic,
-                        map_arg_to_zp_arg(DNNL_ARG_DIFF_SRC), true);
 
-                ds = conv_res;
+                int dst_zp = has_dst_zp
+                        ? dst_zps.get_elem(dst_zp_mask > 0 ? g * ICG + ic : 0)
+                        : 0;
+                ds = conv_res * dst_scale + dst_zp;
             });
 }
 
@@ -294,7 +363,7 @@ void compute_ref_bwd_weights(const prb_t *prb, const args_t &args) {
         }
     };
 
-    dnnl::impl::parallel_nd(G, OCG, ICG, KD, KH, KW,
+    benchdnn_parallel_nd(G, OCG, ICG, KD, KH, KW,
             [&](int64_t g, int64_t oc, int64_t ic, int64_t kd, int64_t kh,
                     int64_t kw) {
                 size_t wei_off = wei_off_f(prb, g, oc, ic, kd, kh, kw);
@@ -312,7 +381,7 @@ void compute_ref_bwd_bias(const prb_t *prb, const args_t &args) {
     const int64_t OCG = OC / G;
     const int64_t OD = prb->od, OH = prb->oh, OW = prb->ow;
 
-    dnnl::impl::parallel_nd(G, OCG, [&](int64_t g, int64_t oc) {
+    benchdnn_parallel_nd(G, OCG, [&](int64_t g, int64_t oc) {
         size_t bia_off = bia_off_f(prb, g, oc);
         double sum = 0;
 
@@ -334,7 +403,7 @@ void compute_ref_direct_bwd_w(const prb_t *prb, const args_t &args) {
 }
 
 void compute_ref_fwd(
-        const prb_t *prb, dnnl_primitive_t prim_ref, const args_t &args) {
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
     if (prim_ref) {
         SAFE_V(execute_and_wait(prim_ref, args));
         return;
@@ -348,7 +417,7 @@ void compute_ref_fwd(
 }
 
 void compute_ref_bwd_d(
-        const prb_t *prb, dnnl_primitive_t prim_ref, const args_t &args) {
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
     if (prim_ref) {
         SAFE_V(execute_and_wait(prim_ref, args));
         return;
@@ -362,7 +431,7 @@ void compute_ref_bwd_d(
 }
 
 void compute_ref_bwd_w(
-        const prb_t *prb, dnnl_primitive_t prim_ref, const args_t &args) {
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
     if (prim_ref) {
         SAFE_V(execute_and_wait(prim_ref, args));
         return;
@@ -375,4 +444,14 @@ void compute_ref_bwd_w(
     }
 }
 
+void compute_ref(
+        const prb_t *prb, const args_t &args, dnnl_primitive_t prim_ref) {
+    if (prb->dir & FLAG_FWD)
+        compute_ref_fwd(prb, args, prim_ref);
+    else if (prb->dir == BWD_D)
+        compute_ref_bwd_d(prb, args, prim_ref);
+    else if (prb->dir & FLAG_BWD && prb->dir & FLAG_WEI)
+        compute_ref_bwd_w(prb, args, prim_ref);
+}
+
 } // namespace conv
