@@ -23,6 +23,7 @@
 #include <string>
 
 #include <algorithm>
+#include <iostream>
 #include <sstream>
 
 #include "oneapi/dnnl/dnnl.h"
@@ -30,6 +31,7 @@
 #include "src/common/math_utils.hpp"
 
 #include "common.hpp"
+#include "conv/conv_dw_fusion.hpp"
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
 #include "dnnl_debug.hpp"
@@ -112,7 +114,6 @@ const char *data_kind2str(data_kind_t kind) {
         case ACC: return "ACC";
         case MEAN: return "MEAN";
         case VAR: return "VAR";
-        case SS: return "SS";
         case SC: return "SC";
         case SH: return "SH";
         case DST_ITER: return "DST_ITER";
@@ -215,9 +216,10 @@ int attr_t::get_default_mask(policy_t policy) {
     }
 }
 
-// This function takes input string, extracts float value and runtime, if
-// present, from the string. Updates @value and @runtime with extracted values.
-int parse_value_and_runtime(float &value, bool &runtime, const std::string &s) {
+// This function takes input string, extracts float value and asteriks, if
+// present, from the string. Updates @value with extracted values.
+// TODO: remove asteriks from all inputs and update doc.
+int parse_value_and_runtime(float &value, const std::string &s) {
     // process value
     size_t scale_pos = 0;
     try {
@@ -229,16 +231,48 @@ int parse_value_and_runtime(float &value, bool &runtime, const std::string &s) {
                 "Expected input: \'VAL[*]\'. See help for proper syntax.");
         exit(1);
     }
-    runtime = false;
     if (scale_pos + 1 < s.size()) return FAIL;
     if (scale_pos == s.size()) return OK;
     if (s.back() != '*') return FAIL;
-    runtime = true;
     return OK;
 }
 
-int attr_t::scale_t::from_str(const std::string &s) {
-    *this = scale_t();
+int attr_t::arg_scales_t::entry_t::policy2mask(
+        int arg, dnnl_primitive_kind_t prim_kind, bool has_groups) const {
+    const auto policy = this->policy;
+
+    if (arg != DNNL_ARG_WEIGHTS || policy == policy_t::COMMON)
+        return attr_t::get_default_mask(policy);
+
+    // Handle of weights mask for various primitives.
+    if (prim_kind == dnnl_convolution || prim_kind == dnnl_deconvolution
+            || prim_kind == dnnl_inner_product) {
+        switch (policy) {
+            case PER_OC:
+                if (has_groups)
+                    return attr_t::get_default_mask(PER_DIM_01);
+                else
+                    return attr_t::get_default_mask(PER_DIM_0);
+            default: SAFE(FAIL, CRIT); return -1;
+        }
+    } else if (prim_kind == dnnl_matmul) {
+        switch (policy) {
+            // TODO: add batch dimension?
+            case PER_OC: return attr_t::get_default_mask(PER_DIM_1);
+            default: SAFE(FAIL, CRIT); return -1;
+        }
+    } else {
+        assert(prim_kind == dnnl_undefined_primitive);
+        assert(!"Weights may have specific mask for a given primitive. "
+                "Please re-direct new primitive to one of two branches "
+                "above");
+        SAFE(FAIL, CRIT);
+        return -1;
+    }
+}
+
+int attr_t::arg_scales_t::entry_t::from_str(const std::string &s) {
+    *this = arg_scales_t::entry_t();
     if (s.empty()) return OK;
 
     size_t start_pos = 0;
@@ -248,8 +282,8 @@ int attr_t::scale_t::from_str(const std::string &s) {
     if (start_pos == std::string::npos) return OK;
     if (start_pos >= s.size()) return FAIL; // to catch dangling ':'
 
-    SAFE(parse_value_and_runtime(this->scale, this->runtime,
-                 parser::get_substr(s, start_pos, ':')),
+    SAFE(parse_value_and_runtime(
+                 this->scale, parser::get_substr(s, start_pos, ':')),
             WARN);
     if (this->scale < 0) return FAIL;
     return OK;
@@ -275,11 +309,10 @@ int attr_t::zero_points_t::from_str(const std::string &s) {
             return FAIL;
 
         float zp = 0;
-        bool runtime = false;
         SAFE(parse_value_and_runtime(
-                     zp, runtime, parser::get_substr(subs, subs_pos, '\0')),
+                     zp, parser::get_substr(subs, subs_pos, '\0')),
                 WARN);
-        set(arg, policy, static_cast<int>(zp), runtime);
+        set(arg, policy, static_cast<int>(zp));
     }
     return OK;
 }
@@ -303,7 +336,7 @@ int attr_t::arg_scales_t::from_str(const std::string &s) {
                 || subs_pos >= s.size())
             return FAIL;
 
-        scale_t arg_scale;
+        arg_scales_t::entry_t arg_scale;
         SAFE(arg_scale.from_str(parser::get_substr(subs, subs_pos, '\0')),
                 WARN);
         set(arg, arg_scale);
@@ -329,8 +362,6 @@ static po_table_entry_t kind_table[] = {
         // eltwise
         {pk_t::ELTWISE_START, {"eltwise_undef"}, dnnl_alg_kind_undef},
         {pk_t::ABS, {"abs", "eltwise_abs"}, dnnl_eltwise_abs},
-        {pk_t::BRELU, {"bounded_relu", "eltwise_bounded_relu", "brelu"},
-                dnnl_eltwise_bounded_relu},
         {pk_t::CLIP, {"clip", "eltwise_clip"}, dnnl_eltwise_clip},
         {pk_t::CLIP_V2, {"clip_v2", "eltwise_clip_v2"}, dnnl_eltwise_clip_v2},
         {pk_t::CLIP_V2_DST, {"clip_v2_dst", "eltwise_clip_v2_use_dst_for_bwd"},
@@ -356,8 +387,6 @@ static po_table_entry_t kind_table[] = {
         {pk_t::LOGISTIC_DST,
                 {"logistic_dst", "eltwise_logistic_use_dst_for_bwd"},
                 dnnl_eltwise_logistic_use_dst_for_bwd},
-        {pk_t::LOGSIGMOID, {"logsigmoid", "eltwise_logsigmoid"},
-                dnnl_eltwise_logsigmoid},
         {pk_t::MISH, {"mish", "eltwise_mish"}, dnnl_eltwise_mish},
         {pk_t::POW, {"pow", "eltwise_pow"}, dnnl_eltwise_pow},
         {pk_t::RELU, {"relu", "eltwise_relu"}, dnnl_eltwise_relu},
@@ -370,8 +399,6 @@ static po_table_entry_t kind_table[] = {
         {pk_t::SQUARE, {"square", "eltwise_square"}, dnnl_eltwise_square},
         {pk_t::SRELU, {"soft_relu", "eltwise_soft_relu", "srelu"},
                 dnnl_eltwise_soft_relu},
-        {pk_t::SRELU_V2, {"soft_relu_v2", "eltwise_soft_relu_v2", "srelu_v2"},
-                dnnl_eltwise_soft_relu_v2},
         {pk_t::SWISH, {"swish", "eltwise_swish"}, dnnl_eltwise_swish},
         {pk_t::TANH, {"tanh", "eltwise_tanh"}, dnnl_eltwise_tanh},
         {pk_t::TANH_DST, {"tanh_dst", "eltwise_tanh_use_dst_for_bwd"},
@@ -406,7 +433,9 @@ pk_t attr_t::post_ops_t::str2kind(const std::string &str) {
             if (s == name) return e.kind;
         }
     }
-    assert(!"unknown attr_t::post_ops_t::kind_t kind");
+    BENCHDNN_PRINT(0, "%s\'%s\' %s\n", "Error: ", str.c_str(),
+            "kind of post operation entry was not recognized.");
+
     const auto table_size = sizeof(kind_table) / sizeof(*kind_table);
     return kind_table[table_size - 1].kind;
 }
@@ -511,7 +540,14 @@ int attr_t::post_ops_t::from_str(const std::string &s) {
             if (subs_pos >= subs.size()) return FAIL; // to catch dangling ':'
 
             auto scale_str = parser::get_substr(subs, subs_pos, '+');
-            SAFE(e.convolution.oscale.from_str(scale_str), WARN);
+            SAFE(e.convolution.wei_scale.from_str(scale_str), WARN);
+            size_t dst_scale_pos = 0;
+            for (int i = 0; i < 2; ++i)
+                dst_scale_pos = scale_str.find(":", dst_scale_pos + 1);
+            if (dst_scale_pos != std::string::npos) {
+                auto dst_scale_str = scale_str.substr(dst_scale_pos + 1);
+                SAFE(e.convolution.dst_scale.from_str(dst_scale_str), WARN);
+            }
         } else if (e.is_eltwise_kind()) {
             e.eltwise.alpha
                     = std::stof(parser::get_substr(subs, subs_pos, ':'));
@@ -521,10 +557,6 @@ int attr_t::post_ops_t::from_str(const std::string &s) {
             e.eltwise.beta = std::stof(parser::get_substr(subs, subs_pos, ':'));
             if (subs_pos == std::string::npos) continue;
             if (subs_pos >= subs.size()) return FAIL; // to catch dangling ':'
-
-            e.eltwise.scale
-                    = std::stof(parser::get_substr(subs, subs_pos, ':'));
-            if (e.eltwise.scale <= 0) return FAIL;
         } else if (e.is_binary_kind()) {
             e.binary.src1_dt
                     = str2dt(parser::get_substr(subs, subs_pos, ':').c_str());
@@ -554,9 +586,8 @@ int attr_t::post_ops_t::from_str(const std::string &s) {
 }
 
 bool attr_t::is_def(bool skip_fpmath) const {
-    return oscale.is_def() && scales.is_def() && zero_points.is_def()
-            && post_ops.is_def()
-            && scratchpad_mode == dnnl_scratchpad_mode_library
+    return scales.is_def() && zero_points.is_def() && post_ops.is_def()
+            && scratchpad_mode == get_default_scratchpad_mode()
             && IMPLICATION(
                     !skip_fpmath, fpmath_mode == dnnl_fpmath_mode_strict);
 }
@@ -618,20 +649,21 @@ std::ostream &operator<<(std::ostream &s, const policy_t &policy) {
     return s;
 }
 
-std::ostream &operator<<(std::ostream &s, const attr_t::scale_t &scale) {
-    s << scale.policy << ":" << scale.scale;
-    if (scale.runtime) s << '*';
+std::ostream &operator<<(
+        std::ostream &s, const attr_t::arg_scales_t::entry_t &scale) {
+    // TODO: remove '*'
+    s << scale.policy << ":" << scale.scale << '*';
     return s;
 }
 
 std::ostream &operator<<(
         std::ostream &s, const attr_t::zero_points_t &zero_points) {
     const char *delim = "";
+    // TODO: remove '*'
     for (const auto &point : zero_points.points) {
         s << delim;
         s << arg2str(point.first) << ":" << point.second.policy << ":"
-          << point.second.value;
-        if (point.second.runtime) s << '*';
+          << point.second.value << '*';
         delim = "+";
     }
 
@@ -674,25 +706,23 @@ std::ostream &operator<<(std::ostream &s, const attr_t::post_ops_t &post_ops) {
                 s << ":k" << e.convolution.kernel << "s" << e.convolution.stride
                   << "p" << e.convolution.padding;
             }
-            const auto &co = e.convolution.oscale;
-            if (e.convolution.dst_dt != dnnl_f32 || !co.is_def())
+            const auto &c_ws = e.convolution.wei_scale;
+            const auto &c_ds = e.convolution.dst_scale;
+            if (e.convolution.dst_dt != dnnl_f32 || !c_ws.is_def()
+                    || !c_ds.is_def())
                 s << ":" << e.convolution.dst_dt;
-            if (!co.is_def()) s << ":" << co;
+            if (!c_ws.is_def() || !c_ds.is_def()) s << ":" << c_ws;
+            if (!c_ds.is_def()) s << ":" << c_ds;
         } else if (e.is_eltwise_kind()) {
-            if (e.eltwise.scale != 1.f)
-                s << ":" << e.eltwise.alpha << ":" << e.eltwise.beta << ":"
-                  << e.eltwise.scale;
-            else if (e.eltwise.beta != 0.f)
+            if (e.eltwise.beta != 0.f)
                 s << ":" << e.eltwise.alpha << ":" << e.eltwise.beta;
             else if (e.eltwise.alpha != 0.f)
                 s << ":" << e.eltwise.alpha;
         } else if (e.is_binary_kind()) {
             s << ":" << e.binary.src1_dt;
-            if (e.binary.policy != policy_t::COMMON) {
+            if (e.binary.policy != policy_t::COMMON || e.binary.tag != tag::any)
                 s << ":" << e.binary.policy;
-                if (attr_t::get_default_mask(e.binary.policy) >= 4)
-                    s << ":" << e.binary.tag;
-            }
+            if (e.binary.tag != tag::any) s << ":" << e.binary.tag;
         } else if (e.is_prelu_kind()) {
             if (e.prelu.policy != policy_t::COMMON) {
                 s << ":" << e.prelu.policy;
@@ -718,13 +748,12 @@ std::ostream &operator<<(std::ostream &s, dnnl_fpmath_mode_t fm) {
 
 std::ostream &operator<<(std::ostream &s, const attr_t &attr) {
     if (!attr.is_def()) {
-        if (!attr.oscale.is_def()) s << "--attr-oscale=" << attr.oscale << " ";
         if (!attr.scales.is_def()) s << "--attr-scales=" << attr.scales << " ";
         if (!attr.zero_points.is_def())
             s << "--attr-zero-points=" << attr.zero_points << " ";
         if (!attr.post_ops.is_def())
             s << "--attr-post-ops=" << attr.post_ops << " ";
-        if (attr.scratchpad_mode != dnnl_scratchpad_mode_library)
+        if (attr.scratchpad_mode != attr_t::get_default_scratchpad_mode())
             s << "--attr-scratchpad=" << attr.scratchpad_mode << " ";
         if (attr.fpmath_mode != dnnl_fpmath_mode_strict)
             s << "--attr-fpmath=" << attr.fpmath_mode << " ";
@@ -732,6 +761,36 @@ std::ostream &operator<<(std::ostream &s, const attr_t &attr) {
     return s;
 }
 
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+std::ostream &operator<<(std::ostream &s, dnnl_sparse_encoding_t se) {
+    s << sparse_encoding2str(se);
+    return s;
+}
+
+std::ostream &operator<<(
+        std::ostream &s, const sparse_options_t &sparse_options) {
+    if (!sparse_options.is_def()) {
+        s << "--encoding=";
+        const std::vector<int> args
+                = {DNNL_ARG_SRC, DNNL_ARG_WEIGHTS, DNNL_ARG_DST};
+
+        for (int i = 0; i < (int)args.size(); i++) {
+            const int arg = args[i];
+            if (!sparse_options.is_encoding_def(arg)) {
+                s << sparse_options.get_encoding(arg);
+                if (!sparse_options.is_sparsity_def(arg))
+                    s << "+" << sparse_options.get_sparsity(arg);
+            }
+            if (i != (int)args.size() - 1)
+                s << ":";
+            else
+                s << " ";
+        }
+    }
+    return s;
+}
+#endif
+
 std::ostream &operator<<(std::ostream &s, bench_mode_t mode) {
     if (is_bench_mode(RUN) && !(is_bench_mode(CORR) || is_bench_mode(PERF)))
         s << "R";
@@ -739,6 +798,7 @@ std::ostream &operator<<(std::ostream &s, bench_mode_t mode) {
     if (is_bench_mode(PERF)) s << "P";
     if (is_bench_mode(LIST)) s << "L";
     if (is_bench_mode(PROF)) s << "O";
+    if (is_bench_mode(INIT)) s << "I";
     return s;
 }
 
@@ -802,7 +862,7 @@ dnnl_scratchpad_mode_t str2scratchpad_mode(const char *str) {
         return dnnl_scratchpad_mode_user;
 
     assert(!"not expected");
-    return dnnl_scratchpad_mode_library;
+    return attr_t::get_default_scratchpad_mode();
 }
 
 dnnl_fpmath_mode_t str2fpmath_mode(const char *str) {
@@ -830,11 +890,6 @@ dnnl_fpmath_mode_t str2fpmath_mode(const char *str) {
 #undef CASE
 }
 
-void attr_args_t::prepare_output_scales(
-        const attr_t &attr, const void *vals, int64_t count, int mask) {
-    insert(DNNL_ARG_ATTR_OUTPUT_SCALES, vals, count, mask, attr.oscale.runtime);
-}
-
 struct post_ops_rhs_tensor_entry_t {
     dnnl_data_type_t dt;
     policy_t policy;
@@ -860,8 +915,11 @@ post_ops_rhs_tensor_entry_t get_po_rhs_tensor_entry(
 } // namespace
 
 int attr_args_t::prepare_post_ops_mds(
-        const attr_t &attr, int ndims, const dnnl_dims_t dims) {
+        const attr_t &attr, int ndims, const dnnl_dims_t prb_dims) {
     const auto &po = attr.post_ops;
+    dnnl_dims_t dims;
+    for (int d = 0; d < ndims; ++d)
+        dims[d] = prb_dims[d];
     // iterate over all post ops and prepare md for each binary
     for (int idx = 0; idx < po.len(); ++idx) {
         const auto &e = po.entry[idx];
@@ -876,29 +934,25 @@ int attr_args_t::prepare_post_ops_mds(
             for (auto d = 0; d < ndims; ++d)
                 rhs_tensor_dims[d] = (!(mask & (1 << d))) ? 1 : dims[d];
 
-            dnnl_memory_desc_t rhs_tensor_desc;
-            rhs_tensor_desc = dnn_mem_t::init_md(ndims, rhs_tensor_dims,
+            auto rhs_tensor_desc = dnn_mem_t::init_md(ndims, rhs_tensor_dims,
                     po_rhs_tensor_entry.dt, po_rhs_tensor_entry.tag);
             mds.emplace((DNNL_ARG_ATTR_MULTIPLE_POST_OP(idx)
                                 | po_rhs_tensor_entry.arg_attr_mask),
-                    rhs_tensor_desc);
+                    std::move(rhs_tensor_desc));
+        } else if (e.is_convolution_kind()) {
+            // Update dims for post operations appended after conv_dw
+            conv_dw_fusion::get_fused_conv_dst_dims(ndims, e, dims, dims);
         }
     }
 
     return OK;
 }
 
-void attr_args_t::prepare_dw_post_op(const attr_t &attr,
-        dnnl_data_type_t wei_dt, dnnl_data_type_t bia_dt, const void *vals,
-        int64_t count, int mask) {
+void attr_args_t::prepare_dw_post_op(
+        const attr_t &attr, dnnl_data_type_t wei_dt, dnnl_data_type_t bia_dt) {
     const int dw_idx = attr.post_ops.convolution_index();
     if (dw_idx == -1) return;
 
-    const auto &dw = attr.post_ops.entry[dw_idx].convolution;
-    // insert output scale which applies in fused convolution
-    insert(DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_ATTR_OUTPUT_SCALES, vals, count,
-            mask, dw.oscale.runtime);
-
     dw_entry.wei_dt = wei_dt;
     dw_entry.bia_dt = bia_dt;
 }
@@ -908,30 +962,22 @@ dnnl_primitive_attr_t create_dnnl_attr(
     dnnl_primitive_attr_t dnnl_attr = nullptr;
     DNN_SAFE_V(dnnl_primitive_attr_create(&dnnl_attr));
 
-    if (!attr.oscale.is_def()) {
-        const auto &os_args = attr_args.get(DNNL_ARG_ATTR_OUTPUT_SCALES);
-        const auto &policy = attr.oscale.policy;
-
-        const auto count = os_args.get_count(policy);
-        const auto mask = os_args.get_mask(policy);
-        const auto scales = os_args.get_float_ptr();
-
-        DNN_SAFE_V(dnnl_primitive_attr_set_output_scales(
-                dnnl_attr, count, mask, scales));
-    } else if (!attr.scales.is_def()) {
+    if (!attr.scales.is_def()) {
         const auto &as = attr.scales;
         for (const auto &arg : as.scales) {
             const int arg_name = arg.first;
             if (as.is_def(arg_name)) continue;
 
             const auto &e = arg.second;
-            // Only common policy is supported in the library at this point
-            int64_t count = 1;
-            int mask = attr_t::get_default_mask(e.policy);
-            const float *scales = e.runtime ? &DNNL_RUNTIME_F32_VAL : &e.scale;
+            // Weights mask differs from primitive to primitive, that's why it
+            // is stashed in `attr_args` at primitive creation time.
+            int mask = (arg_name == DNNL_ARG_WEIGHTS
+                               && e.policy != policy_t::COMMON)
+                    ? attr_args.get_mask(arg_name)
+                    : e.policy2mask(arg_name);
 
-            DNN_SAFE_V(dnnl_primitive_attr_set_scales(
-                    dnnl_attr, arg_name, count, mask, scales));
+            DNN_SAFE_V(dnnl_primitive_attr_set_scales_mask(
+                    dnnl_attr, arg_name, mask));
         }
     }
 
@@ -942,14 +988,10 @@ dnnl_primitive_attr_t create_dnnl_attr(
             if (zp.is_def(arg_name)) continue;
 
             const auto &e = arg.second;
-            // Only common policy/single RT value are supported in the library
-            // at this point
-            int64_t count = 1;
             int mask = attr_t::get_default_mask(e.policy);
-            const auto values = e.runtime ? &DNNL_RUNTIME_S32_VAL : &e.value;
 
-            DNN_SAFE_V(dnnl_primitive_attr_set_zero_points(
-                    dnnl_attr, arg_name, count, mask, values));
+            DNN_SAFE_V(dnnl_primitive_attr_set_zero_points_mask(
+                    dnnl_attr, arg_name, mask));
         }
     }
 
@@ -961,33 +1003,39 @@ dnnl_primitive_attr_t create_dnnl_attr(
         for (int idx = 0; idx < po.len(); ++idx) {
             const auto &e = po.entry[idx];
             if (e.is_sum_kind()) {
-                DNN_SAFE_V(dnnl_post_ops_append_sum_v3(
+                DNN_SAFE_V(dnnl_post_ops_append_sum(
                         ops, e.sum.scale, e.sum.zero_point, e.sum.dt));
             } else if (e.is_convolution_kind()) {
                 const auto wei_dt = attr_args.get_dw_arg(DNNL_ARG_WEIGHTS);
                 const auto bia_dt = attr_args.get_dw_arg(DNNL_ARG_BIAS);
 
-                const auto &os_args = attr_args.get(
-                        DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_ATTR_OUTPUT_SCALES);
-                const auto scales = os_args.get_float_ptr();
-                const auto &policy = e.convolution.oscale.policy;
-                // API expects count=0 if output_scale was not set
-                const auto count = scales ? os_args.get_count(policy) : 0;
-                const auto mask = os_args.get_mask(policy);
-
                 DNN_SAFE_V(dnnl_post_ops_append_dw(ops, wei_dt, bia_dt,
                         e.convolution.dst_dt, e.convolution.kernel,
-                        e.convolution.stride, e.convolution.padding, count,
-                        mask, scales));
+                        e.convolution.stride, e.convolution.padding));
+
+                const auto &wei_scale = e.convolution.wei_scale;
+                int wei_mask = wei_scale.policy2mask(DNNL_ARG_WEIGHTS,
+                        dnnl_convolution, /* has_groups = */ true);
+                if (!wei_scale.is_def())
+                    DNN_SAFE_V(dnnl_primitive_attr_set_scales_mask(dnnl_attr,
+                            DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_WEIGHTS,
+                            wei_mask));
+
+                const auto &dst_scale = e.convolution.dst_scale;
+                int dst_mask = dst_scale.policy2mask(DNNL_ARG_DST);
+                if (!dst_scale.is_def())
+                    DNN_SAFE_V(dnnl_primitive_attr_set_scales_mask(dnnl_attr,
+                            DNNL_ARG_ATTR_POST_OP_DW | DNNL_ARG_DST, dst_mask));
+
             } else if (e.is_eltwise_kind()) {
-                DNN_SAFE_V(dnnl_post_ops_append_eltwise(ops, e.eltwise.scale,
-                        e.eltwise.alg, e.eltwise.alpha, e.eltwise.beta));
+                DNN_SAFE_V(dnnl_post_ops_append_eltwise(
+                        ops, e.eltwise.alg, e.eltwise.alpha, e.eltwise.beta));
             } else if (e.is_binary_kind()) {
                 const auto &src1_md = attr_args.get_md(
                         (DNNL_ARG_ATTR_MULTIPLE_POST_OP(idx) | DNNL_ARG_SRC_1));
-                assert(src1_md.ndims != 0);
+                assert(query_md_ndims(src1_md) != 0);
                 DNN_SAFE_V(dnnl_post_ops_append_binary(
-                        ops, e.binary.alg, &src1_md));
+                        ops, e.binary.alg, src1_md));
             } else if (e.is_prelu_kind()) {
                 const auto &policy = e.prelu.policy;
                 const auto mask = attr_t::get_default_mask(policy);
@@ -1289,11 +1337,17 @@ int check_tag(const std::string &tag_, bool check_enum_tags_only) {
     return check_abc_tag(tag, check_enum_tags_only);
 }
 
-void maybe_oscale(
-        const attr_t &attr, float &d, const float *scales, int64_t oc) {
-    if (!attr.oscale.is_def()) {
-        int64_t idx = attr.oscale.policy == policy_t::COMMON ? 0 : oc;
-        d *= scales[idx];
+void maybe_scale(const attr_t &attr, float &d, const float *scales, int64_t c,
+        int arg, bool opposite_scale) {
+    if (attr.scales.is_def(arg)) return;
+
+    const auto &e = attr.scales.get(arg);
+    if (!e.is_def()) {
+        int64_t idx = e.policy == policy_t::COMMON ? 0 : c;
+        if (opposite_scale)
+            d /= scales[idx];
+        else
+            d *= scales[idx];
     }
 }
 
@@ -1309,46 +1363,41 @@ void maybe_zero_point(const attr_t &attr, float &d, const int32_t *zero_points,
     }
 }
 
-float compute_eltwise_fwd(
-        pk_t kind, float src, float scale, float alpha, float beta) {
+float compute_eltwise_fwd(pk_t kind, float src, float alpha, float beta) {
     // don't compute on nan, propagate it
     if (std::isnan(src)) return NAN;
 
     using namespace dnnl::impl::math;
 
     switch (kind) {
-        case pk_t::RELU: return scale * relu_fwd(src, alpha);
-        case pk_t::TANH: return scale * tanh_fwd(src);
-        case pk_t::ELU: return scale * elu_fwd(src, alpha);
-        case pk_t::SQUARE: return scale * square_fwd(src);
-        case pk_t::ABS: return scale * abs_fwd(src);
-        case pk_t::SQRT: return scale * sqrt_fwd(src);
-        case pk_t::LINEAR: return scale * linear_fwd(src, alpha, beta);
-        case pk_t::BRELU: return scale * bounded_relu_fwd(src, alpha);
-        case pk_t::SRELU: return scale * soft_relu_fwd(src);
-        case pk_t::SRELU_V2: return scale * soft_relu_v2_fwd(src, alpha);
-        case pk_t::LOGSIGMOID: return scale * logsigmoid_fwd(src);
-        case pk_t::MISH: return scale * mish_fwd(src);
-        case pk_t::LOGISTIC: return scale * logistic_fwd(src);
-        case pk_t::EXP: return scale * exp_fwd(src);
-        case pk_t::GELU_TANH: return scale * gelu_tanh_fwd(src);
-        case pk_t::SWISH: return scale * swish_fwd(src, alpha);
-        case pk_t::LOG: return scale * log_fwd(src);
-        case pk_t::CLIP: return scale * clip_fwd(src, alpha, beta);
-        case pk_t::CLIP_V2: return scale * clip_v2_fwd(src, alpha, beta);
-        case pk_t::POW: return scale * pow_fwd(src, alpha, beta);
-        case pk_t::GELU_ERF: return scale * gelu_erf_fwd(src);
-        case pk_t::ROUND: return scale * round_fwd(src);
-        case pk_t::HARDSWISH: return scale * hardswish_fwd(src);
-        case pk_t::HARDSIGMOID:
-            return scale * hardsigmoid_fwd(src, alpha, beta);
-        case pk_t::RELU_DST: return scale * relu_fwd(src, alpha);
-        case pk_t::TANH_DST: return scale * tanh_fwd(src);
-        case pk_t::ELU_DST: return scale * elu_fwd(src, alpha);
-        case pk_t::SQRT_DST: return scale * sqrt_fwd(src);
-        case pk_t::LOGISTIC_DST: return scale * logistic_fwd(src);
-        case pk_t::EXP_DST: return scale * exp_fwd(src);
-        case pk_t::CLIP_V2_DST: return scale * clip_v2_fwd(src, alpha, beta);
+        case pk_t::RELU: return relu_fwd(src, alpha);
+        case pk_t::TANH: return tanh_fwd(src);
+        case pk_t::ELU: return elu_fwd(src, alpha);
+        case pk_t::SQUARE: return square_fwd(src);
+        case pk_t::ABS: return abs_fwd(src);
+        case pk_t::SQRT: return sqrt_fwd(src);
+        case pk_t::LINEAR: return linear_fwd(src, alpha, beta);
+        case pk_t::SRELU: return soft_relu_fwd(src, alpha);
+        case pk_t::MISH: return mish_fwd(src);
+        case pk_t::LOGISTIC: return logistic_fwd(src);
+        case pk_t::EXP: return exp_fwd(src);
+        case pk_t::GELU_TANH: return gelu_tanh_fwd(src);
+        case pk_t::SWISH: return swish_fwd(src, alpha);
+        case pk_t::LOG: return log_fwd(src);
+        case pk_t::CLIP: return clip_fwd(src, alpha, beta);
+        case pk_t::CLIP_V2: return clip_v2_fwd(src, alpha, beta);
+        case pk_t::POW: return pow_fwd(src, alpha, beta);
+        case pk_t::GELU_ERF: return gelu_erf_fwd(src);
+        case pk_t::ROUND: return round_fwd(src);
+        case pk_t::HARDSWISH: return hardswish_fwd(src, alpha, beta);
+        case pk_t::HARDSIGMOID: return hardsigmoid_fwd(src, alpha, beta);
+        case pk_t::RELU_DST: return relu_fwd(src, alpha);
+        case pk_t::TANH_DST: return tanh_fwd(src);
+        case pk_t::ELU_DST: return elu_fwd(src, alpha);
+        case pk_t::SQRT_DST: return sqrt_fwd(src);
+        case pk_t::LOGISTIC_DST: return logistic_fwd(src);
+        case pk_t::EXP_DST: return exp_fwd(src);
+        case pk_t::CLIP_V2_DST: return clip_v2_fwd(src, alpha, beta);
 
         default: assert(!"unknown attr::post_ops::kind");
     };
@@ -1367,10 +1416,7 @@ float compute_eltwise_bwd(
         case pk_t::ABS: return abs_bwd(d_dst, src);
         case pk_t::SQRT: return sqrt_bwd(d_dst, src);
         case pk_t::LINEAR: return linear_bwd(d_dst, src, alpha, beta);
-        case pk_t::BRELU: return bounded_relu_bwd(d_dst, src, alpha);
-        case pk_t::SRELU: return soft_relu_bwd(d_dst, src);
-        case pk_t::SRELU_V2: return soft_relu_v2_bwd(d_dst, src, alpha);
-        case pk_t::LOGSIGMOID: return logsigmoid_bwd(d_dst, src);
+        case pk_t::SRELU: return soft_relu_bwd(d_dst, src, alpha);
         case pk_t::MISH: return mish_bwd(d_dst, src);
         case pk_t::LOGISTIC: return logistic_bwd(d_dst, src);
         case pk_t::EXP: return exp_bwd(d_dst, src);
@@ -1381,7 +1427,7 @@ float compute_eltwise_bwd(
         case pk_t::CLIP_V2: return clip_v2_bwd(d_dst, src, alpha, beta);
         case pk_t::POW: return pow_bwd(d_dst, src, alpha, beta);
         case pk_t::GELU_ERF: return gelu_erf_bwd(d_dst, src);
-        case pk_t::HARDSWISH: return hardswish_bwd(d_dst, src);
+        case pk_t::HARDSWISH: return hardswish_bwd(d_dst, src, alpha, beta);
         case pk_t::HARDSIGMOID: return hardsigmoid_bwd(d_dst, src, alpha, beta);
 
         case pk_t::RELU_DST: return relu_bwd_use_dst(d_dst, src, alpha);
@@ -1446,10 +1492,9 @@ void maybe_post_ops(const attr_t &attr, float &val, float sum_val,
         } else if (e.is_convolution_kind()) {
             continue;
         } else if (e.is_eltwise_kind()) {
-            const auto &s = e.eltwise.scale;
             const auto &a = e.eltwise.alpha;
             const auto &b = e.eltwise.beta;
-            val = compute_eltwise_fwd(e.kind, val, s, a, b);
+            val = compute_eltwise_fwd(e.kind, val, a, b);
         } else if (e.is_binary_kind()) {
             val = compute_binary(e.kind, val, *it_po);
             it_po++;
@@ -1461,17 +1506,59 @@ void maybe_post_ops(const attr_t &attr, float &val, float sum_val,
 }
 
 void update_cpu_ref_attrs(attr_t &attr, dnnl_data_type_t new_dt) {
-    auto &os = attr.oscale;
-    os.runtime = false;
-
     auto &po = attr.post_ops;
     for (int idx = 0; idx < po.len(); ++idx) {
         auto &e = po.entry[idx];
         if (!e.is_binary_kind()) continue;
 
         e.binary.src1_dt = new_dt;
-        e.binary.tag = tag::abx; // Hardcoded in setup_binary_po as well.
+        e.binary.tag = tag::abx; // Hardcoded in local fill functions.
     }
 }
 
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+int sparse_options_t::from_str(const std::string &s) {
+    *this = sparse_options_t();
+    if (s.empty()) return OK;
+
+    const auto get_arg = [](int i) {
+        switch (i) {
+            case 0: return DNNL_ARG_SRC;
+            case 1: return DNNL_ARG_WEIGHTS;
+            case 2: return DNNL_ARG_DST;
+            default: return -1;
+        }
+    };
+
+    int options_count = 0;
+    size_t start_pos = 0;
+    while (start_pos != std::string::npos) {
+        auto subs = parser::get_substr(s, start_pos, ':');
+
+        if (subs.empty()) {
+            add(get_arg(options_count), sparse_options_t::def_encoding,
+                    sparse_options_t::def_sparsity);
+            options_count++;
+            continue;
+        }
+
+        if (subs.find("+") == std::string::npos) {
+            add(get_arg(options_count), str2sparse_encoding(subs.c_str()),
+                    sparse_options_t::def_sparsity);
+        } else {
+            size_t subs_pos = 0;
+            auto encoding_str = parser::get_substr(subs, subs_pos, '+');
+            auto sparsity_str = parser::get_substr(subs, subs_pos, '+');
+            if (encoding_str.empty() || sparsity_str.empty()) { return FAIL; }
+            add(get_arg(options_count),
+                    str2sparse_encoding(encoding_str.c_str()),
+                    atof(sparsity_str.c_str()));
+        }
+        options_count++;
+    }
+    static const int expected_num_options = 3;
+    return options_count == expected_num_options ? OK : FAIL;
+}
+#endif
+
 #undef BENCHDNN_DNNL_ARG_UNDEF
