@@ -30,6 +30,7 @@
 
 #include "common.hpp"
 #include "oneapi/dnnl/dnnl_types.h"
+#include "utils/wrapper.hpp"
 
 namespace tag {
 extern const char *x;
@@ -82,7 +83,6 @@ enum data_kind_t {
     SRC_1,
     MEAN,
     VAR,
-    SS,
     SC,
     SH,
     // rnn
@@ -121,52 +121,39 @@ struct attr_t {
     static const char *policy2str(policy_t policy);
     static int get_default_mask(policy_t policy);
 
-    struct scale_t {
-        scale_t(policy_t apolicy = COMMON, float ascale = 1.,
-                bool aruntime = false)
-            : policy(apolicy), scale(ascale), runtime(aruntime) {}
-
-        int from_str(const std::string &s);
-
-        bool is_def() const {
-            return policy == COMMON && scale == 1. && runtime == false;
-        }
-
-        policy_t policy = COMMON;
-        float scale = 1.;
-        bool runtime = false;
-    };
-
     struct zero_points_t {
         struct entry_t {
-            entry_t(policy_t apolicy = COMMON, int avalue = 0,
-                    bool aruntime = false)
-                : policy(apolicy), value(avalue), runtime(aruntime) {}
+            entry_t(policy_t apolicy = COMMON, int avalue = 0)
+                : policy(apolicy), value(avalue) {}
 
             entry_t(const entry_t &other)
-                : policy(other.policy)
-                , value(other.value)
-                , runtime(other.runtime) {}
+                : policy(other.policy), value(other.value) {}
 
-            bool is_def() const {
-                return policy == COMMON && value == 0 && runtime == false;
-            }
+            bool is_def() const { return policy == COMMON && value == 0; }
 
             policy_t policy = COMMON;
             int value = 0;
-            bool runtime = false;
         };
 
         int from_str(const std::string &s);
 
         int operator[](int arg) const { return get(arg).value; }
-        bool runtime(int arg) const { return get(arg).runtime; }
 
-        bool is_def(int arg) const { return get(arg).is_def(); }
-        bool is_def() const { return points.empty(); }
+        bool is_def(int arg) const {
+            return points.empty() || get(arg).is_def();
+        }
+        bool is_def() const {
+            if (points.empty()) return true;
 
-        void set(int arg, policy_t policy, int value, bool runtime) {
-            set(arg, entry_t(policy, value, runtime));
+            bool def = true;
+            for (const auto &e : points) {
+                def = def && is_def(e.first);
+            }
+            return def;
+        }
+
+        void set(int arg, policy_t policy, int value) {
+            set(arg, entry_t(policy, value));
         }
         void set(int arg, const entry_t &entry) {
             if (!entry.is_def()) points[arg] = entry;
@@ -188,20 +175,53 @@ struct attr_t {
     };
 
     struct arg_scales_t {
-        void set(int arg, scale_t scale) { scales[arg] = scale; }
+        struct entry_t {
+            entry_t(policy_t apolicy = COMMON, float ascale = 1.f)
+                : policy(apolicy), scale(ascale) {}
 
-        scale_t get(int arg) const {
+            int from_str(const std::string &s);
+
+            bool is_def() const { return policy == COMMON && scale == 1.f; }
+
+            int policy2mask(int arg,
+                    dnnl_primitive_kind_t prim_kind = dnnl_undefined_primitive,
+                    bool has_groups = false) const;
+
+            policy_t policy = COMMON;
+            float scale = 1.f;
+        };
+
+        void set(int arg, entry_t scale) { scales[arg] = scale; }
+
+        entry_t get(int arg) const {
             const auto &s = scales.find(arg);
-            return s == scales.end() ? scale_t() : s->second;
+            return s == scales.end() ? entry_t() : s->second;
         }
 
-        bool is_def(int arg) const { return get(arg).is_def(); }
-        bool is_def() const { return scales.empty(); }
+        int get_mask(int arg,
+                dnnl_primitive_kind_t prim_kind = dnnl_undefined_primitive,
+                bool has_groups = false) const {
+            const auto &e = get(arg);
+            return e.policy2mask(arg, prim_kind, has_groups);
+        }
+
+        bool is_def(int arg) const {
+            return scales.empty() || get(arg).is_def();
+        }
+        bool is_def() const {
+            if (scales.empty()) return true;
+
+            bool def = true;
+            for (const auto &e : scales) {
+                def = def && is_def(e.first);
+            }
+            return def;
+        }
         int from_str(const std::string &s);
 
         arg_scales_t() : scales() {} // needed for debug icc190 build;
 
-        std::map<int, scale_t> scales;
+        std::map<int, entry_t> scales;
     };
 
     struct post_ops_t {
@@ -215,7 +235,6 @@ struct attr_t {
             // eltwise
             ELTWISE_START, // a guard to check kind is eltwise
             ABS,
-            BRELU,
             CLIP,
             CLIP_V2,
             CLIP_V2_DST,
@@ -231,7 +250,6 @@ struct attr_t {
             LOG,
             LOGISTIC,
             LOGISTIC_DST,
-            LOGSIGMOID,
             MISH,
             POW,
             RELU,
@@ -241,7 +259,6 @@ struct attr_t {
             SQRT_DST,
             SQUARE,
             SRELU,
-            SRELU_V2,
             SWISH,
             TANH,
             TANH_DST,
@@ -276,7 +293,9 @@ struct attr_t {
                 } else if (is_eltwise_kind()) {
                     eltwise.alg = kind2dnnl_kind(kind);
                 } else if (is_convolution_kind()) {
-                    convolution.oscale = scale_t();
+                    convolution.src_scale = arg_scales_t::entry_t();
+                    convolution.wei_scale = arg_scales_t::entry_t();
+                    convolution.dst_scale = arg_scales_t::entry_t();
                     if (kind != DW) {
                         convolution.kernel = 3;
                         convolution.stride = kind == DW_K3S1P1 ? 1 : 2;
@@ -297,14 +316,15 @@ struct attr_t {
                 dnnl_alg_kind_t alg = dnnl_alg_kind_undef;
                 float alpha = 0.f;
                 float beta = 0.f;
-                float scale = 1.f;
             } eltwise;
             struct {
                 int kernel = 0;
                 int stride = 0;
                 int padding = 0;
                 dnnl_data_type_t dst_dt = dnnl_f32;
-                scale_t oscale;
+                arg_scales_t::entry_t src_scale;
+                arg_scales_t::entry_t wei_scale;
+                arg_scales_t::entry_t dst_scale;
             } convolution;
             struct {
                 dnnl_alg_kind_t alg = dnnl_alg_kind_undef;
@@ -342,7 +362,7 @@ struct attr_t {
     };
 
     attr_t()
-        : scratchpad_mode(dnnl_scratchpad_mode_library)
+        : scratchpad_mode(get_default_scratchpad_mode())
         , fpmath_mode(dnnl_fpmath_mode_strict) {}
 
     template <typename First, typename... Rest>
@@ -351,14 +371,16 @@ struct attr_t {
         if (sizeof...(rest) > 0) this->insert(rest...);
     }
 
-    void insert(const scale_t &s) { this->oscale = s; }
     void insert(const arg_scales_t &as) { this->scales = as; }
     void insert(const zero_points_t &zp) { this->zero_points = zp; }
     void insert(const post_ops_t &po) { this->post_ops = po; }
     void insert(dnnl_scratchpad_mode_t sm) { this->scratchpad_mode = sm; }
     void insert(dnnl_fpmath_mode_t fpm) { this->fpmath_mode = fpm; }
 
-    scale_t oscale;
+    static dnnl_scratchpad_mode_t get_default_scratchpad_mode() {
+        return dnnl_scratchpad_mode_library;
+    }
+
     arg_scales_t scales;
     zero_points_t zero_points;
     post_ops_t post_ops;
@@ -409,8 +431,67 @@ struct isa_hints_t {
 
 using policy_t = attr_t::policy_t;
 
+#ifdef DNNL_EXPERIMENTAL_SPARSE
+struct sparse_options_t {
+    static constexpr dnnl_sparse_encoding_t def_encoding
+            = dnnl_sparse_encoding_undef;
+    static constexpr float def_sparsity = 0.9;
+
+    sparse_options_t() = default;
+    sparse_options_t(int arg, dnnl_sparse_encoding_t encoding, float sparsity) {
+        add(arg, encoding, sparsity);
+    }
+
+    void add(int arg, dnnl_sparse_encoding_t encoding, float sparsity) {
+        options_.insert({arg, {encoding, sparsity}});
+    }
+
+    dnnl_sparse_encoding_t get_encoding(int arg) const {
+        if (options_.count(arg) == 0) return dnnl_sparse_encoding_undef;
+        return options_.at(arg).first;
+    }
+
+    float get_sparsity(int arg) const {
+        if (options_.count(arg) == 0) return 0.0f;
+        return options_.at(arg).second;
+    }
+
+    bool is_encoding_def(int arg) const {
+        return get_encoding(arg) == def_encoding;
+    }
+
+    bool is_sparsity_def(int arg) const {
+        return get_sparsity(arg) == def_sparsity;
+    }
+
+    bool is_def() const {
+        bool ret = true;
+        for (const auto &opt : options_) {
+            ret = ret && is_encoding_def(opt.first)
+                    && is_sparsity_def(opt.first);
+        }
+        return ret;
+    }
+
+    std::vector<int> get_args() const {
+        std::vector<int> args;
+        for (const auto &opt : options_) {
+            args.push_back(opt.first);
+        }
+        return args;
+    }
+
+    int from_str(const std::string &s);
+
+private:
+    std::unordered_map<int, std::pair<dnnl_sparse_encoding_t, float>> options_;
+};
+
+std::ostream &operator<<(
+        std::ostream &s, const sparse_options_t &sparse_options);
+#endif
+
 std::ostream &operator<<(std::ostream &s, const policy_t &policy);
-std::ostream &operator<<(std::ostream &s, const attr_t::scale_t &scale);
 std::ostream &operator<<(
         std::ostream &s, const attr_t::zero_points_t &zero_points);
 std::ostream &operator<<(std::ostream &s, const attr_t::arg_scales_t &scales);
@@ -423,30 +504,6 @@ std::ostream &operator<<(std::ostream &s, const attr_t &attr);
 // A container for additional data and info, not available from user's input at
 // parse time, but which are required to create the library attributes.
 struct attr_args_t {
-    struct entry_t {
-        entry_t(const void *vals = NULL, int64_t count = 1, int mask = -1,
-                bool runtime = false)
-            : vals(vals), count(count), mask(mask), runtime(runtime) {}
-
-        int64_t get_count(policy_t policy) const {
-            return (policy == policy_t::COMMON || runtime) ? 1 : count;
-        }
-
-        int get_mask(policy_t policy) const {
-            return mask == -1 ? attr_t::get_default_mask(policy) : mask;
-        }
-
-        const float *get_float_ptr() const {
-            return runtime ? &DNNL_RUNTIME_F32_VAL
-                           : static_cast<const float *>(vals);
-        }
-
-        const void *vals = NULL;
-        int64_t count = 1;
-        int mask = -1;
-        bool runtime = false;
-    };
-
     struct dw_t {
         dnnl_data_type_t wei_dt = dnnl_data_type_undef;
         dnnl_data_type_t bia_dt = dnnl_data_type_undef;
@@ -454,24 +511,25 @@ struct attr_args_t {
 
     attr_args_t() = default;
 
-    void prepare_output_scales(
-            const attr_t &attr, const void *vals, int64_t count, int mask = -1);
+    void prepare_scales(const attr_t &attr, int arg, int mask = -1) {
+        entries.insert(std::make_pair(arg, mask));
+    };
 
     int prepare_post_ops_mds(
-            const attr_t &attr, int ndims, const dnnl_dims_t dims);
+            const attr_t &attr, int ndims, const dnnl_dims_t prb_dims);
 
     void prepare_dw_post_op(const attr_t &attr, dnnl_data_type_t wei_dt,
-            dnnl_data_type_t bia_dt, const void *vals, int64_t count,
-            int mask = -1);
+            dnnl_data_type_t bia_dt);
 
-    entry_t get(int arg) const {
+    // Returns mask set for correspondent `arg`. The default value is `-1`.
+    int get_mask(int arg) const {
         const auto it = entries.find(arg);
-        return it == entries.end() ? entry_t() : it->second;
+        return it == entries.end() ? -1 : it->second;
     }
 
     dnnl_memory_desc_t get_md(int arg) const {
         const auto it = mds.find(arg);
-        return it == mds.end() ? dnnl_memory_desc_t() : it->second;
+        return it == mds.end() ? nullptr : (dnnl_memory_desc_t)it->second;
     }
 
     dnnl_data_type_t get_dw_arg(int arg) const {
@@ -486,14 +544,8 @@ struct attr_args_t {
     }
 
 private:
-    void insert(
-            int arg, const void *vals, int64_t count, int mask, bool runtime) {
-        entries.insert(
-                std::make_pair(arg, entry_t(vals, count, mask, runtime)));
-    }
-
-    std::map<int, entry_t> entries;
-    std::map<int, dnnl_memory_desc_t> mds;
+    std::map<int, int /* mask*/> entries;
+    std::map<int, benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t>> mds;
     dw_t dw_entry; // only single dw fusion is supported
 };
 
@@ -522,12 +574,12 @@ dnnl_engine_kind_t str2engine_kind(const char *str);
 dnnl_scratchpad_mode_t str2scratchpad_mode(const char *str);
 dnnl_fpmath_mode_t str2fpmath_mode(const char *str);
 
-void maybe_oscale(
-        const attr_t &attr, float &d, const float *scales, int64_t oc);
+void maybe_scale(const attr_t &attr, float &d, const float *scales, int64_t c,
+        int arg, bool opposite_scale = false);
 void maybe_zero_point(const attr_t &attr, float &d, const int32_t *zero_points,
         int64_t c, int arg, bool opposite_zero_point = false);
-float compute_eltwise_fwd(attr_t::post_ops_t::kind_t kind, float src,
-        float scale, float alpha, float beta);
+float compute_eltwise_fwd(
+        attr_t::post_ops_t::kind_t kind, float src, float alpha, float beta);
 float compute_eltwise_bwd(attr_t::post_ops_t::kind_t kind, float d_dst,
         float src, float alpha, float beta);
 float compute_binary(attr_t::post_ops_t::kind_t kind, float src0, float src1);
