@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -21,71 +21,57 @@
 
 #include "oneapi/dnnl/dnnl.h"
 
-#include "tests/test_thread.hpp"
+#include "utils/parallel.hpp"
 
 #include "dnnl_common.hpp"
 #include "dnnl_memory.hpp"
-#include "utils/compare.hpp"
 
 #include "binary/binary.hpp"
 #include "eltwise/eltwise.hpp"
 
 namespace eltwise {
 
-static int init_pd(dnnl_engine_t engine, const prb_t *prb,
-        dnnl_primitive_desc_t &epd, res_t *res, dir_t dir,
-        const_dnnl_primitive_desc_t hint) {
-    dnnl_eltwise_desc_t ed;
-    dnnl_memory_desc_t data_d;
+dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
+    const prb_t *prb = init_pd_args.prb;
+    const dir_t dir = init_pd_args.dir;
 
-    SAFE(init_md(&data_d, prb->ndims, prb->dims.data(), prb->dt, prb->tag),
-            CRIT);
+    auto src_d = dnn_mem_t::init_md(
+            prb->ndims, prb->dims.data(), prb->dt, prb->tag);
+    auto dst_d = dnn_mem_t::init_md(
+            prb->ndims, prb->dims.data(), prb->dt, tag::any);
 
     dnnl_alg_kind_t alg = attr_t::post_ops_t::kind2dnnl_kind(prb->alg);
 
-    if (prb->dir & FLAG_FWD) {
-        auto prop = prb->dir & FLAG_INF ? dnnl_forward_inference
-                                        : dnnl_forward_training;
-
-        DNN_SAFE(dnnl_eltwise_forward_desc_init(
-                         &ed, prop, alg, &data_d, prb->alpha, prb->beta),
-                WARN);
-    } else {
-        dnnl_memory_desc_t diff_data_d;
-        DNN_SAFE(dnnl_memory_desc_init_by_tag(&diff_data_d, prb->ndims,
-                         prb->dims.data(), prb->dt, dnnl_format_tag_any),
-                WARN);
-        DNN_SAFE(dnnl_eltwise_backward_desc_init(&ed, alg, &diff_data_d,
-                         &data_d, prb->alpha, prb->beta),
-                WARN);
-    }
-
     attr_args_t attr_args;
     attr_args.prepare_post_ops_mds(prb->attr, prb->ndims, prb->dims.data());
     auto dnnl_attr = make_benchdnn_dnnl_wrapper(
             create_dnnl_attr(prb->attr, attr_args));
 
-    dnnl_status_t init_status
-            = dnnl_primitive_desc_create(&epd, &ed, dnnl_attr, engine, nullptr);
+    if (dir & FLAG_FWD) {
+        auto prop = prb->dir & FLAG_INF ? dnnl_forward_inference
+                                        : dnnl_forward_training;
 
-    if (init_status == dnnl_unimplemented)
-        return res->state = UNIMPLEMENTED, OK;
-    else
-        SAFE(init_status, WARN);
-
-    res->impl_name = query_impl_info(epd);
-    if (maybe_skip(res->impl_name)) {
-        BENCHDNN_PRINT(2, "SKIPPED: oneDNN implementation: %s\n",
-                res->impl_name.c_str());
-        return res->state = SKIPPED, res->reason = SKIP_IMPL_HIT, OK;
+        DNN_SAFE_STATUS(dnnl_eltwise_forward_primitive_desc_create(
+                &init_pd_args.pd, init_pd_args.engine, prop, alg,
+                init_pd_args.src_md ? init_pd_args.src_md : src_d, dst_d,
+                prb->alpha, prb->beta, dnnl_attr));
     } else {
-        BENCHDNN_PRINT(
-                5, "oneDNN implementation: %s\n", res->impl_name.c_str());
+        auto diff_src_d = dnn_mem_t::init_md(
+                prb->ndims, prb->dims.data(), prb->dt, tag::any);
+        auto diff_dst_d = dnn_mem_t::init_md(
+                prb->ndims, prb->dims.data(), prb->dt, tag::any);
+        if (prb->use_dst()) // Need to create with proper tag
+            dst_d = dnn_mem_t::init_md(
+                    prb->ndims, prb->dims.data(), prb->dt, prb->tag);
+        auto &data_d = prb->use_dst() ? dst_d : src_d;
+
+        DNN_SAFE_STATUS(dnnl_eltwise_backward_primitive_desc_create(
+                &init_pd_args.pd, init_pd_args.engine, alg, diff_src_d,
+                diff_dst_d, data_d, prb->alpha, prb->beta, init_pd_args.hint,
+                dnnl_attr));
     }
 
-    SAFE(check_pd_w_and_wo_attr(res, prb->attr, ed), WARN);
-
-    return OK;
+    return dnnl_success;
 }
 
 static bool check_abs_err(const prb_t *prb, const float &s, const float &trh) {
@@ -135,19 +121,13 @@ static bool check_abs_err(const prb_t *prb, const float &s, const float &trh) {
             // is high and tanh(s) is close to 1.
             return (prb->dir & FLAG_BWD) && (1.f - s * s) <= comp_err;
         case alg_t::SRELU:
-            // when s is negative, expf(s) -> 0 rapidly
-            // which leads to log1pf(expf(s)) -> 0
+            // when `alpha * s` is negative, expf(alpha * s) -> 0 rapidly
+            // which leads to log1pf(expf(alpha * s)) -> 0
             // which leads to high relative error,
             // while abs error is still low.
             // (10.f is magic scale for bf16)
-            return (prb->dir & FLAG_FWD) && std::signbit(s)
-                    && log1pf(expf(s)) <= 10.f * comp_err;
-        case alg_t::LOGSIGMOID:
-            // same situation like in SRELU
-            // in logsigmoid when s is positive
-            // results -> 0
-            return (prb->dir & FLAG_FWD) && !std::signbit(s)
-                    && log1pf(expf(-s)) <= 10.f * comp_err;
+            return (prb->dir & FLAG_FWD) && std::signbit(prb->alpha * s)
+                    && log1pf(expf(prb->alpha * s)) <= 10.f * comp_err;
         case alg_t::MISH:
             // same situation like in SRELU
             return (prb->dir & FLAG_FWD) && std::signbit(s)
@@ -158,6 +138,11 @@ static bool check_abs_err(const prb_t *prb, const float &s, const float &trh) {
             // catastrohic cancellation.
             return (prb->dir & FLAG_BWD) && !std::signbit(s)
                     && (1.f / (1.f + expf(s))) <= comp_err;
+        case alg_t::LOGISTIC_DST:
+            // when s = logistic(x) ~~ 1, it leads to high relative error of
+            // s * (1 - s) due to catastrohic cancellation.
+            return (prb->dir & FLAG_BWD)
+                    && ((1 - s) <= comp_err || s <= comp_err);
         case alg_t::SWISH: {
             // catch cancellation happening when W(s) ~~ -1 in (1 + W(s))
             // formula part on backward.
@@ -176,22 +161,18 @@ float get_eltwise_threshold(dnnl_data_type_t dt, alg_t alg, bool is_fwd) {
     // Tolerate bigger compute errors for complex algorithms.
     const bool alg_has_higher_tolerance = alg == alg_t::GELU_TANH
             || alg == alg_t::ELU || alg == alg_t::SWISH || alg == alg_t::TANH
-            || alg == alg_t::SRELU || alg == alg_t::LOGSIGMOID
-            || alg == alg_t::MISH || alg == alg_t::LOG
+            || alg == alg_t::SRELU || alg == alg_t::MISH || alg == alg_t::LOG
             || ((alg == alg_t::ELU_DST || alg == alg_t::TANH_DST) && is_fwd);
     if (dt == dnnl_f32 && alg_has_higher_tolerance) trh = 4e-5;
     return trh;
 }
 
 static float get_eltwise_zero_trust_percent(const prb_t *prb) {
-    float ztp = 60.f; // default for eltwise due to filling.
+    float ztp = 65.f; // default for eltwise due to filling.
     switch (prb->alg) {
         case alg_t::LINEAR:
             if (prb->alpha == 0) ztp = 100.f;
             break;
-        case alg_t::BRELU:
-            if ((prb->alpha == 0) || (prb->dir & FLAG_BWD)) ztp = 100.f;
-            break;
         case alg_t::CLIP:
         case alg_t::CLIP_V2:
         case alg_t::CLIP_V2_DST:
@@ -220,7 +201,7 @@ int fill_data(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
     const int64_t chunk_size = div_up(nelems, n_chunks);
     const bool is_log = prb->alg == alg_t::LOG;
 
-    dnnl::impl::parallel_nd(n_chunks, [&](int64_t idx_chunk) {
+    benchdnn_parallel_nd(n_chunks, [&](int64_t idx_chunk) {
         int64_t idx_start = idx_chunk * chunk_size;
         int64_t idx_end = MIN2(idx_start + chunk_size, nelems);
         // Note 1: we use a different seed for each chunk to avoid
@@ -283,16 +264,17 @@ int fill_data(const prb_t *prb, data_kind_t kind, dnn_mem_t &mem_dt,
     return OK;
 }
 
-void check_known_skipped_case(const prb_t *prb, res_t *res) {
-    check_known_skipped_case_common({prb->dt}, prb->dir, res);
-    if (res->state == SKIPPED) return;
+void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
+    skip_unimplemented_data_type({prb->dt}, prb->dir, res);
+    skip_unimplemented_sum_po(prb->attr, res);
+}
 
+void skip_invalid_prb(const prb_t *prb, res_t *res) {
     bool is_invalid = false;
     switch (prb->alg) {
         case alg_t::CLIP:
         case alg_t::CLIP_V2:
         case alg_t::CLIP_V2_DST: is_invalid = prb->beta < prb->alpha; break;
-        case alg_t::BRELU:
         case alg_t::ELU_DST:
         case alg_t::RELU_DST: is_invalid = prb->alpha < 0; break;
         case alg_t::ROUND:
@@ -305,143 +287,185 @@ void check_known_skipped_case(const prb_t *prb, res_t *res) {
         return;
     }
 
-    if (is_nvidia_gpu()) {
-        if (!is_nvidia_eltwise_ok(prb->dir, prb->alg, prb->alpha)
-                || !prb->attr.post_ops.is_def()) {
-            res->state = SKIPPED, res->reason = CASE_NOT_SUPPORTED;
-            return;
-        }
+    // Since source is needed for non-use-dst algorithms, it is incorrect to
+    // let forward path overwrite it.
+    is_invalid = (prb->dir & FLAG_BWD) && !prb->use_dst() && prb->inplace;
+    if (is_invalid) {
+        res->state = SKIPPED, res->reason = INVALID_CASE;
+        return;
     }
+
+    // See `skip_invalid_inplace` for details.
+    if (prb->inplace) {
+        skip_invalid_inplace(res, prb->dt, prb->dt, prb->tag, prb->tag);
+        if (res->state == SKIPPED) return;
+    }
+}
+
+bool eltwise_alg_returns_nan_or_inf(alg_t alg) {
+    static const std::vector<alg_t> nan_inf_alg
+            = {alg_t::EXP, alg_t::EXP_DST, alg_t::LOG, alg_t::POW, alg_t::SQRT,
+                    alg_t::SQRT_DST, alg_t::SQUARE};
+    return std::any_of(nan_inf_alg.cbegin(), nan_inf_alg.cend(),
+            [alg](const alg_t _alg) { return (_alg == alg); });
+}
+
+bool eltwise_alg_returns_nan_or_inf(const attr_t &attr) {
+    const auto &po = attr.post_ops;
+    for (int i = 0; i < po.len(); i++) {
+        if (eltwise_alg_returns_nan_or_inf(po.entry[i].kind)) return true;
+    }
+    return false;
+}
+
+void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
+        const args_t &ref_args) {
+    const float trh
+            = get_eltwise_threshold(prb->dt, prb->alg, prb->dir & FLAG_FWD);
+    cmp.set_threshold(trh);
+
+    cmp.set_zero_trust_percent(get_eltwise_zero_trust_percent(prb));
+    cmp.set_op_output_has_nans(eltwise_alg_returns_nan_or_inf(prb->alg));
+
+    // Since lambda is called when stack is unavailable, need to capture `prb`
+    // by value to avoid using dangling references.
+    const auto eltwise_add_check =
+            [&, prb](const compare::compare_t::driver_check_func_args_t &args) {
+                // Some algorithms require absolute value comparison for inputs
+                // where catastrophic cancellation may happen.
+                const auto &src = ref_args.find(DNNL_ARG_SRC);
+                const auto &dst = ref_args.find(DNNL_ARG_DST);
+                const auto &source
+                        = ((prb->dir & FLAG_BWD) && prb->use_dst()) ? dst : src;
+                const float s = source.get_elem(args.idx);
+                if (check_abs_err(prb, s, args.trh))
+                    return args.diff <= args.trh;
+                if (prb->attr.post_ops.binary_index() != -1)
+                    return args.diff <= args.trh;
+                return false;
+            };
+    cmp.set_driver_check_function(eltwise_add_check);
+}
+
+std::vector<int> supported_exec_args(dir_t dir) {
+    static const std::vector<int> exec_fwd_args = {
+            DNNL_ARG_SRC,
+            DNNL_ARG_DST,
+    };
+    static const std::vector<int> exec_bwd_args = {
+            DNNL_ARG_SRC,
+            DNNL_ARG_DST,
+            DNNL_ARG_DIFF_DST,
+            DNNL_ARG_DIFF_SRC,
+    };
+    return (dir & FLAG_FWD) ? exec_fwd_args : exec_bwd_args;
+};
+
+int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res, dir_t dir,
+        dnnl_primitive_t prim_ref) {
+    const auto &ref_engine = get_cpu_engine();
+
+    for (auto &entry : mem_map) {
+        const int exec_arg = entry.first;
+        auto &mem = entry.second; // `mem` is modified by filler (reorder).
+
+        ref_mem_map.emplace(
+                exec_arg, dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+        auto &ref_mem = ref_mem_map[exec_arg];
+
+        switch (exec_arg) {
+            case DNNL_ARG_SRC:
+                SAFE(fill_data(prb, SRC, mem, ref_mem), WARN);
+                break;
+            case DNNL_ARG_DIFF_DST:
+                SAFE(fill_data(prb, DST, mem, ref_mem), WARN);
+                break;
+            case DNNL_ARG_SCRATCHPAD: break;
+            default: { // Process all attributes here
+                int post_ops_range = DNNL_ARG_ATTR_MULTIPLE_POST_OP(31)
+                        - DNNL_ARG_ATTR_MULTIPLE_POST_OP(0);
+                bool is_post_ops_arg = (exec_arg & post_ops_range);
+                if (is_post_ops_arg) {
+                    SAFE(binary::fill_mem(exec_arg, mem, ref_mem), WARN);
+                }
+            } break;
+        }
+        // Don't keep reference memory if it is not used further.
+        if (!is_bench_mode(CORR)) ref_mem_map.clear();
+    }
+
+    // Drop destination memory for in-place case. `args` will take care of rest.
+    const bool inplace_fwd = prb->inplace && (prb->dir & FLAG_FWD);
+    const bool inplace_bwd = prb->inplace && (dir & FLAG_BWD);
+    if (inplace_fwd) {
+        mem_map[DNNL_ARG_DST] = dnn_mem_t();
+    } else if (inplace_bwd) {
+        mem_map[DNNL_ARG_DIFF_SRC] = dnn_mem_t();
+    }
+
+    if (!is_bench_mode(CORR)) return OK;
+
+    // Use inplace reference computation every time.
+    if (dir & FLAG_FWD) {
+        ref_mem_map.emplace(DNNL_ARG_DST, dnn_mem_t());
+    } else {
+        ref_mem_map.emplace(DNNL_ARG_DIFF_SRC, dnn_mem_t());
+    }
+
+    return OK;
 }
 
 int doit(const prb_t *prb, res_t *res) {
     if (bench_mode == LIST) return res->state = LISTED, OK;
 
-    check_known_skipped_case(prb, res);
-    check_sum_post_ops(prb->attr, res);
-    if (res->state == SKIPPED) return OK;
-
     benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim;
-    SAFE(init_prim(prim, init_pd, prb, res), WARN);
+    SAFE(init_prim(prb->ctx_init, prim, init_pd, prb, res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
+    if (is_bench_mode(INIT)) return OK;
 
-    const_dnnl_primitive_desc_t const_pd;
-    DNN_SAFE(dnnl_primitive_get_primitive_desc(prim, &const_pd), CRIT);
-
-    if (check_mem_size(const_pd) != OK) {
-        return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
-    }
-
-    const auto q = [&](int index = 0) -> const dnnl_memory_desc_t & {
-        return *dnnl_primitive_desc_query_md(
-                const_pd, dnnl_query_exec_arg_md, index);
-    };
-
-    const bool is_fwd = prb->dir & FLAG_FWD;
-    const auto &src_md = q(DNNL_ARG_SRC);
-    const auto &dst_md = q(DNNL_ARG_DST);
-    const auto &data_md = !is_fwd && prb->use_dst() ? dst_md : src_md;
-    const auto &scratchpad_md = q(DNNL_ARG_SCRATCHPAD);
-    const auto &test_engine = get_test_engine();
-    const auto &ref_engine = get_cpu_engine();
-
-    dnn_mem_t src_fp(data_md, dnnl_f32, tag::abx, ref_engine);
-    dnn_mem_t src_dt(data_md, test_engine);
-
-    // we need src_fp for proper comparison, => no in-place reference
-    dnn_mem_t dst_fp(data_md, dnnl_f32, tag::abx, ref_engine);
-    dnn_mem_t placeholder_dst_dt;
-    if (!prb->inplace) { placeholder_dst_dt = dnn_mem_t(data_md, test_engine); }
-    dnn_mem_t &dst_dt = prb->inplace ? src_dt : placeholder_dst_dt;
-
-    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
-    std::vector<dnn_mem_t> binary_po_fp, binary_po_dt;
-    std::vector<int> binary_po_args;
-    SAFE(binary::setup_binary_po(
-                 const_pd, binary_po_args, binary_po_dt, binary_po_fp),
+    dnn_mem_map_t mem_map, ref_mem_map;
+    init_memory_args<prb_t>(mem_map, prb, prim, supported_exec_args(FLAG_FWD));
+    SAFE(init_ref_memory_args(ref_mem_map, mem_map, prim, prb, res, FLAG_FWD),
             WARN);
 
-    dnn_mem_t d_dst_dt, placeholder_d_src_dt;
+    args_t args(mem_map), ref_args(ref_mem_map);
 
-    SAFE(fill_data(prb, SRC, src_dt, src_fp), WARN);
-
-    args_t args;
-
-    dnn_mem_t &arg_fp = !is_fwd && prb->use_dst() ? dst_fp : src_fp;
-
-    // Shouldn't be defined inside since not available when `eltwise_add_check`
-    // is invoked due to removed from stack.
-    const float trh = get_eltwise_threshold(prb->dt, prb->alg, is_fwd);
-    compare::compare_t cmp;
-    if (is_bench_mode(CORR)) {
-        cmp.set_threshold(trh);
-        cmp.set_zero_trust_percent(get_eltwise_zero_trust_percent(prb));
-
-        const auto eltwise_add_check =
-                [&](const compare::compare_t::driver_check_func_args_t &args) {
-                    // Some algorithms require absolute value comparison for inputs
-                    // where catastrophic cancellation may happen.
-                    const float src = arg_fp.get_elem(args.idx);
-                    if (check_abs_err(prb, src, trh)) return args.diff <= trh;
-                    if (prb->attr.post_ops.binary_index() != -1)
-                        return args.diff <= trh;
-                    return false;
-                };
-        cmp.set_driver_check_function(eltwise_add_check);
-    }
+    if (!is_bench_mode(INIT)) SAFE(execute_and_wait(prim, args, res), WARN);
 
     if (prb->dir & FLAG_FWD) {
-        args.set(DNNL_ARG_SRC, src_dt);
-        args.set(DNNL_ARG_DST, dst_dt);
-        args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
-        args.set(binary_po_args, binary_po_dt);
-
-        SAFE(execute_and_wait(prim, args), WARN);
-
         if (is_bench_mode(CORR)) {
-            TIME_REF(compute_ref_fwd(prb, src_fp, binary_po_fp, dst_fp));
-            SAFE(cmp.compare(dst_fp, dst_dt, prb->attr, res), WARN);
-        }
-    } else {
-        const auto &d_data_md = q(DNNL_ARG_DIFF_DST);
-
-        dnn_mem_t d_dst_fp
-                = dnn_mem_t(d_data_md, dnnl_f32, tag::abx, ref_engine);
-        d_dst_dt = dnn_mem_t(d_data_md, test_engine);
-
-        dnn_mem_t &d_src_fp = d_dst_fp; // in-place reference
-        if (!prb->inplace) {
-            placeholder_d_src_dt = dnn_mem_t(d_data_md, test_engine);
-        }
-        dnn_mem_t &d_src_dt = prb->inplace ? d_dst_dt : placeholder_d_src_dt;
-
-        SAFE(fill_data(prb, DST, d_dst_dt, d_dst_fp), WARN);
-
-        args.set(DNNL_ARG_DIFF_DST, d_dst_dt);
-        args.set(DNNL_ARG_DIFF_SRC, d_src_dt);
-        args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
-
-        if (prb->use_dst()) {
-            if (is_bench_mode(CORR))
-                TIME_REF(compute_ref_fwd(prb, src_fp, binary_po_fp, dst_fp));
-            SAFE(dst_dt.reorder(dst_fp), WARN);
-            // make dst_fp of same values as for bf16, otherwise there are high
-            // relative and absolute errors due to initial difference in source
-            // values which become worse particularly when (1 - x) is used.
-            if (dst_dt.dt() != dst_fp.dt()) SAFE(dst_fp.reorder(dst_dt), WARN);
-            args.set(DNNL_ARG_DST, dst_dt);
-        } else {
-            args.set(DNNL_ARG_SRC, src_dt);
-        }
-        SAFE(execute_and_wait(prim, args), WARN);
-
-        if (is_bench_mode(CORR)) {
-            TIME_REF(compute_ref_bwd(prb, arg_fp, d_dst_fp, d_src_fp));
-            SAFE(cmp.compare(d_src_fp, d_src_dt, prb->attr, res), WARN);
+            check_correctness(prb, {DST}, args, ref_args, setup_cmp, res);
         }
     }
 
-    return measure_perf(res, prim, args);
+    if (prb->dir & FLAG_BWD) {
+        benchdnn_dnnl_wrapper_t<dnnl_primitive_t> tmp_prim;
+        SAFE(init_prim(prb->ctx_init, tmp_prim, init_pd, prb, res, FLAG_BWD,
+                     query_pd(prim)),
+                WARN);
+        if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
+        if (is_bench_mode(INIT)) return OK;
+        prim.reset(tmp_prim.release());
+
+        // Pass same memory map as we need data from forward on backward.
+        init_memory_args<prb_t>(
+                mem_map, prb, prim, supported_exec_args(FLAG_BWD));
+        SAFE(init_ref_memory_args(
+                     ref_mem_map, mem_map, prim, prb, res, FLAG_BWD),
+                WARN);
+
+        args = args_t(mem_map);
+        ref_args = args_t(ref_mem_map);
+
+        SAFE(execute_and_wait(prim, args, res), WARN);
+
+        if (is_bench_mode(CORR)) {
+            check_correctness(prb, {SRC}, args, ref_args, setup_cmp, res);
+        }
+    }
+
+    return measure_perf(prb->ctx_exe, res, prim, args);
 }
 
 } // namespace eltwise
