@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -32,6 +32,12 @@ struct desc_t {
     int64_t mb, oc, ic, id, ih, iw;
     std::string name;
     int ndims;
+
+    dims_t src_dims() const;
+    dims_t wei_dims() const;
+    dims_t bia_dims() const;
+    dims_t dst_dims() const;
+    int64_t desc_nelems(int arg, int mask) const;
 };
 int str2desc(desc_t *desc, const char *str);
 std::ostream &operator<<(std::ostream &s, const desc_t &d);
@@ -69,12 +75,29 @@ struct settings_t : public base_settings_t {
     }
 
     void reset() { *this = settings_t(perf_template); }
+
+    bool has_single_setup() const override {
+        return dir.size() == 1 && cfg.size() == 1 && stag.size() == 1
+                && wtag.size() == 1 && dtag.size() == 1
+                && base_settings_t::has_single_setup();
+    }
 };
 
 struct prb_t : public desc_t {
+    // A ctor with common interface across all drivers.
+    prb_t(const settings_t &s)
+        : prb_t(s.desc, s.mb[0], s.dir[0], s.cfg[0], s.stag[0], s.wtag[0],
+                s.dtag[0],
+                settings_t::get_attr(s.scales[0], s.zero_points[0],
+                        s.post_ops[0], s.scratchpad_mode[0], s.fpmath_mode[0]),
+                s.ctx_init[0], s.ctx_exe[0]) {
+        SAFE_V(s.has_single_setup() ? OK : FAIL);
+    }
+
     prb_t(const desc_t &desc, int64_t mb, dir_t dir, const dt_conf_t *cfg,
             const std::string &stag, const std::string &wtag,
-            const std::string &dtag, const attr_t &attr)
+            const std::string &dtag, const attr_t &attr,
+            const thr_ctx_t &ctx_init, const thr_ctx_t &ctx_exe)
         : desc_t(desc)
         , dir(dir)
         , cfg(cfg)
@@ -82,25 +105,23 @@ struct prb_t : public desc_t {
         , wtag(wtag)
         , dtag(dtag)
         , attr(attr)
+        , ctx_init(ctx_init)
+        , ctx_exe(ctx_exe)
         , user_mb(mb)
-        , ops(0)
-        , scales(NULL) {
+        , ops(0) {
         if (mb) this->mb = mb;
         count_ops();
-        generate_oscales();
-    }
-    ~prb_t() {
-        if (scales) zfree(scales);
+        repro = set_repro_line(); // must be last in ctor to collect right info
     }
 
     dir_t dir;
     const dt_conf_t *cfg;
     std::string stag, wtag, dtag;
     attr_t attr;
+    thr_ctx_t ctx_init, ctx_exe;
     int64_t user_mb;
 
     double ops;
-    float *scales;
 
     void count_ops() {
         if (ops > 0) return;
@@ -113,11 +134,20 @@ struct prb_t : public desc_t {
                 : cfg[dk];
     }
 
-    void generate_oscales();
+    // Used to construct memory desc when dimensions are runtime since such mds
+    // can't be used directly from query and memory objects can't be constructed.
+    benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> get_md(int arg) const {
+        assert(!"No runtime dimensions support for this driver!");
+        return make_benchdnn_dnnl_wrapper<dnnl_memory_desc_t>(nullptr);
+    }
 
-    BENCHDNN_DISALLOW_COPY_AND_ASSIGN(prb_t);
+    const char *str() const { return repro.c_str(); }
+
+private:
+    std::string repro;
+
+    std::string set_repro_line();
 };
-std::ostream &operator<<(std::ostream &s, const prb_t &prb);
 
 const dt_conf_t *str2cfg(const char *str);
 std::ostream &operator<<(std::ostream &s, const dt_conf_t *cfg);
@@ -143,6 +173,8 @@ struct perf_report_t : public base_perf_report_t {
 
     double ops() const override { return p_->ops; }
     const attr_t *attr() const override { return &p_->attr; }
+    const thr_ctx_t *ctx_init() const override { return &p_->ctx_init; }
+    const thr_ctx_t *ctx_exe() const override { return &p_->ctx_exe; }
     const int64_t *user_mb() const override { return &p_->user_mb; }
     const std::string *name() const override { return &p_->name; }
     const dir_t *dir() const override { return &p_->dir; }
@@ -174,6 +206,14 @@ inline size_t dst_off_f(const prb_t *prb, int64_t mb, int64_t oc) {
     return mb * prb->oc + oc;
 }
 
+dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args);
+void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
+        const args_t &ref_args);
+std::vector<int> supported_exec_args(dir_t dir);
+int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res, dir_t dir,
+        dnnl_primitive_t prim_ref = nullptr);
+
 void skip_unimplemented_prb(const prb_t *prb, res_t *res);
 void skip_invalid_prb(const prb_t *prb, res_t *res);
 void compute_ref(const prb_t *prb, const args_t &args,
