@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,8 +18,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <sstream>
-
 #include "dnnl_common.hpp"
 #include "utils/parser.hpp"
 
@@ -36,48 +34,58 @@ void check_correctness(const settings_t &s) {
     for_(const auto &i_tag : s.tag)
     for_(const auto &i_stat_tag : s.stat_tag)
     for_(const auto &i_flags : s.flags)
-    for_(const auto &i_oscale : s.oscale)
+    for_(const auto &i_scales : s.scales)
     for_(const auto &i_scratchpad_mode : s.scratchpad_mode)
+    for_(const auto &i_ctx_init : s.ctx_init)
+    for_(const auto &i_ctx_exe : s.ctx_exe)
     for (auto i_inplace : s.inplace) {
-        auto attr = settings_t::get_attr(i_oscale, i_scratchpad_mode);
-
-        static constexpr int n_inputs = 2;
-        if (i_dt.size() != 1 && i_dt.size() != n_inputs) {
-            fprintf(stderr,
-                    "ERROR: lnorm driver: `dt` option expects either a single "
-                    "input or `%d` inputs in SRC, DST order. Current size is: "
-                    "\"%ld\"\n",
-                    n_inputs, (long)i_dt.size()),
-                    fflush(stderr);
-            SAFE_V(FAIL);
-        }
+        auto attr = settings_t::get_attr(i_scales, i_scratchpad_mode);
 
         const prb_t prb(s.prb_dims, i_tag, i_stat_tag, i_dir, i_dt, i_flags,
-                attr, i_inplace, s.check_alg);
-        std::stringstream ss;
-        ss << prb;
-        const std::string cpp_pstr = ss.str();
-        const char *pstr = cpp_pstr.c_str();
-
-        if (s.pattern && !match_regex(pstr, s.pattern)) return;
-        BENCHDNN_PRINT(1, "run: %s\n", pstr);
+                attr, i_ctx_init, i_ctx_exe, i_inplace, s.check_alg);
+        if (s.pattern && !match_regex(prb.str(), s.pattern)) return;
+        BENCHDNN_PRINT(1, "run: %s\n", prb.str());
 
         res_t res {};
         doit(&prb, &res);
 
-        parse_result(res, pstr);
+        parse_result(res, prb.str());
 
         if (is_bench_mode(PERF)) {
             perf_report_t pr(&prb, s.perf_template);
-            pr.report(&res, pstr);
+            pr.report(&res, prb.str());
         }
     }
 }
 
+int verify_input(const settings_t &s) {
+    for_(const auto &i_scales : s.scales)
+    for (const auto &e : i_scales.scales) {
+        if (e.second.policy != policy_t::COMMON) {
+            BENCHDNN_PRINT(
+                    0, "%s\n", "ERROR: scales support only `common` policy.");
+            return FAIL;
+        }
+    }
+
+    static constexpr int n_inputs = 2;
+    for (const auto &i_dt : s.dt) {
+        if (i_dt.size() != 1 && i_dt.size() != n_inputs) {
+            BENCHDNN_PRINT(0, "%s%d%s%ld%s\n",
+                    "ERROR: `dt` option expects either 1 or ", n_inputs,
+                    " inputs in SRC:DST format. Current size is: \"",
+                    (long)i_dt.size(), "\".");
+            return FAIL;
+        }
+    }
+
+    return OK;
+}
+
 static const std::string help_flags
         = "FLAGS    (Default: not specified)\n    Specifies normalization "
-          "flags. `FLAGS` values are:\n    * `G` for global_stats.\n    * `S` "
-          "for scaleshift.\n    * `C` for scale.\n    * `H` for shift.\n";
+          "flags. `FLAGS` values are:\n    * `G` for global_stats.\n    * `C` "
+          "for scale.\n    * `H` for shift.\n";
 
 int bench(int argc, char **argv) {
     driver_name = "lnorm";
@@ -94,9 +102,11 @@ int bench(int argc, char **argv) {
                 || parse_vector_option(s.flags, def.flags, str2flags, argv[0],
                         "flags", help_flags)
                 || parse_inplace(s.inplace, def.inplace, argv[0])
-                || parse_attr_oscale(s.oscale, argv[0])
+                || parse_attr_scales(s.scales, argv[0])
                 || parse_attr_scratchpad_mode(
                         s.scratchpad_mode, def.scratchpad_mode, argv[0])
+                || parse_ctx_init(s.ctx_init, def.ctx_init, argv[0])
+                || parse_ctx_exe(s.ctx_exe, def.ctx_exe, argv[0])
                 || parse_test_pattern_match(s.pattern, argv[0])
                 || parse_perf_template(s.perf_template, s.perf_template_def,
                         s.perf_template_csv(), argv[0])
@@ -105,6 +115,8 @@ int bench(int argc, char **argv) {
             catch_unknown_options(argv[0]);
 
             parse_prb_dims(s.prb_dims, argv[0]);
+
+            SAFE(verify_input(s), WARN);
             check_correctness(s);
         }
     }
