@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -17,13 +17,13 @@
 #ifndef LNORM_HPP
 #define LNORM_HPP
 
+#include <iostream>
+
 #include <assert.h>
 #include <limits.h>
 #include <numeric>
 #include <stdint.h>
 
-#include <iostream>
-
 #include "common.hpp"
 #include "dnn_types.hpp"
 #include "dnnl_common.hpp"
@@ -39,7 +39,6 @@ using check_alg_t = bnorm::check_alg_t;
 using flags_t = bnorm::flags_t;
 const flags_t NONE = bnorm::NONE;
 const flags_t GLOB_STATS = bnorm::GLOB_STATS;
-const flags_t USE_SCALESHIFT = bnorm::USE_SCALESHIFT;
 const flags_t USE_SCALE = bnorm::USE_SCALE;
 const flags_t USE_SHIFT = bnorm::USE_SHIFT;
 const auto flags2str = bnorm::flags2str;
@@ -68,13 +67,30 @@ struct settings_t : public base_settings_t {
     }
 
     void reset() { *this = settings_t(perf_template); }
+
+    bool has_single_setup() const override {
+        return dir.size() == 1 && dt.size() == 1 && tag.size() == 1
+                && stat_tag.size() == 1 && flags.size() == 1
+                && base_settings_t::has_single_setup();
+    }
 };
 
 struct prb_t : public prb_dims_t {
+    // A ctor with common interface across all drivers.
+    prb_t(const settings_t &s)
+        : prb_t(s.prb_dims, s.tag[0], s.stat_tag[0], s.dir[0], s.dt[0],
+                s.flags[0],
+                settings_t::get_attr(s.scales[0], s.zero_points[0],
+                        s.post_ops[0], s.scratchpad_mode[0], s.fpmath_mode[0]),
+                s.ctx_init[0], s.ctx_exe[0], s.inplace[0], s.check_alg) {
+        SAFE_V(s.has_single_setup() ? OK : FAIL);
+    }
+
     prb_t(const prb_dims_t &prb_dims, const std::vector<std::string> &tag,
             const std::string &stat_tag, dir_t dir,
             const std::vector<dnnl_data_type_t> &dt, flags_t flags,
-            const attr_t &attr, bool inplace, check_alg_t check_alg)
+            const attr_t &attr, const thr_ctx_t &ctx_init,
+            const thr_ctx_t &ctx_exe, bool inplace, check_alg_t check_alg)
         : prb_dims_t(prb_dims)
         , check_alg(check_alg)
         , tag(tag)
@@ -84,7 +100,8 @@ struct prb_t : public prb_dims_t {
         , flags(flags)
         , inplace(inplace)
         , attr(attr)
-        , scales(NULL) {
+        , ctx_init(ctx_init)
+        , ctx_exe(ctx_exe) {
         n = 1;
         for (int d = 0; d < ndims - 1; d++)
             n *= dims[d];
@@ -97,11 +114,7 @@ struct prb_t : public prb_dims_t {
             this->dt.assign(2, val);
         }
         if (tag.size() == 1) { this->tag.push_back(tag::any); }
-
-        generate_oscales();
-    }
-    ~prb_t() {
-        if (scales) zfree(scales);
+        repro = set_repro_line(); // must be last in ctor to collect right info
     }
 
     check_alg_t check_alg;
@@ -112,18 +125,28 @@ struct prb_t : public prb_dims_t {
     flags_t flags;
     bool inplace;
     attr_t attr;
+    const thr_ctx_t ctx_init, ctx_exe;
     int64_t n, c;
     float eps;
 
-    bool use_ss() const { return flags & USE_SCALESHIFT; }
+    bool use_stats() const { return flags & GLOB_STATS; }
     bool use_sc() const { return flags & USE_SCALE; }
     bool use_sh() const { return flags & USE_SHIFT; }
 
-    float *scales;
-    void generate_oscales();
-};
+    // Used to construct memory desc when dimensions are runtime since such mds
+    // can't be used directly from query and memory objects can't be constructed.
+    benchdnn_dnnl_wrapper_t<dnnl_memory_desc_t> get_md(int arg) const {
+        assert(!"No runtime dimensions support for this driver!");
+        return make_benchdnn_dnnl_wrapper<dnnl_memory_desc_t>(nullptr);
+    }
 
-std::ostream &operator<<(std::ostream &s, const prb_t &prb);
+    const char *str() const { return repro.c_str(); }
+
+private:
+    std::string repro;
+
+    std::string set_repro_line();
+};
 
 struct perf_report_t : public base_perf_report_t {
     perf_report_t(const prb_t *prb, const char *perf_template)
@@ -144,6 +167,9 @@ struct perf_report_t : public base_perf_report_t {
         s << flags2str(p_->flags);
     }
 
+    const attr_t *attr() const override { return &p_->attr; }
+    const thr_ctx_t *ctx_init() const override { return &p_->ctx_init; }
+    const thr_ctx_t *ctx_exe() const override { return &p_->ctx_exe; }
     const std::string *name() const override { return &p_->name; }
     const dir_t *dir() const override { return &p_->dir; }
     const std::vector<dnnl_data_type_t> *sdt() const override {
@@ -158,6 +184,14 @@ private:
     std::string stat_tag_;
 };
 
+dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args);
+void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
+        const args_t &ref_args);
+std::vector<int> supported_exec_args(dir_t dir);
+int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res, dir_t dir,
+        dnnl_primitive_t prim_ref = nullptr);
+
 void skip_unimplemented_prb(const prb_t *prb, res_t *res);
 void skip_invalid_prb(const prb_t *prb, res_t *res);
 void compute_ref(const prb_t *prb, const args_t &args,
