@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2017-2022 Intel Corporation
+* Copyright 2017-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -18,8 +18,6 @@
 #include <stdlib.h>
 #include <string.h>
 
-#include <sstream>
-
 #include "dnnl_common.hpp"
 #include "utils/parser.hpp"
 
@@ -33,24 +31,23 @@ void check_correctness(const settings_t &s) {
     for_(const auto &i_tag : s.tag)
     for_(const auto &i_alg : s.alg)
     for_(const auto &i_mb : s.mb)
-    for (const auto &i_scratchpad_mode : s.scratchpad_mode) {
+    for_(const auto &i_scratchpad_mode : s.scratchpad_mode)
+    for_(const auto &i_ctx_init : s.ctx_init)
+    for (const auto &i_ctx_exe : s.ctx_exe) {
         auto attr = settings_t::get_attr(i_scratchpad_mode);
 
-        const prb_t prb(s.desc, i_mb, i_dir, i_dt, i_tag, i_alg, attr);
-        std::stringstream ss;
-        ss << prb;
-        const std::string cpp_pstr = ss.str();
-        const char *pstr = cpp_pstr.c_str();
-        BENCHDNN_PRINT(1, "run: %s\n", pstr);
+        const prb_t prb(s.desc, i_mb, i_dir, i_dt, i_tag, i_alg, attr,
+                i_ctx_init, i_ctx_exe);
+        BENCHDNN_PRINT(1, "run: %s\n", prb.str());
 
         res_t res {};
         doit(&prb, &res);
 
-        parse_result(res, pstr);
+        parse_result(res, prb.str());
 
         if (is_bench_mode(PERF)) {
             perf_report_t pr(&prb, s.perf_template);
-            pr.report(&res, pstr);
+            pr.report(&res, prb.str());
         }
     }
 }
@@ -70,6 +67,8 @@ int bench(int argc, char **argv) {
                 || parse_mb(s.mb, def.mb, argv[0])
                 || parse_attr_scratchpad_mode(
                         s.scratchpad_mode, def.scratchpad_mode, argv[0])
+                || parse_ctx_init(s.ctx_init, def.ctx_init, argv[0])
+                || parse_ctx_exe(s.ctx_exe, def.ctx_exe, argv[0])
                 || parse_perf_template(s.perf_template, s.perf_template_def,
                         s.perf_template_csv(), argv[0])
                 || parse_reset(s, argv[0]) || parse_help(argv[0]);
