@@ -86,69 +86,45 @@ int fill_data(data_kind_t kind, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp) {
     return OK;
 }
 
-int setup_prelu_po(const_dnnl_primitive_desc_t pd, std::vector<int> &args,
-        std::vector<dnn_mem_t> &ref_mem, std::vector<dnn_mem_t> &prim_mem) {
-    const auto &dst_md = query_md(pd, DNNL_ARG_DST);
-    auto const_attr_po = query_post_ops(pd);
-    const auto po_len = dnnl_post_ops_len(const_attr_po);
-    for (int idx = 0; idx < po_len; ++idx) {
-        const auto kind = dnnl_post_ops_get_kind(const_attr_po, idx);
-        if (kind != dnnl_prelu) continue;
-
-        const auto ndims = dst_md.ndims;
-        int mask = 0;
-        dnnl_dims_t dims = {0};
-        dnnl_post_ops_get_params_prelu(const_attr_po, idx, &mask);
-
-        // Deduce prelu weights dims based on input policy.
-        for (int d = 0; d < ndims; ++d) {
-            dims[d] = (mask & (1 << d)) ? dst_md.dims[d] : 1;
-        }
-
-        // Following call can not be executed if po_md has runtime dimension due
-        // to undefined size.
-        ref_mem.emplace_back(ndims, dims, dnnl_f32, tag::abx, get_cpu_engine());
-        prim_mem.emplace_back(
-                ndims, dims, dnnl_f32, tag::axb, get_test_engine());
-        args.push_back(DNNL_ARG_ATTR_MULTIPLE_POST_OP(idx) | DNNL_ARG_WEIGHTS);
-        fill_data(WEI, prim_mem.back(), ref_mem.back());
-    }
-    return OK;
-}
-
 dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
     const prb_t *prb = init_pd_args.prb;
 
-    dnnl_prelu_desc_t pd;
-
     const auto &src_dims = prb->vdims[0];
     const auto &weight_dims = prb->vdims[1];
 
-    auto data_d = dnn_mem_t::init_md(
+    auto src_d = dnn_mem_t::init_md(
             prb->ndims, src_dims.data(), prb->sdt[0], prb->stag[0]);
     auto weights_d = dnn_mem_t::init_md(
             prb->ndims, weight_dims.data(), prb->sdt[1], prb->stag[1]);
 
-    if (prb->dir & FLAG_FWD) {
-        auto prop = prb->dir & FLAG_INF ? dnnl_forward_inference
-                                        : dnnl_forward_training;
-        DNN_SAFE_STATUS(
-                dnnl_prelu_forward_desc_init(&pd, prop, &data_d, &weights_d));
-    } else {
-        auto diff_data_d = dnn_mem_t::init_md(
-                prb->ndims, src_dims.data(), prb->sdt[0], prb->stag[0]);
-        auto diff_weights_d = dnn_mem_t::init_md(
-                prb->ndims, weight_dims.data(), prb->sdt[1], prb->stag[1]);
-
-        DNN_SAFE_STATUS(dnnl_prelu_backward_desc_init(
-                &pd, &data_d, &weights_d, &diff_data_d, &diff_weights_d));
-    }
-
     auto dnnl_attr = make_benchdnn_dnnl_wrapper(
             create_dnnl_attr(prb->attr, attr_args_t()));
 
-    return dnnl_primitive_desc_iterator_create(&init_pd_args.pd_it, &pd,
-            dnnl_attr, init_pd_args.engine, init_pd_args.hint);
+    if (prb->dir & FLAG_FWD) {
+        auto dst_d = dnn_mem_t::init_md(
+                prb->ndims, src_dims.data(), prb->sdt[0], tag::any);
+
+        auto prop = prb->dir & FLAG_INF ? dnnl_forward_inference
+                                        : dnnl_forward_training;
+        DNN_SAFE_STATUS(dnnl_prelu_forward_primitive_desc_create(
+                &init_pd_args.pd, init_pd_args.engine, prop,
+                init_pd_args.src_md ? init_pd_args.src_md : src_d, weights_d,
+                dst_d, dnnl_attr));
+    } else {
+        auto diff_src_d = dnn_mem_t::init_md(
+                prb->ndims, src_dims.data(), prb->sdt[0], tag::any);
+        auto diff_weights_d = dnn_mem_t::init_md(
+                prb->ndims, weight_dims.data(), prb->sdt[1], tag::any);
+        auto diff_dst_d = dnn_mem_t::init_md(
+                prb->ndims, src_dims.data(), prb->sdt[0], tag::any);
+
+        DNN_SAFE_STATUS(dnnl_prelu_backward_primitive_desc_create(
+                &init_pd_args.pd, init_pd_args.engine, src_d, weights_d,
+                diff_src_d, diff_weights_d, diff_dst_d, init_pd_args.hint,
+                dnnl_attr));
+    }
+
+    return dnnl_success;
 }
 
 void skip_unimplemented_prb(const prb_t *prb, res_t *res) {
@@ -169,91 +145,90 @@ void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
     cmp.set_zero_trust_percent(zero_trust_percent);
 }
 
+std::vector<int> supported_exec_args(dir_t dir) {
+    static const std::vector<int> exec_fwd_args = {
+            DNNL_ARG_SRC,
+            DNNL_ARG_WEIGHTS,
+            DNNL_ARG_DST,
+    };
+    static const std::vector<int> exec_bwd_args = {
+            DNNL_ARG_SRC,
+            DNNL_ARG_WEIGHTS,
+            DNNL_ARG_DIFF_SRC,
+            DNNL_ARG_DIFF_WEIGHTS,
+            DNNL_ARG_DIFF_DST,
+    };
+    return (dir & FLAG_FWD) ? exec_fwd_args : exec_bwd_args;
+};
+
+int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res, dir_t dir,
+        dnnl_primitive_t prim_ref) {
+    const auto &ref_engine = get_cpu_engine();
+
+    for (auto &entry : mem_map) {
+        const int exec_arg = entry.first;
+        auto &mem = entry.second; // `mem` is modified by filler (reorder).
+
+        ref_mem_map.emplace(
+                exec_arg, dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+        auto &ref_mem = ref_mem_map[exec_arg];
+
+        switch (exec_arg) {
+            case DNNL_ARG_SRC: SAFE(fill_data(SRC, mem, ref_mem), WARN); break;
+            case DNNL_ARG_WEIGHTS:
+                SAFE(fill_data(WEI, mem, ref_mem), WARN);
+                break;
+            case DNNL_ARG_DIFF_DST:
+                SAFE(fill_data(DST, mem, ref_mem), WARN);
+                break;
+            case DNNL_ARG_SCRATCHPAD: break;
+            default: break;
+        }
+        // Don't keep reference memory if it is not used further.
+        if (!is_bench_mode(CORR)) ref_mem_map.clear();
+    }
+
+    return OK;
+}
+
+std::vector<data_kind_t> get_kinds_to_check(const prb_t *prb) {
+    std::vector<data_kind_t> check_kinds;
+    if (prb->dir & FLAG_FWD) {
+        check_kinds = {DST};
+    } else if (prb->dir & FLAG_BWD) {
+        check_kinds = {SRC, WEI};
+    } else {
+        assert(!"unexpected!");
+        SAFE_V(FAIL);
+    }
+    assert(!check_kinds.empty());
+    return check_kinds;
+}
+
 int doit(const prb_t *prb, res_t *res) {
     if (bench_mode == LIST) return res->state = LISTED, OK;
 
     benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim;
-    SAFE(init_prim(prim, init_pd, prb, res), WARN);
+    SAFE(init_prim(prb->ctx_init, prim, init_pd, prb, res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
+    if (is_bench_mode(INIT)) return OK;
 
-    auto const_pd = query_pd(prim);
+    dnn_mem_map_t mem_map, ref_mem_map;
+    init_memory_args<prb_t>(mem_map, prb, prim, supported_exec_args(prb->dir));
+    SAFE(init_ref_memory_args(ref_mem_map, mem_map, prim, prb, res, prb->dir),
+            WARN);
 
-    if (check_mem_size(const_pd) != OK) {
-        return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
+    args_t args(mem_map), ref_args(ref_mem_map);
+
+    SAFE(execute_and_wait(prim, args, res), WARN);
+
+    if (is_bench_mode(CORR)) {
+        check_correctness(
+                prb, get_kinds_to_check(prb), args, ref_args, setup_cmp, res);
     }
 
-    const auto &data_md = query_md(const_pd, DNNL_ARG_SRC);
-    const auto &weight_md = query_md(const_pd, DNNL_ARG_WEIGHTS);
-    const auto &scratchpad_md = query_md(const_pd, DNNL_ARG_SCRATCHPAD);
-    const auto &test_engine = get_test_engine();
-    const auto &ref_engine = get_cpu_engine();
-
-    dnn_mem_t src_fp(data_md, dnnl_f32, tag::abx, ref_engine);
-    dnn_mem_t weights_fp(weight_md, dnnl_f32, tag::abx, ref_engine);
-
-    dnn_mem_t src_dt(data_md, test_engine);
-    dnn_mem_t weights_dt(weight_md, test_engine);
-    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
-
-    SAFE(fill_data(SRC, src_dt, src_fp), WARN);
-    SAFE(fill_data(WEI, weights_dt, weights_fp), WARN);
-
-    args_t args, ref_args;
-
-    args.set(DNNL_ARG_SRC, src_dt);
-    args.set(DNNL_ARG_WEIGHTS, weights_dt);
-    args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
-
-    dnn_mem_t dst_dt, d_src_fp, d_src_dt, d_dst_fp, d_dst_dt, d_weights_fp,
-            d_weights_dt;
-
-    if (prb->dir & FLAG_FWD) {
-        dnn_mem_t dst_fp(data_md, dnnl_f32, tag::abx, ref_engine);
-        dst_dt = dnn_mem_t(data_md, test_engine);
-
-        args.set(DNNL_ARG_DST, dst_dt);
-
-        SAFE(execute_and_wait(prim, args, res), WARN);
-
-        if (is_bench_mode(CORR)) {
-            ref_args.set(DNNL_ARG_SRC, src_fp);
-            ref_args.set(DNNL_ARG_WEIGHTS, weights_fp);
-            ref_args.set(DNNL_ARG_DST, dst_fp);
-
-            check_correctness(prb, {DST}, args, ref_args, setup_cmp, res);
-        }
-    } else {
-        const auto &d_data_md = query_md(const_pd, DNNL_ARG_DIFF_DST);
-        const auto &d_weights_md = query_md(const_pd, DNNL_ARG_DIFF_WEIGHTS);
-
-        dnn_mem_t d_src_fp(d_data_md, dnnl_f32, tag::abx, ref_engine);
-        dnn_mem_t d_weights_fp(d_weights_md, dnnl_f32, tag::abx, ref_engine);
-        dnn_mem_t d_dst_fp(d_data_md, dnnl_f32, tag::abx, ref_engine);
-
-        d_src_dt = dnn_mem_t(d_data_md, test_engine);
-        d_weights_dt = dnn_mem_t(d_weights_md, test_engine);
-        d_dst_dt = dnn_mem_t(d_data_md, test_engine);
-
-        SAFE(fill_data(DST, d_dst_dt, d_dst_fp), WARN);
-
-        args.set(DNNL_ARG_DIFF_DST, d_dst_dt);
-        args.set(DNNL_ARG_DIFF_SRC, d_src_dt);
-        args.set(DNNL_ARG_DIFF_WEIGHTS, d_weights_dt);
-
-        SAFE(execute_and_wait(prim, args, res), WARN);
-
-        if (is_bench_mode(CORR)) {
-            ref_args.set(DNNL_ARG_SRC, src_fp);
-            ref_args.set(DNNL_ARG_WEIGHTS, weights_fp);
-            ref_args.set(DNNL_ARG_DIFF_DST, d_dst_fp);
-            ref_args.set(DNNL_ARG_DIFF_SRC, d_src_fp);
-            ref_args.set(DNNL_ARG_DIFF_WEIGHTS, d_weights_fp);
-
-            check_correctness(prb, {SRC, WEI}, args, ref_args, setup_cmp, res);
-        }
-    }
-
-    return measure_perf(res, prim, args);
+    return measure_perf(prb->ctx_exe, res, prim, args);
 }
 
 } // namespace prelu
