@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2018-2022 Intel Corporation
+* Copyright 2018-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -41,13 +41,38 @@ void check_correctness(const settings_t &s) {
     for_(const auto &i_activation : s.activation)
     for_(auto i_skip_nonlinear : s.skip_nonlinear)
     for_(auto i_trivial_strides : s.trivial_strides)
+    for_(const auto &i_flags : s.flags)
     for_(const auto &i_n_layer : s.n_layer)
     for_(const auto &i_n_iter : s.n_iter)
     for_(const auto &i_mb : s.mb)
     for_(const auto &i_scratchpad_mode : s.scratchpad_mode)
+    for_(const auto &i_ctx_init : s.ctx_init)
+    for_(const auto &i_ctx_exe : s.ctx_exe)
     for (const auto &i_fpmath_mode : s.fpmath_mode) {
-        if (i_with_peephole && i_alg != VANILLA_LSTM) continue;
+        auto attr = settings_t::get_attr(i_scratchpad_mode, i_fpmath_mode);
 
+        const prb_t prb(s.desc, dt_conf_t::create(i_cfg, attr), i_prop, i_alg,
+                i_with_peephole, i_with_projection, i_direction, i_scale_policy,
+                i_scale_proj_policy, i_flags, i_activation, attr, i_ctx_init,
+                i_ctx_exe, s.alpha, s.beta, i_skip_nonlinear, i_trivial_strides,
+                i_n_layer, i_n_iter, i_mb);
+        BENCHDNN_PRINT(1, "run: %s\n", prb.str());
+
+        res_t res {};
+        doit(prb, &res);
+
+        parse_result(res, prb.str());
+
+        if (is_bench_mode(PERF)) {
+            perf_report_t pr(&prb, s.perf_template);
+            pr.report(&res, prb.str());
+        }
+    }
+}
+
+int verify_input(const settings_t &s) {
+
+    for (const auto &i_scale_policy : s.scale_policy) {
         if (!(i_scale_policy == policy_t::COMMON
                     || i_scale_policy == policy_t::PER_OC)) {
             std::stringstream ss;
@@ -61,30 +86,9 @@ void check_correctness(const settings_t &s) {
                     fflush(stderr);
             SAFE_V(FAIL);
         }
-
-        auto attr = settings_t::get_attr(i_scratchpad_mode, i_fpmath_mode);
-
-        const prb_t prb(s.desc, dt_conf_t::create(i_cfg, attr), i_prop, i_alg,
-                i_with_peephole, i_with_projection, i_direction, i_scale_policy,
-                i_scale_proj_policy, s.flags, i_activation, attr, s.alpha,
-                s.beta, i_skip_nonlinear, i_trivial_strides, i_n_layer,
-                i_n_iter, i_mb);
-        std::stringstream ss;
-        ss << prb;
-        const std::string cpp_pstr = ss.str();
-        const char *pstr = cpp_pstr.c_str();
-        BENCHDNN_PRINT(1, "run: %s\n", pstr);
-
-        res_t res {};
-        doit(prb, &res);
-
-        parse_result(res, pstr);
-
-        if (is_bench_mode(PERF)) {
-            perf_report_t pr(&prb, s.perf_template);
-            pr.report(&res, pstr);
-        }
     }
+
+    return OK;
 }
 
 static const std::string help_direction
@@ -115,6 +119,10 @@ static const std::string help_with_projection
         = "BOOL    (Default: `false`)\n    When set to `true`, enables LSTM "
           "projection extension.\n";
 
+static const std::string help_flags
+        = "FLAGS    (Default: not specified)\n    Specifies rnn flags. `FLAGS` "
+          "values are:\n    * `O` for diff_weights_overwrite.\n";
+
 int bench(int argc, char **argv) {
     driver_name = "rnn";
     using namespace parser;
@@ -143,6 +151,8 @@ int bench(int argc, char **argv) {
                         s.skip_nonlinear, def.skip_nonlinear, argv[0])
                 || parse_trivial_strides(
                         s.trivial_strides, def.trivial_strides, argv[0])
+                || parse_vector_option(s.flags, def.flags, str2flags, argv[0],
+                        "flags", help_flags)
                 || parse_vector_option(s.with_peephole, def.with_peephole,
                         str2bool, argv[0], "with-peephole", help_with_peephole)
                 || parse_vector_option(s.with_projection, def.with_projection,
@@ -152,6 +162,8 @@ int bench(int argc, char **argv) {
                         s.scratchpad_mode, def.scratchpad_mode, argv[0])
                 || parse_attr_fpmath_mode(
                         s.fpmath_mode, def.fpmath_mode, argv[0])
+                || parse_ctx_init(s.ctx_init, def.ctx_init, argv[0])
+                || parse_ctx_exe(s.ctx_exe, def.ctx_exe, argv[0])
                 || parse_perf_template(s.perf_template, s.perf_template_def,
                         s.perf_template_csv(), argv[0])
                 || parse_reset(s, argv[0]) || parse_help(argv[0]);
@@ -159,6 +171,8 @@ int bench(int argc, char **argv) {
             catch_unknown_options(argv[0]);
 
             SAFE(str2desc(&s.desc, argv[0]), CRIT);
+
+            SAFE(verify_input(s), WARN);
             check_correctness(s);
         }
     }
