@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2022 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -14,6 +14,7 @@
 * limitations under the License.
 *******************************************************************************/
 
+#include <algorithm>
 #include <random>
 
 #include <float.h>
@@ -23,6 +24,7 @@
 
 #include "oneapi/dnnl/dnnl.h"
 
+#include "utils/fill.hpp"
 #include "utils/parallel.hpp"
 
 #include "dnnl_common.hpp"
@@ -35,14 +37,15 @@ namespace softmax {
 dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
     const prb_t *prb = init_pd_args.prb;
 
-    dnnl_softmax_v2_desc_t sd;
-
     auto dst_d = dnn_mem_t::init_md(
             prb->ndims, prb->dims.data(), prb->ddt, prb->dtag);
 
     dnnl_alg_kind_t alg_kind = dnnl_softmax_accurate;
     if (prb->alg == LOGSOFTMAX) alg_kind = dnnl_softmax_log;
 
+    auto dnnl_attr = make_benchdnn_dnnl_wrapper(
+            create_dnnl_attr(prb->attr, attr_args_t()));
+
     if (prb->dir & FLAG_FWD) {
         auto src_d = dnn_mem_t::init_md(
                 prb->ndims, prb->dims.data(), prb->sdt, prb->stag);
@@ -50,8 +53,10 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
         auto prop = prb->dir & FLAG_INF ? dnnl_forward_inference
                                         : dnnl_forward_training;
 
-        DNN_SAFE_STATUS(dnnl_softmax_v2_forward_desc_init(
-                &sd, prop, alg_kind, &src_d, &dst_d, prb->axis));
+        DNN_SAFE_STATUS(dnnl_softmax_forward_primitive_desc_create(
+                &init_pd_args.pd, init_pd_args.engine, prop, alg_kind,
+                init_pd_args.src_md ? init_pd_args.src_md : src_d, dst_d,
+                prb->axis, dnnl_attr));
     } else {
         // Re-create dst_md with source tag if dst was not specified, immitating
         // default value.
@@ -65,17 +70,12 @@ dnnl_status_t init_pd(init_pd_args_t<prb_t> &init_pd_args) {
         auto diff_dst_d = dnn_mem_t::init_md(
                 prb->ndims, prb->dims.data(), prb->ddt, tag::any);
 
-        DNN_SAFE_STATUS(dnnl_softmax_v2_backward_desc_init(
-                &sd, alg_kind, &diff_src_d, &diff_dst_d, &dst_d, prb->axis));
+        DNN_SAFE_STATUS(dnnl_softmax_backward_primitive_desc_create(
+                &init_pd_args.pd, init_pd_args.engine, alg_kind, diff_src_d,
+                diff_dst_d, dst_d, prb->axis, init_pd_args.hint, dnnl_attr));
     }
 
-    attr_args_t attr_args;
-    attr_args.prepare_output_scales(prb->attr, prb->scales, 1);
-    auto dnnl_attr = make_benchdnn_dnnl_wrapper(
-            create_dnnl_attr(prb->attr, attr_args));
-
-    return dnnl_primitive_desc_iterator_create(&init_pd_args.pd_it, &sd,
-            dnnl_attr, init_pd_args.engine, init_pd_args.hint);
+    return dnnl_success;
 }
 
 int fill_data_fwd(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp) {
@@ -141,7 +141,8 @@ int fill_data_fwd(const prb_t *prb, dnn_mem_t &mem_dt, dnn_mem_t &mem_fp) {
             i = 0;
             for (int64_t as = 0; as < axis_size; as++) {
                 auto offset = inner_size * (idx * axis_size + as) + in;
-                float value = INT_MIN;
+                float value
+                        = std::max(lowest_dt(mem_dt.dt()), lowest_dt(prb->ddt));
                 if (as >= axis_idx_start && as < axis_idx_start + n_sum) {
                     value = top_val[i];
                     n_top[i]--;
@@ -225,99 +226,118 @@ void setup_cmp(compare::compare_t &cmp, const prb_t *prb, data_kind_t kind,
     cmp.set_driver_check_function(softmax_add_check);
 }
 
+std::vector<int> supported_exec_args(dir_t dir) {
+    static const std::vector<int> exec_fwd_args = {
+            DNNL_ARG_SRC,
+            DNNL_ARG_DST,
+    };
+    static const std::vector<int> exec_bwd_args = {
+            DNNL_ARG_DST,
+            DNNL_ARG_DIFF_SRC,
+            DNNL_ARG_DIFF_DST,
+    };
+    return (dir & FLAG_FWD) ? exec_fwd_args : exec_bwd_args;
+};
+
+int init_ref_memory_args(dnn_mem_map_t &ref_mem_map, dnn_mem_map_t &mem_map,
+        dnnl_primitive_t prim, const prb_t *prb, res_t *res, dir_t dir,
+        dnnl_primitive_t prim_ref) {
+    const auto &ref_engine = get_cpu_engine();
+
+    for (auto &entry : mem_map) {
+        const int exec_arg = entry.first;
+        auto &mem = entry.second; // `mem` is modified by filler (reorder).
+
+        ref_mem_map.emplace(
+                exec_arg, dnn_mem_t(mem.md_, dnnl_f32, tag::abx, ref_engine));
+        auto &ref_mem = ref_mem_map[exec_arg];
+
+        switch (exec_arg) {
+            case DNNL_ARG_SRC:
+                SAFE(fill_data_fwd(prb, mem, ref_mem), WARN);
+                break;
+            case DNNL_ARG_DST:
+                if (dir & FLAG_BWD) {
+                    const bool neg_sign = prb->alg == SOFTMAX ? true : false;
+                    SAFE(fill_data_bwd(prb, mem, ref_mem, neg_sign), WARN);
+                }
+                break;
+            case DNNL_ARG_DIFF_DST: {
+                const bool neg_sign = prb->alg == SOFTMAX ? true : false;
+                SAFE(fill_data_bwd(prb, mem, ref_mem, !neg_sign), WARN);
+            } break;
+            case DNNL_ARG_SCRATCHPAD: break;
+            default: { // Process all attributes here
+                bool is_scales_arg = (exec_arg & DNNL_ARG_ATTR_SCALES);
+                if (is_scales_arg) {
+                    int exec_src_arg = exec_arg ^ DNNL_ARG_ATTR_SCALES;
+                    SAFE(fill_scales(prb->attr, exec_src_arg, mem, ref_mem),
+                            WARN);
+                }
+            } break;
+        }
+        // Don't keep reference memory if it is not used further.
+        if (!is_bench_mode(CORR)) ref_mem_map.clear();
+    }
+
+    // Drop destination memory for in-place case. `args` will take care of rest.
+    const bool inplace_fwd = prb->inplace && (prb->dir & FLAG_FWD);
+    const bool inplace_bwd = prb->inplace && (dir & FLAG_BWD);
+    if (inplace_fwd) {
+        mem_map[DNNL_ARG_DST] = dnn_mem_t();
+    } else if (inplace_bwd) {
+        mem_map[DNNL_ARG_DIFF_SRC] = dnn_mem_t();
+    }
+
+    if (!is_bench_mode(CORR)) return OK;
+
+    // Use inplace reference computation every time.
+    if (dir & FLAG_FWD) {
+        ref_mem_map.emplace(DNNL_ARG_DST, dnn_mem_t());
+    } else {
+        ref_mem_map.emplace(DNNL_ARG_DIFF_SRC, dnn_mem_t());
+    }
+
+    return OK;
+}
+
+std::vector<data_kind_t> get_kinds_to_check(const prb_t *prb) {
+    std::vector<data_kind_t> check_kinds;
+    if (prb->dir & FLAG_FWD) {
+        check_kinds = {DST};
+    } else if (prb->dir & FLAG_BWD) {
+        check_kinds = {SRC};
+    } else {
+        assert(!"unexpected!");
+        SAFE_V(FAIL);
+    }
+    assert(!check_kinds.empty());
+    return check_kinds;
+}
+
 int doit(const prb_t *prb, res_t *res) {
     if (bench_mode == LIST) return res->state = LISTED, OK;
 
     benchdnn_dnnl_wrapper_t<dnnl_primitive_t> prim;
-    SAFE(init_prim(prim, init_pd, prb, res), WARN);
+    SAFE(init_prim(prb->ctx_init, prim, init_pd, prb, res), WARN);
     if (res->state == SKIPPED || res->state == UNIMPLEMENTED) return OK;
+    if (is_bench_mode(INIT)) return OK;
 
-    auto const_pd = query_pd(prim);
+    dnn_mem_map_t mem_map, ref_mem_map;
+    init_memory_args<prb_t>(mem_map, prb, prim, supported_exec_args(prb->dir));
+    SAFE(init_ref_memory_args(ref_mem_map, mem_map, prim, prb, res, prb->dir),
+            WARN);
 
-    if (check_mem_size(const_pd) != OK) {
-        return res->state = SKIPPED, res->reason = NOT_ENOUGH_RAM, OK;
+    args_t args(mem_map), ref_args(ref_mem_map);
+
+    SAFE(execute_and_wait(prim, args, res), WARN);
+
+    if (is_bench_mode(CORR)) {
+        check_correctness(
+                prb, get_kinds_to_check(prb), args, ref_args, setup_cmp, res);
     }
 
-    const auto &scratchpad_md = query_md(const_pd, DNNL_ARG_SCRATCHPAD);
-    const auto &test_engine = get_test_engine();
-    const auto &ref_engine = get_cpu_engine();
-
-    dnn_mem_t src_dt, placeholder_dst_dt;
-    dnn_mem_t &dst_dt = prb->inplace && (prb->dir & FLAG_FWD)
-            ? src_dt
-            : placeholder_dst_dt;
-    dnn_mem_t scratchpad_dt(scratchpad_md, test_engine);
-
-    dnn_mem_t d_dst_dt, placeholder_d_src_dt;
-    dnn_mem_t &d_src_dt = prb->inplace ? d_dst_dt : placeholder_d_src_dt;
-
-    args_t args, ref_args;
-
-    if (prb->dir & FLAG_FWD) {
-        const auto &src_md = query_md(const_pd, DNNL_ARG_SRC);
-        const auto &dst_md = query_md(const_pd, DNNL_ARG_DST);
-
-        src_dt = dnn_mem_t(src_md, test_engine);
-        if (!prb->inplace) {
-            placeholder_dst_dt = dnn_mem_t(dst_md, test_engine);
-        }
-
-        dnn_mem_t src_fp(src_md, dnnl_f32, tag::abx, ref_engine);
-        dnn_mem_t &dst_fp = src_fp; // in-place reference
-
-        SAFE(fill_data_fwd(prb, src_dt, src_fp), WARN);
-        dnn_mem_t scales;
-        maybe_prepare_runtime_scales(scales, prb->attr.oscale, 1, prb->scales);
-
-        args.set(DNNL_ARG_SRC, src_dt);
-        args.set(DNNL_ARG_DST, dst_dt);
-        args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
-        args.set(DNNL_ARG_ATTR_OUTPUT_SCALES, scales);
-
-        SAFE(execute_and_wait(prim, args, res), WARN);
-
-        if (is_bench_mode(CORR)) {
-            ref_args.set(DNNL_ARG_SRC, src_fp);
-            ref_args.set(DNNL_ARG_DST, dst_fp);
-
-            check_correctness(prb, {DST}, args, ref_args, setup_cmp, res);
-        }
-    } else {
-        const auto &dst_md = query_md(const_pd, DNNL_ARG_DST);
-        const auto &d_dst_md = query_md(const_pd, DNNL_ARG_DIFF_DST);
-        const auto &d_src_md = query_md(const_pd, DNNL_ARG_DIFF_SRC);
-
-        placeholder_dst_dt = dnn_mem_t(dst_md, test_engine);
-        d_dst_dt = dnn_mem_t(d_dst_md, test_engine);
-        if (!prb->inplace) {
-            placeholder_d_src_dt = dnn_mem_t(d_src_md, test_engine);
-        }
-
-        dnn_mem_t dst_fp(dst_md, dnnl_f32, tag::abx, ref_engine);
-        dnn_mem_t d_dst_fp(d_dst_md, dnnl_f32, tag::abx, ref_engine);
-        dnn_mem_t &d_src_fp = d_dst_fp; // in-place reference
-
-        const bool neg_sign = prb->alg == SOFTMAX ? true : false;
-        SAFE(fill_data_bwd(prb, dst_dt, dst_fp, neg_sign), WARN);
-        SAFE(fill_data_bwd(prb, d_dst_dt, d_dst_fp, !neg_sign), WARN);
-
-        args.set(DNNL_ARG_DST, dst_dt);
-        args.set(DNNL_ARG_DIFF_DST, d_dst_dt);
-        args.set(DNNL_ARG_DIFF_SRC, d_src_dt);
-        args.set(DNNL_ARG_SCRATCHPAD, scratchpad_dt);
-
-        SAFE(execute_and_wait(prim, args, res), WARN);
-
-        if (is_bench_mode(CORR)) {
-            ref_args.set(DNNL_ARG_DST, dst_fp);
-            ref_args.set(DNNL_ARG_DIFF_DST, d_dst_fp);
-            ref_args.set(DNNL_ARG_DIFF_SRC, d_src_fp);
-
-            check_correctness(prb, {SRC}, args, ref_args, setup_cmp, res);
-        }
-    }
-
-    return measure_perf(res, prim, args);
+    return measure_perf(prb->ctx_exe, res, prim, args);
 }
 
 } // namespace softmax
