@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2016-2021 Intel Corporation
+* Copyright 2016-2022 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -36,8 +36,8 @@ void compute_ref_inner_product_bwd_bias(const test_inner_product_descr_t &ipd,
 
     const memory::desc diff_bias_d = diff_bias.get_desc();
     const memory::desc diff_dst_d = diff_dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper diff_bias_mdw(diff_bias_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.data);
+    const dnnl::impl::memory_desc_wrapper diff_bias_mdw(diff_bias_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.get());
 
     dnnl::impl::parallel_nd(ipd.oc, [&](memory::dim oc) {
         data_t *db = &diff_bias_data[diff_bias_mdw.off_l(oc, true)];
@@ -59,11 +59,12 @@ void compute_ref_inner_product_bwd_weights(int ndims,
     const memory::desc src_d = src.get_desc();
     const memory::desc diff_weights_d = diff_weights.get_desc();
     const memory::desc diff_dst_d = diff_dst.get_desc();
-    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_weights_mdw(diff_weights_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.data);
+    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_weights_mdw(
+            diff_weights_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.get());
 
-    auto padded_ic = src_d.data.padded_dims[1];
+    auto padded_ic = src_d.get_padded_dims()[1];
 
     bool has_spatial = ipd.kh > 1 || ipd.kw > 1;
     if (ndims == 5) has_spatial = has_spatial || ipd.kd > 1;
@@ -206,21 +207,19 @@ protected:
                 : create_md({}, data_type, p.diff_bias_format);
 
         // Create inner product forward (hint for backward)
-        auto ip_fwd_desc = inner_product_forward::desc(prop_kind::forward,
-                ip_src_desc, ip_diff_weights_desc, ip_diff_dst_desc);
         auto ip_fwd_pdesc
-                = inner_product_forward::primitive_desc(ip_fwd_desc, eng);
-
-        // Create inner product backward
-        auto ip_desc = with_bias
-                ? inner_product_backward_weights::desc(ip_src_desc,
-                        ip_diff_weights_desc, ip_diff_bias_desc,
-                        ip_diff_dst_desc)
-                : inner_product_backward_weights::desc(
+                = inner_product_forward::primitive_desc(eng, prop_kind::forward,
                         ip_src_desc, ip_diff_weights_desc, ip_diff_dst_desc);
 
-        auto ip_primitive_desc = inner_product_backward_weights::primitive_desc(
-                ip_desc, eng, ip_fwd_pdesc);
+        // Create inner product backward
+        auto ip_primitive_desc = with_bias
+                ? inner_product_backward_weights::primitive_desc(eng,
+                        ip_src_desc, ip_diff_weights_desc, ip_diff_bias_desc,
+                        ip_diff_dst_desc, ip_fwd_pdesc)
+                : inner_product_backward_weights::primitive_desc(eng,
+                        ip_src_desc, ip_diff_weights_desc, ip_diff_dst_desc,
+                        ip_fwd_pdesc);
+
         ip_primitive_desc = inner_product_backward_weights::primitive_desc(
                 ip_primitive_desc.get()); // test construction from a C pd
 
@@ -256,6 +255,9 @@ protected:
                             query::exec_arg_md, DNNL_ARG_DIFF_BIAS)
                 == ip_primitive_desc.diff_bias_desc());
 
+        ASSERT_EQ(
+                ip_primitive_desc.get_prop_kind(), prop_kind::backward_weights);
+
         EXPECT_ANY_THROW(inner_product_backward_weights(ip_primitive_desc, {}));
         inner_product_backward_weights(ip_primitive_desc)
                 .execute(strm,
