@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2023 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -105,17 +105,21 @@ protected:
                         && ((p.attr.zero_points.src & P::PER_N)
                                 || (p.attr.zero_points.dst & P::PER_N)),
                 "Per dimensional zero points are not supported on GPU");
-        SKIP_IF(get_test_engine_kind() == engine::kind::cpu
-                        && p.base.src.tag == impl::format_tag::AB8a4b,
-                "Don't test blocked formats on CPU");
 
         SKIP_IF_CUDA((p.attr.zero_points.src != 0 || p.attr.zero_points.dst != 0
                              || p.attr.zero_points.weights != 0),
                 "Zero points not supported for CUDA");
 
+        SKIP_IF_HIP((p.attr.zero_points.src != 0 || p.attr.zero_points.dst != 0
+                            || p.attr.zero_points.weights != 0),
+                "Zero points not supported for HIP");
+
         SKIP_IF_CUDA((p.attr.scale_flags & P::MASK_MASK) == P::PER_N,
                 "Per dimensional scaling is not supported for CUDA");
 
+        SKIP_IF_HIP((p.attr.scale_flags & P::MASK_MASK) == P::PER_N,
+                "Per dimensional scaling is not supported for HIP");
+
         catch_expected_failures(
                 [=]() { Test(); }, p.expect_to_fail, p.expected_status, false);
     }
@@ -151,36 +155,10 @@ protected:
     }
 
     static void create_attr(const matmul_test_params_t &p, primitive_attr &attr,
-            memory &scales_m, memory &zero_points_src_m,
-            memory &zero_points_weights_m, memory &zero_points_dst_m,
-            engine &eng) {
+            memory &zero_points_src_m, memory &zero_points_weights_m,
+            memory &zero_points_dst_m, engine &eng) {
         const int ndims = (int)p.base.dst.dims.size();
 
-        // output scales
-        if (p.attr.scale_flags != P::NONE) {
-            ASSERT_TRUE(p.attr.scale_flags & P::SCALES);
-
-            unsigned scales_mask = p.attr.scale_flags & P::MASK_MASK;
-            ASSERT_TRUE(scales_mask == P::COMMON || scales_mask == P::PER_N);
-
-            int mask = scales_mask == P::PER_N ? 1 << (ndims - 1) : 0;
-            memory::dim scale_size = mask ? p.base.dst.dims[ndims - 1] : 1;
-
-            if (p.attr.scale_flags & P::RUNTIME) {
-                attr.set_output_scales(mask, {DNNL_RUNTIME_F32_VAL});
-
-                scales_m = test::make_memory(
-                        {{scale_size}, memory::data_type::f32, {1}}, eng);
-                auto s = map_memory<float>(scales_m);
-                GTEST_EXPECT_NE(s, nullptr);
-                for (memory::dim i = 0; i < scale_size; ++i)
-                    s[i] = 2.f;
-            } else {
-                std::vector<float> scales(scale_size, 2.f);
-                attr.set_output_scales(mask, scales);
-            }
-        }
-
         // zero points
         auto handle_zero_points = [&](int arg, unsigned flags,
                                           const matmul_base_t::md_t &md,
@@ -210,19 +188,13 @@ protected:
             int mask = zero_points_mask == P::PER_N ? 1 << (ndims - 1) : 0;
             memory::dim zero_points_size = mask ? md.dims[ndims - 1] : 1;
 
-            if (flags & P::RUNTIME) {
-                attr.set_zero_points(arg, mask, {DNNL_RUNTIME_S32_VAL});
-                zero_points_m = test::make_memory(
-                        {{zero_points_size}, memory::data_type::s32, {1}}, eng);
-                auto z = map_memory<int32_t>(zero_points_m);
-                GTEST_EXPECT_NE(z, nullptr);
-                for (memory::dim i = 0; i < zero_points_size; ++i)
-                    z[i] = (arg % 7) - 3;
-            } else {
-                std::vector<int32_t> zero_points(
-                        zero_points_size, (arg % 7) - 3);
-                attr.set_zero_points(arg, mask, zero_points);
-            }
+            attr.set_zero_points_mask(arg, mask);
+            zero_points_m = test::make_memory(
+                    {{zero_points_size}, memory::data_type::s32, {1}}, eng);
+            auto z = map_memory<int32_t>(zero_points_m);
+            GTEST_EXPECT_NE(z, nullptr);
+            for (memory::dim i = 0; i < zero_points_size; ++i)
+                z[i] = (arg % 7) - 3;
         };
 
         handle_zero_points(DNNL_ARG_SRC, p.attr.zero_points.src, p.base.src,
@@ -238,7 +210,7 @@ protected:
             switch (post_op.kind) {
                 case primitive::kind::sum: po.append_sum(); break;
                 case primitive::kind::eltwise:
-                    po.append_eltwise(1.f, post_op.alg, 0.f, 0.f);
+                    po.append_eltwise(post_op.alg, 0.f, 0.f);
                     break;
                 default: ASSERT_TRUE(!"unknown post op kind");
             }
@@ -276,15 +248,14 @@ protected:
                     init_md({bia_dims, p.base.bia_dt, bia_tag}), eng);
         }
 
-        auto matmul_d = matmul::desc(src_md, weights_md, bia_md, dst_md);
-
         primitive_attr attr;
         memory scales_m, zero_points_src_m, zero_points_weights_m,
                 zero_points_dst_m;
-        create_attr(p, attr, scales_m, zero_points_src_m, zero_points_weights_m,
+        create_attr(p, attr, zero_points_src_m, zero_points_weights_m,
                 zero_points_dst_m, eng);
 
-        auto matmul_pd = matmul::primitive_desc(matmul_d, attr, eng);
+        auto matmul_pd = matmul::primitive_desc(
+                eng, src_md, weights_md, bia_md, dst_md, attr);
 
         ASSERT_TRUE(matmul_pd.query_md(query::exec_arg_md, DNNL_ARG_SRC)
                 == matmul_pd.src_desc());
@@ -306,8 +277,11 @@ protected:
         auto set_to_zero = [](memory &m) {
             if (m) {
                 auto p = map_memory<char>(m);
-                GTEST_EXPECT_NE(p, nullptr);
-                memset(p, 0, m.get_desc().get_size());
+                auto size = m.get_desc().get_size();
+                if (size > 0) {
+                    GTEST_EXPECT_NE(p, nullptr);
+                    memset(p, 0, size);
+                }
             }
         };
         set_to_zero(src_m);
@@ -321,7 +295,6 @@ protected:
                         {DNNL_ARG_WEIGHTS, weights_m},
                         {DNNL_ARG_BIAS, bia_m},
                         {DNNL_ARG_DST, dst_m},
-                        {DNNL_ARG_ATTR_OUTPUT_SCALES, scales_m},
                         {DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_SRC,
                                 zero_points_src_m},
                         {DNNL_ARG_ATTR_ZERO_POINTS | DNNL_ARG_WEIGHTS,
@@ -346,38 +319,41 @@ HANDLE_EXCEPTIONS_FOR_TEST_P(
 
     const auto &tensor_dims = std::get<0>(GetParam());
     const auto format_tag = std::get<2>(GetParam());
+    const auto &binary_po_mem_dt = std::get<3>(GetParam());
 
-    auto src_md = memory::desc(tensor_dims, memory::data_type::u8, format_tag);
-    auto weights_md
-            = memory::desc(tensor_dims, memory::data_type::s8, format_tag);
-    auto dst_md = memory::desc(tensor_dims, memory::data_type::s8, format_tag);
+    SKIP_IF(unsupported_data_type(binary_po_mem_dt),
+            "Engine does not support this data type.");
+
+    // Currently, f16 binary post-ops are only supported for f16 primitive.
+    const auto src_dt = binary_po_mem_dt == memory::data_type::f16
+            ? memory::data_type::f16
+            : memory::data_type::u8;
+    const auto wei_dt = binary_po_mem_dt == memory::data_type::f16
+            ? memory::data_type::f16
+            : memory::data_type::s8;
+    const auto dst_dt = binary_po_mem_dt == memory::data_type::f16
+            ? memory::data_type::f16
+            : memory::data_type::s8;
+
+    auto src_md = memory::desc(tensor_dims, src_dt, format_tag);
+    auto weights_md = memory::desc(tensor_dims, wei_dt, format_tag);
+    auto dst_md = memory::desc(tensor_dims, dst_dt, format_tag);
     auto bia_md = memory::desc();
 
-    auto matmul_d = matmul::desc(src_md, weights_md, bia_md, dst_md);
-
     std::string impl_info_no_postops;
-    auto matmul_pd = matmul::primitive_desc(matmul_d, e);
+    auto matmul_pd
+            = matmul::primitive_desc(e, src_md, weights_md, bia_md, dst_md);
     ASSERT_NO_THROW(impl_info_no_postops = matmul_pd.impl_info_str(););
 
     dnnl::primitive_attr attr;
-    const float scale = 1.f;
     const float alpha = 1.f;
     const float beta = 1.f;
-    const float oscale = 1.5f;
-
-    const int ndims = std::get<4>(GetParam());
-    // per-channel output scales
-    std::vector<float> oscales(tensor_dims[1], oscale);
-    attr.set_output_scales(1 << (ndims - 1), oscales);
 
     dnnl::post_ops ops;
     ops.append_sum(1.0);
-    ops.append_eltwise(scale, algorithm::eltwise_relu, alpha, beta);
+    ops.append_eltwise(algorithm::eltwise_relu, alpha, beta);
 
     const auto &binary_po_tensor_dims = std::get<1>(GetParam());
-    const auto &binary_po_mem_dt = std::get<3>(GetParam());
-    SKIP_IF(unsupported_data_type(binary_po_mem_dt),
-            "Engine does not support this data type.");
     memory::desc src1_po_md(
             binary_po_tensor_dims, binary_po_mem_dt, format_tag);
     ops.append_binary(algorithm::binary_add, src1_po_md);
@@ -386,7 +362,8 @@ HANDLE_EXCEPTIONS_FOR_TEST_P(
 
     std::string impl_info_with_postops;
 
-    matmul_pd = matmul::primitive_desc(matmul_d, attr, e);
+    matmul_pd = matmul::primitive_desc(
+            e, src_md, weights_md, bia_md, dst_md, attr);
     ASSERT_NO_THROW(impl_info_with_postops = matmul_pd.impl_info_str(););
     ASSERT_EQ(impl_info_no_postops, impl_info_with_postops);
 }
@@ -468,47 +445,90 @@ static auto cases_ef = []() {
     //                           {{10, 20}, data_type::u8, tag::ab}},
     //        {}, true, dnnl_unimplemented});
 
-    // unimplemented formats (GPU only)
-    cases.push_back({{{{16, 16}, data_type::f32, tag::AB8a4b},
-                             {{16, 16}, data_type::f32, tag::AB8a4b},
-                             {{16, 16}, data_type::f32, tag::AB8a4b}},
-            {}, true, dnnl_unimplemented});
+    // broken broadcast case
+    cases.push_back(
+            {{{{1, 10, 2}, data_type::f32, tag::abc},
+                     {{1, 2, 20}, data_type::f32, tag::abc},
+                     {{0, 10, 20}, data_type::f32, tag::abc}, data_type::undef},
+                    {}, true, dnnl_invalid_arguments});
+
+    // broken broadcast case
+    cases.push_back(
+            {{{{0, 10, 2}, data_type::f32, tag::abc},
+                     {{2, 2, 20}, data_type::f32, tag::abc},
+                     {{0, 10, 20}, data_type::f32, tag::abc}, data_type::undef},
+                    {}, true, dnnl_invalid_arguments});
+
+    // broken broadcast case
+    cases.push_back(
+            {{{{1, 10, 2}, data_type::f32, tag::abc},
+                     {{0, 2, 20}, data_type::f32, tag::abc},
+                     {{1, 10, 20}, data_type::f32, tag::abc}, data_type::undef},
+                    {}, true, dnnl_invalid_arguments});
 
     return ::testing::ValuesIn(cases);
 };
 INSTANTIATE_TEST_SUITE_P(EF, iface, cases_ef());
 
+static auto cases_zd = [](memory::data_type dt) {
+    std::vector<matmul_test_params_t> cases;
+
+    // simple case M=0
+    cases.push_back({{{{0, 2}, dt, tag::ab}, {{2, 20}, dt, tag::ab},
+                             {{0, 20}, dt, tag::ab}, data_type::undef},
+            {}});
+    // simple case K=0
+    cases.push_back({{{{10, 0}, dt, tag::ab}, {{0, 20}, dt, tag::ab},
+                             {{10, 20}, dt, tag::ab}, data_type::undef},
+            {}});
+    // simple case N=0
+    cases.push_back({{{{10, 2}, dt, tag::ab}, {{2, 0}, dt, tag::ab},
+                             {{10, 0}, dt, tag::ab}, data_type::undef},
+            {}});
+    // broadcast case all MB=0
+    cases.push_back({{{{0, 10, 2}, dt, tag::abc}, {{0, 2, 20}, dt, tag::abc},
+                             {{0, 10, 20}, dt, tag::abc}, data_type::undef},
+            {}});
+    // broadcast case wei MB!=0
+    cases.push_back({{{{0, 10, 2}, dt, tag::abc}, {{1, 2, 20}, dt, tag::abc},
+                             {{0, 10, 20}, dt, tag::abc}, data_type::undef},
+            {}});
+    // broadcast case src MB!=0
+    cases.push_back({{{{1, 10, 2}, dt, tag::abc}, {{0, 2, 20}, dt, tag::abc},
+                             {{0, 10, 20}, dt, tag::abc}, data_type::undef},
+            {}});
+
+    return ::testing::ValuesIn(cases);
+};
+INSTANTIATE_TEST_SUITE_P(ZeroDim_f32, iface, cases_zd(data_type::f32));
+
 static auto cases_f = [](memory::data_type dt) {
     std::vector<matmul_test_params_t> cases;
 
+    auto bias_dt = data_type::f32;
+#ifdef DNNL_SYCL_HIP
+    if (dt == data_type::f16) bias_dt = data_type::f16;
+#endif
+
     // simple case
     cases.push_back({{{{10, 2}, dt, tag::ab}, {{2, 20}, dt, tag::ab},
                              {{10, 20}, dt, tag::ab}, data_type::undef},
             {}});
     // simple case + leading dimensions
-    cases.push_back({{{{10, 1}, dt, tag::ab, P::SRC | P::LEADING_DIM},
-                             {{1, 3}, dt, tag::ba},
-                             {{10, 3}, dt, tag::ab, P::DST | P::LEADING_DIM},
-                             data_type::f32},
-            {}});
+    cases.push_back(
+            {{{{10, 1}, dt, tag::ab, P::SRC | P::LEADING_DIM},
+                     {{1, 3}, dt, tag::ba},
+                     {{10, 3}, dt, tag::ab, P::DST | P::LEADING_DIM}, bias_dt},
+                    {}});
     // simple case + leading dimensions + runtime dims
     cases.push_back(
             {{{{1, 10}, dt, tag::ab, P::SRC | P::LEADING_DIM | P::RUNTIME},
                      {{10, 2}, dt, tag::ba, P::WEIGHTS | P::RUNTIME},
                      {{1, 2}, dt, tag::ab,
                              P::DST | P::LEADING_DIM | P::RUNTIME},
-                     data_type::f32},
+                     bias_dt},
                     {}});
 
-    // output scales
-    cases.push_back({{{{10, 2}, dt, tag::ab}, {{2, 20}, dt, tag::ab},
-                             {{10, 20}, dt, tag::ab}, data_type::undef},
-            {P::SCALES | P::COMMON}});
-    // output scales + per_n + runtime
-    cases.push_back({{{{10, 2}, dt, tag::ab}, {{2, 20}, dt, tag::ab},
-                             {{10, 20}, dt, tag::ab}, data_type::undef},
-            {P::SCALES | P::PER_N | P::RUNTIME}});
-
     // post-ops
     cases.push_back({{{{10, 1}, dt, tag::ab}, {{1, 20}, dt, tag::ab},
                              {{10, 20}, dt, tag::ab}},
@@ -524,21 +544,20 @@ static auto cases_f = [](memory::data_type dt) {
 
     // gemm like: output scale + post-ops(sum)
     cases.push_back({{{{10, 1}, dt, tag::ab}, {{1, 20}, dt, tag::ab},
-                             {{10, 20}, dt, tag::ab}, data_type::f32},
+                             {{10, 20}, dt, tag::ab}, bias_dt},
             {P::SCALES | P::COMMON, {}, {{primitive::kind::sum}}}});
     // gemm like: output scale + post-ops(sum) + all runtime
-    cases.push_back({{{{10, 1}, dt, tag::ab, P::SRC | P::RUNTIME},
-                             {{1, 20}, dt, tag::ab, P::WEIGHTS | P::RUNTIME},
-                             {{10, 20}, dt, tag::ab, P::DST | P::RUNTIME},
-                             data_type::f32},
-            {P::SCALES | P::COMMON | P::RUNTIME, {},
-                    {{primitive::kind::sum}}}});
+    cases.push_back(
+            {{{{10, 1}, dt, tag::ab, P::SRC | P::RUNTIME},
+                     {{1, 20}, dt, tag::ab, P::WEIGHTS | P::RUNTIME},
+                     {{10, 20}, dt, tag::ab, P::DST | P::RUNTIME}, bias_dt},
+                    {P::SCALES | P::COMMON, {}, {{primitive::kind::sum}}}});
 
     return ::testing::ValuesIn(cases);
 };
 
-GPU_INSTANTIATE_TEST_SUITE_P(Generic_f16, iface, cases_f(data_type::f16));
-GPU_INSTANTIATE_TEST_SUITE_P(Generic_bf16, iface, cases_f(data_type::bf16));
+INSTANTIATE_TEST_SUITE_P(Generic_f16, iface, cases_f(data_type::f16));
+INSTANTIATE_TEST_SUITE_P(Generic_bf16, iface, cases_f(data_type::bf16));
 INSTANTIATE_TEST_SUITE_P(Generic_f32, iface, cases_f(data_type::f32));
 
 static auto cases_x8 = [](memory::data_type src_dt, memory::data_type dst_dt) {
@@ -565,17 +584,6 @@ static auto cases_x8 = [](memory::data_type src_dt, memory::data_type dst_dt) {
                      data_type::u8},
                     {}});
 
-    // output scales
-    cases.push_back(
-            {{{{10, 2}, src_dt, tag::ab}, {{2, 20}, data_type::s8, tag::ab},
-                     {{10, 20}, dst_dt, tag::ab}, data_type::undef},
-                    {P::SCALES | P::COMMON}});
-    // output scales + per_n + runtime
-    cases.push_back(
-            {{{{10, 2}, src_dt, tag::ab}, {{2, 20}, data_type::s8, tag::ab},
-                     {{10, 20}, dst_dt, tag::ab}, data_type::undef},
-                    {P::SCALES | P::PER_N | P::RUNTIME}});
-
     // zero points
     cases.push_back(
             {{{{10, 2}, src_dt, tag::ba}, {{2, 20}, data_type::s8, tag::ab},
@@ -591,16 +599,15 @@ static auto cases_x8 = [](memory::data_type src_dt, memory::data_type dst_dt) {
                      {{10, 20}, dst_dt, tag::ab}, data_type::f32},
                     {P::SCALES | P::COMMON | P::RUNTIME,
                             {P::ZERO_POINTS | P::SRC | P::COMMON, P::NONE,
-                                    P::ZERO_POINTS | P::DST | P::COMMON
-                                            | P::RUNTIME}}});
+                                    P::ZERO_POINTS | P::DST | P::COMMON}}});
 
     // per_dim_1 zero points + runtime
-    cases.push_back({{{{10, 2}, src_dt, tag::ba},
-                             {{2, 20}, data_type::s8, tag::ab},
-                             {{10, 20}, dst_dt, tag::ab}, data_type::f32},
-            {P::SCALES | P::COMMON | P::RUNTIME,
-                    {P::ZERO_POINTS | P::SRC | P::PER_N | P::RUNTIME, P::NONE,
-                            P::ZERO_POINTS | P::DST | P::PER_N | P::RUNTIME}}});
+    cases.push_back(
+            {{{{10, 2}, src_dt, tag::ba}, {{2, 20}, data_type::s8, tag::ab},
+                     {{10, 20}, dst_dt, tag::ab}, data_type::f32},
+                    {P::SCALES | P::COMMON | P::RUNTIME,
+                            {P::ZERO_POINTS | P::SRC | P::PER_N, P::NONE,
+                                    P::ZERO_POINTS | P::DST | P::PER_N}}});
     // post-ops
     cases.push_back({{{{10, 1}, src_dt, tag::ab},
                              {{1, 20}, data_type::s8, tag::ab},
@@ -617,23 +624,22 @@ static auto cases_x8 = [](memory::data_type src_dt, memory::data_type dst_dt) {
                                             algorithm::eltwise_relu}}}});
 
     // igemm like: output scale + post-ops(sum)
-    cases.push_back({{{{10, 1}, src_dt, tag::ab},
-                             {{1, 20}, data_type::s8, tag::ab},
-                             {{10, 20}, dst_dt, tag::ab}, data_type::s8},
-            {P::SCALES | P::COMMON,
-                    {P::ZERO_POINTS | P::SRC | P::COMMON, P::NONE,
-                            P::ZERO_POINTS | P::DST | P::COMMON | P::RUNTIME},
-                    {{primitive::kind::sum}}}});
+    cases.push_back(
+            {{{{10, 1}, src_dt, tag::ab}, {{1, 20}, data_type::s8, tag::ab},
+                     {{10, 20}, dst_dt, tag::ab}, data_type::s8},
+                    {P::SCALES | P::COMMON,
+                            {P::ZERO_POINTS | P::SRC | P::COMMON, P::NONE,
+                                    P::ZERO_POINTS | P::DST | P::COMMON},
+                            {{primitive::kind::sum}}}});
     // igemm like: output scale + post-ops(sum) + all runtime
-    cases.push_back({{{{10, 2}, src_dt, tag::ba},
-                             {{2, 20}, data_type::s8, tag::ba},
-                             {{10, 20}, dst_dt, tag::ab}, data_type::s8},
-            {P::SCALES | P::PER_N | P::RUNTIME,
-                    {P::ZERO_POINTS | P::SRC | P::COMMON | P::RUNTIME,
-                            P::ZERO_POINTS | P::WEIGHTS | P::COMMON
-                                    | P::RUNTIME,
-                            P::ZERO_POINTS | P::DST | P::COMMON | P::RUNTIME},
-                    {{primitive::kind::sum}}}});
+    cases.push_back(
+            {{{{10, 2}, src_dt, tag::ba}, {{2, 20}, data_type::s8, tag::ba},
+                     {{10, 20}, dst_dt, tag::ab}, data_type::s8},
+                    {P::SCALES | P::PER_N,
+                            {P::ZERO_POINTS | P::SRC | P::COMMON,
+                                    P::ZERO_POINTS | P::WEIGHTS | P::COMMON,
+                                    P::ZERO_POINTS | P::DST | P::COMMON},
+                            {{primitive::kind::sum}}}});
 
     return ::testing::ValuesIn(cases);
 };
@@ -657,6 +663,9 @@ INSTANTIATE_TEST_SUITE_P(TensorDims, attr_test_t,
                         memory::data_type::f32, 4),
                 std::make_tuple(memory::dims {2, 10, 10, 10},
                         memory::dims {2, 10, 10, 10}, tag::abcd,
-                        memory::data_type::bf16, 4)));
+                        memory::data_type::bf16, 4),
+                std::make_tuple(memory::dims {2, 10, 10, 10},
+                        memory::dims {2, 10, 10, 10}, tag::abcd,
+                        memory::data_type::f16, 4)));
 
 } // namespace dnnl
