@@ -1,5 +1,5 @@
 /*******************************************************************************
-* Copyright 2019-2021 Intel Corporation
+* Copyright 2019-2022 Intel Corporation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
@@ -66,8 +66,8 @@ void compute_ref_resampling_fwd(const resampling_test_params_t &p,
     const memory::desc src_d = src_m.get_desc();
     const memory::desc dst_d = dst_m.get_desc();
 
-    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.data);
-    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.data);
+    const dnnl::impl::memory_desc_wrapper src_mdw(src_d.get());
+    const dnnl::impl::memory_desc_wrapper dst_mdw(dst_d.get());
 
     auto pd = p.test_pd;
     auto padded_c = src_mdw.padded_dims()[1];
@@ -131,8 +131,8 @@ void compute_ref_resampling_bwd(const resampling_test_params_t &p,
     const memory::desc diff_src_d = diff_src_m.get_desc();
     const memory::desc diff_dst_d = diff_dst_m.get_desc();
 
-    const dnnl::impl::memory_desc_wrapper diff_src_mdw(diff_src_d.data);
-    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.data);
+    const dnnl::impl::memory_desc_wrapper diff_src_mdw(diff_src_d.get());
+    const dnnl::impl::memory_desc_wrapper diff_dst_mdw(diff_dst_d.get());
 
     auto pd = p.test_pd;
     auto padded_c = diff_src_mdw.padded_dims()[1];
@@ -210,6 +210,7 @@ private:
     std::shared_ptr<test_memory> src, dst, diff_src, diff_dst;
     std::shared_ptr<memory::desc> src_desc, dst_desc;
     std::vector<float> factors;
+    std::vector<float> expected_factors;
     resampling_forward::primitive_desc resampling_pd;
 
     resampling_test_params_t p;
@@ -269,28 +270,38 @@ protected:
         dst_desc = std::make_shared<memory::desc>(
                 dst_dims, data_type, p.src_format);
 
+        expected_factors.resize(src_desc->get_ndims() - 2);
+        for (int i = 0; i < src_desc->get_ndims() - 2; i++) {
+            expected_factors[i] = (double)dst_desc->get_dims()[2 + i]
+                    / src_desc->get_dims()[2 + i];
+        }
+
         Forward();
         Backward();
     }
 
     void Forward() {
-        auto resampling_desc = resampling_forward::desc(
-                p.aprop_kind, p.aalgorithm, *src_desc, *dst_desc);
-
-        resampling_pd
-                = resampling_forward::primitive_desc(resampling_desc, eng);
+        resampling_pd = resampling_forward::primitive_desc(
+                eng, p.aprop_kind, p.aalgorithm, *src_desc, *dst_desc);
         resampling_pd = resampling_forward::primitive_desc(
                 resampling_pd.get()); // test construction from a C pd
 
-        if (true) {
-            auto resampling_desc_no_dst = resampling_forward::desc(p.aprop_kind,
-                    p.aalgorithm, factors, resampling_pd.src_desc());
-            auto resampling_pd_no_dst = resampling_forward::primitive_desc(
-                    resampling_desc_no_dst, eng);
+        {
+            auto resampling_desc_no_dst
+                    = resampling_forward::primitive_desc(eng, p.aprop_kind,
+                            p.aalgorithm, factors, resampling_pd.src_desc());
+            auto resampling_pd_no_dst
+                    = resampling_forward::primitive_desc(eng, p.aprop_kind,
+                            p.aalgorithm, factors, resampling_pd.src_desc());
             ASSERT_EQ(
                     resampling_pd.dst_desc(), resampling_pd_no_dst.dst_desc());
+            ASSERT_EQ(resampling_pd_no_dst.get_factors(), expected_factors);
         }
 
+        ASSERT_EQ(resampling_pd.get_prop_kind(), p.aprop_kind);
+        ASSERT_EQ(resampling_pd.get_algorithm(), p.aalgorithm);
+        ASSERT_EQ(resampling_pd.get_factors(), expected_factors);
+
         auto src = test::make_memory(resampling_pd.src_desc(), eng);
         auto dst = test::make_memory(resampling_pd.dst_desc(), eng);
         auto dst_ref = test::make_memory(resampling_pd.dst_desc(), eng);
@@ -311,11 +322,8 @@ protected:
     }
 
     void Backward() {
-        auto resampling_bwd_desc = resampling_backward::desc(
-                p.aalgorithm, factors, *src_desc, *dst_desc);
-
-        auto resampling_bwd_pd = resampling_backward::primitive_desc(
-                resampling_bwd_desc, eng, resampling_pd);
+        auto resampling_bwd_pd = resampling_backward::primitive_desc(eng,
+                p.aalgorithm, factors, *src_desc, *dst_desc, resampling_pd);
 
         auto diff_src
                 = test::make_memory(resampling_bwd_pd.diff_src_desc(), eng);
@@ -324,6 +332,10 @@ protected:
         auto diff_src_ref
                 = test::make_memory(resampling_bwd_pd.diff_src_desc(), eng);
 
+        ASSERT_EQ(resampling_bwd_pd.get_prop_kind(), prop_kind::backward_data);
+        ASSERT_EQ(resampling_bwd_pd.get_algorithm(), p.aalgorithm);
+        ASSERT_EQ(resampling_bwd_pd.get_factors(), expected_factors);
+
         fill_data<data_t>(
                 diff_dst.get_desc().get_size() / sizeof(data_t), diff_dst);
         check_zero_tail<data_t>(1, diff_dst);
